/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $Id: alt.c,v 1.16 92/05/29 14:45:32 boyd Exp $
   Purpose:
       Create the window that contains the alternative
       definitions.  It also is the ONLY place that the user can type a
       symbol into DeveGram.  All of the callbacks are included in this file
       for the appropriate button and keypress events.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.16 $
   Date:        $Date: 92/05/29 14:45:32 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  SensitizeAltButtons
                                           IsFirstAlt
					   IsLastAlt
   HIDDEN  AltMove
                                           FindAlt
			   SensitizeAltButtons
			                   UpdateInterface
   HIDDEN  ChangeType
   HIDDEN  ChangeOccurrence
                                           FindOccurW
					   GetOccurAttr
					   RemoveOccur
   EXPORT  CreateOccurW

   EXPORT  CreateTypeW

   EXPORT  CreateNodeW
					   GetStringWidthInPixels
   EXPORT  NodeSelect
					   FindNodeW
					   UpdateInterface
   EXPORT  NodeDone
					   AddSymbol
					   FindNodeW
					   GetFirstChild
					   GetLabel
					   GetNextNode
					   GetOccurAttr
					   GetSymbolAttr
					   IsLastNode
					   IsRoot
					   IsTerminal
					   MakeSubtree
					   NewNode
					   PutDefined
					   PutSymbolType
					   PutTerminal
					   RemoveNode
					   UpdateInterface
   EXPORT  CreateAltDisplay
*/

/* HISTORY
   $Log:	alt.c,v $
 * Revision 1.16  92/05/29  14:45:32  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.15  92/04/27  20:24:20  ware
 * Fix problem with not being able to insert before first symbol in production window
 * 
 * Revision 1.14  92/03/18  14:14:32  ware
 * Working on consistent interface.
 * 
 * Revision 2.3  92/03/05  17:36:02  boyd
 * Changes to keep boxes around text fields in production:
 *  . replaced nodeframe with a Group widget
 *  . replaced nodew widgets with IcaText
 *  . changing border_width, instead of setting border to BG
 * My Xt calls are recognizable by XtVa style.
 * 
 * Revision 1.13  1992/02/20  04:51:54  cso
 * Because of problems occurring when changing a symbol name and some
 * other duplicities (like updating the interface twice instead of once).
 * A rewrite of NodeDone was in order to better process a symbol.  In so
 * doing it created the opportunity to offer a cancel option to the user
 * on the attribute panel.  These changes fixed some problems and made
 * things much more asynchronous.
 *
 * Revision 1.12  1992/02/11  16:24:33  cso
 * Changed the focus feedback to a switch of bg/fg colors and caret display
 *
 * Revision 1.11  1992/01/28  14:40:59  cso
 * Aesthetic changes
 *
 * Revision 1.10  1992/01/14  14:55:55  cso
 *  - Reorganized header files.
 *  - Added an action for a translation table for keypad accelerators
 * while the focus is on the symbols.
 *  - Increased the range of the keyboard accelerators to the main
 * interface.
 *
 * Revision 1.9  1992/01/03  19:42:33  cso
 * Modified the name to widget call to find nodeframe.
 *
 * Revision 1.8  1992/01/02  23:28:01  cso
 * *** empty log message ***
 *
 * Revision 1.7  1992/01/02  22:36:40  cso
 * In ChangeType:
 * 	Modified the routine to use PutSymbolType for toggling
 * between Group and Part types.
 *
 * Renamed ChangeOccurrence to SetOccurW:
 * 	Use a menu to change the occurrence but first update Tree to
 * point to the correct node.
 *
 * Changed the classes of widgets for the type and occurrence.
 *
 * In NodeDone:
 * 	Restructured to use a panel for the user to describe the
 * symbol's type and occurrence, this still needs refinement.
 * 	Fixed the keyboard focus problem when empty placeholders are
 * removed.
 *
 * Revision 1.6  1991/11/22  23:34:36  cso
 * Finally, have the display working as expected, perhaps a bit quicker,
 * but there is still a small amount of flicker on the redraws.  The
 * fixes included some small problems when removing empty place holders
 * and complete alternatives.
 *
 * Revision 1.5  1991/11/20  00:01:50  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.4  1991/11/19  18:16:21  cso
 * Added the Enter translation to be the same as for Return.
 *
 * Revision 1.3  1991/11/14  14:59:43  cso
 * Added a translation for linefeed as well as return.
 *
 * Revision 1.2  1991/11/14  14:30:49  cso
 * types.h utils.c utils.h CVS: CVS: Enter Log.  Lines beginning with
 * 'CVS: ' are removed automatically CVS:
 * ----------------------------------------------------------------------
 * Made the necessary additions and changes to provide a mechanism to
 * change symbol types between nonterminal types Nonterminal & Grouping.
 * In SGML terms, therefore, intermediate form terms, this is the
 * difference between having element tags and not.
 *
 * Revision 1.1.1.3  1991/10/04  20:15:22  jbm
 * First HaL mods to ICA
 *
 * Revision 2.1  91/05/28  09:58:13  cso
 * Removed Chosen_Symbol.
 * 
 * Revision 2.0  91/03/30  13:46:20  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.15  91/03/28  14:01:22  cso
 * Moved the AddActions routine to interface.c because it only needs to be 
 * done once.
 * 
 * Revision 1.14  91/02/21  09:36:40  cso
 * Completed documentation.
 * Because of concerns about checking for depth, some of SensitizeAlt
 * got muddied - it will be cleaned up again.
 * Used a typedef for Direction of the movement buttons and combined
 * PrevAlt and NextAlt into AltMove - they were too similar.
 * 
 * Revision 1.13  90/11/26  13:10:09  cso
 * RemoveNode call needs to know tree display orientation (vertical).
 * 
 * Revision 1.12  90/11/20  15:16:07  cso
 * Changed ptr name to aptr in SensitizeAltButtons.
 * Added ChangeOccurrence so that an occurrence can be changed without having
 * to delete a symbol.
 * Finally, added a way to alter the widget size around the size of the text
 * in CreateNodeW.
 * If a symbol name has been changed an additional check was needed
 * in NodeDone to not ask for a new occurrence.
 * Other changes based on lint.
 * 
 * Revision 1.11  90/11/05  17:52:29  cso
 * Added function template to all functions, but still need to fill it in.
 * Fixed PrevAlt so that 3D stuff works at least with these buttons.
 * Mostly cosmetics.  I was able to make sure that no spaces ever get enterred.
 * I am ready to change occurw to a command widget.
 * 
 * Revision 1.10  90/10/24  13:24:45  cso
 * Added SYMBOLSIZE, so that I could eliminate the malloc in LoadFile.  I also
 * added the check while a user enters symbols to check against that size.
 * The other changes were based  on the new enum types in types.h.
 * 
 * Revision 1.9  90/10/04  15:51:15  cso
 * Removed unnecessary resource declarations for nodeframe.
 * 
 * Revision 1.8  90/10/02  13:27:37  cso
 * If SensitizeAltButtons receives a NULL Alt pointer,
 * the semantics should be that it is the Root and therefore act
 * like it was programmed to, i.e., Set Prev and Next insensitive.
 * 
 * Revision 1.7  90/09/07  09:34:41  cso
 * Altered the dependency on GetSymbol/OccurAttr to continue the program
 * and put it in node_done.
 * 
 * Revision 1.6  90/09/03  18:22:24  cso
 * Function name changes.
 * 
 * Revision 1.5  90/09/03  14:26:08  cso
 * Adjusting node_done to use Node functions.  Added SensitizeAltButtons
 * to make NextAlt and PrevAlt easier to follow.
 * 
 * Revision 1.4  90/08/29  17:43:03  cso
 * Updating to using GetSymbolAttr() and GetOccurAttr().  Using popups
 * to get attributes instead of all of the setsensitive stuff.
 * 
 * Revision 1.3  90/08/26  14:12:55  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "att.h"
#include "get.h"
#include "interface.h"
#include "main.h"
#include "node.h"
#include "put.h"
#include "remove.h"
#include "status.h"
#include "symtab.h"
#include "utils.h"



/*
   FUNCTION: 		SensitizeAltButtons

   Synopsis: 
     (De)Sensitizes the previous and next alternative buttons depending
     on the alt ptr given.

   Initial purpose:
     Same as above.

   Present purpose:
     Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
      Frame              To get the appropriate widgets

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SensitizeAltButtons(aptr)

Alt
   *aptr;     /* The alt ptr under consideration */

{
   Widget
      prevw,  /* Previous alternative button widget */
      nextw;  /* Next alternative button widget */

   /*
    * Get the proper widgets
    */
   prevw = XtNameToWidget(Frame, "*prevalt");
   nextw = XtNameToWidget(Frame, "*nextalt");

   /*
    * Check if the alt ptr is NULL.
    * A NULL alt ptr better be the root symbol.  
    * An error cannot be detected if it is not the Root and is NULL.
    */
   if (aptr != NULL)
   {
      /*
       * If aptr is the only alt ptr, both buttons false and no depth
       * Else there is depth, but where is the pointer? first or last or middle
       * If aptr is the first alternative then prev is F, next is T
       * Else if aptr is the last alternative then prev is T, next is F
       * Else it is in the middle so both buttons are true
       */
      if ((IsFirstAlt(aptr) == True) && (IsLastAlt(aptr) == True))
      {
	 XtSetSensitive(prevw, False);
	 XtSetSensitive(nextw, False);
      }
      else if (IsFirstAlt(aptr) == True)
      {
	 XtSetSensitive(prevw, False);
	 XtSetSensitive(nextw, True);
      }
      else if (IsLastAlt(aptr) == True)
      {
	 XtSetSensitive(prevw, True);
	 XtSetSensitive(nextw, False);
      }
      else
      {
	 XtSetSensitive(prevw, True);
	 XtSetSensitive(nextw, True);
      }
   }
   else
   {
      /*
       * It must be the Root symbol else it is an undetectable error 
       */
      XtSetSensitive(prevw, False);
      XtSetSensitive(nextw, False);
   }
   return;
}




/*
   FUNCTION:              AltMove

   Synopsis:
     Show the previous/next alternative to the user, but only if there is one.

   Initial purpose:
     Only showed one direction, thus two callbacks.

   Present purpose:
     One callback with a direction being passed in client_data.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
      Tree               Points to the current node in the tree being used

   Globals Modified:
      Tree               Points to the first node in the previous alternative

   Return value:	 void

   Range:		 none
*/

static void
AltMove(w, direction, call_data)

Widget
   w;            /* The widget using this callback */
DepthDirection
   direction;    /* The direction of the move */
caddr_t
   call_data;    /* Not used */

{
   Alt
      *aptr;     /* Pointer to the alternative definition displayed */
   Node
      *oldptr;   /* Initial node being pointed to by Tree */
   
   /*
    * Remember the node pointed to by Tree
    */
   oldptr = Tree;

   /*
    * Find the alternative containing Tree
    * If it is not NULL, try to move to another alternative
    */
   if ((aptr = FindAlt(Tree)) != NULL)
   {
      switch (direction)
      {
      default:
	 /*
	  * This is an error
	  */
	 fprintf(stderr, "Illegal direction passed to AltMove\n");
	 exit(1);
	 break;

      case Previous:
	 /*
	  * If I cannot move to a previous alternative, inform the user
	  * SensitizeAltButtons should rectify this situation
	  * Else move and set Tree to be the first node in the previous alt
	  */
	 if (aptr->prev == NULL)
	 {
	    XicaStatus(Statw, "Sorry, no previous alternative definition\n");
	 }
	 else
	 {
	    aptr = aptr->prev;
	    Tree = aptr->prod;
	 }
	 break;

      case Next:
	 /*
	  * If I cannot move to a next alternative, inform the user
	  * SensitizeAltButtons should rectify this situation
	  * Else move and set Tree to be the first node in the next alt
	  */
	 if (aptr->next == NULL)
	 {
	    XicaStatus(Statw, "Sorry, no next alternative definition\n");
	 }
	 else
	 {
	    aptr = aptr->next;
	    Tree = aptr->prod;
	 }
	 break;
      }
   }
   /*
    * Sensitize the alt buttons based on new position
    * Update the entire interface
    */
   SensitizeAltButtons(aptr);
   UpdateInterface(oldptr, Tree);
   return;
}




/*
   FUNCTION: 		ChangeType

   Synopsis: 
     Change the symbol type of a particular symbol.

   Initial purpose:
     Same as above.

   Present purpose:
     Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Operating System issues:
      CALL		 PURPOSE

   Globals used:
      Tree               To find the symbol associated with the occurrence

   Globals Modified:
      none

   Return value:	void

   Range:		none
*/

static void
ChangeType(w, client_data, call_data)

Widget
   w;             /* The particular widget calling this callback */
caddr_t
   client_data,   /* Not used. */
   call_data;     /* Not used. */

{
   Node
      *nptr;      /* Pointer to the symbol associated with
		     the widget selected */
   SymType
      oldtype;   /* The old occurrence value */
   
   /*
    * Find the symbol associated with the occurrence widget
    */
   nptr = FindTypeW(w, Tree);

   /*
    * If a symbol exists (Non-NULL), change its occurrence
    * Else make the widget insensitive and inform the user
    */
   if ((nptr != NULL) && (*nptr->symbol != '\0'))
   {
      /*
       * Remember the old type
       * Get the new value
       * Remove the old value
       * If the old and new values are not the same then set the Dirtyflag
       */
      oldtype = GetSymbolType(nptr->symbol);
      if (oldtype == Part)
      {
	 PutSymbolType(nptr->symbol, Group);
      }
      else
      {
	 PutSymbolType(nptr->symbol, Part);
      }
      Dirtyflag = True;
   }
   else
   {
      XicaStatus(Statw, "Cannot change the type of a NULL symbol\n");
      XtSetSensitive(w, False);
   }
   return;
}




/*
   FUNCTION: 		ChangeOccurrence

   Synopsis: 
     Change the occurrence value of a particular symbol.

   Initial purpose:
     Same as above.

   Present purpose:
     Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Operating System issues:
      CALL		 PURPOSE

   Globals used:
      Tree               To find the symbol associated with the occurrence

   Globals Modified:
      none

   Return value:	void

   Range:		none
*/

static void
SetOccurw(w, client_data, call_data)

Widget
   w;             /* The particular widget calling this callback */
caddr_t
   client_data,   /* Not used. */
   call_data;     /* Not used. */

{
   Node
      *nptr;      /* Pointer to the symbol associated with
		     the widget selected */
   
   /*
    * Find the symbol associated with the occurrence widget
    */
   nptr = FindOccurW(w, Tree);

   /*
    * If a symbol exists (Non-NULL), change its occurrence
    * Else make the widget insensitive and inform the user
    */
   if ((nptr != NULL) && (*nptr->symbol != '\0'))
   {
      if (Tree != nptr)
      {
	 UpdateInterface(Tree, nptr);
      }
      Tree = nptr;
   }
   else
   {
      XicaStatus(Statw, "Cannot change the occurrence of a NULL symbol\n");
      XtSetSensitive(w, False);
   }
   return;
}




/*
   FUNCTION: 		AltAction

   Synopsis: 
       A way to have wider keyboard focus and still manipulate the
       compass.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AltAction(w, event, params, num)

Widget
   w;          /* The widget calling this procedure */
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   if (**params == 'p')
   {
      XtCallCallbacks(XtNameToWidget(Frame, "*prevalt"),
		      XtNcallback, NULL);
   }
   else if (**params == 'n')
   {
      XtCallCallbacks(XtNameToWidget(Frame, "*nextalt"),
		      XtNcallback, NULL);
   }
   return;
}



/*
   FUNCTION: 		CreateOccurW

   Synopsis: 
       Creates the occurrence widget for a particular node.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
      Frame              To get the parent of this widget

   Globals Modified:
      none

   Return value:	Widget

   Range:		A Widget structure if XtCreateManagedWidget worked
*/

Widget
CreateOccurW(left)

Widget
   left;         /* The widget to my left */

{
   int
      cnt;       /* Number of arguments */
   Arg
      args[MAXARGS];/* List of arguments */
   static String 
      menutransstr = "<Btn1Down>: set() notify() PopupMenu() unset()";
   Widget
      t1,        /* Temporary widget to be returned */
      nodeframe; /* The parent widget */
   XtTranslations 
      menutrans;
   
   /*
    * Get my parent widget
    */
   nodeframe = XtNameToWidget(Frame, "*nodeframe");

   /*
    * Set up my arguments
    * Create the widget structure
    * Add the callback and return the new widget
    */
   t1 = XtVaCreateManagedWidget( 
            "occurw", menuButtonWidgetClass, nodeframe,
            XtNlabel,       " ",
            XtNsensitive,   False,
            XtNmenuName,    "occurmenu",
            XtNalignment,   gaCenter,
            XtNmargin,      2,
            XtNfromSibling, left,
            NULL );
   XtAddCallback(t1, XtNcallback, SetOccurw, NULL);

   menutrans = XtParseTranslationTable(menutransstr);
   XtOverrideTranslations(t1, menutrans);

   return(t1);
}




/*
   FUNCTION: 		CreateTypeW

   Synopsis: 
       Creates the type widget for a particular node.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
      Frame              To get the parent of this widget

   Globals Modified:
      none

   Return value:	Widget

   Range:		A Widget structure if XtCreateManagedWidget worked
*/

Widget
CreateTypeW(left)

Widget
   left;         /* The widget to my left */

{
   int
      cnt;       /* Number of arguments */
   Arg
      args[MAXARGS];/* List of arguments */
   Widget
      t1,        /* Temporary widget to be returned */
      nodeframe; /* The parent widget */
   
   /*
    * Get my parent widget
    */
   nodeframe = XtNameToWidget(Frame, "*nodeframe");

   /*
    * Create the widget structure
    */
   t1 = XtVaCreateManagedWidget(
            "typew", toggleWidgetClass, nodeframe,
            XtNlabel,         " ",
            XtNsensitive,     False,
            XtNalignment,     gaCenter,
            XtNmargin,        2,
            XtNfromSibling,   left,
            NULL );

   /*
    * Add the callback and return the new widget
    */
   XtAddCallback(t1, XtNcallback, ChangeType, NULL);
   return(t1);
}




/*
   FUNCTION: 		CreateNodeW

   Synopsis: 
       Create a widget representing a node in the grammar tree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE     REASON
       cso	 91-03-28 Corrected the search path to find ks_names.h
       			  Moved the AddActions call to interface.c so
			   that it is only done once.

   Globals used:
        Frame            To obtain the parent widget

   Globals Modified:
	none

   Return value:	Widget

   Range:		A Widget structure if XtCreateManagedWidget worked
*/

Widget
CreateNodeW(left, str)

Widget
   left;             /* The widget to my left */
String
   str;              /* The symbol string to be placed in the widget */

{
   int
      cnt;           /* Number of arguments */
   Arg
      args[MAXARGS]; /* List of arguments */
   Dimension
      text_width;    /* Width of the string in pixels */
   Widget
      t1,            /* Temporary widget being created and returned */
      nodeframe;     /* My parent widget */
   XtTranslations
      node_trans;    /* The translation table after parsing */
                    
   /*
      Translations to add to this widget
      When user hits return, execute NodeDone
      When this widget is selected, highlight the text,
      and remember the symbol and node selected
      Replace whitespace with '_' - it cause problems in tokenizing
      Look for <Key> symbol names in /usr/include/X11/keysymdef.h. 
    */
   static String
      node_trans_str = "<Key>Return:   NodeDone()\n\
                        <Key>KP_Enter: NodeDone()\n\
                        <Key>Linefeed: NodeDone()\n\
                        <Btn1Down>,<Btn1Up>: select-all() NodeSelect()\n\
                        <Key>space: insert-string(\"_\")\n\
                        <Key>Tab:   insert-string(\"_\")\n\
			<Key>R9:     	AltAction(\"p\")\n\
			<Key>R15:     	AltAction(\"n\")\n\
			<Key>Up:     	CompassAction(\"n\")\n\
			<Key>Down:	CompassAction(\"s\")\n\
			<Key>Right:	CompassAction(\"e\")\n\
			<Key>Left:	CompassAction(\"w\")";

   /*
    * Get the parent widget
    */
   nodeframe = XtNameToWidget(Frame, "*nodeframe");

   /*
    * Set the arguments
    */
   cnt = 0;
   /*
    * Create the widget structure
    */
   t1 = XtVaCreateManagedWidget( 
            "nodew", icaTextWidgetClass, nodeframe,
            XtNeditType,     XawtextEdit,
            XtNresize,       XawtextResizeWidth,
            XtNborderWidth,  1,
            XtNborder,       Foreground,
            XtNdisplayCaret, False,
            XtNalignment,    gaCenter,
            XtNmargin,       2,
            XtNfromSibling,  left,
	    XtNinsertAfter,  (left != NULL),
            NULL );

   /*
    * Add the translations to the widget
    */
   node_trans = XtParseTranslationTable(node_trans_str);
   XtOverrideTranslations(t1, node_trans);

   /*
    * If a string is known, get its size, and set the appropriate
    * arguments
    */
   if ((str != NULL) && (*str != '\0'))
   {
      text_width = GetStringWidthInPixels(t1, str);
      cnt = 0;
      XtSetArg(args[cnt], XtNstring, str);        cnt++;
      XtSetArg(args[cnt], XtNwidth,  text_width); cnt++;
      XtSetValues(t1, args, cnt);
   }
   return(t1);
}




/*
   FUNCTION: 		NodeSelect

   Synopsis:
       Handle a selection of a node and update the interface.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO      DATE     REASON
       cso	91-03-28 Made the procedure external so that the
		          actions could be added.
       cso	91-05-23 Removed Chosen_Symbol instead use a temp variable.
       cso      91-07-31 Removed the reference to Chosen_Node because
                          UpdateInterface takes care of that.

   Globals used:
       Tree              Points to the present node

   Globals Modified:
       Tree              Points to the node selected

   Return value:	 void

   Range:		 none
*/

void
NodeSelect(w, event, params, num)

Widget
   w;       /* The widget calling this procedure */
XEvent
   *event;  /* Unused */
String
   *params; /* Unused */
int
   num;     /* Unused */

{
   Node
      *oldptr;  /* Track where Tree was pointing */

   /*
    * Remember where Tree was
    * Find the node whose widget was selected
    * Update the interface
    */
   oldptr = Tree;
   Tree   = FindNodeW(w, Tree);
   UpdateInterface(oldptr, Tree);
   return;
}




/*
   FUNCTION: 		NodeDone

   Synopsis: 
       Handles the event when a user hits return after typing 
       in a symbol name.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO      DATE     REASON
       cso	91-03-28 Made the procedure external so that the
		          actions could be added.

   Globals used:
       Dirtyflag       Change to the tree means it needs to be saved
       Tree             Traces the current node being manipulated

   Globals Modified:
       Dirtyflag       Might get set to True
       Tree             Might change its focus

   Return value:	void

   Range:		none
*/

void
NodeDone(w, event, params, num)

Widget
   w;          /* The widget calling this procedure */
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   String
      oldstr,  /* Remembers the symbol in case the user has changed it */
      str;     /* The string the user has presently enterred */
   Node
      *oldptr; /* Remembers the position of Tree */
   
   /*
    * Get the string that the user just enterred into the widget
    */
   str    = GetLabel(w);
#ifdef DEBUG
   fprintf(stderr, "str = [%s]\n", str);
#endif

   /*
    * Remember the present position of Tree
    * Make Tree point to the node that contains the calling widget if
    *  it isn't already
    * Remember the present symbol string
    * If the user hit return on a NULL string, remove the widget & node
    * Else it might be a string that to be placed into the node tree
    * Else if the string is too long inform the user and reject the symbol
    * if the old and new string are the same but not NULL, return
    * doing nothing - may have been a mistake by the user
    */
   oldptr = Tree;
   if (Tree->nodew != w)
   {
      Tree = FindNodeW(w, Tree);
   }
   oldstr = XtNewString(Tree->symbol);
   if (*str == '\0')
   {
      XtSetKeyboardFocus(Frame, None);
      Tree   = RemoveNode(Tree, True);
      oldptr = Root;
      UpdateInterface(oldptr, Tree);
   }
   else if (strlen(str) >= SYMBOLSIZE)
   {
      /*
       * If the length of the enterred string is too long
       * (for the load routine primarily), inform the user how to handle
       * changes and return from the callback without accepting this string
       * Else accept this string as a new symbol name
       */
      XicaStatus(Statw, "Symbol [%s] exceeds allowable symbol length [%d]\n",
		 str, SYMBOLSIZE);
      XicaStatus(Statw, "Increase SYMBOLSIZE in defs.h and recompile\n");
      XicaStatus(Statw, "Else please truncate the symbol string, Thanks!\n");
      return;
   }
   else if (strcmp(str, oldstr) == 0)
   {
      return;
   }
   else if (strlen(oldstr) != 0)
   {
      Dirtyflag = True;
      AddSymbol(Tree, str, GetSymbolType(oldstr), False);
      UpdateInterface(oldptr, Tree);
   }
   else
   {
      /*
       * If it is the Root, we know certain properties about it
       * Definitely a nonterminal, and it will be defined
       * we also need to make a subtree under it - at least one alt
       * we need to move Tree to point to Root's first child.
       */
      if (IsRoot(Tree) == True)
      {
	 Dirtyflag = True;
	 AddSymbol(Tree, str, Part, False);
	 PutTerminal(Tree, TriF);
	 PutDefined(Tree, True);
	 MakeSubtree(Tree);
	 Tree = GetFirstChild(Tree);
	 UpdateInterface(oldptr, Tree);
      }
      else /* it isn't the root node */
      {
	 /*
	  * It is not the Root, so we need to find out some properties
	  * about the symbol
	  * we need to know if it is a terminal or nonterminal
	  * if it is unknown - TriU - get that attribute
	  */
	 if (IsTerminal(str) == TriU)
	 {
	    Tree->symbol = XtNewString(str);
	    PopupAttPanel(Attnon, True);
	 }
	 else if (IsTerminal(str) == TriF)
	 {
	    Tree->symbol = XtNewString(str);
	    PutDefined(Tree, IsDefined(Tree->symbol));
	    if (GetSymbolType(Tree->symbol) == Part)
	    {
	       PopupAttPanel(Attnon, False);
	    }
	    else
	    {
	       PopupAttPanel(Attgrp, False);
	    }
	 }
	 else /* IsTerminal == TriT */
	 {

	    /*
	     * A terminal symbol - TriT - does not get an occurrence
	     * and it is always defined (from a syntactic view, not
	     * necessarily lexical view)
	     */
	    Dirtyflag = True;
	    AddSymbol(Tree, str, GetSymbolType(str), False);
      	    PutDefined(Tree, True);
	    if (Tree->occurw != NULL)
	    {
	       XtDestroyWidget(Tree->occurw);
	       Tree->occurw = NULL;
	    }
	    Tree->occurrence = "^";
	    if (Tree->typew != NULL)
	    {
	       XtDestroyWidget(Tree->typew);
	       Tree->typew = NULL;
	    }

	    /*
	     * If this is the last node in the production create a 
	     * sibling and move Tree to point to it
	     * This would be a place to add a personal choice -
	     * like type over or insert mode in an editor -
	     * always add a new node or make the user select it
	     */
	    if (Tree->next == NULL)
	    {
	       Tree->next = NewNode(Tree->parent, Tree);
	       Tree       = GetNextNode(Tree);
	    }

	    UpdateInterface(oldptr, Tree);
	 } /****** ELSE it must've been a terminal */
      } /****** ELSE it wasn't the root symbol */
   } /***** ELSE it must be a symbol being newly placed */

   XtFree(oldstr);
   return;
}




/*
   FUNCTION: 		CreateAltDisplay

   Synopsis: 
       Create the display that allows the user to manipulate definitions.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	void

   Range:		none
*/

Widget CreateAltDisplay( parent )
Widget parent;
{
   Widget
      altframe, top, bottom, left,right,
      altcmds, altlbls,
      altlabel,
      focus,   
      nodeview, 
      nodeframe,
      nextalt, 
      prevalt;  

   /*
    *   --------------------------------------------------
    *   |          altframe (table)                      |
    *   |                                                |
    *   | [altlabel (label)]  [focus (label)]            |
    *   |                                                |
    *   | [prevalt (command)] -------------------------- |
    *   |                     | nodeview (viewport)    | |
    *   |                     |                        | |
    *   | [nextalt (command)] | ---------------------- | |
    *   |                     | | nodeframe (form)   | | |
    *   |                     | |                    | | |
    *   |                     | | [nodew]s (command) | | |
    *   |                     | ---------------------- | |
    *   |                     -------------------------- |
    *   --------------------------------------------------
    *   [nodew]s are created dynamically by CreateNodeW
    */
   altframe = XtVaCreateManagedWidget( "altframe",
       tableWidgetClass, parent,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       XtNhorzMargin,  0,
       XtNvertMargin,  0,
       XtNhorzSpacing, 8,
       XtNvertSpacing, 0,
       NULL );
   left = XtVaCreateWidget( "left",
       tableColObjectClass, altframe,
       NULL );
   right = XtVaCreateWidget( "right",
       tableColObjectClass, altframe,
       XtNstretch, True,
       NULL );
   top = XtVaCreateWidget( "top",
       tableRowObjectClass, altframe,
       NULL );
   bottom = XtVaCreateWidget( "bottom",
       tableRowObjectClass, altframe,
       NULL );

   altlbls = XtVaCreateManagedWidget( "altlbls",
       groupWidgetClass, altframe,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     0,
       XtNtableRow,    top,
       XtNtableCol,    right,
       NULL );

   altlabel = XtVaCreateManagedWidget( "altlabel",
       labelWidgetClass, altlbls,
       XtNlabel,        "Production for",
       XtNborderWidth,  0,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );

   /*
    * Create the label containing the symbol string that is the parent
    * of the definition displayed in nodeframe
    */
   focus = XtVaCreateManagedWidget( "focus",
       labelWidgetClass, altlbls,
       XtNlabel,        "",
       XtNborderWidth,  0,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );

   altcmds = XtVaCreateManagedWidget( "altcmds",
       groupWidgetClass, altframe,
       XtNspacing,       0,
       XtNplacement,     gpFlush,
       XtNvertAlignment, taFill,
       XtNtableRow,      bottom,
       XtNtableCol,      left,
       NULL );

   prevalt = XtVaCreateManagedWidget( "prevalt",
       commandWidgetClass, altcmds,
       XtNlabel,     "Previous",
       XtNsensitive, False,
       NULL );
   XtAddCallback( prevalt, XtNcallback, AltMove, Previous );

   nextalt = XtVaCreateManagedWidget( "nextalt",
       commandWidgetClass, altcmds,
       XtNlabel,     "  Next  ",
       XtNsensitive, False,
       NULL );
   XtAddCallback( nextalt, XtNcallback, AltMove, Next );

   /*
    * Create the viewport through which to view productions
    * Although the scrollbars are handled by a viewport if the
    * user directly manipulates them, AdjustScrollBars will alter
    * them so that the node being currently viewed is in view.  
    * Refer to utils.c
    */
   nodeview = XtVaCreateManagedWidget( "nodeview",
       viewportWidgetClass, altframe,
       XtNallowHoriz,    True,
       XtNuseBottom,     False,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     550,
       XtNminWidth,      100,
*/
       XtNminHeight,     48,
       XtNtableRow,      bottom,
       XtNtableCol,      right,
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       NULL );

   /*
    * Create the form widget that will contain nodes in a production
    */
   nodeframe = XtVaCreateManagedWidget( "nodeframe",
       groupWidgetClass, nodeview,
       XtNorientation, XtorientHorizontal,
       XtNplacement,   gpTopOrLeft,
       XtNmargin,      0,
       XtNspacing,     2,
       XtNborderWidth, 0,
       NULL );

   return(altframe);
}
