/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: att.c,v $
   Purpose:
       To get the attributes of the symbols from the user.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.12 $
   Date:        $Date: 92/05/29 14:45:40 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  GetOccurAttr
					   PutLabel
					   PutOccur
					   XlwhAlertGetButton
					   XlwhPopup
   EXPORT  GetSymbolAttr
                                           PutLabel
					   PutSymbolType
					   PutTerminal
					   XlwhAlertGetButton
					   XlwhPopup
   EXPORT  CreateAttribShell
                                           XlwhMakeAlertShell
*/

/* HISTORY
   $Log:	att.c,v $
 * Revision 1.12  92/05/29  14:45:40  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.11  92/03/18  14:14:41  ware
 * Working on consistent interface.
 * 
 * Revision 1.10  1992/03/10  17:11:58  cso
 * Got keyboard focus to always be in Panel while it is up
 *
 * Revision 1.9  1992/02/21  02:41:10  cso
 * Removed an extra newline from an accel string
 *
 * Revision 1.8  1992/02/20  04:51:58  cso
 * Because of problems occurring when changing a symbol name and some
 * other duplicities (like updating the interface twice instead of once).
 * A rewrite of NodeDone was in order to better process a symbol.  In so
 * doing it created the opportunity to offer a cancel option to the user
 * on the attribute panel.  These changes fixed some problems and made
 * things much more asynchronous.
 *
 * Revision 1.7  1992/01/28  14:41:45  cso
 * Added translations for linefeed and enter
 *
 * Revision 1.6  1992/01/14  15:57:49  cso
 *  - Reorganization of header files.
 *  - Adjusted the positioning of the Attribute panel
 *  - Made the toggles in the panel (one of many)
 *
 * Revision 1.5  1992/01/03  21:55:17  cso
 * Created a new attribute panel for getting type and occurrence info for
 * a symbol.
 *
 * Revision 1.4  1991/11/25  14:36:13  cso
 * Made lint modifications to ensure that function and variable types are
 * being used and declared correctly and consistently.
 *
 * Revision 1.3  1991/11/20  00:02:02  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.2  1991/11/14  14:31:06  cso
 * Made the necessary additions and changes to provide a mechanism to
 * change symbol types between nonterminal types Nonterminal & Grouping.
 * In SGML terms, therefore, intermediate form terms, this is the
 * difference between having element tags and not.
 *
 * Revision 1.1.1.3  1991/10/04  20:15:26  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:46:38  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.13  91/03/30  13:25:56  cso
 * Changed the names of the buttons so that keyboard accelerators in 
 * a resource file can be used for the symbol types and occurrence types.
 * According to page 173 of X Toolkit Intrinsics, a ComponentName (widget name)
 * can only be [a-zA-Z0-9_-]*
 * 
 * Revision 1.12  91/02/21  09:45:13  cso
 * Completed documentation.
 * 
 * Revision 1.11  90/11/20  15:20:22  cso
 * Some changes based on lint.
 * 
 * Revision 1.10  90/11/05  17:56:41  cso
 * Added function template to all functions, but still need to fill it in.
 * Mostly cosmetics.
 * 
 * Revision 1.9  90/10/24  15:32:01  cso
 * Changed mallocs to XtMallocs.
 * 
 * Revision 1.8  90/10/02  13:29:38  cso
 * Removed the XtNokay resource.
 * 
 * Revision 1.7  90/09/25  15:37:21  cso
 * Had to comment out XtNhelp b/c of loop constraints
 * 
 * Revision 1.6  90/09/07  09:36:29  cso
 * looked for the string assigned to the okay button instead of "okay"
 * And elimianted the responsibility of the Attr routines to comtinue th
 * the program
 * 
 * Revision 1.5  90/09/03  18:22:38  cso
 * Function name changes.
 * 
 * Revision 1.4  90/09/03  14:27:17  cso
 * Added #ifdef XLWH for easier testing without the library.
 * 
 * Revision 1.3  90/08/29  17:44:28  cso
 * Complete change over to using Popups to get the attributes.
 * Had to use an AlertShell, but it works nicely except the XtNokay 
 * resource isn't working quite right yet.  Also, the callbacks are
 * virtually eliminated because of the GET routines in utils.c
 * 
 * Revision 1.2  90/08/26  14:13:01  cso
 * added module template
 * 
*/


/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "main.h"
#include "node.h"
#include "put.h"
#include "remove.h"
#include "status.h"
#include "symtab.h"
#include "utils.h"



/*
   FUNCTION: 		OccurSel

   Synopsis: 
       The new occurrence for a symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
OccurSel(w, occursetting, call_data)

Widget
   w;       /* The widget using this callback */
Attribute
   occursetting;
caddr_t
   call_data;    /* Not used */

{
   String
      oldoccur;
   
   /*
    * Remember the old occurrence
    * Get the new value
    * Remove the old value
    * If the old and new values are not the same then set the Dirtyflag
    */
   oldoccur = XtNewString(Tree->occurrence);
   switch (occursetting)
   {
   default:
      break;
      
   case Attexact:
      Tree->occurrence = "_";
      break;
      
   case Attopt:
      Tree->occurrence = "?";
      break;

   case Attstar:
      Tree->occurrence = "*";
      break;

   case Attplus:
      Tree->occurrence = "+";
      break;
   }
   if (*oldoccur != *Tree->occurrence)
   {
      PutOccur(Tree->symbol, Tree->occurrence[0]);
      PutLabel(Tree->occurw, Tree->occurrence);
      RemoveOccur(GetIndex(Tree->symbol), *oldoccur);
      Dirtyflag = True;
   }
   XtFree(oldoccur);
   return;
}




/*
   FUNCTION: 		ApplyAtts

   Synopsis: 

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
ApplyAtts(w, parent, call_data)

Widget
   w,
   parent;       /* The widget and its surrounding form using this callback */
caddr_t
   call_data;    /* Not used */

{
   Attribute
      attsetting;
   Boolean
      getoccur;
   Node
      *oldptr;
   String
      symbol;
   Widget
      exactw,
      nontermw;
   
   
   oldptr = Tree;
   /*
    * Hack around AddSymbol b/c it is expecting to place a symbol into
    * a virgin spot |-)
    */
   symbol = XtNewString(Tree->symbol);
   
   exactw   = XtNameToWidget(parent, "*Exact");
   nontermw = XtNameToWidget(parent, "*Nonterminal");
   if ((exactw == NULL) || (nontermw == NULL))
   {
      XicaStatus(Statw, "ApplyAtts: Could not access attribute settings\n");
      return;
   }

   getoccur = True;
   attsetting = (Attribute) XawToggleGetCurrent(nontermw);
   Dirtyflag = True;
   if (attsetting == Attnon)
   {
      getoccur = True;
      PutTerminal(Tree, TriF);
      XtFree(Tree->symbol);
      Tree->symbol = "";
      AddSymbol(Tree, symbol, Part, False);
   }
   else if (attsetting == Attterm)
   {
      getoccur = False;
      PutTerminal(Tree, TriT);
      XtFree(Tree->symbol);
      Tree->symbol = "";
      AddSymbol(Tree, symbol, Part, False);
      PutDefined(Tree, True);
      if (Tree->occurw != NULL)
      {
	 XtDestroyWidget(Tree->occurw);
	 Tree->occurw = NULL;
      }
      Tree->occurrence = "^";
      if (Tree->typew != NULL)
      {
	 XtDestroyWidget(Tree->typew);
	 Tree->typew = NULL;
      }
   }
   else if (attsetting == Attgrp)
   {
      getoccur = True;
      PutTerminal(Tree, TriF);
      XtFree(Tree->symbol);
      Tree->symbol = "";
      AddSymbol(Tree, symbol, Group, False);
   }
   else if (attsetting == Attfile)
   {
      getoccur = True;
      PutTerminal(Tree, TriF);
      XtFree(Tree->symbol);
      Tree->symbol = "";
      AddSymbol(Tree, symbol, File, False);
   }
   else
   {
      XicaStatus(Statw, "ApplyAtts: Bad attribute type setting\n");
      return;
   }
   if (IsTerminal(Tree->symbol) != TriT)
   {
      XtSetSensitive(Tree->typew, True);
   }
   
   if (getoccur == True)
   {
      attsetting = (Attribute) XawToggleGetCurrent(exactw);
      if (attsetting == Attexact)
      {
	 Tree->occurrence = "_";
      }
      else if (attsetting == Attopt)
      {
	 Tree->occurrence = "?";
      }
      else if (attsetting == Attstar)
      {
	 Tree->occurrence = "*";
      }
      else if (attsetting == Attplus)
      {
	 Tree->occurrence = "+";
      }
      else
      {
	 XicaStatus(Statw, "ApplyAtts: Bad attribute occur setting\n");
	 return;
      }

      /*
       * Put the occurrence into the symbol table
       * and into the occurrence widget and set the sensitivity
       */
      PutOccur(Tree->symbol, Tree->occurrence[0]);
      PutLabel(Tree->occurw, Tree->occurrence);
      XtSetSensitive(Tree->occurw, True);
   } /***** IF we need to check the occurrence setting */

   /*
    * If this is the last node in the production create a 
    * sibling and move Tree to point to it
    * This would be a place to add a personal choice -
    * like type over or insert mode in an editor -
    * always add a new node or make the user select it
    */
   XtSetSensitive(Tree->nodew, True);
   if (IsLastNode(Tree) == True)
   {
      Tree->next = NewNode(Tree->parent, Tree);
      Tree       = GetNextNode(Tree);
   }
   
   if (Tree->nodew != NULL)
   {
      XtSetKeyboardFocus(Frame, Tree->nodew);
   }
   
   XtPopdown(parent);
   
   /*
    * Update the interface and return
    */
   UpdateInterface(oldptr, Tree);
   
   return;
}



/*
   FUNCTION: 		DefaultAtts

   Synopsis: 
       Reset the interface to the default attribute values.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
DefaultAtts(w, parent, call_data)

Widget
   w,
   parent;       /* The widget  and its parent using this callback */
caddr_t
   call_data;    /* Not used */

{
   Widget
      exactw,
      nontermw;
   
   exactw   = XtNameToWidget(parent, "*Exact");
   nontermw = XtNameToWidget(parent, "*Nonterminal");
   if ((exactw == NULL) || (nontermw == NULL))
   {
      XicaStatus(Statw, "DefaultAtts: Could not access default attributes\n");
      return;
   }
   XawToggleSetCurrent(exactw,   Attexact);
   XawToggleSetCurrent(nontermw, Attnon);
   
   return;
}



/*
   FUNCTION: 		CancelAtts

   Synopsis: 
       Cancel the entering of this symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CancelAtts(w, panel, call_data)

Widget
   w,
   panel;        /* The widget and its parent using this callback */
caddr_t
   call_data;    /* Not used */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   
   XtFree(Tree->symbol);
   Tree->symbol = "";
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, ""); cnt++;
   if (Tree->nodew != NULL)
   {
      XtSetValues(Tree->nodew, args, cnt);
      XtSetKeyboardFocus(Frame, Tree->nodew);
   }
   XtPopdown(panel);
   XtSetSensitive(Tree->nodew, True);

   return;
}



/*
   FUNCTION: 		PopupAttPanel

   Synopsis: 
       Prepare the panel to be popped up in a certain location and
       with certain attribute settings (Terminal insensitive, for
       example), and pop it up.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PopupAttPanel(symset, termflag)

Attribute
   symset;
Boolean
   termflag;

{
   Dimension
      height,
      width1,
      width2;
   Position
      popx,
      popy;
   Widget
      attribform,
      attribshell;
   
   attribshell = XtNameToWidget(Toplevel, "*Attributes");
   attribform  = XtNameToWidget(Toplevel, "*attribform");
   XtSetKeyboardFocus(Frame, attribform);
   if ((attribshell == NULL) || (Tree == NULL) || (attribform == NULL))
   {
      XicaStatus(Statw,
		 "PopupAttPanel: Cannot get attribute or tree handle\n");
      return;
   }
	 
   XtSetSensitive(Tree->nodew, False);

   XtVaGetValues( XtNameToWidget(Toplevel, "*nodeview"),
       XtNheight, &height,
       XtNwidth,  &width1,
       NULL );
	 
   XtVaGetValues( attribform, XtNwidth, &width2, NULL );
   
   XtTranslateCoords(XtNameToWidget(Toplevel, "*nodeview"),
		     0, 0, &popx, &popy);
   
   XtVaSetValues( attribshell,
      XtNx, popx + ((width1 - width2)/2),
      XtNy, popy + height - 10,
      NULL );

   PutLabel(XtNameToWidget(attribshell, "*attriblbl"), Tree->symbol);
   XawToggleSetCurrent(XtNameToWidget(attribshell, "*Nonterminal"), symset);
   XawToggleSetCurrent(XtNameToWidget(attribshell, "*Exact"), Attexact);
   XtSetSensitive(XtNameToWidget(attribshell, "*Terminal"), termflag);
   XtInstallAllAccelerators(attribform, attribform);
   XtPopup(attribshell, XtGrabExclusive);
   return;
}



/*
   FUNCTION: 		CreateAttribShell

   Synopsis: 
       Create the shell widgets for symbol attributes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	
   Modifications:
       WHO       DATE    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	void

   Range:		none
*/

void
CreateAttribShell(parent)

Widget
   parent;         /* The parent widget of the attribute shells */

{
   static String
      appacctab  = "<Key>Return:    set() notify() reset()\n\
                    <Key>KP_Enter:  set() notify() reset()\n\
                    <Key>Linefeed:  set() notify() reset()",
      canacctab  = "<Key>Delete:    set() notify() reset()\n\
                    <Key>BackSpace: set() notify() reset()",
      nonacctab  = "<Key>n:	unset() set() notify()",
      termacctab = "<Key>t:	unset() set() notify()",
      grpacctab  = "<Key>g:	unset() set() notify()",
      fileacctab = "<Key>f:	unset() set() notify()",
      /*      Occurrence      */
      exacctab   = "<Key>_:	unset() set() notify()\n\
		    <Key>-:	unset() set() notify()",
      optacctab  = "<Key>?:	unset() set() notify()\n\
		    <Key>/:	unset() set() notify()",
      plusacctab = "<Key>+:	unset() set() notify()\n\
		    <Key>=:	unset() set() notify()",
      staracctab = "<Key>*:	unset() set() notify()\n\
		    <Key>8:	unset() set() notify()";
   Widget
      attribshell,
      attribform, attlbls, attsymocc, attbtns,
      applyw,
      attriblbl, attlblttl,
      cancelw,
      defaultsw,
      exactsel,
      exactw,
      filenamew,
      groupw,
      nontermw,
      occurform,
      occurlbl,
      occurmenu,
      optsel,
      optionalw,
      plussel,
      plusw,
      starsel,
      starw,
      termw,
      symbolform,
      symbollbl;
   XtAccelerators
      appaccels,
      canaccels,
      nonaccels,
      termaccels,
      grpaccels,
      fileaccels,
      exaccels,
      optaccels,
      staraccels,
      plusaccels;
   XtTranslations 
      toggletrans;
   static String
      toggletransstr = "<EnterWindow>:	     highlight(Always)\n\
			<LeaveWindow>:	     unhighlight()\n\
			<Btn1Down>,<Btn1Up>: set() notify()";
   
   toggletrans = XtParseTranslationTable(toggletransstr);
   
   appaccels  = XtParseAcceleratorTable(appacctab);
   canaccels  = XtParseAcceleratorTable(canacctab);
   nonaccels  = XtParseAcceleratorTable(nonacctab);
   termaccels = XtParseAcceleratorTable(termacctab);
   grpaccels  = XtParseAcceleratorTable(grpacctab);
   fileaccels = XtParseAcceleratorTable(fileacctab);
   exaccels   = XtParseAcceleratorTable(exacctab);
   optaccels  = XtParseAcceleratorTable(optacctab);
   staraccels = XtParseAcceleratorTable(staracctab);
   plusaccels = XtParseAcceleratorTable(plusacctab);
   
   occurmenu = XtVaCreatePopupShell( "occurmenu",
       simpleMenuWidgetClass, parent,
       NULL );

   exactsel = XtVaCreateManagedWidget( "exactsel",
       smeBSBObjectClass, occurmenu,
       XtNlabel, "Exactly once",
       NULL );
   XtAddCallback(exactsel, XtNcallback, OccurSel, Attexact); 

   optsel = XtVaCreateManagedWidget( "optsel",
       smeBSBObjectClass, occurmenu,
       XtNlabel, "Zero or once",
       NULL );
   XtAddCallback(optsel, XtNcallback, OccurSel, Attopt); 

   starsel = XtVaCreateManagedWidget( "starsel",
       smeBSBObjectClass, occurmenu,
       XtNlabel, "Zero or more",
       NULL );
   XtAddCallback(starsel, XtNcallback, OccurSel, Attstar); 

   plussel = XtVaCreateManagedWidget( "plussel",
       smeBSBObjectClass, occurmenu,
       XtNlabel, "One or more",
       NULL );
   XtAddCallback(plussel, XtNcallback, OccurSel, Attplus); 

   /*
    */
   attribshell = XtVaCreatePopupShell( "Attributes",
       transientShellWidgetClass, parent,
       XtNallowShellResize, True,
       NULL );

   attribform = XtVaCreateManagedWidget( "attribform",
       groupWidgetClass, attribshell,
       XtNspacing, 6,
       NULL );

   attlbls = XtVaCreateManagedWidget( "attlbls",
       groupWidgetClass, attribform,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     0,
       XtNmargin,      6,
       NULL );

   attlblttl = XtVaCreateManagedWidget( "attlblttl",
       labelWidgetClass, attlbls,
       XtNborderWidth, 0,
       XtNlabel,       "Attributes for:",
       XtNmargin,      0,
       NULL );
   
   attriblbl = XtVaCreateManagedWidget( "attriblbl",
       labelWidgetClass, attlbls,
       XtNborderWidth, 0,
       XtNlabel,       "Symbol",
       XtNmargin,      0,
       NULL );
   

   attsymocc = XtVaCreateManagedWidget( "attsymocc",
       groupWidgetClass, attribform,
       XtNorientation, XtorientHorizontal,
       XtNalignment,   gaFill,
       XtNspacing,     6,
       XtNmargin,      0,
       NULL );

   symbolform = XtVaCreateManagedWidget( "symbolform",
       groupWidgetClass, attsymocc,
       XtNmargin,      0,
       XtNspacing,     4,					
       NULL );

   symbollbl = XtVaCreateManagedWidget( "symbollbl",
       labelWidgetClass, symbolform,
       XtNborderWidth, 0,
       XtNlabel,       "Choose a symbol type:",
       XtNmargin,      0,
       NULL );
   
   nontermw = XtVaCreateManagedWidget( "Nonterminal",
       toggleWidgetClass, symbolform,
       XtNaccelerators, nonaccels,
       XtNradioGroup,   NULL,
       XtNradioData,    Attnon,
       XtNstate,        True,
       XtNlabel,        "Nonterminal",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(nontermw, toggletrans);
   
   termw = XtVaCreateManagedWidget( "Terminal",
       toggleWidgetClass, symbolform,
       XtNaccelerators, termaccels,
       XtNradioGroup,   nontermw,
       XtNradioData,    Attterm,
       XtNlabel,        "Terminal",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(termw, toggletrans);

   groupw = XtVaCreateManagedWidget( "Grouping",
       toggleWidgetClass, symbolform,
       XtNaccelerators, grpaccels,
       XtNradioGroup,   nontermw,
       XtNradioData,    Attgrp,
       XtNlabel,        "Grouping",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(groupw, toggletrans);

   filenamew = XtVaCreateManagedWidget( "Filename",
       toggleWidgetClass, symbolform,
       XtNaccelerators, fileaccels,
       XtNradioGroup,   nontermw,
       XtNradioData,    Attfile,
       XtNlabel,        "Filename",
       XtNsensitive,    False,
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(filenamew, toggletrans);


   occurform = XtVaCreateManagedWidget( "occurform",
       groupWidgetClass, attsymocc,
       XtNmargin,      0,
       XtNspacing,     4,					
       NULL );

   occurlbl = XtVaCreateManagedWidget( "occurlbl",
       labelWidgetClass, occurform,
       XtNborderWidth, 0,
       XtNlabel,       "Choose an occurrence:",
       XtNmargin,      4,
       NULL );

   exactw = XtVaCreateManagedWidget( "Exact",
       toggleWidgetClass, occurform,
       XtNaccelerators, exaccels,
       XtNradioGroup,   NULL,
       XtNradioData,    Attexact,
       XtNstate,        True,
       XtNlabel,        "Exactly once",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(exactw, toggletrans);
   
   optionalw = XtVaCreateManagedWidget( "Optional",
       toggleWidgetClass, occurform,
       XtNfromVert,     exactw,
       XtNaccelerators, optaccels,
       XtNradioGroup,   exactw,
       XtNradioData,    Attopt,
       XtNlabel,        "Zero or once",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(optionalw, toggletrans);

   starw = XtVaCreateManagedWidget( "Star",
       toggleWidgetClass, occurform,
       XtNfromVert,     optionalw,
       XtNaccelerators, staraccels,
       XtNradioGroup,   exactw,
       XtNradioData,    Attstar,
       XtNlabel,        "Zero or more",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(starw, toggletrans);

   plusw = XtVaCreateManagedWidget( "Plus",
       toggleWidgetClass, occurform, 
       XtNfromVert,     starw,
       XtNaccelerators, plusaccels,
       XtNradioGroup,   exactw,
       XtNradioData,    Attplus,
       XtNlabel,        "One or more",
       XtNalignment,    gaFill,
       XtNmargin,       0,
       NULL );
   XtOverrideTranslations(plusw, toggletrans);

   attbtns = XtVaCreateManagedWidget( "attbtns",
       groupWidgetClass, attribform,
       XtNorientation,  XtorientHorizontal,
       XtNalignment,    gaCenter,
       XtNspacing,      8,
       XtNmargin,       8,
       NULL );
   
   applyw = XtVaCreateManagedWidget( "Apply",
       commandWidgetClass, attbtns,
       XtNaccelerators,  appaccels,
       XtNborderWidth,   2,
       XtNlabel,         "Apply",
       XtNalignment,     gaCenter,
       XtNmargin,        4,
       NULL );
   XtAddCallback(applyw, XtNcallback, ApplyAtts, attribshell);
   
   defaultsw = XtVaCreateManagedWidget( "Defaults",
       commandWidgetClass, attbtns,
       XtNlabel,         "Defaults",
       XtNalignment,     gaCenter,
       XtNmargin,        4,
       NULL );
   XtAddCallback(defaultsw, XtNcallback, DefaultAtts, attribform);

   cancelw = XtVaCreateManagedWidget( "Cancel",
       commandWidgetClass, attbtns,
       XtNaccelerators,  canaccels,
       XtNlabel,         "Cancel",
       XtNalignment,     gaCenter,
       XtNmargin,        4,
       NULL );
   XtAddCallback(cancelw, XtNcallback, CancelAtts, attribshell);
   
   XtRealizeWidget(attribshell);

   XtInstallAllAccelerators(attribform, attribform);
   
   return;
}
