/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: compass.c,v $
   Purpose:
       To navigate using a compass.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/05/29 14:45:46 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  UpdateCompass
					   GetFirstChild
					   IsFirstNode
					   IsLastNode
                                           IsRoot
                                           PutLabel
   HIDDEN  CompassMove
                                           UpdateInterface
   EXPORT  CreateCompass
                                           GetWidgetHeightOffset
*/

/* HISTORY
   $Log:	compass.c,v $
 * Revision 1.5  92/05/29  14:45:46  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.4  92/03/18  14:14:56  ware
 * Working on consistent interface.
 * 
 * Revision 1.3  1992/01/14  17:47:02  cso
 *  - Reorganizing header files.
 *  - Added an action for keypad accelerators.
 *  -
 *
 * Revision 1.2  1991/11/20  00:02:12  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:15:33  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:47:05  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.10  91/03/28  14:36:04  cso
 * Removed the accelerator stuff, so it is up to the user to have a resource
 * file for the application to take advantage of it.
 * 
 * 
 * Revision 1.9  91/02/21  09:54:16  cso
 * Completed documentation.
 * Added ifdefs instead of comments around path calls.
 * 
 * Revision 1.8  90/11/20  15:21:11  cso
 * Added a way to adjust the height offset of the west widget so that the 
 * compass will look correct no matter what size font is used.
 * 
 * Revision 1.7  90/11/05  17:58:37  cso
 * Added function template to all functions, but still need to fill it in.
 * Used XtNameToWidget as little as possible to speed things up.  A
 * added characters to CompassMove to speed that up.
 * Else cosmetic changes.
 * 
 * Revision 1.6  90/10/24  15:33:16  cso
 * Changed the order of UpdateInterface.
 * 
 * Revision 1.5  90/09/03  18:23:28  cso
 * Function name changes.
 * 
 * Revision 1.4  90/09/03  14:28:54  cso
 * 
 * Also, use_button has been replaced by PutLabel.
 * 
 * Revision 1.3  90/08/26  14:13:07  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "main.h"
#include "node.h"
#include "put.h"
#include "status.h"   
#include "utils.h"



/*
   FUNCTION: 		UpdateCompass

   Synopsis: 
       Update the focus and surrounding symbols in the compass.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Toplevel        To access other widgets.

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void 
UpdateCompass(nptr)

Node
   *nptr;     /* The new focus of the environment */

{
   Widget
      east,  /* The right widget */
      here,  /* The center, "You are here", widget */
      north, /* The top widget */
      south, /* The bottom widget */
      west;  /* The left widget */

   /*
    * Access each of the compass widgets
    */
   east  = XtNameToWidget(Toplevel, "*compassframe*east");
   here  = XtNameToWidget(Toplevel, "*compassframe*here");
   north = XtNameToWidget(Toplevel, "*compassframe*north");
   south = XtNameToWidget(Toplevel, "*compassframe*south");
   west  = XtNameToWidget(Toplevel, "*compassframe*west");
   /*
    * Put the symbol in focus into the middle
    */
   PutLabel( here, nptr->symbol );

   /*
    * For each direction around nptr place the correct
    * symbol into the surrounding widgets.  If No symbol exists
    * in a particular direction then make that widget insensitive.
    */

   /*
    * Update the north widget
    */
   if (IsRoot(nptr) == True)
   {
      PutLabel(north, "");
      XtSetSensitive(north, False);
   }
   else
   {
      PutLabel(north, nptr->parent->symbol);
      XtSetSensitive(north, True);
   }

   /*
    * Update the west widget
    */
   if (IsFirstNode(nptr) == True)
   {
      PutLabel(west, "");
      XtSetSensitive(west, False);
   }
   else
   {
      PutLabel(west, nptr->prev->symbol);
      XtSetSensitive(west, True);
   }

   /*
    * Update the east widget
    */
   if (IsLastNode(nptr) == True)
   {
      PutLabel(east, "");
      XtSetSensitive(east, False);
   }
   else
   {
      PutLabel(east, nptr->next->symbol);
      XtSetSensitive(east, True);
   }

   /*
    * Update the south widget
    */
   nptr = GetFirstChild(nptr);
   if (nptr == NULL)
   {
      PutLabel(south, "");
      XtSetSensitive(south, False);
   }
   else
   {
      PutLabel(south, nptr->symbol);
      XtSetSensitive(south, True);
   }

   return;
}




/*
   FUNCTION: 		CompassMove

   Synopsis: 
       Callback routine for the compass buttons to move in a direction.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Tree              To access the current pointer location

   Globals Modified:
       Tree              If possible move in the direction requested

   Return value:	void

   Range:		none
*/

static void
CompassMove(w, direction, call_data)

Widget
   w;           /* The widget calling this function */
char
   direction;   /* The direction corresponding to the widget, but it saves
		   having to call XtNameToWidget */
caddr_t
   call_data;   /* Unused */

{
   Node
      *oldptr;  /* The current position of Tree */


   /*
    * Get the current position of Tree
    */
   oldptr = Tree;

   /*
    * Depending on the direction, check, move, and update the interface
    */
   switch (direction)
   {
   default:
      XicaStatus(Statw, "CompassMove: bad direction value [%c]\n", direction);
      break;

   case 'e':
      if (IsLastNode(Tree) == False)
      {
	 Tree = Tree->next;
      }
      break;

   case 'n':
      if (IsRoot(Tree) == False)
      {
#ifdef PATH
	 Tree = pop();
#endif
	 Tree = Tree->parent;
      }
      break;

   case 's':
      if (GetFirstChild(Tree) != NULL)
      {
#ifdef PATH
	 push(Tree);
#endif
	 Tree = GetFirstChild(Tree);
      }
      break;

   case 'w':
      if (IsFirstNode(Tree) == False)
      {
	 Tree = Tree->prev;
      }
      break;
   }
   UpdateInterface(oldptr, Tree);
   return;
}




/*
   FUNCTION: 		CompassAction

   Synopsis: 
       A way to have wider keyboard focus and still manipulate the
       compass.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CompassAction(w, event, params, num)

Widget
   w;          /* The widget calling this procedure */
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   switch (**params)
   {
   default:
      break;
      
   case 'n':
      XtCallCallbacks(XtNameToWidget(Toplevel, "*north"),
		      XtNcallback, NULL);
      break;
      
   case 's':
      XtCallCallbacks(XtNameToWidget(Toplevel, "*south"),
		      XtNcallback, NULL);
      break;
      
   case 'w':
      XtCallCallbacks(XtNameToWidget(Toplevel, "*west"),
		      XtNcallback, NULL);
      break;
      
   case 'e':
      XtCallCallbacks(XtNameToWidget(Toplevel, "*east"),
		      XtNcallback, NULL);
      break;
   }
   return;
}



/*
   FUNCTION: 		CreateCompass

   Synopsis: 
       Create the interface layout of a compass.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO	DATE     REASON
       cso	91-03-28 Removed accelerator info.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget CreateCompass( parent )
Widget
   parent;            /* The parent widget */
{
   int
      height_offset;  /* Offset from the top edge to place the west widget */
   Widget
      compass,
      compassframe,
      left, center, right,  /* table columns */
      top, middle, bottom,  /* table rows */
      east,           /* East widget */
      here,           /* Focus widget */
      label,	      /* Label for the compass */
      north,          /* North widget */
      south,          /* South widget */
      west;           /* West widget */

   compass = XtVaCreateManagedWidget( "compass",
       groupWidgetClass, parent,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       XtNspacing,     0,
       NULL );

   label  = XtVaCreateManagedWidget( "compasslabel",
       labelWidgetClass, compass,
       XtNlabel,       "Compass",
       XtNborderWidth, 0,
       XtNmargin,      0,
       NULL );

   compassframe = XtVaCreateManagedWidget( "compassframe",
       tableWidgetClass, compass,
       XtNalignment,   gaFill,
       XtNmargin,      0,
       XtNhorzMargin,  0,
       XtNvertMargin,  0,
       XtNhorzSpacing, 6,
       XtNvertSpacing, 4,
       XtNhorzPlacement, tpCenter,
       NULL );
   left = XtVaCreateWidget( "left",
       tableColObjectClass, compassframe,
       NULL );
   center = XtVaCreateWidget( "center",
       tableColObjectClass, compassframe,
       NULL );
   right = XtVaCreateWidget( "right",
       tableColObjectClass, compassframe,
       NULL );
   top = XtVaCreateWidget( "top",
       tableRowObjectClass, compassframe,
       NULL );
   middle = XtVaCreateWidget( "middle",
       tableRowObjectClass, compassframe,
       NULL );
   bottom = XtVaCreateWidget( "bottom",
       tableRowObjectClass, compassframe,
       NULL );

   west = XtVaCreateManagedWidget( "west",
       commandWidgetClass, compassframe,
       XtNlabel, "",
       XtNtableRow, middle,
       XtNtableCol, left,
       XtNvertAlignment, taCenter,
       XtNhorzAlignment, taBottomOrRight,
       NULL );
   XtAddCallback( west, XtNcallback, CompassMove, 'w' );
   
   north = XtVaCreateManagedWidget( "north",
       commandWidgetClass, compassframe,
       XtNlabel, "",
       XtNtableRow, top,
       XtNtableCol, center,
       XtNvertAlignment, taCenter,
       XtNhorzAlignment, taCenter,
       NULL );
   XtAddCallback( north, XtNcallback, CompassMove, 'n' );

   here = XtVaCreateManagedWidget( "here",
       labelWidgetClass, compassframe,
       XtNlabel, "",
       XtNtableRow, middle,
       XtNtableCol, center,
       XtNvertAlignment, taCenter,
       XtNhorzAlignment, taFill,
       NULL );
   
   south = XtVaCreateManagedWidget( "south",
       commandWidgetClass, compassframe,
       XtNlabel, "",
       XtNtableRow, bottom,
       XtNtableCol, center,
       XtNvertAlignment, taCenter,
       XtNhorzAlignment, taCenter,
       NULL );
   XtAddCallback( south, XtNcallback, CompassMove, 's' );
   
   east = XtVaCreateManagedWidget( "east",
       commandWidgetClass, compassframe,
       XtNlabel, "",
       XtNtableRow, middle,
       XtNtableCol, right,
       XtNvertAlignment, taCenter,
       XtNhorzAlignment, taTopOrLeft,
       NULL );
   XtAddCallback( east, XtNcallback, CompassMove, 'e' );

   return(compass);
}
