/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: display.c,v $
   Purpose:     
       This module handles all of the display routines for the tree
       interface.  Most of the original functions conatined in here
       were from module, tree.c, so look there for a log of changes.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.13 $
   Date:        $Date: 92/03/18 14:15:14 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  WipeWindow

   EXPORT  HideDisplayTree
                           HideDisplayTree
                                           GetExposedAlt
   EXPORT  ShowDisplayTree
                           ShowDisplayTree
                                           GetExposedAlt
   EXPORT  DrawDogLegs

   HIDDEN  PositionParentW
                                           GetDimensions
   HIDDEN  PositionLeafW
                                           GetDimensions
   HIDDEN  DetermineFromW
                                           GetDimensions
   HIDDEN  HighlightDepth

   EXPORT  PrettyPrintTree
                           DetermineFromW
			   DrawDogLegs
			   HighlightDepth
			   PositionLeafW
			   PositionParentW
			   PrettyPrintTree
			   WipeWindow
					   GetNextNode
					   IsRoot
   HIDDEN  HighlightPath
                                           GetDefinition
   HIDDEN  UpdateAltsDisplayed
                           HideDisplayTree
			   PrettyPrintTree
			   ShowDisplayTree
                                           FindAlt
   EXPORT  UpdateTree
			   HighlightPath
			   UpdateAltsDisplayed
   EXPORT  PutSymbolInTree
                           PrettyPrintTree
			                   CreateTreeW
					   PutLabel
   EXPORT  PrepareDisplayTree
                           PrepareDisplayTree
   EXPORT  HighlightDefinedStatus
                                           GetIndex
*/

/* HISTORY
   $Log:	display.c,v $
 * Revision 1.13  92/03/18  14:15:14  ware
 * Working on consistent interface.
 * 
 * Revision 1.12  1992/01/28  14:49:21  cso
 * Still having problems when removing subtree items...  Tried delaying
 * the updates under the form widget until after prettyprinting.
 * Prettyprinting may still need work - something about correcting all
 * alternatives instead of just the displayed plane might help...
 * Added a function ZeroOutTree in an attempt to help to reset values -
 * no luck.
 *
 * Revision 1.11  1992/01/14  17:56:14  cso
 *  - Reorganizing header files.
 *
 * Revision 1.10  1991/11/25  14:36:33  cso
 * Made lint modifications to ensure that function and variable types are
 * being used and declared correctly and consistently.
 *
 * Revision 1.9  1991/11/22  23:35:00  cso
 * Finally, have the display working as expected, perhaps a bit quicker,
 * but there is still a small amount of flicker on the redraws.  The
 * fixes included some small problems when removing empty place holders
 * and complete alternatives.
 *
 * Revision 1.8  1991/11/22  17:01:42  cso
 * Fixed some of the multiple blinking when redrawing.
 *
 * Revision 1.4  1991/11/07  12:05:25  cso
 * Fixed Bad Window error!!!!  Combination of two functions causing the
 * problem.  In HighlightDefinedStatus, I was always managing the tree
 * widget after modifying its border width - now I check to see if it was
 * already managed before managing it after the change.  In DefineSym,
 * the call to MakeSubtree for the symbol being defined was creating a
 * new alt with exposed set to true and the Tree ptr being bumped down to
 * that alt, but the interface hadn't been updated to get to the correct
 * plane yet - now if the planes of the tree aren't the same, I hide the
 * old plane first update the oldptr and then bump Tree ptr along.
 *
 * Revision 1.3  1991/11/06  16:11:42  cso
 * Initialized uninitialized variables found by QA_C...
 *
 * Revision 1.2  1991/10/17  18:30:28  cso
 * Added a check to make sure that a widget was managed before reshaping
 * it.  Is this the right thing???
 *
 * Revision 1.1  1991/10/04  19:05:23  jbm
 * Initial revision
 *
 * Revision 2.2  91/05/28  09:59:11  cso
 * Cosmetics.
 * 
 * 
 * Revision 2.1  91/04/01  10:53:56  cso
 * Attempted to speed up the prettyprinting only those widgets that needed
 * repositioning are actually (un)managed.
 * 
 * Revision 2.0  91/03/30  13:47:33  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.7  91/03/30  13:28:13  cso
 * Added a check to see if GetDefinition returns a NULL pointer.
 * 
 * Revision 1.6  91/03/28  14:38:06  cso
 * Corrected the exposure of the different planes of a grammar.
 * Made sure that all instances of a symbol having more than one 
 * definition will be highlighted appropriately.
 * Moved the creation of Threed to CreateInterface.
 * Only Highlight the depth of nonNULL nodes.
 * Fixed UpdateAltsDisplayed
 * Fixed PrepareDisplayTree to set the exposed flag correctly.
 * 
 * Revision 1.5  91/02/26  15:17:28  cso
 * Positionparentw: removed old_x and old_y
 * 		    added leftedge and topedge for getting new
 * 		    position values for placing the parent
 * Prettyuprinttree: New philosophy in positioning tree widgets
 * 	Don't use fromHoriz/fromVert resources (except for 
 * 	relative position of parent-child) for siblings.
 * 	Thus use the x/y and horiz/vertdistance resources.
 * 	Using leftedge and topedge for that minimal edge of the span
 * 	over which to place the parent.
 * Positionleafw:  Always place the leaf defaultspace away from from_widget
 * 	Again don't use the fromHoriz/Vert resource
 * 
 * Revision 1.4  91/02/22  10:27:36  cso
 * Completed documentation.
 * Changed PositionParentW to working only with widgets.
 * Changed the calculation of the span and the parent's position.
 * Added a function to highlight a widget as having depth.
 * Trying to update PrettyPrintTree.
 * Modified HighlightPath to do that (not just one widget).
 * 
 * Revision 1.3  90/11/29  14:22:13  cso
 * Moved GetDimensions to get.c
 * 
 * Revision 1.2  90/11/26  13:11:05  cso
 * lint modifications.
 * 
 * Revision 1.1  90/11/20  15:23:05  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "get.h"
#include "main.h"
#include "node.h"
#include "put.h"
#include "status.h"
#include "tree.h"
#include "utils.h"



/*
   FUNCTION: 		WipeWindow

   Synopsis: 
       Wipe the window displaying the tree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Treeshell         Get access to the widget to wipe

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
WipeWindow()

{
   Widget
      w;   /* Widget to wipe */

   /*
    * Get the widget that is going to be cleared, clear it, and return
    */
   w = XtNameToWidget(Treeshell, "*treeframe");
   XClearWindow(XtDisplay(w), XtWindow(w));
   return;
}




/*
   FUNCTION: 		HideDisplayTree

   Synopsis: 
       From the initial node down the subtree, hide that subtree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
HideDisplayTree(nptr)

Node
   *nptr;     /* Pointer whose tree widget will be hidden and whose 
		 children's widgets will be hidden too */

{
   Alt
      *aptr;  /* The alternative for nptr that is exposed */
   Node
      *qptr;  /* Pointer to hide the children of nptr */

   /*
    * Check if nptr is NULL
    */
   if (nptr != NULL)
   {
      /*
       * If nptr has a tree widget, hide it
       * Get the alternative for nptr that is exposed
       * If the alternative is not NULL, hide the children
       */
      if (nptr->treew != NULL)
      {
	 XtUnmanageChild(nptr->treew);
      }
      aptr = GetExposedAlt(nptr);
      if (aptr != NULL)
      {
	 /*
	  * The alternative will no longer be exposed
	  * For each node in the alternative, hide it and its subtree
	  */
	 aptr->exposed = False;
	 for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	 {
	    HideDisplayTree(qptr);
	 }
      }
   }
   return;
}




/*
   FUNCTION: 		DrawDogLegs

   Synopsis: 
       Draw a dogleg from a parent widget to one its children.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DrawDogLegs(nptr, gc, vertical)

Node
   *nptr;            /* The child  whose dogleg is being drawn */
GC
   gc;		     /* The graphics context to use for drawing */
Boolean 
   vertical;         /* The direction of the dogleg */

{
   Arg
      args[MAXARGS];      /* List of arguments */
   Dimension
      parent_width,
      parent_height, /* parent's width and height */
      child_width,
      child_height;  /* child's width and height */
   int
      cnt;           /* Number of arguments */
   Position
      parent_x,
      parent_y,      /* parent's x and y values */
      child_x,
      child_y,       /* child's x and y values */
      start_x,
      start_y,       /* beginning of line */
      end_x,
      end_y;         /* ending of line */
   Widget
      canvas;
   

   if ((nptr == NULL) || (nptr->parent == NULL))
   {
      return;
   }
   
   canvas = XtParent(nptr->treew);
   
   /*
    * Get dimensions of parent's tree widget 
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNx,      &parent_x);      cnt++;
   XtSetArg(args[cnt], XtNy,      &parent_y);      cnt++;
   XtSetArg(args[cnt], XtNwidth,  &parent_width);  cnt++;
   XtSetArg(args[cnt], XtNheight, &parent_height); cnt++; 
   XtGetValues(nptr->parent->treew, args, cnt);
   

   /*
    * Get dimensions of child's tree widget
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNx,      &child_x);      cnt++; 
   XtSetArg(args[cnt], XtNy,      &child_y);      cnt++;  
   XtSetArg(args[cnt], XtNwidth,  &child_width);  cnt++;
   XtSetArg(args[cnt], XtNheight, &child_height); cnt++;
   XtGetValues(nptr->treew, args, cnt);
   

   /*
    * Determine the direction and draw the three lines comprising the dogleg
    */
   if (vertical == True)
   {

      /*
       * For vertical orientation:
       *     | parent |
       *     ----------
       * line1 -> |
       * line2 -> -----
       * line3 ->     |
       *          ---------
       *          | child |
       */
      start_x = parent_x + (parent_width / 2);
      start_y = parent_y + parent_height;
      end_x   = start_x;
      end_y   = start_y + ((child_y - start_y) / 2);
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
      
      /* start_x stays the same */
      start_y = end_y;
      end_x   = child_x + (child_width / 2);
      /* end_y stays the same */
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
      
      start_x = end_x;
      /* start_y stays the same */
      /* end_x stays the same */
      end_y   = child_y;
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
   }
   else
   {
      /*
       * For horizontal orientation:
       *                 ---------
       *              |--| child |
       *              |  ---------
       *  ----------  |
       *  | parent |--
       *  ----------
       *            ^ ^ ^
       *      lines 1 2 3
       */
      start_x = parent_x + parent_width;
      start_y = parent_y + (parent_height / 2);
      end_x   = start_x + ((child_x - start_x) / 2);
      end_y   = start_y;
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
      
      start_x = end_x;
      /* start_y stays the same */
      /* end_x stays the same */
      end_y   = child_y + (child_height / 2);
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
      
      /* start_x stays the same */
      start_y = end_y;
      end_x   = child_x;
      /* end_y stays the same */
      XDrawLine(XtDisplay(canvas), XtWindow(canvas), gc,
		start_x, start_y, end_x, end_y);
   }
   return;
}      




/*
   FUNCTION: 		ShowDisplayTree

   Synopsis: 
       Show a subtree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO	DATE     REASON
       cso	91-03-28 Added a check for a subtree that has not
       			 been exposed yet but exists.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ShowDisplayTree(nptr)

Node
   *nptr;     /* The pointer whose widget and subtree will be shown */

{
   Alt
      *aptr;  /* The alternative to expose */
   Node
      *qptr;  /* Pointer for each node in the alternative */


   /*
    * If nptr is not NULL, show its tree widget and its subtree
    */
   if (nptr != NULL)
   {

      /*
       * If nptr has a tree widget, show it
       * Get an alternative for it to show
       * If an alternative exists show that subtree
       */
      if (nptr->treew != NULL)
      {
	 XtManageChild(nptr->treew);
	 DrawDogLegs(nptr, Gc, True);
      }

      aptr = GetExposedAlt(nptr);
      if ((aptr == NULL) && (nptr->alts != NULL))
      {
	 /*
	  * If no exposed alternative is returned, but one exists
	  * Make the first alternative the one to expose.
	  */
	 aptr = nptr->alts;
	 aptr->exposed = True;
      }

      if (aptr != NULL)
      {
	 /*
	  * For each node in the alternative, show its widget and subtree
	  */
	 for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	 {
	    ShowDisplayTree(qptr);
	 }
      }
   }
   return;
}




/*
   FUNCTION: 		PositionParentW

   Synopsis: 
       Places the parent widget centered over its children or 
       on the left edge if its width is wider than the display 
       of its children.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/04/01   cso    Added a check if a modification needs to take
                          place before actually performing the
			  (Un)managing of a widget.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PositionParentW(parentw, min_w, max_w, vertical)

Widget
   parentw,   /* Parent widget being placed */
   min_w,     /* Widget to the left  (in vertical mode) */
   max_w;     /* Widget to the right (in vertical mode) */
Boolean
   vertical;  /* Orientation of display */

{
   Arg
      args[MAXARGS];      /* List of arguments */
   Boolean
      modified;      /* Flag to indicate a change in a widget's */
		     /* position */
   Dimension
      min_width,     /* First child's width  */
      min_height,    /* First child's height */
      max_width,     /* Last child's width   */
      max_height,    /* Last child's height  */
      parent_width,  /* Parent's width       */
      parent_height; /* Parent's height      */
   int
      cnt,           /* Number of arguments               */
      min_hdist,     /* First child's horizontal distance */
      min_vdist,     /* First child's vertical distance   */
      max_hdist,     /* Last child's horizontal distance  */
      max_vdist,     /* Last child's vertical distance    */
      parent_hdist,  /* Parent's horizontal distance      */
      parent_vdist;  /* Parent's vertical distance        */
   Position
      leftedge,      /* Left edge of the span to place the parent */
      min_x,         /* First child's x value    */
      min_y,         /* First child's y value    */
      max_x,         /* Last child's x value     */
      max_y,         /* Last child's y value     */
      parent_x,      /* Parent's new x value     */
      parent_y,      /* Parent's new y value     */
      span,          /* The span over which we try to position parentw */
      topedge;       /* Top edge of the span to place the parent */


   /*
    * Return if the parentw is NULL
    */
   if (parentw == NULL)
   {
      XicaStatus(Statw,
	      "WARNING: PositionParentW: parentw is NULL\n");
      return;
   }

   /*
    * Get the min widget's dimensions
    */
   GetDimensions(min_w, True,
		 &min_x,     &min_y,
		 &min_width, &min_height,
		 &min_hdist, &min_vdist);

   /*
    * Get the max widget's dimensions
    */
   GetDimensions(max_w, True,
		 &max_x,     &max_y,
		 &max_width, &max_height,
		 &max_hdist, &max_vdist);

   /*
    * Get the parent's dimensions
    */
   GetDimensions(parentw, False,
		 &parent_x,     &parent_y,
		 &parent_width, &parent_height,
		 &parent_hdist, &parent_vdist);

   /*
    * Initialize modified flag
    * Depending on orientation, calculate new coordinates of parent
    */
   modified = False;
   if (vertical == True)
   {
      /*
       * Find the span over which to put the parent widget
       * If parent's width is less than span of subtree widgets
       * over which it
       * will be placed, calculate that new position
       * Else set it to the first child's position
       */
      leftedge = min_x + min_width + DEFAULTSPACE;
      span     = max_x + max_width - leftedge;
      if (parent_width < span)
      {
	 parent_x = (Position) (span / 2) - (parent_width / 2) +
	    leftedge;
	 modified = True;
      }
      else if (parent_x != leftedge)
      {
	 parent_x = leftedge;
	 modified = True;
      }
      parent_hdist = parent_x;
   }
   else
   {
      /*
       * If parent's width is less than span of children over which it
       * will be placed, calculate that new position
       * Else set it to the first child's position
       */
      topedge = min_y + min_height + DEFAULTSPACE;
      span    = max_y + max_height - topedge;
      if (parent_height < span)
      {
	 parent_y = (Position) (span / 2) - (parent_height / 2) + topedge;
	 modified = True;
      }
      else if (parent_y != topedge)
      {
	 parent_y = topedge;
	 modified = True;
      }
      parent_vdist = parent_y;
   }

   /*
    * If a modification has taken place
    * Unmanage parent, reset arguments, set the values, manage, and flush
    */
   if (modified == True)
   {
      XtUnmanageChild(parentw);
      cnt = 0;
      XtSetArg(args[cnt], XtNx,             parent_x);     cnt++;
      XtSetArg(args[cnt], XtNy,             parent_y);     cnt++;
      XtSetArg(args[cnt], XtNhorizDistance, parent_hdist); cnt++;
      XtSetArg(args[cnt], XtNvertDistance,  parent_vdist); cnt++;
      XtSetValues(parentw, args, cnt);
      XtManageChild(parentw);
   }
   return;
}




/*
   FUNCTION: 		PositionLeafW

   Synopsis: 
   Position a leaf treew from the fromw which is either to the left or 
   above this treew.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/04/01   cso    Added a check if a modification needs to take
                          place before actually performing the
			  (Un)managing of a widget.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Widget

   Range:		Might be NULL
*/

static Widget
PositionLeafW(leafw, fromw, vertical)

Widget
   leafw,          /* A leaf widget in the tree */
   fromw;          /* The rightmost widget so far */
Boolean
   vertical;       /* Tree orientation */

{
   Arg
      args[MAXARGS];    /* List of arguments */
   Boolean
      modified;      /* Flag to indicate a change in a widget's */
		     /* position */
   Dimension
      from_width,  /* Width of from widget  */
      from_height, /* Height of from widget */
      leaf_width,  /* Width of leaf widget  */
      leaf_height; /* Height of leaf widget */
   int
      cnt,         /* Number of arguments                */
      from_hdist,  /* Horizontal distance of from widget */
      from_vdist,  /* Vertical distance of from widget   */
      leaf_hdist,  /* Horizontal distance of leaf widget */
      leaf_vdist;  /* Vertical distance of leaf widget   */
   Position
      from_x,      /* X coordinate of from widget */
      from_y,      /* Y coordinate of from widget */
      leaf_x,      /* X coordinate of leaf widget */
      leaf_y;      /* Y coordinate of leaf widget */


   /*
    * If either widget is NULL, return
    * Else get their dimensions and position the leaf widget
    */
   if (leafw != NULL)
   {
      /*
       * Get from widget dimensions
       */
      GetDimensions(fromw,       True,
		    &from_x,     &from_y,
		    &from_width, &from_height,
		    &from_hdist, &from_vdist);

      /*
       * Get leaf widget dimensions
       */
      GetDimensions(leafw,       True,
		    &leaf_x,     &leaf_y,
		    &leaf_width, &leaf_height,
		    &leaf_hdist, &leaf_vdist);
      

      /*
       * Depending on orientation, position the leaf widget
       * However, first check if a change needs to take place
       */
      modified = False;
      if (vertical == True)
      {
	 if (leaf_x != (from_x + from_width + DEFAULTSPACE))
	 {
	    leaf_x     = from_x + from_width + DEFAULTSPACE;
	    leaf_hdist = leaf_x;
	    modified   = True;
	 }
      }
      else
      {
	 if (leaf_y != (from_y + from_height + DEFAULTSPACE))
	 {
	    leaf_y     = from_y + from_height + DEFAULTSPACE;
	    leaf_vdist = leaf_y;
	    modified   = True;
	 }
      }

      /*
       * If a modification took place
       * Unmanage leaf, set arguments and values, manage leaf and flush
       */
      if (modified == True)
      {
	 XtUnmanageChild(leafw);
	 cnt = 0;
	 XtSetArg(args[cnt], XtNx,             leaf_x);      cnt++;
	 XtSetArg(args[cnt], XtNy,             leaf_y);      cnt++;
	 XtSetArg(args[cnt], XtNhorizDistance, leaf_hdist);  cnt++;
	 XtSetArg(args[cnt], XtNvertDistance,  leaf_vdist);  cnt++;
	 XtSetValues(leafw, args, cnt);
	 XtManageChild(leafw);
      }
   }
   else
   {
      /*
       * In the case that a NULL symbol may be placed just return the 
       * present fromw.
       */
      return(fromw);
   }

   /*
    * Leaf widget becomes the from widget
    */
   return(leafw);
}




/*
   FUNCTION: 		DetermineFromW

   Synopsis: 
   Based on parent's position and the from_widget's position, determine
   which is further in the tree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Widget

   Range:		Might be NULL
*/

static Widget
DetermineFromW(nodew, fromw, vertical)

Widget
   nodew,    /* A node widget being compared against the current from widget */
   fromw;    /* The current from widget */
Boolean
   vertical; /* Tree orientation */

{
   Dimension
      from_width,  /* Width of from widget  */
      from_height, /* Height of from widget */
      node_width,  /* Width of node widget  */
      node_height; /* Height of node widget */
   int
      from_hdist,  /* Horizontal distance of from widget */
      from_vdist,  /* Vertical distance of from widget   */
      node_hdist,  /* Horizontal distance of node widget */
      node_vdist;  /* Vertical distance of node widget   */
   Position
      from_x,      /* X coordinate of from widget */
      from_y,      /* Y coordinate of from widget */
      node_x,      /* X coordinate of node widget */
      node_y;      /* Y coordinate of node widget */

   /*
    * If node widget is not NULL, determine which widget
    * is further to the right (or down) in the tree
    */
   if (nodew != NULL)
   {
      /*
       * Get the dimensions of the from widget
       */
      GetDimensions(fromw,       True,
		    &from_x,     &from_y,
		    &from_width, &from_height,
		    &from_hdist, &from_vdist);

      /*
       * Get the dimensions of the node widget
       */
      GetDimensions(nodew,       False,
		    &node_x,     &node_y,
		    &node_width, &node_height,
		    &node_hdist, &node_vdist);

      /*
       * Depending on the orientation, determine if the from widget
       * should remain the from widget, or if the node widget should 
       * become the from widget
       */
      if (vertical == True)
      {
	 if ((node_x + node_width) > (from_x + from_width))
	 {
	    fromw = nodew;
	 }
      }
      else
      {
	 if ((node_y + node_height) > (from_y + from_height))
	 {
	    fromw = nodew;
	 }
      }
   }
   return(fromw);
}




/*
   FUNCTION: 		HighlightDepth

   Synopsis: 
       Using a visual cue to show that a node has depth.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO    REASON
       91/01/16 cso    Change it so that PrettyPrintTree will use it.
       91/03/28 cso    Moved the Threed creation to interface.c
       		        Added the check to see if where the symbol is
		        actually defined has more than one definition,
		        but changes the present widget under consideration.
       91/03/29 cso    Added a check in case GetDefinition returns NULL.

   Globals used:
       Threed    To highlight a node

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
HighlightDepth(nptr)

Node
   *nptr;       /* Node under consideration */

{
   Arg
      args[MAXARGS];   /* List of arguments */
   int
      cnt;        /* Number of arguments */
   Widget
      w;          /* to make some of the calls easier */

   /*
    * If the node is not NULL, check its depth
    */
   if (nptr != NULL)
   {

      /*
       * assign the tree widget to w for subsequent calls
       */
      w = nptr->treew;

      /*
       * If a widget exists to highlight
       */
      if (w != NULL)
      {
	 /*
	  * Get the definition of this symbol and check to see if more
	  * than one definition exists.
	  * Unmanage the widget, (un)highlight, manage the widget
	  */
	 nptr = GetDefinition(nptr->symbol);
	 XtUnmanageChild(w);
	 if ((nptr != NULL) &&
	     (nptr->alts != NULL) &&
	     (nptr->alts->next != NULL))
	 {
	    cnt = 0;
	    XtSetArg(args[cnt], XtNbackgroundPixmap, Threed); cnt++;
	    XtSetValues(w, args, cnt);
	 }
	 else
	 {
	    cnt = 0;
	    XtSetArg(args[cnt], XtNbackgroundPixmap, XtUnspecifiedPixmap); 
	    cnt++;
	    XtSetValues(w, args, cnt);
	 }
	 XtManageChild(w);
      }
   }
   return;
}




/*
   FUNCTION: 		PrettyPrintTree

   Synopsis: 
       Display the tree using a postfix traversal,
       centering parent widgets over their span of children.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/28	  cso	 Only highlight the depth of a non NULL node.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Widget

   Range:               Might be NULL
*/

Widget
PrettyPrintTree(node, from_widget, vertical)

Node
   *node;       /* Prettyprinting the subtree under this node and/or
		   then this node */
Widget
   from_widget; /* current right-most widget in window */
Boolean
   vertical;    /* True = vertical tree, False = on its side */

{
   Alt
      *aptr;    /* Loop pointer to prettyprint all subtrees */
   Node
      *nptr;    /* Loop pointer to access each child under node */
   Widget
      child_fromw,  /* From widget returned from prettyprinting each subtree */
      max_fromw;    /* From widget furthest out in all subtrees */

   /*
    * If there is a node to prettyprint
    */
   if (node != NULL)
   {
      /*
       * If node is Root, wipe the "slate" clean
       * If node has children, prettyprint them then itself
       * Else node is a leaf
       */
      if (IsRoot(node) == True) 
      {
	 WipeWindow();
      }

      if (node->alts != NULL)
      {
	 /*
	  * For each child under node, prettyprint it 
	  * After the children are done, position the node
	  * Determine which should become the next from_widget
	  */
	 max_fromw = from_widget;
	 aptr = GetExposedAlt(node);
	 if (aptr != NULL)
	 {
	    child_fromw = from_widget;
	    for (nptr = aptr->prod; nptr != NULL; nptr = GetNextNode(nptr))
	    {
	       /*
		* Place all of the children first
		* Find the max from_widget
		*/
	       child_fromw = PrettyPrintTree(nptr, child_fromw, vertical);

	       max_fromw = DetermineFromW(child_fromw, max_fromw, vertical);
	    }

	    PositionParentW(node->treew, from_widget, max_fromw, vertical);
	    from_widget = DetermineFromW(node->treew, max_fromw, vertical);
	 }
      }
      else
      {
	 /*
	  * Position the leaf node
	  */
	 from_widget = PositionLeafW(node->treew, from_widget, vertical);
      }

      /*
       * (Un)Highlight a node's depth appropriately
       */
      HighlightDepth(node);
   }
   return(from_widget);
}




/*
   FUNCTION: 		HighlightPath

   Synopsis: 
       Indicate the path to a node by changing the shapes of the widgets.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
HighlightPath(nptr, highlight)

Node
   *nptr;     /* Node to which a path needs to be highlighted */
Boolean
   highlight; /* Highlight = True, Unhighlight = False  */

{
   Node
      *qptr;    /* Loop pointer to access the next node to highlight */
   

   /*
    * Beginning at the initial node go up the tree until the path
    * is (un)highlighted
    */
   if (nptr == NULL)
   {
      return;
   }

   for (qptr = nptr; qptr->parent != NULL; qptr = qptr->parent)
   {
      /*
       * If a widget exists to (un)highlight
       */
      if (qptr->treew != NULL)
      {
	 /*
	  * Highlight   using dashed  connecting lines
	  * Unhighlight using regular connecting lines
	  */
	 if (highlight == True)
	 {
	    qptr->treegc = Pathgc;
	 }
	 else
	 {
	    qptr->treegc = Gc;
	 }
	 if (XtIsManaged(qptr->treew) == True)
	 {
	    if (qptr->treegc == Pathgc)
	    {
	       DrawDogLegs(qptr, Gc, True);
	    }
	    DrawDogLegs(qptr, qptr->treegc, True);
	 }
      }
   }
   return;
}





/*
   FUNCTION: 		UpdateAltsDisplayed

   Synopsis: 
       Display the plane that contains newptr.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/28   cso    

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
UpdateAltsDisplayed(newptr)

Node
   *newptr;  /* The current focus of the interface */

{
   Alt
      *aptr, /* Alternative containing newptr */
      *bptr, /* Alternative presently exposed */
      *cptr; /* Alternative being exposed while searching for the */
	     /* alternative to hide */
   
   Node
      *nptr, /* Loop pointer to access nodes being hidden/shown */
      *optr; /* Node pointer in the alternative about to be exposed */
	     /* but is not currently exposed */

   /*
    * If the new node is not NULL and has a tree widget AND that tree
    * widget is not managed at present, that widget exists is an
    * unexposed alternative so update that situation
    */
   if ((newptr != NULL) && (newptr->treew != NULL) &&
       (XtIsManaged(newptr->treew) == False))
   {
      /*
       * Find the alternative containing newptr
       * If that alternative exists, newptr is not Root
       */
      aptr = FindAlt(newptr);
      if (aptr != NULL)
      {
	 /*
	  * If newptr's alternative is not exposed,
	  * hide the plane that is exposed and show this new plane
	  */
	 if (aptr->exposed != True)
	 {
	    /*
	     * Find the alternative that is exposed
	     * This is done by moving up the tree until an exposed
	     * alternative is found
	     * Initialize nptr to be the parent of newptr, and optr to
	     * be newptr.  It is possible that newptr's parent has an
	     * exposed alternative.
	     */
	    nptr = newptr->parent;
	    optr = newptr;
	    while ((bptr = GetExposedAlt(nptr)) == NULL)
	    {
	       /*
		* Find the alternative of the current hidden plane's
		* child.
		* Set that alternative to be exposed.
		* Remember the parent of where we are.
		* Go up another level in the tree.
		*/
	       cptr = FindAlt(optr);
	       cptr->exposed = True;
	       optr = nptr;
	       nptr = nptr->parent;
	    }

	    /*
	     * Hide the presently exposed plane
	     */
	    bptr->exposed = False;
	    for (nptr = bptr->prod; nptr != NULL; nptr = nptr->next)
	    {
	       HideDisplayTree(nptr);
	    }
	    
	    /*
	     * Expose the plane containing newptr
	     * Find the currently unexposed plane.
	     * Set it to be True.  Show that subtree.
	     */
	    cptr          = FindAlt(optr);
	    cptr->exposed = True;
	    for (nptr = cptr->prod; nptr != NULL; nptr = nptr->next)
	    {
	       ShowDisplayTree(nptr);
	    }
	 }
	    
	 /*
	  * Prettyprint the tree
	  */
	 XawFormDoLayout(XtNameToWidget(Toplevel, "*treeframe"),
			 False);
	 (void) PrettyPrintTree(Root, (Widget) NULL, True);
	 XawFormDoLayout(XtNameToWidget(Toplevel, "*treeframe"),
			 True);
      }
   }
   return;
}




/*
   FUNCTION: 		UpdateTree

   Synopsis: 
       Update the tree being displayed to the user.  This includes a possible
       change in the plane being viewed and visual cues as to depth and path.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/28   cso    UpdateAltsDisplayed only needs the new node.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
UpdateTree(oldptr, newptr)

Node
   *oldptr, /* The previous focus of the interface */
   *newptr; /* The current focus of the interface */

{
   /*
    * If the focus has changed, unhighlighting and plane changing may occur
    */
   if (newptr != oldptr)
   {

      /*
       * Unhighlight the path to the Root
       */
      HighlightPath(oldptr, False);

#ifdef ThreeD
      /*
       * Update the planes being viewed
       */
      UpdateAltsDisplayed(newptr);
#endif
   }

   /*
    * Highlight the path from newptr to the Root
    */
   HighlightPath(newptr, True);
   return;
}



/*
   FUNCTION: 		PutSymbolInTree

   Synopsis: 
       Place the symbol string into the tree, which may include creating 
       a widget to put the string.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutSymbolInTree(nptr, loadflag)

Node
   *nptr;    /* The node being placed into the tree */
Boolean
   loadflag; /* Is this occurring because a file is being loaded? */

{

   /*
    * If a widget doesn't exist, create one
    * Else place the symbol in the label of the widget
    */
   if (nptr->treew == NULL)
   {
      nptr->treew = CreateTreeW(nptr, True);
   }
   else
   {
      PutLabel(nptr->treew, nptr->symbol);
   }

   /*
    * If this is not a call from the loadfile procedure then take the time
    * to prettyprint the tree
    */
   if (loadflag == False)
   {
      (void) PrettyPrintTree(Root, (Widget) NULL, True);
   }
   return;
}




/*
   FUNCTION: 		PrepareDisplayTree

   Synopsis: 
       Initialize the first view that a user will see of the tree.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/28   cso    Changed the initial exposeflag assignment
       			 because whole alternatives may be hidden so
			 even the first alternative of a symbol may be
			 set to False.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PrepareDisplayTree(nptr, exposeflag)

Node
   *nptr;      /* Node under consideration */
Boolean
   exposeflag; /* Should this plane be exposed or not? */

{
   Node
      *qptr; /* Loop variable for accessing children of nptr */
   Alt
      *aptr; /* Loop variable for accessing all alternatives of nptr */


   /*
    * If the node exists and is not a leaf
    */
   if ((nptr != NULL) && (nptr->alts != NULL))
   {
      /*
       * Set the expose flag to the passed in value
       * Hide any other alternatives
       * If this plane is supposed to be shown, manage the tree widgets
       * Else make sure all children were unmanaged
       */
      nptr->alts->exposed = exposeflag;
      for (aptr = nptr->alts->next; aptr != NULL; aptr = aptr->next)
      {
	 aptr->exposed = False;
      }
      if (exposeflag == True)
      {
	 /*
	  * For each alternative, hide or show the children and 
	  * then call PrepareDisplayTree with each child with their
	  * expose mode
	  */
	 for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
	 {
	    for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	    {
	       if (qptr->treew != NULL)
	       {
		  if (aptr->exposed == True)
		  {
		     XtManageChild(qptr->treew);
		  }
		  else
		  {
		     XtUnmanageChild(qptr->treew);
		  }
	       }
	       PrepareDisplayTree(qptr, aptr->exposed);
	    }
	 }
      }
      else
      {
	 /*
	  * All of these children and so on are to be hidden
	  */
	 for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
	 {
	    for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	    {
	       if (qptr->treew != NULL)
	       {
		  XtUnmanageChild(qptr->treew);
	       }
	       PrepareDisplayTree(qptr, False);
	    }
	 }
      }
   }
   return;
}



/*
   FUNCTION: 		HighlightDefinedStatus

   Synopsis: 
       Highlight the nodes via borderwidth thickness to show if the
       symbol has been defined yet.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
       11/07/91 cso      When I was changing the highlighting,
                          I was managing each treew being modified
			  instead of checking to see if it was already
			  managed - added exposed flag to catch it.

   Globals used:
       Ndx	To check if the symbol exists in the symbol table.
       Info	To access the list of nodes where this symbol is being used.

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
HighlightDefinedStatus(str, defflag)

String
   str;      /* The symbol under consideration */
Boolean
   defflag;  /* The flag indicating the defined status */

{
   Arg
      args[MAXARGS];  /* Argument list */
   Boolean
      exposed;   /* Flag to indicate whether the widget is exposed - */
		 /* do NOT Manage an unexposed widget! */
   int
      cnt,       /* Count of arguments */
      i;         /* Index into the symbol table */
   Node
      *nptr;     /* A particular node for that symbol */
   NodeList
      *nlptr;    /* Pointer in the list of nodes for that symbol */
   
   /*
    * Get the index into the symbol table for that symbol
    * if the symbol exists (< Ndx), then ...
    */
   i = GetIndex(str);
   if (i < Ndx)
   {
      /*
       * For each node for this symbol
       * If it has a tree widget, modify its borderwidth
       *  according to the defined status: True:normal, False:thick
       */
      for (nlptr = Info[i].definition; nlptr != NULL; nlptr = nlptr->next)
      {
	 nptr = nlptr->nodeptr;
	 if ((nptr != NULL) && (nptr->treew != NULL))
	 {
	    cnt = 0;
	    if (defflag == True)
	    {
	       XtSetArg(args[cnt], XtNborderWidth, 1);
	    }
	    else
	    {
	       XtSetArg(args[cnt], XtNborderWidth, 3);
	    }
	    cnt++;
	    exposed = False;
	    if (XtIsManaged(nptr->treew) == True)
	    {
	       exposed = True;
	    }
	    XtUnmanageChild(nptr->treew);
	    XtSetValues(nptr->treew, args, cnt);
	    if (exposed == True)
	    {
	       XtManageChild(nptr->treew);
	    } /***** IF exposed widget is already exposed */
	 } /***** IF the pointer and the widget are not NULL */
      } /***** FOR each occurrence of the symbol in the tree structure */
   } /***** IF the symbol exists in the symbol table */
   return;
}



/*
   FUNCTION: 		ZeroOutTree

   Synopsis: 
       Zero out all of the x/y horiz/vert values so that when they are
       pretty printed these widgets will not be larger than the from
       widget that they are being set relative to.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ZeroOutTree(nptr)

Node
   *nptr;

{
   Alt
      *aptr;
   Arg
      args[MAXARGS];
   int
      cnt;
   Node
      *qptr;
   
   if (nptr == NULL)
   {
      return;
   }
   for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
   {
      for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
      {
	 if (qptr->treew != NULL)
	 {
	    cnt = 0;
	    XtSetArg(args[cnt], XtNx,             0);    cnt++;
	    XtSetArg(args[cnt], XtNy,             0);    cnt++;
	    XtSetArg(args[cnt], XtNhorizDistance, 0);    cnt++;
	    XtSetArg(args[cnt], XtNvertDistance,  0);    cnt++;
	    XtSetArg(args[cnt], XtNfromHoriz,     NULL); cnt++;
	    XtSetArg(args[cnt], XtNfromVert,      NULL); cnt++;
	    XtSetValues(qptr->treew, args, cnt);
	 }
	 ZeroOutTree(qptr);
      }
   }
   return;
}

	    
