/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: main.c,v $
   Purpose:
       The main program initializes, and loops through events.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/05/29 14:46:14 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  Initialize
                                           CreateNodeW
					   KillWidgets
					   NewNode
					   UpdateInterface
					   WipeWindow
   HIDDEN  InitSensitivityCursors

   EXPORT  main
			   Initialize
                           InitSensitivityCursors
			                   CreateInterface
					   InstallSignalHandlers
*/

/* HISTORY
   $Log:	main.c,v $
 * Revision 1.5  92/05/29  14:46:14  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.4  92/03/18  14:16:22  ware
 * Working on consistent interface.
 * 
 * Revision 1.3  1992/01/14  20:17:20  cso
 *  - Reorganizing header files.  So globals are declared here.
 *  - Trying to keep the attribute panel popped up (even with autoraise).
 *
 * Revision 1.2  1991/11/20  00:04:14  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:15  jbm
 * First HaL mods to ICA
 *
 * Revision 2.1  91/05/28  10:13:15  cso
 * Removed Chosen_Symbol.
 * 
 * Revision 2.0  91/03/30  13:50:14  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.16  91/03/28  14:45:38  cso
 * Removed the XFlush calls.
 * 
 * Revision 1.15  91/02/22  09:47:20  cso
 * Completed documentation.
 * Combined and removed Conclude with Quit.
 * 
 * Revision 1.14  90/11/29  14:25:25  cso
 * Added a check so that NULL grammars are not checked for termination.
 * 
 * Revision 1.13  90/11/26  13:15:09  cso
 * Cosmetic and lint modifications.
 * 
 * Revision 1.12  90/11/20  15:52:50  cso
 * Modified Initialize to just resetting Info up to the symbol table size.
 * 
 * Revision 1.11  90/11/05  18:20:28  cso
 * Added function template to all functions, but still need to fill it in.
 * Removed some global initializations.  Changed Conclude to only checking
 * termination.  Added the call to install signal handlers.
 * 
 * Revision 1.10  90/10/29  10:24:31  cso
 * Removed a couple of XtSetSensitive calls.
 * 
 * Revision 1.9  90/10/24  15:35:00  cso
 * Changes based on the new types in types.h.
 * 
 * Revision 1.8  90/10/02  13:35:48  cso
 *  Added #ifdef AUX instead of comments.
 * 
 * Revision 1.7  90/09/03  18:25:05  cso
 * Function name changes.
 * 
 * Revision 1.6  90/09/03  14:35:41  cso
 * Changed function names to be more consistent.
 * 
 * Revision 1.5  90/08/29  17:49:31  cso
 * Eliminated Alert_Status and attframe and the global file stuff.
 * 
 * Revision 1.4  90/08/26  17:41:26  cso
 * Comment all calls RE: auxiliary information.
 * 
 * Revision 1.3  90/08/26  14:13:25  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "alt.h"
#include "display.h"
#include "interface.h"
#include "utils.h"

/*  EXTERNS */

extern void
   InstallSignalHandlers();

/*  GLOBALS */

Boolean
   Dirtyflag;
GrammarData
   Info[MAXSYMBOLS];
int
   Defndx,                 /* Index into the available symbols list.
			      It always points to the next 
			      available position for a symbol, thus
			      it always indicates the number of elements. */
   Ndx,                    /* Index into the symbol table.
			      It always points to the next 
			      available position for a symbol, thus
			      it always indicates the number of elements. */
   Pathndx,                /* Index into the stack of elements implementing
			      the path mechanism of navigation
			      (currently unused).
			      It always points to the next 
			      available position for a node, thus
			      it always indicates the number of elements. */
   Undndx;                 /* Index into the undefined symbols list.
			      It always points to the next 
			      available position for a symbol, thus
			      it always indicates the number of elements. */
Node
   *Chosennode,           /* The node selected by the user. */
   *Pathlist[MAXLEVELS],   /* The stack for the path navigation mechanism */
   *Root,                  /* This pointer ALWAYS points at the root node in 
			      the node data structure. */ 
   *Tree;                  /* This pointer traverses the tree, and follows the
			      focus of the user.*/
Pixmap
  Threed;		   /* The background pixmap used to show
			      that a symbol has more than one definition */
String
   Defsymlist[MAXSYMBOLS], /* The available symbols list */
   Undsymlist[MAXSYMBOLS]; /* The undefined symbols list */
Widget
   Frame,                  /* The form widget containing the main interface */
   Toplevel;               /* The shell widget associated with the 
			      application context. */
XtAppContext
   Appcon;                /* The application context for this environment */




/*
   FUNCTION: 		Initialize

   Synopsis: 
       Initialize the data structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       05-23-91   cso    Removed Chosen_Symbol.

   Globals used:
       Toplevel  Needed for initializing some windows

   Globals Modified:
       The following are initialized:
       Info
       Ndx
       Root
       Tree
       Defsymlist
       Undsymlist
       Defndx
       Undndx
       Dirtyflag
       Tree
       Chosennode
       Pathndx
       
   Return value:	void

   Range:		none
*/

void
Initialize(symtablimit)

int
   symtablimit;  /* The symbol table limit to be initialized */

{
   int
      i;  /* Loop counter */
   
   /*
    * Initialize the symbol table (also the max in the symbol lists
    */
   for (i = 0; i < symtablimit; i++)
   {
      Info[i].symbol     = "";
      Info[i].symtype    = Unknown;
      Info[i].filename   = "";
      Info[i].defined    = False;
      Info[i].definition = NULL;
      Info[i].count      = 0;
      Info[i].star       = 0;
      Info[i].plus       = 0;
      Info[i].optional   = 0;
      Info[i].terminal   = TriU;
      Info[i].terminate  = TriU;
      Info[i].visits     = 0;
      Defsymlist[i]      = NULL;
      Undsymlist[i]      = NULL;
   }

   /*
    * Clear the tree display 
    * Destroy the widgets if there are any
    * Change the lists displayed in the symbol lists
    */
   WipeWindow();
   KillWidgets(Root);

   XawListChange(XtNameToWidget(Frame, "*listframe*deflist"),
		 Defsymlist, 0, 0, True);
   XawListChange(XtNameToWidget(Frame, "*listframe*undlist"),
		 Undsymlist, 0, 0, True);

   /*
    * Initialize indices
    *            selected symbol/node
    * Create the Root node
    * Assign Tree to point there
    * Create a widget for it
    * A new interface is up so dirty-flag should be false
    * Update the interface
    */
   Ndx           = 0;
   Defndx        = 0;
   Undndx        = 0;
   Pathndx       = 0;
   Chosennode    = NULL;
   Root          = NewNode((Node *) NULL, (Node *) NULL);
   Tree          = Root;
   Tree->nodew   = CreateNodeW((Widget) NULL, "");
   Dirtyflag     = False;

   UpdateInterface(Root, Tree);

   return;
}




/*
   FUNCTION: 		InitSensitivityCursors

   Synopsis: 
       Initialize the sensitivity of some widgets and cursors.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Frame
       Toplevel - both used to access other widgets

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
InitSensitivityCursors()

{
   Widget
      deflist, /* Widget for the available symbols list */
      undlist; /* Widget for the undefined symbols list */

   /*
    * All compass widgets start out insensitive
    */
   XtSetSensitive(XtNameToWidget(Toplevel, "*compassframe*north"),    False);
   XtSetSensitive(XtNameToWidget(Toplevel, "*compassframe*west"),     False);
   XtSetSensitive(XtNameToWidget(Toplevel, "*compassframe*east"),     False);
   XtSetSensitive(XtNameToWidget(Toplevel, "*compassframe*south"),    False);
#ifdef AUX
   /*
    * aux widgets
    */
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*value"),        False);
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*Previous"),     False);
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*Next"),         False);
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*Done"),         False);
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*Remove"),       False);
   XtSetSensitive(XtNameToWidget(Frame, "*auxframe*Add"),          False);
#endif

   /*
    * Define a global cursor, the same for the lists
    */
   XDefineCursor(XtDisplay(Toplevel), XtWindow(Toplevel),
		 XCreateFontCursor(XtDisplay(Toplevel), XC_hand2));

   deflist = XtNameToWidget(Frame, "*listframe*deflist");
   undlist = XtNameToWidget(Frame, "*listframe*undlist");
   XDefineCursor(XtDisplay(deflist), XtWindow(deflist),
		 XCreateFontCursor(XtDisplay(deflist), XC_hand2));

   XDefineCursor(XtDisplay(undlist), XtWindow(undlist),
		 XCreateFontCursor(XtDisplay(undlist), XC_hand2));
   return;
}




/*
   FUNCTION: 		main

   Synopsis: 
       Coordinate and initialize the environment.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	int

   Range:		0 - everything is okay
*/

int
main(argc, argv)

int
   argc;   /* Number of command line options */
char
   **argv; /* List of command line options */

{
   Display
      *paneldsp;
   Widget
      panel;
   Window
      panelwdw;
   XEvent
      event;
   
   /*
    * Install the signal handlers
    * Create the whole interface
    * Initialize data structures
    * Initialize the widget sensitivity and cursors
    * Run the event mainloop and go with it!
    */
   InstallSignalHandlers();

   CreateInterface(argc, argv);

   Initialize(MAXSYMBOLS);

   InitSensitivityCursors();

   panel    = XtNameToWidget(Toplevel, "*Attributes*attribform");
   paneldsp = XtDisplay(panel);
   panelwdw = XtWindow(panel);
   
   for ( ; ; )
   {
      XtAppNextEvent(Appcon, &event);
      XtDispatchEvent(&event);
      if (XtIsManaged(panel))
      {
	 XRaiseWindow(paneldsp, panelwdw);
      }
   }
}
