/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menubar.c,v $
   Purpose:
       To handle the menus.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 14:46:19 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  CreateMenubar
                                           CreateFileMenu
					   CreateGrammarMenu
					   CreateHelpMenu
					   CreateNotationsMenu
					   CreateSemanticMenu
*/

/* HISTORY
   $Log:	menubar.c,v $
 * Revision 1.6  92/05/29  14:46:19  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.5  92/03/18  14:16:29  ware
 * Working on consistent interface.
 * 
 * Revision 1.4  1992/01/28  15:40:32  cso
 * Put border around the menubar again
 *
 * Revision 1.3  1992/01/14  17:56:42  cso
 *  - Reorganizing header files.
 *
 * Revision 1.2  1991/11/20  00:04:25  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:18  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:50:29  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.4  91/02/22  09:48:29  cso
 * Completed documentation.
 * 
 * Revision 1.3  90/11/05  18:22:30  cso
 * Added function template to all functions, but still need to fill it in.
 * Else cosmetics.
 * 
 * Revision 1.2  90/10/29  10:25:21  cso
 * Added the appropriate decls to make it work after breaking up this file
 * into separate menu files.
 * 
 * Revision 1.1  90/10/25  13:47:25  cso
 * Initial revision
 * 
 * Revision 1.13  90/10/25  13:35:08  cso
 * Separated out the individual menu selections into separate modules
 * to make this module manageable.
 * 
 * Revision 1.12  90/10/24  15:35:47  cso
 * Using XtMallocs instead of mallocs.
 * 
 * Revision 1.11  90/10/04  15:57:25  cso
 * Cosmetic changes.
 * 
 * Revision 1.10  90/10/02  13:36:18  cso
 *  Added #ifdef AUX instead of comments.
 * 
 * Revision 1.9  90/09/03  18:26:55  cso
 * Function name changes.
 * 
 * Revision 1.8  90/09/03  14:36:26  cso
 * Changed function names to be more consistent.
 * 
 * Revision 1.7  90/08/29  17:50:37  cso
 * Eliminated the Alert_Status flag.  Rearranged menu buttons again.
 * 
 * Revision 1.6  90/08/26  18:44:45  cso
 * Changed order in the menubar.
 * 
 * Revision 1.5  90/08/26  17:41:32  cso
 * Comment all calls RE: auxiliary information.
 * 
 * Revision 1.4  90/08/26  14:13:28  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"

#include "filemenu.h"
#include "grammenu.h"
#include "helpmenu.h"
#include "notemenu.h"
#include "semmenu.h"



/*
   FUNCTION: 		CreateMenubar

   Synopsis: 
       Create the menubar and call the appropriate functions to create
       the options.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget CreateMenubar( parent )
Widget parent; /* The parent of the menubar */
{
   Widget
      menubar,  /* The menubar itself */
      filebut,  /* The file button */
      filemenu, /* The file menu list */
      grambut,  /* The Grammar button */
      grammenu, /* The Grammar menu list */
      helpbut,  /* The Help button */
      helpmenu, /* The Help menu list */
      notebut,  /* The Notations button */
      notemenu, /* The Notations menu list */
      sembut,   /* The Semantics button */
      semmenu;  /* The Semantics menu list */

   /*
    * Create the menubar
    * [file] [grammar] [notations] [semantics] [help]
    */
   menubar = XtVaCreateManagedWidget( "menubar",
       groupWidgetClass, parent,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     1,
       XtNmargin,      7,
       XtNalignment,   gaFill,    			     
       NULL );

   /*
    * Create the FILE menu
    */
   filebut = XtVaCreateManagedWidget( "File",
       menuButtonWidgetClass, menubar,
       XtNlabel,     "  File  ",
       XtNmenuName,  "filemenu",
       XtNmargin,    1,
       XtNstretch,   True,
       NULL );

   filemenu = XtVaCreatePopupShell( "filemenu",
       simpleMenuWidgetClass, parent,
       NULL );
   CreateFileMenu(filemenu);

   /*
    * Create the GRAMMAR menu
    */
   grambut = XtVaCreateManagedWidget( "GrammarCommands",
       menuButtonWidgetClass, menubar,
       XtNlabel,     " Grammar Commands ",
       XtNmenuName,  "grammenu",
       XtNmargin,    1,
       XtNstretch,   True,
       NULL );

   grammenu = XtVaCreatePopupShell( "grammenu",
       simpleMenuWidgetClass, parent,
       NULL );
   CreateGrammarMenu(grammenu);

   /*
    * Create the NOTATIONS Menu
    */
   notebut = XtVaCreateManagedWidget( "Notations",
       menuButtonWidgetClass, menubar,
       XtNlabel,     " Notations ",
       XtNmenuName,  "notemenu",
       XtNmargin,    1,
       XtNstretch,   True,
       NULL );

   notemenu = XtVaCreatePopupShell( "notemenu",
       simpleMenuWidgetClass, parent,
       NULL );
   CreateNotationsMenu(notemenu);

   /*
    * Create the SEMANTIC menu
    */
   sembut = XtVaCreateManagedWidget( "Semantic Views",
       menuButtonWidgetClass, menubar,
       XtNlabel,     " Semantic Views ",
       XtNmenuName,  "semmenu",
       XtNmargin,    1,
       XtNstretch,   True,
       NULL );

   semmenu = XtVaCreatePopupShell( "semmenu",
       simpleMenuWidgetClass, parent,
       NULL );
   CreateSemanticMenu(semmenu);

   /*
    * Create the HELP menu
    */
   helpbut = XtVaCreateManagedWidget( "Help",
       menuButtonWidgetClass, menubar,
       XtNlabel,     "  Help  ",
       XtNmenuName,  "helpmenu",
       XtNmargin,    1,
       XtNstretch,   True,
       NULL );

   helpmenu = XtVaCreatePopupShell( "helpmenu",
       simpleMenuWidgetClass, parent,
       NULL );
   CreateHelpMenu(helpmenu);

   return(menubar);
}
