/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: node.c,v $
   Purpose:
       To contain all routines for manipulating the Node data 
       structures.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/18 14:16:36 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  IsFirstAlt

   EXPORT  IsLastAlt

   EXPORT  IsFirstNode

   EXPORT  IsLastNode

   EXPORT  GetPrevAlt

   EXPORT  GetNextAlt

   EXPORT  GetPrevNode

   EXPORT  GetNextNode

   EXPORT  GetFirstChild

   EXPORT  IsRoot

*/

/* HISTORY
   $Log:	node.c,v $
 * Revision 1.4  92/03/18  14:16:36  ware
 * Working on consistent interface.
 * 
 * Revision 1.3  1992/01/14  17:56:47  cso
 *  - Reorganizing header files.
 *
 * Revision 1.2  1991/11/20  00:04:30  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:22  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:50:41  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.5  91/02/22  09:48:52  cso
 * Completed documentation.
 * 
 * Revision 1.4  90/11/05  18:22:54  cso
 * Added function template to all functions, but still need to fill it in.
 * Added explicit returns.
 * 
 * Revision 1.3  90/10/24  15:36:33  cso
 * Changed a return value from UNKNOWN to NULL.
 * 
 * Revision 1.2  90/09/03  14:37:01  cso
 * Fleshed out some routines for Node utilities.
 * 
 * Revision 1.1  90/08/26  17:41:38  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"

#include "main.h"
#include "status.h"



/*
   FUNCTION: 		IsFirstAlt

   Synopsis: 
       Return a boolean indicating whether the pointer is the first 
       definition or not.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Boolean

   Range:		True or False
*/

Boolean
IsFirstAlt(aptr)

Alt
   *aptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL check if it is the first one or not
    * Else inform the user of the error.
    */
   if (aptr != NULL)
   {
      /*
       * If prev is NULL return True, else return False
       */
      if (aptr->prev == NULL)
      {
	 return(True);
      }
      else
      {
	 return(False);
      }
   }
   else
   {
      XicaStatus(Statw, "IsFirstAlt received a NULL Alt pointer\n");
   }
   return(False);
}




/*
   FUNCTION: 		IsLastAlt

   Synopsis: 
       Return a boolean indicating whether the pointer is the last
       definition or not.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Boolean

   Range:		True or False
*/

Boolean
IsLastAlt(aptr)

Alt
   *aptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL check if it is the last one or not
    * Else inform the user of the error.
    */
   if (aptr != NULL)
   {
      /*
       * If next is NULL return True, else return False
       */
      if (aptr->next == NULL)
      {
	 return(True);
      }
      else
      {
	 return(False);
      }
   }
   else
   {
      XicaStatus(Statw, "IsLastAlt received a NULL Alt pointer\n");
   }
   return(False);
}




/*
   FUNCTION: 		IsFirstNode

   Synopsis: 
       Return a boolean indicating whether the pointer is the first 
       node in a production or not.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Boolean

   Range:		True or False
*/

Boolean
IsFirstNode(nptr)

Node
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL check if it is the first one or not
    * Else inform the user of the error.
    */
   if (nptr != NULL)
   {
      /*
       * If prev is NULL return True, else return False
       */
      if (nptr->prev == NULL)
      {
	 return(True);
      }
      else
      {
	 return(False);
      }
   }
   else
   {
      XicaStatus(Statw, "IsFirstNode received a NULL Node pointer\n");
   }
   return(False);
}




/*
   FUNCTION: 		IsLastNode

   Synopsis: 
       Return a boolean indicating whether the pointer is the last
       node in a production or not.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Boolean

   Range:		True or False
*/

Boolean
IsLastNode(nptr)

Node
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL check if it is the last one or not
    * Else inform the user of the error.
    */
   if (nptr != NULL)
   {
      /*
       * If next is NULL return True, else return False
       */
      if (nptr->next == NULL)
      {
	 return(True);
      }
      else
      {
	 return(False);
      }
   }
   else
   {
      XicaStatus(Statw, "IsLastNode received a NULL Node pointer\n");
   }
   return(False);
}






/*
   FUNCTION: 		*GetPrevAlt

   Synopsis: 
       Return the pointer previous to the one passed in.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Alt *

   Range:		NULL or an actual pointer
*/

Alt
*GetPrevAlt(aptr)

Alt
   *aptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL, return the pointer that is previous
    * Else inform the user of the error
    */
   if (aptr != NULL)
   {
      return(aptr->prev);
   }
   else
   {
      XicaStatus(Statw, "GetPrevAlt received a NULL Alt pointer\n");
      return(NULL);
   }
}




/*
   FUNCTION: 		*GetNextAlt

   Synopsis: 
       Return the pointer next to the one passed in.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Alt *

   Range:		NULL or an actual pointer
*/

Alt
*GetNextAlt(aptr)

Alt
   *aptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL, return the pointer that is next
    * Else inform the user of the error
    */
   if (aptr != NULL)
   {
      return(aptr->next);
   }
   else
   {
      XicaStatus(Statw, "GetNextAlt received a NULL Alt pointer\n");
      return(NULL);
   }
}




/*
   FUNCTION: 		*GetPrevNode

   Synopsis: 
       Return the pointer previous to the one passed in.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Node *

   Range:		NULL or an actual pointer
*/

Node
*GetPrevNode(nptr)

Node
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL, return the pointer that is previous
    * Else inform the user of the error
    */
   if (nptr != NULL)
   {
      return(nptr->prev);
   }
   else
   {
      XicaStatus(Statw, "GetPrevNode received a NULL Node pointer\n");
      return(NULL);
   }
}




/*
   FUNCTION: 		*GetNextNode

   Synopsis: 
       Return the pointer next to the one passed in.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Node *

   Range:		NULL or an actual pointer
*/

Node
*GetNextNode(nptr)

Node
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL, return the pointer that is next
    * Else inform the user of the error
    */
   if (nptr != NULL)
   {
      return(nptr->next);
   }
   else
   {
      XicaStatus(Statw, "GetNextNode received a NULL Node pointer\n");
      return(NULL);
   }
}




/*
   FUNCTION: 		*GetFirstChild

   Synopsis: 
       Returns the first child of a particular node.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Node *

   Range:		NULL or an actual pointer
*/

Node
*GetFirstChild(nptr)

Node 
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL, check its definitions.
    * Else inform the user.
    */
   if (nptr != NULL)
   {
      /*
       * If the node has definitions, return the first node in the 
       * first production
       * Else return NULL
       */
      if (nptr->alts != NULL)
      {
	 return(nptr->alts->prod);
      }
      else
      {
	 return(NULL);
      }
   }
   else
   {
      XicaStatus(Statw, "GetFirstChild received a NULL Node pointer\n");
      return(NULL);
   }
}





/*
   FUNCTION: 		IsRoot

   Synopsis: 
       Make a determination on whether the node is the Root or not.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	Boolean

   Range:		True or False
*/

Boolean
IsRoot(nptr)

Node
   *nptr;  /* Pointer under consideration */

{
   /*
    * If the pointer is not NULL check if it has a parent or not
    * Else inform the user of the error.
    */
   if (nptr != NULL)
   {
      /*
       * If the node has no parent then return True,
       * Else return False
       */
      if (nptr->parent == NULL)
      {
	 return(True);
      }
      else
      {
	 return(False);
      }
   }
   else
   {
      XicaStatus(Statw, "IsRoot received a NULL Node pointer\n");
      return(False);
   }
}
