/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: notemenu.c,v $
   Purpose:     
       Create a menu of and provide the functions for grammar notations
       that can be generated.

   Programmer:  Con O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/03/18 14:16:43 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  BNF_Notation
                                           BNF_Gen
					   OpenFile
   HIDDEN  DTD_Notation
					   DTD_Declaration
                                           DTD_Elements
					   DTD_Entities
					   OpenFile
   HIDDEN  ICA_Notation
                                           ICA_Gen
					   OpenFile
   HIDDEN  YaccNotation
					   OpenFile
                                           YaccBNF
					   YaccGen
					   YaccPre
   HIDDEN  IFYaccNotation
					   OpenFile
                                           IFYaccBNF
					   IFYaccGen
					   IFYaccPre
   EXPORT  CreateNotationsMenu
*/

/* HISTORY
   $Log:	notemenu.c,v $
 * Revision 1.8  92/03/18  14:16:43  ware
 * Working on consistent interface.
 * 
 * Revision 1.7  1992/02/11  16:25:43  cso
 * Commented out the IF parser option for the release
 *
 * Revision 1.6  1992/01/14  17:56:52  cso
 *  - Reorganizing header files.
 *
 * Revision 1.5  1992/01/03  20:22:17  cso
 * Notify the user with the status widget that the various notations have
 * been generated.
 *
 * Revision 1.4  1991/11/20  00:04:39  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.3  1991/11/12  23:19:15  cso
 * Added modifications to generate intermediate form yacc parsers.  Still
 * need to generate the lex side of things and to provide the auxiliary
 * information extensions.
 *
 * Revision 1.2  1991/11/07  12:41:19  cso
 * Changed CDATA to ICA_Char_DATA to avoid possible name conflicts.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:25  jbm
 * First HaL mods to ICA
 *
 * Revision 2.1  91/05/28  10:13:41  cso
 * Added the calculation of the current date/time for SGML DTD creation.
 * Added the new markers in the ICA file.
 * Output the MaxNameLength for the retagger
 * 
 * Revision 2.0  91/03/30  13:50:53  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.7  91/03/28  14:45:59  cso
 * Added a new notation (DBM) for Database Mappings to be selected.
 * 
 * Revision 1.6  91/02/22  09:50:25  cso
 * Completed documentation.
 * Removed the options for MDL and STRUCT notations.
 * Broke down the DTD and Yacc generation calls to the different pieces.
 * Thus, no system calls are necessary.
 * 
 * Revision 1.5  90/11/26  13:15:39  cso
 * Cosmetic and lint modifications.
 * 
 * Revision 1.4  90/11/20  15:54:12  cso
 * Added blank lines in menu.
 * 
 * Revision 1.3  90/11/05  18:23:10  cso
 * Added function template to all functions, but still need to fill it in.
 * Taking advantage of the new OpenFile.  But I need to fix some other
 * file things that get done and maybe eliminate some system calls.
 * Else, cosmetics.
 * 
 * Revision 1.2  90/10/29  10:26:16  cso
 * Added the appropriate decls to make it work.
 * 
 * Revision 1.1  90/10/25  13:35:47  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "fileio.h"
#include "generate.h"
#include "main.h"
#include "sgml.h"
#include "status.h"
#include "symtab.h"

static char
   *ISOEntities[] =
{
   "<!ENTITY % ISOamsa  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Arrow Relations//EN\">",
   "<!ENTITY % ISOamsb  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Binary Operators//EN\">",
   "<!ENTITY % ISOamsc  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Delimiters//EN\">",
   "<!ENTITY % ISOamsn  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Negated Relations//EN\">",
   "<!ENTITY % ISOamso  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Ordinary//EN\">",
   "<!ENTITY % ISOamsr  PUBLIC \"ISO 8879-1986//ENTITIES Added Math Symbols: Relations//EN\">",
   "<!ENTITY % ISObox   PUBLIC \"ISO 8879-1986//ENTITIES Box and Line Drawing//EN\">",
   "<!ENTITY % ISOcyr1  PUBLIC \"ISO 8879-1986//ENTITIES Russian Cyrillic//EN\">",
   "<!ENTITY % ISOcyr2  PUBLIC \"ISO 8879-1986//ENTITIES Non-Russian Cyrillic//EN\">",
   "<!ENTITY % ISOdia   PUBLIC \"ISO 8879-1986//ENTITIES Diacritical Marks//EN\">",
   "<!ENTITY % ISOgrk1  PUBLIC \"ISO 8879-1986//ENTITIES Greek Letters//EN\">",
   "<!ENTITY % ISOgrk2  PUBLIC \"ISO 8879-1986//ENTITIES Monotoniko Greek//EN\">",
   "<!ENTITY % ISOgrk3  PUBLIC \"ISO 8879-1986//ENTITIES Greek Symbols//EN\">",
   "<!ENTITY % ISOlat1  PUBLIC \"ISO 8879-1986//ENTITIES Added Latin 1//EN\">",
   "<!ENTITY % ISOlat2  PUBLIC \"ISO 8879-1986//ENTITIES Added Latin 2//EN\">",
   "<!ENTITY % ISOnum   PUBLIC \"ISO 8879-1986//ENTITIES Numeric and Special Graphic//EN\">",
   "<!ENTITY % ISOpub   PUBLIC \"ISO 8879-1986//ENTITIES Publishing//EN\">",
   "<!ENTITY % ISOtech  PUBLIC \"ISO 8879-1986//ENTITIES General Technical//EN\">",
   "",
   0
};


static char
   *ISOuse[] =
{
   "%ISOamsa;",
   "%ISOamsb;",
   "%ISOamsc;",
   "%ISOamsn;",
   "%ISOamso;",
   "%ISOamsr;",
   "%ISObox;",
   "%ISOcyr1;",
   "%ISOcyr2;",
   "%ISOdia;",
   "%ISOgrk1;",
   "%ISOgrk2;",
   "%ISOgrk3;",
   "%ISOlat1;",
   "%ISOlat2;",
   "%ISOnum;",
   "%ISOpub;",
   "%ISOtech;",
   0
};



/*
   FUNCTION: 		BNF_Notation

   Synopsis: 
       Open a file and call the appropriate routines to generate the BNF
       grammar notation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	static void

   Range:		none
*/

static void
BNF_Notation(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;   /* The file for the generated notation */

   /*
    * Get an open file from the user
    * If it is not NULL, proceed with generating the notation
    */
   fptr = OpenFile("bnf", False);
   if (fptr != NULL)
   {
      /*
       * Generate the BNF notation
       * Close the file
       */
      BNF_Gen(Root, fptr);
      fclose(fptr);
      XicaStatus(Statw, "BNF file generated\n");
   }
   return;
}




/*
   FUNCTION: 		DTD_Notation

   Synopsis: 
       Open a file and call the appropriate routines to generate the SGML DTD
       grammar notation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/14   cso    Added the ISO symbol entities to the output
       91/05/23   cso    Added the time of generation and some other
                          cosmetic changes.

   Globals used:
	none

   Globals Modified:
	none

   Return value:	static void

   Range:		none
*/

static void
DTD_Notation(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;   /* The file for the generated notation */
   int
      i;       /* Loop counter */
   String
      dgmdate; /* The date the DeveGram produced the DTD */
   time_t
      dgmtime; /* The time in seconds from THE epoch ??? */

   /*
    * Get an open file from the user
    * If it is not NULL, proceed with generating the notation
    */
   fptr = OpenFile("dtd", False);
   if (fptr != NULL)
   {
      /*
       * Generate the SGML Declaration
       * Output the DOCTYPE declaration
       * Generate the parameter ENTITY declarations
       * Generate the ELEMENT declarations
       * Output the end of the doctype
       * Close the file
       */
      DTD_Declaration(fptr);

      fprintf(fptr, "<!-- This DTD was produced by DeveGram ");
      dgmtime = time(NULL);
      if (dgmtime != (time_t)-1)
      {
	 dgmdate = asctime(localtime(&dgmtime));
	 dgmdate[strlen(dgmdate) - 1] = '\0';
	 fprintf(fptr, "on %s ", dgmdate);
      }
      fprintf(fptr, "-->\n\n");
      fprintf(fptr, "<!DOCTYPE %s [\n\n", Root->symbol);
      fprintf(fptr, "<!--     Parameter Entities       -->\n\n");
      fprintf(fptr, "<!--        ISO Symbols           -->\n\n");
      for (i = 0; ISOEntities[i] != 0; i++)
      {
	 fprintf(fptr, "%s\n", ISOEntities[i]);
      }
      fprintf(fptr, "\n");
      fprintf(fptr, "<!--      Terminal symbols        -->\n\n");
      DTD_Tokens(fptr);
      ResetVisits();
      fprintf(fptr, "<!--    Non-ELEMENT symbols       -->\n\n");
      DTD_Entities(Root, fptr);
      fprintf(fptr, "<!--     Document structure       -->\n\n");
      DTD_Elements(Root, fptr);
      for (i = 0; ISOuse[i] != 0; i++)
      {
	 fprintf(fptr, "%s\n", ISOuse[i]);
      }
      fprintf(fptr, "\n");
      fprintf(fptr, "]>\n");
      fclose(fptr);
      XicaStatus(Statw, "DTD file generated\n");
   }
   return;
}




/*
   FUNCTION: 		ICA_Notation

   Synopsis: 
       Open a file and call the appropriate routines to generate the ICA
       grammar notation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/14   cso    Added the CDATA productions for terminals
       91/04/05   cso    Added GetMaxNameLength call for RTT (max
                          length of a symbol).
       91/05/23   cso    Added the top marker for the MSI and
                          CDATAPLUS production.
       91/08/02   cso    Changed CDATAPLUS to CDATASTAR for possible
                          empty content.
       91/11/07   cso    Changed CDATASTAR to ICA_Char_DATASTAR to
                          avoid possible name conflicts with CDATA.

   Globals used:
	Info   To access the terminal symbols in the table
	Ndx    To limit access to the symbol table

   Globals Modified:
	none

   Return value:	static void

   Range:		none
*/

static void
ICA_Notation(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;   /* The file for the generated notation */
   int
      i;       /* Loop counter */
   
   /*
    * Get an open file from the user
    * If it is not NULL, proceed with generating the notation
    */
   fptr = OpenFile("ica", False);
   if (fptr != NULL)
   {
      /*
       * Generate the grammar portion of the ICA file used
       * by the MSI
       * Output the RTT separator in the .ica file
       * Generate the RTT list of tags
       * Output the ITT separator in the .ica file
       * Generate the ITT dependency list
       * Close the file
       */
      fprintf(fptr, "$$$$\n$$$$\n");
      ICA_Gen(Root, fptr);

      /*
       * Output terminal symbol productions for MSI and the translator
       * generators to handle.  This is similar to the #PCDATA stuff
       * in DTDs.
       */
      for (i = 0; i < Ndx; i++)
      {
	 if (Info[i].terminal == TriT)
	 {
	    fprintf(fptr, "%s\t: ICA_Char_DATASTAR\n", Info[i].symbol);
	    fprintf(fptr, "\t(ICA_Char_DATASTAR)\n\t;\n\n");
	 }
      }
      
      fprintf(fptr, "#### Replace Tag Tool List ####\n");
      fprintf(fptr, "%d\n", GetMaxNameLength());
      ICA_RTT(fptr);
      fprintf(fptr, "#### Insert Tag Tool Rules ####\n");
      ICA_ITT(fptr);
      fclose(fptr);
      XicaStatus(Statw, "ICA file generated\n");
   }
   return;
}




/*
   FUNCTION: 		YaccNotation

   Synopsis: 
       Open a file and call the appropriate routines to generate the Yacc
       grammar notation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	static void

   Range:		none
*/

static void
YaccNotation(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;   /* The file for the generated notation */

   /*
    * Get an open file from the user
    * If it is not NULL, proceed with generating the notation
    */
   fptr = OpenFile("yacc", False);
   if (fptr != NULL)
   {
      /*
       * Generate the header info of a yacc specification
       * (%token and %start)
       * Generate the grammar portion
       * Generate the expanded grammar notation for *, +, and ?
       * Close the file
       */
      YaccPre(fptr);
      YaccGen(Root, fptr);
      YaccBNF(fptr);
      fclose(fptr);
      XicaStatus(Statw, "Yacc file generated\n");
   }
   return;
}




/*
   FUNCTION: 		IFYaccNotation

   Synopsis: 
       Open a file and call the appropriate routines to generate the Yacc
       grammar notation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
	none

   Globals Modified:
	none

   Return value:	static void

   Range:		none
*/

static void
IFYaccNotation(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;   /* The file for the generated notation */

   /*
    * Get an open file from the user
    * If it is not NULL, proceed with generating the notation
    */
   fptr = OpenFile("ifx", False);
   if (fptr != NULL)
   {
      /*
       * Generate the header info of a yacc specification
       * (%token and %start)
       * Generate the grammar portion
       * Generate the expanded grammar notation for *, +, and ?
       * Close the file
       */
      IFYaccPre(fptr);
      IFYaccGen(Root, fptr);
      IFYaccBNF(fptr);
      fclose(fptr);
      XicaStatus(Statw, "Intermediate Form parser file generated\n");
   }
   return;
}




/*
   FUNCTION: 		CreateNotationsMenu

   Synopsis: 
       Create the menu list of the notations available to be generated.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/14   cso    Added a selection for the DBM notation

   Globals used:
	none

   Globals Modified:
	none

   Return value:	void

   Range:		none
*/

void
CreateNotationsMenu(parent)

Widget
   parent;  /* The parent of this menu list */

{
   Arg
      args[MAXARGS];/* List of arguments */
   int
      cnt;       /* Number of arguments */
   Widget
      bnfsel,    /* BNF selection */
      dtdsel,    /* SGML DTD selection */
      icasel,    /* ICA selection */
      yaccsel,   /* Yacc selection */
      dbm_sel;   /* DBM notation */
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "Backus-Naur Form"); cnt++;
   bnfsel = XtCreateManagedWidget("bnfsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(bnfsel, XtNcallback, BNF_Notation, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "SGML DTD"); cnt++;
   dtdsel = XtCreateManagedWidget("dtdsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(dtdsel, XtNcallback, DTD_Notation, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "ICA specification"); cnt++;
   icasel = XtCreateManagedWidget("icasel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(icasel, XtNcallback, ICA_Notation, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "Yacc"); cnt++;
   yaccsel = XtCreateManagedWidget("yaccsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(yaccsel, XtNcallback, YaccNotation, NULL);

/*
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "IF parser"); cnt++;
   dbm_sel = XtCreateManagedWidget("dbmsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(dbm_sel, XtNcallback, IFYaccNotation, NULL);
*/
   return;
}
