/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: put.c,v $
   Purpose:     
   To provide Put routines for the data structures and widgets.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.7 $
   Date:        $Date: 92/05/29 14:46:24 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  PutLabel

   EXPORT  PutString
                                           GetStringWidthInPixels
   EXPORT  PutDefined
                                           GetIndex
					   InsertUndItem
					   RemoveUndItem
   EXPORT  PutDefinition
                                           GetIndex
   EXPORT  PutTerminal
                                           GetIndex
   EXPORT  PutSymbol
                                           GetIndex
   EXPORT  PutSymbolType
                                           GetIndex
   EXPORT  PutOccur
                                           GetIndex
*/

/* HISTORY
   $Log:	put.c,v $
 * Revision 1.7  92/05/29  14:46:24  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.6  92/03/18  14:16:50  ware
 * Working on consistent interface.
 * 
 * Revision 1.5  1992/01/14  17:56:59  cso
 *  - Reorganizing header files.
 *
 * Revision 1.4  1992/01/03  20:18:42  cso
 * PutSymbolType:
 * 	When putting the label type into the toggle widget maintain an
 * off state for now.
 *
 * Revision 1.3  1991/11/20  00:04:45  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.2  1991/11/14  14:31:18  cso
 * types.h utils.c utils.h CVS: CVS: Enter Log.  Lines beginning with
 * 'CVS: ' are removed automatically CVS:
 * ----------------------------------------------------------------------
 * Made the necessary additions and changes to provide a mechanism to
 * change symbol types between nonterminal types Nonterminal & Grouping.
 * In SGML terms, therefore, intermediate form terms, this is the
 * difference between having element tags and not.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:29  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:51:15  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.2  91/02/22  09:52:41  cso
 * Completed documentation.
 * Changed the call to PutOccur to use a character instead of a string
 * so that a switch could be used.
 * 
 * Revision 1.1  90/11/26  13:15:45  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "get.h"
#include "main.h"
#include "status.h"
#include "undlist.h"



/*
   FUNCTION: 		PutLabel

   Synopsis: 
       Put a string into a label widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void PutLabel( w, str )
Widget
   w;    /* The widget being changed */
String
   str;  /* The string being placed in the widget */
{
   /*
    * Assign the string to the label widget
    */
   XtVaSetValues( w, XtNlabel, str, NULL );

   return;
}




/*
   FUNCTION: 		PutString

   Synopsis: 
       Put a string into an ascii text widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutString(w, str)

Widget
   w;    /* The widget being changed */
String
   str;  /* The string being placed in the widget */

{
   Arg
      args[MAXARGS];/* List of arguments */
   int
      cnt;         /* Number of arguments */
   Dimension
      text_width;  /* The width of the string in pixels */

   /*
    * Get the size of the string in pixels so that the widget 
    * displaying it can be adjusted accordingly
    * Set the appropriate resources
    */
   text_width = GetStringWidthInPixels(w, str);
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, str);        cnt++;
   XtSetArg(args[cnt], XtNwidth,  text_width); cnt++;
   XtSetValues(w, args, cnt);
   return;
}




/*
   FUNCTION: 		PutDefined

   Synopsis: 
       Put a flag into the defined part of the symbol table
       for a particular symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutDefined(nptr, flag)

Node
   *nptr; /* The node under consideration */
Boolean
   flag;  /* The flag indicating whether or not the node is defined */

{
   int
      i;  /* Index into the symbol table */

   /*
    * Get the index into the symbol table
    * Set the flag
    * Update the undefined list of symbols
    */
   i = GetIndex(nptr->symbol);
   Info[i].defined = flag;
   if (flag == True)
   {
      RemoveUndItem(nptr->symbol);
   }
   else
   {
      InsertUndItem(nptr->symbol);
   }
   return;
}

   


/*
   FUNCTION: 		PutDefinition

   Synopsis: 
       Add a pointer to a node in a production to the list of
       definitions (NodeList).

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Info     Update the symbol table to add a new node to the 
                definition list

   Globals Modified:
       Info     A new node is added to the definition list of a symbol

   Return value:	void

   Range:		none
*/

void
PutDefinition(nptr)

Node
   *nptr;  /* Node being added to the definition list */

{
   int
      i;      /* Index into the symbol table */
   NodeList
      *nlptr, /* Loop pointer to search the definition list for duplicates
		 and to find an available spot for the new node */
      *olptr; /* Pointer to the next location for the new node */

   /*
    * Get the index into the symbol table for the node's symbol
    * If this is the first definition, create a new node list
    * Else make sure of uniqueness and add a new node list
    */
   i = GetIndex(nptr->symbol);
   if (Info[i].definition == NULL)
   {
      /*
       * A new node list is created and the node is added
       */
      Info[i].definition          = XtNew(NodeList);
      Info[i].definition->prev    = NULL;
      Info[i].definition->next    = NULL;
      Info[i].definition->nodeptr = nptr;
   }
   else
   {
      /*
       * Initialize the place holder pointer
       * Look for duplicates and update the place holder
       */
      olptr = NULL;
      for (nlptr = Info[i].definition; nlptr != NULL; nlptr = nlptr->next)
      {
	 olptr = nlptr;
	 if (nlptr->nodeptr == nptr)
	 {
	    return;
	 }
      }
      /*
       * If the place holder is not NULL
       * Create a new node list structure
       * Connect it to the node list
       * Add the node
       */
      if (olptr != NULL)
      {
	 olptr->next    = XtNew(NodeList);
	 nlptr          = olptr->next;
	 nlptr->prev    = olptr;
	 nlptr->next    = NULL;
	 nlptr->nodeptr = nptr;
      }
   }
   return;
}

   


/*
   FUNCTION: 		PutTerminal

   Synopsis: 
       This routine sets the boolean terminal to indicate 
       True, False, or Unknown (TriT, TriF, TriU).

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutTerminal(nptr, flag)

Node
   *nptr;  /* The node under consideration */
TriState
   flag;   /* The flag indicating terminal, nonterminal, unknown */

{
   int
      i;   /* Index into the symbol table */

   /*
    * Get the index into the symbol table
    * Assign the terminal flag
    */
   i = GetIndex(nptr->symbol);
   Info[i].terminal = flag;
   return;
}




/*
   FUNCTION: 		PutSymbol

   Synopsis: 
       Put a symbol into the symbol table or if it already
       exists increment the counter.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutSymbol(str)

String
   str;  /* The symbol string being placed in the symbol table */

{
   int
      i; /* Index into the symbol table */

   /*
    * Get the index into the symbol table
    * If the index returned is equal to the defined maximum
    * Inform the user to increase that variable and return
    * Else If the index is equal to Ndx, it is a new symbol
    * Else increment its count
    */
   i = GetIndex(str);
   if (i == MAXSYMBOLS)
   {
      XicaStatus(Statw,
		 "Out of space in symbol table - increase MAXSYMBOLS\n");
      return;
   }
   else if (i == Ndx)
   {
      /*
       * Assign the symbol string
       * Initialize its count
       * Increment Ndx
       */
      Info[Ndx].symbol = XtNewString(str);
      Info[Ndx].count = 1;
      Ndx++;
   }
   else
   {
      Info[i].count++;
   }
   return;
}




/*
   FUNCTION: 		PutSymbolType

   Synopsis: 
       Put a symbol's type into the symbol table.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutSymbolType(str, type)

String
   str;  /* Symbol under consideration */
SymType
   type; /* Symbol type */

{
   Arg
      args[MAXARGS];
   int
      cnt,
      i;  /* Index into the symbol table */
   NodeList
      *nlptr;
   String
      typelbl;
   
   /*
    * Get the index into the symbol table
    * Assign the symbol type
    */
   i = GetIndex(str);
   Info[i].symtype = type;
   switch (type)
   {
   default:
      typelbl = " ";
      break;
      
   case Part:
      typelbl = "N";
      break;
      
   case Group:
      typelbl = "G";
      break;
      
   case File:
      typelbl = "F";
      break;
   }
   for (nlptr = Info[i].definition; nlptr != NULL; nlptr = nlptr->next)
   {
      if ((nlptr->nodeptr != NULL) && (nlptr->nodeptr->typew != NULL))
      {
	 cnt = 0;
	 XtSetArg(args[cnt], XtNlabel, typelbl); cnt++;
	 XtSetArg(args[cnt], XtNstate, False);	 cnt++;
	 XtSetValues(nlptr->nodeptr->typew, args, cnt);
      }
   }
   return;
}




/*
   FUNCTION: 		PutOccur

   Synopsis: 
       Increment the occurrence counter (star, plus or optional).
       These counters are useful for notation generation routines.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PutOccur(str, occurchar)

String
   str;       /* Symbol under consideration */
char
   occurchar; /* Occurrence character */

{
   int
      i;      /* Index into the symbol table */

   /*
    * Get the index into the symbol table for the symbol
    */
   i = GetIndex(str);

   /*
    * If the symbol exists, increment the appropriate counter
    */
   if (i < Ndx)
   {
      switch (occurchar)
      {
      default:
	 /*
	  * Probably a terminal symbol occurrence so nothing happens
	  */
	 break;

      case '_':
	 /*
	  * I don't keep track of required uses of a symbol
	  */
	 break;

      case '?':
	 Info[i].optional++;
	 break;

      case '*':
	 Info[i].star++;
	 break;

      case '+':
	 Info[i].plus++;
	 break;
      }
   }
   return;
}


