/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: semmenu.c,v $
   Purpose:     
       Provide the menu of options and the functions for showing the semantics
       of a grammar.

   Programmer:  Con O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/18 14:17:04 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  View
                                           CreateViewShell
					   DetermineViewPtr
					   GenerateViewFile
					   ResetVisits
   EXPORT  CreateSemanticMenu
*/

/* HISTORY
   $Log:	semmenu.c,v $
 * Revision 1.4  92/03/18  14:17:04  ware
 * Working on consistent interface.
 * 
 * Revision 1.3  1992/01/14  17:57:44  cso
 *  - Reorganizing header files.
 *
 * Revision 1.2  1991/11/20  00:05:08  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:36  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:52:10  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.7  91/02/26  15:25:23  cso
 * Changed Show to View (the module changed to that as well).
 * 
 * Revision 1.6  91/02/22  09:56:37  cso
 * Completed documentation.
 * 
 * Revision 1.5  90/11/26  13:16:21  cso
 * Some cosmetic and lint modifications.
 * Significant changes because of routines developed in show.c
 * (should be view.c).
 * 
 * Revision 1.4  90/11/20  15:54:36  cso
 * Added blank lines in menu.
 * 
 * Revision 1.3  90/11/05  18:24:42  cso
 * Added function template to all functions, but still need to fill it in.
 * Else, cosmetics.  Still trying to get Refresh to work.
 * 
 * Revision 1.2  90/10/29  10:26:44  cso
 * Added the appropriate decls to make it work.
 * Also, trying to use separate shells for each individual menu choice, and 
 * allowing the Refresh button to make it reread the file, but it is not
 * working quite right yet.
 * 
 * Revision 1.1  90/10/25  13:35:55  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "get.h"
#include "main.h"
#include "status.h"
#include "symtab.h"
#include "view.h"




/*
   FUNCTION: 		View

   Synopsis: 
       Assign a filename, call the functions to generate the file, and show it.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static void

   Range:		none
*/

static void
View(w, semtype, call_data)

Widget
   w;         /* Unused */
SemanticFlag
   semtype;   /* The type of semantic to be created and shown */
caddr_t
   call_data; /* Unused */

{
   String
      filename; /* Filename to generate the view */
   Node
      *nptr;    /* The reference node to start the generation function */

   /*
    * Reinitialize the visits part of the symbol table
    * Depending on the semantic type, assign a filename
    */
   ResetVisits();
   switch (semtype)
   {
   default:
      XicaStatus(Statw, "Bad Semantic Type passed!\n");
      return;
      break;

   case LinearRoot:
      filename = "/tmp/LR.view";
      break;

   case LinearSym:
      filename = "/tmp/LS.view";
      break;

   case BarestRoot:
      filename = "/tmp/BR.view";
      break;

   case BarestSym:
      filename = "/tmp/BS.view";
      break;

   case ExpandRoot:
      filename = "/tmp/ER.view";
      break;

   case ExpandSym:
      filename = "/tmp/ES.view";
      break;
   }

   /*
    * Get the reference node
    * Generate teh semantic view
    * Create a shell and show that view to the user
    */
   nptr = DetermineViewPtr(semtype);
   GenerateViewFile(filename, nptr, semtype);
   CreateViewShell(filename, nptr, semtype);
   return;
}




/*
   FUNCTION: 		SymbolTerm

   Synopsis: 
       Check the termination starting at a particular symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
SymbolTerm(w, checkflag, call_data)

Widget
   w;         /* Unused */
TriState
   checkflag; /* The initial symbol */
XtPointer
   call_data; /* Unused */

{
   int
      i;
   Node
      *nptr;
   
   /*
    * Initialize the symbol table for doing this
    */
   InitTerminate();

   /*
    * If the checkflag is True, check termination of the whole
    * grammar, else just check for Chosennode.
    */
   switch (checkflag)
   {
   default:
      break;
      
   case TriT:
      /*
       * Check the termination starting at Root.
       * If it returns False tell the user it doesn't terminate
       * Else tell the user that the grammar does terminate
       */
      if (CheckTerminates(Root) == False)
      {
	 XicaStatus(Statw,
		 "WARNING: Grammar does not terminate from the root\n");
      }
      else
      {
	 XicaStatus(Statw, "Grammar terminates from the root\n");
      }
      break;
      
   case TriF:
      if (Chosennode != NULL)
      {
	 /*
	  * Check the termination starting at Chosennode.
	  * If it returns False tell the user it doesn't terminate
	  * Else tell the user that the grammar does terminate
	  */
	 if (CheckTerminates(Chosennode) == False)
	 {
	    XicaStatus(Statw, "WARNING: [%s] does not terminate\n",
		    Chosennode->symbol);
	 }
	 else
	 {
	    XicaStatus(Statw, "[%s] terminates\n", Chosennode->symbol);
	 }
      }
      else
      {
	 XicaStatus(Statw, "Select a non-empty place holder\n");
      }
      break;

   case TriU:
      for (i = 0; i < Ndx; i++)
      {
	 nptr = GetDefinition(Info[i].symbol);
	 if ((nptr             != NULL) &&
	     (Info[i].terminal == TriF) &&
	     (Info[i].symtype  == Part))
	 {
	    /*
	     * Initialize the symbol table for doing this
	     */
	    InitTerminate();

	    if (CheckTerminates(nptr) == False)
	    {
	       XicaStatus(Statw, "WARNING: [%s] does not terminate\n",
		       nptr->symbol);
	    }
	    else
	    {
	       XicaStatus(Statw, "[%s] terminates\n", nptr->symbol);
	    }
	 } /***** IF a definition exists */
      } /***** FOR each symbol in the grammar */
      break;
   }
   
   return;
}




/*
   FUNCTION: 		CreateSemanticMenu

   Synopsis: 
       Create the menu list of semantic view options.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateSemanticMenu(parent)

Widget
   parent;        /* The parent of the menu list */

{
   Arg
      args[MAXARGS];/* List of arguments */
   int
      cnt;        /* Number of arguments */
   Widget
      baresel1,   /* Barest Root selection */
      baresel2,   /* Barest Symbol selection */
      expsel1,    /* Expanded Root selection */
      expsel2,    /* Expanded Symbol selection */
      inedsel,    /* Instance Editor invocation */
      linesel1,   /* Linear Root selection */
      linesel2,   /* Linear Symbol selection */
      termsel1,   /* Linear Root selection */
      termsel2,   /* Linear Symbol selection */
      termsel3;   /* Linear Symbol selection */

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);           cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Linear (Root)"); cnt++;
   linesel1 = XtCreateManagedWidget("semsel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(linesel1, XtNcallback, View, LinearRoot);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);             cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Linear (Symbol)"); cnt++;
   linesel2 = XtCreateManagedWidget("semsel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(linesel2, XtNcallback, View, LinearSym);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);           cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Barest (Root)"); cnt++;
   baresel1 = XtCreateManagedWidget("baresel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(baresel1, XtNcallback, View, BarestRoot);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);             cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Barest (Symbol)"); cnt++;
   baresel2 = XtCreateManagedWidget("baresel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(baresel2, XtNcallback, View, BarestSym);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);             cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Expanded (Root)"); cnt++;
   expsel1 = XtCreateManagedWidget("expsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(expsel1, XtNcallback, View, ExpandRoot);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);               cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Expanded (Symbol)"); cnt++;
   expsel2 = XtCreateManagedWidget("expsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(expsel2, XtNcallback, View, ExpandSym);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);                      cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Check Termination (Root)"); cnt++;
   termsel1 = XtCreateManagedWidget("termsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(termsel1, XtNcallback, SymbolTerm, TriT);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);                        cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Check Termination (Symbol)"); cnt++;
   termsel2 = XtCreateManagedWidget("termsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(termsel2, XtNcallback, SymbolTerm, TriF);

   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, True);
   cnt++;
   XtSetArg(args[cnt], XtNlabel,    "Check Termination (All Symbols)"); cnt++;
   termsel3 = XtCreateManagedWidget("termsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(termsel3, XtNcallback, SymbolTerm, TriU);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   /*
    * For the time-being the instance editor cannot be invoked from DeveGram
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNsensitive, False);                    cnt++;
   XtSetArg(args[cnt], XtNlabel,     "Invoke Instance Editor"); cnt++;
   inedsel = XtCreateManagedWidget("inedsel", smeBSBObjectClass,
				   parent, args, cnt);
/*
   XtAddCallback(inedsel, XtNcallback, Invoke, NULL);
*/
   return;
}
