/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: sgml.c,v $
   Purpose:     
       To generate a legal SGML DTD that expresses the same grammar
       as what has been enterred in DeveGram.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.7 $
   Date:        $Date: 92/06/09 11:29:55 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  RightSide
					   GetDefinition
			                   GetSymbolType
					   IsFirstNode
                                           PrintOccur
   EXPORT  DTD_Declaration
                                           GetMaxNameLength
   EXPORT  DTD_Tokens

   EXPORT  DTD_Entities

   EXPORT  DTD_Elements
                           RightSide
			   DTD_Elements
                                           GetSymbolType
*/

/* HISTORY
   $Log:	sgml.c,v $
 * Revision 1.7  92/06/09  11:29:55  ware
 * 
 * CVS:
 * ----------------------------------------------------------------------
 * automatically CVS: CVS: Committing in CVS: CVS: Modified Files: CVS:
 * sgml.c CVS:
 * ----------------------------------------------------------------------
 * Added three environment variables so the top, middle and end of an
 * SGML DTD could be customized.  Each variable specifies a file to be
 * used.  If the variables is not set or the file does not exist, the
 * default values are used.  The variables are
 * ICA_DEVEGRAM_{TOP,MID,END}.  The part between TOP and MID is
 * "LCNMCHAR" and "UCNMCHAR".  The part between MID and END sets
 * "QUANTITY_SGMLREF" for "NAMELEN" and TAGLVL.
 * 
 * Revision 1.6  92/03/18  14:17:11  ware
 * Working on consistent interface.
 * 
 * Revision 1.5  1992/01/28  16:02:20  cso
 * When generating the entity definitions check to make sure that the
 * symbol is not empty, so that the print occur warning does not come
 * out.
 *
 * Revision 1.4  1992/01/14  17:57:38  cso
 *  - Reorganizing header files.
 *
 * Revision 1.3  1991/11/25  14:37:25  cso
 * Made lint modifications to ensure that function and variable types are
 * being used and declared correctly and consistently.
 *
 * Revision 1.2  1991/10/17  18:37:14  cso
 * I had to update the DTD generation functions so that ENTITY generation
 * mirrored a lot of the ELEMENT generation.  Possible function overlap
 * here.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:40  jbm
 * First HaL mods to ICA
 *
 * Revision 2.2  91/05/28  12:22:46  cso
 * Added tests so that NULL symbols are not printed.
 * 
 * Revision 2.1  91/05/28  10:15:52  cso
 * Added the capability to generate entities so some extra variables
 * were unnecessary per lint.
 * 
 * Revision 2.0  91/03/30  13:52:26  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.1  91/02/22  09:57:07  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "generate.h"
#include "get.h"
#include "main.h"
#include "node.h"
#include "symtab.h"

/* Globals */

/* This character string makes up the top part of the SGML declaration. */
static char
   *Topdecl[] =
{
   "<!SGML  \"ISO 8879-1986\"",
   "-- Default SGML declaration using the Reference concrete syntax --",
   "",
   "CHARSET",
   "         BASESET  \"ISO 646-1983//CHARSET",
   "                   International Reference Version (IRV)//ESC 2/5 4/0\"",
   "         DESCSET  0 256 0",
   "",
   "CAPACITY PUBLIC \"ISO 8879-1986//CAPACITY Reference//EN\"",
   "SCOPE    DOCUMENT",
   "SYNTAX   ",
   "         SHUNCHAR CONTROLS 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18",
   "                           19 20 21 22 23 24 25 26 27 28 29 30 31 127 255",
   "         BASESET  \"ISO 646-1983//CHARSET",
   "                   International Reference Version (IRV)//ESC 2/5 4/0\"",
   "         DESCSET  0 128 0",
   "         FUNCTION RE          13",
   "                  RS          10",
   "                  SPACE       32",
   "                  TAB SEPCHAR  9",
   "         NAMING   LCNMSTRT \"\"",
   "                  UCNMSTRT \"\"",
   0
};

/* This character string makes up the middle part of the SGML declaration.
 After Some of the naming variables have been done. */
static char
   *Middecl[] =
{
   "                  NAMECASE GENERAL YES",
   "                           ENTITY  NO",
   "         DELIM    GENERAL  SGMLREF",
   "                  SHORTREF SGMLREF",
   "         NAMES    SGMLREF",
   0
};

/* This character string makes up the bottom part of the SGML declaration.
 This string occurs after the QUANTITY section has been completed.  By default
 all of the FEATURES are set to NO, but eventually, CONCUR will be allowed. */
static char
   *Features[] =
{
   "FEATURES",
   "         MINIMIZE DATATAG  NO     OMITTAG   NO     RANK     NO",
   "                  SHORTTAG NO",
   "         LINK     SIMPLE   NO     IMPLICIT  NO     EXPLICIT NO",
   "         OTHER    CONCUR   NO     SUBDOC    NO     FORMAL   NO",
   "APPINFO  NONE",
   ">",
   0
};




/*
   FUNCTION: 		RightSide

   Synopsis: 
       Because of the use of ENTITIES in SGML, I was treating them as macros,
       so I had to change the way that righthand sides were generated.  
       However, this will change and this function will probably disappear.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/05/28   cso    Added a test so that NULL symbols are not printed.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
RightSide(nptr, caller, fptr)

Node
   *nptr;  /* Node whose rightside we are creating */
ProdStyle
   caller; /* What style needs to be output: BNF, DTD */
FILE
   *fptr;  /* The file that is being output */

{
   Alt
      *aptr; /* Loop pointer to do each of nptr's definitions */
   Node
      *qptr; /* Loop pointer to each node of a definition */

   /*
    * If nptr is NULL, or if it is not an ELEMENT, return.  For each
    * definition of nptr output them as a righthand side of a
    * production.
    */
   if ((nptr == NULL) || (GetSymbolType(nptr->symbol) != Part))
   {
      return;
   }
   for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
   {
      /*
       * If this is not the first definition, then the | operator
       * needs to be used.
       */
      if (aptr->prev != NULL)
      {
	 switch (caller)
	 {
	 case BNF:
	    fprintf(fptr, "\t|\t");
	    break;
	    
	 case DTD:
	    fprintf(fptr, " |\n\t\t\t(");
	    break;
	 }
      }

      /*
       * Now output each node in the production
       */
      for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
      {
	 if (*qptr->symbol != '\0')
	 {
	    /*
	     * If qptr is not the first node, output a ","
	     */
	    if (IsFirstNode(qptr) == False)
	    {
	       fprintf(fptr, ", ");
	    }
	    
	    /*
	     * Depending on the symbol type, output it appropriately.
	     */
	    switch (GetSymbolType(qptr->symbol))
	    {
	    default:
	       break;
	       
	    case Part:
	       if (IsTerminal(qptr->symbol) == TriT)
	       {
		  fprintf(fptr, "%%%s;", qptr->symbol);
	       }
	       else
	       {
		  fprintf(fptr, "%s", qptr->symbol);
	       }
	       break;
	       
	    case Group:
	       fprintf(fptr, "(%%%s;)", qptr->symbol);
	       break;
	       
	    case File:
	       break;
	    } /* SWITCH */
	    
	    /*
	     * After the symbol/group is output, output its occurrence
	     */
	    PrintOccur(fptr, *qptr->occurrence, "", "", "?", "*", "+");
	 }
      } /* FOR each qptr in a production */

      /*
       * Output the end of a production,
       * depending on the caller.
       */
      switch (caller)
      {
      case BNF:
	 fprintf(fptr, "\n");
	 break;
	 
      case DTD:
	 fprintf(fptr, ")");
	 break;
      }
   } /* FOR each alternative definition of nptr  */
   return;
}




/*
 * FUNCTION:	copy_file_or_string
 * Synopsis:	If the specified filename is non-null then copy the contents
 *		to fptr.  Otherwise copy the list of lines to fptr
 */

static void
copy_file_or_string (filename, strlist, fptr)
	char	*filename;
	char	**strlist;
	FILE	*fptr;
{
	FILE	*inp_fptr;
	int	ch;
	int	i; /* Loop counter */

	if (filename != NULL && (inp_fptr = fopen (filename, "r")) != NULL)
	{
		while ((ch = getc (inp_fptr)) != EOF)
			putc (ch, fptr);
		fclose (inp_fptr);
	}
	else
	{
		for (i = 0; strlist[i] != 0; i++)
		{
			fprintf(fptr, "%s\n", strlist[i]);
		}
	}
}

/*
   FUNCTION: 		DTD_Declaration

   Synopsis: 
       Output the SGML declaration.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       Features  Used the string array to output the declaration
       Topdecl   Used the string array to output the declaration

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DTD_Declaration(fptr)

FILE
   *fptr;  /* File being output */

{
   String
      oddstr; /* To hold the string of odd characters to be output 
		 as allowable within tag names */

   /*
    * Output the top part of the declaration
    * Output odd characters
    * Output the middle part of the declaration
    * Output the quantity section
    * Output the features part of the declaration
    */
   copy_file_or_string (getenv ("ICA_DEVEGRAM_TOP"), Topdecl, fptr);

   oddstr = XtNewString(GetOddChars());
   fprintf(fptr, "                  LCNMCHAR \"%s\"\n", oddstr);
   fprintf(fptr, "                  UCNMCHAR \"%s\"\n", oddstr);

   copy_file_or_string (getenv ("ICA_DEVEGRAM_MID"), Middecl, fptr);
   fprintf(fptr, "         QUANTITY SGMLREF\n");
   fprintf(fptr, "                  NAMELEN  %d\n", GetMaxNameLength());
   fprintf(fptr, "                  TAGLVL   64\n");
   fprintf(fptr, "\n");

   copy_file_or_string (getenv ("ICA_DEVEGRAM_END"), Features, fptr);
   fprintf(fptr, "\n");
   return;
}


	


/*
   FUNCTION: 		DTD_Tokens

   Synopsis: 
       Output terminal symbols as general ENTITY declarations to be used
       macro substitutes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       Info    To access all Group symbols
       Ndx     To limit access

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DTD_Tokens(fptr)

FILE
   *fptr;

{
   int
      i;  /* Loop counter to go through the symbol table */

   /*
    * For each symbol
    * If it is a terminal symbol output an entity to refer to it as #PCDATA
    */
   for (i = 0; i < Ndx; i++)
   {
      if (Info[i].terminal == TriT)
      {
	 fprintf(fptr, "<!ENTITY %% %s \"#PCDATA\" >\n\n", Info[i].symbol);
      }
   }
   return;
}




/*
   FUNCTION: 		DTD_Entities

   Synopsis: 
       Output Group symbols as Parameter ENTITY declarations to be used in
       the ELEMENT declaration section.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
       91/05/23 cso      Cosmetic changes to the ENTITY decls.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DTD_Entities(nptr, fptr)

Node
   *nptr;  /* Node under consideration */
FILE
   *fptr;  /* File being output */

{
   Alt
      *aptr;  /* Loop pointer to visit each of my definitions */
   Node
      *qptr;  /* Loop pointer to visit all of my children */

   /*
    * If a node and its definitions exist
    * Do a depth-first search to find a Group symbol
    */
   if ((nptr != NULL) && (nptr->alts != NULL))
   {
      /*
       * Find the leaf node
       * If that node is a Group symbol
       * Generate an ENTITY declaration for it
       */
      for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
      {
	 for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	 {
	    DTD_Entities(qptr, fptr);
	 }
      }
      if ((GetSymbolType(nptr->symbol) == Group) && 
	  (Visited(nptr->symbol) == 1))
      {
	 /*
	  * If this is the first time it was seen
	  * Generate the ENTITY declaration
	  */
	 fprintf(fptr, "<!ENTITY %% %s \"(", nptr->symbol);

	 /*
	  * If the node has more than one definition, output an extra "("
	  * Similar to ((cond1) | (cond2)), the extra "(" for cond1 is output
	  */
	 if (nptr->alts->next != NULL)
	 {
	    fprintf(fptr, "(");
	 }

	 for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
	 {
	    if (IsFirstAlt(aptr) == False)
	    {
	       fprintf(fptr, " |\n\t\t\t(");
	    }
	    for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	    {
	       if (*qptr->symbol != '\0')
	       {
		  if (IsFirstNode(qptr) == False)
		  {
		     fprintf(fptr, ", ");
		  }
		  switch (GetSymbolType(qptr->symbol))
		  {
		  default:
		     break;

		  case Part:
		     if (IsTerminal(qptr->symbol) == TriT)
		     {
			fprintf(fptr, "%%%s;", qptr->symbol);
		     }
		     else
		     {
			fprintf(fptr, "%s", qptr->symbol);
		     }
		     break;

		  case Group:
		     fprintf(fptr, "(%%%s;)", qptr->symbol);
		     break;

		  case File:
		     break;
		  } /* SWITCH */
		  
		  /*
		   * After the symbol/group is output, output its occurrence
		   */
		  PrintOccur(fptr, *qptr->occurrence, "", "", "?", "*", "+");
	       } /***** IF it is not empty */
	    } /* FOR each node */

	    fprintf(fptr, ")");
	    
	 } /* FOR each production */

	 /*
	  * If the node has more than one definition, output an extra ")"
	  * Similar to ((cond1) | (cond2)), the extra ")" for cond2 is output
	  */
	 if (nptr->alts->next != NULL)
	 {
	    fprintf(fptr, ")");
	 }

	 fprintf(fptr, "\" >\n\n");
      } /* IF a Group */
   } /* if a node with definitions */
   return;
}




/*
   FUNCTION: 		DTD_Elements

   Synopsis: 
       Generate an SGML DTD - easier said than done.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DTD_Elements(nptr, fptr)

Node
   *nptr;     /* Node under consideration */
FILE
   *fptr;     /* File being output */

{
   Alt
      *aptr,  /* Loop pointer to look at each definition */
      *bptr;  /* Loop pointer to look at each definition of a group symbol */
   Node
      *qptr,  /* Loop pointer to look at each node in aptr's definition */
      *rptr;  /* Loop pointer to look at each node in bptr's definition */

   /*
    * If a node has a definition
    */
   if ((nptr != NULL) && (nptr->alts != NULL))
   {
      if (GetSymbolType(nptr->symbol) == Part)
      {
	 /*
	  * Output an ELEMENT definition,
	  */
	 fprintf(fptr, "<!ELEMENT %s\t- -  (", nptr->symbol);
	 
	 /*
	  * If the node has more than one definition, output an extra "("
	  * Similar to ((cond1) | (cond2)), the extra "(" for cond1 is output
	  */
	 if (nptr->alts->next != NULL)
	 {
	    fprintf(fptr, "(");
	 }
	 
	 /*
	  * Generate the rest of the right side of the definition
	  */
	 RightSide(nptr, DTD, fptr);
	 
	 /*
	  * If the node has more than one definition, output an extra ")"
	  * Similar to ((cond1) | (cond2)), the extra ")" for cond2 is output
	  */
	 if (nptr->alts->next != NULL)
	 {
	    fprintf(fptr, ")");
	 }
	 
	 /*
	  * Output the end of the ELEMENT definition
	  */
	 fprintf(fptr, ">\n\n");
      }
      
      /*
       * Expand all of the definitions' nodes, 
       * For each definition, for each node in it, expand
       */
      for (aptr = nptr->alts; aptr != NULL; aptr = aptr->next)
      {
	 for (qptr = aptr->prod; qptr != NULL; qptr = qptr->next)
	 {
	    /*
	     * Expanding a node gets a little complicated because
	     * there is a difference between Part and Group
	     * Part == ELEMENT, Group == ENTITY
	     * Presently I treat ENTITY as macro substitutes, so...
	     * If a node is a Part, expand it normally,
	     * Else for each of the Group's definitions and definitions' nodes
	     * expand them
	     */
	    if (GetSymbolType(qptr->symbol) == Part)
	    {
	       DTD_Elements(qptr, fptr);
	    }
	    else
	    {
	       for (bptr = qptr->alts; bptr != NULL; bptr = bptr->next)
	       {
		  for (rptr = bptr->prod; rptr != NULL; rptr = rptr->next)
		  {
		     DTD_Elements(rptr, fptr);
		  }
	       }
	    }
	 }
      }
   }
   return;
}
