/*
  $Log:	types.h,v $
 * Revision 1.6  92/03/18  14:17:48  ware
 * Working on consistent interface.
 * 
 * Revision 1.5  1992/01/28  15:44:15  cso
 * Removed the use of left and right alts - they weren't being used anyway.
 *
 * Revision 1.4  1992/01/03  20:13:29  cso
 * Removed the old alert types.
 * Rearranged the typedefs alphabetically.
 * Added symbol attribute types.
 *
 * Revision 1.3  1991/11/22  15:35:35  cso
 * 	Added a graphics context to the node type for easier
 * manipualtion of the line drawing in the navigation tree.  Tried to
 * eliminate the number of refreshes occurring for a particular expose
 * event, tough to debug in synchronous mode.
 *
 * Revision 1.2  1991/11/14  14:31:24  cso
 * Made the necessary additions and changes to provide a mechanism to
 * change symbol types between nonterminal types Nonterminal & Grouping.
 * In SGML terms, therefore, intermediate form terms, this is the
 * difference between having element tags and not.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:55  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:53:31  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.11  91/02/22  10:08:46  cso
 * Added some documentation.
 * Added a new type for the direction of alt buttons.
 * 
 * Revision 1.10  90/11/20  16:06:15  cso
 * Removed the type FileString.  Changed exposed to a Boolean.
 * 
 * Revision 1.9  90/11/05  18:35:19  cso
 * Added a NodeListType for the definition pointer.
 * 
 * Revision 1.8  90/10/29  10:32:12  cso
 * Expanded SemanticFlagType to distinguish between Symbol and Root
 * versions of the semantic view.
 * Also changed the type name to Semantic from View.
 * 
 * Revision 1.7  90/10/24  15:50:34  cso
 * Added new enum types based on the old defines in defs.h to make for 
 * better type checking in compiles.
 * 
 * Revision 1.6  90/09/07  09:43:17  cso
 * Added an expose flag to Alt for three-dimensional layout.
 * 
 * Revision 1.5  90/09/03  18:31:46  cso
 * Removed T_Struct, and added, instead, left and right Alt ptrs to NodeType.
 * 
 * Revision 1.4  90/09/03  14:55:42  cso
 * Changed char* to String and Bool to Boolean and a few other cosmetics.
 * 
 * Revision 1.3  90/08/26  17:44:57  cso
 * Added the RCS Log message at the top for these messages.
 * 
*/

/*
  The different types of attributes for a symbol.
 */
typedef enum AttributeType
{
   Attnon   = 1,
   Attterm  = 2,
   Attgrp   = 3,
   Attfile  = 4,
   Attexact = 5,
   Attopt   = 6,
   Attstar  = 7,
   Attplus  = 8
} Attribute;


/*
  Indicate the direction used by the alternative definition buttons.
 */
typedef enum DepthDirectionType
{
   Previous = 0,
   Next     = 1
} DepthDirection;

/*
  The production style produced by Rightside().
 */
typedef enum ProdStyleType
{
   BNF = 0,
   DTD = 1
} ProdStyle;

/*
  The six types of semantic information available.
 */
typedef enum SemanticFlagType
{
   LinearRoot = 0,
   LinearSym  = 1,
   BarestRoot = 2,
   BarestSym  = 3,
   ExpandRoot = 4,
   ExpandSym  = 5
} SemanticFlag;

/*
  Used for indicating the type of symbol.  As of 02-19-91, the File type is
  not implemented.
 */
typedef enum SymbolType
{
   Unknown = -1,
   Part    =  0,
   Group   =  1,
   File    =  2
} SymType;

/*
  This is used for instances when true and false are insufficient because
  unknown conditions exist as well.
 */
typedef enum TriStateType
{
   TriU = -1,
   TriF =  0,
   TriT =  1
} TriState;




/*
  A node in the node data structure (they are all the same, except the Root
  never has a parent nor an occurrence nor a previous or next sibling).
 */
typedef struct NodeType
{
   GC
      treegc;      /* The line graphics context to use        */
   String
      symbol,      /* Name of a symbol in the grammar         */
      occurrence;  /* [_?*+^]                                 */
   struct NodeType
      *parent,     /* Ptr to parent node                      */
      *next,       /* Ptr to the next node in a prod          */
      *prev;       /* Ptr to the prev node in a prod          */
   struct AltType
      *alts;       /* Ptr to definitions of this node         */
   Widget
      nodew,       /* Widget displaying this node             */
      occurw,      /* Widget displaying the occurrence        */
      treew,       /* Widget in the tree display              */
      typew;       /* Widget displaying a symbol's type       */
} Node;


/*
  A nodelist points to each instance of a node in a definition.
 */
typedef struct NodeListType
{
   Node
      *nodeptr; /* A pointer to where this node occurs     */
   struct NodeListType
      *next,    /* Pointer to the next ptr in the list     */
      *prev;    /* Pointer to the previous ptr in the list */
} NodeList;


/*
  The structure connecting alternative definitions of a node.
 */
typedef struct AltType
{
   Node
      *whom,       /* Ptr to node whom this is for            */
      *prod;       /* Ptr to a list of nodes                  */
   struct AltType
      *next,       /* Ptr to the next alternative             */
      *prev;       /* Ptr to the prev alternative             */
   struct AuxType
      *auxinf;     /* Ptr to aux info for this alt            */
   Boolean
      exposed;     /* Flag for tree display determination     */
} Alt;


/*
  The structure connecting auxiliary information for a particular definition.
 */
typedef struct AuxType
{
   Alt
      *which;      /* Ptr to alt which this is for            */
   String  
      type,        /* Type of info, e.g., yacc                */
      info;        /* String of information                   */
   struct AuxType
      *next,       /* Ptr to next auxinfo                     */
      *prev;       /* Ptr to prev auxinfo                     */
} Aux;


/*
  The symbol table structure.
 */
typedef struct GrammarDataType
{
   String
      symbol,       /* Name of a symbol in the grammar         */
      filename;     /* Name of a file when symtype=2           */
   SymType
      symtype;      /* Symbol type [file=2, group=1, part=0]   */
   Boolean
      defined;      /* Flag to indicate a symbol is defined    */
   NodeList
      *definition;  /* Ptr to a list of Nodes where this symbol
                       is used everywhere in the tree.
                       There should be one per count.          */
   int
      count,        /* Count of the use of this symbol         */
      star,         /* Count of the use of this occurrence     */
      plus,         /* Count of the use of this occurrence     */
      optional,     /* Count of the use of this occurrence     */
      visits;       /* for tree traversal, # of visits         */
   TriState
      terminal,     /* Flag to indicate terminal               */
      terminate;    /* Flag to indicate termination            */
} GrammarData;
