/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: undlist.c,v $
   Purpose:
       To handle the undefined symbol list.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/05/29 14:46:41 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   EXPORT  RemoveUndItem
                                           HighlightDefinedStatus
   EXPORT  InsertUndItem
                                           HighlightDefinedStatus
                                           MakeUpper
   HIDDEN  UndSelect
                                           GetDefinition
					   UpdateInterface
   EXPORT  CreateUndSymbolList
*/

/* HISTORY
   $Log:	undlist.c,v $
 * Revision 1.8  92/05/29  14:46:41  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.7  92/03/18  14:17:52  ware
 * Working on consistent interface.
 * 
 * Revision 1.6  1992/02/20  04:46:28  cso
 * Made the insertion routines case sensitive although the
 * alphabetization is case insensitive, i.e., find the spot where the
 * symbol should go case-insensitively (there's a new term...), but check
 * to see if it is there in a case-sensitive manner.
 *
 * Revision 1.5  1992/01/28  15:55:41  cso
 * Added a define action for double clicking in Undefined list
 *
 * Revision 1.4  1992/01/14  20:24:33  cso
 *  - Reorganizing header files.
 *  - Check to make sure a non-malloc'd string is not being freed.
 *
 * Revision 1.3  1991/11/20  00:05:41  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.2  1991/10/17  18:39:34  cso
 * Added a level of debugging so that it is not the default.
 *
 * Revision 1.1.1.3  1991/10/04  20:16:57  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:53:42  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.8  91/02/22  10:04:05  cso
 * Completed documentation.
 * Changed the way Undndx is used to be more like Ndx.
 * 
 * Revision 1.7  90/11/05  18:30:56  cso
 * Added function template to all functions, but still need to fill it in.
 * Using Make Upper so that it'll alphabetize correctly and fixed it so that
 * it shows a NULL list when appropriate.
 * Else, cosmetics.
 * 
 * 
 * Revision 1.6  90/10/24  15:39:40  cso
 * Changed the ordering of parameters to UpdateInterface.
 * 
 * Revision 1.5  90/10/02  13:39:46  cso
 * Shortened list size.
 * 
 * Revision 1.4  90/09/03  18:30:12  cso
 * Function name changes.
 * 
 * Revision 1.3  90/09/03  14:43:04  cso
 * Changed function names to be consistent.
 * 
 * Revision 1.2  90/08/29  17:52:05  cso
 * Changed the look of its display.
 * 
 * Revision 1.1  90/08/26  17:43:37  cso
 * Initial revision
 * 
 * Revision 1.2  90/08/26  14:13:22  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "display.h"
#include "get.h"
#include "main.h"
#include "status.h"
#include "utils.h"




/*
   FUNCTION: 		DefineSymAction

   Synopsis: 
       A way to have define a symbol with a double-click from the
       undefined symbols list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DefineSymAction(w, event, params, num)

Widget
   w;          /* The widget calling this procedure */
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   XtCallCallbacks(XtNameToWidget(Toplevel, "*definesel"),
		   XtNcallback, NULL);
   return;
}



/*
   FUNCTION: 		RemoveUndItem

   Synopsis: 
       Remove an undefined symbol from that list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/07/31   cso    Added a way to show that a symbol is defined.

   Globals used:
       none

   Globals Modified:
       Undndx       To limit the search for the symbol
       Undsymlist   A symbol is removed

   Return value:	void

   Range:		none
*/

void
RemoveUndItem(str)

String
   str;  /* The symbol to remove */

{
   int
      i, /* Loop counter to search for the symbol */
      j; /* Index to adjust the list after removing the symbol */

   /*
    * Search for the symbol
    */
   for (i = 0; ((i < Undndx) && (strcmp(str, Undsymlist[i]) != 0)); i++)
   {
   }

   /*
    * If the index i is less than Undndx, 
    * means that the symbol was found at position i
    */
   if (i < Undndx)
   {
      /*
       * From i down the list move every string up, freeing up strings
       * Decrement Undndx
       * Reinitialize the bottom string
       * Update the list widget
       * Update the defined status of that symbol in the display
       */
      for (j = i; j < (Undndx - 1); j++)
      {
	 XtFree(Undsymlist[j]);
	 Undsymlist[j] = XtNewString(Undsymlist[j+1]);
      }
      Undndx--;
      XtFree(Undsymlist[Undndx]);
      Undsymlist[Undndx] = NULL;
#if (DEBUG == 2)
      fprintf(stderr, "Undndx = [%d]\n", Undndx);
#endif
      XawListChange(XtNameToWidget(Frame, "*listframe*undlist"),
		    Undsymlist, Undndx, 0, True);
      HighlightDefinedStatus(str, True);
   }
   return;
}




/*
   FUNCTION: 		InsertUndItem

   Synopsis: 
       Insert a symbol into the undefined symbol list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/07/31   cso    Added a way to show that a symbol is defined.

   Globals used:
       none

   Globals Modified:
       Undndx       Index will be incremented
       Undsymlist   A symbol may be added

   Return value:	void

   Range:		none
*/

void
InsertUndItem(str)

String
   str;          /* The symbol to add */

{
   int
      i,         /* Loop pointer to search for the symbol */
      j;         /* Index to augment the list */
   String
      upperstr;  /* The symbol in uppercase */

   /*
    * Highlight all of the tree widgets to show that it is undefined
    * Get the upper case version of the symbol
    * Search to see if the symbol is already there
    */
   HighlightDefinedStatus(str, False);
   upperstr = MakeUpper(str);
   for (i = 0;
	((i < Undndx) &&
	 (strcmp(upperstr, MakeUpper(Undsymlist[i])) > 0));
	i++)
   {
   }

   /*
    * If the symbol is not found (i == Undndx) or
    * If the symbol should be inserted at the current spot
    * Insert the symbol in the list
    */
   if ((i == Undndx) ||
       (strcmp(upperstr, MakeUpper(Undsymlist[i])) < 0) ||
       (strcmp(str, Undsymlist[i]) != 0))
   {
      /*
       * Open up a spot for the symbol to be inserted
       * Insert the symbol
       * Increment Undndx
       * Update the list widget
       */
      for (j = Undndx; j > i; j--)
      {
	 if (j != Undndx)
	 {
	    XtFree(Undsymlist[j]);
	 }
	 Undsymlist[j] = XtNewString(Undsymlist[j-1]);
      }
      Undsymlist[i] = XtNewString(str);
      Undndx++;
      XawListChange(XtNameToWidget(Frame, "*listframe*undlist"),
		    Undsymlist, Undndx, 0, True);
   }
   return;
}




/*
   FUNCTION: 		UndSelect

   Synopsis: 
       The user has selected a symbol in the list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static void

   Range:		none
*/

static void
UndSelect(w, client_data, call_data)

Widget
   w;           /* The list widget, the user selected */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   XawListReturnStruct
      *current; /* Pointer to the structure containing the index into 
		   the list and the string */
   Node
      *oldptr;  /* Pointer to the current node */

   /*
    * Get the symbol selected in the list by the user
    * Unhighlight the symbol
    */
   current = XawListShowCurrent(w);
   XawListUnhighlight(w);
#if (DEBUG == 2)
   fprintf(stderr, "selection = [%s], position = [%d]\n",
	   current->string, current->list_index);
#endif

   /*
    * Remember the current node
    * Get the definition for that symbol, assign it to Tree
    * Update the interface
    */
   oldptr = Tree;
   Tree = GetDefinition(current->string);
   UpdateInterface(oldptr, Tree);
   return;
}




/*
   FUNCTION: 		CreateUndSymbolList

   Synopsis: 
       Create the list widget for the undefined symbols.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void CreateUndSymbolList( parent, col, top, bottom )
Widget
   parent,
   col,             /* column gadget */
   top, bottom;     /* row gadgets */
{
   Widget
      undlabel,      /* Label describing the list */
      undlist,       /* List widget */
      undview;       /* Viewport into the list widget */
   XtTranslations
      listtrans;
   static String
      listtransstr = "<Btn1Up>(2):         DefineSymAction()\n\
                      <Btn1Down>,<Btn1Up>: Set() Notify()";

   listtrans = XtParseTranslationTable(listtransstr);

   undlabel = XtVaCreateManagedWidget( "undlabel",
       labelWidgetClass, parent,
       XtNlabel,       "Undefined Symbols:",
       XtNborderWidth, 0,
       XtNtableRow,    top,
       XtNtableCol,    col,
       NULL );

   undview = XtVaCreateManagedWidget( "undview",
       viewportWidgetClass, parent,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNuseBottom,     False,
       XtNuseRight,      True,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     (FRAMEWIDTH/2),
       XtNminWidth,      100,
       XtNprefHeight,    300,
       XtNminHeight,     50,
*/
       XtNtableRow,      bottom,
       XtNtableCol,      col,
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       NULL );

   undlist = XtVaCreateManagedWidget( "undlist",
       listWidgetClass, undview,
       XtNdefaultColumns, 1,
       XtNforceColumns,   True,
       XtNlist,           Undsymlist,
       XtNnumberStrings,  0,
       NULL );
   XtAddCallback( undlist, XtNcallback, UndSelect, NULL );
   XtOverrideTranslations( undlist, listtrans );

   return;
}
