/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: view.c,v $
   Purpose:
       To show the diffierent views of the grammar.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 14:47:02 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  CreateLabelStr
   EXPORT  DetermineViewPtr
   EXPORT  GenerateViewFile
                                           BareGen
					   ExpandGen
					   LinearGen
   HIDDEN  ViewDone
   HIDDEN  ViewPrint
   HIDDEN  ViewRefresh
                           DetermineViewPtr
			   GenerateViewFile
			                   ResetVisits
   EXPORT  CreateViewShell
                           CreateLabelStr
*/

/* HISTORY
   $Log:	view.c,v $
 * Revision 1.6  92/05/29  14:47:02  boyd
 * Changes for improved resizing, consisting mostly of replacing Form widgets
 * with Group, Table, and Viewform widgets.
 * 
 * 
 * Revision 1.5  92/03/18  14:18:10  ware
 * Working on consistent interface.
 * 
 * Revision 1.4  1992/01/14  17:57:05  cso
 *  - Reorganizing header files.
 *
 * Revision 1.3  1991/11/22  15:35:54  cso
 * 	Added a graphics context to the node type for easier
 * manipualtion of the line drawing in the navigation tree.  Tried to
 * eliminate the number of refreshes occurring for a particular expose
 * event, tough to debug in synchronous mode.
 *
 * Revision 1.2  1991/11/20  00:06:11  cso
 * Reorganized the interface by removing borders from around widgets and
 * relocating widgets.
 * Added a status widget to display info that was being output to stderr.
 * The xterm used to instantiate DeveGram is no longer required b/c of
 * this widget.
 * Created a couple of constants for maximum arguments used and width of
 * the interface.
 *
 * Revision 1.1.1.3  1991/10/04  20:17:05  jbm
 * First HaL mods to ICA
 *
 * Revision 2.1  91/05/28  10:19:47  cso
 * Changed the shell to a popup.
 * and added an explicit return.
 * 
 * Revision 2.0  91/03/30  13:54:21  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.1  91/02/22  10:46:55  cso
 * Initial revision
 * 
 * Revision 1.13  91/02/22  09:57:31  cso
 * Completed documentation.
 * 
 * Revision 1.12  90/11/26  13:17:49  cso
 * name should be changed to view.c.
 * Added functions to break down Show and Refresh routines.
 * Fixed changing files by unmanaging/managing widget.
 * Added a label to the shell to let user know which shell is showing what.
 * 
 * Revision 1.11  90/11/20  15:55:29  cso
 * changed the name from "tree" to view.
 * 
 * Revision 1.10  90/11/05  18:25:26  cso
 * Added function template to all functions, but still need to fill it in.
 * Else, cosmetics.  Still trying to get Refresh to work.
 * 
 * Revision 1.9  90/10/29  10:28:30  cso
 * Added the appropriate decls to make it work.
 * Also, trying to use separate shells for each individual menu choice, and 
 * allowing the Refresh button to make it reread the file, but it is not
 * working quite right yet.
 * 
 * Revision 1.8  90/10/24  15:36:51  cso
 * Most of the changes were cosmetic, but were based on the changes occurring
 * in types.h, i.e., I eliminated Showform.
 * 
 * Revision 1.7  90/10/02  13:36:44  cso
 *  Added #ifdef AUX instead of comments.
 * 
 * Revision 1.6  90/09/03  14:38:53  cso
 * Changed function name to be consistent.
 * 
 * Revision 1.5  90/08/26  17:43:15  cso
 * Comment all calls RE: auxiliary information.
 * 
 * Revision 1.4  90/08/26  14:13:34  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "std.h"
#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "generate.h"
#include "get.h"
#include "main.h"
#include "status.h"
#include "symtab.h"



static Widget ShellParent(w)
Widget w;
/*
   Return the nearest parent of w which is a shell widget.
*/
{
    Widget parent;

    while (NULL != (parent = XtParent(w)) && !XtIsShell(parent))  w = parent;

    return (parent);
}


/*
   FUNCTION: 		CreateLabelStr

   Synopsis: 
       Create a label for the view being presented to the user.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static String

   Range:		none
*/

static String
CreateLabelStr(semtype)

SemanticFlag
   semtype;   /* The type of view being presented to the user */

{
   /*
    * Depending on the type of view, return a label
    */
   switch (semtype)
   {
   default:
      return("");
      break;
      
   case LinearRoot:
      return("  Linear View (Root)");
      break;
      
   case LinearSym:
      return("  Linear View (Symbol)");
      break;
      
   case BarestRoot:
      return("  Barest View (Root)");
      break;
      
   case BarestSym:
      return("  Barest View (Symbol)");
      break;
      
   case ExpandRoot:
      return("  Expanded View (Root)");
      break;
      
   case ExpandSym:
      return("  Expanded View (Symbol)");
      break;
   }
   return("");
}




/*
   FUNCTION: 		DetermineViewPtr

   Synopsis: 
       Determine the pointer to use for generating the view.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Node

   Range:		none
*/

Node
*DetermineViewPtr(semtype)

SemanticFlag
   semtype;  /* Type of view */

{
   /*
    * Depending on the semantic view, return the pointer to use
    */
   switch (semtype)
   {
   default:
      
   case LinearRoot:
      
   case BarestRoot:
      
   case ExpandRoot:
      return(Root);
      break;
      
   case LinearSym:
      
   case BarestSym:
      
   case ExpandSym:
      if (Chosennode != NULL)
      {
	 return(Chosennode);
      }
      else
      {
	 return(Root);
      }
      break;
   }

   return(Root);
}




/*
   FUNCTION: 		GenerateViewFile

   Synopsis: 
       Generate the appropriate semantic information into a file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateViewFile(filename, nptr, semtype)

String
   filename;  /* File to be used for the generation function */
Node
   *nptr;     /* The node to be use start the generation */
SemanticFlag
   semtype;   /* The semantic view */

{
   FILE
      *fptr;  /* File pointer */

   /*
    * Open the file to hold the semantic info
    * If unable to open one, inform the user
    */
   if ((fptr = fopen(filename, "w")) == NULL)
   {
      XicaStatus(Statw, "Could not open View file [%s]\n", filename);
   }
   else
   {
      /*
       * Depending on the view, generate the semantic info
       * Close the file
       */
      switch (semtype)
      {
      default:
	 break;

      case LinearRoot:

      case LinearSym:
	 LinearGen(nptr, fptr);
	 break;

      case BarestRoot:

      case BarestSym:
	 BareGen(nptr, fptr, -1);
	 break;

      case ExpandRoot:

      case ExpandSym:
	 ExpandGen(nptr, fptr, -1);
	 break;
      }
      fclose(fptr);
   }
   return;
}




/*
   FUNCTION: 		ViewDone

   Synopsis: 
       The function for indicating the user is done with this view.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static void

   Range:		none
*/

static void
ViewDone( w, client_data, call_data )
Widget
   w;           /* Used to access the parent widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */
{
   Widget
      semshell = (Widget) client_data,
      semdefs;  /* The widget showing the semantic info */
   String
      filename, /* The file containing the semantic info */
      sysstr;   /* System call to remove the file */

   /*
    * Access the text widget showing the semantic info
    * Get the filename containing the info
    */
   semdefs  = XtNameToWidget( ShellParent(w), "*semdefs" );
   filename = XtNewString( GetViewFilename(semdefs) );
#ifdef DEBUG
   fprintf(stderr, "Deleting viewing file [%s]\n", filename);
#endif

   /*
    * Remove the file
    */
   sysstr   = XtMalloc((unsigned) strlen(filename) + 32);
   sprintf(sysstr, "/usr/bin/rm -f %s", filename);
   system(sysstr);

   /*
    * Destroy the shell
    */
   XtPopdown(semshell);
   XtDestroyWidget(semshell);

   return;
}




/*
   FUNCTION: 		ViewPrint

   Synopsis: 
       Print the semantic info on the line printer.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static void

   Range:		none
*/

static void
ViewPrint(w, client_data, call_data)

Widget
   w;           /* Used to access the parent widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   Widget
      semdefs;  /* The widget showing the semantic info */
   String
      filename, /* The file containing the semantic info */
      sysstr;   /* System call to print the file */

   /*
    * Access the text widget showing the semantic info
    * Get the filename containing the info
    */
   semdefs  = XtNameToWidget( ShellParent(w), "*semdefs" );
   filename = XtNewString(GetViewFilename(semdefs));
#ifdef DEBUG
   fprintf(stderr, "Printing viewing file [%s]\n", filename);
#endif

   /*
    * Make the system call to use the default line printer
    */
   sysstr = XtMalloc((unsigned) strlen(filename) + 32);
   sprintf(sysstr, "/usr/ucb/lpr %s", filename);
   system(sysstr);
   return;
}




/*
   FUNCTION: 		ViewRefresh

   Synopsis: 
       Refresh the view with perhaps a different node.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	static void

   Range:		none
*/

static void
ViewRefresh(w, semtype, call_data)

Widget
   w;           /* Used to access the parent widget */
SemanticFlag
   semtype;     /* The semantic view being shown the user */
caddr_t
   call_data;   /* Unused */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Node
      *nptr;
   Widget
      semdefs;  /* The widget showing the semantic info */
   String
      filename; /* The file containing the semantic info */

   /*
    * Access the text widget showing the semantic info
    * Get the filename containing the info
    */
   semdefs  = XtNameToWidget( ShellParent(w), "*semdefs" );
   filename = XtNewString(GetViewFilename(semdefs));

   /*
    * Reset the visits for the generation of the semantic info
    * Determine the pointer to be used for the generation function
    * Generate the semantic info
    */
   ResetVisits();
#ifdef DEBUG
   fprintf(stderr, "Refreshing file [%s]\n", filename);
#endif
   nptr = DetermineViewPtr(semtype);
   GenerateViewFile(filename, nptr, semtype);

   /*
    * Update the widget showing the user the semantic info
    */
   XtUnmanageChild(semdefs);
   cnt =0;
   XtSetArg(args[cnt], XtNstring, filename);     cnt++;
   XtSetArg(args[cnt], XtNtype,   XawAsciiFile); cnt++;
   XtSetValues(semdefs, args, cnt);
   XtManageChild(semdefs);
   XawTextDisplay(semdefs);
   return;
}




/*
   FUNCTION: 		CreateViewShell

   Synopsis: 
       Create the shell that shows the user the semantic info.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/05/23   cso    Changed the Shell to a popup.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void CreateViewShell( filename, nptr, semtype )
String
   filename;  /* Filename of the semantic info */
Node
   *nptr;     /* Unused */
SemanticFlag
   semtype;   /* The type of semantic info */
{
   int
      cnt;       /* Number of arguments */
   Arg
      args[MAXARGS];/* List of arguments */
   String
      labelstr;  /* Label describing the view */
   Widget
      semdefs,   /* Widget containing semantic info */
      semdone,   /* Widget for the user to indicate that he is finished */
      semframe,  /* Widget for containing the text, and button widgets */
      sembar,
      semfresh,  /* Widget for refreshing the view being shown */
      semlabel,  /* Widget containing the label of the semantic view */
      semprint,  /* Widget for indicating the desire for a print out */
      semshell;  /* The actual shell being popped up for the user to place */

   /*
    * -----------------------------------------
    * |      shell                            |
    * | ------------------------------------- |
    * | |    form                           | |
    * | |                                   | |
    * | | [Refresh] [Print] [Done]  [label] | |
    * | |                                   | |
    * | | [semantic info]                   | |
    * | |                                   | |
    * | ------------------------------------- |
    * -----------------------------------------
    */
   semshell = XtVaCreatePopupShell( "Semantic View",
       applicationShellWidgetClass, Toplevel,
       XtNallowShellResize, True,
       NULL );

   semframe = XtVaCreateManagedWidget( "semframe",
       groupWidgetClass, semshell,
       XtNspacing, 8,
       NULL );

   sembar = XtVaCreateManagedWidget( "sembar",
       groupWidgetClass, semframe,
       XtNorientation, XtorientHorizontal,
       XtNspacing, 1,
       XtNmargin,  7,
       NULL );

   semdone = XtVaCreateManagedWidget( "semdone",
       commandWidgetClass, sembar,
       XtNlabel,  "  Done  ",
       XtNmargin, 0,
       NULL );
   XtAddCallback(semdone, XtNcallback, ViewDone, (XtPointer) semshell );

   semfresh = XtVaCreateManagedWidget( "semfresh",
       commandWidgetClass, sembar,
       XtNlabel,  " Refresh ",
       XtNmargin, 0,
       NULL );
   XtAddCallback(semfresh, XtNcallback, ViewRefresh, (XtPointer) semtype);

   semprint = XtVaCreateManagedWidget( "semprint",
       commandWidgetClass, sembar,
       XtNlabel,  "  Print  ",
       XtNmargin, 0,
       NULL );
   XtAddCallback(semprint, XtNcallback, ViewPrint, NULL);

   labelstr = XtNewString(CreateLabelStr(semtype));
   semlabel = XtVaCreateManagedWidget( "semlabel",
       labelWidgetClass, sembar,
       XtNlabel,       labelstr,
       XtNborderWidth, 0,
       NULL );

   semdefs = XtVaCreateManagedWidget( "semdefs",
       asciiTextWidgetClass, semframe,
       XtNtype,             XawAsciiFile,
       XtNstring,           filename,
       XtNscrollHorizontal, XawtextScrollWhenNeeded,
       XtNscrollVertical,   XawtextScrollAlways,
       XtNpreference,       False,
/*
       XtNprefWidth,        450,
       XtNminWidth,         100,
       XtNprefHeight,       300,
       XtNminHeight,        50,
*/
       XtNalignment,        gaFill,
       XtNstretch,          True,
       XtNmargin,           8,
       NULL );

   XtPopup(semshell, XtGrabNone);

   return;
}
