/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: generate.c,v $
   Purpose:     Generate the translator code.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/03/31 15:12:06 $

   HISTORY
   $Log:	generate.c,v $
 * Revision 1.8  92/03/31  15:12:06  ware
 * From Con: add name
 * 
 * Revision 1.7  92/03/16  12:19:43  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.5  1992/03/10  21:55:13  cso
 * Added the functions required for generating the code for translating
 * from a general form to a specific form with the possibility of
 * specializing an instance based on the immediate enclosing tags.
 *
 * Also, modified the lex side to push and pop the enclosing tags.
 *
 * Also, managed to consider separator tags in the code generated.
 *
 * Revision 1.4  1992/02/28  14:45:11  cso
 * Removed needless function; added the global Maproot
 *
 * Revision 1.3  1992/02/21  23:08:09  cso
 * Merging the down translator generator into the interface
 *
 * Revision 1.2  1992/02/18  18:40:46  cso
 * Aesthetics
 *
 * Revision 1.4  1992/02/06  06:15:55  cso
 * Aesthetics
 *
 * Revision 1.3  1992/01/28  17:15:24  cso
 * Fixed a generation problem with multiply-used symbols in a production
 * and the occurrence being generated.
 *
 * Revision 1.2  1992/01/14  20:54:38  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/10  02:23:48  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"

#include <icatypes.h>

#include "output.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "initialize.h"
#include "mapping.h"
#include "symtab.h"
#include "utils.h"


/*  GLOBALS  */

TokenMapping
   Maproot;




/*
   FUNCTION: 		FindMapping

   Synopsis: 
	Find the mapping pointer in the data structure.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

RSFList
*FindMapping(tagstr)

String
   tagstr;

{
   RSFList
      *rptr;
   
   for (rptr = Downmapping.segptr; rptr != NULL; rptr= rptr->next)
   {
      if (strcmp(tagstr, rptr->tag) == 0)
      {
	 return(rptr);
      }
   }
   return(NULL);
}



/*
   FUNCTION: 		PrintTokens

   Synopsis: 
       Output the token list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintTokens(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   String
      estr,
      sstr;
   SymbolTable
      *tptr;
   
   fprintf(fptr, "%%token <data>\n");
   fprintf(fptr, " ICA_Char_DATA\n");
   fprintf(fptr, "%%token <ival>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      if (tptr->symbol[1] != '/')
      {
	 sstr = (char *) makestarttag(tptr->symbol);
	 fprintf(fptr, " %s\n", sstr);
	 XtFree(sstr);
      }
   }
   fprintf(fptr, "%%token <sepflag>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      if (tptr->symbol[1] == '/')
      {
	 estr = (char *) makeendtag(tptr->symbol);
	 fprintf(fptr, " %s\n", estr);
	 XtFree(estr);
      }
   }
   return;
}




/*
   FUNCTION: 		PrintType

   Synopsis: 
       Output the nonterminal list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintType(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   SymbolTable
      *tptr;
      
   fprintf(fptr, "\n%%type <trans>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "  %s", tptr->symbol);
      if (tptr->optional > 0)
      {
	 fprintf(fptr, " %sOPTL", tptr->symbol);
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, " %sSTAR", tptr->symbol);
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, " %sPLUS", tptr->symbol);
      }
      fprintf(fptr, "\n");
   }
   return;
}




/*
   FUNCTION: 		PrintSwitch

   Synopsis: 
	Print the switch statement including the test for sepflag.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
PrintSwitch(fptr, tagstr, indent)

FILE
   *fptr;
String
   tagstr,
   indent;

{
   RSFList
      *rptr,
      *rptr1;
   String
      duptag;
   
   rptr = FindMapping(tagstr);
   if (rptr == NULL)
   {
      return;
   }
   fprintf(fptr, "%sswitch (ICA_stack[Top])\n%s{\n", indent, indent);
   fprintf(fptr, "%sdefault:\n", indent);
   fprintf(fptr, "%s   $$ = structconcat($$, ", indent);
   if ((rptr->instance == NULL)  ||
       (*rptr->instance == '\0') ||
       (strcmp(rptr->instance, "\"\"") == 0))
   {
      if (tagstr[1] == ',')
      {
	 duptag = XtNewString(tagstr);
	 duptag[1] = '/';
	 rptr1 = FindMapping(duptag);
	 XtFree(duptag);
	 if ((rptr1 != NULL) &&
	     (rptr1->instance != NULL) && (*rptr1->instance != '\0'))
	 {
	    fprintf(fptr, "structnew(%s),\n", rptr1->instance);
	 }
	 else
	 {
	    fprintf(fptr, "structnew(\"\"),\n");
	 }
      }
      else
      {
	 fprintf(fptr, "structnew(\"\"),\n");
      }
   }
   else
   {
      fprintf(fptr, "structnew(%s),\n", rptr->instance);
   }
   fprintf(fptr, "%s                     NULL);\n", indent);
   fprintf(fptr, "%s   break;\n", indent);
   rptr = rptr->next;
   while ((rptr != NULL) && (rptr->dupflag == True))
   {
      duptag = makestarttag(&(rptr->tag[4]));
      fprintf(fptr, "\n%scase %s:\n", indent, duptag);
      fprintf(fptr, "%s   $$ = structconcat($$, ", indent);
      if ((rptr->instance == NULL) || (*rptr->instance == '\0'))
      {
	 fprintf(fptr, "structnew(\"\"),\n");
      }
      else
      {
	 fprintf(fptr, "structnew(%s),\n", rptr->instance);
      }
      fprintf(fptr, "%s                     NULL);\n", indent);
      fprintf(fptr, "%s   break;\n", indent);
      XtFree(duptag);
      rptr = rptr->next;
   }
   fprintf(fptr, "%s}\n", indent);
   return;
}



/*
   FUNCTION: 		PrintAction

   Synopsis: 
	Print the action routine for a production.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
PrintAction(fptr, permptr, prodptr)

FILE
   *fptr;
SymbolList
   *permptr,
   *prodptr;

{
   String
      comma;
   
   fprintf(fptr, "\t\t  $$ = structnew(\"\");\n");
   for ( ; permptr != NULL; permptr = permptr->next)
   {
      if (permptr->symbolstr[0] == '<')
      {
	 if (permptr->symbolstr[1] == '/')
	 {
	    fprintf(fptr, "\t\t  if ($%d == 0) {\n",
		    permptr->position);
	    /* Now we need to print the switch for the end tag */
	    PrintSwitch(fptr, permptr->symbolstr, "\t\t     ");
	    fprintf(fptr, "\t\t  }\n\t\t  else {\n");
	    /* Search and use the separator tag stuff here */
	    comma = XtNewString(permptr->symbolstr);
	    comma[1] = ',';
	    PrintSwitch(fptr, comma, "\t\t     ");
	    XtFree(comma);
	    fprintf(fptr, "\t\t  }\n");
	 }
	 else
	 {
	    PrintSwitch(fptr, permptr->symbolstr, "\t\t  ");
	 }
      }
      else
      {
	 fprintf(fptr, "\t\t  $$ = structconcat($$, $%d, NULL);\n",
		 permptr->position);
      }
   }
   return;
}



/*
   FUNCTION: 		PrintProductions

   Synopsis: 
       Output the productions.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintProductions(fptr)

FILE
   *fptr;

{
   String
      estr,
      sstr;
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr,
      *sptr1;
   PermutationList
      *pptr;
   
   for (i = 0; i < Gmax; i++)
   {
      fprintf(fptr, "%s\t: ", Grammar[i].lhs);
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    if ((sptr->symbolstr[0] == '<') && (sptr->symbolstr[1] == '/'))
	    {
	       estr = (char *) makeendtag(sptr->symbolstr);
	       fprintf(fptr, "%s ", estr);
	       XtFree(estr);
	    }
	    else if (sptr->symbolstr[0] == '<')
	    {
	       sstr = (char *) makestarttag(sptr->symbolstr);
	       fprintf(fptr, "%s ", sstr);
	       XtFree(sstr);
	    }
	    else
	    {
	       fprintf(fptr, "%s", sptr->symbolstr);
	       switch (sptr->occurstr[0])
	       {
	       default:
		  fprintf(fptr, " ");
		  break;

	       case '?':
		  fprintf(fptr, "OPTL ");
		  break;
		  
	       case '*':
		  fprintf(fptr, "STAR ");
		  break;

	       case '+':
		  fprintf(fptr, "PLUS ");
		  break;
	       }
	    }
	 } /* for sptr */
	 
	 fprintf(fptr, "\n\t\t{\n");
	 pptr = rptr->perminstance;
	 if ((pptr != NULL) && (pptr->prev == NULL))
	 {
	    sptr1 = rptr->symbol;
	    if (pptr->permsymbol != NULL)
	    {
	       sptr = pptr->permsymbol;
	    }
	    else
	    {
	       sptr = rptr->symbol;
	    }
	    PrintAction(fptr, sptr, sptr1);
	 }
	 else
	 {
	    fprintf(fptr, "\t\t  $$ = structconcat(NULL);\n");
	 }
	 
	 if (i == 0)
	 {
	    fprintf(fptr, "\t\t  structprint($$);\n");
	 }
	 fprintf(fptr, "\t\t}\n");
	 if (rptr->next == NULL)
	 {
	    fprintf(fptr, "\t\t;\n\n");
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
      } /* for rptr */
   } /* for i */
   return;
}




/*
   FUNCTION: 		PrintExtras

   Synopsis: 
       Output the extra productions for yacc (*, ?, +).

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintExtras(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   String
      str;
   SymbolTable
      *tptr;
      
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      str = tptr->symbol;
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "%sOPTL\t: %s\n", str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "%sSTAR\t: %sSTAR %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "%sPLUS\t: %sPLUS %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| %s\n", str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
   }
   fprintf(fptr, "ICA_Char_DATASTAR\t: ICA_Char_DATASTAR ICA_Char_DATA\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat($1, structnew($2), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t| /* empty content */\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t;\n\n");
   return;
}




/*
   FUNCTION: 		GenerateYacc

   Synopsis: 
       Generate the yacc spec.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

SymbolTable *
GenerateYacc(fptr)

FILE
   *fptr;

{
   SymbolTable
      *stable,
      *ttable;
   
   Output(fptr, yacchead);
   ttable = NULL;
   ttable = MakeTokenTable(ttable);
   PrintTokens(fptr, ttable);
   stable = NULL;
   stable = MakeSymbolTable(stable);
   PrintType(fptr, stable);
   fprintf(fptr, "\n%%start %s\n\n%%%%\n", Grammar[0].lhs);
   PrintProductions(fptr);
   PrintExtras(fptr, stable);
   FreeSymbolTable(stable);
   return(ttable);
}




/*
   FUNCTION: 		PrintLexSpec

   Synopsis: 
       Output the regexps.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintLexSpec(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   RSFList
      *rptr;
   String
      estr,
      sstr,
      prevstart;
   SymbolTable
      *tptr;
   
   for (rptr = Downmapping.symptr; rptr != NULL; rptr = rptr->next)
   {
      if (rptr->tag[0] == '&')
      {
	 fprintf(fptr, "\"%s\"\t{\n", rptr->tag);
	 fprintf(fptr, "\t yylval.data = makestr(%s);\n",
		 rptr->instance);
	 fprintf(fptr, "\t return(ICA_Char_DATA);\n\t}\n\n");
      }
   }
   
   prevstart = NULL;
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      estr = (char *) makeendtag(tptr->symbol);
      sstr = (char *) makestarttag(tptr->symbol);
      if (tptr->symbol[1] == '/')
      {
	 fprintf(fptr, "\"%s\"/\"%s\"\t{\n", tptr->symbol, prevstart);
	 fprintf(fptr, "\t BEGIN 0;\n");
	 fprintf(fptr, "\t pop(%s);\n", &(sstr[1]));
	 fprintf(fptr, "\t yylval.sepflag = 1;\n");
	 fprintf(fptr, "\t return(%s);\n", estr);
	 fprintf(fptr, "\t}\n\n");
	 fprintf(fptr, "\"%s\"\t{\n", tptr->symbol);
	 fprintf(fptr, "\t BEGIN 0;\n");
	 fprintf(fptr, "\t pop(%s);\n", &(sstr[1]));
	 fprintf(fptr, "\t yylval.sepflag = 0;\n");
	 fprintf(fptr, "\t return(%s);\n", estr);
	 fprintf(fptr, "\t}\n\n");
      }
      else
      {	
	 if (prevstart != NULL)
	 {
	    XtFree(prevstart);
	 }
	 prevstart = XtNewString(tptr->symbol);
	 fprintf(fptr, "\"%s\"\t{\n", tptr->symbol);
	 fprintf(fptr, "\t BEGIN 0;\n");
	 fprintf(fptr, "\t push(%s);\n", sstr);
	 fprintf(fptr, "\t return(%s);\n", sstr);
	 fprintf(fptr, "\t}\n\n");
      }	 
      XtFree(estr);
      XtFree(sstr);
   }
   return;
}





/*
   FUNCTION: 		GenerateLex

   Synopsis: 
       Generate the lex spec.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateLex(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   Output(fptr, lexhead);
   PrintLexSpec(fptr, table);
   Output(fptr, lextail);
   FreeSymbolTable(table);
   return;
}
