/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      
   Purpose:     To load a user file which conforms to frank's grammer.
   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/03/16 12:20:11 $
*/

/* HISTORY
   $Log:	load.y,v $
 * Revision 1.5  92/03/16  12:20:11  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.3  1992/03/10  20:15:48  cso
 * Changed the language of the spec file to load and save specs
 *
 * Revision 1.2  1992/02/28  14:51:13  cso
 * The latest incarnations recognizing and installing the mapping
 * information at the top of the g2s file.
 *
 * Revision 1.1  1992/02/06  05:33:28  cso
 * Initial revision
 *
 * Revision 1.3  1992/01/14  20:54:59  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:02  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:55:12  cso
 * Initial revision
 *
*/


%{
#include <stdio.h>
#include <X11/Intrinsic.h>
#include "types.h"

#include "generate.h"
#include "initialize.h"
#include "main.h"
#include "mapping.h"

extern void
   yyerror();

int
   Poscnt;
%}

%union
{
   struct 
   {
      String
	 str;
      String
	 occur;
      int
	 pos;
   } symbol;

   CtxtList
      *ctxtptr;
   MapPtr
      *mapptr;
   NestingInfo
      *nestptr;
   PermutationList
     *permptr;
   RhsList
     *rhsptr;
   RSFList
     *rsfptr;
}

%token <symbol>
AR_CLOSE
AR_OPEN
CTXTBEG
CTXTEND
ENDTAG
IN
LINEFEED
MAPPING
OCCURRENCE
OR
POS_OPEN
POSITION
POS_CLOSE
SEMICOLON
SEPARATOR
STARTTAG
SYMBOL
TOKENSEPARATOR

%type <ctxtptr>
tagS

%type <mapptr>
contextinfo

%type <nestptr>
nesting

%type <permptr>
action_routine
ar_list
ar_sym_list
ar_sym_listOPTL

%type <rhsptr>
mapping
rhs
rhs_list

%type <rsfptr>
dupline
duplineS

%type <symbol>
lhs
occurOPTL
posOPTL
symbol
tag

%start Gen2Spec

%%
Gen2Spec	: mapinfo productionPLUS intaginfo
		{
		}	
		;

mapinfo		: TOKENSEPARATOR tokenlineSTAR TOKENSEPARATOR
		{
		}
		;

tokenlineSTAR	: tokenlineSTAR tokenline
		{
		}
		| /* empty token line, i.e., ICA file */
		{
		}
		;

tokenline	: MAPPING MAPPING MAPPING LINEFEED contextinfo
		{
		   RSFList
		      *ptr,
		      *tailptr;
		   
		   ptr = InstallMappingInfo($2.str, $1.str, $3.str);
		   if ((ptr != NULL) && ($5 != NULL))
		   {
		      if ($5->rptr != NULL)
		      {
			 for (tailptr = $5->rptr;
			      tailptr->next != NULL;
			      tailptr = tailptr->next)
			 {
			 }
			 tailptr->next = ptr->next;
		      }
		      ptr->next = $5->rptr;
		      InstallNesting($5->nptr);
		   }
		   
		   XtFree($1.str);
		   XtFree($2.str);
		   XtFree($3.str);
		   XtFree($5);
		}
		;

productionPLUS	: productionPLUS production
		{
		}
		| production
		{
		}
		;

production	: lhs SEPARATOR rhs_list SEMICOLON
		{
		   int
		      i;
		   RhsList
		      *rl;
		   
		   Grammar[Gmax].lhs       = XtNewString($1.str);
		   Grammar[Gmax].rhs       = $3;
		   Grammar[Gmax].activerhs = $3;
		   for (i = 0, rl = Grammar[Gmax].rhs; rl != NULL; 
			i++, rl = rl->next)
		   {
		   }
		   Grammar[Gmax].numrhs    = i;
		   Gmax++;
		   XtFree($1.str);
		}
		;

lhs		: SYMBOL
		{
		   $$.str   = $1.str;
		   $$.occur = XtNewString("");
		   $$.pos   = 0;
		}
		;

rhs_list	: mapping 
		{
		   $$ = $1;
		}
		| rhs_list OR mapping 
		{
		   RhsList
		      *rptr;
		   
                   rptr = $1;
		   if (rptr != NULL)
		   {
		      for ( ; rptr->next != NULL; rptr = rptr->next)
		      {
		      }
		      rptr->next = $3;
		      if ($3 != NULL)
		      {
			 $3->prev = rptr;
		      }
		      $$ = $1;
		   }
		   else 
		   {
		      fprintf(stderr, "rhslist: $1->symbol is NULL\n");
		      exit(1);
		   }
		}
		;

mapping		: rhs ar_list 
		{
		   $1->perminstance = $2;
		   $1->activeperm   = $2;
		   $$               = $1;
		}
		;

rhs		: symbol
		{
		   RhsList
		      *rptr;
		   
		   Poscnt                  = 1;

		   rptr                    = XtNew(RhsList);
		   rptr->next              = NULL;
		   rptr->prev              = NULL;
		   rptr->perminstance      = NULL;
		   rptr->activeperm        = NULL;
		   rptr->symbol            = XtNew(SymbolList);
		   rptr->symbol->prev      = NULL;
		   rptr->symbol->next      = NULL;
		   rptr->symbol->symbutton = NULL;
		   rptr->symbol->symbolstr = XtNewString($1.str);
		   rptr->symbol->occurstr  = XtNewString($1.occur);
		   if ($1.pos == 0)
		   {
		      rptr->symbol->position = Poscnt;
		   }
		   else
		   {
		      rptr->symbol->position = $1.pos;
		   }
		   $$                      = rptr;
		   Poscnt++;
		   XtFree($1.str);
		   XtFree($1.occur);
		}
		| rhs symbol 
		{
		   SymbolList
		      *sptr;
		   
		   sptr = $1->symbol;
		   if (sptr != NULL)
		   {
		      for ( ; sptr->next != NULL; sptr = sptr->next)
		      {
		      }
		      sptr->next            = XtNew(SymbolList);
		      sptr->next->prev      = sptr;
		      sptr->next->next      = NULL;
		      sptr->next->symbutton = NULL;
		      sptr->next->symbolstr = XtNewString($2.str);
		      sptr->next->occurstr  = XtNewString($2.occur);
		      if ($2.pos == 0)
		      {
			 sptr->next->position = Poscnt;
		      }
		      else
		      {
			 sptr->next->position = $2.pos;
		      }
		      $$                    = $1;
		      Poscnt++;
		   }
		   else 
		   {
		      fprintf(stderr,
			      "rhs: $1->symbol is NULL\n");
		      exit(1);
		   }
		   XtFree($2.str);
		   XtFree($2.occur);
		}
		;

symbol		: STARTTAG posOPTL
		{
		   $$.str   = $1.str;
		   $$.occur = XtNewString("");
		   $$.pos   = $2.pos;
		}
                | ENDTAG posOPTL
		{
		   $$.str   = $1.str;
		   $$.occur = XtNewString("");
		   $$.pos   = $2.pos;
		}
		| SYMBOL occurOPTL posOPTL
		{
		   if ((strcmp($1.str, "CDATASTAR") == 0) ||
		       (strcmp($1.str, "CDATAPLUS") == 0))
		   {
		      XtFree($1.str);
		      $$.str = XtNewString("ICA_Char_DATASTAR");
		      Dirtyflag = True;
		      
		   }
		   else
		   {
		      $$.str   = $1.str;
		   }
		   
		   $$.occur = $2.occur;
		   $$.pos   = $3.pos;
		}
		;

occurOPTL	: OCCURRENCE
		{
		   $$.occur = $1.occur;
		}
		| /* Exactly once */
		{
		   $$.occur = XtNewString("");
		}
		;
    
posOPTL		: POS_OPEN POSITION POS_CLOSE
		{
		   $$.pos = $2.pos;
		}
		| /* Exactly once */
		{
		   $$.pos = 0;
		}
		;
    
ar_list		: action_routine
		{
		   $$ = $1;
		}
		| ar_list action_routine 
		{
		   PermutationList
		      *pptr;
		   
		   if ($1 != NULL)
		   {
		      for (pptr = $1; pptr->next != NULL;
			   pptr = pptr->next)
		      {
		      }
		      pptr->next = $2;
		      if ($2 != NULL)
		      {
			 $2->prev = pptr;
		      }
		      $$ = $1;
		   }
		}
		;

action_routine	: AR_OPEN ar_sym_listOPTL AR_CLOSE 
		{
		   $$ = $2;
		}
		;

ar_sym_listOPTL	: ar_sym_list
		{
		   $$ = $1;
		}
		| /* Empty action routine */
		{
		   /*
		    * In the first cut at this tool,
		    * an empty action routine meant the default
		    * ordering, so to maintain backward compatibility
		    * copy the current rhs.
		    * This will have to work on the other side, after
		    * loading the data structures because of the
		    * inherited attributes in yacc
		    */
		   PermutationList
		      *pptr;

		   pptr             = XtNew(PermutationList);
		   pptr->next       = NULL;
		   pptr->prev       = NULL;
		   pptr->permsymbol = NULL;
		   $$               = pptr;
		}
		;

ar_sym_list	: symbol
		{
		   PermutationList
		      *pptr;

		   Poscnt                      = 1;

		   pptr                        = XtNew(PermutationList);
		   pptr->next                  = NULL;
		   pptr->prev                  = NULL;
		   pptr->permsymbol            = XtNew(SymbolList);
		   pptr->permsymbol->prev      = NULL;
		   pptr->permsymbol->next      = NULL;
		   pptr->permsymbol->symbutton = NULL;
		   pptr->permsymbol->symbolstr = XtNewString($1.str);
		   pptr->permsymbol->occurstr  = XtNewString($1.occur);
		   if ($1.pos == 0)
		   {
		      pptr->permsymbol->position = Poscnt;
		   }
		   else
		   {
		      pptr->permsymbol->position = $1.pos;
		   }
		   $$                          = pptr;
		   Poscnt++;
		   XtFree($1.str);
		   XtFree($1.occur);
		}
		| ar_sym_list symbol
		{
		   SymbolList
		      *sptr;
		   
		   sptr = $1->permsymbol;
		   if (sptr != NULL)
		   {
		      for ( ; sptr->next != NULL; sptr = sptr->next)
		      {
		      }
		      sptr->next            = XtNew(SymbolList);
		      sptr->next->prev      = sptr;
		      sptr->next->next      = NULL;
		      sptr->next->symbutton = NULL;
		      sptr->next->symbolstr = XtNewString($2.str);
		      sptr->next->occurstr  = XtNewString($2.occur);
		      if ($2.pos == 0)
		      {
			 sptr->next->position = Poscnt;
		      }
		      else
		      {
			 sptr->next->position = $2.pos;
		      }
		      Poscnt++;
		   }
		   $$ = $1;
		   XtFree($2.str);
		   XtFree($2.occur);
		}
		;

intaginfo	: nesting intaginfo
		{
		   InstallNesting($1);
		}
		| /* empty in g2s files */
		{
		}
		;

contextinfo	: CTXTBEG nesting duplineS CTXTEND
		{
		   $$ = XtNew(MapPtr);
		   $$->nptr = $2;
		   $$->rptr = $3;
		}
		| /* empty in g2s files */
		{
		   $$ = NULL;
		}
		;

nesting		: tag IN tagS LINEFEED
		{
		   $$          = XtNew(NestingInfo);
		   $$->tag     = XtNewString($1.str);
		   $$->list    = $3;
		   $$->rexlist = NULL;
		   XtFree($1.str);
		}
		;

tagS		: STARTTAG tagS
		{
		   $$          = XtNew(CtxtList);
		   $$->context = XtNewString($1.str);
		   $$->button  = NULL;
		   $$->next    = $2;
		   XtFree($1.str);
		}
		| /* maybe just toplevel */
		{
		   $$ = NULL;
		}
		;

tag		: STARTTAG ENDTAG
		{
		   $$.str = $1.str;
		   XtFree($2.str);
		}    
		| STARTTAG
		{
		   $$.str = $1.str;
		}
		| ENDTAG
		{
		   $$.str = $1.str;
		}
		;

duplineS	: dupline duplineS
		{
		   $$       = $1;
		   $$->next = $2;
		}    
		|
		{
		   $$ = NULL;
		}
		;

dupline		: MAPPING MAPPING MAPPING LINEFEED
		{
		   RSFList
		      *ptr;
		   
		   ptr           = XtNew(RSFList);
		   ptr->tag      = XtNewString($2.str);
		   ptr->regexp   = XtNewString($1.str);
		   ptr->instance = XtNewString($3.str);
		   ptr->dupflag  = True;
		   ptr->type     = Start;
		   ptr->context  = NULL;
		   ptr->next     = NULL;
		   ptr->tagw     = NULL;
		   ptr->rexw     = NULL;
		   ptr->insw     = NULL;
		   $$ = ptr;
		   free($1.str);
		   free($2.str);
		   free($3.str);
		}
		;

%%

