/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: mapping.c,v $
   Purpose:     
       Provide the capability of mapping tags to a string.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 16:20:22 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	mapping.c,v $
 * Revision 1.6  92/05/29  16:20:22  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.5  92/03/16  12:20:22  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.4  1992/03/10  22:09:18  cso
 * Added message regarding generated instance
 *
 * Revision 1.3  1992/03/10  22:01:09  cso
 * Added the install and other routines for managing the mapping
 * interface, including using the string generator that uses the regular
 * expression from rsf files for a specific tag to give the user a start
 * string to confirm - need to add a message of that fact.
 *
 * Revision 1.2  1992/02/28  23:49:18  cso
 * Added the SetFocus action to provide feedback about keyboard focus to
 * the user.
 * Added a function to convert the instance string into a simulated
 * stream for the compare_string function that compares it to a regular
 * expression to feedback info to the user about the validity of their
 * enterred instance.
 * Fixed a resizing problem in the mapping shell.
 *
 * Revision 1.1  1992/02/28  14:57:15  cso
 * Initial revision
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include <icatypes.h>

/*  EXTERNS  */

#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "panel.h"
#include "status.h"
#include "utils.h"

/*  GLOBALS  */

Pixel
   Background,
   Foreground;
RSFList
   *Activemap;
TagList
   Downmapping;
Widget 
   Dtmpopup,
   Mapfocus; 



/*
   FUNCTION: 		MakeUpper

   Synopsis: 
       Make a string into the uppercase equivalent.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		An upper case version of the input string
*/

static String
MakeUpper(str)

String
   str;  /* The string being transformed */

{
   String
      tmp; /* A temporary copy */
   int
      i;   /* Loop counter to examine each character */
   
   /*
    * Make a copy of the string
    * For each character, consider if it should be changed
    */
   tmp = XtNewString(str);
   for (i = 0; i < strlen(tmp); i++)
   {
      /*
       * If the character is a lower case alphabetic character
       * Make it upper case
       */
      if (islower(tmp[i]))
      {
	 tmp[i] = toupper(tmp[i]);
      }
   }
   return(tmp);
}




/*
   FUNCTION: 		WhichType

   Synopsis: 
	Determine the type of tag being processed.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


static Markup
WhichType(tag)

String
   tag;

{
   if (tag[0] == '&')
   {
      return(Symbol);
   }

   switch (tag[1]) 
   {
   default:
      return(Start);
      break;

   case '/':
      return(End);
      break;

   case ',':
      return(Comma);
      break;

   case '?':
      return(Process);
      break;
   }
}



/*
   FUNCTION: 		CollateData

   Synopsis: 
	Collate of the mapping data so that the interface can
	work with it.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
CollateData()

{
   Boolean
      dupflag;
   CtxtList
      *clptr,
      *clptr1;
   RSFList
      *mptr,
      *mptr1;
   
   for (mptr = Downmapping.segptr; mptr != NULL; mptr = mptr->next)
   {
      for (mptr1 = mptr->next;
	   ((mptr1 != NULL) && (strcmp(mptr->tag, mptr1->tag) == 0));
	   )
      {
	 /* If it is worth checking about adding the regexp */
	 if (strcmp(mptr1->regexp, NOREGEXP) != 0)
	 {
	    if ((strcmp(mptr->regexp, mptr1->regexp) != 0) &&
		(strcmp(mptr->regexp, NOREGEXP) == 0))
	    {
	       XtFree(mptr->regexp);
	       mptr->regexp = XtNewString(mptr1->regexp);
	    }
	    else if (strcmp(mptr->regexp, mptr1->regexp) != 0)		     
	    {
	       /* place this regexp into the list of other regexps */
	       if (mptr->context == NULL)
	       {
		  mptr->context                   = XtNew(NestingInfo);
		  mptr->context->tag              = XtNewString(mptr1->tag);
		  mptr->context->list             = NULL;
		  mptr->context->rexlist          = XtNew(CtxtList);
		  mptr->context->rexlist->context = XtNewString(mptr1->regexp);
		  mptr->context->rexlist->button  = NULL;
		  mptr->context->rexlist->next    = NULL;
	       }
	       else
	       {
		  clptr1  = NULL;
		  dupflag = False;
		  for (clptr = mptr->context->rexlist; clptr != NULL;
		       clptr = clptr->next)
		  {
		     if (strcmp(clptr->context, mptr1->regexp) == 0)
		     {
			dupflag = True;
			break;
		     }
		     clptr1 = clptr;
		  } /** FOR - checking for duplicates */
		  if ((dupflag != True) && (clptr1 != NULL))
		  {
		     clptr1->next          = XtNew(CtxtList);
		     clptr1->next->context = XtNewString(mptr1->regexp);
		     clptr1->next->button  = NULL;
		     clptr1->next->next    = NULL;
		  }
		  else if (dupflag != True)
		  {
		     mptr->context->rexlist = XtNew(CtxtList);
		     clptr1                 = mptr->context->rexlist;
		     clptr1->context        = XtNewString(mptr1->regexp);
		     clptr1->button         = NULL;
		     clptr1->next           = NULL;
		  }
	       } /** ELSE - there is at least another regexp besides */
		 /** the default */
	    } /** ELSE - the duplicate tag has a different but */
	      /** nonempty regexp */
	 } /** IF the duplicate tag has a non empty regexp */

	 mptr->next = mptr1->next;
	 XtFree(mptr1->tag);
	 XtFree(mptr1->regexp);
	 XtFree(mptr1->instance);
	 /*  Have to free up the rest of the pointer stuff */
	 if (mptr1->context != NULL)
	 {
	    XtFree(mptr1->context->tag);
	    FreeContext(mptr1->context->list);
	    FreeContext(mptr1->context->rexlist);
	    XtFree(mptr1->context);
	 }
	 XtFree(mptr1);
	 mptr1       = mptr->next;
      } /** FOR each duplicate tag immediately following the first */
	/** encounter */
   } /** FOR each tag in the mapping of seg tags */
   return;
}




/*
   FUNCTION: 		InstallMappingInfo

   Synopsis: 
	Install the tag, regular expression, instance string, and uses info.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

RSFList
*InstallMappingInfo(tagstr, regexp, instance)

String
   tagstr,
   regexp,
   instance;

{
   Boolean
      found;
   RSFList
      *dptr,
      *dptr1,
      *head,
      *new;
   
   if (tagstr[1] == '#')
   {
      /* 
       * if the tag has a # character as the 2nd character
       * then it's the root symbol.  We don't care about 
       * that, so we'll get rid of it by sliding everything
       * over 1 (i.e. move 1st char. into 2nd char position)
       * and bumping the string pointer up by one.
       */
      tagstr[1] = tagstr[0];
      tagstr++;
   }
   new           = XtNew(RSFList);
   new->tag      = XtNewString(tagstr);
   new->regexp   = XtNewString(regexp);
   new->instance = XtNewString(instance);
   new->type     = WhichType(tagstr);
   new->dupflag  = False;
   new->tagw     = NULL;
   new->rexw     = NULL;
   new->insw     = NULL;
   new->next     = NULL;
   new->context  = NULL;
   
   found = False;
   dptr1 = NULL;
   switch (new->type)
   {
   default:
      head = NULL;
      break;
      
   case Symbol:
      head = Downmapping.symptr;
      break;
      
   case Process:
      head = Downmapping.pinptr;
      break;
      
   case Start:
   case End:
   case Comma:
      head = Downmapping.segptr;
      break;
   }

   for (dptr = head; ((dptr != NULL) && (found != True)); )
   {
      if ((new->type == dptr->type) && (strcmp(new->tag, dptr->tag) <= 0))
      {
	 found = True;
      }
      else if (new->type < dptr->type)
      {
	 found = True;
      }
      else
      {
	 dptr1 = dptr;
	 dptr  = dptr->next;
      }
   }

   if ((dptr == NULL) && (dptr1 == NULL))
   {
      switch (new->type)
      {
      default:
	 break;
	 
      case Symbol:
	 Downmapping.symptr = new;
	 break;
	 
      case Process:
	 Downmapping.pinptr = new;
	 break;
	 
      case Start:
      case End:
      case Comma:
	 Downmapping.segptr = new;
	 break;
      }
   }
   else if (dptr == NULL)
   {
      new->next   = dptr1->next;
      dptr1->next = new;
   }
   else if (found == True)
   {
      /* The new tag goes in front of the head of the ptr list */
      if (dptr1 == NULL)
      {
	 new->next = head;
	 switch (new->type)
	 {
	 case Symbol:
	    Downmapping.symptr = new;
	    break;
	    
	 case Process:
	    Downmapping.pinptr = new;
	    break;
	    
	 case Start:
	 case End:
	 case Comma:
	    Downmapping.segptr = new;
	    break;
	 }
      }
      else
      {
	 while ((dptr1->next != NULL) &&
		(strcmp(new->tag, dptr1->next->tag) == 0))
	 {
	    dptr1 = dptr1->next;
	 }
	 new->next   = dptr1->next;
	 dptr1->next = new;
      }
   }
   return(new);
}



/*
   FUNCTION: 		InstallNesting

   Synopsis: 
	Install the nesting information into the data structures.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
InstallNesting(nptr)

NestingInfo
   *nptr;

{
   CtxtList
      *clptr1,
      *clptr2,
      *clptr3;
   RSFList
      *mptr;
   String
      infostr,
      mapstr;
   
   if (nptr == NULL)
   {
      return;
   }
   
   /* Eliminate any duplicate parent strings hanging out from the ica */
   /* file */
   for (clptr1 = nptr->list; clptr1 != NULL; clptr1 = clptr1->next)
   {
      clptr3 = clptr1;
      for (clptr2 = clptr1->next; clptr2 != NULL; )
      {
	 if (strcmp(clptr1->context, clptr2->context) == 0)
	 {
	    clptr3->next = clptr2->next;
	    XtFree(clptr2->context);
	    XtFree(clptr2);
	    clptr2 = clptr3->next;
	 }
	 else
	 {
	    clptr3 = clptr2;
	    clptr2 = clptr2->next;
	 }
      }
   }

   /* Now search the mapping DS to find all of the tags that can use */
   /* this info */
   infostr = StripTagString(nptr->tag);
   for (mptr = Downmapping.segptr; mptr != NULL; mptr = mptr->next)
   {
      mapstr = StripTagString(mptr->tag);
      if ((strcmp(infostr, mapstr) == 0) && (mptr->context == NULL))
      {
	 /* Duplicate the nesting info */
	 mptr->context          = XtNew(NestingInfo);
	 mptr->context->tag     = XtNewString(nptr->tag);
	 mptr->context->rexlist = NULL;
	 if (nptr->list == NULL)
	 {
	    mptr->context->list = NULL;
	 }
	 else
	 {
	    mptr->context->list = XtNew(CtxtList);
	 }
	 clptr2                 = mptr->context->list;
	 for (clptr1 = nptr->list; clptr1 != NULL; clptr1 = clptr1->next)
	 {
	    clptr2->context = XtNewString(clptr1->context);
	    clptr2->button  = NULL;
	    
	    if (clptr1->next == NULL)
	    {
	       clptr2->next = NULL;
	    }
	    else
	    {
	       clptr2->next = XtNew(CtxtList);
	       clptr2       = clptr2->next;
	    }
	 }
      }
      XtFree(mapstr);
   }
   XtFree(infostr);
   
   /* Free up the nesting info b/c it's been copied to all of the */
   /* appropriate places, so we don't need the memory any longer */
   FreeContext(nptr->list);
   XtFree(nptr->tag);
   XtFree(nptr);
   
   return;
}



/*
   FUNCTION: 		ScrollTogether

   Synopsis: 
	Coordinates all of the scrollbars...

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


/*************************************************************************/
/* This function scrolls the regular exp and clauses vertical scroll bars*/
/* when the tags scroll bars is moved using the middle mouse button.     */
/*************************************************************************/

static void
ScrollTogether(w, client_data, call_data)

Widget 
   w;
XtPointer 
   client_data,                  /* not used */
   call_data;                     /* not used */

{
   Arg 
      args[MAXARGS];
   int
      cnt;
   Position 
      x,
      y;
   float 
      top,
      shown;
   Widget
      tags,
      tvert,
      rexp,
      rvert,
      inst,
      ivert;
   
   tvert = XtNameToWidget(Dtmpopup, "*dtmtagsview*vertical");
   rvert = XtNameToWidget(Dtmpopup, "*dtmregexpview*vertical");
   ivert = XtNameToWidget(Dtmpopup, "*dtminstview*vertical");
   tags  = XtNameToWidget(Dtmpopup, "*dtmtagsform");
   rexp  = XtNameToWidget(Dtmpopup, "*dtmregexpform");
   inst  = XtNameToWidget(Dtmpopup, "*dtminstform");
   cnt = 0;
   XtSetArg(args[cnt], XtNtopOfThumb, &top);   cnt++;
   XtSetArg(args[cnt], XtNshown,      &shown); cnt++;
   XtGetValues(w, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNx, &x); cnt++;
   XtSetArg(args[cnt], XtNy, &y); cnt++;
   if (w == tvert)
   {
      XtGetValues(tags, args, cnt);
      XawScrollbarSetThumb(rvert, top, shown);
      XtMoveWidget(rexp, x, y);
      XawScrollbarSetThumb(ivert, top, shown);
      XtMoveWidget(inst, x, y);
   }
   else if (w == rvert)
   {
      XtGetValues(rexp, args, cnt);
      XawScrollbarSetThumb(tvert, top, shown);
      XtMoveWidget(tags, x, y);
      XawScrollbarSetThumb(ivert, top, shown);
      XtMoveWidget(inst, x, y);
   }
   else if (w == ivert)
   {
      XtGetValues(inst, args, cnt);
      XawScrollbarSetThumb(tvert, top, shown);
      XtMoveWidget(tags, x, y);
      XawScrollbarSetThumb(rvert, top, shown);
      XtMoveWidget(rexp, x, y);
   }
   return;
}




/*
   FUNCTION: 		GetStringWidthInPixels

   Synopsis: 
       Using XTextExtents, find the width of a string in pixels, making
       the application font independent.
       
   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Dimension

   Range:		0 <=
*/

Dimension
GetStringWidthInPixels(w, str)

Widget
   w;   /* The widget under consideration */
String
   str; /* The string being measured */

{
   Arg
      args[MAXARGS];        /* List of arguments */
   Dimension
      width;           /* Width value to be returned */
   int
      cnt,             /* Number of arguments */
      direction_hint,  /* Direction of the font */
      ascent,          /* Amount above the baseline used by the font */
      descent;         /* Amount below the baseline used by the font */
   Position
      lmargin,         /* The left margin allowed in the widget */
      rmargin;         /* The right margin allowed in the widget */
   XCharStruct
      overall;         /* Structure describing the font */
   XFontStruct
      *font;           /* The font structure used by the widget */
   
   /*
    * Get the font and margins of the widget
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNfont,        &font);    cnt++;
   XtSetArg(args[cnt], XtNleftMargin,  &lmargin); cnt++;
   XtSetArg(args[cnt], XtNrightMargin, &rmargin); cnt++;
   XtGetValues(w, args, cnt);
   
   /*
    * Get some of the dimensions used in the font of the widget
    */
   XTextExtents(font, str, strlen(str), &direction_hint, &ascent, &descent,
		&overall);
   
   /*
    * The width returned equals the margins of the widget (to make sure
    * we don't cut it off too short) plus the left and right bearings that
    * define the rectangle around the string in that font
    */
   width = lmargin + rmargin + overall.lbearing + overall.rbearing;
   return(width);
}




/*
   FUNCTION: 		ConvertStream

   Synopsis: 
	Convert a stream into a sample instance string.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

String
ConvertStream(stream)

String
   stream;

{
   int
      i,
      length,
      ndx;
   String
      quoted;
   
   length = strlen(stream);
   /* Because it is possible for every character to need escaping plus */
   /* the begin and end quotes and the null character to mark the end  */
   quoted = (String) XtMalloc((unsigned) (2 * length + 4));

   ndx = 0;
   quoted[ndx++] = '"';
   for (i = 0; i < length; i++)
   {
      switch (stream[i])
      {
      default:
	 quoted[ndx++] = stream[i];
	 break;
	 
      case '\\':
      case '"':
	 quoted[ndx++] = '\\';
	 quoted[ndx++] = stream[i];
	 break;
	 
      case '\b':
	 quoted[ndx++] = '\\';
	 quoted[ndx++] = 'b';
	 break;
	 
      case '\n':
	 quoted[ndx++] = '\\';
	 quoted[ndx++] = 'n';
	 break;
	 
      case '\r':
	 quoted[ndx++] = '\\';
	 quoted[ndx++] = 'r';
	 break;

      case '\t':
	 quoted[ndx++] = '\\';
	 quoted[ndx++] = 't';
	 break;
      }
   }
   quoted[ndx++] = '"';
   quoted[ndx]   = '\0';

   XtFree(stream);
   return(quoted);
}



/*
   FUNCTION: 		CreateMappingLine

   Synopsis: 
	Create a line in the mapping window.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
CreateMappingLine(dptr, segflag, tvert, rvert, ivert)

RSFList
   *dptr;
Boolean
   segflag;
Widget
   *tvert,
   *rvert,
   *ivert;

{
   Arg 
      args[MAXARGS];
   Dimension
      text_width;
   int 
      cnt;
   String
      dpyinstance,
      rexstr,
      stream;
   Widget
      tparent,
      rparent,
      iparent;
   XtTranslations 
      dup_trans,
      tag_trans,
      text_trans;
   static String
      dup_trans_str =
	 "<Btn1Down>,<Btn1Up>: RemoveDup()",
      tag_trans_str =
	 "<Btn1Down>,<Btn1Up>: DoPanel()",
      text_trans_str =
	 "<Key>Return:         TextReturn()\n\
          <Key>KP_Enter:       TextReturn()\n\
          <Key>Linefeed:       TextReturn()\n\
          <Btn1Down>,<Btn1Up>: SetFocus()";

   /* translations for the ascii text widgets for instance strings */
   
   dup_trans  = XtParseTranslationTable(dup_trans_str);
   tag_trans  = XtParseTranslationTable(tag_trans_str);
   text_trans = XtParseTranslationTable(text_trans_str);

   tparent = XtNameToWidget(Dtmpopup, "*dtmtagsform");
   rparent = XtNameToWidget(Dtmpopup, "*dtmregexpform");
   iparent = XtNameToWidget(Dtmpopup, "*dtminstform");

   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,     *tvert);      cnt++;
   XtSetArg(args[cnt], XtNbottom,       XtChainTop);  cnt++;
   XtSetArg(args[cnt], XtNdisplayCaret, False);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Background);  cnt++;
   XtSetArg(args[cnt], XtNborderWidth,  2);           cnt++;
   XtSetArg(args[cnt], XtNeditType,     XawtextRead); cnt++;
   XtSetArg(args[cnt], XtNstring,       dptr->tag);   cnt++;
   dptr->tagw = XtCreateWidget(dptr->tag, asciiTextWidgetClass,
			       tparent, args, cnt);
   text_width = GetStringWidthInPixels(dptr->tagw, dptr->tag);
   cnt = 0;
   XtSetArg(args[cnt], XtNwidth,  text_width); cnt++;
   XtSetValues(dptr->tagw, args, cnt);
   XtManageChild(dptr->tagw);

   if (segflag == True)
   {
      if (dptr->dupflag == True)
      {
	 XtOverrideTranslations(dptr->tagw, dup_trans);
      }
      else
      {
	 XtOverrideTranslations(dptr->tagw, tag_trans);
      }
   }
   *tvert = dptr->tagw;
   
   if (strcmp(dptr->regexp, NOREGEXP) == 0)
   {
      rexstr = "Functionality Mismatch";
   }
   else
   {
      rexstr = dptr->regexp;
   }
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,     *rvert);      cnt++;
   XtSetArg(args[cnt], XtNbottom,       XtChainTop);  cnt++;
   XtSetArg(args[cnt], XtNdisplayCaret, False);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Background);  cnt++;
   XtSetArg(args[cnt], XtNborderWidth,  2);           cnt++;
   XtSetArg(args[cnt], XtNeditType,     XawtextRead); cnt++;
   XtSetArg(args[cnt], XtNstring,       rexstr);      cnt++;
   dptr->rexw = XtCreateWidget(dptr->tag, asciiTextWidgetClass,
			       rparent, args, cnt);
   text_width = GetStringWidthInPixels(dptr->rexw, rexstr);
   cnt = 0;
   XtSetArg(args[cnt], XtNwidth,  text_width); cnt++;
   XtSetValues(dptr->rexw, args, cnt);
   XtManageChild(dptr->rexw);
   *rvert = dptr->rexw;
   
   if ((dptr->instance == '\0') || (strcmp(dptr->instance, "\"\"") == 0))
   {
      if (strcmp(dptr->regexp, NOREGEXP) != 0)
      {
	 stream = strgen(dptr->regexp);
	 dpyinstance = ConvertStream(stream);
	 XicaStatus(Statw, "\nGenerating instance [%s] for tag [%s]\n",
		    dpyinstance, dptr->tag);
      }
      else
      {
	 dpyinstance = XtNewString("\"\"");
      }
   }
   else
   {
      dpyinstance = XtNewString(dptr->instance);
   }
   
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,     *ivert);             cnt++;
   XtSetArg(args[cnt], XtNbottom,       XtChainTop);         cnt++;
   XtSetArg(args[cnt], XtNresizable,    True);               cnt++;
   XtSetArg(args[cnt], XtNresize,       XawtextResizeWidth); cnt++;
   XtSetArg(args[cnt], XtNdisplayCaret, False);              cnt++;
   XtSetArg(args[cnt], XtNborder,       Background);         cnt++;
   XtSetArg(args[cnt], XtNborderWidth,  2);                  cnt++;
   XtSetArg(args[cnt], XtNeditType,     XawtextEdit);        cnt++;
   XtSetArg(args[cnt], XtNstring,       dpyinstance);        cnt++;
   dptr->insw = XtCreateWidget(dptr->tag, asciiTextWidgetClass, 
			       iparent, args, cnt);
   XtOverrideTranslations(dptr->insw, text_trans);
   text_width = GetStringWidthInPixels(dptr->insw, dpyinstance);
   cnt = 0;
   XtSetArg(args[cnt], XtNwidth,  text_width); cnt++;
   XtSetValues(dptr->insw, args, cnt);
   XtManageChild(dptr->insw);
   *ivert = dptr->insw;
   if (Mapfocus == NULL)
   {
      Mapfocus = *ivert;
   }
   XtFree(dpyinstance);

   return;
}




/*
  FUNCTION: 		UpdateMappingW
  
   Synopsis: 
	Create all of the widgets for the set of tags.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
UpdateMappingW()

{
   Arg
      args[MAXARGS];
   int
      cnt;
   RSFList
      *dptr;
   Widget
      parent,
      tvert,
      rvert,
      ivert;
   
   tvert    = NULL;
   rvert    = NULL;
   ivert    = NULL;
   Mapfocus = NULL;
   
   for (dptr = Downmapping.symptr; dptr != NULL; dptr = dptr->next)
   {
      CreateMappingLine(dptr, False, &tvert, &rvert, &ivert);
      XicaStatus(Statw, ".");
   }
   if (Downmapping.symptr != NULL)  XicaStatus(Statw, "\n");

   for (dptr = Downmapping.pinptr; dptr != NULL; dptr = dptr->next)
   {
      /***  This works - almost...  The problem is that it returns */
      /***  what would be in a stream and not what would be in a */
      /***  string, so for now we won't use it. */
      /*****
      if ((dptr->instance == '\0') || (strcmp(dptr->instance, "\"\"") == 0))
      {
	 if (strcmp(dptr->regexp, NOREGEXP) != 0)
	 {
	    dptr->instance = XtNewString(strgen(dptr->regexp));
	    Dirtyflag = True;
	 }
      }
      *****/
   }
   if (Downmapping.pinptr != NULL)  XicaStatus(Statw, ".\n");

   for (dptr = Downmapping.segptr; dptr != NULL; dptr = dptr->next)
   {
      CreateMappingLine(dptr, True, &tvert, &rvert, &ivert);
      XicaStatus(Statw, ".");
   }
   if (Downmapping.segptr != NULL)  XicaStatus(Statw, "\n");

   if (Mapfocus != NULL)
   {
      parent = XtParent(XtParent(XtParent(Mapfocus)));
      XtSetKeyboardFocus( parent, Mapfocus );
      cnt = 0;
      XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
      XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;
      XtSetValues(Mapfocus, args, cnt);
   }
   XicaStatus(Statw, "Please confirm any instances generated above.\n");
   
   return;
}



/*
   FUNCTION: 		ConvertInstance

   Synopsis: 
	Convert an instance string into a stream of characters that
	can be validated.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

String
ConvertInstance(instance)

String
   instance;

{
   int
      i,
      length,
      offset;
   String
      stream;
   
   stream = XtNewString(instance);
   length = strlen(instance);
   offset = 1; /* Bypass the open quote */
   for (i = offset; i < (length - 1); i++)
   {
      if (stream[i] == '\\')
      {
	 i++;
	 offset++;
      }
      stream[i - offset] = stream[i];
   }
   stream[i - offset] = '\0';
   return(stream);
}



/*
   FUNCTION: 		TextReturn

   Synopsis: 
	The action for when the user hits return.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

/*************************************************************************/
/* This function is called when a return is hit in the ascii text widgets*/
/* of the regexp's. It highlights the reg exp., then verifies if the     */
/* regular expression is valid or not and prints a message to stdout     */
/* specifying that.                                                      */
/*************************************************************************/

void
TextReturn(w, event, params, num_params)

Widget 
   w;
XEvent 
   *event;
String 
   *params;
int 
   num_params;

{
   Arg
      args[MAXARGS];
   Boolean
      found,
      valid;
   Dimension
      oldwidth,
      width;
   int
      cnt;
   RSFList
      *csptr,
      *dptr;
   String
      compstr,
      textstr;
   Widget
      parent,
      nextw,
      oldfocus;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, &textstr);  cnt++;
   XtSetArg(args[cnt], XtNwidth,  &oldwidth); cnt++;
   XtGetValues(w, args, cnt);
   width = GetStringWidthInPixels(w, textstr);
   if (oldwidth != width)
   {
      cnt = 0;
      XtSetArg(args[cnt], XtNwidth, width); cnt++;
      XtSetValues(w, args, cnt);
   }
   
   XawTextSetSelection(w, 0, strlen(textstr));

   found = False;
   valid = True;
   csptr = NULL;
   
   for (dptr = Downmapping.symptr;
	((dptr != NULL) && (found == False));
	dptr = dptr->next)
   {
      if (dptr->insw == w)
      {
	 found = True;
	 csptr = dptr;
      }
   }
   for (dptr = Downmapping.pinptr;
	((dptr != NULL) && (found == False));
	dptr = dptr->next)
   {
      if (dptr->insw == w)
      {
	 found = True;
	 csptr = dptr;
      }
   }
   for (dptr = Downmapping.segptr;
	((dptr != NULL) && (found == False));
	dptr = dptr->next)
   {
      if (dptr->insw == w)
      {
	 found = True;
	 csptr = dptr;
      }
   }
   if ((found == True) && (strcmp(csptr->instance, textstr) != 0))
   {
      if (strcmp(csptr->regexp, NOREGEXP) != 0)
      {
	 /* Validate instance given */
	 compstr = ConvertInstance(textstr);
	 switch (compare_string(csptr->regexp, &compstr, False, False))
	 {
	 case STRING_MATCH:
	 case STRING_MATCH_RECOPIED:
	    XicaStatus(Statw, "Regular expression accepts [%s]\n",
		       textstr);
	    valid = True;
	    break;
	    
	 case STRING_NO_MATCH:
	    OkMessage(Toplevel,
		      "WARNING: String does not match regular expression");
	    valid = False;
	    break;
	    
	 case REX_SYNTAX_ERROR:
	    OkMessage(Toplevel,
		      "WARNING: Syntax error in the regular expression");
	    valid = False;
	    break;
	    
	 case STRING_OTHER_ERROR:
	    OkMessage
	       (Toplevel,
		"ERROR: String contains an error: Check escape characters");
	    valid = False;
	    break;
	 }
	 XtFree(compstr);
      }
      XtFree(csptr->instance);
      csptr->instance = XtNewString(textstr);
      Dirtyflag = True;
   }
   if (valid == True)
   {
      /* Bump the focus for the user */
      /* Find the next legal focus widget */
      nextw = Mapfocus;
      if (csptr->next != NULL)
      {
	 nextw = csptr->next->insw;
      }
      else if (csptr->type == Symbol)
      {
	 if (Downmapping.segptr != NULL)
	 {
	    nextw = Downmapping.segptr->insw;
	 }
      }
      
      if ((Mapfocus != nextw) && (nextw != NULL))
      {
	 oldfocus = Mapfocus;
	 Mapfocus = nextw;
      
	 if (oldfocus != NULL)
	 {
#ifdef DEBUG
	    fprintf(stderr, "oldfocus = [%s]\n", XtName(oldfocus));
#endif
	    cnt = 0;
	    XtSetArg(args[cnt], XtNdisplayCaret, False);      cnt++;
	    XtSetArg(args[cnt], XtNborder,       Background); cnt++;
	    XtSetValues(oldfocus, args, cnt);
	 }
	 parent = XtParent(XtParent(XtParent(Mapfocus)));
	 XtSetKeyboardFocus( parent, Mapfocus );
	 cnt = 0;
	 XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
	 XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;
	 XtSetValues(Mapfocus, args, cnt);
      }
   }
   return;
}



/*
  FUNCTION: 		SetFocus

   Synopsis:
       The callback used whenever the user selects an instance widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SetFocus(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      parent,
      oldfocus;
   
   if (Mapfocus == w)
   {
      return;
   }
   
#ifdef DEBUG
   fprintf(stderr, "w = [%s]\n", XtName(w));
#endif
   oldfocus = Mapfocus;
   Mapfocus = w;

   if (oldfocus != NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "oldfocus = [%s]\n", XtName(oldfocus));
#endif
      cnt = 0;
      XtSetArg(args[cnt], XtNdisplayCaret, False);      cnt++;
      XtSetArg(args[cnt], XtNborder,       Background); cnt++;
      XtSetValues(oldfocus, args, cnt);
   }
   parent = XtParent(XtParent(XtParent(w)));
   XtSetKeyboardFocus( parent, w );
   cnt = 0;
   XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;
   XtSetValues(w, args, cnt);
   
   return;
}



/*
   FUNCTION: 		CreateMappingShell

   Synopsis: 
	Create the popup shell containing the mapping interface.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON
       pete	3/6/92	 Making titles consistent.

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

/************************************************************************/
/* This function is called from the retag.c file main function. It      */
/* the entire implicit segmenting interface widget tree adding all the  */
/* necessary callbacks and menus. It does not popup the shell, though.  */
/* That is done in mainw.c.                                             */
/************************************************************************/

void
CreateMappingShell(parent)

Widget
   parent;

{
   Widget 
      dtmbuttonform,
      dtmdone,
      dtmhelp,
      dtmrem,
      dtmtags,
      dtmregexp,
      dtminst,
      dtmregexpview,
      dtmregexpform,
      dtminstform,
      dtmtagsview,
      dtminstview,
      dtmtagsform,
      dtmtagsmenu,
      dtmtagsmenuentry,
      dtmtopform,
      dtmlblrow, dtmviewrow,
      dtmtagcol, dtmregexpcol, dtminstcol,
      dtmvert1,
      dtmvert2,
      dtmvert3;
   

   Dtmpopup = XtVaCreatePopupShell( "map-spec-form",
       applicationShellWidgetClass, Toplevel,
       XtNallowShellResize, True,
       NULL );
   
   dtmtopform = XtVaCreateManagedWidget( "dtmtopform",
       tableWidgetClass, Dtmpopup,
       XtNhorzPlacement, tpFlush,
       XtNhorzSpacing,   6,
       XtNvertSpacing,   2,
       XtNhorzMargin,    6,
       XtNvertMargin,    6,
       NULL );
   dtmlblrow = XtVaCreateWidget( "dtmlblrow",
       tableRowObjectClass, dtmtopform,
       NULL );
   dtmviewrow = XtVaCreateWidget( "dtmviewrow",
       tableRowObjectClass, dtmtopform,
       XtNstretch, True,
       NULL );
   dtmtagcol = XtVaCreateWidget( "dtmtagscol",
       tableColObjectClass, dtmtopform,
       NULL );
   dtmregexpcol = XtVaCreateWidget( "dtmregexpcol",
       tableColObjectClass, dtmtopform,
       NULL );
   dtminstcol = XtVaCreateWidget( "dtminstcol",
       tableColObjectClass, dtmtopform,
       NULL );
   
   XtVaGetValues( dtmtopform, 
       XtNborder,     &Foreground,
       XtNbackground, &Background,
       NULL );

   dtmtags = XtVaCreateManagedWidget( "dtmtags",
       labelWidgetClass, dtmtopform,
       XtNborderWidth, 0,
       XtNlabel,       "General-Markup Tags:",
       XtNtableRow,    dtmlblrow,
       XtNtableCol,    dtmtagcol,
       NULL );
   
   dtmregexp = XtVaCreateManagedWidget( "dtmregexp",
       labelWidgetClass, dtmtopform,
       XtNborderWidth, 0,
       XtNlabel,       "Regular Expressions:",
       XtNtableRow,    dtmlblrow,
       XtNtableCol,    dtmregexpcol,
       NULL );
   
   dtminst = XtVaCreateManagedWidget( "dtminst",
       labelWidgetClass, dtmtopform,
       XtNborderWidth, 0,
       XtNlabel,       "Specific-Markup Strings:",
       XtNtableRow,    dtmlblrow,
       XtNtableCol,    dtminstcol,
       NULL );
   
   dtmtagsview = XtVaCreateManagedWidget( "dtmtagsview",
       viewportWidgetClass, dtmtopform,
       XtNallowVert,     True,
       XtNallowHoriz,    True,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     150,
       XtNminWidth,      100,
       XtNprefHeight,    500,
       XtNminHeight,     50,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      dtmviewrow,
       XtNtableCol,      dtmtagcol,
       NULL );
   dtmvert1 = XtNameToWidget(dtmtagsview, "vertical");
   XtAddCallback(dtmvert1, XtNscrollProc, ScrollTogether, (XtPointer) NULL);
   XtAddCallback(dtmvert1, XtNjumpProc,   ScrollTogether, (XtPointer) NULL);
   
   dtmregexpview = XtVaCreateManagedWidget( "dtmregexpview",
       viewportWidgetClass, dtmtopform,
       XtNallowVert,     True,
       XtNallowHoriz,    True,
       XtNforceBars,     True,
       XtNuseRight,      True,
       XtNpreference,    False,
/*
       XtNprefWidth,     150,
       XtNminWidth,      100,
       XtNprefHeight,    500,
       XtNminHeight,     50,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      dtmviewrow,
       XtNtableCol,      dtmregexpcol,
       NULL );
   dtmvert2 = XtNameToWidget(dtmregexpview, "vertical");
   XtAddCallback(dtmvert2, XtNscrollProc, ScrollTogether, (XtPointer) NULL);
   XtAddCallback(dtmvert2, XtNjumpProc,   ScrollTogether, (XtPointer) NULL);

   XtUnmanageChild(dtmvert2);
   XtVaSetValues( dtmvert2,
       XtNwidth,       1,
       XtNborderWidth, 0,
       NULL );

   dtminstview = XtVaCreateManagedWidget( "dtminstview",
       viewportWidgetClass, dtmtopform,
       XtNallowVert,     True,
       XtNallowHoriz,    True,
       XtNforceBars,     True,
       XtNuseRight,      True,
       XtNpreference,    False,
/*
       XtNprefWidth,     250,
       XtNminWidth,      100,
       XtNprefHeight,    500,
       XtNminHeight,     50,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      dtmviewrow,
       XtNtableCol,      dtminstcol,
       NULL );

   dtmvert3 = XtNameToWidget(dtminstview, "vertical");
   XtAddCallback(dtmvert3, XtNscrollProc, ScrollTogether, (XtPointer) NULL);
   XtAddCallback(dtmvert3, XtNjumpProc,   ScrollTogether, (XtPointer) NULL);
   
   dtmtagsform = XtVaCreateManagedWidget( "dtmtagsform",
       viewformWidgetClass, dtmtagsview,
       XtNresizable, True,
       NULL );
   
   dtmregexpform = XtVaCreateManagedWidget( "dtmregexpform",
       viewformWidgetClass, dtmregexpview,
       XtNresizable, True,
       NULL );
   
   dtminstform = XtVaCreateManagedWidget( "dtminstform",
       viewformWidgetClass, dtminstview,
       XtNresizable, True,
       NULL );
   
   XtRealizeWidget(Dtmpopup);

   XtPopup(Dtmpopup, XtGrabNone);
   
   CreateMapPanel(Dtmpopup);
   
   return;
}

