/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menucalls.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.7 $
   Date:        $Date: 92/05/29 16:20:35 $

   HISTORY
   $Log:	menucalls.c,v $
 * Revision 1.7  92/05/29  16:20:35  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.6  92/03/16  12:20:38  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.4  1992/02/28  14:52:46  cso
 * Integrated the call to load in multiple rsf specs in a NEW load - ugly
 *
 * Revision 1.3  1992/02/21  23:07:11  cso
 * Removed perm operations (unnecessary); added new make procedure
 *
 * Revision 1.2  1992/02/18  16:58:28  cso
 * Made the appropriate g2s mods from s2g
 *
 * Revision 1.5  1992/02/11  22:49:16  cso
 * Fixed the call to GetFilename/OpenFile
 *
 * Revision 1.4  1992/02/06  06:07:41  cso
 * Fixed file dependencies and function decls
 *
 * Revision 1.3  1992/01/14  20:55:10  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:08  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:56:05  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "fileio.h"
#include "generate.h"
#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "mapping.h"
#include "prodlist.h"
#include "save.h"
#include "status.h"
#include "utils.h"

#ifdef FLEX
extern void
   rsfrestart(),
   yyrestart();
#endif

/*  GLOBALS  */




/*
   FUNCTION: 		SaveChanges

   Synopsis: 
       Check if the  current session has changed a grammar description.
       Offer the user to save or forget changes AND if unsuccessful
       in saving let them know and let them leave the function
       invoking this one without losing the changes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
	3/6/92	pete	 Making tool names/prompts consistent.

   Globals used:
       Dirtyflag  

   Globals Modified:
       none

   Return value:	Boolean

   Range:		True or False
*/

static Boolean
SaveChanges()

{
   Boolean
      continue_oper,
      response;
   FILE
      *fptr;

   /*
    * If the current session has changed, give the user a chance to
    * save the file or at least realize that changes have occurred.
    */
   continue_oper = True;
   if (Dirtyflag == True)
   {
      /*
       * Ask the user if they wish to save the changes, if Continue, 
       * then get a file name and save the file
       */
      continue_oper = YesNoCancelMessage(Toplevel, "Save changes?",
					 True, &response);
      
      if ((continue_oper == True) && (response == True))
      {
	 /*
	  * Get the file to save the current session
	  * If successful in getting a file, save out the current session
	  * Else alert the user that changes will be lost
	  */
	 fptr = OpenFile("Save a [.g2s]",
			 "Save:",
			 "Save",
			 "*.g2s",
			 "g2s",
			 False);
	 
	 if (fptr != NULL)
	 {
	    SaveFile(fptr);
	    fclose(fptr);
	    Dirtyflag = False;
	 }
	 else
	 {
	    /*
	     * If the user does not wish to lose changes,
	     * then return False, preventing whatever action would lose
	     * the changes from occurring.
	     */
	    return(YesNoMessage(Toplevel, "Lose changes?", False));
	 }
      }
   }
   return(continue_oper);
}




/*
   FUNCTION: 		Reset

   Synopsis: 
       Reset the interface.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
Reset(w, client_data, call_data)

Widget
   w;
caddr_t
   client_data,
   call_data;

{
   Arg
      args[MAXARGS];
   int 
      cnt;
   
   if (SaveChanges() == False)
   {
      return;
   }
   
   if (YesNoMessage(Toplevel, "Reset interface?", True) == True)
   {
      /*
       * Reset data structures.
       * Blank the focus widget.
       */
      InitializeGram();
      cnt = 0;
      XtSetArg(args[cnt], XtNlabel, ""); cnt++;
      XtSetValues(XtNameToWidget(Toplevel, "*focus"), args, cnt);
      Dirtyflag = False;
   }
   return;
}



/*
   FUNCTION: 		FileLoad

   Synopsis: 
       Load in a file to be edited.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
	3/6/92	pete	 Making tool names/prompts consistent.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void 
FileLoad(w, newflag, call_data)

Widget
   w;
Boolean
   newflag;
XtPointer
   call_data;   /* Unused */

{
   extern FILE
      *rsfin,
      *yyin;
   extern int
      linecount;

   if (SaveChanges() == False)
   {
      return;
   }
   InitializeGram();
   if (newflag == True)
   {
      /* weird kludge/hack to read in RSF files    */
      /* keep selecting the files and read them    */
      /* in, until we don't get an open descriptor */
      /* back.  I don't like this method.          */
      rsfin = OpenFile("Load a specification [.rsf]\n\
 Use Cancel after all files have been loaded",
		      "Load:",
		      "Load",
		      "*.rsf",
		      "rsf",
		      True);
      while (rsfin != NULL)
      {
	 Dirtyflag = True;
	 /* deal with the RSF file */
	 linecount = 1;
	 if (rsfparse() != 0)
	 {
	    XicaStatus(Statw, "[.rsf] file not in correct format\n");
	 }
	 fclose(rsfin);
#ifdef FLEX
	 rsfrestart(rsfin);
#endif
	 XicaStatus(Statw, "Successfully loaded [.rsf] file\n");
	 rsfin = OpenFile("Load a specification [.rsf]\n\
 Use Cancel after all files have been loaded",
			  "Load:",
			  "Load",
			  "*.rsf",
			  "rsf",
			  True);
      }

      yyin = OpenFile("Load an ICA Specification [.ica]",
		      "Load:",
		      "Load",
		      "*.ica",
		      "ica",
		      True);
   }
   else
   {
      yyin = OpenFile("Load a specification [.g2s]",
		      "Load:",
		      "Load",
		      "*.g2s",
		      "g2s",
		      True);
   }
   
   if (yyin == NULL)
   {
      return;
   }
   
   linecount = 1;
   if (yyparse() == 0) 
   {
      CreateSymbolButtons();
      if (newflag == True)
      {
	 XicaStatus(Statw, "[.ica] file load completed.\n");
	 XicaStatus(Statw, "Do not forget to SAVE this specification!\n");
      }
      else
      {
	 XicaStatus(Statw, "[.g2s] file load completed.\n");
      }
   }
   else
   {
      XicaStatus(Statw, "Input file not in correct format\n");
   }
   fclose(yyin);
#ifdef FLEX
   yyrestart(yyin);
#endif
   
   if (Gmax > 0) 
   {
      Gindex = 0;
#ifdef DEBUG
      fprintf(stderr, "Root symbol = [%s]\n", Grammar[0].lhs);
#endif
      GenerateProdList();
      UpdateInterface(Gindex, Gindex);
      XicaStatus(Statw, "Collating data structures\n");
      CollateData();
      XicaStatus(Statw, "Creating widgets\n");
      UpdateMappingW();
   }

   return;
}




/*
   FUNCTION: 		FileSave

   Synopsis: 
       Save out the specification file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FileSave(w, client_data, call_data)

Widget 
   w;			/* unused */
caddr_t 
   client_data,		/* unused */
   call_data;		/* unused */

{
   FILE
      *fptr;

   if (Dirtyflag == False)
   {
      if (YesNoMessage(Toplevel,
		       "No changes made, save anyway?", False) == False)
      {
	 return;
      }
   }
   
   fptr = OpenFile("Save a specification [.g2s]",
		   "Save:",
		   "Save",
		   "*.g2s",
		   "g2s",
		   False);
	 
   if (fptr == NULL)
   {
      return;
   }
   SaveFile(fptr);
   fclose(fptr);
   Dirtyflag = False;
   return;
}



/*
   FUNCTION: 		Quit

   Synopsis: 
       Quit.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
Quit(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   if (SaveChanges() == False)
   {
      return;
   }
   XtDestroyWidget(Toplevel);
   exit(0);
}



/*
   FUNCTION: 		Make

   Synopsis: 
       Make a translator

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
	3/6/92	pete	 Making tool names/prompts consistent.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Make(w, panel, call_data)

Widget
   w;
XtPointer
   panel;
XtPointer
   call_data;

{
   FILE
      *lexptr,
      *yaccptr;
   String
      target;
   SymbolTable
      *tokentab;
   
   if (Gmax == 0)
   {
      XicaStatus(Statw, "Please load a specification file first.\n");
      return;
   }
   
   target= GetFilename("Making a translator [.gsx]",
		       "Make:",
		       "Make",
		       "*.gsx",
		       "gsx",
		       False);

   if (*target == '\0')
   {
      return;
   }
   
   lexptr  = OpenGenerationFile(target, False);
   yaccptr = OpenGenerationFile(target, True);

   tokentab = GenerateYacc(yaccptr);
   fclose(yaccptr);
   GenerateLex(lexptr, tokentab);
   fclose(lexptr);
   
   MakeTranslator(target);

   XtFree(target);
   return;
}



/*
   FUNCTION: 		Run

   Synopsis: 
       Run an up translator.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
	3/6/92	pete	 Making tool names/prompts consistent.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Run(w, client_data, call_data)

Widget
   w;
XtPointer
   client_data,
   call_data;

{
   String
      execfile,
      infile,
      outfile,
      sysstr;

   /* pick a document to translate */
   infile = GetFilename("Choose an input file [*]",
			"Ok:",
			"Ok",
			"*",
			"*",
			True);

   if (*infile == '\0')
   {
      return;
   }
   
   /* choose an output file name */
   outfile = GetFilename("Choose an output file [*]\n Cancel to use stdout",
			"Ok:",
			"Ok",
			"*",
			"*",
			False);

   /* pick an executable */
   execfile = GetFilename("Execute a translator [.gsx]",
			  "Run:",
			  "Run",
			  "*.gsx",
			  "gsx",
			  True);

   if (*execfile == '\0')
   {
      return;
   }
   
   if (*outfile != '\0')
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) +
			strlen(outfile) + 32);
      sprintf(sysstr, "%s < %s > %s", execfile, infile, outfile);
      XtFree(outfile);
   } /***** IF the user chose a file for output */
   else
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) + 32);
      sprintf(sysstr, "%s < %s", execfile, infile);
   } /***** ELSE put the output to stdout */

#ifdef DEBUG
   fprintf(stderr, "RUN str = [%s]\n", sysstr);
#endif
   XicaStatus(Statw, "Running translator\n%s\n", sysstr);
   errno = 0;
   if (system(sysstr) < 0)
   {
      perror("System call to RUN translator failed");
      XicaStatus(Statw, "Run failed\n");
   }
   else
   {
      XicaStatus(Statw, "Run finished\n");
   }
   
   
   XtFree(sysstr);
   XtFree(infile);
   XtFree(execfile);
   return;
}

