/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: panel.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.3 $
   Date:        $Date: 92/05/29 16:20:40 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	panel.c,v $
 * Revision 1.3  92/05/29  16:20:40  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.2  92/03/16  12:20:48  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.1  1992/03/10  20:39:19  cso
 * Initial revision
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include <icatypes.h>

/*  EXTERNS  */

#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "mapping.h"
#include "status.h"

/*  GLOBALS  */

Widget
   Mappanel;



/*
   FUNCTION: 		CorrectVertical

   Synopsis: 
	Correct the vertical widget for two pointers.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
CorrectVertical(above, below)

RSFList
   *above,
   *below;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   
   if (below == NULL)
   {
      return;
   }
   
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, above->tagw); cnt++;
   XtSetValues(below->tagw, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, above->rexw); cnt++;
   XtSetValues(below->rexw, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, above->insw); cnt++;
   XtSetValues(below->insw, args, cnt);
   return;
}



/*
   FUNCTION: 		ClearPanel

   Synopsis: 
	Clear out the buttons created.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
ClearPanel(form, rptr)

Widget
   form;
RSFList
   *rptr;

{
   CtxtList
      *clptr;
   
   if ((rptr != NULL) && (rptr->context != NULL))
   {
      for (clptr = rptr->context->list; clptr != NULL;
	   clptr = clptr->next)
      {
	 if (clptr->button != NULL)
	 {
	    XtDestroyWidget(clptr->button);
	    clptr->button = NULL;
	 }
      }
      for (clptr = rptr->context->rexlist; clptr != NULL;
	   clptr = clptr->next)
      {
	 if (clptr->button != NULL)
	 {
	    XtDestroyWidget(clptr->button);
	    clptr->button = NULL;
	 }
      }
   }
   return;
}



/*
   FUNCTION: 		PopupPanel

   Synopsis: 
	Set up and popup the map panel.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
PopupPanel(mptr)

RSFList
   *mptr;

{
   Arg
      args[MAXARGS];
   CtxtList
      *clptr;
   Dimension
      height,
      width;
   int
      cnt;
   Position
      popx,
      popy;
   Widget
      form,
      parent,
      vert;
   XtTranslations 
      toggletrans;
   static String
      toggletransstr = "<EnterWindow>:	     highlight(Always)\n\
			<LeaveWindow>:	     unhighlight()\n\
			<Btn1Down>,<Btn1Up>: set() notify()";
   
   toggletrans = XtParseTranslationTable(toggletransstr);
   
   form = XtNameToWidget(Mappanel, "*panelform");
   if (form == NULL)
   {
      return;
   }

   XtVaSetValues( XtNameToWidget(form, "*taglbl"), 
       XtNlabel, mptr->tag,
       NULL );
   
   XtVaGetValues( mptr->tagw,
       XtNheight, &height,
       XtNwidth,  &width,
       NULL );
   XtTranslateCoords( mptr->tagw, 0, 0, &popx, &popy );
   XtVaSetValues( Mappanel,
       XtNx, popx + 10,
       XtNy, popy + height + 10,
       NULL );

   if ((mptr != NULL) && (mptr->context != NULL))  {
      vert = NULL;
      parent = XtNameToWidget(form, "*ctxtform");
      XawFormDoLayout( parent, False );
      for (clptr = mptr->context->list; clptr != NULL;
	   clptr = clptr->next)  {
	 clptr->button = XtVaCreateManagedWidget( clptr->context,
             toggleWidgetClass, parent,
	     XtNfromVert,   vert,
	     XtNradioData,  clptr,
	     XtNradioGroup, vert,
	     XtNlabel,      clptr->context,
             NULL );
	 XtOverrideTranslations(clptr->button, toggletrans);
	 vert = clptr->button;
      }
      XawFormDoLayout( parent, True );

      vert = NULL;
      parent = XtNameToWidget(form, "*rexform");
      XawFormDoLayout( parent, False );
      for (clptr = mptr->context->rexlist; clptr != NULL;
	   clptr = clptr->next)  {
	 clptr->button = XtVaCreateManagedWidget( clptr->context,
             toggleWidgetClass, parent,
	     XtNfromVert,   vert,
	     XtNradioData,  clptr,
	     XtNradioGroup, vert,
	     XtNlabel,      clptr->context,
             NULL );
	 XtOverrideTranslations(clptr->button, toggletrans);
	 vert = clptr->button;
      }
      XawFormDoLayout( parent, True );
   }
   
   XtPopup(Mappanel, XtGrabExclusive);
   return;
}



/*
   FUNCTION: 		DoPanel

   Synopsis:
       The action to find the tag that is to be specialized and to do
       some cursory checks that it is valid to specialize it.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DoPanel(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   RSFList
      *mptr;
   
   for (mptr = Downmapping.segptr; mptr != NULL; mptr = mptr->next)
   {
      if (w == mptr->tagw)
      {
	 break;
      }
   }
   if (mptr == NULL)
   {
      return;
   }
   if (mptr->dupflag == True)
   {
      OkMessage(Toplevel, "Select the tag you wish to specialize");
      return;
   }
   else if (strcmp(mptr->regexp, NOREGEXP) == 0)
   {
      OkMessage(Toplevel,
		"This tag has no regular expression to use for specializing");
      return;
   }
   else if ((mptr->context == NULL) ||
	    (mptr->context->list == NULL))
   {
      OkMessage(Toplevel, "This tag is contained within a default context");
      return;
   }
   else if (mptr->context->list->next == NULL)
   {
      OkMessage(Toplevel, "This tag is contained within a default context");
      return;
   }

   PopupPanel(mptr);
   return;
}



/*
   FUNCTION: 		RemoveDup

   Synopsis:
       Remove a duplicate entry.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
RemoveDup(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   CtxtList
      *clptr,
      *clptr1;
   int
      offset = 4;
   RSFList
      *mptr,
      *mptr1;
   String
      oldctxt;
   
   mptr1 = NULL;
   for (mptr = Downmapping.segptr; mptr != NULL; mptr = mptr->next)
   {
      if (w == mptr->tagw)
      {
	 break;
      }
      if (mptr->dupflag == False)
      {
	 mptr1 = mptr;
      }
   }
   if ((mptr == NULL) && (mptr1 == NULL))
   {
      return;
   }
   if (mptr->dupflag == True)
   {
      if (YesNoMessage(Toplevel, "Remove this context?", False) == False)
      {
	 return;
      }
   }
   oldctxt = (String) XtMalloc(strlen(mptr->tag));
   strncpy(oldctxt, &(mptr->tag[offset]), strlen(mptr->tag));
   if (mptr1->context != NULL)
   {
      for (clptr = mptr1->context->list; clptr != NULL; clptr = clptr->next)
      {
	 clptr1 = clptr;
      }
      clptr1->next   = XtNew(CtxtList);
      clptr          = clptr1->next;
      clptr->context = XtNewString(oldctxt);
      clptr->button  = NULL;
      clptr->next    = NULL;
   }
   while (mptr1->next != mptr)
   {
      mptr1 = mptr1->next;
   }
   mptr1->next = mptr->next;
   if (mptr->tagw != NULL)
   {
      XtUnmanageChild(mptr->tagw);
      XtDestroyWidget(mptr->tagw);
      mptr->tagw = NULL;
   }
   if (mptr->rexw != NULL)
   {
      XtUnmanageChild(mptr->rexw);
      XtDestroyWidget(mptr->rexw);
      mptr->rexw = NULL;
   }
   if (mptr->insw != NULL)
   {
      XtUnmanageChild(mptr->insw);
      XtDestroyWidget(mptr->insw);
      mptr->insw = NULL;
   }
   XtFree(mptr->tag);
   XtFree(mptr->regexp);
   XtFree(mptr->instance);
   if (mptr->context != NULL)
   {
      XtFree(mptr->context->tag);
      FreeContext(mptr->context->list);
      FreeContext(mptr->context->rexlist);
      XtFree(mptr->context);
   }
   XtFree(mptr);
   
   CorrectVertical(mptr1, mptr1->next);
   Dirtyflag = True;
   return;
}



/*
   FUNCTION: 		CreateDuplicateEntry

   Synopsis: 
	The user has selected a special context be created - do it.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
CreateDuplicateEntry(rsfptr, ctxtptr, rexptr)

RSFList
   *rsfptr;
CtxtList
   *ctxtptr,
   *rexptr;

{
   CtxtList
      *clptr,
      *clptr1;
   RSFList
      *new;
   Widget
      tvert,
      rvert,
      ivert;
   
   new           = XtNew(RSFList);
   new->tag      = XtMalloc((unsigned) (strlen(ctxtptr->context) + 8));
   sprintf(new->tag, " -> %s", ctxtptr->context);
   if (rexptr != NULL)
   {
      new->regexp = XtNewString(rexptr->context);
   }
   else
   {
      new->regexp = XtNewString(rsfptr->regexp);
   }
   new->instance = XtNewString("\"\"");
   new->type     = rsfptr->type;
   new->dupflag  = True;
   new->context  = NULL;

   tvert = rsfptr->tagw;
   rvert = rsfptr->rexw;
   ivert = rsfptr->insw;

   CreateMappingLine(new, True, &tvert, &rvert, &ivert);

   /* Coordinate the vertical widgets and connect up the pointers. */
   /* Also, remove the context from the master pointer (the mapping */
   /* symbol that was duplicated */
   new->next    = rsfptr->next;
   rsfptr->next = new;
   if (new->next != NULL)
   {
      CorrectVertical(new, new->next);
   }
   
   clptr1 = NULL;
   for (clptr = rsfptr->context->list; clptr != NULL; clptr = clptr->next)
   {
      if (clptr == ctxtptr)
      {
	 if (clptr1 == NULL)
	 {
	    rsfptr->context->list = clptr->next;
	 }
	 else
	 {
	    clptr1->next = clptr->next;
	 }
	 break;
      }
      clptr1 = clptr;
   }
   if (clptr != NULL)
   {
      XtFree(clptr->context);
      if (clptr->button != NULL)
      {
	 XtDestroyWidget(clptr->button);
	 clptr->button = NULL;
      }
      XtFree(clptr);
   }
   return;
}



/*
   FUNCTION: 		Okay

   Synopsis: 
	The user hits return or uses the button to use what is displayed.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
Okay(w, form, call_data)

Widget
   w,
   form;
XtPointer
   call_data;

{
   Arg
      args[MAXARGS];
   Boolean
      state;
   CtxtList
      *ctxtchoice,
      *rexchoice;
   int
      cnt;
   RSFList
      *rptr;
   
   XtPopdown(Mappanel);

   for (rptr = Downmapping.segptr; rptr != NULL; rptr = rptr->next)
   {
      if ((rptr->context != NULL) &&
	  (rptr->context->list != NULL) &&
	  (rptr->context->list->button != NULL))
      {
	 break;
      }
   }
   if (rptr == NULL)
   {
      return;
   }
   
   ctxtchoice = (CtxtList *) XawToggleGetCurrent(rptr->context->list->button);
   if (rptr->context->rexlist != NULL)
   {
      rexchoice = (CtxtList *)
	 XawToggleGetCurrent(rptr->context->rexlist->button);
      if ((rexchoice == NULL) && (rptr->context->rexlist->button != NULL))
      {
	 cnt = 0;
	 XtSetArg(args[cnt], XtNstate, &state); cnt++;
	 XtGetValues(rptr->context->rexlist->button, args, cnt);
	 if (state == True)
	 {
	    rexchoice = rptr->context->rexlist;
	 }
      }
   }
   else
   {
      rexchoice = NULL;
   }
   
   XawFormDoLayout(form, False);
   ClearPanel(form, rptr);
   XawFormDoLayout(form, True);

   if (ctxtchoice != NULL)
   {
      /* The user set something so we need to create the new entry and */
      /* adjust the d.s. accordingly */
      CreateDuplicateEntry(rptr, ctxtchoice, rexchoice);
      Dirtyflag = True;
   }

   return;
}



/*
   FUNCTION: 		Cancel

   Synopsis: 
	The user hits return or uses the button to use what is displayed.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

static void
Cancel(w, form, call_data)

Widget
   w,
   form;
XtPointer
   call_data;

{
   RSFList
      *rptr;
   
   XtPopdown(Mappanel);

   for (rptr = Downmapping.segptr; rptr != NULL; rptr = rptr->next)
   {
      if ((rptr->context != NULL) &&
	  (rptr->context->list != NULL) &&
	  (rptr->context->list->button != NULL))
      {
	 break;
      }
   }
   if (rptr != NULL)
   {
      XawFormDoLayout(form, False);
      ClearPanel(form, rptr);
      XawFormDoLayout(form, True);
   }
   return;
}



/*
   FUNCTION: 		CreateMapPanel

   Synopsis: 
	Create the panel that will be popped up if/when the user wants
	to make a different instance based on context.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
CreateMapPanel(parent)

Widget
   parent;

{
   Widget
      panelform, panelttl,
      prefacew,
      taglbl,
      choiceform,
      labelrow, viewrow, ctxcol, rexcol,
      actiongrp,
      okayw,
      cancelw,
      ctxtlbl,
      ctxtview,
      ctxtform,
      rexlbl,
      rexview,
      rexform;
   XtAccelerators
      appaccels,
      canaccels;
   static String
      appacctab  = "<Key>Return:    set() notify() reset()\n\
                    <Key>KP_Enter:  set() notify() reset()\n\
                    <Key>Linefeed:  set() notify() reset()",
      canacctab  = "<Key>Delete:    set() notify() reset()\n\
                    <Key>BackSpace: set() notify() reset()";
   
   Mappanel = XtVaCreatePopupShell( "Special Case Selection",
       transientShellWidgetClass, parent,
       XtNallowShellResize, True,
       NULL );

   panelform = XtVaCreateManagedWidget( "panelform",
       groupWidgetClass, Mappanel,
       XtNspacing, 4,
       NULL );
   
   panelttl = XtVaCreateManagedWidget( "panelttl",
       groupWidgetClass, panelform,
       XtNorientation, XtorientHorizontal,
       XtNspacing, 0,
       XtNmargin,  6,
       NULL );
   prefacew = XtVaCreateManagedWidget( "prefacew",
       labelWidgetClass, panelttl,
       XtNborderWidth, 0,
       XtNlabel,       "Tag:",
       NULL );
   taglbl = XtVaCreateManagedWidget( "taglbl",
       labelWidgetClass, panelttl,
       XtNborderWidth, 0,
       XtNlabel,       "",
       NULL );
   
   choiceform = XtVaCreateManagedWidget("choiceform",
       tableWidgetClass, panelform,
       XtNborderWidth,   1,
       XtNhorzPlacement, tpFlush,
       XtNhorzSpacing,   6,
       XtNvertSpacing,   2,
       XtNhorzMargin,    6,
       XtNvertMargin,    6,
       XtNalignment,     gaFill,
       XtNstretch,       True,
       XtNmargin,        6,
       NULL );
   labelrow = XtVaCreateWidget( "labelrow",
       tableRowObjectClass, choiceform,
       NULL );
   viewrow = XtVaCreateWidget( "viewrow",
       tableRowObjectClass, choiceform,
       XtNstretch, True,
       NULL );
   ctxcol = XtVaCreateWidget( "ctxcol",
       tableColObjectClass, choiceform,
       XtNstretch, True,
       NULL );
   rexcol = XtVaCreateWidget( "rexcol",
       tableColObjectClass, choiceform,
       XtNstretch, True,
       NULL );

   ctxtlbl = XtVaCreateManagedWidget( "ctxtlbl",
       labelWidgetClass, choiceform,
       XtNborderWidth, 0,
       XtNlabel,       "Contexts:           ",
       XtNtableRow,    labelrow,
       XtNtableCol,    ctxcol,
       NULL );
   ctxtview = XtVaCreateManagedWidget( "ctxtview",
       viewportWidgetClass, choiceform,
       XtNallowVert,     True,
       XtNallowHoriz,    True,
       XtNforceBars,     False,
       XtNpreference,    False,
/*
       XtNprefWidth,     150,
       XtNminWidth,      100,
       XtNprefHeight,    100,
       XtNminHeight,     50,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      ctxcol,
       NULL );
   ctxtform = XtVaCreateManagedWidget( "ctxtform",
       viewformWidgetClass, ctxtview,
       XtNresizable,   True,
       NULL );

   rexlbl = XtVaCreateManagedWidget( "rexlbl",
       labelWidgetClass, choiceform,
       XtNborderWidth, 0,
       XtNlabel,       "Regular Expressions:",
       XtNtableRow,    labelrow,
       XtNtableCol,    rexcol,
       NULL );
   rexview = XtVaCreateManagedWidget( "rexview",
       viewportWidgetClass, choiceform,
       XtNallowVert,     True,
       XtNallowHoriz,    True,
       XtNforceBars,     False,
       XtNpreference,    False,
/*
       XtNprefWidth,     150,
       XtNminWidth,      100,
       XtNprefHeight,    100,
       XtNminHeight,     50,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      rexcol,
       NULL );
   rexform = XtVaCreateManagedWidget( "rexform",
       viewformWidgetClass, rexview,
       XtNresizable,   True,
       NULL );

   actiongrp = XtVaCreateManagedWidget( "actiongrp",
       groupWidgetClass, panelform,
       XtNorientation,  XtorientHorizontal,
       XtNspacing,      8,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );

   appaccels  = XtParseAcceleratorTable(appacctab);
   canaccels  = XtParseAcceleratorTable(canacctab);

   okayw = XtVaCreateManagedWidget( "okayw",
       commandWidgetClass, actiongrp,
       XtNborderWidth,   2,
       XtNlabel,         " OK ",
       XtNaccelerators,  appaccels,
       XtNalignment,     gaCenter,
       XtNmargin,        2,
       NULL );
   XtAddCallback(okayw, XtNcallback, Okay, panelform);

   cancelw = XtVaCreateManagedWidget( "cancelw",
       commandWidgetClass, actiongrp,
       XtNlabel,         "Cancel",
       XtNaccelerators,  canaccels,
       XtNalignment,     gaCenter,
       XtNmargin,        2,
       NULL );
   XtAddCallback(cancelw, XtNcallback, Cancel, panelform);

   XtInstallAllAccelerators(panelform, panelform);

   XtRealizeWidget(Mappanel);
   
   return;
}


