/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: permute.c,v $
   Purpose:     Create the permutation part of the interface.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 16:20:46 $

   HISTORY
   $Log:	permute.c,v $
 * Revision 1.6  92/05/29  16:20:46  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.5  92/05/20  15:49:22  ware
 * 
 * For some reason, certain window manager settings prevent the event
 * handler from ever getting called.  I don't quite understand why.  What
 * I do know is that making it a translation fixes the problem.  That is
 * about all that happens.  I change SymbolPlace from being an event
 * handler to being an action proc.
 * 
 * Revision 1.4  92/03/16  12:20:56  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.3  1992/02/21  23:02:45  cso
 * Removed spec2gen related routines
 *
 * Revision 1.2  1992/02/18  18:26:57  cso
 * Restructured the interface for the opposite xlation direction
 *
 * Revision 1.1  1992/02/06  05:35:18  cso
 * Initial revision
 *
 * Revision 1.2  1992/01/14  20:55:30  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/08  19:56:54  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/*  EXTERNS  */

#include "callbacks.h"

/*  GLOBALS  */



/*
   FUNCTION: 		CreateMapDisplay

   Synopsis: 
       Create the display for performing permutations.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


Widget CreateMapDisplay( parent, left, right )
Widget
   parent,        /* Parent widget of this display */
   left,          /* left column */
   right;         /* right column */
{
   Widget
      lblrow, specrow, genrow,
      /* general stuff */
      genlbls, gencmds,
      altlabel,   /* Label widget for the symbol in focus */
      focus,      /* Display of the symbol in focus */
      nextalt,    /* Command to move to the next alternative */
      nodeframe,  /* Form widget for the production widgets */
      nodeview,   /* The viewport of the node frame */
      prevalt,    /* Command to move to the previous alternative */
      prodlabel,  /* Label between previous and next buttons */
      /* specific stuff */
      speccmds,
      nextperm,    /* Command to move to the next permutation */
      pnodeframe, /* Form widget for the production widgets */
      pnodeview,  /* The viewport of the node frame */
      prevperm,   /* Command to move to the previous permutation */
      permlabel;  /* Label between previous and next buttons */
   XtTranslations 
      formtrans;
   static String
      formtransstr = "<Btn1Down>,<Btn1Up>: SymbolPlace()";

   /*
    *   [altlabel (label)]  [focus (label)]
    */

   lblrow = XtVaCreateWidget( "prodlbls",
       tableRowObjectClass, parent,
       NULL );
   genrow = XtVaCreateWidget( "genrow",
       tableRowObjectClass, parent,
       NULL );
   specrow = XtVaCreateWidget( "specrow",
       tableRowObjectClass, parent,
       NULL );

   genlbls = XtVaCreateManagedWidget( "genlbls",
       groupWidgetClass, parent,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     0,
       XtNtableRow,    lblrow,
       XtNtableCol,    right,
       NULL );

   altlabel = XtVaCreateManagedWidget( "altlabel",
       labelWidgetClass, genlbls,
       XtNlabel,        "Permutations and Production for",
       XtNborderWidth,  0,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );

   /*
    * Create the label containing the symbol string that is the parent
    * of the definition displayed in nodeframe
    */
   focus = XtVaCreateManagedWidget( "focus",
       labelWidgetClass, genlbls,
       XtNlabel,        "",
       XtNborderWidth,  0,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );


   gencmds = XtVaCreateManagedWidget( "gencmds",
       groupWidgetClass, parent,
       XtNspacing,       0,
       XtNplacement,     gpFlush,
       XtNvertAlignment, taFill,
       XtNtableRow,      genrow,
       XtNtableCol,      left,
       NULL );

   prevalt = XtVaCreateManagedWidget( "prevalt",
       commandWidgetClass, gencmds,
       XtNlabel,     "Previous",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtAddCallback( prevalt, XtNcallback, AltMove, Previous );

   prodlabel = XtVaCreateManagedWidget( "deflabel",
       labelWidgetClass, gencmds,
       XtNlabel,       "General\nOrder",
       XtNborderWidth, 0,
       NULL );

   nextalt = XtVaCreateManagedWidget( "nextalt",
       commandWidgetClass, gencmds,
       XtNlabel,     "  Next  ",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtAddCallback( nextalt, XtNcallback, AltMove, Next );

   /*
    * Create the viewport through which to view productions
    * Although the scrollbars are handled by a viewport if the
    * user directly manipulates them, AdjustScrollBars will alter
    * them so that the node being currently viewed is in view.  
    * Refer to utils.c
    */
   nodeview = XtVaCreateManagedWidget( "nodeview",
       viewportWidgetClass, parent,
       XtNallowHoriz,    True,
       XtNuseBottom,     False,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     550,
       XtNminWidth,      100,
*/
       XtNprefHeight,    70,
       XtNminHeight,     48,
       XtNtableRow,      genrow,
       XtNtableCol,      right,
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       NULL );

   /*
    * Create the form widget that will contain nodes in a production
    */
   nodeframe = XtVaCreateManagedWidget( "nodeframe",
       viewformWidgetClass, nodeview,
       XtNresizable,       True,
       XtNdefaultDistance, DEFAULTSPACE,
       NULL );


   speccmds = XtVaCreateManagedWidget( "speccmds",
       groupWidgetClass, parent,
       XtNspacing,       0,
       XtNplacement,     gpFlush,
       XtNvertAlignment, taFill,
       XtNtableRow,      specrow,
       XtNtableCol,      left,
       NULL );

   prevperm = XtVaCreateManagedWidget( "prevperm",
       commandWidgetClass, speccmds,
       XtNlabel,     "Previous",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtUnmanageChild(prevperm);
/*
   XtAddCallback( prevperm, XtNcallback, PermMove, Previous );
*/
   permlabel = XtVaCreateManagedWidget( "permlabel",
       labelWidgetClass, speccmds,
       XtNlabel,       "Specific\nOrder",
       XtNborderWidth, 0,
       NULL );

   nextperm = XtVaCreateManagedWidget( "nextperm",
       commandWidgetClass, speccmds,
       XtNlabel,     "  Next  ",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtUnmanageChild(nextperm);
/*
   XtAddCallback( nextperm, XtNcallback, PermMove, Next );
*/

   /*
    * Create the viewport through which to view permutations
    */
   pnodeview = XtVaCreateManagedWidget( "pnodeview",
       viewportWidgetClass, parent,
       XtNallowHoriz,    True,
       XtNuseBottom,     True,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     550,
       XtNminWidth,      100,
*/
       XtNprefHeight,    70,
       XtNminHeight,     48,
       XtNtableRow,      specrow,
       XtNtableCol,      right,
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       NULL );

   pnodeframe = XtVaCreateManagedWidget( "pnodeframe",
       viewformWidgetClass, pnodeview,
       XtNresizable,       True,
       XtNdefaultDistance, DEFAULTSPACE,
       NULL );
   formtrans = XtParseTranslationTable(formtransstr);
   XtOverrideTranslations(pnodeframe, formtrans);

   return(nextperm);
}
