
/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: prodlist.c,v $
   Purpose:
       To handle the available symbol list.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/05/29 16:20:51 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  ProdSelect

   EXPORT  CreateProdList
                               
*/

/* HISTORY
   $Log:	prodlist.c,v $
 * Revision 1.4  92/05/29  16:20:51  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.3  92/03/16  12:21:02  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.2  1992/02/18  18:23:44  cso
 * Fixed some dependencies
 *
 * Revision 1.1  1992/02/06  05:35:36  cso
 * Initial revision
 *
 * Revision 1.3  1992/01/14  20:55:38  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:22  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:57:09  cso
 * Initial revision
 *
 * Revision 1.1.1.3  1991/10/04  20:17:27  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/*  EXTERNS */

#include "callbacks.h"
#include "display.h"
#include "initialize.h"
#include "interface.h"

/*  GLOBALS */

String
   Prodlist[MAXRHS];




/*
   FUNCTION: 		GenerateProdlist

   Synopsis: 
       Create the list of productions.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateProdList()

{
   int
      i,
      len;
   RhsList
      *rl;
   SymbolList
      *sl;
   
   Pindex = 0;
   for (i = 0; i < Gmax; i++)
   {
      for (rl = Grammar[i].rhs; rl != NULL; rl = rl->next)
      {
	 len = strlen(Grammar[i].lhs) + 8;
	 for (sl = rl->symbol; sl != NULL; sl = sl->next)
	 {
	    len = len + strlen(sl->symbolstr) + 2;
	 }
	 Prodlist[Pindex] = (String) XtMalloc(len + 1);
	 if (rl-> prev == NULL)
	 {
	    sprintf(Prodlist[Pindex], "%s   ::=", Grammar[i].lhs);
	 }
	 else 
	 {
	    sprintf(Prodlist[Pindex], "     |");
	 }
	 
	 for (sl = rl->symbol; sl != NULL; sl = sl->next)
	 {
	    Prodlist[Pindex] = strcat(Prodlist[Pindex], " ");
	    Prodlist[Pindex] = strcat(Prodlist[Pindex], sl->symbolstr);
	    Prodlist[Pindex] = strcat(Prodlist[Pindex], sl->occurstr);
	 }
	 Pindex++;
      }
   }
   XawListChange(XtNameToWidget(Toplevel, "*prodlist"),
		 Prodlist, Pindex, -1, True);

   return;
}




/*
   FUNCTION: 		ProdSelect

   Synopsis: 
       If the user selects an itme in the list, reuse that symbol or
       move to that symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       Tree              The current focus of the interface

   Globals Modified:
       Tree              May be moved to a different symbol

   Return value:	void

   Range:		none
*/

static void
ProdSelect(w, client_data, current)

Widget
   w;           /* The list widget calling this function */
caddr_t
   client_data; /* Unused */
XawListReturnStruct
   *current; /* The symbol and its index selected by the user */

{
   int
      i,
      ndx;
   RhsList
      *rl;
   
#ifdef DEBUG
   fprintf(stderr, "selection = [%s]\nposition = [%d]\n\n",
	   current->string, current->list_index);
#endif
   ndx = current->list_index + 1;
   
   for (i = 0; i < Gmax; i++)
   {
      if (ndx > Grammar[i].numrhs)
      {
	 ndx = ndx - Grammar[i].numrhs;
      }
      else
      {
	 break;
      }
   }      
   for (rl = Grammar[i].rhs;
	((rl != NULL) && (ndx > 1)); rl = rl->next, ndx--)
   {
   }
   if (rl != NULL)
   {
      Grammar[i].activerhs = rl;
   }
   ndx = Gindex;
   Gindex = i;
   UpdateInterface(ndx, Gindex);
 
   return;
}




/*
   FUNCTION: 		CreateProdList

   Synopsis: 
       Create the interface for the defined symbol list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above, except that the list is for all symbols.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget CreateProdList( parent, left, right )
Widget
   parent,          /* Parent of this interface */
   left,            /* left column */
   right;           /* right column */
{
   Widget
      lblrow, prodrow,
      prodcmds,
      movelabel,
      prodlabel,     /* Label widget with title of list */
      prodlist,      /* List widget */
      nextprod,
      prevprod,
      prodview;      /* Viewport so that list can grow beyond screen */

   lblrow = XtVaCreateWidget( "prodlbls",
       tableRowObjectClass, parent,
       NULL );
   prodrow = XtVaCreateWidget( "prodrow",
       tableRowObjectClass, parent,
       XtNstretch,     True,
       NULL );

   prodlabel = XtVaCreateManagedWidget( "gramlabel",
       labelWidgetClass, parent,
       XtNlabel,       "Grammar Productions:",
       XtNborderWidth, 0,
       XtNtableRow,    lblrow,
       XtNtableCol,    right,
       NULL );

   prodcmds = XtVaCreateManagedWidget( "prodcmds",
       groupWidgetClass, parent,
       XtNspacing,       0,
       XtNplacement,     gpTopOrLeft,
       XtNvertAlignment, taFill,
       XtNtableRow,      prodrow,
       XtNtableCol,      left,
       NULL );

   prevprod = XtVaCreateManagedWidget( "prevprod",
       commandWidgetClass, prodcmds,
       XtNlabel,     "Previous",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtAddCallback( prevprod, XtNcallback, ProdMove, Previous );

   movelabel = XtVaCreateManagedWidget( "movelabel",
       labelWidgetClass, prodcmds,
       XtNlabel,       "General\nForm",
       XtNborderWidth, 0,
       NULL );

   nextprod = XtVaCreateManagedWidget( "nextprod",
       commandWidgetClass, prodcmds,
       XtNlabel,     "  Next  ",
       XtNsensitive, False,
       XtNalignment, gaFill,
       NULL );
   XtAddCallback( nextprod, XtNcallback, ProdMove, Next );

   prodview = XtVaCreateManagedWidget( "prodview",
       viewportWidgetClass, parent,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNuseBottom,     False,
       XtNuseRight,      False,
       XtNforceBars,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     550,
       XtNminWidth,      100,
       XtNprefHeight,    300,
       XtNminHeight,     50,
*/
       XtNtableRow,      prodrow,
       XtNtableCol,      right,
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       NULL );

   prodlist = XtVaCreateManagedWidget( "prodlist",
       listWidgetClass, prodview,
       XtNdefaultColumns, 1,
       XtNforceColumns,   True,
       XtNlist,           Prodlist,
       XtNnumberStrings,  0,
       NULL );
   XtAddCallback(prodlist, XtNcallback, ProdSelect, NULL);

   return(prodview);
}
