/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: rsfparse.y,v $
   Purpose:     To parse and validate a .rsf file.  If get_file ==
   True, it is used to load a file into the interface, else it is used to
   generate the temp.[yacc/lex] files from the Make option.

   Programmer:  Conleth O'Connell, Julie Barnes
   Reviewer:    
   Revision:    $Revision: 1.1 $
   Date:        $Date: 92/05/22 15:28:47 $

*/

/* HISTORY
   $Log:	rsfparse.y,v $
 * Revision 1.1  92/05/22  15:28:47  ware
 * renamed
 * 
 * Revision 1.3  92/03/16  12:21:12  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.2  1992/03/10  20:15:41  cso
 * Changed the language of the spec file to load and save specs
 *
 * Revision 1.1  1992/02/28  15:01:17  cso
 * Initial revision
 *
 * Revision 1.8  1992/02/25  02:52:55  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.7  1992/02/20  04:55:35  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.6  1992/01/28  18:37:35  cso
 * Used allocation of constant strings so they can be freed
 *
 * Revision 1.5  1991/12/05  16:02:59  cso
 * Added the separator tag support code.
 * 	- changed the parsing language to find the sep. tag info
 * 	- added the capability of being backward compatible and
 * 		updating the data structures
 *
 * Changed the initialization of some variables.
 *
 * Added checks to catch user-modified spec files (that they aren't
 * supposed to do...|-), and to recover...
 *
 * Revision 1.4  1991/11/08  23:04:32  cso
 * Removed the possibility of generating bogus cpde from partial specs.
 * The situation was rare but possible, when a user would only specify
 * one of the two regular expressions necessary for explicit segmenting
 * tags.
 *
 * Revision 1.3  1991/11/07  21:59:49  cso
 * Modified the interface load parse to put a space after the context
 * clause keywords so that variable-width fonts will not make the
 * interface look bad.  Still need to adjust the dialog to do the same.
 * Although this changes the rsf spec language, the parse is backwards
 * compatible with old specifications so on subsequent loads and saves
 * the update is transparent to the user.
 *
 * Revision 1.2  1991/11/06  17:05:55  cso
 * *** empty log message ***
 *
 * Revision 1.1.1.2  1991/10/04  20:20:10  jbm
 * First HaL mods to ICA
 *
Revision 1.2  90/11/06  17:35:37  cso
Removed TABs.

Revision 1.1  90/10/05  13:19:13  cso
Initial revision

*/
/* Yacc specification for the retag specification language */
/* compiler.                                               */
/*                                                         */
/* Copyright 1990. The Ohio State University.              */

%{

#include "widgets.h"
#include "defs.h"
#include "types.h"

#include "mapping.h"
#include "status.h"

extern int
   linecount;

extern void
   yyerror();
%}

%union
{
 int ival;
 char *trans;
}

%token <trans>
AMBIGCHAR
DATACHAR
DEFINE
DELTAG
END
EXPTAG
IMPTAG
IN
LINEFEED
NEST
PROCIN
SEPTAG
START
STRING
SYMBOL
TAGCHAR

%type <trans>
dataalph
datatagalph
delimiting
explicit
implicit
oldornew
oldtagline
oldtaglines
optctxtclause
optstring
procin
retagfile
segmenting
separatorOPTL
separator
symbol
tagalph
tagline
taglines
tagspec

%start retagfile

%%

retagfile	: dataalph tagalph datatagalph tagspec
		{
		}
		;

dataalph	: DATACHAR STRING LINEFEED
		{
		   free($2);
		}
		;

tagalph		: TAGCHAR optstring LINEFEED
		{
		}
		;

datatagalph	: AMBIGCHAR optstring LINEFEED
		{
		}
		;

optstring	: STRING
		{
		   free($1);
		}
		| /* null */
		{
		}
		;



tagspec		: symbol taglines procin taglines segmenting
		{
		}
		;

segmenting	: implicit taglines oldornew separatorOPTL
		{
		}
		;

oldornew	: delimiting oldtaglines
		{
		}
		| explicit taglines
		{
		}
		;

separatorOPTL	: separator taglines
		{
		}
		| /* empty */
		{
		}
		;
   
symbol		: SYMBOL
		{
		}
		;

procin		: PROCIN
		{
		}
		;

implicit	: IMPTAG
		{
		}
		;

explicit	: EXPTAG
		{
		}
		;

delimiting	: DELTAG
		{
		}
		;

separator	: SEPTAG
		{
		}
		;
   
tagline		: STRING STRING optctxtclause LINEFEED
		{
		   /* REXP TAG optctxtclause LINEFEED */
		   (void) InstallMappingInfo($2, $1, "\"\"");
		   free($1);
		   free($2);
		}
		;

oldtagline	: STRING STRING STRING STRING optctxtclause LINEFEED
		{
		   /* REXP REXP BTAG ETAG optctxtclause LINEFEED */
		   (void) InstallMappingInfo($3, $1, "\"\"");
		   (void) InstallMappingInfo($4, $2, "\"\"");
		   free($1);
		   free($2);
		   free($3);
		   free($4);
		}
		;

optctxtclause	: START STRING
		{
		   free($2);
		}
		| END STRING
		{
		   free($2);
		}
		| IN STRING
		{
		   free($2);
		}
		| NEST STRING IN STRING
		{
		   free($2);
		   free($4);
		}
		| DEFINE STRING
		{
		   free($2);
		}
		| /* null */
		{
		}
		;

taglines	: taglines tagline
		{
		}
		| /* null */
		{
		}
		;

oldtaglines	: oldtaglines oldtagline
		{
		}
		| /* null */
		{
		}
		;

%%
