/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: save.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/03/16 12:21:20 $

   HISTORY
   $Log:	save.c,v $
 * Revision 1.5  92/03/16  12:21:20  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.3  1992/03/10  20:15:36  cso
 * Changed the language of the spec file to load and save specs
 *
 * Revision 1.2  1992/02/28  14:53:25  cso
 * Integrated the call to save the mapping info
 *
 * Revision 1.1  1992/02/06  05:35:54  cso
 * Initial revision
 *
 * Revision 1.3  1992/01/14  20:55:45  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:27  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:57:21  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "initialize.h"
#include "mapping.h"

/*  GLOBALS  */

void
SaveFile(fptr)

FILE
   *fptr;

{
   CtxtList
      *clptr;
   int
      i;
   PermutationList
      *pl;
   RhsList
      *rl;
   RSFList
      *dptr;
   SymbolList
      *sl;
   
   fprintf(fptr, "$$$$\n");
   for (dptr = Downmapping.symptr; dptr != NULL; dptr = dptr->next)
   {
      fprintf(fptr, "%s\t%s\t", dptr->regexp, dptr->tag);
      if ((dptr->instance != NULL) && (*dptr->instance != '\0'))
      {
	 fprintf(fptr, "%s\n", dptr->instance);
      }
      else
      {
	 fprintf(fptr, "\"\"\n");
      }
   }
   for (dptr = Downmapping.pinptr; dptr != NULL; dptr = dptr->next)
   {
      fprintf(fptr, "%s\t%s\t", dptr->regexp, dptr->tag);
      if ((dptr->instance != NULL) && (*dptr->instance != '\0'))
      {
	 fprintf(fptr, "%s\n", dptr->instance);
      }
      else
      {
	 fprintf(fptr, "\"\"\n");
      }
   }
   for (dptr = Downmapping.segptr; dptr != NULL; dptr = dptr->next)
   {
      if (dptr->dupflag == False)
      {
	 fprintf(fptr, "%s\t%s\t", dptr->regexp, dptr->tag);
	 if ((dptr->instance != NULL) && (*dptr->instance != '\0'))
	 {
	    fprintf(fptr, "%s\n", dptr->instance);
	 }
	 else
	 {
	    fprintf(fptr, "\"\"\n");
	 }
	 if ((dptr->context != NULL) && (dptr->context->rexlist != NULL))
	 {
	    for (clptr = dptr->context->rexlist; clptr != NULL;
		 clptr = clptr->next)
	    {
	       fprintf(fptr, "%s\t%s\t\"\"\n", clptr->context, dptr->tag);
	    }
	 }	    
	 
	 fprintf(fptr, "\\\\\\\\\n");
	 fprintf(fptr, "%s\t:IN:", dptr->tag);
	 if (dptr->context != NULL)
	 {
	    for (clptr = dptr->context->list; clptr != NULL;
		 clptr = clptr->next)
	    {
	       fprintf(fptr, "\t%s", clptr->context);
	    }
	 }	    
	 fprintf(fptr, "\n");

	 while ((dptr->next != NULL) && (dptr->next->dupflag == True))
	 {
	    dptr = dptr->next;
	    fprintf(fptr, "%s\t%s\t", dptr->regexp, dptr->tag);
	    if ((dptr->instance != NULL) && (*dptr->instance != '\0'))
	    {
	       fprintf(fptr, "%s\n", dptr->instance);
	    }
	    else
	    {
	       fprintf(fptr, "\"\"\n");
	    }
	 }
	 fprintf(fptr, "////\n");
      }
   }
   fprintf(fptr, "$$$$\n");

   /*
    * Step through all the productions
    */
   for (i = 0; i < Gmax; i++)
   {
      fprintf(fptr, "%s\t:", Grammar[i].lhs);

      /*
       * step through each production
       */
      for (rl = Grammar[i].rhs; rl != NULL; rl = rl->next)
      {
	 /*
	  * print out all the symbols in the production (and occurence) 
	  */
	 for (sl = rl->symbol; sl != NULL; sl = sl->next)
	 {
	    /*
	     * only print occurence string if it exists 
	     * print the position
	     */
	    if (sl->occurstr != NULL)
	    {
	       fprintf(fptr, " %s%s", sl->symbolstr, sl->occurstr);
	    }
	    else
	    {
	       fprintf(fptr, " %s", sl->symbolstr);
	    }
	    fprintf(fptr, "#[%d]", sl->position);
	 }
	 fprintf(fptr, "\n");
	 
	 /*
	  * now go through all the permutations for this alternative 
	  */
	 for (pl = rl->perminstance; pl != NULL; pl = pl->next)
	 {
	    fprintf(fptr, "\t(");

	    /*
	     * print out the symbols for the permutation 
	     */
	    for (sl = pl->permsymbol; sl != NULL; sl = sl->next)
	    {
	       fprintf(fptr, "%s",    sl->symbolstr);
	       if (sl->occurstr != NULL)
	       {
		  fprintf(fptr, "%s", sl->occurstr);
	       }
	       fprintf(fptr, "#[%d]", sl->position);
	       if (sl->next != NULL)
	       {
		  fprintf(fptr, " ");
	       }
	    }
	    fprintf(fptr, ")\n");  
	 }
	 if (rl->next != NULL)
	 {
	    fprintf(fptr, "\t|");
	 }
      }
      fprintf(fptr, ";\n\n");
   }
   return;
}
