/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: symtab.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.3 $
   Date:        $Date: 92/03/16 12:21:39 $

   HISTORY
   $Log:	symtab.c,v $
 * Revision 1.3  92/03/16  12:21:39  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.2  1992/03/10  20:14:00  cso
 * Used XtNew to create symbol table instance
 *
 * Revision 1.1  1992/02/06  05:36:43  cso
 * Initial revision
 *
 * Revision 1.2  1992/01/14  20:55:51  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/10  02:24:07  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "initialize.h"

/*  GLOBALS  */



/*
   FUNCTION: 		PutSymbol

   Synopsis: 
       Put a symbol in a table.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


static SymbolTable *
PutSymbol(tableroot, symbolstr)

SymbolTable
   *tableroot;
String
   symbolstr;

{
   int
      newflag;
   SymbolTable
      *tmptab,
      *tptr;

   tmptab = XtNew(SymbolTable);
   newflag = 1;
   tptr    = tableroot;
   if (tableroot != NULL)
   {
      if (strcmp(tableroot->symbol, symbolstr) == 0)
      {
	 newflag = 0;
	 tptr = tableroot;
      }
      else
      {
	 for (tptr = tableroot; tptr->next != NULL; tptr = tptr->next)
	 {
	    if (strcmp(tptr->next->symbol, symbolstr) == 0)
	    {
	       newflag = 0;
	       tptr = tptr->next;
	       break;
	    }
	 }
      }	 
   }
   if (newflag == 1)
   {
      tmptab->symbol   = symbolstr;
      tmptab->optional = 0;
      tmptab->star     = 0;
      tmptab->plus     = 0;
      tmptab->next     = NULL;
      
      if (tableroot == NULL)
      {
	 tableroot = tmptab;
      }
      else
      {
	 tptr->next = tmptab;
      }
   }
   else
   {
      XtFree(tmptab);
      tmptab = tptr;
   }
   return(tmptab);
}




/*
   FUNCTION: 		MakeSymbolTable

   Synopsis: 
       Create a symbol table.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

SymbolTable *
MakeSymbolTable(tableroot)

SymbolTable
   *tableroot;

{
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr;
   SymbolTable
      *tptr;
      
   tableroot = NULL;
   
   tptr = PutSymbol(tableroot, Grammar[0].lhs);
   if (tableroot == NULL)
   {
      tableroot = tptr;
   }
   
   for (i = 0; i < Gmax; i++)
   {
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    if (sptr->symbolstr[0] != '<')
	    {
	       tptr = PutSymbol(tableroot, sptr->symbolstr);
	       if (tableroot == NULL)
	       {
		  tableroot = tptr;
	       }
	       
	       
	       switch (sptr->occurstr[0])
	       {
	       default:
		  break;
		  
	       case '*':
		  tptr->star++;
		  break;
		  
	       case '+':
		  tptr->plus++;
		  break;
		  
	       case '?':
		  tptr->optional++;
		  break;
	       }
	    }
	 }
      }
   }
   return(tableroot);
}




/*
   FUNCTION: 		MakeTokenTable

   Synopsis: 
       Create the token table.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

SymbolTable *
MakeTokenTable(tableroot)

SymbolTable
   *tableroot;

{
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr;
   SymbolTable
      *tptr;
      
   tableroot = NULL;
   for (i = 0; i < Gmax; i++)
   {
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    if (sptr->symbolstr[0] == '<')
	    {
	       tptr = PutSymbol(tableroot, sptr->symbolstr);
	       if (tableroot == NULL)
	       {
		  tableroot = tptr;
	       }
	    }
	 }
      }
   }
   return(tableroot);
}



/*
   FUNCTION: 		FreeSymbolTable

   Synopsis: 
       Free up the symbol table.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FreeSymbolTable(table)

SymbolTable
   *table;

{
   SymbolTable
      *sptr1,
      *sptr2;

   sptr2 = table;
   for (sptr1 = table; sptr1 != NULL; sptr1 = sptr2)
   {
      sptr2 = sptr1->next;
      XtFree(sptr1);
   }
   return;
}
