/*
  $Log:	types.h,v $
 * Revision 1.5  92/03/16  12:21:45  ware
 * Merging cso and OSU versions.  Changes for titles.
 * 
 * Revision 1.4  1992/03/10  20:13:23  cso
 * Added some new types
 *
 * Revision 1.3  1992/02/28  14:54:00  cso
 * Integrated the mapping info data structures
 *
 * Revision 1.2  1992/02/21  23:06:06  cso
 * Added the string mapping structure
 *
 * Revision 1.1  1992/02/06  05:36:59  cso
 * Initial revision
 *
 * Revision 1.2  1992/01/10  02:25:31  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:57:52  cso
 * Initial revision
 *
 * Revision 1.1.1.3  1991/10/04  20:17:58  jbm
 * First HaL mods to ICA
 *
 * Revision 1.10  90/11/20  16:06:15  cso
 * Removed the type FileString.  Changed exposed to a Boolean.
 * 
 * Revision 1.9  90/11/05  18:35:19  cso
 * Added a NodeListType for the definition pointer.
 * 
 * Revision 1.8  90/10/29  10:32:12  cso
 * Expanded SemanticFlagType to distinguish between Symbol and Root
 * versions of the semantic view.
 * Also changed the type name to Semantic from View.
 * 
 * Revision 1.7  90/10/24  15:50:34  cso
 * Added new enum types based on the old defines in defs.h to make for 
 * better type checking in compiles.
 * 
 * Revision 1.6  90/09/07  09:43:17  cso
 * Added an expose flag to Alt for three-dimensional layout.
 * 
 * Revision 1.5  90/09/03  18:31:46  cso
 * Removed T_Struct, and added, instead, left and right Alt ptrs to NodeType.
 * 
 * Revision 1.4  90/09/03  14:55:42  cso
 * Changed char* to String and Bool to Boolean and a few other cosmetics.
 * 
 * Revision 1.3  90/08/26  17:44:57  cso
 * Added the RCS Log message at the top for these messages.
 * 
*/

typedef enum DirectionType
{
   Previous = 0,
   Next     = 1
} Direction;
   
typedef enum MarkupType
{
   Start   = 1,
   End     = 2,
   Comma   = 3,
   Symbol  = 4,
   Process = 5
} Markup;



/*
 * The list of tokens (tags) and their corresponding text
 * translations.  This probably needs to be expanded for using
 * contextual information like production lhs or adding the translation
 * to the symbol table structure.
 */
typedef struct TokenMappingType {
  char                    *token;
  char                    *text;
  struct TokenMappingType *next;
} TokenMapping;

/*
  The symbol table structure.
 */
typedef struct SymbolTableType
{
   char   *symbol;     /* Name of a symbol in the grammar         */
   int    star;        /* Count of the use of this occurrence     */
   int    plus;        /* Count of the use of this occurrence     */
   int    optional;    /* Count of the use of this occurrence     */
   struct SymbolTableType *next;
} SymbolTable;

typedef struct SymbolListType
{
   int
      position;
   String
      symbolstr,
      occurstr;
   Widget
      symbutton;
   struct SymbolListType
      *next,
      *prev;
} SymbolList;

typedef struct PermutationListType
{
   SymbolList
      *permsymbol;
   struct PermutationListType
      *next,
      *prev;
} PermutationList;

typedef struct RhsListType
{
   SymbolList
      *symbol;
   PermutationList
      *perminstance,
      *activeperm;
   struct RhsListType
      *next,
      *prev;
} RhsList;

typedef struct ProductionType
{
   String
      lhs;
   RhsList
      *rhs,
      *activerhs;
   int
      numrhs;
   struct ProductionType
      *next,
      *prev;
} Production;



typedef struct CtxtListType
{
   String
      context;
   Widget
      button;
   struct CtxtListType
      *next;
} CtxtList;


typedef struct NestingInfoType
{
   String
      tag;
   CtxtList
      *list,
      *rexlist;
} NestingInfo;

typedef struct RSFListType 
{
   Boolean
      dupflag;
   Markup
      type;
   NestingInfo
      *context;
   String
      tag,
      regexp,
      instance;
   Widget
      tagw,
      rexw,
      insw;
   struct RSFListType
      *next;
} RSFList;

typedef struct TagListType
{
   RSFList
      *symptr,  /* symbol tag info */
      *pinptr,  /* processing instruction info */
      *segptr;  /* segmenting tag info */
} TagList;

typedef struct MapPtrType
{
   NestingInfo
      *nptr;
   CtxtList
      *cptr;
   RSFList
      *rptr;
} MapPtr;

