/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: display.c,v $
   Purpose:     Handle all display updates.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 16:08:59 $

   HISTORY
   $Log:	display.c,v $
 * Revision 1.6  92/05/29  16:08:59  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.  The interactive translators have not (yet) been
 * converted in this manner.
 * 
 * 
 * Revision 1.5  92/03/16  13:01:57  ware
 * Conditionalize the title
 * 
 * Revision 1.4  1992/02/21  23:11:06  cso
 * Trivial aesthetics: messages, formatting...
 *
 * Revision 1.3  1992/01/14  20:54:21  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:24:45  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:54:13  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "callbacks.h"
#include "initialize.h"
#include "interface.h"
#include "permshell.h"
#include "status.h"

/*  GLOBALS  */



/*
   FUNCTION: 		SensitizeAltButtons

   Synopsis: 
       Sensitize the movement buttons based on the data structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SensitizeAltButtons(rl)

RhsList
   *rl;     /* The alt ptr under consideration */

{
   Widget
      prevw,  /* Previous alternative button widget */
      nextw;  /* Next alternative button widget */

   /*
    * Get the proper widgets
    */
   prevw = XtNameToWidget(Toplevel, "*prevalt");
   nextw = XtNameToWidget(Toplevel, "*nextalt");

   if (rl == NULL)
   {
      XtSetSensitive(prevw, False);
      XtSetSensitive(nextw, False);
      return;
   }
   
   if (rl->prev == NULL)
   {
      XtSetSensitive(prevw, False);
   }
   else
   {
      XtSetSensitive(prevw, True);
   }
   if (rl->next == NULL)
   {
      XtSetSensitive(nextw, False);
   }
   else
   {
      XtSetSensitive(nextw, True);
   }
   return;
}



/*
   FUNCTION: 		SensitizePermButtons

   Synopsis: 
       Sensitize the permuation movement buttons based on the data
       structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
SensitizePermButtons(pl)

PermutationList
   *pl;

{
   Widget
      prevw,  /* Previous alternative button widget */
      nextw;  /* Next alternative button widget */

   /*
    * Get the proper widgets
    */
   prevw = XtNameToWidget(Toplevel, "*prevperm");
   nextw = XtNameToWidget(Toplevel, "*nextperm");

   if (pl == NULL)
   {
      XtSetSensitive(prevw, False);
      XtSetSensitive(nextw, False);
      return;
   }
   
   if (pl->prev == NULL)
   {
      XtSetSensitive(prevw, False);
   }
   else
   {
      XtSetSensitive(prevw, True);
   }
   if (pl->next == NULL)
   {
      XtSetSensitive(nextw, False);
   }
   else
   {
      XtSetSensitive(nextw, True);
   }
   return;
}




/*
   FUNCTION: 		SensitizeProdButtons

   Synopsis: 
       Sensitize the buttons based on data structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
SensitizeProdButtons(ndx)

int
   ndx;

{
   Widget
      prevw,  /* Previous alternative button widget */
      nextw;  /* Next alternative button widget */

   /*
    * Get the proper widgets
    */
   prevw = XtNameToWidget(Toplevel, "*prevprod");
   nextw = XtNameToWidget(Toplevel, "*nextprod");

   if (ndx <= 0)
   {
      XtSetSensitive(prevw, False);
   }
   else
   {
      XtSetSensitive(prevw, True);
   }
   
   if (ndx >= (Gmax - 1))
   {
      XtSetSensitive(nextw, False);
   }
   else
   {
      XtSetSensitive(nextw, True);
   }
   return;
}




/*
   FUNCTION: 		SensitizeAltToggles

   Synopsis: 
       Sensitize the grammar symbols based on if they are used in a
       permutation.


   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SensitizeAltToggles()
{
   PermutationList
      *pl;
   RhsList
      *rl;
   SymbolList
      *sl,
      *tsl;

   rl = Grammar[Gindex].activerhs;
   if (rl == NULL)
   {
      return;
   }
   
   /* 
    *  make all of the alts sensitive, and then we'll unsensitize
    *  them individually.
    */
   for (sl = rl->symbol; sl != NULL; sl = sl->next)
   {
      if (sl->symbutton != NULL)
      {
	 XtSetSensitive(sl->symbutton, True);
      }
   }
   
   pl = rl->activeperm;
   
   if (pl == NULL)
   {
      return;
   }
   
   /* for each perm symbol ... */
   for (tsl = pl->permsymbol; tsl != NULL; tsl = tsl->next)
   {
      /* for each alt symbol ...*/
      for (sl = rl->symbol; sl != NULL; sl = sl->next)
      {
	 /* check if they're the same ... */
	 if (tsl->position == sl->position)
	 {
	    /* and make it insensitive, if they match */
	    if (sl->symbutton != NULL)
	    {
	       XtSetSensitive(sl->symbutton, False);
	    }
	 }	/* if        */
      }		/* for (sl) */
   }		/* for (tsl)  */
   return;
}



/*
   FUNCTION: 		SymbolManage

   Synopsis: 
       Manage or unmanage the symbol buttons.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SymbolManage(ptr, manageflag)

SymbolList
   *ptr;
Boolean
   manageflag;

{
   /* unmanage all the symbol widgets */
   for ( ; ptr != NULL; ptr = ptr->next)
   {
      if (ptr->symbutton != NULL)
      {
	 if (manageflag == True)
	 {
	    XtManageChild(ptr->symbutton);
	 }
	 else
	 {
	    XtUnmanageChild(ptr->symbutton);
	 }
      }
   }
   return;
}



/*
   FUNCTION: 		UnmanageProd

   Synopsis: 
       Unmanage the production widgets.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
UnmanageProd(prodnum)

int
   prodnum;

{
   PermutationList
      *pl;
   RhsList
      *rl;

   for (rl = Grammar[prodnum].rhs; rl != NULL; rl = rl->next)
   {
      SymbolManage(rl->symbol, False);
      for (pl = rl->perminstance; pl != NULL; pl = pl->next)
      {
	 SymbolManage(pl->permsymbol, False);
      }
   }
   return;
}



/*
   FUNCTION: 		AdjustVerticalScrollbar

   Synopsis: 
       Adjust the vertical scrollbar on the production list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
AdjustVerticalScrollbar(selected)

int
   selected;


{
   Arg
      args[MAXARGS];    /* List of arguments */
   Dimension
      height, /* Height of the form widget */
      length;
   
   float
      new_top,     /* New top of thumb location */
      shown,       /* Percentage of the form being viewed */
      top;         /* Location where the view begins */
   int
      cnt,         /* Number of arguments */
      temp;
   Position
      x, y,
      new_y;       /* New Y coordinate for the form widget */
   Widget
      vertw,
      listw;
   
   vertw = XtNameToWidget(Toplevel, "*prodview*vertical");
   listw = XtNameToWidget(Toplevel, "*prodlist");
   /*
    * Get values of the vertical scrollbar
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNlength,     &length); cnt++;
   XtSetArg(args[cnt], XtNshown,      &shown);  cnt++;
   XtSetArg(args[cnt], XtNtopOfThumb, &top);    cnt++;
   XtGetValues(vertw, args, cnt);
#ifdef DEBUG
   fprintf(stderr, "Vertical scrollbar:\n");
   fprintf(stderr, "\tlength = [%d], shown = [%7.4f], top = [%7.4f]\n",
	   length, shown, top);
#endif
   
   /*
    * If the amount shown is less than all of the data, 
    * Recalculate scrollbar and form widget attributes
    */
   if (shown < 1.0)
   {
      /*
       * Get the dimensions of the form and node widgets
       */
      cnt = 0;
      XtSetArg(args[cnt], XtNx, &x); cnt++;
      XtSetArg(args[cnt], XtNy, &y); cnt++;
      XtSetArg(args[cnt], XtNheight, &height); cnt++;
      XtGetValues(listw, args, cnt);

#ifdef DEBUG
      fprintf(stderr, "\tlist: height = [%4d]\n", height);
#endif

      /*
       * If the length is the same as the form height,
       * Reset the scrollbar to show everything
       * Else get new values
       */
      if (length >= height)
      {
	 new_top = (float)    0.0;
	 new_y   = (Position) 0;
	 shown   = (float)    1.0;
      }
      else
      {

	 /*
	  * Calculate the coverage
	  * If the coverage is more than the length,
	  * Calculate the new form widget Y as (length - coverage)
	  * This is usually negative to move the data up of
	  * the viewport.
	  */
	 temp    = (int) (selected - (shown * Pindex) / 2);
	 if (temp < 0)
	 {
	    temp = 0;
	 }
	 new_top = (float) temp / Pindex;
	 new_y   = (Position) 0 - (Position) (new_top * height);
      }
#ifdef DEBUG
      fprintf(stderr, "\tnew_top = [%7.4f], new_y = [%d], shown = [%7.4f]\n",
	      new_top, new_y, shown);
#endif

      /*
       * Set the new values of the scrollbar
       * Move the form widget
       */
      if (y != new_y)  {
	  XawScrollbarSetThumb(vertw, new_top, shown);
	  XtMoveWidget(listw, x, new_y);
      }
   }
   return;
}



/*
   FUNCTION: 		HighlightProd

   Synopsis: 
       Highlight the current production being permuted.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
HighlightProd(position)

int
   position;

{
   int
      i,
      ndx;
   RhsList
      *rl;
   
   ndx = 0;
   for (i = 0; i < position; i++)
   {
      ndx = ndx + Grammar[i].numrhs;
   }
   for (rl = Grammar[position].rhs;
	((rl != NULL) && (rl != Grammar[position].activerhs));
	rl = rl->next)
   {
      ndx++;
   }
#ifdef DEBUG
   fprintf(stderr, "list index = [%d]\n", ndx);
#endif
   AdjustVerticalScrollbar(ndx);
   XawListHighlight(XtNameToWidget(Toplevel, "*prodlist"), ndx);
   return;
}



/*
   FUNCTION: 		MakeSymbol

   Synopsis: 
       Make the widgets for the symbols.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



static void
MakeSymbol(ptr, parent, toggleflag)

SymbolList
   *ptr;
Widget
   parent;
Boolean
   toggleflag;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   String
      labelstr;
   Widget
      horiz;
   XtTranslations 
      toggletrans;
   static String
      toggletransstr = "<EnterWindow>:	     highlight(Always)\n\
			<LeaveWindow>:	     unhighlight()\n\
			<Btn1Down>,<Btn1Up>: set() notify()";

   /*
    * Make the command and toggle widgets for
    * the production symbols and the permutation
    * symbols.  The toggleflag determines if   
    * we are making a toggle or command widget
    */
   horiz = NULL;
   toggletrans = XtParseTranslationTable(toggletransstr);
   for ( ; ptr != NULL; ptr = ptr->next)
   {
      labelstr = (String) XtMalloc(strlen(ptr->symbolstr) +
				   strlen(ptr->occurstr) + 1);
      sprintf(labelstr, "%s%s", ptr->symbolstr, ptr->occurstr);
      
      cnt = 0;
      XtSetArg(args[cnt], XtNfromVert,  NULL);       cnt++;
      XtSetArg(args[cnt], XtNfromHoriz, horiz);      cnt++;
      XtSetArg(args[cnt], XtNtop,       XtChainTop); cnt++;
      XtSetArg(args[cnt], XtNbottom,    XtChainTop); cnt++;
      XtSetArg(args[cnt], XtNlabel,     labelstr);   cnt++;
      if (toggleflag == True)
      {
	 XtSetArg(args[cnt], XtNradioGroup, horiz); cnt++;
	 XtSetArg(args[cnt], XtNradioData,  ptr);   cnt++;
	 ptr->symbutton = XtCreateWidget(ptr->symbolstr, toggleWidgetClass,
					 parent, args, cnt);
	 XtOverrideTranslations(ptr->symbutton, toggletrans);
      }
      else
      {
	 ptr->symbutton = XtCreateWidget(ptr->symbolstr, commandWidgetClass,
					 parent, args, cnt);
	 /* add event handler for deleting an entry */
	 XtAddCallback(ptr->symbutton, XtNcallback, DeletePermSymbol, ptr);
      }
      horiz = ptr->symbutton;
      XtFree(labelstr);
   }
   return;
}



/*
   FUNCTION: 		CreateSymbolButtons

   Synopsis: 
       Create all of the widgets for the symbols.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateSymbolButtons()

{
   int
      i;
   RhsList
      *rl;
   PermutationList
      *pl;
   Widget
      nodeframe,
      pnodeframe;
   
   nodeframe  = XtNameToWidget(Toplevel, "*nodeframe");
   pnodeframe = XtNameToWidget(Toplevel, "*pnodeframe");
   for (i = 0; i < Gmax; i++)
   {
      /*
       * for each production 
       * Make production symbols
       * then the permutation symbols
       */
      for (rl = Grammar[i].rhs; rl != NULL; rl = rl->next)
      {
	 MakeSymbol(rl->symbol, nodeframe, True);
	 for (pl = rl->perminstance; pl != NULL; pl = pl->next)
	 {
	    MakeSymbol(pl->permsymbol, pnodeframe, False);
	 }
      } 	/* while (rl...) */
   }		/* for (i) */
   return;
}



/*
   FUNCTION: 		MakePermButton

   Synopsis: 
       Make the symbol button for a permutation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakePermButton(sl, prevsl, nextsl)

SymbolList
   *sl,
   *prevsl,
   *nextsl;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   String
      labelstr;
   Widget
      parent;
   
   parent = XtNameToWidget(Toplevel, "*pnodeframe");
   XawFormDoLayout(parent, False);
   
   labelstr = (String) XtMalloc(strlen(sl->symbolstr) +
				strlen(sl->occurstr) + 1);
   sprintf(labelstr, "%s%s", sl->symbolstr, sl->occurstr);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,  NULL);          cnt++;
   if (prevsl != NULL)
   {
      XtSetArg(args[cnt], XtNfromHoriz, prevsl->symbutton); cnt++;
   }
   XtSetArg(args[cnt], XtNtop,    XtChainTop); cnt++;
   XtSetArg(args[cnt], XtNbottom, XtChainTop); cnt++;
   XtSetArg(args[cnt], XtNlabel,  labelstr);   cnt++;
   sl->symbutton = XtCreateManagedWidget(sl->symbolstr, commandWidgetClass,
					 parent, args, cnt);
   XtAddCallback(sl->symbutton, XtNcallback, DeletePermSymbol, sl);

   if (nextsl != NULL)
   {
      XtUnmanageChild(nextsl->symbutton);
      cnt = 0;
      XtSetArg(args[cnt], XtNfromHoriz, sl->symbutton); cnt++;
      XtSetValues(nextsl->symbutton, args, cnt);
      XtManageChild(nextsl->symbutton);
   }
   XawFormDoLayout(parent, True);
   return;
}



/*
   FUNCTION: 		FindLeftSibling

   Synopsis: 
       Find the ptr for from horzontal resource.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

SymbolList *
FindLeftSibling(pl, x)

PermutationList
   *pl;
Position
   x;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Position
      slx;
   SymbolList
      *sl,
      *tsl;
   
   tsl = NULL;
   for (sl = pl->permsymbol; sl != NULL; sl = sl->next)
   {
      cnt = 0;
      XtSetArg(args[cnt], XtNx, &slx); cnt++;
      XtGetValues(sl->symbutton, args, cnt);

      if (slx >= x)
      {
	 return(tsl);
      }
      tsl = sl;
   }
   return(tsl);
}



/*
   FUNCTION: 		UpdateInterface

   Synopsis: 
       Update everything in the display based on some change.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
UpdateInterface(oldindex, newindex)

int 
   oldindex, 
   newindex;

{
   Arg 
      args[MAXARGS];
   int 
      cnt;
   PermutationList 
      *pl;
   RhsList 
      *rl;
   Widget 
      focus,
      nodeframe,
      pnodeframe;

   /*
    * Update the focus (lhs production label)
    */
   focus = XtNameToWidget( Toplevel, "*focus" );
   nodeframe = XtNameToWidget( Toplevel, "*nodeframe" );
   pnodeframe = XtNameToWidget( Toplevel, "*pnodeframe" );

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, Grammar[newindex].lhs); cnt++;
   XtSetValues(focus, args, cnt);

   XtUnmanageChild(nodeframe);
   XtUnmanageChild(pnodeframe);

   UnmanageProd(oldindex);

   if (Grammar[newindex].rhs == NULL)
   {
      XicaStatus(Statw, "Strangeness in production [%d]\n", newindex);
   }      
   if (Grammar[newindex].activerhs == NULL)
   {
      Grammar[newindex].activerhs = Grammar[newindex].rhs;
   }
   rl = Grammar[newindex].activerhs;
   if (rl == NULL)
   {
      XicaStatus(Statw, "Strangeness in production [%d]\n", newindex);
   }

   /* we have an active rhs */
   SymbolManage(rl->symbol, True);
   if (rl->activeperm == NULL)
   {
      rl->activeperm = rl->perminstance;
   }
   pl = rl->activeperm;
   if (pl != NULL)
   {
      /* we have an active perm, skip if none active */
      SymbolManage(pl->permsymbol, True);
   }
   SensitizeAltToggles();
   SensitizeAltButtons(rl);
   SensitizePermButtons(pl);
   SensitizeProdButtons(newindex);
   HighlightProd(newindex);

   UpdatePermlist();

   XtManageChild(nodeframe);
   XtManageChild(pnodeframe);

   return;
}

