/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: generate.c,v $
   Purpose:     Generate the translator code.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/03/16 13:02:11 $

   HISTORY
   $Log:	generate.c,v $
 * Revision 1.5  92/03/16  13:02:11  ware
 * Conditionalize the title
 * 
 * Revision 1.4  1992/02/06  06:15:55  cso
 * Aesthetics
 *
 * Revision 1.3  1992/01/28  17:15:24  cso
 * Fixed a generation problem with multiply-used symbols in a production
 * and the occurrence being generated.
 *
 * Revision 1.2  1992/01/14  20:54:38  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/10  02:23:48  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"

#include "output.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "initialize.h"
#include "symtab.h"
#include "utils.h"

/* for the ICA string library */

extern char
   *makeendtag(),
   *makestarttag();

/*  GLOBALS  */



/*
   FUNCTION: 		PrintTokens

   Synopsis: 
       Output the token list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintTokens(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   String
      estr,
      sstr;
   SymbolTable
      *tptr;
   
   fprintf(fptr, "%%token <data>\n");
   fprintf(fptr, " ICA_Char_DATA\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      if (tptr->symbol[1] == '/')
      {
	 estr = (char *) makeendtag(tptr->symbol);
	 fprintf(fptr, " %s", estr);
	 XtFree(estr);
      }
      else
      {
	 sstr = (char *) makestarttag(tptr->symbol);
	 fprintf(fptr, " %s", sstr);
	 XtFree(sstr);
      }
      fprintf(fptr, "\n");
   }
   return;
}




/*
   FUNCTION: 		PrintType

   Synopsis: 
       Output the nonterminal list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintType(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   SymbolTable
      *tptr;
      
   fprintf(fptr, "\n%%type <trans>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "  %s", tptr->symbol);
      if (tptr->optional > 0)
      {
	 fprintf(fptr, " %sOPTL", tptr->symbol);
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, " %sSTAR", tptr->symbol);
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, " %sPLUS", tptr->symbol);
      }
      fprintf(fptr, "\n");
   }
   return;
}




/*
   FUNCTION: 		PrintOccur

   Synopsis: 
       Output the occurrence suffix.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintOccur(fptr, pos, sptr)

FILE
   *fptr;
int
   pos;
SymbolList
   *sptr;

{
   for ( ; sptr != NULL; sptr = sptr->next)
   {
      if (pos == sptr->position)
      {
	 switch (sptr->occurstr[0])
	 {
	 default:
	    fprintf(fptr, " ");
	    break;
	    
	 case '?':
	    fprintf(fptr, "OPTL ");
	    break;
	    
	 case '*':
	    fprintf(fptr, "STAR ");
	    break;
	    
	 case '+':
	    fprintf(fptr, "PLUS ");
	    break;
	 }
      }
   }
   return;
}




/*
   FUNCTION: 		PrintProductions

   Synopsis: 
       Output the productions.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintProductions(fptr)

FILE
   *fptr;

{
   String
      estr,
      mapstr,
      sstr;
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr,
      *sptr1;
   PermutationList
      *pptr;
   
   for (i = 0; i < Gmax; i++)
   {
      fprintf(fptr, "%s\t: ", Grammar[i].lhs);
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (pptr = rptr->perminstance; pptr != NULL; pptr = pptr->next)
	 {
	    if (pptr->permsymbol != NULL)
	    {
	       sptr = pptr->permsymbol;
	    }
	    else
	    {
	       sptr = rptr->symbol;
	    }
	    for ( ; sptr != NULL; sptr = sptr->next)
	    {
	       if ((sptr->symbolstr[0] == '<') && (sptr->symbolstr[1] == '/'))
	       {
		  estr = (char *) makeendtag(sptr->symbolstr);
		  fprintf(fptr, "%s ", estr);
		  XtFree(estr);
	       }
	       else if (sptr->symbolstr[0] == '<')
	       {
		  sstr = (char *) makestarttag(sptr->symbolstr);
		  fprintf(fptr, "%s ", sstr);
		  XtFree(sstr);
	       }
	       else if (sptr->symbolstr[1] != '?')
	       {
		  fprintf(fptr, "%s", sptr->symbolstr);
		  PrintOccur(fptr, sptr->position, rptr->symbol);
	       }
	    } /* for sptr */
	 
	    fprintf(fptr, "\n\t\t{\n\t\t  $$ = structconcat(");
	    if (pptr->permsymbol != NULL)
	    {
	       sptr1 = pptr->permsymbol;
	    }
	    else
	    {
	       sptr1 = rptr->symbol;
	    }
	    for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	    {
	       mapstr = GetMapping(sptr->position, sptr->symbolstr, sptr1);
	       fprintf(fptr, "%s, ", mapstr);
	       XtFree(mapstr);
	    }
	    fprintf(fptr, "NULL);\n");
	    if (i == 0)
	    {
	       fprintf(fptr, "\t\t  structprint($$);\n");
	    }
	    fprintf(fptr, "\t\t}\n");
	    if (pptr->next != NULL)
	    {
	       fprintf(fptr, "\t\t| ");
	    }
	 }  /* for pptr */
	 if (rptr->next == NULL)
	 {
	    fprintf(fptr, "\t\t;\n\n");
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
      } /* for rptr */
   }
   return;
}




/*
   FUNCTION: 		PrintExtras

   Synopsis: 
       Output the extra productions for yacc (*, ?, +).

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintExtras(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   String
      str;
   SymbolTable
      *tptr;
      
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      str = tptr->symbol;
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "%sOPTL\t: %s\n", str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "%sSTAR\t: %sSTAR %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "%sPLUS\t: %sPLUS %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| %s\n", str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
   }
   fprintf(fptr, "ICA_Char_DATASTAR\t: ICA_Char_DATASTAR ICA_Char_DATA\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat($1, structnew($2), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t| /* empty content */\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t;\n\n");
   return;
}




/*
   FUNCTION: 		GenerateYacc

   Synopsis: 
       Generate the yacc spec.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

SymbolTable *
GenerateYacc(fptr)

FILE
   *fptr;

{
   SymbolTable
      *stable,
      *ttable;
   
   Output(fptr, yacchead);
   ttable = NULL;
   ttable = MakeTokenTable(ttable);
   PrintTokens(fptr, ttable);
   stable = NULL;
   stable = MakeSymbolTable(stable);
   PrintType(fptr, stable);
   fprintf(fptr, "\n%%start %s\n\n%%%%\n", Grammar[0].lhs);
   PrintProductions(fptr);
   PrintExtras(fptr, stable);
   FreeSymbolTable(stable);
   return(ttable);
}




/*
   FUNCTION: 		PrintLexSpec

   Synopsis: 
       Output the regexps.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PrintLexSpec(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   String
      estr,
      sstr;
   SymbolTable
      *tptr;
   
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "\"%s\"\t{\n", tptr->symbol);
      fprintf(fptr, "\t BEGIN 0;\n");
      fprintf(fptr, "#ifndef NOINTERFACE\n");
      fprintf(fptr, "\t UpdateHighlight(yyleng);\n");
      fprintf(fptr, "#endif\n");
      fprintf(fptr, "\t return(");
      if (tptr->symbol[1] == '/')
      {
	 estr = (char *) makeendtag(tptr->symbol);
	 fprintf(fptr, "%s", estr);
	 XtFree(estr);
      }
      else
      {
	 sstr = (char *) makestarttag(tptr->symbol);
	 fprintf(fptr, "%s", sstr);
	 XtFree(sstr);
      }	 
      fprintf(fptr, ");\n\t}\n\n");
   }
   return;
}





/*
   FUNCTION: 		GenerateLex

   Synopsis: 
       Generate the lex spec.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
GenerateLex(fptr, table, pitype)

FILE
   *fptr;
SymbolTable
   *table;
UpInterface
   pitype;

{
   Output(fptr, lexhead);
   if (pitype == With)
   {
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
   }
   Output(fptr, lexmid);
   PrintLexSpec(fptr, table);
   Output(fptr, lextail);
   FreeSymbolTable(table);
   return;
}
