/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: initialize.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/16 13:02:18 $

   HISTORY
   $Log:	initialize.c,v $
 * Revision 1.4  92/03/16  13:02:18  ware
 * Conditionalize the title
 * 
 * Revision 1.3  1992/02/06  06:07:47  cso
 * Fixed file dependencies and function decls
 *
 * Revision 1.2  1992/01/14  20:54:44  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/08  19:54:40  cso
 * Initial revision
 *

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "interface.h"

/*  GLOBALS  */

Production
   Grammar[MAXPRODS];
String
   Permlist[MAXRHS],
   Prodlist[MAXRHS];
int
   Gindex,
   Gmax,
   Pindex;




/*
   FUNCTION: 		FreeSymbol

   Synopsis: 
       Free up the symbol pointers.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
FreeSymbol(ptr)

SymbolList
   *ptr;

{
   SymbolList
      *temp;

   /*
    * kill all the symbol list nodes and strings 
    */
   while (ptr != NULL)
   {
      XtFree(ptr->symbolstr);
      XtFree(ptr->occurstr);
      if (ptr->symbutton != NULL)
      {
	 XtDestroyWidget(ptr->symbutton);
	 ptr->symbutton = NULL;
      }
      temp = ptr;
      ptr  = ptr->next;
      XtFree(temp);
   }
   return;
}



/*
   FUNCTION: 		InitializeGram

   Synopsis: 
       Initialize the grammar data structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void 
InitializeGram()

{
   int
      i;
   PermutationList
      *pl,
      *tpl;
   RhsList
      *rl,
      *trl;
   
   /*
    * NULL out the production list
    * Reset the list and scrollbars
    * Initialize all the data structs for the grammar.
    * Reset the global Gmax & Pindex to 0.
    */
   for (i = 0; i < MAXRHS; i++)
   {
      if (Prodlist[i] != NULL)
      {
	 XtFree(Prodlist[i]);
	 Prodlist[i] = NULL;
      }
   }

   XawListChange(XtNameToWidget(Toplevel, "*prodlist"), Prodlist, -1, -1,
		 True);
   XawScrollbarSetThumb(XtNameToWidget(Toplevel, "*prodview*vertical"),
			(float) 0.0, (float) 1.0);
   
   for (i = 0; i < Gmax; i++)
   {
      /*
       * for each production -- 
       * free the string, trace and free the RHS 
       */
      (void) XtFree(Grammar[i].lhs);
      
      for (rl = Grammar[i].rhs; rl != NULL; )
      {
	 /*
	  * track and free the SymbolList and PermList 
	  * free the PermList 
	  */
	 FreeSymbol(rl->symbol);
	 
	 for (pl = rl->perminstance; pl != NULL; )
	 {
	    /*
	     * kill all symbols lists in a perm instance 
	     * free the node
	     */
	    FreeSymbol(pl->permsymbol);

	    tpl = pl;
	    pl  = pl->next;
	    XtFree(tpl);
	 } /***** FOR (pl...) */

	 /*
	  * and now free up the RhsList node 
	  */
	 trl = rl;
	 rl  = rl->next;
	 XtFree(trl);
      } /***** FOR (rl...) */

      Grammar[i].rhs = NULL;
   } /***** FOR (i) */

   Gmax   = 0;
   Pindex = 0;
   SensitizeAltButtons( NULL);
   SensitizePermButtons(NULL);
   SensitizeProdButtons(-1);

   return;
}

