/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: interface.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/06/01 20:02:42 $

   HISTORY
   $Log:	interface.c,v $
 * Revision 1.5  92/06/01  20:02:42  boyd
 * Didn't quite get Pete's SymbolPlace change merged completely - it's better now.
 * 
 * 
 * Revision 1.4  92/05/29  16:09:04  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.  The interactive translators have not (yet) been
 * converted in this manner.
 * 
 * 
 * Revision 1.3  92/05/20  11:47:33  ware
 * 
 * For some reason, certain window manager settings prevent the event
 * handler from ever getting called.  I don't quite understand why.  What
 * I do know is that making it a translation fixes the problem.  That is
 * about all that happens.  I change SymbolPlace from being an event
 * handler to being an action proc.
 * 
 * Revision 1.2  92/03/16  13:02:24  ware
 * Conditionalize the title
 * 
 * Revision 1.1  1992/01/08  19:54:52  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "menubar.h"
#include "permute.h"
#include "prodlist.h"
#include "callbacks.h"
#include "status.h"

/*  EXTERNS  */

/*  GLOBALS  */

static String
   Fallbacks[] = {
   /************************/
   /***   Accelerators   ***/
   /************************/
   "*prevprod*accelerators:	<Key>Home:	set() notify() reset()",
   "*nextprod*accelerators:	<Key>End:	set() notify() reset()",
   "*prevalt*accelerators:	<Key>R9:	set() notify() reset()",
   "*nextalt*accelerators:	<Key>R15:	set() notify() reset()",
   "*prevperm*accelerators:	<Key>Up:	set() notify() reset()",
   "*nextperm*accelerators:	<Key>Down:	set() notify() reset()",
   "Spec2Gen.title:		Map Specific to General Structure",
   "Spec2Gen.iconName:		Map Specific to General Structure",
   "*Permutations.title:      Map Specific to General Structure: Permutations",
   "*Permutations.iconName:   Map Specific to General Structure: Permutations",
   /************************/
   /***     Sizing       ***/
   /************************/
   "*pnodeview.prefWidth:   550",
   "*pnodeview.minWidth:    100",
   "*nodeview.prefWidth:    550",
   "*nodeview.minWidth:     100",
   "*prodview.prefWidth:    550",
   "*prodview.minWidth:     100",
   "*prodview.prefHeight:   300",
   "*prodview.minHeight:    50",
   "*permview.prefWidth:    500",
   "*permview.minWidth:     100",
   "*permview.prefHeight:   200",
   "*permview.minHeight:    50",
   "*Statw.prefHeight:      96",
   "*Statw.minHeight:       20",
   "*Statw.stretch:         False",
   "*statframe.stretch:     False",
   (char *) NULL
   };
static XtActionsRec
   TextActions[] = {
		     {"SymbolPlace",SymbolPlace},
                     {NULL,         NULL}
		   };
Widget
   Toplevel;



/*
   FUNCTION: 		QuickQuit

   Synopsis: 
       Quits during development on key strokes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
QuickQuit(w, client_data, event)

Widget
   w;
caddr_t
   client_data;
XEvent
   *event;

{
   int
      i;
   char
      text[10];
   KeySym
      keys;
   
   /*
    * quit the program if the user hits a 'q'
    */
   i = XLookupString(event, text, 10, &keys, 0);
   if ((i == 1) && ((text[0] == 'Q') || (text[0] == 'q')))
   {
      exit(0);
   }
   return;
}



/*
   FUNCTION: 		CreateInterface

   Synopsis: 
       Open up the display connection and call the appropriate
       creation routines.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
       3/3/92	pete	 ifdef'd hardcoded title so we could easily have
       			 consistent titles.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	XtAppContext

   Range:		NULL or an application context
*/

XtAppContext
CreateInterface(argc, argv)

int
   argc;
char
   **argv;

{
   Widget
      frame,
      menubar,
      listframe,
      listlbls, listviews,
      mapping,
      prods;
   XtAppContext
      appcon;
   
   Toplevel = XtVaAppInitialize( 
       &appcon, "Spec2Gen", NULL, 0, &argc, argv, Fallbacks,
       XtNinput,            True,
#ifdef HARDCODEDTITLE
       XtNtitle,            TITLE,
       XtNiconName,         TITLE,
#endif
       XtNallowShellResize, True,
       NULL );

#ifdef SYNC
   XSynchronize(XtDisplay(Toplevel), True);
#endif

   XtAppAddActions(appcon, TextActions, XtNumber(TextActions));

   frame = XtVaCreateManagedWidget( "Frame",
       groupWidgetClass, Toplevel,
       XtNspacing, 8,
       NULL );

   CreateMenubar(frame);

   listframe = XtVaCreateManagedWidget( "listframe",
       tableWidgetClass, frame,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       XtNhorzMargin,  0,
       XtNvertMargin,  0,
       XtNhorzSpacing, 8,
       XtNvertSpacing, 8,
       XtNstretch,     True,
       NULL );
   listlbls = XtVaCreateWidget( "listlbls",
       tableColObjectClass, listframe,
       NULL );
   listviews = XtVaCreateWidget( "listviews",
       tableColObjectClass, listframe,
       XtNstretch, True,
       NULL );

   mapping = CreateMapDisplay( listframe, listlbls, listviews );
   prods   = CreateProdList( listframe, listlbls, listviews );

   CreateStatus(frame);

   XtRealizeWidget(Toplevel);

   SetupFileDialog(Toplevel);

#ifdef DEBUG
   XtAddEventHandler(frame, KeyPressMask, False, QuickQuit, NULL);
#endif

   XtInstallAllAccelerators(frame, frame);

   return(appcon);
}
