/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menubar.c,v $
   Purpose:     The menubar choices plus callbacks.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/05/29 16:09:09 $

   HISTORY
   $Log:	menubar.c,v $
 * Revision 1.5  92/05/29  16:09:09  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.  The interactive translators have not (yet) been
 * converted in this manner.
 * 
 * 
 * Revision 1.4  92/03/16  13:02:48  ware
 * Conditionalize the title
 * 
 * Revision 1.3  1992/02/21  06:06:35  cso
 * Made the use of the new Make callback compile-time selectable
 *
 * Revision 1.2  1992/01/28  17:13:01  cso
 * Put border around menubar
 *
 * Revision 1.1  1992/01/08  19:55:30  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

#include "interface.h"
#include "menucalls.h"
#include "panel.h"
#include "permshell.h"



/*
   FUNCTION: 		CreateFileMenu

   Synopsis: 
       Create the file options.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


static void
CreateFileMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS]; /* List of arguments */
   int
      cnt;      /* Number of arguments */
   Widget
      loadsel,  /* Load  option widget */
      newsel,   /* New   option widget */
      quitsel,  /* Quit  option widget */
      resetsel, /* Reset option widget */
      savesel;  /* Save  option widget */
   
   /*
    * Offer RESET first, a line to separate, LOAD and SAVE,
    * another line separator, and lastly QUIT
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Reset    "); cnt++;
   resetsel = XtCreateManagedWidget("resetsel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(resetsel, XtNcallback, Reset, NULL);
   
    cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "New     "); cnt++;
   newsel = XtCreateManagedWidget("newsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(newsel, XtNcallback, FileLoad, True);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Edit     "); cnt++;
   loadsel = XtCreateManagedWidget("loadsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(loadsel, XtNcallback, FileLoad, False);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Save     "); cnt++;
   savesel = XtCreateManagedWidget("savesel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(savesel, XtNcallback, FileSave, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Quit     "); cnt++;
   quitsel = XtCreateManagedWidget("quitsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(quitsel, XtNcallback, Quit, NULL);

   return;
}



/*
   FUNCTION: 		CreateOpsMenu

   Synopsis: 
       Create the operations menu.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



static void
CreateOpsMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      ops1sel,
      ops2sel,
      ops3sel,
      shell;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Show   Permutations"); cnt++;
   ops3sel = XtCreateManagedWidget("ops3sel", smeBSBObjectClass,
				   parent, args, cnt);
   shell = CreatePermShell(Toplevel, ops3sel);
   XtAddCallback(ops3sel, XtNcallback, XtCallbackNone, shell);
   XtAddCallback(ops3sel, XtNcallback, PermUpdate,     NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Add    Permutation "); cnt++;
   ops1sel = XtCreateManagedWidget("ops1sel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(ops1sel, XtNcallback, PermOp, Add);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Delete Permutation "); cnt++;
   ops2sel = XtCreateManagedWidget("ops2sel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(ops2sel, XtNcallback, PermOp, Delete);

   return;
}



/*
   FUNCTION: 		CreateCodeMenu

   Synopsis: 
       The menu dealing with making and running.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


static void
CreateCodeMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      makesel,
      runsel,
      shell;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Make"); cnt++;
   makesel = XtCreateManagedWidget("makesel", smeBSBObjectClass,
				 parent, args, cnt);
#ifdef PANEL
   shell = CreateMakePanel(makesel);
   XtAddCallback(makesel, XtNcallback, XtCallbackNone, shell);
#else
   XtAddCallback(makesel, XtNcallback, Make, NULL);
#endif

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Run"); cnt++;
   runsel = XtCreateManagedWidget("runsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(runsel, XtNcallback, Run, NULL);
   
   return;
}



/*
   FUNCTION: 		CreateHelpMenu

   Synopsis: 
       

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CreateHelpMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      help1sel;
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "Creating  "); cnt++;
   help1sel = XtCreateManagedWidget("help1sel", smeBSBObjectClass,
				    parent, args, cnt);
/*
   XtAddCallback(help1sel, XtNcallback, Help, "create.hlp");
*/
   return;
}



/*
  FUNCTION: 		CreateMenubar
  
  Synopsis: 
  Create the menubar for the interface.
  
  Initial purpose:
  Same as above.
  
  Present purpose:
  Same as above.
  
  Creator:	Conleth O'Connell
  Reviewer:	
  
  Modifications:
  DATE     WHO      REASON
  
  Globals used:
  none
  
  Globals Modified:
  none
  
  Return value:	void
  
  Range:		none
  */


Widget CreateMenubar(parent)
Widget
   parent;
{
   Widget
      menubar,
      filebut,
      filemenu,
      opsmenu,
      opsbut,
      codemenu,
      codebut,
      helpbut,
      helpmenu;
   
   menubar = XtVaCreateManagedWidget( "menubar",
       groupWidgetClass, parent,
       XtNorientation, XtorientHorizontal,
       XtNspacing, 1,
       XtNmargin,  7,
       NULL );
   
   /*********** File menu **************/

   filebut = XtVaCreateManagedWidget( "File",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   File   ",
       XtNmenuName, "filemenu",
       XtNmargin,   0,
       NULL );
   
   filemenu = XtVaCreatePopupShell( "filemenu",
       simpleMenuWidgetClass, parent, 
       NULL );

   CreateFileMenu(filemenu);
   
   /*********** Perm Ops **************/
   
   opsbut = XtVaCreateManagedWidget( "Operations",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "  Operations  ",
       XtNmenuName, "opsmenu",
       XtNmargin,   0,
       NULL );
   
   opsmenu = XtVaCreatePopupShell( "opsmenu",
       simpleMenuWidgetClass, parent, 
       NULL );

   CreateOpsMenu(opsmenu);
   
   /*********** Code Generation **************/
   
   codebut = XtVaCreateManagedWidget( "Code",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   Code   ",
       XtNmenuName, "codemenu",
       XtNmargin,   0,
       NULL );
   
   codemenu = XtVaCreatePopupShell( "codemenu",
       simpleMenuWidgetClass, parent, 
       NULL );

   CreateCodeMenu(codemenu);
   
#ifndef NOHELP
   /*********** Help **************/

   helpbut = XtVaCreateManagedWidget( "Help",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   Help   ",
       XtNmenuName, "helpmenu",
       XtNmargin,   0,
       NULL );
   
   helpmenu = XtVaCreatePopupShell( "helpmenu",
       simpleMenuWidgetClass, parent, 
       NULL );

   CreateHelpMenu(helpmenu);
#endif /* NOHELP */;

   return(menubar);
}

