/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menucalls.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.10 $
   Date:        $Date: 92/05/20 11:49:55 $

   HISTORY
   $Log:	menucalls.c,v $
 * Revision 1.10  92/05/20  11:49:55  ware
 * Insure translator is not interactive for reduce/reduce errors.
 * 
 * Revision 1.9  92/03/16  13:02:55  ware
 * Conditionalize the title
 * 
 * Revision 1.8  1992/03/10  18:24:30  cso
 * Freed up filenames
 *
 * Revision 1.7  1992/02/21  23:11:09  cso
 * Trivial aesthetics: messages, formatting...
 *
 * Revision 1.6  1992/02/21  06:04:00  cso
 *  - Added a cancel option to the save changes dialog
 *  - Added a Make callback to replace the make panel for now - to be
 * more consistent, and to avoid other problems for now
 *
 * Revision 1.5  1992/02/11  22:49:16  cso
 * Fixed the call to GetFilename/OpenFile
 *
 * Revision 1.4  1992/02/06  06:07:41  cso
 * Fixed file dependencies and function decls
 *
 * Revision 1.3  1992/01/14  20:55:10  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:08  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:56:05  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "fileio.h"
#include "generate.h"
#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "permshell.h"
#include "prodlist.h"
#include "save.h"
#include "status.h"
#include "utils.h"

#ifdef FLEX
extern void
   yyrestart();
#endif

/*  GLOBALS  */




/*
   FUNCTION: 		SaveChanges

   Synopsis: 
       Check if the  current session has changed a grammar description.
       Offer the user to save or forget changes AND if unsuccessful
       in saving let them know and let them leave the function
       invoking this one without losing the changes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       Dirtyflag  

   Globals Modified:
       none

   Return value:	Boolean

   Range:		True or False
*/

static Boolean
SaveChanges()

{
   Boolean
      continue_oper,
      response;
   FILE
      *fptr;

   /*
    * If the current session has changed, give the user a chance to
    * save the file or at least realize that changes have occurred.
    */
   continue_oper = True;
   if (Dirtyflag == True)
   {
      /*
       * Ask the user if they wish to save the changes, if Continue, 
       * then get a file name and save the file
       */
      continue_oper = YesNoCancelMessage(Toplevel, "Save changes?",
					 True, &response);
      
      if ((continue_oper == True) && (response == True))
      {
	 /*
	  * Get the file to save the current session
	  * If successful in getting a file, save out the current session
	  * Else alert the user that changes will be lost
	  */
	 fptr = OpenFile("Save a Specific to General \
Translation Specification [.s2g]",
			 "Save:",
			 "Save",
			 "*.s2g",
			 "s2g",
			 False);
	 
	 if (fptr != NULL)
	 {
	    SaveFile(fptr);
	    fclose(fptr);
	    Dirtyflag = False;
	 }
	 else
	 {
	    /*
	     * If the user does not wish to lose changes,
	     * then return False, preventing whatever action would lose
	     * the changes from occurring.
	     */
	    return(YesNoMessage(Toplevel, "Lose changes?", False));
	 }
      }
   }
   return(continue_oper);
}




/*
   FUNCTION: 		Reset

   Synopsis: 
       Reset the interface.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
Reset(w, client_data, call_data)

Widget
   w;
caddr_t
   client_data,
   call_data;

{
   Arg
      args[MAXARGS];
   int 
      cnt;
   
   if (SaveChanges() == False)
   {
      return;
   }
   
   if (YesNoMessage(Toplevel, "Reset interface?", True) == True)
   {
      /*
       * Reset data structures.
       * Blank the focus widget.
       */
      InitializeGram();
      cnt = 0;
      XtSetArg(args[cnt], XtNlabel, ""); cnt++;
      XtSetValues(XtNameToWidget(Toplevel, "*focus"), args, cnt);
      Dirtyflag = False;
   }
   return;
}



/*
   FUNCTION: 		FileLoad

   Synopsis: 
       Load in a file to be edited.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void 
FileLoad(w, newflag, call_data)

Widget
   w;
Boolean
   newflag;
XtPointer
   call_data;   /* Unused */

{
   extern FILE
      *yyin;
   extern int
      linecount;
   
   if (SaveChanges() == False)
   {
      return;
   }
   if (newflag == True)
   {
      yyin = OpenFile("Load an ICA Specification [.ica]",
		      "Load:",
		      "Load",
		      "*.ica",
		      "ica",
		      True);
   }
   else
   {
      yyin = OpenFile("Load a Specific to General Translation \
Specification [.s2g]",
		      "Load:",
		      "Load",
		      "*.s2g",
		      "s2g",
		      True);
   }
   
   if (yyin != NULL)
   {
      InitializeGram();
      linecount = 1;
      if (yyparse() == 0) 
      {
	 CreateSymbolButtons();
      }
      else
      {
	 XicaStatus(Statw, "Input file was not in correct format\n");
      }
      fclose(yyin);
#ifdef FLEX
      yyrestart(yyin);
#endif
      if (newflag == True)
      {
	 Dirtyflag = True;
      }

      if (Gmax > 0) 
      {
	 Gindex = 0;
#ifdef DEBUG
	 fprintf(stderr, "Root symbol = [%s]\n", Grammar[0].lhs);
#endif
	 GenerateProdList();
	 UpdateInterface(Gindex, Gindex);
      }
   }

   return;
}




/*
   FUNCTION: 		FileSave

   Synopsis: 
       Save out the specification file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FileSave(w, client_data, call_data)

Widget 
   w;			/* unused */
caddr_t 
   client_data,		/* unused */
   call_data;		/* unused */

{
   FILE
      *fptr;

   if (Dirtyflag == False)
   {
      if (YesNoMessage(Toplevel,
		       "No changes made, save anyway?", False) == False)
      {
	 return;
      }
   }
   
   fptr = OpenFile("Save a Specific to General \
Translation Specification [.s2g]",
		   "Save:",
		   "Save",
		   "*.s2g",
		   "s2g",
		   False);
	 
   if (fptr == NULL)
   {
      return;
   }
   SaveFile(fptr);
   fclose(fptr);
   Dirtyflag = False;
   return;
}



/*
   FUNCTION: 		Quit

   Synopsis: 
       Quit.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
Quit(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   if (SaveChanges() == False)
   {
      return;
   }
   XtDestroyWidget(Toplevel);
   exit(0);
}



/*
   FUNCTION: 		PermOp

   Synopsis: 
       Add or delete a permutation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
PermOp(w, op, call_data)

Widget
   w;           /* Unused */
Operation
   op;		/* Add or Delete */
caddr_t
   call_data;   /* Unused */

{
   PermutationList
      *pl;
   RhsList 
      *rl;

   rl = Grammar[Gindex].activerhs;
   if (rl != NULL)
   {
      pl = rl->activeperm;
   }
   else
   {
      XicaStatus(Statw, "Operation failed\n");
      return;
   }
   
   switch (op)
   {
   default:
      XicaStatus(Statw, "PermOp: called with invalid operation\n");
      break;
      
   case Add:

      if (pl != NULL)
      {
	 
	 for ( ; pl->next != NULL; pl = pl->next)
	 {
	 }
	 
	 pl->next       = XtNew(PermutationList);
	 pl->next->next = NULL;
	 pl->next->prev = pl;
	 pl             = pl->next;
	 pl->permsymbol = NULL;
      }
      else
      {
	 pl               = XtNew(PermutationList);
	 pl->next         = NULL;
	 pl->prev         = NULL;
	 pl->permsymbol   = NULL;
	 rl->perminstance = pl;
	 
      }
      
      rl->activeperm = pl;
      Dirtyflag = True;
      break;
      
   case Delete:
      if (pl->prev == NULL)
      {
	 XicaStatus(Statw, "Cannot delete the first permutation\n");
	 XicaStatus(Statw, "Try editing it\n");
      }
      else if (YesNoMessage(Toplevel,
			    "Delete Current Permutation?", False) == True)
      {
	 SymbolManage(pl->permsymbol, False);
	 if (pl->next != NULL)
	 {
	    pl->next->prev = pl->prev;
	 }
	 pl->prev->next = pl->next;

	 FreeSymbol(pl->permsymbol);
	 rl->activeperm = pl->prev;
	 XtFree(pl);
	 Dirtyflag = True;
      }
   }

   UpdateInterface(Gindex, Gindex);
   return;
}



/*
   FUNCTION: 		PermUpdate

   Synopsis: 
       Call the UpdatePerm routine.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PermUpdate(w, client_data, call_data)

Widget
   w;
XtPointer
   client_data,
   call_data;

{
   UpdatePermlist();
   return;
}



/*
   FUNCTION: 		Make

   Synopsis:  Make the specification files and compile them into a tool.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Make(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *lexptr,
      *yaccptr;
   String
      target;
   SymbolTable
      *tokentab;
   UpInterface
      uitype;
   
   if (Gmax == 0)
   {
      XicaStatus(Statw, "Please load a specification file first.\n");
      return;
   }
   
   if (YesNoMessage(Toplevel,
		    "Make an interactive translator?", False) == False)
   {
      uitype = Without;
   }
   else
   {
      uitype = With;
   }
   
   target = GetFilename("Making a specific to general translator [.sgx]",
			"Make:",
			"Make",
			"*.sgx",
			"sgx",
			False);

   if (*target == '\0')
   {
      return;
   }

   lexptr  = OpenGenerationFile(target, False);
   yaccptr = OpenGenerationFile(target, True);

   tokentab = GenerateYacc(yaccptr);
   fclose(yaccptr);
   GenerateLex(lexptr, tokentab, Without);
   fclose(lexptr);
   
   MakeTranslator(target, uitype, Without);

   XtFree(target);
   
   return;
} /***** Make */



/*
   FUNCTION: 		Run

   Synopsis: 
       Run an up translator.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Run(w, client_data, call_data)

Widget
   w;
XtPointer
   client_data,
   call_data;

{
   String
      execfile,
      infile,
      outfile,
      sysstr;

   /* pick a document to translate */
   infile = GetFilename("Choose an input file [*]",
			"Ok:",
			"Ok",
			"*",
			"*",
			True);

   if (*infile == '\0')
   {
      XtFree(infile);
      return;
   }
   
   /* choose an output file name */
   outfile = GetFilename("Choose an output file [*]\n Cancel to use stdout",
			"Ok:",
			"Ok",
			"*",
			"*",
			False);

   /* pick an executable */
   execfile = GetFilename("Execute a specific to general translator [.sgx]",
			  "Run:",
			  "Run",
			  "*.sgx",
			  "sgx",
			  True);

   if (*execfile == '\0')
   {
      XtFree(infile);
      XtFree(outfile);
      XtFree(execfile);
      return;
   }
   
   if (*outfile != '\0')
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) +
			strlen(outfile) + 32);
      sprintf(sysstr, "%s < %s > %s", execfile, infile, outfile);
      XtFree(outfile);
   } /***** IF the user chose a file for output */
   else
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) + 32);
      sprintf(sysstr, "%s < %s", execfile, infile);
   } /***** ELSE put the output to stdout */

#ifdef DEBUG
   fprintf(stderr, "RUN str = [%s]\n", sysstr);
#endif
   XicaStatus(Statw, "Running translator\n%s\n", sysstr);
   errno = 0;
   if (system(sysstr) < 0)
   {
      perror("System call to RUN translator failed");
      XicaStatus(Statw, "Run failed\n");
   }
   else
   {
      XicaStatus(Statw, "Run finished\n");
   }
   
   
   XtFree(sysstr);
   XtFree(infile);
   XtFree(execfile);
   return;
}

