/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: panel.c,v $
   Purpose:     A Panel to ontain information to make a translator.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/05/29 16:09:14 $

   HISTORY
   $Log:	panel.c,v $
 * Revision 1.4  92/05/29  16:09:14  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.  The interactive translators have not (yet) been
 * converted in this manner.
 * 
 * 
 * Revision 1.3  92/03/16  13:03:06  ware
 * Conditionalize the title
 * 
 * Revision 1.2  1992/01/14  20:55:16  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/08  19:56:25  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "callbacks.h"
#include "interface.h"

/*  GLOBALS  */



/*
   FUNCTION: 		CreateMakePanel

   Synopsis: 
       Create the panel for obtaining info to produce a translator.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget
CreateMakePanel(enable)

Widget
   enable;

{
   Widget
      applyw,
      autotog,
      cancelw,
      contenttog,
      defaultsw,
      elementtog,
      grainlbl,
      leavetog,
      panel,
      panelform,
      paneltbl,
      resrow, granrow, pinrow, labelcol, tglcol1, tglcol2,
      panelbtns,
      procinlbl,
      removetog,
      resolvelbl,
      usertog;
   XtPopdownID
      pdptr;   /* Pointer to the popdown record that will take care of */
	       /* sensitizing the menu button */
   XtTranslations 
      toggletrans;
   static String
      toggletransstr = "<EnterWindow>:	     highlight(Always)\n\
			<LeaveWindow>:	     unhighlight()\n\
			<Btn1Down>,<Btn1Up>: set() notify()";
   
   toggletrans = XtParseTranslationTable(toggletransstr);

   panel = XtVaCreatePopupShell( "MakePanel",
       applicationShellWidgetClass, Toplevel,
       XtNtitle,     "Make Options",
       XtNresizable, True,
       NULL );
   
   panelform = XtVaCreateManagedWidget( "panelform",
       groupWidgetClass, panel,
       XtNspacing, 6,
       NULL );
   paneltbl = XtVaCreateManagedWidget( "paneltbl",
       tableWidgetClass, panelform,
       XtNhorzSpacing,   6,
       XtNvertSpacing,   6,
       XtNhorzPlacement, tpCenter,
       XtNvertPlacement, tpCenter,
       XtNalignment,     gaFill,
       XtNstretch,       True,
       XtNmargin,        6,
       NULL );
   resrow = XtVaCreateManagedWidget( "resrow",
       tableRowObjectClass, paneltbl,
       NULL );
   granrow = XtVaCreateManagedWidget( "granrow",
       tableRowObjectClass, paneltbl,
       NULL );
   pinrow = XtVaCreateManagedWidget( "pinrow",
       tableRowObjectClass, paneltbl,
       NULL );
   labelcol = XtVaCreateManagedWidget( "labelcol",
       tableColObjectClass, paneltbl,
       NULL );
   tglcol1 = XtVaCreateManagedWidget( "tglcol1",
       tableColObjectClass, paneltbl,
       NULL );
   tglcol2 = XtVaCreateManagedWidget( "tglcol2",
       tableColObjectClass, paneltbl,
       NULL );
   
   resolvelbl = XtVaCreateManagedWidget( "resolvelbl",
       labelWidgetClass, paneltbl,
       XtNlabel,         "Ambiguity Resolution:",
       XtNborderWidth,   0,
       XtNtableRow,      resrow,
       XtNtableCol,      labelcol,
       XtNhorzAlignment, taBottomOrRight,
       NULL );

   autotog = XtVaCreateManagedWidget( "autotog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Batch",
       XtNradioGroup,    NULL,
       XtNradioData,     Without,
       XtNstate,         True,
       XtNtableRow,      resrow,
       XtNtableCol,      tglcol1,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(autotog, toggletrans);

   usertog = XtVaCreateManagedWidget( "usertog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Interactive",
       XtNradioGroup,    autotog,
       XtNradioData,     With,
       XtNtableRow,      resrow,
       XtNtableCol,      tglcol2,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(usertog, toggletrans);

   grainlbl = XtVaCreateManagedWidget( "grainlbl",
       labelWidgetClass, paneltbl,
       XtNlabel,         "Ambiguity Granularity:",
       XtNborderWidth,   0,
       XtNtableRow,      granrow,
       XtNtableCol,      labelcol,
       XtNhorzAlignment, taBottomOrRight,
       NULL );

   elementtog = XtVaCreateManagedWidget( "elementtog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Element",
       XtNradioGroup,    NULL,
       XtNradioData,     Without,
       XtNstate,         True,
       XtNtableRow,      granrow,
       XtNtableCol,      tglcol1,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(elementtog, toggletrans);

   contenttog = XtVaCreateManagedWidget( "contenttog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Content",
       XtNradioGroup,    elementtog,
       XtNradioData,     With,
       XtNtableRow,      granrow,
       XtNtableCol,      tglcol2,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(contenttog, toggletrans);

   procinlbl = XtVaCreateManagedWidget( "procinlbl",
       labelWidgetClass, paneltbl,
       XtNlabel,         "Processing Instructions:",
       XtNborderWidth,   0,
       XtNtableRow,      pinrow,
       XtNtableCol,      labelcol,
       XtNhorzAlignment, taBottomOrRight,
       NULL );

   removetog = XtVaCreateManagedWidget( "removetog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Remove",
       XtNradioGroup,    NULL,
       XtNradioData,     Without,
       XtNstate,         True,
       XtNtableRow,      pinrow,
       XtNtableCol,      tglcol1,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(removetog, toggletrans);

   leavetog = XtVaCreateManagedWidget( "leavetog",
       toggleWidgetClass, paneltbl,
       XtNlabel,         "Retain",
       XtNradioGroup,    removetog,
       XtNradioData,     With,
       XtNtableRow,      pinrow,
       XtNtableCol,      tglcol2,
       XtNhorzAlignment, taFill,
       NULL );
   XtOverrideTranslations(leavetog, toggletrans);

   panelbtns = XtVaCreateManagedWidget( "panelbtns",
       groupWidgetClass, panelform,
       XtNorientation, XtorientHorizontal,
       XtNspacing, 6,
       XtNalignment, gaCenter,
       XtNmargin,  0,
       NULL );

   applyw = XtVaCreateManagedWidget( "Apply",
       commandWidgetClass, panelbtns,
       XtNlabel,        "Make",
       XtNborderWidth,  2,
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );
   XtAddCallback(applyw, XtNcallback, MakeApply, panelform);

   defaultsw = XtVaCreateManagedWidget( "Defaults",
       commandWidgetClass, panelbtns,
       XtNlabel,        "Defaults",
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );
   XtAddCallback(defaultsw, XtNcallback, MakeDefaults, panelform);

   cancelw = XtVaCreateManagedWidget( "Cancel",
       commandWidgetClass, panelbtns,
       XtNlabel,        "Cancel",
       XtNalignment,    gaCenter,
       XtNmargin,       0,
       NULL );
   pdptr                = XtNew(XtPopdownIDRec);
   pdptr->shell_widget  = panel;
   pdptr->enable_widget = enable;
   XtAddCallback(cancelw, XtNcallback, XtCallbackPopdown, pdptr);

   return(panel);
}
