/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: permshell.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/29 16:09:19 $

   HISTORY
   $Log:	permshell.c,v $
 * Revision 1.6  92/05/29  16:09:19  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.  The interactive translators have not (yet) been
 * converted in this manner.
 * 
 * 
 * Revision 1.5  92/03/16  13:03:13  ware
 * Conditionalize the title
 * 
 * Revision 1.4  1992/02/06  06:07:30  cso
 * Fixed file dependencies and function decls
 *
 * Revision 1.3  1992/01/14  20:55:22  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:25:13  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:56:39  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "initialize.h"
#include "interface.h"

/*  GLOBALS  */



/*
   FUNCTION: 		AdjustVerticalPermScrollbar

   Synopsis: 
       what the title says.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AdjustVerticalPermScrollbar(selected)

int
   selected;


{
   Arg
      args[10];    /* List of arguments */
   Dimension
      height, /* Height of the form widget */
      length;
   float
      new_top,     /* New top of thumb location */
      shown,       /* Percentage of the form being viewed */
      top;         /* Location where the view begins */
   int
      cnt,         /* Number of arguments */
      temp;
   Position
      x,
      new_y;       /* New Y coordinate for the form widget */
   Widget
      vertw,
      listw;
   

   vertw = XtNameToWidget(Toplevel, "*permview*vertical");
   listw = XtNameToWidget(Toplevel, "*permlist");
   /*
    * Get values of the vertical scrollbar
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNlength,     &length); cnt++;
   XtSetArg(args[cnt], XtNshown,      &shown);  cnt++;
   XtSetArg(args[cnt], XtNtopOfThumb, &top);    cnt++;
   XtGetValues(vertw, args, cnt);
#ifdef DEBUG
   fprintf(stderr, "Vertical scrollbar:\n");
   fprintf(stderr, "\tlength = [%d], shown = [%7.4f], top = [%7.4f]\n",
	   length, shown, top);
#endif
   
   /*
    * If the amount shown is less than all of the data, 
    * Recalculate scrollbar and form widget attributes
    */
   if (shown < (float) 1.0)
   {
      /*
       * Get the dimensions of the form and node widgets
       */
      cnt = 0;
      XtSetArg(args[cnt], XtNx, &x); cnt++;
      XtSetArg(args[cnt], XtNheight, &height); cnt++;
      XtGetValues(listw, args, cnt);

#ifdef DEBUG
      fprintf(stderr, "\tlist: height = [%4d]\n", height);
#endif

      /*
       * If the length is the same as the form height,
       * Reset the scrollbar to show everything
       * Else get new values
       */
      if (length >= height)
      {
	 new_top = (float)    0.0;
	 new_y   = (Position) 0;
	 shown   = (float)    1.0;
      }
      else
      {
	 /*
	  * Calculate the coverage
	  * If the coverage is more than the length,
	  * Calculate the new form widget Y as (length - coverage)
	  * This is usually negative to move the data up of
	  * the viewport.
	  */
	 temp    = (int) (selected - (shown * Pindex) / 2);
	 if (temp < 0)
	 {
	    temp = 0;
	 }
	 new_top = (float) temp / Pindex;
	 new_y   = (Position) 1 - (new_top * height);
      }
#ifdef DEBUG
      fprintf(stderr, "\tnew_top = [%7.4f], new_y = [%d], shown = [%7.4f]\n",
	      new_top, new_y, shown);
#endif

      /*
       * Set the new values of the scrollbar
       * Move the form widget
       */
      XawScrollbarSetThumb(vertw, new_top, shown);
      XtMoveWidget(listw, x, new_y);
   }
   return;
}




/*
   FUNCTION: 		UpdatePermlist

   Synopsis: 
       Updates the permutation list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
UpdatePermlist()

{
   int
      activeperm,
      i,
      len,
      permindex;
   PermutationList
      *pl;
   RhsList
      *rl;
   SymbolList
      *sl;
   Widget
      permlist;

   rl = Grammar[Gindex].activerhs;

   if (rl == NULL)
   {
      return;
   }

   /*
    * clear out old values in Permlist[] 
    */
   for (i = 0; Permlist[i] != NULL; i++)
   {
      XtFree(Permlist[i]);
      Permlist[i] = NULL;
   }

   permindex  = 0;
   activeperm = -1;
   
   /*
    * print out the permutations for current alternative 
    */
   for (pl = rl->perminstance; pl != NULL; pl = pl->next)
   {
      if (pl == rl->activeperm)
      {
	 activeperm = permindex;
      }
      
      len = 0;
      for (sl = pl->permsymbol; sl != NULL; sl = sl->next)
      {
	 len = len + strlen(sl->symbolstr) + 1;
      }
      /*
       * add 3 to length for "()" and NULL at end 
       */
      Permlist[permindex] = (String) XtMalloc(len + 3);
      Permlist[permindex] = strcpy(Permlist[permindex], "(");
      for (sl = pl->permsymbol; sl != NULL; sl = sl->next)
      {
	 Permlist[permindex] = strcat(Permlist[permindex], sl->symbolstr);
         if (sl->next != NULL)
	 {
	    Permlist[permindex] = strcat(Permlist[permindex], " ");
	 }
      }
      Permlist[permindex] = strcat(Permlist[permindex], ")");
      permindex++;
   }
     
   permlist = XtNameToWidget(Toplevel, "*permlist");
   if (permlist != NULL)
   {
      XawListChange(permlist, Permlist, permindex, 0, True);
   }
   AdjustVerticalPermScrollbar(permindex);
   if (activeperm >= 0)
   {
      XawListHighlight(permlist, activeperm);
   }
   return;
}



/*
   FUNCTION: 		PermSelect

   Synopsis: 
       Select a permutation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
PermSelect(w, client_data, current)

Widget
   w;
caddr_t
   client_data;
XawListReturnStruct
   *current; /* The symbol and its index selected by the user */

{
   int
      ndx;
   PermutationList
     *pl;
   RhsList
      *rl;

   rl = Grammar[Gindex].activerhs;

#ifdef DEBUG
   fprintf(stderr, "selection = [%s]\nposition = [%d]\n\n",
	   current->string, current->list_index);
#endif
   ndx = current->list_index + 1;
 
   SymbolManage(rl->activeperm->permsymbol, False);

   /*
    * walk down to the ndx'th entry in the permlist 
    */
   for (pl = rl->perminstance; --ndx > 0; pl = pl->next)
   {
   }
   rl->activeperm = pl;

   SymbolManage(pl->permsymbol, True);

   SensitizePermButtons(pl);
   SensitizeAltToggles();
   UpdatePermlist();
   /* move us to the selected permutation */
   return;
}



/*
   FUNCTION: 		CreatePermShell

   Synopsis: 
       Create the shell displaying the permutations.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget
CreatePermShell(parent, enable)

Widget
   parent,
   enable;

{
   Widget
      permdone,
      permform,
      permlist,
      permshell,
      permview;
   XtPopdownID
      pdptr;   /* Pointer to the popdown record that will take care of */
	       /* sensitizing the menu button */

   /*
    * -----------------------------------------
    * |      shell                            |
    * | ------------------------------------- |
    * | |    form                           | |
    * | | [done]                            | |
    * | |                                   | |
    * | | |---------------------|           | |
    * | | |   viewport          |           | |
    * | | | [permutation list]  |           | |
    * | | |---------------------|           | |
    * | |                                   | |
    * | ------------------------------------- |
    * -----------------------------------------
    */
   permshell = XtVaCreatePopupShell( "Permutations",
       applicationShellWidgetClass, parent,
       XtNallowShellResize, True,
       NULL );

   permform = XtVaCreateManagedWidget( "permform",
       groupWidgetClass, permshell,
       XtNspacing, 6,
       NULL );

   permdone = XtVaCreateManagedWidget( "permdone",
       commandWidgetClass, permform,
       XtNlabel,  "Done",
       XtNmargin, 6,
       NULL );

   pdptr                = XtNew(XtPopdownIDRec);
   pdptr->shell_widget  = permshell;
   pdptr->enable_widget = enable;
   XtAddCallback(permdone, XtNcallback, XtCallbackPopdown, pdptr);
   
   permview = XtVaCreateManagedWidget( "permview",
       viewportWidgetClass, permform,
       XtNfromVert,     permdone,
       XtNallowHoriz,   True,
       XtNallowVert,    True,
       XtNforceBars,    True,
       XtNuseBottom,    False,
       XtNuseRight,     False,
       XtNpreference,   False,
/*
       XtNprefWidth,    500,
       XtNminWidth,     150,
       XtNprefHeight,   200,
       XtNminHeight,    50,
*/
       XtNalignment,    gaFill,
       XtNstretch,      True,
       XtNmargin,       6,
       NULL );
   
   permlist = XtVaCreateManagedWidget( "permlist",
       listWidgetClass, permview,
       XtNresizable,      True,
       XtNdefaultColumns, 1,
       XtNforceColumns,   True,
       XtNlist,           Permlist,
       XtNnumberStrings,  0,
       NULL );
   XtAddCallback(permlist, XtNcallback, PermSelect, NULL);

   XtRealizeWidget(permshell);
   return(permshell);
}
