/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: utils.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/03/16 13:04:04 $

   HISTORY
   $Log:	utils.c,v $
 * Revision 1.5  92/03/16  13:04:04  ware
 * Conditionalize the title
 * 
 * Revision 1.4  1992/02/25  02:49:11  cso
 * Provided the distinction between batch and interactive xlator makes
 *
 * Revision 1.3  1992/02/21  05:55:15  cso
 *  - Do not die b/c of lack of malloc space, inform and return.
 *  - Do not display the entire makefile call, just inform that a make is
 * occurring
 *
 * Revision 1.2  1992/01/14  20:55:57  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.1  1992/01/10  02:24:14  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "initialize.h"
#include "status.h"

/*  GLOBALS  */



/*
   FUNCTION: 		MakeUpper

   Synopsis: 
       Make a string into the uppercase equivalent.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		An upper case version of the input string
*/

String
MakeUpper(str)

String
   str;  /* The string being transformed */

{
   String
      tmp; /* A temporary copy */
   int
      i;   /* Loop counter to examine at each character */
   
   /*
    * Make a copy of the string
    * For each character, consider if it should be changed
    */
   tmp = XtNewString(str);
   for (i = 0; i < strlen(tmp); i++)
   {
      /*
       * If the character is a lower case alphabetic character
       * Make it upper case
       */
      if (islower(tmp[i]))
      {
	 tmp[i] = toupper(tmp[i]);
      }
   }
   return(tmp);
}



/*
   FUNCTION: 		CreateEmptyTag

   Synopsis: 
       Create an empty tag for require tags not in the mapping.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

String
CreateEmptyTag(tag)

String
   tag;

{
   String
      emstr,
      uptag;
   int
      i;
   
   emstr = (String) XtMalloc((unsigned) (2 * strlen(tag)) + 16);
   if (emstr == NULL)
   {
      fprintf(stderr, "CreateEmptyTag: Unable to allocate memory\n");
      exit(1);
   }
   for (i = 0; i < Gmax; i++)
   {
      if (strcmp(tag, Grammar[i].lhs) == 0)
      {
	 if ((Grammar[i].rhs != NULL) && (Grammar[i].rhs->symbol != NULL))
	 {
	    if (*Grammar[i].rhs->symbol->symbolstr == '<')
	    {
	       uptag = MakeUpper(tag);
	       sprintf(emstr, "structnew(\"<%s></%s>\")", uptag, uptag);
	       XtFree(uptag);
	       return(emstr);
	    } /***** IF this symbol in not a grouping symbol */
	 } /***** IF a rhs exists */
      } /***** IF this is the production */
   } /***** FOR all productions */

   sprintf(emstr, "structnew(\"\")");
   return(emstr);
}




/*
   FUNCTION: 		GetMapping

   Synopsis: 
       Get the mapping position.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

String
GetMapping(pos, tag, sptr1)

int
   pos;
String
   tag;
SymbolList
   *sptr1;

{
   String
      str;
   int
      i;
   SymbolList
      *sptr;

   if (tag[0] == '<')
   {
      str = (String) XtMalloc((unsigned) (strlen(tag) + 32));
      if (str == NULL)
      {
	 fprintf(stderr, "GetMapping: Could not allocate [%d] bytes\n",
		 strlen(tag) + 32);
	 exit(1);
      }
      sprintf(str, "structnew(\"%s\\n\")", tag);
      return(str);
   }
   else
   {
      for (sptr = sptr1, i = 1;
	   ((sptr != NULL) && (sptr->position != pos));
	   sptr = sptr->next, i++)
      {
      }
      if (sptr != NULL)
      {
	 str = (String) XtMalloc((unsigned) (sizeof(int) + 2));
	 if (str == NULL)
	 {
	    fprintf(stderr, "GetMapping: Could not allocate [16] bytes\n");
	    exit(1);
	 }
	 sprintf(str, "$%d", i);
	 return(str);
      }
      else
      {
	 /*
	  * This symbol is not included in the permutation, so create
	  * an empty content tag for it if it is not a grouping symbol.
	  */
	 return(CreateEmptyTag(tag));
      }
   }
   /* Unnecessary return
    *  return(tag);
    */
}



/*
   FUNCTION: 		Output

   Synopsis: 
       Output standard header strings.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Output(fptr, str)

FILE
   *fptr;
String
   str[];

{
   int
      i;
   
   for (i = 0; str[i] != 0; i++)
   {
      fprintf(fptr, "%s\n", str[i]);
   }
   return;
}



/*
   FUNCTION: 		MakeTranslator

   Synopsis: 
       Create the system call to make the translator.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakeTranslator(target, upitype, granularity)

String
   target;
UpInterface
   upitype,
   granularity;

{
   String
      makestring;
   int
      len;
   
   len = strlen(target);
   makestring = (String) XtMalloc((unsigned) (len + 128));
   if (makestring == NULL)
   {
      fprintf(stderr, "MakeTranslator: unable to allocate [%d] bytes\n",
	      len + 128);
      return;
   }
   switch (upitype)
   {
   default:
      break;
      
   case With:
      if (granularity == Without)
      {
	 sprintf(makestring,
         "make -f $ICAROOT/lib/Makefile.up 'TARGET=%s' 'INTFLAGS=-DAUTOSHIFT'",
		 target);
      }
      else
      {
	 sprintf(makestring,
	 "make -f $ICAROOT/lib/Makefile.up 'TARGET=%s' 'INTFLAGS='",
		 target);
      }
      break;
      
   case Without:
      sprintf(makestring,
      "make -f $ICAROOT/lib/Makefile.gen 'TARGET=%s' 'MAINLIB=txmain'",
	      target);
      break;
   }
   
   XicaStatus(Statw, "Making...\n");
   errno = 0;
   if (system(makestring) < 0)
   {
      perror("System call to Make translator failed");
      XicaStatus(Statw, "Make failed\n");
   }
   else
   {
      XicaStatus(Statw, "Make completed\n");
   }
      
   XtFree(makestring);
   
   return;
}




/*
   FUNCTION: 		OpenGenerationFile

   Synopsis: 
       Open a file to generate code (yacc, or lex).

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

FILE
*OpenGenerationFile(fileroot, yaccflag)

String
   fileroot;
Boolean
   yaccflag;

{
   FILE
      *fptr;
   String
      filename;
   
   filename = (String) XtMalloc((unsigned) (strlen(fileroot) + 4));
   if (yaccflag == True)
   {
      sprintf(filename,  "%s.y",  fileroot);
   }
   else
   {
      sprintf(filename,  "%s.l",  fileroot);
   }
   errno = 0;
   if ((fptr = fopen(filename, "w")) == NULL) 
   {
      (void) perror("Error opening file to generate transator");
      exit(1);
   }
   
   return(fptr);
}
