/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: callbacks.c,v $
   Purpose:     Provide the callback routines for the various
                  interface pieces.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.13 $
   Date:        $Date: 92/04/29 20:11:33 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	callbacks.c,v $
 * Revision 1.13  92/04/29  20:11:33  boyd
 * 
 * Fixed problem with duplicates overlaying subsequent lines, by adding DoLayout
 * calls to Duplicate (although this _really_ shouldn't be necessary, it doesn't
 * work otherwise).
 * 
 * Revision 1.12  92/03/16  13:52:37  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.13  92/03/05  18:34:36  boyd
 * Removed unnecessary Manage/Unmanage calls in ScrollTogether, eliminating
 * the flicker problem.
 * 
 * Revision 1.12  92/03/05  17:54:14  boyd
 * Changed message for dummy help to go to Status.
 * 
 * Revision 1.11  92/03/05  17:50:26  boyd
 * As received from cso/ware, before my changes.
 * 
 * Revision 1.10  1992/02/25  02:56:32  cso
 * Provided the use of a general Makefile
 *
 * Revision 1.9  1992/02/21  05:52:28  cso
 * Added a cancel option to the save changes dialog
 *
 * Revision 1.8  1992/02/20  04:58:44  cso
 * Fixed a problem with using yyrestart
 *
 * Revision 1.7  1992/02/11  17:00:48  cso
 * Added the appropriate Dirtyflag changes
 *
 * Revision 1.6  1992/02/06  05:14:21  cso
 * Fixed the titles that appear in the file dialogs to be correct
 * depending on context.
 *
 * Revision 1.5  1992/01/28  22:36:02  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "fileio.h"
#include "generate.h"
#include "itool.h"
#include "liftab.h"
#include "status.h"
#include "tagedit.h"
#include "utils.h"
#include "xinterface.h"

/* External from Flex */

#ifdef FLEX
extern void
   yyrestart();
#endif

/*  GLOBALS  */

Boolean
   Dirtyflag;



/*
   FUNCTION:               SaveChanges

   Synopsis: 
	Check and see if changes occurred to make sure they are saved.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON
	pete	3/5/92	Working on consistent titles

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


static Boolean
SaveChanges()

{
   Boolean
      continue_oper,
      response;
   FILE
      *fptr;

   /*
    * If the current session has changed, give the user a chance to
    * save the file or at least realize that changes have occurred.
    */
   continue_oper = True;
   if (Dirtyflag == True)
   {
      /*
       * Ask the user if they wish to save the changes, if Continue, 
       * then get a file name and save the file
       */
      continue_oper = YesNoCancelMessage(Frame, "Save changes?",
					 True, &response);
      
      if ((continue_oper == True) && (response == True))
      {
	 /*
	  * Get the file to save the current session
	  */
	 fptr = OpenFile("Save a specification [.isf]",
			 "Save:",
			 "Save",
			 "*.isf",
			 "isf",
			 False);
	 
	 /*
	  * If successful in getting a file, save out the current session
	  * Else alert the user that changes will be lost
	  */
	 if (fptr != NULL)
	 {
	    SaveIspec(fptr);
	    fclose(fptr);
	    Dirtyflag = False;
	 }
	 else
	 {
	    /*
	     * If the user does not wish to lose changes,
	     * then return False, preventing whatever action would lose
	     * the changes from occurring.
	     */
	    return(YesNoMessage(Frame, "Lose changes?", False));
	 }
      }
   }
   return(continue_oper);
}




/* FUNCTION:  Reset
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

void
Reset(w, client_data, call_data)

Widget 
   w;
caddr_t 
   client_data,
   call_data;

{
   if (SaveChanges() == True)
   {
      /*
       * Initialize devegram, after confirming with user
       */
      if (YesNoMessage(Frame, "Reset interface?", True) == True)
      {
	 ResetValues();
	 Dirtyflag = False;
      }
   }
   return;
}




/* FUNCTION:  NewFile

   Synopsis:
      Loads in an ICA specification.

   Initial purpose:
      Same as above.

   Present purpose:
      Same as above.

   Creator: Conleth O'Connell
   Reviewer:
   Modifications:
       WHO       DATE    REASON

   Globals used:
      none
   Globals Modified:
      none

   Return value:           void
   Range:                  none
*/


void
NewFile(w, client_data, call_data)

Widget 
   w;
caddr_t 
   client_data,
   call_data;

{
   extern FILE 
      *yyin;

   /* reset everything*/
   /* check if any buffers have been changed since last save */
   if (SaveChanges() == True)
   {
      yyin = OpenFile("Load an ICA Specification [.ica]",
		      "Load:",
		      "Load",
		      "*.ica",
		      "ica",
		      True);

      if (yyin != NULL)
      {
	 ResetValues();
	 XicaStatus(Statw, "Loading new intag specification.\n");
	 if (LoadIspec() != 0)
	 {
	    XicaStatus(Statw,
		       "There was an error in the specification file\n");
	 }
	 fclose(yyin);
#ifdef FLEX
	 yyrestart(yyin);
#endif
	 UpdatePriority();
	 SortLiftab();
	 UpdateEditor();
	 XicaStatus(Statw, "Load completed.\n");
	 XicaStatus(Statw, "Don't forget to save this specification.\n");
	 Dirtyflag = True;
      }
      Lifentry = NULL;
   }
   return;
}



/* FUNCTION:  GetFile
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON
	pete	3/5/92	Working on consistent titles

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

/* ************************************************************************ */

void
GetFile(w, client_data, call_data)

Widget 
   w;
caddr_t 
   client_data,
   call_data;

{
   extern FILE 
      *yyin;

   /* reset everything*/
   /* check if any buffers have been changed since last save */
   if (SaveChanges() == True)
   {
      yyin = OpenFile("Edit a specification [.isf]",
		      "Edit:",
		      "Edit",
		      "*.isf",
		      "isf",
		      True);

      if (yyin != NULL)
      {
	 ResetValues();
	 XicaStatus(Statw, "Loading intag specification.\n");
	 if (LoadIspec() != 0)
	 {
	    XicaStatus(Statw,
		       "There was an error in the specification file\n");
	 }
	 fclose(yyin);
#ifdef FLEX
	 yyrestart(yyin);
#endif
	 UpdatePriority();
	 SortLiftab();
	 UpdateEditor();
	 XicaStatus(Statw, "Load completed.\n");
	 Dirtyflag = False;
      }
      Lifentry = NULL;
   }
   return;
}



/* FUNCTION:  SaveFile
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON
	pete	3/5/92	Working on consistent titles

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/


/* ************************************************************************ */

void
SaveFile(w, client_data, call_data)

Widget 
   w;
caddr_t 
   client_data,
   call_data;

{
   FILE 
      *fptr;

   /* call dialog box to get file name */
   fptr = OpenFile("Save a specification [.isf]",
		   "Save:",
		   "Save",
		   "*.isf",
		   "isf",
		   False);

   if (fptr != NULL)
   {
      XicaStatus(Statw, "Saving intag specification.\n");
      SaveIspec(fptr);
      fclose(fptr);
      XicaStatus(Statw, "Save completed.\n");
      Dirtyflag = False;
   }
   return;
}



/* FUNCTION:  Quit
   Synopsis:  When the Quit button is pressed, it checks to see if any
   files have been changed but not saved, and if so, puts up an alert
   widget to confirm the quit action. If confirmed, it goes ahead.
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:    Shantanu Sarkar
   Modifications:
       WHO       DATE    REASON
       ss        10/90   Added new alert widgets
   Interface issues:
      FUNCTION                PURPOSE
      XawAsciiSourceChanged   Check if modifications have been made.
                               Note: This does not work properly.

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

void
Quit(w, client_data, call_data)

Widget 
   w;
caddr_t  
   client_data,
   call_data;

{
   /* check if any buffers have been changed since last save */
   if (SaveChanges() == True)
   {
      exit(0);
   }
   return;
}



/* FUNCTION:  MakeIntag
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON
	pete	3/5/92	Working on consistent titles

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/



void
MakeIntag(w, client_data, call_data)

Widget 
     w;
caddr_t  
     client_data,
     call_data;

{
   Entry
      *tmpptr;
   extern FILE 
      *yyin;
   String
      filestr,
      sysstr;

   /* check if any buffers have been changed since last save */
   if (SaveChanges() == True)
   {
      yyin = OpenFile("Make a specification [.isf] into a [.ix]",
		   "Load:",
		   "Load",
		   "*.isf",
		   "isf",
		   True);

      if (yyin != NULL)
      {
	 filestr = GetFilename("Make a specification [.isf] into a [.ix]",
			       "Make:",
			       "Make",
			       "*.ix",
			       "ix",
			       False);
	 
	 if (*filestr != '\0')
	 {
	    XicaStatus(Statw, "Loading intag specification.\n");
	    tmpptr = Liftable;
	    Liftable = NULL;
	    Lifentry = NULL;
	    InitLif();
	    if (LoadIspec() == 0)
	    {
	       FixDuplicates();
	       UpdatePriority();
	       
	       XicaStatus(Statw, "ISF Compiler working...\n");
	       GenerateLex(filestr);
	       GenerateYacc(filestr);
	       sysstr = XtMalloc(strlen(filestr) + ICAlen + 64);
	       sprintf(sysstr,
	       "make -f %s/lib/Makefile.gen 'TARGET=%s' 'MAINLIB=intag'",
		       ICApath, filestr);
#ifdef DEBUG
	       fprintf(stderr, "sysstr = [%s]\n", sysstr);
#endif
	       errno = 0;
	       if (system(sysstr) < 0)
	       {
		  perror("WARNING: problem with the system call to make");
	       }
	    }
	    else
	    {
	       XicaStatus(Statw, "There was an error in the specification.\n");
	       XicaStatus(Statw, "Compilation aborted.\n\n");
	    }
	    ResetValues();
	    XtFree(Liftable);
	    Liftable = tmpptr;
	 } /***** The user has selected an ix file to create */
	 fclose(yyin);
#ifdef FLEX
	 if (*filestr != '\0')
	 {
	    yyrestart(yyin);
	 }
#endif
      } /***** The user has chosen a spec file to compile */
   }
   return;
}




/* FUNCTION:  RunIntag
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON
	pete	3/5/92	Working on consistent titles

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

/* ************************************************************************ */

void
RunIntag(w, client_data, call_data)

Widget 
   w;
caddr_t  
   client_data,
   call_data;

{
   String
      execfile,
      infile,
      outfile,
      str;

   /* pick a document to intag */
   infile = GetFilename("Choose an input file [*]",
			"Ok:",
			"Ok",
			"*",
			"*",
			True);
   
   if (*infile == '\0')
   {
      XtFree(infile);
      return;
   }
      
   /* choose an output file name */
   outfile = GetFilename("Choose an output file [*]\n\
 Cancel to use stdout",
			 "Ok:",
			 "Ok",
			 "*",
			 "*",
			 False);
   
   /* pick an executable */
   execfile = GetFilename("Execute a [.ix]",
			  "Run:",
			  "Run",
			  "*.ix",
			  "ix",
			  True);
   
   if (*execfile == '\0')
   {
      XtFree(infile);
      XtFree(outfile);
      XtFree(execfile);
      return;
   }
   
   if (*outfile != '\0')
   {
      str = XtMalloc(strlen(execfile) + strlen(infile) +
		     strlen(outfile) + 32);
      sprintf(str, "%s < %s > %s &", execfile, infile, outfile);
#ifdef DEBUG
      fprintf(stderr, "%s\n", str);
#endif
      system(str);
      XtFree(outfile);
   }
   else
   {
      str = XtMalloc(strlen(execfile) + strlen(infile) + 32);
      sprintf(str, "%s < %s &", execfile, infile);
#ifdef DEBUG
      fprintf(stderr, "%s\n", str);
#endif
      system(str);
   }
   XtFree(str);
   XtFree(infile);
   XtFree(execfile);
   return;
}



void
Help(w, client_data, call_data)

Widget
   w;           /* Unused */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   XicaStatus( Statw, "Help was selected--There's no help for you!\n" );
   return;
}



/*
   FUNCTION: 		AttSet

   Synopsis: 
	Set the attribute state.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
AttSet(w, tagstr, call_data)

Widget
   w;
String
   tagstr;
caddr_t
   call_data;

{
   Arg
      args[MAXARGS];
   Boolean
      dupflag,
      state;
   int
      cnt;
   Entry
      *eptr,
      *eptr1,
      *eptr2;
   String
      str;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNstate, &state); cnt++;
   XtSetArg(args[cnt], XtNlabel, &str);   cnt++;
   XtGetValues(w, args, cnt);
   eptr    = FindEntry(tagstr, XtParent(w));
   dupflag = GetDuplicateEntries(eptr, &eptr1, &eptr2);
   
   if (eptr != NULL)
   {
      switch (*str)
      {
      default:
	 XicaStatus(Statw, "Error in AttSet\n");
	 break;
	 
      case 'S':
	 if ((state == False) && (eptr->end == False))
	 {
	    XicaStatus(Statw, "Can't turn both off!!\n");
	    SetState(XtNameToWidget(eptr->boolform, "start"), eptr->start);
	 }
	 else
	 {
	    eptr->start = state;
	    Dirtyflag = True;
	 }
	 break;
	 
      case 'E':
	 if ((state == False) && (eptr->start == False))
	 {
	    XicaStatus(Statw, "Can't turn both off!!\n");
	    SetState(XtNameToWidget(eptr->boolform, "end"), eptr->end);
	 }
	 else
	 {
	    eptr->end = state;
	    Dirtyflag = True;
	 }
	 break;
	 
      case 'N':
	 eptr->nullflag = state;
	 Dirtyflag = True;
	 break;
      }      

      if (dupflag == True)
      {
	 CheckDuplicates(eptr, eptr1, str);
	 if (eptr2 != NULL)
	 {
	    CheckDuplicates(eptr, eptr2, str);
	 }
      }
   }
   return;
}



/*
   FUNCTION: 		ScrollTogether

   Synopsis: 
	Scroll the two viewports together.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/

void
ScrollTogether(w, client_data, call_data)

Widget 
   w;
caddr_t
   client_data, /* not used */
   call_data;   /* not used */

{
   Arg 
      args[MAXARGS];
   int
      cnt;
   Position 
      x,
      y;
   float 
      top,
      shown;
   Widget
      tform,
      tvert,
      lform,
      lvert;
   
   tvert = XtNameToWidget(Frame, "*tagview*vertical");
   lvert = XtNameToWidget(Frame, "*lvlview*vertical");
   tform = XtNameToWidget(Frame, "*tagform");
   lform = XtNameToWidget(Frame, "*lvlform");
   
   cnt = 0;
   XtSetArg(args[cnt], XtNtopOfThumb, &top);   cnt++;
   XtSetArg(args[cnt], XtNshown,      &shown); cnt++;
   XtGetValues(w, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNx, &x); cnt++;
   XtSetArg(args[cnt], XtNy, &y); cnt++;
   if (w == tvert)
   {
      XtGetValues(tform, args, cnt);
      XawScrollbarSetThumb(lvert, top, shown);
      XtMoveWidget(lform, x, y);
   }
   else
   {
      XtGetValues(lform, args, cnt);
      XawScrollbarSetThumb(tvert, top, shown);
      XtMoveWidget(tform, x, y);
   }
   
   return;
}



/*
   FUNCTION: 		SetLifentry

   Synopsis: 
	Set the global Lif ptr.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
SetLifentry(w, eptr, call_data)

Widget 
   w;
Entry
   *eptr;
caddr_t
   call_data;   /* not used */

{
   Lifentry = eptr;
   return;
}




/*
   FUNCTION: 		Duplicate

   Synopsis: 
	Duplicate an entry.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/



void
Duplicate(w, client_data, call_data)

Widget 
   w;
caddr_t
   client_data, /* not used */
   call_data;   /* not used */

{
   Boolean
      dupflag;
   Entry
      *eptr,
      *eptr1,
      *eptr2,
      *neweptr;
   Widget
      lvlform,
      tagform;

   lvlform = XtNameToWidget(Frame, "*lvlform");
   tagform = XtNameToWidget(Frame, "*tagform");

   if (Lifentry == NULL)
   {
      XicaStatus(Statw, "Try using Button 1 (usually the left)\n");
      return;
   }
   eptr = Lifentry;
   dupflag = GetDuplicateEntries(eptr, &eptr1, &eptr2);
   if ((dupflag == True) && (eptr1 != NULL) && (eptr2 != NULL))
   {
      XicaStatus(Statw, "No more duplicates allowed\n");
      return;
   }
   XawFormDoLayout( tagform, False );
   XawFormDoLayout( lvlform, False );

   neweptr = InsertEntry(eptr, eptr1);
   if (neweptr != NULL)
   {
      Dirtyflag = True;
      if (eptr1 != NULL)
      {
	 if ((eptr->start == True) && (eptr1->start == True))
	 {
	    neweptr->start = False;
	    neweptr->end   = True;
	 }
	 else if ((eptr->end == True) && (eptr1->end == True))
	 {
	    neweptr->start = True;
	    neweptr->end   = False;
	 }
	 else
	 {
	    neweptr->start = True;
	    neweptr->end   = True;
	 }
      }
      else
      {
	 if ((eptr->start == True) && (eptr->end == True))
	 {
	    neweptr->start = True;
	    neweptr->end   = False;
	 }
	 else
	 {
	    neweptr->start = True;
	    neweptr->end   = True;
	 }
      }
      SetState(XtNameToWidget(neweptr->boolform, "start"), neweptr->start);
      SetState(XtNameToWidget(neweptr->boolform, "end"),   neweptr->end);
   }
   Lifentry = NULL;

   XawFormDoLayout( tagform, True );
   XawFormDoLayout( lvlform, True );

   return;
}



/*
   FUNCTION: 		Remove

   Synopsis: 
	Remove an independent entry.  Independence is classified as a
	duplicate or an entry that is not depended upon as a parnet of
	another entry.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
Remove(w, client_data, call_data)

Widget 
   w;
caddr_t
   client_data, /* not used */
   call_data;   /* not used */

{
   Container
      *cptr;
   Entry
      *eptr,
      *eptr1,
      *eptr2;
   String
      alertstr;
   
   if (Lifentry == NULL)
   {
      XicaStatus(Statw, "Try using Button 1 (usually the left)\n");
      return;
   }
   eptr = Lifentry;
/*
   if (GetDuplicateEntries(eptr, &eptr1, &eptr2) == False)
   {
      XicaStatus(Statw, "Cannot delete a single entry\n");
      eptr->start = True;
      eptr->end   = True;
      SetState(XtNameToWidget(eptr->boolform, "start"), eptr->start);
      SetState(XtNameToWidget(eptr->boolform, "end"),   eptr->end);
      return;
   }
*/   
   if ((CheckDependencies(eptr->tag) == True) &&
       (GetDuplicateEntries(eptr, &eptr1, &eptr2) == False))
   {
      XicaStatus(Statw,
	      "Cannot delete a single-entry tag being depended upon\n");
      return;
   }
   else if (GetDuplicateEntries(eptr, &eptr1, &eptr2) == False)
   {
      for (cptr = eptr->cptr; cptr != NULL; cptr = cptr->next)
      {
	 eptr1 = GetEntry(cptr->str);
	 if (eptr1->type == Tags)
	 {
	    eptr1->type = Mixed;
	    ChangeLabel(XtNameToWidget(eptr1->boolform, "*content"),
			"Both");
	 }
      }
   }
   alertstr = XtMalloc(strlen(eptr->tag) + 32);
   sprintf(alertstr, "Remove a specification for [%s]?", eptr->tag);
   if (YesNoMessage(Frame, alertstr, False) == False)
   {
      return;
   }
   RemoveEntry(eptr);
   Dirtyflag = True;
   Lifentry = NULL;
   return;
}
