/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: fileio.c,v $
   Purpose:     Contains all the headers for the file fileio.c

   Programmer:  Satish Menon
   Reviewer:    
   Revision:    $Revision: 1.9 $
   Date:        $Date: 92/03/16 13:52:48 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	fileio.c,v $
 * Revision 1.9  92/03/16  13:52:48  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.10  92/03/07  18:46:05  boyd
 * Removed prompt from FileDlg, to be consistent with devegram.
 * 
 * Revision 1.9  92/03/07  18:44:42  boyd
 * As received from cso, before my changes.
 * 
 * Revision 1.8  1992/02/11  21:46:54  cso
 * Fixed the creation of the file dialog in terms of title/prompt
 *
 * Revision 1.7  1992/02/06  05:14:13  cso
 * Fixed the titles that appear in the file dialogs to be correct
 * depending on context.
 *
 * Revision 1.6  1992/01/31  15:17:57  cso
 * CHanged the name of the file dialog
 *
 * Revision 1.5  1992/01/28  22:36:09  cso
 * Restructured dependencies
 *
 * Revision 1.4  1992/01/28  18:08:50  cso
 * Fixed a bad return, and added a cast to please lint
 *
 * Revision 1.3  1992/01/02  23:37:47  cso
 * Upgraded to the Xica widget set from Xlwh.  Using Message boxes
 * instead of the alert shells, and the new file dialog.
 *
 * Revision 1.2  1991/11/27  00:29:00  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.1.1.3  1991/10/04  20:19:49  jbm
 * First HaL mods to ICA
 *
 * Revision 1.2  90/10/24  16:08:20  cso
 * *** empty log message ***
 * 
 * Revision 1.1  90/10/12  14:17:38  cso
 * Initial revision
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS */

#include "status.h"
#include "xinterface.h"



/*
   FUNCTION: 		GetFilename

   Synopsis: 
       Get a filename from the user by using Loyde's XlwhPopup, which will 
       not popdown until a valid filename has been specified.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		"" - no filename, e.g., the user changed his mind
                        a valid filename including path and extension
*/

String
GetFilename(title, prompt, okbut, filter, exten, loadflag)

String
   title,
   prompt,
   okbut,
   filter,
   exten;             /* The extension to be used with the filename */
Boolean
   loadflag;          /* Specifies the operation */

{
   Arg
      args[MAXARGS];  /* List of arguments */
   int
      cnt;            /* Number of arguments */
   String
      filestr,        /* The filename to be returned */
      name,
      name_ext,
      path;
   Widget
      dialog;         /* The dialog widget requiring modification */

   filestr = XtNewString("./test.dgm");
   
   dialog = XtNameToWidget(XtParent(Frame), "*filedlg");

   if (dialog == NULL)
   {
      XicaStatus(Statw, "Error GetFilename: Unable to find filedlg\n");
      return("");
   }
   
   /*
    * Build a filename as a default, using any previous name and
    * the new extension.
    * If a previous name is present, use it with the
    * new extension.
    * Set up the dialog for this invocation.
    * Dialog title text
    * file spec that must be matched to exit
    * prompt for file name input field
    * label for open button
    * must file exist to be opened?
    * read access expecte? 
    * write access expected?
    * confirm overwrite
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNname, &name); cnt++;
   XtGetValues(dialog, args, cnt);

   if (strlen(name) > 0)
   {
      name_ext = (String) XtMalloc(strlen(name) + strlen(exten) + 2);
      strcpy(name_ext, name);
      strcat(name_ext, ".");
      strcat(name_ext, exten);
   }
   else
   {
      name_ext = NULL;
   }

   cnt = 0;
   if ((name != NULL) && (strlen(name) > 0))
   {
      XtSetArg(args[cnt], XtNname, name_ext); cnt++;
   }
   else
   {
      XtSetArg(args[cnt], XtNname, ""); cnt++;
   }
   XtSetArg(args[cnt], XtNtitle,      "File Dialog"); cnt++;
   XtSetArg(args[cnt], XtNdlgTitle,   title);         cnt++;
   XtSetArg(args[cnt], XtNspec,       filter);        cnt++;
   XtSetArg(args[cnt], XtNprompt,     NULL);          cnt++;
   XtSetArg(args[cnt], XtNbutton,     okbut);         cnt++;
   XtSetArg(args[cnt], XtNmustExist,  loadflag);      cnt++;
   XtSetArg(args[cnt], XtNdirsFirst,  False);         cnt++;
   XtSetArg(args[cnt], XtNfilesFirst, True);          cnt++;
   XtSetArg(args[cnt], XtNwillRead,   True);          cnt++;
   XtSetArg(args[cnt], XtNwillWrite,  !loadflag);     cnt++;
   XtSetArg(args[cnt], XtNconfirm,    True);          cnt++;
   XtSetValues(dialog, args, cnt);
   
   /*
    * Pop up the dialog and wait for a response from the user
    */
   DoFileDialog((FileDlgWidget) dialog);

   /*
    * If a file was selected, the name will have contents.
    * Concatenate the returned path and name.
    * In all cases, the name will also have an extension; strip the
    * extension from the name so a new one can be appended next time.
    * Also, if the dialog was cancelled, we want to keep the name,
    * but it will have been cleared, so use the one that was passed in.
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNname, &name); cnt++;
   XtSetArg(args[cnt], XtNpath, &path); cnt++;
   XtGetValues(dialog, args, cnt);

   XtFree(filestr);
   filestr = (String) XtMalloc(strlen(path) + strlen(name) + 1);
   if (strlen(name) == 0)
   {
      strcpy(filestr, "");
   }
   else
   {
      strcpy(filestr, path);
      strcat(filestr, name);
      if (name_ext != NULL)
      {
	 XtFree(name_ext);
      }
      name_ext = (String) XtMalloc(strlen(name) + 1);
      strcpy(name_ext, name);
   }
   if (name_ext != NULL)
   {
      /*
       * Cut off the extension
       * Set the name (for next time) to be the nonextended filename
       */
      if (strcmp(exten, "*") != 0)
      {	 
	 name_ext[strlen(name_ext) - strlen(exten) - 1] = '\0';
      }
      cnt = 0;
      XtSetArg(args[cnt], XtNname, name_ext); cnt++;
      XtSetArg(args[cnt], XtNpath, path);     cnt++;
      XtSetValues(dialog, args, cnt);
      XtFree(name_ext);
   }
   
#ifdef DEBUG
   fprintf(stderr,"filestr = [%s]\n", filestr);
#endif
   return(filestr);
}




/*
   FUNCTION: 		OpenFile

   Synopsis: 
       Get a filename from the user and open that file and return the 
       file pointer.

   Initial purpose:
       Get a filename from the user..

   Present purpose:
       Get the name and perform the opening of the file as the function
       name suggests.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	FILE *

   Range:		NULL or a valid file pointer
*/

FILE
*OpenFile(title, prompt, okbut, filter, exten, loadflag)

String
   title,
   prompt,
   okbut,
   filter,
   exten;    /* The extension for the file */
Boolean
   loadflag; /* Determines the operation load/save - read/write */

{
   String
      filestr;  /* The name of the file including explicit path */
   FILE
      *fptr;    /* File pointer to return */

   /*
    * Get a filename from the user
    */
   filestr = GetFilename(title, prompt, okbut, filter, exten, loadflag);

   /*
    * If the filename is not "", perform the open based on the loadflag
    * Else return NULL, no file pointer for a nonexistent file.
    *
    * The pointer returned is NULL because of two reasons - 
    *   1) a nonexistent file was chosen by the user, i.e., "", or 
    *   2) opening the file failed
    * Or it is a valid file pointer to a file opened per loadflag
    */
   if (*filestr != '\0')
   {
      if (loadflag == True)
      {
	 fptr = fopen(filestr, "r");
      }
      else
      {
	 fptr = fopen(filestr, "w");
      }

      /*
       * If we were unsuccessful in opening a file, inform the user
       */
      if (fptr == NULL)
      {
	 XicaStatus(Statw, "Unable to open file\n\t[%s]\n", filestr);
      }
      return(fptr);
   }
   else
   {
      return(NULL);
   }
}




/*
   FUNCTION: 		CreateFileIOShell

   Synopsis: 
	Create the popup shell.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
CreateFileIOShell(parent)

Widget
   parent;

{
   SetupFileDialog(parent);

   return;
}


