/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: generate.c,v $
   Purpose:     Generate the code that is executed.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/16 13:52:56 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	generate.c,v $
 * Revision 1.4  92/03/16  13:52:56  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.3  1992/01/28  22:36:19  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "output.h"
#include "print.h"
#include "status.h"



/*
   FUNCTION: 		Output

   Synopsis: 
	Output the static string.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


static void
Output(fptr, str)

FILE
   *fptr;
String
   str[];

{
   int
      i;
   
   for (i = 0; str[i] != 0; i++)
   {
      fprintf(fptr, "%s\n", str[i]);
   }
}



/*
   FUNCTION: 		GenerateLex

   Synopsis: 
	Generate the lex spec.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
GenerateLex(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      lexfile;

   lexfile = XtMalloc(strlen(filestr) + 8);
   sprintf(lexfile, "%s.l", filestr);
   if  ((fptr = fopen(lexfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open lex file [%s]\n", lexfile);
      XtFree(lexfile);
      return;
   }
   Output(fptr, headlex);
   PrintTags(fptr);
   Output(fptr, taillex);
   fclose(fptr);
   XtFree(lexfile);
   return;
}



/*
   FUNCTION: 		GenerateYacc

   Synopsis: 
	Generate the yacc spec.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
GenerateYacc(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      yaccfile;
   
   yaccfile = XtMalloc(strlen(filestr) + 8);
   sprintf(yaccfile, "%s.y", filestr);
   if  ((fptr = fopen(yaccfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open yacc file [%s]\n", yaccfile);
      XtFree(yaccfile);
      return;
   }
   Output(fptr, headyacc);
   PrintTokens(fptr);
   Output(fptr, midyacc1);
   PrintNterms(fptr);
   Output(fptr, midyacc2);
   PrintProds(fptr);
   Output(fptr, tailyacc);
   fclose(fptr);
   XtFree(yaccfile);
   return;
}
