/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: itool.c,v $
   Purpose:

   Programmer:  Shantanu Sarkar
   Reviewer:
   Revision:    $Revision: 1.5 $
   Date:        $Date: 92/03/16 13:53:03 $

NOTE:
   AVAIL can be:
        HIDDEN for internal use only
        EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	itool.c,v $
 * Revision 1.5  92/03/16  13:53:03  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.4  1992/02/20  04:58:47  cso
 * Fixed a problem with using yyrestart
 *
 * Revision 1.3  1992/02/11  21:51:35  cso
 * Use of delaying form layout to speed interface changes
 *
 * Revision 1.2  1992/01/28  22:36:30  cso
 * Restructured dependencies
 *
 * Revision 1.1.1.3  1991/10/04  20:18:29  jbm
 * First HaL mods to ICA
 *
 * Revision 1.6  90/11/30  14:50:55  shantanu
 * *** empty log message ***
 * 
 * Revision 1.5  90/11/05  15:23:48  shantanu
 * *** empty log message ***
 * 
 * Revision 1.4  90/10/24  11:36:50  shantanu
 * *** empty log message ***
 * 
 * Revision 1.3  90/10/19  15:38:41  shantanu
 * new header files dialog.h
 * 
 * Revision 1.2  90/10/16  12:58:50  shantanu
 * New alert widgets added
 * 
 * Revision 1.1  90/10/12  12:03:45  shantanu
 * Initial revision
 * 
*/

/* NOTE: Only include the header files that are absolutely necessary */
/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "liftab.h"
#include "xinterface.h"



/* FUNCTION:  ResetValues
   Synopsis:
   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

/* ************************************************************************ */
void
ResetValues()

{
   Container
      *cptr,
      *cptr1;
   Entry
      *eptr,
      *eptr1;
   Widget
      tform,
      lform;

   tform = XtNameToWidget(Frame, "*tagform");
   lform = XtNameToWidget(Frame, "*lvlform");
   XawFormDoLayout(tform, False);
   XawFormDoLayout(lform, False);
   for (eptr = Liftable; eptr != NULL; )
   {
      eptr1 = eptr->next;
      if (eptr->tagw != NULL)
      {
	 XtDestroyWidget(eptr->tagw);
	 eptr->tagw = NULL;
      }
      if (eptr->boolform != NULL)
      {
	 XtDestroyWidget(eptr->boolform);
	 eptr->boolform = NULL;
      }
      XtFree(eptr->tag);
      eptr->tag = NULL;
      for (cptr = eptr->cptr; cptr != NULL; )
      {
	 cptr1 = cptr->next;
	 if (cptr->w != NULL)
	 {
	    XtDestroyWidget(cptr->w);
	    cptr->w = NULL;
	 }
	 XtFree(cptr->str);
	 cptr->str  = NULL;
	 cptr->next = NULL;
	 cptr->prev = NULL;
	 XtFree(cptr);
	 cptr       = cptr1;
      }
      eptr->cptr = NULL;
      eptr->next = NULL;
      eptr->prev = NULL;
      XtFree(eptr);
      eptr       = eptr1;
   }
   Liftable = NULL;
   Lifentry = NULL;
   XawFormDoLayout(tform, True);
   XawFormDoLayout(lform, True);
   InitLif();
   return;
}




/* FUNCTION:  LoadIspec
   Synopsis:
     This procedure actually prints the spec file, retrieving 
     the values from the various subwindows.                  
     Called by GetFile() and MakeIntag(). 


   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

int
LoadIspec()

{
   int
      result;
   
   ISFlinecount = 0;
   result = yyparse();
#ifdef DEBUG
   DumpLiftab();
#endif
   return(result);
}
   


/* FUNCTION:  SaveIspec
   Synopsis:
     This procedure actually prints the spec file, retrieving 
     the values from the various subwindows.                  
     Called by SaveFile().                                    


   Initial purpose:
   Informal description of the initial intent of the function

   Present purpose:
   What does it really do now.

   Creator:
   Reviewer:
   Modifications:
       WHO       DATE    REASON

   Interface issues:
      FUNCTION     PURPOSE

   Operating System issues:
      CALL         PURPOSE

   Globals used:
   Globals Modified:       none
   System files modified:  none

   Return value:           void
   Range:                  none
*/

/* ************************************************************************ */

void
SaveIspec(fptr)

FILE 
    *fptr;

{
   Container
      *cptr;
   Entry
      *eptr;

   for (eptr = Liftable->next; eptr != NULL; eptr = eptr->next)
   {
      if (eptr->start == True)
      {
	 fprintf(fptr, "<%s>\t", eptr->tag);
      }
      if (eptr->end == True)
      {
	 fprintf(fptr, "</%s>\t", eptr->tag);
      }
      switch (eptr->type)
      {
      default:
	 fprintf(stderr, "WARNING: bad content type\n");
	 break;
	 
      case Mixed:
	 fprintf(fptr, ":MIXED:\t");
	 break;
	 
      case Tags:
	 fprintf(fptr, ":TAGS:\t");
	 break;
	 
      case Data:
	 fprintf(fptr, ":DATA:\t");
	 break;
      }
      if (eptr->nullflag == True)
      {
	 fprintf(fptr, ":NULL:\t");
      }
      fprintf(fptr, ":IN:\t");
      for (cptr = eptr->cptr; cptr != NULL; cptr = cptr->next)
      {
	 if (strcmp(cptr->str, TOPLEVEL) == 0)
	 {
	    fprintf(fptr, "%s\t", cptr->str);
	 }
	 else
	 {
	    fprintf(fptr, "<%s>\t", cptr->str);
	 }
      }
      fprintf(fptr, "\n");
   }
   return;
}
