/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: liftab.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/03/16 13:53:17 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	liftab.c,v $
 * Revision 1.6  92/03/16  13:53:17  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.5  1992/02/11  22:40:03  cso
 * Removed the resetting of Dirtyflag b/c it was redundant
 *
 * Revision 1.4  1992/01/28  22:36:35  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "status.h"
#include "xinterface.h"

/*  GLOBALS  */


/* *************************************************************** */
/* Liftable is a list of lif tags to be used in the ITT.  Each lif */
/* tag has a level value associated with it which indicates what   */
/* level of textag production the tag should appear in.            */
/* *************************************************************** */



void
InitLif()

/* This function initializes the Liftable pointer */

{
   Liftable             = XtNew(Entry);
   Liftable->tag        = XtNewString(TOPLEVEL);
   Liftable->nullflag   = False;
   Liftable->start      = False;
   Liftable->end        = False;
   Liftable->type       = Mixed;
   Liftable->priority   = 0;
   Liftable->next       = NULL;
   Liftable->prev       = NULL;
   Liftable->cptr       = NULL;
   Liftable->boolform   = NULL;
   Liftable->tagw       = NULL;
   Lifentry             = Liftable;
   return;
}



void
DumpLiftab()

/* This function dumps the Liftable.                                  */
/* Best when called from the root production of icomp.yacc.           */

{
   Container
      *cptr;
   Entry
      *eptr;

   fprintf(stderr, "Dump of Liftable.\n");
   for (eptr = Liftable; eptr != NULL; eptr = eptr->next)
   {
      fprintf(stderr, "tag = [%s]\n  start = %d end = %d nullflag = %d\n",
	      eptr->tag, eptr->start, eptr->end, eptr->nullflag);
      fprintf(stderr, "  ConType = %d\tPriority = %d\n",
	      eptr->type, eptr->priority);
      fprintf(stderr, "  Containers = ");
      for (cptr = eptr->cptr; cptr != NULL; cptr = cptr->next)
      {
	 fprintf(stderr, "%s ", cptr->str);
      }
      fprintf(stderr, "\n");
   }
   fprintf(stderr, "End of dump of Liftable.\n");
}



/*
   FUNCTION: 		StripTagString

   Synopsis: 
	Strip the tag down to the identifier.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


String
StripTagString(tagstr)

String
   tagstr;

{
   String
      tmpstr;

#ifdef DEBUG
   fprintf(stderr, "Stripped string [%s] to ", tagstr);
#endif
   if (strcmp(tagstr, TOPLEVEL) == 0)
   {
      return(tagstr);
   }
   
   if (tagstr[1] == '/')
   {
      tmpstr = &tagstr[2];
   }
   else
   {
      tmpstr = &tagstr[1];
   }
   tmpstr[strlen(tmpstr) - 1] = '\0';
#ifdef DEBUG
   fprintf(stderr, "[%s]\n", tmpstr);
#endif
   return(tmpstr);
}



/*
   FUNCTION: 		InstallTag

   Synopsis: 
	Put it in the appropriate data structure(s).

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
InstallTag(tagstr, contype, nullval, start, end)

String
   tagstr;
Content
   contype;
int
   nullval;
Boolean
   start,
   end;

{
   Boolean
      nullflag;
   Entry
      *eptr;
   
   tagstr = StripTagString(tagstr);
   if (nullval == 1)
   {
      nullflag = True;
   }
   else
   {
      nullflag = False;
   }
   
#ifdef DEBUG
   fprintf(stderr, "Installing tagstr [%s]\n", tagstr);
#endif
   for (eptr = Liftable; eptr != NULL; eptr = eptr->next)
   {
      Lifentry = eptr;
      if (strcmp(tagstr, eptr->tag) == 0)
      {
	 if ((eptr->start == start) && (eptr->end == end))
	 {
	     XicaStatus(Statw,
			"WARNING: Duplicate tag definition found [%s]\n",
			tagstr);
	     XicaStatus(Statw, "\tCollapsing level specifications\n");
	     if ((nullflag == True) && (eptr->nullflag == False))
	     {
		eptr->nullflag = True;
	     }
	     return;
	  }
      }
   }

   if (eptr == NULL)
   {
      eptr           = XtNew(Entry);
      eptr->tag      = XtNewString(tagstr);
      eptr->nullflag = nullflag;
      eptr->start    = start;
      eptr->end      = end;
      eptr->type     = (Content) contype;
      eptr->priority = -1;
      eptr->boolform = NULL;
      eptr->tagw     = NULL;
      eptr->cptr     = NULL;
      if (Lifentry != NULL)
      {
	 Lifentry->next = eptr;
      }
      eptr->prev     = Lifentry;
      eptr->next     = NULL;
      Lifentry       = eptr;
   }
   return;
}




/*
   FUNCTION: 		GetEntry

   Synopsis: 
	Given a tag, retrieve the pointer in the data structure.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/



Entry *
GetEntry(str)

String
   str;

{
   Entry
      *eptr;
   
   for (eptr = Liftable; eptr != NULL; eptr = eptr->next)
   {
      if (strcmp(str, eptr->tag) == 0)
      {
	 return(eptr);
      }
   }
   return(NULL);
}




/*
   FUNCTION: 		InstallLevel

   Synopsis: 
	Install a level value for prioritizing reduce/reduce conflicts.

   Initial purpose:
	Same as above.

   Present purpose:
	Same as above.


   Creator:  Conleth O'Connell, Julie Barnes
   Reviewer:

   Modifications:
       WHO       DATE    REASON

   Globals used:           
	none

   Globals Modified:
	none

   Return value:           void

   Range:                  none
*/


void
InstallLevel(str)

String
   str;

{
   Container
      *cptr,
      *cptr1;
   
   str = StripTagString(str);
   cptr1 = NULL; 
   for (cptr = Lifentry->cptr; cptr != NULL; cptr = cptr->next)
   {
      cptr1 = cptr;
      if (strcmp(cptr->str, str) == 0)
      {
	 break;
      }
   }
   if (cptr == NULL)
   {
      cptr       = XtNew(Container);
      cptr->str  = XtNewString(str);
      cptr->w    = NULL;
      cptr->next = NULL;
      if (cptr1 == NULL)
      {
	 Lifentry->cptr = cptr;
	 cptr->prev     = NULL;
      }
      else
      {
	 cptr1->next = cptr;
	 cptr->prev  = cptr1;
      }
   }
   return;
}



/*
   FUNCTION: 		SortLiftab

   Synopsis: 
       Sort the LIF table in alphabetical order.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Ann Barnes
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SortLiftab()

{
   Entry
      *eptr1,
      *eptr2,
      *eptr3;
   

   XicaStatus(Statw, "\n\nSorting");
   for (eptr1 = Liftable->next; eptr1 != NULL; )
   {
      XicaStatus(Statw, ".");

      /*
       * Find the smallest string in the remainder of the tags
       * If one is found (eptr3 != NULL), swap.
       */
      eptr3 = NULL;
      for (eptr2 = eptr1->next; eptr2 != NULL; eptr2 = eptr2->next)
      {
	 if (strcmp(eptr1->tag, eptr2->tag) > 0)
	 {
	    eptr3 = eptr2;
	 }
      }
      if (eptr3 != NULL)
      {
	 eptr1->prev->next = eptr3;
	 eptr3->prev->next = eptr1;

	 eptr2             = eptr1->prev;
	 eptr1->prev       = eptr3->prev;
	 eptr3->prev       = eptr2;
	 
	 if (eptr1->next != NULL)
	 {
	    eptr1->next->prev = eptr3;
	 }
	 if (eptr3->next != NULL)
	 {
	    eptr3->next->prev = eptr1;
	 }
	 
	 eptr2             = eptr1->next;
	 eptr1->next       = eptr3->next;
	 eptr3->next       = eptr2;

	 eptr1             = eptr3;
      } /***** IF a swap is to be done */
      else
      {
	 eptr1             = eptr1->next;
      } /***** ELSE no swap done */
   } /***** FOR each tag in the table */
   XicaStatus(Statw, "\n\n");

   return;
}
