/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menubar.c,v $
   Purpose:     Handle the menubar operations.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.7 $
   Date:        $Date: 92/05/29 15:33:10 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	menubar.c,v $
 * Revision 1.7  92/05/29  15:33:10  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.6  92/03/16  13:53:24  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.5  1992/02/11  22:41:01  cso
 * Commented out the help menu b/c it was not functional yet
 *
 * Revision 1.4  1992/01/28  22:36:41  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"
#include "Xica/help.h"

/*  EXTERNS  */

#include "callbacks.h"

/*  GLOBALS  */



/***********************************/
/* another one from con            */
/***********************************/

static void
CreateFileMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS]; /* List of arguments */
   int
      cnt;      /* Number of arguments */
   Widget
      loadsel,  /* Load  option widget */
      newsel,   /* New   option widget */
      quitsel,  /* Quit  option widget */
      resetsel, /* Reset option widget */
      savesel;  /* Save  option widget */
   
   /*
    * Offer RESET first, a line to separate, LOAD and SAVE,
    * another line separator, and lastly QUIT
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Reset    "); cnt++;
   resetsel = XtCreateManagedWidget("resetsel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(resetsel, XtNcallback, Reset, NULL); 

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "New      "); cnt++;
   newsel = XtCreateManagedWidget("newsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(newsel, XtNcallback, NewFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Edit     "); cnt++;
   loadsel = XtCreateManagedWidget("loadsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(loadsel, XtNcallback, GetFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Save     "); cnt++;
   savesel = XtCreateManagedWidget("savesel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(savesel, XtNcallback, SaveFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Quit     "); cnt++;
   quitsel = XtCreateManagedWidget("quitsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(quitsel, XtNcallback, Quit, NULL);

   return;
}



static void
CreateOpsMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      ops1sel,
      ops2sel;

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "Make      "); cnt++;
   ops1sel = XtCreateManagedWidget("ops1sel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(ops1sel, XtNcallback, MakeIntag, NULL); 

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel,    "Run       "); cnt++;
   ops2sel = XtCreateManagedWidget("ops2sel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(ops2sel, XtNcallback, RunIntag, NULL);
   
   return;
}



static void
CreateHelpMenu(parent)

Widget
   parent;  /* The parent of this list of menu options */

{
   Widget
      help1sel;

   help1sel = XtVaCreateManagedWidget( "help1sel",
       smeBSBObjectClass, parent,
       XtNlabel,     "Introduction",
       XtNsensitive, False,
       NULL );

   XtAddCallback( help1sel, XtNcallback, XicaHelp, "intag.hlp" );
   return;
}



/***********************************/
/* create the menus                */
/* stolen from con                 */
/***********************************/

Widget CreateMenubar( parent )
Widget parent;
{
   Widget
      menubar,
      filebut,
      filemenu,
      opsmenu,
      opsbut,
      helpbut,
      helpmenu;

   menubar = XtVaCreateManagedWidget( "menubar",
       groupWidgetClass, parent,
       XtNborderWidth, 0,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     1,
       XtNmargin,      8,
       NULL );

   /*********** File menu **************/
   filebut = XtVaCreateManagedWidget( "File",
       menuButtonWidgetClass, menubar,
       XtNlabel,       "   File   ",
       XtNmenuName,    "filemenu",
       XtNborderWidth, 1,
       XtNmargin,      1,
       XtNalignment,   gaCenter,
       NULL );

   filemenu = XtVaCreatePopupShell( "filemenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateFileMenu(filemenu);

   /*********** Tool Ops **************/
   opsbut = XtVaCreateManagedWidget( "Code",
       menuButtonWidgetClass, menubar,
       XtNlabel,       "   Code   ",
       XtNmenuName,    "opsmenu",
       XtNborderWidth, 1,
       XtNmargin,      1,
       XtNalignment,   gaCenter,
       NULL );

   opsmenu = XtVaCreatePopupShell( "opsmenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateOpsMenu(opsmenu);

#ifndef NOHELP
   helpbut = XtVaCreateManagedWidget( "Help",
       menuButtonWidgetClass, menubar,
       XtNlabel,       "   Help   ",
       XtNmenuName,    "helpmenu",
       XtNborderWidth, 1,
       XtNmargin,      1,
       XtNalignment,   gaCenter,
       NULL );

   helpmenu = XtVaCreatePopupShell( "helpmenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateHelpMenu(helpmenu);
#endif /* NOHELP */

   return(menubar);
}
