/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: print.c,v $
   Purpose:     Print out the various pieces of specifications.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/16 13:53:36 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	print.c,v $
 * Revision 1.4  92/03/16  13:53:36  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.3  1992/01/28  22:36:45  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "utils.h"
#include "xinterface.h"

/*  GLOBALS  */





void
PrintNterms(fptr)

FILE
   *fptr;

{
   Entry
      *eptr,
      *eptr1,
      *eptr2;
   String
      str;   

   for (eptr = Liftable; eptr != NULL; )
   {
      if (strcmp(eptr->tag, TOPLEVEL) == 0)
      {
	 str = "Toplevel";
      }
      else
      {
	 str = eptr->tag;
      }
      switch (eptr->type)
      {
      default:
	 break;
	 
      case Mixed:
	 if (eptr->nullflag == True)
	 {
	    fprintf(fptr, "%stextortagsS\n", str);
	 }	 
	 fprintf(fptr, "%stextortagsP\n", str);
	 fprintf(fptr, "%stextortags\n",  str);
	 fprintf(fptr, "%stags\n", str);
	 break;
	 
      case Tags:
	 if (eptr->nullflag == True)
	 {
	    fprintf(fptr, "%stagsS\n", str);
	 }	 
	 fprintf(fptr, "%stagsP\n", str);
	 fprintf(fptr, "%stags\n", str);
	 break;
	 
      case Data:
	 if (eptr->nullflag == True)
	 {
	    fprintf(fptr, "%stextS\n", str);
	 }	 
	 fprintf(fptr, "%stextP\n", str);
	 break;
      }
      (void) GetDuplicateEntries(eptr, &eptr1, &eptr2);
      eptr = NextUniqueTag(eptr, eptr1, eptr2);
   }
   return;
}



static void
PrintNullProd(fptr, eptr)

FILE
   *fptr;
Entry
   *eptr;

{
   switch (eptr->type)
   {
   default:
      break;
      
   case Mixed:
      fprintf(fptr, "%stextortagsS\t: %stextortagsP\n", eptr->tag, eptr->tag);
      break;
      
   case Tags:
      fprintf(fptr, "%stagsS\t: %stagsP\n", eptr->tag, eptr->tag);
      break;
      
   case Data:
      fprintf(fptr, "%stextS\t: %stextP\n", eptr->tag, eptr->tag);
      break;
   }
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t| /* null content */\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
   fprintf(fptr, "\t\t}");
   fprintf(fptr, "\n\t\t;\n\n");
   return;
}




static void
PrintPlusProd(fptr, eptr)

FILE
   *fptr;
Entry
   *eptr;

{
   String
      str;
   
   if (strcmp(eptr->tag, TOPLEVEL) == 0)
   {
      str = "Toplevel";
   }
   else
   {
      str = eptr->tag;
   }
   
   switch (eptr->type)
   {
   default:
      break;
      
   case Mixed:
      fprintf(fptr, "%stextortagsP\t: %stextortags %stextortagsP\n",
	      str, str, str);
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
      fprintf(fptr, "\t\t}\n");
      fprintf(fptr, "\t\t| %stextortags\n", str);
      break;
      
   case Tags:
      fprintf(fptr, "%stagsP\t: %stags %stagsP\n",
	      str, str, str);
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
      fprintf(fptr, "\t\t}\n");
      fprintf(fptr, "\t\t| %stags\n", str);
      break;
      
   case Data:
      fprintf(fptr, "%stextP\t: textplus\n", str);
      break;
   }
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t;\n\n");
   return;
}




static void
PrintContent(fptr, eptr)

FILE
   *fptr;
Entry
   *eptr;

{
   fprintf(fptr, "%s", eptr->tag);
   switch (eptr->type)
   {
   default:
      break;
      
   case Mixed:
      fprintf(fptr, "textortags");
      break;
      
   case Tags:
      fprintf(fptr, "tags");
      break;
      
   case Data:
      fprintf(fptr, "text");
      break;
   }
   if (eptr->nullflag == True)
   {
      fprintf(fptr, "S");
   }
   else
   {
      fprintf(fptr, "P");
   }
   return;
}




static void
PrintTagProds(fptr, eptr)

FILE
   *fptr;
Entry
   *eptr;

{
   int
      arg;
   
   if (eptr == NULL)
   {
      return;
   }

   arg = 1;
   if (eptr->start == True)
   {
      fprintf(fptr, "%sa ", eptr->tag);
      arg++;
   }
   PrintContent(fptr, eptr);
   if (eptr->end == True)
   {
      fprintf(fptr, " %sz", eptr->tag);
   }
   fprintf(fptr, "\n\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat(");
   fprintf(fptr, "structnew(\"<%s>\"), ",  eptr->tag);
   fprintf(fptr, "$%d, ", arg);
   fprintf(fptr, "structnew(\"</%s>\"), ", eptr->tag);
   fprintf(fptr, "NULL);\n");
   fprintf(fptr, "\t\t}\n");
   return;
}




void
PrintProds(fptr)

FILE
   *fptr;

/* Function that prints the list of extra nonterminals and the        */
/* productions of the yacc spec for an intag program, based on the    */
/* info in Liftable.                                                  */
/* Called from yyparse (icomp.yacc).                                  */

{
   Boolean
      firstprod;
   Container
      *cptr;
   Entry
      *eptr,
      *eptr1,
      *eptr2,
      *eptr3,
      *eptr4;
   int
      i,
      max;

   for (eptr1 = Liftable; eptr1 != NULL; )
   {
      if (eptr1->nullflag == True)
      {
	 PrintNullProd(fptr, eptr1);
      }
      PrintPlusProd(fptr, eptr1);
      firstprod = True;
      for (eptr2 = Liftable; eptr2 != NULL; eptr2 = eptr2->next)
      {
	 for (cptr = eptr2->cptr; cptr != NULL; cptr= cptr->next)
	 {
	    if ((eptr1->type == Mixed) || (eptr1->type == Tags))
	    {
	       if (strcmp(eptr1->tag, cptr->str) == 0)
	       {
		  if (firstprod == True)
		  {
		     if (strcmp(eptr1->tag, TOPLEVEL) == 0)
		     {
			fprintf(fptr, "Topleveltags\t: ");
		     }
		     else
		     {
			fprintf(fptr, "%stags\t: ", eptr1->tag);
		     }
		     if (eptr1->type == Tags)
		     {
			fprintf(fptr, "ICA_PROC_IN\n\t\t{\n");
			fprintf(fptr, "\t\t  $$ = structconcat(");
			fprintf(fptr, "structnew($1), NULL);\n");
			fprintf(fptr, "\t\t}\n");
			fprintf(fptr, "\t\t| ");
		     }
		     firstprod = False;
		  }
		  else
		  {
		     fprintf(fptr, "\t\t| ");
		  }
		  (void) GetDuplicateEntries(eptr2, &eptr3, &eptr4);
		  PrintTagProds(fptr, eptr2);
		  if (eptr3 != NULL)
		  {
		     fprintf(fptr, "\t\t| ");
		     PrintTagProds(fptr, eptr3);
		     eptr2 = eptr3;
		  }
		  if (eptr4 != NULL)
		  {
		     fprintf(fptr, "\t\t| ");
		     PrintTagProds(fptr, eptr4);
		     eptr2 = eptr4;
		  }
		  break;
	       }
	    }
	 }
      }
      if (eptr1->type != Data)
      {
	 fprintf(fptr, "\t\t;\n\n");
      }
      (void) GetDuplicateEntries(eptr1, &eptr3, &eptr4);
      eptr1 = NextUniqueTag(eptr1, eptr3, eptr4);
   }
   
   max = GetMaxPriority();
   for (i = max; i > 0; i--)
   {
      for (eptr = Liftable; eptr != NULL; )
      {
	 if (eptr->priority == i)
	 {
	    if (eptr->type == Mixed)
	    {
	       fprintf(fptr, "%stextortags\t: ", eptr->tag);
	       fprintf(fptr, "%stags\n", eptr->tag);
	       fprintf(fptr, "\t\t{\n");
	       fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	       fprintf(fptr, "\t\t}\n");
	       fprintf(fptr, "\t\t| ");
	       fprintf(fptr, "textplus\n");
	       fprintf(fptr, "\t\t{\n");
	       fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	       fprintf(fptr, "\t\t}");
	       fprintf(fptr, "\n\t\t;\n\n");
	    }
	    (void) GetDuplicateEntries(eptr, &eptr3, &eptr4);
	    eptr = NextUniqueTag(eptr, eptr3, eptr4);
	 }
	 else
	 {
	    eptr = eptr->next;
	 }
      }
   }
   return;
}



void
PrintTokens(fptr)

FILE
   *fptr;

/* Function to print the list of token names for the yacc spec.       */
/* Called from yyparse (icomp.yacc).                                  */

{
   Boolean
      both;
   Entry
      *eptr,
      *eptr1,
      *eptr2;

   eptr = Liftable->next;
   while (eptr != NULL)
   {
      both = False;
      if (GetDuplicateEntries(eptr, &eptr1, &eptr2) == True)
      {
	 both = True;
      }
      else
      {
	 if (eptr->start == eptr->end)
	 {
	    both = True;
	 }
      }
      if (both == True)
      {
	 fprintf(fptr, "%sa\n", eptr->tag);
	 fprintf(fptr, "%sz\n", eptr->tag);
      }
      else if (eptr->start == True)
      {
	 fprintf(fptr, "%sa\n", eptr->tag);
      }
      else
      {
	 fprintf(fptr, "%sz\n", eptr->tag);
      }
      eptr = NextUniqueTag(eptr, eptr1, eptr2);
   }
   return;
}



void
PrintTags(fptr)

FILE
   *fptr;

/* Function to print the productions for the lex spec.                */
/* Called from yyparse (icomp.yacc).                                  */

{
   Boolean
      both;
   Entry
      *eptr,
      *eptr1,
      *eptr2;

   eptr = Liftable->next;
   while (eptr != NULL)
   {
      both = False;
      if (GetDuplicateEntries(eptr, &eptr1, &eptr2) == True)
      {
	 both = True;
      }
      else
      {
	 if (eptr->start == eptr->end)
	 {
	    both = True;
	 }
      }
      if (both == True)
      {
	 fprintf(fptr, "\"<%s>\"\t{return(%sa);}\n", eptr->tag, eptr->tag);
	 fprintf(fptr, "\"</%s>\"\t{return(%sz);}\n\n", eptr->tag, eptr->tag);
      }
      else if (eptr->start == True)
      {
	 fprintf(fptr, "\"<%s>\"\t{return(%sa);}\n\n", eptr->tag, eptr->tag);
      }
      else
      {
	 fprintf(fptr, "\"</%s>\"\t{return(%sz);}\n\n", eptr->tag, eptr->tag);
      }
      eptr = NextUniqueTag(eptr, eptr1, eptr2);
   }
   return;
}
