/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: tagedit.c,v $
   Purpose:     Create the tag editor interface.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/05/29 15:33:20 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	tagedit.c,v $
 * Revision 1.8  92/05/29  15:33:20  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.7  92/03/16  13:54:01  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.7  92/03/05  18:24:18  boyd
 * Finished boxing changes.
 * 
 * Revision 1.6  92/03/05  18:02:05  boyd
 * Mostly as received from cso.  Pete has made some changes for boxing
 * label/buttons, etc.  I will finish shortly.
 * 
 * Revision 1.5  1992/02/11  21:51:44  cso
 * Use of delaying form layout to speed interface changes
 *
 * Revision 1.4  1992/01/28  22:36:59  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "callbacks.h"
#include "status.h"
#include "utils.h"
#include "xinterface.h"

/*  GLOBALS  */



void
ChangeLabel(w, str)

Widget
   w;
String
   str;

{
   Arg
      args[MAXARGS];
   int
      cnt;

   XtUnmanageChild(w);
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, str); cnt++;
   XtSetValues(w, args, cnt);
   XtManageChild(w);
   return;
}




Widget
CreateAttWidgets(eptr, parent, top)

Entry
   *eptr;
Widget
   parent,
   top;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      formw,
      togw;
   
   formw = XtVaCreateWidget( "boolform",
       viewformWidgetClass, parent,
       XtNfromVert,        top,
       XtNtop,             XtChainTop,
       XtNbottom,          XtChainTop,
       XtNleft,            XtChainLeft,
       XtNright,           XtChainLeft,
       XtNdefaultDistance, 0,
       XtNresizable,       True,
       XtNborderWidth,     0,
       NULL );

   togw = XtVaCreateManagedWidget( "start",
       toggleWidgetClass, formw,
       XtNstate,       eptr->start,
       XtNlabel,       "S",
/*
       XtNborderWidth, 0,
*/
       NULL );
   XtAddCallback(togw, XtNcallback, AttSet, eptr->tag);

   togw = XtVaCreateManagedWidget( "end",
       toggleWidgetClass, formw,
       XtNfromHoriz,     togw,
       XtNstate,         eptr->end,
       XtNlabel,         "E",
       XtNhorizDistance, 10,
/*
       XtNborderWidth,   0,
*/
       NULL );
   XtAddCallback(togw, XtNcallback, AttSet, eptr->tag);

   cnt = 0;
   togw = XtVaCreateManagedWidget( "null",
       toggleWidgetClass, formw,
       XtNfromHoriz,     togw,
       XtNstate,         eptr->nullflag,
       XtNlabel,         "N",
       XtNhorizDistance, 10,
/*
       XtNborderWidth,   0,
*/
       NULL );
   XtAddCallback(togw, XtNcallback, AttSet, eptr->tag);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromHoriz,     togw);         cnt++;
   XtSetArg(args[cnt], XtNhorizDistance, 10);           cnt++;
   XtSetArg(args[cnt], XtNborderWidth,   0);            cnt++;
   switch (eptr->type)
   {
   default:
      break;
      
   case Mixed:
      XtSetArg(args[cnt], XtNlabel, "Both"); cnt++;
      break;
      
   case Tags:
      XtSetArg(args[cnt], XtNlabel, "Tags"); cnt++;
      break;
      
   case Data:
      XtSetArg(args[cnt], XtNlabel, "Data"); cnt++;
      break;
   }
   togw = XtCreateManagedWidget( "content",
       labelWidgetClass, formw, args, cnt );

   XtManageChild(formw);

   XtUnmanageChild(formw);
   XtVaSetValues( formw, XtNresizable, False, NULL );
   XtManageChild(formw);

   return(formw);
}



void
UpdateEditor()

{
   Arg
      args[MAXARGS];
   Container
      *cptr;
   Entry
      *eptr;
   int
      cnt;
   static String 
      menutransstr = "<Btn1Down>: set() notify() PopupMenu() unset()";
   Widget
      lvlhoriz,
      lvlform,
      tagvert,
      tagform;
   XtTranslations 
      menutrans;
   Pixel
      background;

   

   FixDuplicates();
   
   if (Liftable == NULL)
   {
      return;
   }
   
   XtVaGetValues( Frame, XtNbackground, &background, NULL );

   lvlform = XtNameToWidget(Frame, "*lvlform");
   tagform = XtNameToWidget(Frame, "*tagform");
   XawFormDoLayout(lvlform, False);
   XawFormDoLayout(tagform, False);
   XtUnmanageChild(lvlform);
   XtUnmanageChild(tagform);
   tagvert = NULL;
   menutrans = XtParseTranslationTable(menutransstr);
   XicaStatus(Statw, "Creating widgets");
   
   for (eptr = Liftable->next; eptr != NULL; eptr = eptr->next)
   {
      XicaStatus(Statw, ".");
      eptr->boolform = CreateAttWidgets(eptr, tagform, tagvert);

      cnt = 0;
      XtSetArg(args[cnt], XtNfromHoriz,   eptr->boolform); cnt++;
      XtSetArg(args[cnt], XtNfromVert,    tagvert);        cnt++;
      XtSetArg(args[cnt], XtNtop,         XtChainTop);     cnt++;
      XtSetArg(args[cnt], XtNbottom,      XtChainTop);     cnt++;
      XtSetArg(args[cnt], XtNleft,        XtChainLeft);    cnt++;
      XtSetArg(args[cnt], XtNright,       XtChainLeft);    cnt++;
      XtSetArg(args[cnt], XtNmenuName,    "tagmenu");      cnt++;
      XtSetArg(args[cnt], XtNresizable,   False);          cnt++;
      eptr->tagw = XtCreateManagedWidget(eptr->tag, menuButtonWidgetClass,
					 tagform, args, cnt);
      XtAddCallback(eptr->tagw, XtNcallback, SetLifentry, eptr);

      XtOverrideTranslations(eptr->tagw, menutrans);

      lvlhoriz = NULL;
      for (cptr = eptr->cptr; cptr != NULL; cptr = cptr->next)
      {
	 cnt = 0;
	 if (eptr == Liftable->next)
	 {
	    XtSetArg(args[cnt], XtNfromVert,  NULL);
	 }
	 else
	 {
	    XtSetArg(args[cnt], XtNfromVert,  eptr->prev->cptr->w);
	 }
	 cnt++;
	 XtSetArg(args[cnt], XtNtop,         XtChainTop);  cnt++;
	 XtSetArg(args[cnt], XtNbottom,      XtChainTop);  cnt++;
	 XtSetArg(args[cnt], XtNleft,        XtChainLeft); cnt++;
	 XtSetArg(args[cnt], XtNright,       XtChainLeft); cnt++;
	 XtSetArg(args[cnt], XtNfromHoriz,   lvlhoriz);    cnt++;
	 XtSetArg(args[cnt], XtNlabel,       cptr->str);   cnt++;
         XtSetArg(args[cnt], XtNborderColor, background);  cnt++;
	 cptr->w = XtCreateManagedWidget(cptr->str, labelWidgetClass,
					 lvlform, args, cnt);
	 lvlhoriz = cptr->w;
      }
      tagvert = eptr->tagw;
   }
   XicaStatus(Statw, "\n");

   XawFormDoLayout(lvlform, True);
   XawFormDoLayout(tagform, True);
   XtManageChild(lvlform);
   XtManageChild(tagform);

   return;
}


Widget CreateTagEditor( parent )
Widget parent;
{
   Widget
      editform, labelrow, viewrow, tagcol, lvlcol,
      taglabel,
      lvllabel,
      tagview,
      lvlview,
      tagform,
      lvlform,
      vert,
      tagmenu,
      dupsel,
      remsel;
   
   tagmenu = XtVaCreatePopupShell( "tagmenu",
       simpleMenuWidgetClass, parent,
       NULL );

   dupsel = XtVaCreateManagedWidget( "dupsel",
       smeBSBObjectClass, tagmenu,
       XtNlabel, "Duplicate",
       NULL );
   XtAddCallback( dupsel, XtNcallback, Duplicate, NULL ); 

   remsel = XtVaCreateManagedWidget( "remsel",
       smeBSBObjectClass, tagmenu,
       XtNlabel, "Remove",
       NULL );
   XtAddCallback(remsel, XtNcallback, Remove, NULL);

   editform = XtVaCreateManagedWidget( "editform",
       tableWidgetClass, parent,
       XtNhorzSpacing,   8,
       XtNhorzPlacement, tpFlush,
       XtNalignment,     gaFill,
       XtNmargin,        8,
       XtNstretch,       True,
       NULL );
   labelrow = XtVaCreateWidget( "labelrow",
       tableRowObjectClass, editform,
       NULL );
   viewrow = XtVaCreateWidget( "viewrow",
       tableRowObjectClass, editform,
       XtNstretch, True,
       NULL );
   tagcol = XtVaCreateWidget( "tagcol",
       tableColObjectClass, editform,
       NULL );
   lvlcol = XtVaCreateWidget( "lvlcol",
       tableColObjectClass, editform,
       NULL );

   taglabel = XtVaCreateManagedWidget( "taglabel",
       labelWidgetClass, editform,
       XtNlabel,       "Tags:",
       XtNborderWidth, 0,
       XtNtableRow,    labelrow,
       XtNtableCol,    tagcol,
       NULL );

   tagview = XtVaCreateManagedWidget( "tagview",
       viewportWidgetClass, editform,
       XtNforceBars,     True,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNpreference,    False,
/*
       XtNprefWidth,     300,
       XtNminWidth,      250,
       XtNprefHeight,    250,
       XtNminHeight,     100,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      tagcol,
       NULL );
   
   vert = XtNameToWidget(tagview, "vertical");
   XtAddCallback(vert, XtNscrollProc, ScrollTogether, NULL);
   XtAddCallback(vert, XtNjumpProc,   ScrollTogether, NULL);

   tagform = XtVaCreateManagedWidget( "tagform",
       viewformWidgetClass, tagview,
       XtNresizable, True,
       NULL );
   
   lvllabel = XtVaCreateManagedWidget( "lvllabel",
       labelWidgetClass, editform,
       XtNlabel,       "Hierarchical Levels:",
       XtNborderWidth, 0,
       XtNtableRow,    labelrow,
       XtNtableCol,    lvlcol,
       NULL );

   lvlview = XtVaCreateManagedWidget( "lvlview",
       viewportWidgetClass, editform,
       XtNforceBars,     True,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNuseRight,      True,
       XtNpreference,    False,
/*
       XtNprefWidth,     350,
       XtNminWidth,      100,
       XtNprefHeight,    250,
       XtNminHeight,     100,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      lvlcol,
       NULL );

   vert = XtNameToWidget(lvlview, "vertical");
   XtAddCallback(vert, XtNscrollProc, ScrollTogether, NULL);
   XtAddCallback(vert, XtNjumpProc,   ScrollTogether, NULL);

   XtUnmanageChild(vert);
   XtVaSetValues( vert,
       XtNwidth,       1,
       XtNborderWidth, 0,
       NULL );

   lvlform = XtVaCreateManagedWidget( "lvlform",
       viewformWidgetClass, lvlview,
       XtNresizable, True,
       NULL );

   return(editform);
}
