/* CONTENTS
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: utils.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.10 $
   Date:        $Date: 92/04/27 20:22:28 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   $Id: utils.c,v 1.10 92/04/27 20:22:28 ware Exp $
   HISTORY
   $Log:	utils.c,v $
 * Revision 1.10  92/04/27  20:22:28  ware
 * Get rid of boxes around duplications in the level window. -- John B.
 * 
 * Revision 1.9  92/03/24  11:09:31  ware
 * Fixing a glitch ala Con
 * 
 * Revision 1.8  92/03/16  13:54:12  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.7  1992/03/10  19:56:20  cso
 * Removed (un)managing calls; Fixed an infinite loop for priority checking
 *
 * Revision 1.6  1992/02/11  21:51:40  cso
 * Use of delaying form layout to speed interface changes
 *
 * Revision 1.5  1992/01/28  22:37:05  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "callbacks.h"
#include "liftab.h"
#include "status.h"
#include "tagedit.h"
#include "xinterface.h"

/*  GLOBALS  */




Entry *
FindEntry(tagstr, w)

String
   tagstr;
Widget
   w;

{
   Entry
      *eptr;

   for (eptr = Liftable; eptr != NULL; eptr = eptr->next)
   {
      if ((strcmp(eptr->tag, tagstr) == 0) && (eptr->boolform == w))
      {
	 return(eptr);
      }
   }
   return(NULL);
}




int
GetMaxPriority()

{
   Entry
      *eptr;
   int
      max;
   
   max = 0;
   for (eptr = Liftable; eptr != NULL; eptr = eptr->next)
   {
      if (max < eptr->priority)
      {
	 max = eptr->priority;
      }
   }
   return(max);
}




Boolean
GetDuplicateEntries(eptr1, eptr2, eptr3)

Entry
   *eptr1,
   **eptr2,
   **eptr3;

{
   Boolean
      dupflag;
   Entry
      *eptr;
   
   dupflag = False;
   *eptr2  = NULL;
   *eptr3  = NULL;
   for (eptr = eptr1->prev; eptr != NULL; eptr = eptr->prev)
   {
      if (strcmp(eptr->tag, eptr1->tag) == 0)
      {
	 if (*eptr2 == NULL)
	 {
	    *eptr2  = eptr;
	    dupflag = True;
	 }
	 else if (*eptr3 == NULL)
	 {
	    *eptr3  = eptr;
	 }
      }
      else
      {
	 break;
      }
   }
   for (eptr = eptr1->next; eptr != NULL; eptr = eptr->next)
   {
      if (strcmp(eptr->tag, eptr1->tag) == 0)
      {
	 if (*eptr2 == NULL)
	 {
	    *eptr2  = eptr;
	    dupflag = True;
	 }
	 else if (*eptr3 == NULL)
	 {
	    *eptr3 = eptr;
	 }
      }
      else
      {
	 break;
      }
   }
   return(dupflag);
}




Entry *
NextUniqueTag(eptr1, eptr2, eptr3)

Entry
   *eptr1,
   *eptr2,
   *eptr3;

{
   if ((eptr1->next == NULL) ||
       (eptr1->next != eptr2) && (eptr1->next != eptr3))
   {
      return(eptr1->next);
   }
   else if ((eptr2->next == NULL) ||
	    (eptr2->next != eptr3))
   {
      return(eptr2->next);
   }
   else
   {
      return(eptr3->next);
   }
}




static void
ShiftTags(eptr1, eptr2)

Entry
   *eptr1,
   *eptr2;

{
   if ((eptr1 == NULL) || (eptr2 == NULL))
   {
      return;
   }
   if (eptr1->next != eptr2)
   {
      if (eptr2->prev != NULL)
      {
	 eptr2->prev->next = eptr2->next;
      }
      if (eptr2->next != NULL)
      {
	 eptr2->next->prev = eptr2->prev;
      }
      eptr2->next = eptr1->next;
      eptr2->prev = eptr1;
      if (eptr2->next != NULL)
      {
	 eptr2->next->prev = eptr2;
      }
      eptr1->next = eptr2;
   }
   return;
}



static void
MergeInfo(eptr1)

Entry
   *eptr1;

{
   Entry
      *eptr2;
   Container
      *cptr,
      *cptr1,
      *cptr2;
   
   if (eptr1->next != NULL)
   {
      eptr2 = eptr1->next;
   }
   else
   {
      return;
   }
   
   if ((eptr1->nullflag == True) || (eptr2->nullflag == True))
   {
      eptr1->nullflag = True;
      eptr2->nullflag = True;
   }

   cptr = eptr1->cptr;
   for (cptr1 = eptr1->cptr; cptr1 != NULL; cptr1 = cptr1->next)
   {
      for (cptr2 = eptr2->cptr; cptr2 != NULL; cptr2 = cptr2->next)
      {
	 cptr = cptr2;
	 if (strcmp(cptr1->str, cptr2->str) == 0)
	 {
	    break;
	 }
      }
      if (cptr2 == NULL)
      {
	 cptr2       = XtNew(Container);
	 cptr->next  = cptr2;
	 cptr2->prev = cptr;
	 cptr2->next = NULL;
	 cptr2->w    = NULL;
	 cptr2->str  = XtNewString(cptr1->str);
      }
   }
   
   cptr = eptr2->cptr;
   for (cptr2 = eptr2->cptr; cptr2 != NULL; cptr2 = cptr2->next)
   {
      for (cptr1 = eptr1->cptr; cptr1 != NULL; cptr1 = cptr1->next)
      {
	 cptr = cptr1;
	 if (strcmp(cptr2->str, cptr1->str) == 0)
	 {
	    break;
	 }
      }
      if (cptr1 == NULL)
      {
	 cptr1       = XtNew(Container);
	 cptr->next  = cptr1;
	 cptr1->prev = cptr;
	 cptr1->next = NULL;
	 cptr1->w    = NULL;
	 cptr1->str  = XtNewString(cptr2->str);
      }
   }
   return;   
}



void
FixDuplicates()

{
   Entry
      *eptr1,
      *eptr2;
   
   for (eptr1 = Liftable; eptr1 != NULL; eptr1 = eptr1->next)
   {
      for (eptr2 = eptr1->next; eptr2 != NULL; eptr2 = eptr2->next)
      {
	 if (strcmp(eptr1->tag, eptr2->tag) == 0)
	 {
	    if (eptr1->next != eptr2)
	    {
	       ShiftTags(eptr1, eptr2);
	       MergeInfo(eptr1);
	       eptr1 = eptr1->next;
	    }
	 }
      }
   }
   return;
}




void
SetState(w, state)

Widget
   w;
Boolean
   state;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNstate, state); cnt++;
   XtSetValues(w, args, cnt);
   return;
}




void
CheckDuplicates(eptr, eptr1, str)

Entry
   *eptr,
   *eptr1;
String
   str;

{
   switch (*str)
   {
   default:
      XicaStatus(Statw, "CheckDuplicates passed bad attribute type\n");
      break;
      
   case 'N':
      if (eptr1->nullflag != eptr->nullflag)
      {
	 eptr1->nullflag = eptr->nullflag;
	 SetState(XtNameToWidget(eptr1->boolform, "null"), eptr1->nullflag);
      }
      break;
      
   case 'S':
   case 'E':
      if ((eptr1->start == eptr->start) && (eptr1->end == eptr->end))
      {
#ifdef DEBUG
	 fprintf(stderr, "Uh-oh, a duplicate was formed\n");
#endif
	 if (*str == 'S')
	 {
	    eptr->start = !eptr->start;
	    SetState(XtNameToWidget(eptr->boolform, "start"), eptr->start);
	 }
	 else
	 {
	    eptr->end = !eptr->end;
	    SetState(XtNameToWidget(eptr->boolform, "end"), eptr->end);
	 }
      }
      break;
   }
   return;
}




static Entry *
CreateNewEntry(top)

Entry
   *top;

{
   Arg
      args[MAXARGS];
   Container
      *cptr,
      *cptr1,
      *cptr2;
   Entry
      *new;
   int
     cnt;
   static String 
      menutransstr = "<Btn1Down>: set() notify() PopupMenu() unset()";
   XtTranslations 
      menutrans;
   Widget
      lvlform,
      lvlhoriz,
      tagform;
   Pixel
      background;
   
   XtVaGetValues( Frame, XtNbackground, &background, NULL );

   tagform = XtNameToWidget(Frame, "*tagform");
   lvlform = XtNameToWidget(Frame, "*lvlform");

   new           = XtNew(Entry);
   new->tag      = XtNewString(top->tag);
   new->start    = False;
   new->end      = False;
   new->type     = top->type;
   new->nullflag = top->nullflag;
   new->priority = top->priority;
   new->cptr     = NULL;
   new->boolform = CreateAttWidgets(new, tagform, top->tagw);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromHoriz, new->boolform); cnt++;
   XtSetArg(args[cnt], XtNfromVert,  top->tagw);     cnt++;
   XtSetArg(args[cnt], XtNtop,       XtChainTop);             cnt++;
   XtSetArg(args[cnt], XtNbottom,    XtChainTop);             cnt++;
   XtSetArg(args[cnt], XtNleft,      XtChainLeft);            cnt++;
   XtSetArg(args[cnt], XtNright,     XtChainLeft);            cnt++;
   XtSetArg(args[cnt], XtNmenuName,  "tagmenu");              cnt++;
   XtSetArg(args[cnt], XtNresizable, False);                  cnt++;
   new->tagw = XtCreateManagedWidget(new->tag, menuButtonWidgetClass,
				     tagform, args, cnt);
   XtAddCallback(new->tagw, XtNcallback, SetLifentry, new);
   
   menutrans = XtParseTranslationTable(menutransstr);
   XtOverrideTranslations(new->tagw, menutrans);
   
   lvlhoriz = NULL;
   cptr2    = top->cptr;
   for (cptr = top->cptr; cptr != NULL; cptr = cptr->next)
   {
      cptr1       = XtNew(Container);
      cptr1->str  = XtNewString(cptr->str);
      cptr1->next = NULL;
      cnt = 0;
      XtSetArg(args[cnt], XtNfromVert,  cptr->w);      cnt++;
      XtSetArg(args[cnt], XtNtop,       XtChainTop);   cnt++;
      XtSetArg(args[cnt], XtNbottom,    XtChainTop);   cnt++;
      XtSetArg(args[cnt], XtNleft,      XtChainLeft);  cnt++;
      XtSetArg(args[cnt], XtNright,     XtChainLeft);  cnt++;
      XtSetArg(args[cnt], XtNfromHoriz, lvlhoriz);     cnt++;
      XtSetArg(args[cnt], XtNlabel,     cptr1->str);   cnt++;
      XtSetArg(args[cnt], XtNborderColor, background);  cnt++;
      cptr1->w = XtCreateManagedWidget(cptr1->str, labelWidgetClass,
				       lvlform, args, cnt);
      lvlhoriz = cptr1->w;
      if (new->cptr == NULL)
      {
	 new->cptr   = cptr1;
	 cptr1->prev = NULL;
      }
      else
      {
	 cptr2->next = cptr1;
	 cptr1->prev = cptr2;
      }	 
      cptr2 = cptr1;
   }
   return(new);
}




void
ResetVert(bottom, new)

Entry
   *bottom,
   *new;

{
   Arg
      args[MAXARGS];
   Container
      *cptr;
   int
      cnt;
   Widget
      boolw,
      w,
      tagw;
      

   if (bottom == NULL)
   {
      return;
   }
   
   if (new == NULL)
   {
      boolw = NULL;
      tagw  = NULL;
      w     = NULL;
   }
   else
   {
      boolw = new->boolform;
      tagw  = new->tagw;
      if (new->cptr != NULL)
      {
	 w = new->cptr->w;
      }
      else
      {
	 w = NULL;
      }
   }

   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, boolw); cnt++;
   XtSetValues(bottom->boolform, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, tagw); cnt++;
   XtSetValues(bottom->tagw, args, cnt);

   for (cptr = bottom->cptr; cptr != NULL; cptr = cptr->next)
   {
      cnt = 0;
      XtSetArg(args[cnt], XtNfromVert, w); cnt++;
      XtSetValues(cptr->w, args, cnt);
   }
   return;
}




Entry *
InsertEntry(eptr1, eptr2)

Entry
   *eptr1,
   *eptr2;

{
   Entry
      *newptr,
      *maxdup;
   
   newptr = NULL;
   if ((eptr1->next != eptr2) || (eptr2 == NULL))
   {
      maxdup = eptr1;
   }
   else
   {
      maxdup = eptr2;
   }

   newptr = CreateNewEntry(maxdup);
   newptr->next = maxdup->next;
   newptr->prev = maxdup;
   maxdup->next = newptr;
   if (newptr->next != NULL)
   {
      newptr->next->prev = newptr;
      ResetVert(newptr->next, newptr);
   }

   return(newptr);
}




void
RemoveEntry(eptr)

Entry
   *eptr;

{
   Container
      *cptr,
      *cptr1;
   Widget
      lvlform,
      tagform;
   
   lvlform = XtNameToWidget(Frame, "*lvlform");
   tagform = XtNameToWidget(Frame, "*tagform");
   XawFormDoLayout(lvlform, False);
   XawFormDoLayout(tagform, False);
   if (eptr == Liftable->next)
   {
      ResetVert(eptr->next, NULL);
   }
   else
   {
      ResetVert(eptr->next, eptr->prev);
   }
   if (eptr->next != NULL)
   {
      eptr->next->prev = eptr->prev;
   }
   
   XtFree(eptr->tag);
   if (eptr->tagw != NULL)
   {
      XtDestroyWidget(eptr->tagw);
   }
   if (eptr->boolform != NULL)
   {
      XtDestroyWidget(eptr->boolform);
   }
   if (eptr->prev != NULL)
   {
      eptr->prev->next = eptr->next;
   }
   if (eptr->next != NULL)
   {
      eptr->next->prev = eptr->prev;
   }
   eptr->tag      = NULL;
   eptr->tagw     = NULL;
   eptr->boolform = NULL;
   eptr->next     = NULL;
   eptr->prev     = NULL;
   eptr->start    = False;
   eptr->end      = False;
   eptr->nullflag = False;
   eptr->priority = 0;

   for (cptr = eptr->cptr; cptr != NULL; )
   {
      cptr1 = cptr->next;
      XtFree(cptr->str);
      if (cptr->w != NULL)
      {
	 XtDestroyWidget(cptr->w);
      }
      cptr->str  = NULL;
      cptr->w    = NULL;
      cptr->next = NULL;
      cptr->prev = NULL;
      XtFree(cptr);
      cptr       = NULL;
      cptr       = cptr1;
   }
   eptr->cptr = NULL;
   XtFree(eptr);
   XawFormDoLayout(lvlform, True);
   XawFormDoLayout(tagform, True);
   return;
}




void
CorrectPriority(eptr1)

Entry
   *eptr1;

{
   Container
      *cptr;
   Entry
      *eptr,
      *eptr3,
      *eptr4;

   if ((eptr1 == NULL) || (eptr1->priority > 0))
   {
      return;
   }
   for (cptr = eptr1->cptr; cptr != NULL; cptr = cptr->next)
   {
      for (eptr = Liftable; eptr != NULL; )
      {
	 if (strcmp(eptr->tag, cptr->str) == 0)
	 {
	    if (eptr->priority < 0)
	    {
	       CorrectPriority(eptr);
	    }
	    
	    if (eptr1->priority < (eptr->priority + 1))
	    {
	       eptr1->priority = eptr->priority + 1;
	    }
	 } /***** IF the outer entry is my parent */
	 (void) GetDuplicateEntries(eptr, &eptr3, &eptr4);
	 eptr = NextUniqueTag(eptr, eptr3, eptr4);
      } /***** FOR each table entry */
   } /***** FOR each parent of an entry */

   return;
}



void
UpdatePriority()

{
#ifdef OLD
   Container
      *cptr;
   Entry
      *eptr,
      *eptr1;
#endif
   Entry
      *eptr,
      *eptr3,
      *eptr4;
   
   for (eptr = Liftable; eptr != NULL; )
   {
      CorrectPriority(eptr);
      (void) GetDuplicateEntries(eptr, &eptr3, &eptr4);
      eptr = NextUniqueTag(eptr, eptr3, eptr4);
#ifdef OLD
      for (eptr1 = Liftable; eptr1 != NULL; eptr1 = eptr1->next)
      {
	 for (cptr = eptr1->cptr; cptr != NULL; cptr = cptr->next)
	 {
	    if (strcmp(eptr->tag, cptr->str) == 0)
	    {
	       if (eptr1->priority < (eptr->priority + 1))
	       {
		  eptr1->priority = eptr->priority + 1;
	       }
	    } /***** IF the outer entry is my parent */
	 } /***** FOR each parent of an entry */
      } /***** FOR each table entry */
#endif
   } /***** FOR each table entry */
#ifdef DEBUG
   DumpLiftab();
#endif
   return;
}




Boolean
CheckDependencies(str)

String
   str;

{
   Entry
      *eptr;
   Container
      *cptr;

   for (eptr = Liftable->next; eptr != NULL; eptr = eptr->next)
   {
      if (strcmp(eptr->tag, str) != 0)
      {
         for (cptr = eptr->cptr; cptr != NULL; cptr = cptr->next)
         {
            if (strcmp(str, cptr->str) == 0)
            {
               return(True);
            }
         }
      }
   }
   return(False);
}
