/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: xinterface.c,v $
   Purpose:
       To call the appropriate create routines and handle the "look"
       of the interface.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.9 $
   Date:        $Date: 92/05/29 15:33:31 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  QuickQuit

   EXPORT  CreateInterface
					   CreateFileIOShell
*/

/* HISTORY
   $Log:	xinterface.c,v $
 * Revision 1.9  92/05/29  15:33:31  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table, and Viewform.
 * 
 * 
 * Revision 1.8  92/03/16  13:54:24  ware
 * Make titles consistent.  Fix some consistency problems.
 * 
 * Revision 1.6  1992/02/11  21:42:28  cso
 * Declaration of gloabl Dirtyflag
 *
 * Revision 1.5  1992/02/06  05:16:10  cso
 * Changed the parent of the file dialog for proper positioning
 *
 * Revision 1.4  1992/01/28  22:37:10  cso
 * Restructured dependencies
 *
 * Revision 1.3  1992/01/02  23:38:20  cso
 * Upgraded to the Xica widget set from Xlwh.  Using Message boxes
 * instead of the alert shells, and the new file dialog.
 *
 * Revision 1.2  1991/11/27  16:05:20  cso
 * Mostly aesthetic, minor changes including:
 *  - Adding the status widget
 *  - Changing interface resources
 *  - Using a maxarg variable
 *
 * Revision 1.1.1.3  1991/10/04  20:19:09  jbm
 * First HaL mods to ICA
 *
 * Revision 2.0  91/03/30  13:49:35  cso
 * This module is now in Alpha Release form.
 * 
 * Revision 1.18  91/03/30  13:29:14  cso
 * Taking advantage of XtInstallAllAccelerators.
 * Added the install routines for the symbol and occurrence types.
 * 
 * Revision 1.17  91/03/28  14:44:42  cso
 * Added the AddActions and pixmap creation to CreateInterface.
 * 
 * Revision 1.16  91/02/22  09:44:45  cso
 * Completed documentation.
 * Changed the name of quit2 to QuickQuit.
 * Installed keyboard accelerators for navigating.
 * 
 * Revision 1.15  90/11/26  13:14:53  cso
 * Cosmetic and lint modifications.
 * 
 * Revision 1.14  90/11/05  18:19:25  cso
 * Added function template to all functions, but still need to fill it in.
 * Only allow QuickQuit if debugging. Else, cosmetics.
 * 
 * Revision 1.13  90/10/29  10:23:30  cso
 * Removed CreateViewShell, and placed the call in semmenu.c
 * 
 * Revision 1.12  90/10/24  15:34:28  cso
 * Added ifdef SYNCED for debugging purposes.
 * 
 * Revision 1.11  90/10/04  15:56:24  cso
 * Cosmetic changes.
 * 
 * Revision 1.10  90/10/02  13:34:55  cso
 * Cosmetic changes.
 * 
 * Revision 1.9  90/09/07  09:39:58  cso
 * Cosmetics.
 * 
 * Revision 1.8  90/09/03  18:24:42  cso
 * Function name changes.
 * 
 * Revision 1.7  90/09/03  14:33:31  cso
 * Changed function names to be more consistent.
 * 
 * Revision 1.6  90/08/29  17:48:10  cso
 * changed create_attdisplay to create_attribshell because of the
 * migration to popups.
 * Also,
 *  rearranged the look of the interface.
 * 
 * Revision 1.5  90/08/26  17:41:22  cso
 * Comment all calls RE: auxiliary information.
 * 
 * Revision 1.4  90/08/26  14:13:19  cso
 * added module template
 * 
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"


/*  EXTERNALS */

#include "fileio.h"
#include "liftab.h"
#include "menubar.h"
#include "signal.h"
#include "status.h"
#include "tagedit.h"

/* GLOBALS */

int
   ICAlen,
   ISFlinecount;  /* line counter for the isf compiler */
Entry
   *Lifentry,     /* Running pointer */
   *Liftable;     /* Head pointer into the table */
String
   ICApath;
Widget
   Frame;         /* toplevel widgets */

static char
   *Fallbacks[] = {
   "intag.title:	Insert Tags",
   "intag.iconName:	Insert Tags",
   /*
      Sizing
   */
   "*tagview.prefWidth:   300",
   "*tagview.minWidth:    100",
   "*tagview.prefHeight:  250",
   "*tagview.minHeight:   50",
   "*tagcol.stretch:      False",
   "*lvlview.prefWidth:   350",
   "*lvlview.minWidth:    100",
   "*lvlview.prefHeight:  250",
   "*lvlview.minHeight:   50",
   "*lvlcol.stretch:      True",
   "*Statw.prefHeight:    96",
   "*Statw.minHeight:     20",
   "*Statw.stretch:       False",
   "*statframe.stretch:   False",
   (char *) NULL
   };



/*
   FUNCTION: 		QuickQuit

   Synopsis: 
       Create a quick quit during development.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   System files modified:
	none

   Return value:	void

   Range:		none
*/

static void
QuickQuit(w, client_data, event)

Widget
   w;           /* Unused */
caddr_t
   client_data; /* Unused */
XEvent
   *event;      /* The event causing this routine to be called */

{
   int
      i;        /* Count of characters being returned */
   char
      text[10]; /* String of characters buffered */
   KeySym
      keys;     /* Key structure */
   
   /*
    * Find the string of characters being typed by the user
    * If the user hit a 'Q' or 'q', exit out of the environment
    */
   i = XLookupString(event, text, 10, &keys, 0);
   if ((i == 1) && ((text[0] == 'Q') || (text[0] == 'q')))
   {
      exit(0);
   }
   return;
}




/*
   FUNCTION: 		CreateInterface

   Synopsis: 
       Call all of the routines to create the complete interface.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91/03/22   cso    Added the creation of Threed Pixmap
       91/03/29   cso    Taking advantage of InstallAllAccelerators
                         Added accelerators to the occurrences and
			  symbol types.
   Globals used:
       Treeshell  To create the global pixmap

   Globals Modified:
       App_Con     It's created
       Frame       It's created
       Toplevel    It's created
       Threed      It's created

   Return value:	void

   Range:		none
*/

static XtAppContext CreateInterface( argc, argv )
int
   argc;   /* The number of variable args being passed in */
char
   **argv; /* The list of string options being passed in */
{
   Widget
      toplevel,
      vertw;
   XtAppContext
      appcon;

   /*
    * Create the toplevel shell and initialize the application
    * context, then add some resources.
    */
   toplevel = XtVaAppInitialize(
       &appcon, TITLE, NULL, 0, &argc, argv, Fallbacks,
       XtNinput,            True,
#ifdef HARDCODEDTITLE
       XtNtitle,            TITLE,
       XtNiconName,         TITLE,
#endif
       XtNallowShellResize, True,
       NULL );

   XawSimpleMenuAddGlobalActions(appcon);
#ifdef SYNC
   /*
    * Go into synchronize mode for debugging
    */
   XSynchronize(XtDisplay(toplevel), True);
#endif

   /*
    * Create the Frame that contains the pieces of the main widget
    */
   Frame = XtVaCreateManagedWidget( "Frame",
       groupWidgetClass, toplevel,
       XtNspacing, 8,
       NULL );

   /*
    * Create shells for alerting the user, getting symbol 
    * attributes, and doing file I/O
    */
   CreateFileIOShell(toplevel);

   /*
    * Create the menubar
    * Create the tag editor
    * Realize the interface
    * Install keyboard accelerators
    */

   CreateMenubar(Frame);
   CreateTagEditor(Frame);
   CreateStatus(Frame);

   XtRealizeWidget(toplevel);

   XtInstallAllAccelerators(Frame, Frame);

#ifdef DEBUG
   /*
    * Add the event handler for the quick quit
    */
   XtAddEventHandler(Frame, KeyPressMask, False, QuickQuit, NULL);
#endif

   return(appcon);
}




/*
   FUNCTION:  main 		

   Synopsis: ITT X widget main program

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Ann Barnes
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


/* ************************************************************************ */
void
main( argc, argv )
int argc;
char *argv[];
{
   XtAppContext
      appcon;
   
   ICApath = XtNewString(getenv("ICAROOT"));
#ifdef DEBUG
   fprintf(stderr, "ICAROOT = [%s]\n", ICApath);
#endif

   ICAlen = strlen(ICApath);
   InstallSignalHandlers();

   appcon = CreateInterface( argc, argv );

   if (ICApath == NULL)
   {
      ICApath = ".";
      XicaStatus(Statw, "WARNING: ICAROOT is not set\n");
      XicaStatus(Statw, "Using current directory\n");
   }

   InitLif();

   XtAppMainLoop(appcon);
}
