/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: alpha.c,v $
   Purpose:     To create the main interface with the alphabet info.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.13 $
   Date:        $Date: 92/05/29 15:27:34 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	alpha.c,v $
 * Revision 1.13  92/05/29  15:27:34  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.12  92/03/16  15:00:35  ware
 * Make titles consistent.
 * 
 * Revision 1.11  1992/02/06  04:54:11  cso
 * Changed parent for positioning of file dialog.
 * Changed the labels of the alphabets to be more descriptive.
 *
 * Revision 1.10  1992/01/30  14:15:10  cso
 * Added globals: Background and Foreground for easier focus display.
 * Now, the focus is initialized to the data alphabet.  The borderwidth
 * is always 2, and the colors are changed from background to foreground
 * to show which widget has the focus (no changes in managing status
 * needed - Thanks Dave!)
 *
 * Revision 1.9  1992/01/28  18:41:33  cso
 * Restructured dependencies
 *
 * Revision 1.8  1992/01/02  23:36:39  cso
 * Upgraded to the Xica widget set from Xlwh.  Using Message boxes
 * instead of the alert shells, and the new file dialog.
 *
 * Revision 1.7  1991/12/05  15:13:59  cso
 * Changed the appearance of the main window a bit.
 *
 * Revision 1.6  1991/11/27  15:57:34  cso
 * Made some aesthetic changes.
 *
 * Revision 1.5  1991/11/27  00:28:23  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.4  1991/11/19  18:22:24  cso
 * Added the Enter translation to be the same as for Return.
 *
 * Revision 1.3  1991/11/14  15:01:51  cso
 * Added a translation for linefeed as well as return.
 *
 * Revision 1.2  1991/11/09  16:42:36  cso
 * Made some minor changes to the keyboard focus feedback mechanism so
 * that the interface does not shift around too much.
 * Added a function ShiftScrollbars to keep them up to date, but it did
 * not have the desired effect, so nothing calls it at the moment.
 *
 * Revision 1.1.1.2  1991/10/04  20:19:30  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS */

#include "callbacks.h"
#include "fileio.h"
#include "menubar.h"
#include "retag.h"
#include "status.h"
#include "utils.h"

/*  GLOBALS  */

Boolean
   Dirtyflag;
Pixel
   Background,
   Foreground;
Widget
   Alffocus;




/*
   FUNCTION: 		UpdateInterface

   Synopsis: 

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
UpdateInterface()

{
   Arg
      args[MAXARGS];
   Dimension
      textwidth;
   int
      cnt;
   Widget
      ambg,
      data,
      tags;
   
   ambg = XtNameToWidget(Toplevel, "*ambtext");
   data = XtNameToWidget(Toplevel, "*datatext");
   tags = XtNameToWidget(Toplevel, "*tagtext");
   
   XtVaSetValues( data, XtNstring, RTTinfo.data, NULL );
   XtVaSetValues( tags, XtNstring, RTTinfo.tags, NULL );
   XtVaSetValues( ambg, XtNstring, RTTinfo.ambg, NULL );
   
   return;
}




/*
   FUNCTION: 		CreateInterface

   Synopsis: Creates the main interface with the alphabets.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateInterface()

{
   Dimension
      textwidth,
      width;
   Widget 
      alphaform,
      alphaview,
      amblabel,
      ambtext,
      datalabel,
      datatext,
      frame,
      menubar,
      taglabel,
      tagtext;
   static XtTranslations 
      texttrans;
   static String 
      texttransstr =
         "<Key>Return:         TextReturn()\n\
          <Key>KP_Enter:       TextReturn()\n\
          <Key>Linefeed:       TextReturn()\n\
          <Btn1Down>,<Btn1Up>: AlphaFocus()\n\
          <Key>space:          no-op()\n\
          <Key>Tab:            no-op()";


   frame = XtVaCreateManagedWidget( "frame",
       groupWidgetClass, Toplevel,
       XtNspacing, 8,
       NULL );
   
   CreateFileIOShell(Toplevel);

   menubar = CreateMenubar(frame);
   
   texttrans = XtParseTranslationTable(texttransstr);
/*
   alphaview = XtVaCreateManagedWidget( "alphaview",
       viewportWidgetClass, frame,
       XtNallowVert,  True,
       XtNallowHoriz, True,
       XtNalignment,  gaFill,
       XtNmargin,     8,
       NULL );
*/
   alphaform = XtVaCreateManagedWidget( "alphaform",
       groupWidgetClass, frame,
       XtNspacing,     2,
       XtNalignment,   gaFill,
       XtNmargin,      0,
       NULL );

   datalabel = XtVaCreateManagedWidget( "datalabel",
       labelWidgetClass, alphaform,
       XtNlabel,       "Data Characters:",
       XtNborderWidth, 0,
       XtNmargin,      8,
       NULL );
   
   XtVaGetValues( alphaform,
       XtNborderColor, &Foreground,
       XtNbackground,  &Background,
       NULL );

   datatext = XtVaCreateManagedWidget( "datatext",
       icaTextWidgetClass, alphaform,
       XtNeditType,     XawtextEdit,
       XtNdisplayCaret, True,
       XtNborder,       Foreground,
       XtNborderWidth,  2,
       XtNstring,       ALPHABET,
       XtNalignment,    gaFill,
       XtNmargin,       8,
       NULL );
   XtOverrideTranslations( datatext, texttrans );

   XtSetKeyboardFocus( alphaform, datatext );
   Alffocus = datatext;
   
   taglabel = XtVaCreateManagedWidget( "taglabel",
       labelWidgetClass, alphaform,
       XtNlabel,       "Reserved Characters:",
       XtNborderWidth, 0,
       XtNmargin,      8,
       NULL );
   
   tagtext = XtVaCreateManagedWidget( "tagtext",
       icaTextWidgetClass, alphaform,
       XtNeditType,     XawtextEdit,
       XtNdisplayCaret, False,
       XtNborder,       Background,
       XtNborderWidth,  2,
       XtNstring,       "",
       XtNalignment,    gaFill,
       XtNmargin,       8,
       NULL );
   XtOverrideTranslations( tagtext, texttrans );
   
   amblabel = XtVaCreateManagedWidget( "amblabel",
       labelWidgetClass, alphaform,
       XtNlabel,       "Context-Sensitive Characters:",
       XtNborderWidth, 0,
       XtNmargin,      8,
       NULL );
   
   ambtext = XtVaCreateManagedWidget( "ambtext",
       icaTextWidgetClass, alphaform,
       XtNeditType,     XawtextEdit,
       XtNdisplayCaret, False,
       XtNborder,       Background,
       XtNborderWidth,  2,
       XtNstring,       "",
       XtNalignment,    gaFill,
       XtNmargin,       8,
       NULL );
   XtOverrideTranslations(ambtext, texttrans);

   CreateStatus(frame);

   Dirtyflag = False;
   
   XtRealizeWidget(Toplevel);

   return;
}
