/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: callbacks.c,v $
   Purpose:     To contain the callbacks used from the editors and popups.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.15 $
   Date:        $Date: 92/05/29 15:27:40 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	callbacks.c,v $
 * Revision 1.15  92/05/29  15:27:40  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.12  92/03/16  15:00:44  ware
 * Make titles consistent.
 * 
 * Revision 1.10  1992/02/21  05:51:26  cso
 * Needed to more strongly tie the regular expression validation routine
 * into the interface so that various error messages were displayed in a
 * conspicuous manner instead of the xterm.  Only the Makefile progress
 * is left in the xterm.  This can be masked out easily with the @'s in
 * the makefile.  The changes required a modification to icatypes.h and a
 * switch to handle the different return values, but now an
 * acknowledgement type of popup occurs when there is a problem with the
 * regexp with an indication of the problem.
 *
 * Revision 1.9  1992/02/20  04:55:51  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.8  1992/01/30  14:21:30  cso
 * Variety of mods:
 *  - The state of the implicit toggles is always True (to make it easier
 * 	to understand what should be set in intag).
 *  - Changes in focus have two effects: the display caret is shown and
 * 	the border color changes to the foreground.
 * 	Unmanaging/Managing is no longer needed.
 *  - A message is displayed explaining why the context dialog does not
 * 	pop up for an empty tag.
 *
 * Revision 1.7  1992/01/28  18:41:25  cso
 * Restructured dependencies
 *
 * Revision 1.6  1992/01/02  23:36:46  cso
 * Upgraded to the Xica widget set from Xlwh.  Using Message boxes
 * instead of the alert shells, and the new file dialog.
 *
 * Revision 1.5  1991/12/05  15:14:49  cso
 * Added a separator tag editor.
 *
 * Revision 1.4  1991/11/27  00:28:29  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.3  1991/11/09  16:42:41  cso
 * Made some minor changes to the keyboard focus feedback mechanism so
 * that the interface does not shift around too much.
 * Added a function ShiftScrollbars to keep them up to date, but it did
 * not have the desired effect, so nothing calls it at the moment.
 *
 * Revision 1.2  1991/11/07  22:00:58  cso
 * Made the variable-width fix to the dialog return.
 *
 * Revision 1.1.1.2  1991/10/04  20:19:33  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include <icatypes.h>

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "alpha.h"
#include "list.h"
#include "retag.h"
#include "status.h"
#include "tagedit.h"
#include "utils.h"

/*  GLOBALS  */

ConCode
   Clausecode;
Widget
   Clausew;




/*
   FUNCTION: 		ScrollTogether

   Synopsis: To have the viewports in the editors look as though the
              scrollbars are coordinated to scroll together.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ScrollTogether(w, parent, call_data)

Widget 
   w,
   parent;
XtPointer 
   call_data;                     /* not used */

{
   Arg 
      args[MAXARGS];
   int
      cnt;
   Position 
      x,
      y;
   float 
      top,
      shown;
   Widget
      tags,
      tvert,
      rexp,
      rvert,
      con,
      cvert;
   
   tvert = XtNameToWidget(parent, "*tagview*vertical");
   rvert = XtNameToWidget(parent, "*rexview*vertical");
   cvert = XtNameToWidget(parent, "*conview*vertical");
   tags  = XtNameToWidget(parent, "*tagform");
   rexp  = XtNameToWidget(parent, "*rexform");
   con   = XtNameToWidget(parent, "*conform");

   cnt = 0;
   XtSetArg(args[cnt], XtNtopOfThumb, &top);   cnt++;
   XtSetArg(args[cnt], XtNshown,      &shown); cnt++;
   XtGetValues(w, args, cnt);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNx, &x); cnt++;
   XtSetArg(args[cnt], XtNy, &y); cnt++;
   if (w == tvert)
   {
      XtGetValues(tags, args, cnt);
      XawScrollbarSetThumb(rvert, top, shown);
      XtMoveWidget(rexp, x, y);
      XawScrollbarSetThumb(cvert, top, shown);
      XtMoveWidget(con, x, y);
   }
   else if (w == rvert)
   {
      XtGetValues(rexp, args, cnt);
      XawScrollbarSetThumb(tvert, top, shown);
      XtMoveWidget(tags, x, y);
      XawScrollbarSetThumb(cvert, top, shown);
      XtMoveWidget(con, x, y);
   }
   else if (w == cvert)
   {
      XtGetValues(con, args, cnt);
      XawScrollbarSetThumb(tvert, top, shown);
      XtMoveWidget(tags, x, y);
      XawScrollbarSetThumb(rvert, top, shown);
      XtMoveWidget(rexp, x, y);
   }
   return;
}




/*
   FUNCTION: 		TextReturn

   Synopsis: The callback used whenever the user hits return.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
TextReturn(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Arg
      args[MAXARGS];
   Dimension
      textwidth;
   int
      cnt;
   String
      str,
      textstr;
   Widget
      ambgw,
      dataw,
      tagsw;
   
   ambgw = XtNameToWidget(ShellParent(w), "*ambtext");
   dataw = XtNameToWidget(ShellParent(w), "*datatext");
   tagsw = XtNameToWidget(ShellParent(w), "*tagtext");

#ifdef DEBUG
   fprintf(stderr, "Widget is %s\n", XtName(w));
#endif
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, &textstr); cnt++;
   XtGetValues(w, args, cnt);

   textstr = RemoveDuplicateChars(textstr);
   XawTextSetSelection(w, 0, strlen(textstr));

   if ((w == dataw) && (strcmp(RTTinfo.data, textstr) != 0))
   {
      XtFree(RTTinfo.data);
      RTTinfo.data = XtNewString(textstr);
      str = UpdateAlphabet(ambgw, textstr);
      if (strcmp(RTTinfo.ambg, str) != 0)
      {
	 XtFree(RTTinfo.ambg);
	 RTTinfo.ambg = XtNewString(str);
      }
      str = UpdateAlphabet(tagsw, textstr);
      if (strcmp(RTTinfo.tags, str) != 0)
      {
	 XtFree(RTTinfo.tags);
	 RTTinfo.tags = XtNewString(str);
      }
      Dirtyflag = True;
   }
   else if ((w == tagsw) && (strcmp(RTTinfo.tags, textstr) != 0))
   {
      XtFree(RTTinfo.tags);
      RTTinfo.tags = XtNewString(textstr);
      str = UpdateAlphabet(ambgw, textstr);
      if (strcmp(RTTinfo.ambg, str) != 0)
      {
	 XtFree(RTTinfo.ambg);
	 RTTinfo.ambg = XtNewString(str);
      }
      str = UpdateAlphabet(dataw, textstr);
      if (strcmp(RTTinfo.data, str) != 0)
      {
	 XtFree(RTTinfo.data);
	 RTTinfo.data = XtNewString(str);
      }
      Dirtyflag = True;
   }
   else if ((w == ambgw) && (strcmp(RTTinfo.ambg, textstr) != 0))
   {
      XtFree(RTTinfo.ambg);
      RTTinfo.ambg = XtNewString(textstr);
      str = UpdateAlphabet(dataw, textstr);
      if (strcmp(RTTinfo.data, str) != 0)
      {
	 XtFree(RTTinfo.data);
	 RTTinfo.data = XtNewString(str);
      }
      str = UpdateAlphabet(tagsw, textstr);
      if (strcmp(RTTinfo.tags, str) != 0)
      {
	 XtFree(RTTinfo.tags);
	 RTTinfo.tags = XtNewString(str);
      }
      Dirtyflag = True;
   }

   XtVaSetValues( dataw, XtNstring, RTTinfo.data, NULL );
   XtVaSetValues( tagsw, XtNstring, RTTinfo.tags, NULL );
   XtVaSetValues( ambgw, XtNstring, RTTinfo.ambg, NULL );

   return;
}



/*
   FUNCTION: 		TagReturn

   Synopsis: The callback used whenever the user hits return.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
TagReturn(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Arg
      args[MAXARGS];
   Dimension
      textwidth;
   int
      cnt;
   String
      textstr;
   Tagspec
      *tptr;

#ifdef DEBUG
   fprintf(stderr, "Widget is [%s]\n", XtName(w));
#endif
   tptr = FindTagptr(w);
   if (tptr == NULL)
   {
      XicaStatus(Statw, "TagReturn: Unable to find tag pointer\n");
      return;
   }
   
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, &textstr); cnt++;
   XtGetValues(w, args, cnt);

#ifdef DEBUG
   fprintf(stderr, "String is [%s]\n", textstr);
   fprintf(stderr, "tptr->tag is [%s]\n", tptr->tag);
#endif
   if ((*textstr == '\0') && (w == tptr->tagw))
   {
      textwidth = GetStringWidthInPixels(w, tptr->tag);
      cnt = 0;
      XtSetArg(args[cnt], XtNstring, tptr->tag); cnt++;
      XtSetArg(args[cnt], XtNwidth,  textwidth); cnt++;
      XtSetValues(w, args, cnt);
      XawTextSetSelection(w, 0, strlen(tptr->tag));
      return;
   }
   textwidth = GetStringWidthInPixels(w, textstr);
   cnt = 0;
   XtSetArg(args[cnt], XtNwidth,  textwidth); cnt++;
   XtSetValues(w, args, cnt);
   XawTextSetSelection(w, 0, strlen(textstr));

   if (w == tptr->tagw)
   {
      if (strcmp(tptr->tag, textstr) == 0)
      {
	 return;
      }
      XtFree(tptr->tag);
      tptr->tag = XtNewString(textstr);
      Dirtyflag = True;
   }
   else if (w == tptr->rexw)
   {
      if (strcmp(tptr->regexp, textstr) == 0)
      {
	 return;
      }
      XtFree(tptr->regexp);
      tptr->regexp = XtNewString(textstr);
      Dirtyflag = True;
      if (*textstr == '\0')
      {
	 return;
      }
      switch (is_valid_rex(textstr))
      {
      default:
	 break;
	 
      case BadRepetitionContent:
	 OkMessage(Toplevel,
 "ERROR: Expression contains an illegal character in repetition operator {}");
	 break;
	 
      case BadStartingChar:
	 OkMessage(Toplevel,
		   "ERROR: Expression starting with a bad character");
	 break;
	 
      case BlankFound:
	 OkMessage(Toplevel,
		   "ERROR: Expression may not contain a blank");
	 break;
	 
      case EmptyParens:
	 OkMessage(Toplevel,
		   "ERROR: Expression may not contain empty parentheses ()");
	 break;
	 
      case EndingBackslash:
	 OkMessage(Toplevel,
		   "ERROR: Expression may not end in a backslash \\");
	 break;
	 
      case IllegalOpenBrace:
	 OkMessage(Toplevel,
		   "ERROR: Expression may not start with a {");
	 break;
	 
      case IllegalPercent:
	 OkMessage(Toplevel,
		   "ERROR: Expression may not start with %%");
	 break;
	 
      case MissingCloseBracket:
	 OkMessage(Toplevel,
		   "ERROR: Expression missing a close bracket ]");
	 break;
	 
      case MissingCloseParen:
	 OkMessage(Toplevel,
		   "ERROR: Expression missing a close parenthesis )");
	 break;
	 
      case MissingCloseQuote:
	 OkMessage(Toplevel,
		   "ERROR: Expression missing a close quote \"");
	 break;
	 
      case MissingOpenBrace:
	 OkMessage(Toplevel,
		   "ERROR: Expression missing an open brace {");
	 break;
	 
      case MissingOpenParen:
	 OkMessage(Toplevel,
		   "ERROR: Expression missing an open parenthesis (");
	 break;
	 
      case NonportableRange:
	 OkMessage(Toplevel,
		   "WARNING: Nonportable range specified '-'");
	 break;
	 
      case Valid:
	 XicaStatus(Statw, "Valid regular expression [%s]\n", textstr);
	 break;
	 
      case WhereCloseBrace:
	 OkMessage(Toplevel,
		   "ERROR: Expression has misplaced close brace }");
	 break;
      }
   }
   if ((tptr->tagw == w) && (tptr->next == NULL) &&
       ((tptr->tcode == Symbol) || (tptr->tcode == ProcIn)))
   {
      tptr->next = NewTagptr("", tptr->tcode);
      CreateEditorWidgets( ShellParent(w),
			  tptr->next, XawtextEdit, tptr);
   }
   return;
}



/*
   FUNCTION: 		AttSet

   Synopsis: Set the tag code attribute.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AttSet(w, tptr, call_data)

Widget
   w;
Tagspec
   *tptr;
caddr_t
   call_data;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   
   cnt = 0;
   if (tptr->tcode == ImEnd)
   {
      XtSetArg(args[cnt], XtNlabel, "S");  cnt++;
      tptr->tcode = ImBeg;
   }
   else
   {
      XtSetArg(args[cnt], XtNlabel, "E");   cnt++;
      tptr->tcode = ImEnd;
   }
   XtSetArg(args[cnt], XtNstate, True); cnt++;
   XtSetValues(w, args, cnt);
   Dirtyflag = True;
   return;
}



/*
  FUNCTION: 		AlphaFocus

   Synopsis: The callback used whenever the user selects a text widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AlphaFocus(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      oldfocus;
   
#ifdef DEBUG
   fprintf(stderr, "w = [%s]\n", XtName(w));
#endif
   oldfocus = Alffocus;
   Alffocus = w;

   if (oldfocus != NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "oldfocus = [%s]\n", XtName(oldfocus));
#endif
      cnt = 0;
      XtSetArg(args[cnt], XtNdisplayCaret, False);      cnt++;
      XtSetArg(args[cnt], XtNborder,       Background); cnt++;
      XtSetValues(oldfocus, args, cnt);
   }

   XtSetKeyboardFocus(XtParent(w), w);
   cnt = 0;
   XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;
   XtSetValues(w, args, cnt);
   
   return;
}



/*
   FUNCTION: 		FocusSelect

   Synopsis: The callback used whenever the user selects a text widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
       pete	3/3/92	 I've changed the names of the editor widgets created
       			 in CreateEditorShell() so the title is set in the
			 resources instead of being hardcoded.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FocusSelect(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      oldfocus,
      parent;
   
   parent = ShellParent(w);
#ifdef DEBUG
   fprintf(stderr, "parent = [%s]\n", XtName(parent));
   fprintf(stderr, "w      = [%s]\n", XtName(w));
#endif
   if (parent == XtNameToWidget(Toplevel, "*symsel-editor"))
   {
      oldfocus = Symfocus;
      Symfocus = w;
   }
   else if (parent == XtNameToWidget(Toplevel, "*pinsel-editor"))
   {
      oldfocus = Pinfocus;
      Pinfocus = w;
   }
   else if (parent == XtNameToWidget(Toplevel, "*sepsel-editor"))
   {
      oldfocus = Sepfocus;
      Sepfocus = w;
   }
   else if (parent == XtNameToWidget(Toplevel, "*impsel-editor"))
   {
      oldfocus = Impfocus;
      Impfocus = w;
   }
   else if (parent == XtNameToWidget(Toplevel, "*expsel-editor"))
   {
      oldfocus = Expfocus;
      Expfocus = w;
   }
   else
   {
      oldfocus = NULL;
   }

   if (oldfocus != NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "oldfocus = [%s]\n", XtName(oldfocus));
#endif
      cnt = 0;
      XtSetArg(args[cnt], XtNdisplayCaret, False);      cnt++;
      XtSetArg(args[cnt], XtNborder,       Background); cnt++;
      XtSetValues(oldfocus, args, cnt);
   }

   XtSetKeyboardFocus(parent, w);
   cnt = 0;
   XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;
   XtSetValues(w, args, cnt);
   return;
}



/*
   FUNCTION: 		DialogOk

   Synopsis: Called from the OK routines from the popup b/c they do
   the exact same thing.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
DialogOk(popup)

Widget
   popup;

{
   Arg
      args[MAXARGS];
   Dimension
      textwidth;
   int
      cnt;
   String
      valstr;
   Tagspec
      *tptr;
   Widget
      value;
   
   value = XtNameToWidget(popup, "*value");
   XtVaGetValues( value, XtNstring, &valstr, NULL );
   XtPopdown(popup);
   
   if (*valstr != '\0')
   {
      AddContext(valstr);
      tptr = FindTagptr(Clausew);
      if (tptr != NULL)
      {
	 XtFree(tptr->context);
	 tptr->context = XtMalloc(strlen(valstr) + 16);
	 switch (Clausecode)
	 {
	 default:
	    break;
	    
	 case Start:
	    sprintf(tptr->context, "START: %s", valstr);
	    break;
	    
	 case Stop:
	    sprintf(tptr->context, "END: %s", valstr);
	    break;
	    
	 case In:
	    sprintf(tptr->context, "IN: %s", valstr);
	    break;
	    
	 case Nest:
	    break;
	 }
	 textwidth = GetStringWidthInPixels(tptr->conw, tptr->context);
	 cnt = 0;
	 XtSetArg(args[cnt], XtNstring, tptr->context); cnt++;
	 XtSetArg(args[cnt], XtNwidth,  textwidth);     cnt++;
	 XtSetValues(tptr->conw, args, cnt);
	 Dirtyflag = True;
      } /***** IF a tag pointer was found */
   } /***** IF a string was there when the function was called */
   return;
}




/*
   FUNCTION: 		DialogOkTrans

   Synopsis: The callback used whenever the user hits return in the popup.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DialogOkTrans(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */

{
   Widget
      popup;

   popup = XtNameToWidget(Toplevel, "*dlgpopup");
   DialogOk(popup);
   return;
}



/*
   FUNCTION: 		DialogOkButton

   Synopsis: The callback used whenever the user hits the Ok button.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DialogOkButton(w, popup, call_data)

Widget
   w,
   popup;
XtPointer
   call_data;   /* Unused */

{
   DialogOk(popup);
   return;
}



/*
   FUNCTION: 		DialogList

   Synopsis: The callback used whenever the user hits return in the popup.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DialogList(w, popup, item)

Widget
   w,
   popup;
XawListReturnStruct
   item;

{
   Widget 
      value;

   XawListUnhighlight(w);

   value = XtNameToWidget( popup, "*value" );
   XtVaSetValues( value, XtNstring, item.string, NULL );

   return;
}



/*
   FUNCTION: 		ContextSelect

   Synopsis: The callback used whenever the user popups the Context menu.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ContextSelect(w, event, params, num)

Widget 
     w;
XEvent
   *event;     /* Unused */
String
   *params;    /* Unused */
int
   num;        /* Unused */
{
   Clausew = w;
   return;
}



/*
   FUNCTION: 		ContextMenuSelect

   Synopsis: The user has selected a context item.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
ContextMenuSelect(w, ccode, call_data)

Widget
   w;
ConCode
   ccode;
XtPointer
   call_data;   /* Unused */

{
   Arg
      args[MAXARGS];
   Dimension
      height,
      textwidth,
      width;
   int
      cnt;
   Position
      popx,
      popy;
   Tagspec
      *tptr;
   Widget
      dialog;
   
   /*
    * If the code is not Nope then get the clause string
    * Depending on the code create the context string and
    * Finding the tptr (based on Clausew)
    */
   tptr = FindTagptr(Clausew);
#ifdef DEBUG
   if (tptr != NULL)
   {
      fprintf(stderr, "Clause for [%s]\n", tptr->tag);
   }
#endif
   if ((tptr != NULL) && (tptr->next == NULL) &&
       ((tptr->tcode == Symbol) || (tptr->tcode == ProcIn)))
   {
      XicaStatus(Statw, "Cannot specify a condition for an empty tag\n");
      return;
   }
   if (ccode != Nope)
   {
      Clausecode = ccode;
      
      dialog = XtNameToWidget(Toplevel, "*dlgpopup");
      XtVaGetValues( dialog,
          XtNheight, &height,
          XtNwidth,  &width,
          NULL );

      XtTranslateCoords(XtParent(XtParent(tptr->conw)), 0, 0, &popx, &popy);
      
      XtVaSetValues( dialog,
          XtNx, popx-width,
          XtNy, popy-height,
          NULL );

      UpdateList(dialog);
      
      XtPopup(dialog, XtGrabExclusive);
#ifdef DEBUG
      fprintf(stderr, "returned from popup\n");
#endif
   }
   else if (strcmp(tptr->context, "None") != 0)
   {
      XtFree(tptr->context);
      tptr->context = XtNewString("None");
      textwidth = GetStringWidthInPixels(tptr->conw, tptr->context);
      cnt = 0;
      XtSetArg(args[cnt], XtNstring, tptr->context); cnt++;
      XtSetArg(args[cnt], XtNwidth,  textwidth);     cnt++;
      XtSetValues(tptr->conw, args, cnt);
      Dirtyflag = True;
   }
   return;
}



/*
   FUNCTION: 		DuplicateTag

   Synopsis: Duplicate a tag specification.

   Initial purpose:
       Same as above.

   Present purpose:       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DuplicateTag(w, parent, call_data)

Widget
   w,
   parent;
XtPointer
   call_data; /* Unused */

{
   Tagspec
      *new1,
      *new2,
      *tptr1,
      *tptr2;
   Widget
      editor,
      focus;


   focus = FindFocus(parent);  editor = ShellParent(focus);
   if (focus == NULL)
   {
      XicaStatus(Statw, "\nSelect the regular expression of the tag ");
      XicaStatus(Statw, "that you want duplicated\n");
      return;
   }
   tptr1 = FindTagptr(focus);
   if (tptr1 == NULL)
   {
      XicaStatus(Statw, "DuplicateTag: Unable to find tag from focus\n");
      return;
   }
   if (((tptr1->tcode == Symbol) || (tptr1->tcode == ProcIn)) &&
       (tptr1->next == NULL))
   {
      XicaStatus(Statw, "It is unnecessary to duplicate this tag\n");
      return;
   }

   EditorDoLayout( editor, False );

   tptr2 = NULL;
   if (tptr1->tcode == ExBeg)
   {
      tptr2 = tptr1->next;
   }
   if (tptr1->tcode == ExEnd)
   {
      tptr2 = tptr1;
      tptr1 = RTTinfo.expptr;
      while ((tptr1 != NULL) && (tptr1->next != tptr2))  tptr1 = tptr1->next;
   }

   new1 = NewTagptr(tptr1->tag, tptr1->tcode);
   if (tptr2 != NULL)
   {
      new2          = NewTagptr(tptr2->tag, tptr2->tcode);
      new1->next    = new2;
      new2->next    = tptr2->next;
      tptr2->next   = new1;
      XtFree(new2->regexp);
      new2->regexp  = XtNewString(tptr2->regexp);
      XtFree(new2->context);
      new2->context = XtNewString(tptr2->context);
   }
   else
   {
      new1->next  = tptr1->next;
      tptr1->next = new1;
   }
   XtFree(new1->regexp);
   new1->regexp  = XtNewString(tptr1->regexp);
   XtFree(new1->context);
   new1->context = XtNewString(tptr1->context);
   switch (new1->tcode)
   {
   default:
      break;

   case Symbol:
   case ProcIn:
      CreateEditorWidgets(parent, new1, XawtextEdit, tptr1);
      FixVerticals(new1->next, new1);
      break;
      
   case Comma:
#ifdef ICA
      CreateEditorWidgets(parent, new1, XawtextRead, tptr1);
#else
      CreateEditorWidgets(parent, new1, XawtextEdit, tptr1);
#endif
      FixVerticals(new1->next, new1);
      break;
      
   case ImBeg:
   case ImEnd:
#ifdef ICA
      CreateEditorWidgets(parent, new1, XawtextRead, tptr1);
#else
      CreateEditorWidgets(parent, new1, XawtextEdit, tptr1);
#endif
      FixVerticals(new1->next, new1);
      break;
      
   case ExBeg:
   case ExEnd:
#ifdef ICA
      CreateEditorWidgets(parent, new1, XawtextRead, tptr2);
      CreateEditorWidgets(parent, new2, XawtextRead, new1);
#else
      CreateEditorWidgets(parent, new1, XawtextEdit, tptr2);
      CreateEditorWidgets(parent, new2, XawtextEdit, new1);
#endif
      FixVerticals(new2->next, new2);
      break;
   }
   Dirtyflag = True;
   
   EditorDoLayout( editor, True );

   return;
}



/*
   FUNCTION: 		RemoveTag

   Synopsis: Remove a tag specification.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
RemoveTag(w, parent, call_data)

Widget
   w,
   parent;
XtPointer
   call_data; /* Unused */

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Tagspec
      *rem,
      *tptr;
   Widget
      focus,
      vert;
   
   focus = FindFocus(parent);
   if (focus == NULL)
   {
      XicaStatus(Statw, "\nSelect the regular expression of the tag ");
      XicaStatus(Statw, "that you want removed\n");
      return;
   }
   tptr = FindTagptr(focus);
   if (tptr == NULL)
   {
      XicaStatus(Statw, "RemoveTag: Unable to find tag from focus\n");
      return;
   }
   if (tptr->next == NULL)
   {
      XicaStatus(Statw, "It is unnecessary to remove this tag\n");
      return;
   }

   if (YesNoMessage(Toplevel, "Remove tag specification?", False) == False)
   {
      return;
   }
   
   if ((Pinfocus != NULL) && (parent == ShellParent(Pinfocus)))
   {
      Pinfocus = NULL;
   }
   else if ((Symfocus != NULL) && (parent == ShellParent(Symfocus)))
   {
      Symfocus = NULL;
   }

   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert, &vert); cnt++;
   XtGetValues(tptr->rexw, args, cnt);
   
   if (vert == NULL)
   {
      if (RTTinfo.symptr == tptr)
      {
	 FixVerticals(tptr->next, NULL);
	 RTTinfo.symptr = tptr->next;
      }
      else if (RTTinfo.pinptr == tptr)
      {
	 FixVerticals(tptr->next, NULL);
	 RTTinfo.pinptr = tptr->next;
      }
   } /***** IF the vertical widget is NULL */
   else
   {
      rem = FindTagptr(vert);
      if (rem == NULL)
      {
	 XicaStatus(Statw, "RemoveTag: Could not find the previous tag ptr\n");
	 return;
      }
      FixVerticals(tptr->next, rem);
      rem->next = tptr->next;
   }

   DisposeTagptr(tptr);
   Dirtyflag = True;
   return;
}
