/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: chtab.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.4 $
   Date:        $Date: 92/03/16 15:00:52 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	chtab.c,v $
 * Revision 1.4  92/03/16  15:00:52  ware
 * Make titles consistent.
 * 
 * Revision 1.3  1992/01/28  18:41:19  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */



#include "widgets.h"
#include "defs.h"
#include "types.h"

#include <icatypes.h>

/* #DEFINES  */

/*  EXTERNS  */

#include "contab.h"
#include "retag.h"
#include "status.h"

/*  GLOBALS  */

int
   Chartab[CTABSIZE][MAX_CTXT + 1];


/* Chartab contains an integer value for each printable character     */
/* (space to ~).  Chartab[][0] represents the global uses of the      */
/* characters.  Chartab[][i], 1 <= i <= MAX_CTXT, represents the      */
/* character usage for the start conditions listed in Ctxttab[j],   */
/* j = i - 1.                                                         */
/* This integer indicates the use of the character as text or markup. */
/* If the value is 1, the character is part of the general CDATA      */
/* pattern.  If the value is 2, the character is used exclusively as  */
/* markup.  If the value is 3, the character is ambiguous; Sometimes  */
/* it is markup, sometimes it is text.  A character whose value       */
/* is zero has not been specified in any alphabet.                    */

void
InstallAlfa(str, value)

char
   *str;
int
   value;

/* This function installs the integer values for the characters in   */
/* the different alphabets.                                          */
/* str is the alphabet-string regular expression.                    */
/* value is the integer value to be assigned.                        */
/* Called by yyparse (rcomp.yacc).                                   */

{
   int
      i,
      j;

   Chartab[0][0] = 1;
   for (i = 0; i < strlen(str); i++)
   {
      if ((isalnum(str[i]) != 0) && (str[i + 1] == '-'))
      {
	 /* have a range of alphanumeric characters */
	 for (j = str[i]; j <= str[i + 2]; j++)
	 {
	    Chartab[j - ' '][0] = value;
	 }
	 i = i + 2;  /* point to last char. in range */
      }
      else
      {
	 /* have a single character */
	 Chartab[str[i] - ' '][0] = value;
      }
   }
   return;
}



void
ZeroChartab()

{
   int
      i;
   
   for (i = 0; i < CTABSIZE; i++)
   {
      Chartab[i][0] = 0;
   }
   return;
}



Boolean
CheckAmbig(regexp)

String
   regexp;

{
   int
      i;
   char
      *ambstr;
   
   if ((ambstr = (char *)malloc(8)) == NULL)
   {
      XicaStatus(Statw, "CheckAmbig: Unable to allocate space\n");
      return(False);
   }
   
   ambstr[1] = '\0';
   for (i = 0; i < CTABSIZE; i++)
   {
      if (Chartab[i][0] == 3)
      {
	 ambstr[0] = i + ' ';

	 switch (compare_string(regexp, &ambstr, False, False))
	 {
         case STRING_MATCH:
#ifdef DEBUG
            fprintf(stderr, "String Matched: [%s], [%s]\n", regexp,
		    ambstr);
#endif
	    free(ambstr);
	    return(True);
            break;

         case STRING_MATCH_RECOPIED:
#ifdef DEBUG
            fprintf(stderr, "String Matched Recopied: [%s], [%s]\n",
		    regexp, ambstr);
#endif
	    free(ambstr);
	    return(True);
            break;

         case STRING_NO_MATCH:
#ifdef DEBUG
            fprintf(stderr, "String Not Matched: [%s], [%s]\n",
		    regexp, ambstr);
#endif
            break;

         case REX_SYNTAX_ERROR:
#ifdef DEBUG
            fprintf(stderr, "Rex Syntax Error: [%s], [%s]\n", regexp,
		    ambstr);
#endif
            break;

	 case STRING_OTHER_ERROR:
#ifdef DEBUG
            fprintf(stderr, "String Other Error: [%s], [%s]\n",
		    regexp, ambstr);
#endif
            break;
	 }
      }
   }
   free(ambstr);
   return(False);
}




void
MakeCdata(aptr, nptr, sptr, chndx)

String
   aptr,
   nptr,
   sptr;
int
   chndx;

/* This function prints the CDATA patterns. */
/* Called from yyparse (rcomp.yacc).        */
{
   int
      i,
      aj,
      nj,
      sj;

   aj = 0;
   nj = 0;
   sj = 0;
   for (i = 1; i < CTABSIZE; i++)
   {
      if (Chartab[i][chndx] == 1)
      {
	 switch (i + ' ')
	 {
	 case '-':
	 case '^':
	 case '[':
	 case ']':
	 case '\\':
	    sptr[sj] = '\\';
	    sj++;
	    sptr[sj] = i + ' ';
	    sj++;
	    break;

	 default:
	    if (!isalnum(i + ' '))
	    {
	       sptr[sj] = i + ' ';
	       sj++;
	    }
	    break;
	 }

	 if (((i + 1) < CTABSIZE) && (isalnum(i + ' ')) &&
	     (Chartab[i + 1][chndx] == 1))
	 {
	    /* have a range of alphanumeric characters */
	    if (isalpha(i + ' '))
	    {
	       /* alphabetic */
	       aptr[aj] = i + ' ';
	       aj++;
	       aptr[aj] = '-';
	       aj++;
	       for (; ((Chartab[i][chndx] == 1) && (i < CTABSIZE) &&
		       (isalnum(i + ' '))); i++)
	       {
	       }
	       i--;
	       aptr[aj] = i + ' ';
	       aj++;
	    }
	    else
	    {
	       /* numeric */
	       nptr[nj] = i + ' ';
	       nj++;
	       nptr[nj] = '-';
	       nj++;
	       for (; ((Chartab[i][chndx] == 1) && (i < CTABSIZE) &&
		       (isalnum(i + ' '))); i++)
	       {
	       }
	       i--;
	       nptr[nj] = i + ' ';
	       nj++; 
	    }
	 }
      }
   }
   aptr[aj] = '\0';
   nptr[nj] = '\0';
   sptr[sj] = '\0';
   return;
}



/* ****************************************************************** */

void
InitChartab(numctxt)

int
   numctxt;

/* This function initializes Chartab[][]. */

{
   int
      i,
      j;

   for (i = 0; i < CTABSIZE; i++)
   {
      for (j = 0; j < numctxt; j++)
      {
	 Chartab[i][j + 1] = Chartab[i][0];
      }
   }
   return;
}

/* ****************************************************************** */

void
UpdateChartab(rex, chndx)

char
   *rex;
int
   chndx;

/* This function examines a regular expression and looks for a  */
/* nonalphanumeric character to change its entry to 3 in        */
/* Chartab[][current_ctxt_ndx].                                 */
/* Called from print_*s_prod*.                                  */

{
   int
      i;

   i = 0;
   if (rex[i] == '^')
   {
      i++;
   }
   for ( ; rex[i] != '\0'; i++)
   {
      switch (rex[i])
      {
      case '.':
      case '?':
      case '*':
      case '+':
      case '|':
      case '/':
      case '(':
      case ')':
	 /* operators to be skipped */
	 break;

      case '{':
	 /* repetition or defn - can be skipped */
	 i++;
	 for ( ; rex[i] != '}'; i++)
	 {
	 }
	 break;

      case '$':
	 /* end of line context operator */
	 if (rex[i + 1] != '\0')
	 {
	    Chartab[rex[i] - ' '][chndx] = 3;
	 }
	 break;
	 
      case '\\':
	 /* escaped character */
	 i++;
	 if (!isalnum(rex[i]))
	 {
	    Chartab[rex[i] - ' '][chndx] = 3;
	 }
	 break;

      case '[':
	 /* char. class - no guarantee which char. may appear */
	 /* skip it                                           */
	 i++;
	 for (; rex[i] != ']'; i++)
	 {
	 }
	 break;

      case '\"':
	 /* quoted string - must check all chars. */
	 i++;
	 for (; rex[i] != '\"'; i++)
	 {
	    if ((!isalnum(rex[i])) && (rex[i] != ' '))
	    {
	       if (rex[i] == '\\')
	       {
		  i++;
		  if ((!isalnum(rex[i])) && (rex[i] != ' '))
		  {
		     Chartab[rex[i] - ' '][chndx] = 3;
		  }
	       }
	       else
	       {
		  Chartab[rex[i] - ' '][chndx] = 3;
	       }
	    }
	 }
	 break;

      default:
	 if ((!isalnum(rex[i])) && (rex[i] != ' '))
	 {
	    Chartab[rex[i] - ' '][chndx] = 3;
	 }
	 break;
      } /* SWITCH */
   } /* FOR */
   return;
}

/* ****************************************************************** */

void
DumpChartab()

/* This function dumps the contents of Chartab.             */
/* Best when called from the root production in rcomp.yacc. */

{
   int
      i,
      j;

   fprintf(stderr, "Chartab dump\n");
   for (i = 0; i < CTABSIZE; i++)
   {
      fprintf(stderr, "[%c] = [%d] ", i+' ', Chartab[i][0]);
      for (j = 1; j <= Ctxtndx; j++)
      {
	 fprintf(stderr,"[%d] ", Chartab[i][j]);
      }
      fprintf(stderr, "\n");
   }
   return;
}
