/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: dialog.c,v $
   Purpose:     Contains the routines for the context clause dialog.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/05/29 15:27:47 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	dialog.c,v $
 * Revision 1.8  92/05/29  15:27:47  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.7  92/03/16  15:01:10  ware
 * Make titles consistent.
 * 
 * Revision 1.6  1992/01/28  18:41:10  cso
 * Restructured dependencies
 *
 * Revision 1.5  1991/11/27  00:28:48  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.4  1991/11/19  18:22:31  cso
 * Added the Enter translation to be the same as for Return.
 *
 * Revision 1.3  1991/11/14  15:01:56  cso
 * Added a translation for linefeed as well as return.
 *
 * Revision 1.2  1991/11/06  23:07:50  cso
 * Added double-click capability in the context dialog list.
 *
 * Revision 1.1.1.2  1991/10/04  20:19:44  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "callbacks.h"
#include "retag.h"




/*
   FUNCTION: 		CreateContextDialog

   Synopsis: Create the dialog for obtaining the context.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateContextDialog()

{
   Widget
      popup,
      dlgform,
      dlgview,
      dlglist,
      dialog, dlglbl,
      dlgbtns, dlgok, dlgcancel, dlghelp,
      value;
   XtPopdownID
      pdptr;
   XtTranslations 
      dlgtrans,
      listtrans;
   static String 
      dlgtransstr = "<Key>Return:         DialogOkTrans()\n\
                      <Key>KP_Enter:       DialogOkTrans()\n\
                      <Key>Linefeed:       DialogOkTrans()\n\
                      <Key>space:          no-op()\n\
                      <Key>Tab:            no-op()",
      listtransstr = "<Btn1Up>(2):         DialogOkTrans()\n\
                      <Btn1Down>,<Btn1Up>: Set() Notify()";
   
   dlgtrans  = XtParseTranslationTable(dlgtransstr);
   listtrans = XtParseTranslationTable(listtransstr);
   pdptr     = XtNew(XtPopdownIDRec);
   
   popup = XtVaCreatePopupShell( "dlgpopup",
       transientShellWidgetClass, Toplevel,
       XtNallowShellResize, True,
       NULL );
   
   dlgform = XtVaCreateManagedWidget( "dlgform",
       groupWidgetClass, popup,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     6,
       NULL );

   dlgview = XtVaCreateManagedWidget( "dlgview", 
       viewportWidgetClass, dlgform,
       XtNallowHoriz, True,
       XtNallowVert,  True,
       XtNpreference, False,		
/*		      
       XtNprefWidth,  100,
       XtNminWidth,   50,
       XtNprefHeight, 75,
       XtNminHeight,  50,
*/
       XtNalignment,  gaFill,
       XtNstretch,    True,
       XtNmargin,     6,
       NULL );
   
   dlglist = XtVaCreateManagedWidget( "dlglist",
       listWidgetClass, dlgview,
       XtNdefaultColumns, 1,
       XtNforceColumns,   True,
       NULL );
   XtAddCallback( dlglist, XtNcallback, DialogList, popup );
   XtOverrideTranslations( dlglist, listtrans );
   
   dialog = XtVaCreateManagedWidget( "dialog",
       groupWidgetClass, dlgform,
       XtNborderWidth, 1,
       XtNspacing,     4,
       XtNmargin,      6,
       XtNalignment,   gaFill,
       NULL );

   dlglbl = XtVaCreateManagedWidget( "label",
       labelWidgetClass, dialog,
       XtNborderWidth, 0,
       XtNlabel,       "Enter context:",
       XtNmargin,      6,
       NULL );

   value = XtVaCreateManagedWidget( "value",
       icaTextWidgetClass, dialog,
       XtNeditType,    XawtextEdit,
       XtNstring,      "",
       XtNalignment,   gaFill,
       XtNmargin,      6,
       NULL );

   dlgbtns = XtVaCreateManagedWidget( "buttons",
       groupWidgetClass, dialog,
       XtNorientation, XtorientHorizontal,
       XtNalignment,   gaCenter,
       XtNspacing,     6,
       XtNmargin,      0,
       NULL );

   dlgok = XtVaCreateManagedWidget( "Ok",
       commandWidgetClass, dlgbtns,
       XtNborderWidth, 2,
       XtNalignment,   gaCenter,
       XtNmargin,      2,
       NULL );
   XtAddCallback( dlgok, XtNcallback, DialogOkButton, popup );

   dlgcancel = XtVaCreateManagedWidget( "Cancel",
       commandWidgetClass, dlgbtns,
       XtNalignment,   gaCenter,
       XtNmargin,      2,
       NULL );
   pdptr->shell_widget  = popup;
   pdptr->enable_widget = NULL;
   XtAddCallback( dlgcancel, XtNcallback, XtCallbackPopdown, pdptr );

   dlghelp = XtVaCreateManagedWidget( "Help",
       commandWidgetClass, dlgbtns,
       XtNsensitive,   False,
       XtNalignment,   gaCenter,
       XtNmargin,      2,
       NULL );

   XtOverrideTranslations( value, dlgtrans );
   
   XtSetKeyboardFocus( dlgform, value );

   XtRealizeWidget(popup);
   
   return;
}
