/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: generate.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.12 $
   Date:        $Date: 92/03/16 15:01:25 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	generate.c,v $
 * Revision 1.12  92/03/16  15:01:25  ware
 * Make titles consistent.
 * 
 * Revision 1.11  1992/03/10  19:46:42  cso
 * Removed the plus operator to aid in symbol recognition
 *
 * Revision 1.10  1992/02/25  02:55:06  cso
 * Fixed a string compare of wrong types
 *
 * Revision 1.9  1992/02/20  04:55:54  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.8  1992/02/06  04:49:25  cso
 * Modified the End tag code per ultrix user comment
 *
 * Revision 1.7  1992/01/28  18:41:04  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "chtab.h"
#include "contab.h"
#include "output.h"
#include "retag.h"
#include "status.h"
#include "tables.h"



static void
Output(fptr, str)

FILE
   *fptr;
String
   str[];

{
   int
      i;

   for (i = 0; str[i] != 0; i++)
   {
      fprintf(fptr, "%s\n", str[i]);
   }
   return;
}



static void
PrintStartConds(fptr)

FILE
   *fptr;

{
   int i;
   
   if (Ctxtndx != 0)
   {
      fprintf(fptr, "%%START ");
      for (i = 0; i < Ctxtndx; i++)
      {
	 fprintf(fptr, "%s ", Ctxttab[i]);
      }
      fprintf(fptr, "\n");
   }
}



static void
PrintCdataRules(fptr, chndx)

FILE
   *fptr;
int
   chndx;

{
   char
      abuff[15], /* buffer to build alphabetic cdata string in. */
      nbuff[10], /* buffer to build numeric cdata string in.     */
      sbuff[50]; /* buffer to build special cdata string in.     */

   MakeCdata(abuff, nbuff, sbuff, chndx);
   if (chndx == 0)
   {
      fprintf(fptr, "[%s]+\t{\n", abuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "[%s]+\t{\n", nbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "[%s]\t{\n", sbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
   }
   else
   {
      fprintf(fptr, "<%s>[%s]+\t{\n", Ctxttab[chndx - 1], abuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "<%s>[%s]+\t{\n", Ctxttab[chndx - 1], nbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
      fprintf(fptr, "<%s>[%s]\t{\n", Ctxttab[chndx - 1], sbuff);
      fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
      fprintf(fptr, "\t return(ICA_Char_DATA);\n");
      fprintf(fptr, "\t}\n\n");
   }
   return;
}



static void
PrintSCLexRules(fptr)

FILE
   *fptr;

{
   int
      i,
      j;

   /*
    * Every recognized token is being made into a string to be passed
    * over to the yacc side of things because it is only then that we
    * know whether the recognized token is ambiguous in that it could
    * either be in the text production or as part of a tag production.
    * On the yacc side these strings are currently NOT being freed
    * because of memory allocation efficiency, some day they may have
    * to be.
    */
   for (i = 0; i < Ctxtndx; i++)
   {
      for (j = 0; j < Lexndx; j++)
      {
	 if (strcmp(Lextab[j].context, Ctxttab[i]) == 0)
	 {
	    switch(Lextab[j].ccode)
	    {
	    default:
	       break;

	    case Stop:
	       fprintf(fptr,
		       "<%s>%s\t{\n", Lextab[j].context, Lextab[j].regexp);
	       fprintf(fptr, "\t BEGIN pop(%s);\n", Lextab[j].context);
	       fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	       fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	       fprintf(fptr, "\t return(TOKEN%d);\n", j);
	       fprintf(fptr, "\t}\n\n");
	       UpdateChartab(Lextab[j].regexp, i + 1);
	       break;

	    case In:
	       fprintf(fptr, "<%s>%s\t{\n",
		       Lextab[j].context, Lextab[j].regexp);
	       fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	       fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	       fprintf(fptr, "\t return(TOKEN%d);\n", j);
	       fprintf(fptr, "\t}\n\n");
	       UpdateChartab(Lextab[j].regexp, i + 1);
	       break;

	    case Nest:
	       fprintf(fptr, "<%s>%s\t{\n",
		       Lextab[j].context, Lextab[j].regexp);
	       /* have to access the new field in lex, whatever its */
	       /* name is. */
	       /* What will that new field be called?
	       fprintf(fptr, "\t BEGIN %s;\n", Lextab[i].context2);
	       fprintf(fptr, "\t push(%s);\n", Lextab[i].context2);
	       */
	       fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	       fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	       fprintf(fptr, "\t return(TOKEN%d);\n", j);
	       fprintf(fptr, "\t}\n\n");
	       UpdateChartab(Lextab[j].regexp, i + 1);
	       break;
	    }
	 }
      }
      PrintCdataRules(fptr, i + 1);
   }
   return;
}



static void
PrintLexRules(fptr)

FILE
   *fptr;

{
   int
      i;

   /*
    * Every recognized token is being made into a string to be passed
    * over to the yacc side of things because it is only then that we
    * know whether the recognized token is ambiguous in that it could
    * either be in the text production or as part of a tag production.
    * On the yacc side these strings are currently NOT being freed
    * because of memory allocation efficiency, some day they may have
    * to be.
    */
   for (i = Lexndx - 1; i >= 0; i--)
   {
      if (Lextab[i].ccode == Nope)
      {
	 fprintf(fptr, "%s\t{\n", Lextab[i].regexp);
	 fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	 fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	 fprintf(fptr, "\t return(TOKEN%d);\n", i);
	 fprintf(fptr, "\t}\n\n");
      }
      else if (Lextab[i].ccode == Start)
      {
	 fprintf(fptr, "%s\t{\n", Lextab[i].regexp);
	 fprintf(fptr, "\t BEGIN %s;\n", Lextab[i].context);
	 fprintf(fptr, "\t push(%s);\n", Lextab[i].context);
	 fprintf(fptr, "\t yylval.data = makestr(yytext);\n");
	 fprintf(fptr, "\t linecount  += cntlinefeeds(yytext);\n");
	 fprintf(fptr, "\t return(TOKEN%d);\n", i);
	 fprintf(fptr, "\t}\n\n");
      }
   }
   PrintCdataRules(fptr, 0);
   return;
}



static void
PrintTokens(fptr)

FILE
   *fptr;

{
   int
      i;

   for (i = 0; i < Lexndx; i++)
   {
      fprintf(fptr, "TOKEN%d\n", i);
   }
   return;
}



static void
PrintDocProds(fptr)

FILE
   *fptr;

{
   int
      i;
   Boolean
      firstprod;

   firstprod = True;
   for (i = 0; i < Yaccndx; i++)
   {
      if (Yacctab[i].root == True)
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "document\t: ");
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
	 if (Yacctab[i].segflag == True)
	 {
	    fprintf(fptr, "textortagsstar TOKEN%d textortagsplus ",
		    Yacctab[i].lexndx);
	    fprintf(fptr, "TOKEN%d textortagsstar\n", Yacctab[i].endndx);
	    fprintf(fptr, "\t\t{\n");
	    fprintf(fptr, "\t\t $$ = structconcat($1, ");
	    fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	    fprintf(fptr, "$3, ");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "$5, NULL);\n");
	 }
	 else
	 {
	    switch (Yacctab[i].tcode)
	    {
	    default:
	       break;

	    case Begin:
	       fprintf(fptr, "textortagsstar TOKEN%d textortagsplus\n",
		       Yacctab[i].lexndx);
	       fprintf(fptr, "\t\t{\n");
	       fprintf(fptr, "\t\t $$ = structconcat($1, ");
	       fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	       fprintf(fptr, "$3, NULL);\n");
	       break;

	    case EndT:
	       fprintf(fptr, "textortagsplus TOKEN%d textortagsstar\n", 
		       Yacctab[i].lexndx);
	       fprintf(fptr, "\t\t{\n");
	       fprintf(fptr, "\t\t $$ = structconcat($1, ");
	       fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	       fprintf(fptr, "$3, NULL);\n");
	       break;
	    }
	 }
	 fprintf(fptr, "\t\t structprint($$);\n");
	 fprintf(fptr, "\t\t}\n");
      }
   }

   if (firstprod)
   {
      fprintf(fptr, "document\t: textortagsplus\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat($1, NULL);\n");
      fprintf(fptr, "\t\t structprint($$);\n");
      fprintf(fptr, "\t\t}\n");
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



static void
PrintNSProds(fptr)

FILE
   *fptr;

{
   int
      i;
   Boolean
      firstprod;

   firstprod = True;
   for (i = 0; i < Yaccndx; i++)
   {
      if ((Yacctab[i].segflag == False) && (Yacctab[i].root == False))
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "nonsegmenting\t: TOKEN%d\n", Yacctab[i].lexndx);
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| TOKEN%d\n", Yacctab[i].lexndx);
	 }
	 fprintf(fptr, "\t\t{\n");
	 switch (Yacctab[i].tcode)
	 {
	 default:
	    break;

	 case Begin:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case Comma:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case EndT:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case Symbol:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"&%s;\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;

	 case ProcIn:
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<?%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    break;
	 }
	 fprintf(fptr, "\t\t}\n");
      }
   }
   if (firstprod)
   {
      fprintf(fptr, "nonsegmenting\t: /* null */\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"));\n");
      fprintf(fptr, "\t\t}\n");
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



static void
PrintSProds(fptr)

FILE
   *fptr;

{
   int
      i;
   Boolean
      firstprod;

   firstprod = True;
   for (i = 0; i < Yaccndx; i++)
   {
      if ((Yacctab[i].segflag == True) && (Yacctab[i].root == False))
      {
	 if (firstprod)
	 {
	    fprintf(fptr, "segmenting\t: ");
	    firstprod = False;
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
	 if ((strcmp(NOREGEXP, Lextab[Yacctab[i].lexndx].regexp) != 0) &&
	     (strcmp(NOREGEXP, Lextab[Yacctab[i].endndx].regexp) != 0))
	 {
	    fprintf(fptr, "TOKEN%d textortagsstar ", Yacctab[i].lexndx);
	    fprintf(fptr, "TOKEN%d\n",               Yacctab[i].endndx);
	    fprintf(fptr, "\t\t{\n");
	    fprintf(fptr, "\t\t $$ = structconcat(");
	    fprintf(fptr, "structnew(\"<%s>\"), ",  Yacctab[i].tag);
	    fprintf(fptr, "$2, ");
	    fprintf(fptr, "structnew(\"</%s>\"), ", Yacctab[i].tag);
	    fprintf(fptr, "NULL);\n");
	    fprintf(fptr, "\t\t}\n");
	 } /***** IF neither regular expression is empty */
      } /***** IF the segmenting flag is true and not the root */
   } /***** FOR each symbol in the Yacc table */

   if (firstprod)
   {
      fprintf(fptr, "segmenting\t: /* null */\n");
      fprintf(fptr, "\t\t{\n");
      fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"));\n");
      fprintf(fptr, "\t\t}\n");
   } /***** IF nothing got generated for segmenting tags */

   fprintf(fptr, "\t\t;\n\n");
   return;
}



static void
PrintAmbCharProds(fptr)

FILE
   *fptr;

{
   int
      i;
   
   for (i = 0; i < Lexndx; i++)
   {
      if (Lextab[i].amb == True)
      {
	 fprintf(fptr, "\t\t| TOKEN%d\n", i);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat(structnew($1), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
      }
   }
   fprintf(fptr, "\t\t;\n\n");
   return;
}



void
GenerateLex(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      lexfile;
   
   lexfile = XtMalloc(strlen(filestr) + 8);
   if (lexfile == NULL)
   {
      fprintf(stderr, "GenerateLex: unable to allocate space\n");
      exit(2);
   }
   sprintf(lexfile, "%s.l", filestr);
   if  ((fptr = fopen(lexfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open lex file [%s]\n", lexfile);
      XtFree(lexfile);
      return;
   }
   CreateCtxttab();
   InitChartab(Ctxtndx);
   Output(fptr, headlex);
   PrintStartConds(fptr);
   Output(fptr, midlex);
   PrintSCLexRules(fptr);
   PrintLexRules(fptr);
   Output(fptr, taillex);
   fclose(fptr);
   XtFree(lexfile);
   return;
}



void
GenerateYacc(filestr)

String
   filestr;

{
   FILE
      *fptr;
   String
      yaccfile;
   
   yaccfile = XtMalloc(strlen(filestr) + 8);
   if (yaccfile == NULL)
   {
      fprintf(stderr, "GenerateYacc: unable to allocate space\n");
      exit(2);
   }
   sprintf(yaccfile, "%s.y", filestr);
   if  ((fptr = fopen(yaccfile, "w")) == NULL)
   {
      XicaStatus(Statw, "Unable to open yacc file [%s]\n", yaccfile);
      XtFree(yaccfile);
      return;
   }
   Output(fptr, headyacc);
   PrintTokens(fptr);
   Output(fptr, midyacc1);
   PrintDocProds(fptr);
   Output(fptr, midyacc2);
   PrintNSProds(fptr);
   PrintSProds(fptr);
   Output(fptr, midyacc3);
   PrintAmbCharProds(fptr);
   Output(fptr, tailyacc);
   fclose(fptr);
   XtFree(yaccfile);
   return;
}

