/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: list.c,v $
   Purpose:     Contains the list functions common to RTTinfo.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.10 $
   Date:        $Date: 92/05/29 15:27:52 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	list.c,v $
 * Revision 1.10  92/05/29  15:27:52  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.9  92/03/16  15:01:36  ware
 * Make titles consistent.
 * 
 * Revision 1.8  1992/02/25  02:52:58  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.7  1992/02/20  04:55:48  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.6  1992/02/06  04:49:19  cso
 * Modified the End tag code per ultrix user comment
 *
 * Revision 1.5  1992/01/28  18:40:56  cso
 * Restructured dependencies
 *
 * Revision 1.4  1991/12/05  15:22:14  cso
 * Added the necessary code to accommodate the new separator tag editor:
 *   - the saving of info
 *   - the loading of info
 *   - tracking the focus widget
 *
 * Changed the Unmanaging/Managing calls to FormLayout False/True calls.
 *
 * Revision 1.3  1991/11/07  22:18:23  cso
 * *** empty log message ***
 *
 * Revision 1.2  1991/11/04  16:51:47  cso
 * generate.c, output.h: Upgraded the generation routines to use
 * structconcat() for two orders of magnitude speed up!  (Thanks to jbm
 * for the help).
 *
 * list.c: fixed some uninitialized variables as detected by QA_C
 *
 * menucalls.c: Added the ifdef FLEX and yyrestart call for a flex
 * generated scanner
 *
 * Revision 1.1.1.2  1991/10/04  20:19:54  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "retag.h"
#include "utils.h"

/*  GLOBALS  */

static String
   *Liststr;
Widget
   Expfocus,
   Impfocus,
   Pinfocus,
   Sepfocus,
   Symfocus;



/*
   FUNCTION: 		ResetTaglist

   Synopsis: Reset the list of tags.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ResetTaglist(head)

Tagspec
   *head;

{
   Tagspec
      *tptr1,
      *tptr2;
   
   for (tptr1 = head; tptr1 != NULL; )
   {
      tptr2 = tptr1->next;
      DisposeTagptr(tptr1);
      tptr1 = tptr2;
   }
   return;
}

   


/*
   FUNCTION: 		ResetConlist

   Synopsis: Reset the list of tags.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ResetConlist(head)

ContextList
   *head;

{
   ContextList
      *cptr1,
      *cptr2;
   
   for (cptr1 = head; cptr1 != NULL; )
   {
      cptr2 = cptr1->next;
      XtFree(cptr1->cond);
      XtFree(cptr1);
      cptr1 = cptr2;
   }
   return;
}

   


/*
   FUNCTION: 		SaveTaglist

   Synopsis: Save a tag list out to a file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
SaveTaglist(fptr, head)

FILE
   *fptr;
Tagspec
   *head;

{
   Tagspec
      *tptr;
   
   for (tptr = head; tptr != NULL; tptr = tptr->next)
   {
      if (*tptr->tag != '\0')
      {
	 if (*tptr->regexp == '\0')
	 {
	    fprintf(fptr, "%s\t", NOREGEXP);
	 }
	 else
	 {
	    fprintf(fptr, "%s\t", tptr->regexp);
	 }
	 switch (tptr->tcode)
	 {
	 default:
	    break;
	    
	 case Symbol:
	    fprintf(fptr, "&%s;",  tptr->tag);
	    break;
	    
	 case ProcIn:
	    fprintf(fptr, "<?%s>", tptr->tag);
	    break;
	    
	 case Comma:
	    if (strcmp(tptr->tag, RTTinfo.root) == 0)
	    {
	       fprintf(fptr, "<#,%s>", tptr->tag);
	    }
	    else
	    {
	       fprintf(fptr, "<,%s>",  tptr->tag);
	    }
	    break;
	    
	 case Begin:
	 case ImBeg:
	 case ExBeg:
	    if (strcmp(tptr->tag, RTTinfo.root) == 0)
	    {
	       fprintf(fptr, "<#%s>", tptr->tag);
	    }
	    else
	    {
	       fprintf(fptr, "<%s>",  tptr->tag);
	    }
	    break;
	    
	 case EndT:
	 case ImEnd:
	 case ExEnd:
	    if (strcmp(tptr->tag, RTTinfo.root) == 0)
	    {
	       fprintf(fptr, "<#/%s>", tptr->tag);
	    }
	    else
	    {
	       fprintf(fptr, "</%s>",  tptr->tag);
	    }
	    break;
	 } /***** SWITCH on the tag code */
	 if (strcmp(tptr->context, "None") != 0)
	 {
	    fprintf(fptr, "\t:%s", tptr->context);
	 }
	 fprintf(fptr, "\n");
      } /***** IF the tag is not an empty string "" */
   } /***** FOR each tag in a list */
   
   return;
}




/*
   FUNCTION: 		AddContext

   Synopsis: Add a context string to the context list.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AddContext(constr)

String
   constr;

{
   ContextList
      *cptr,
      *cptr1,
      *cptr2;
   String
      clptr,
      clause,
      upcond,
      upcond1;

   clause = XtNewString(constr);
   /*
     if context = nest: then find the next part of the context, add
     it, then come back strip that part of the string out and add the
     nest context
     */
   for (clptr = clause; ((*clptr != ':') && (*clptr != '\0')); clptr++)
   {
   }
   if (*clptr == ':')
   {
      clptr++;
      clptr++;
   }
   else
   {
      clptr = clause;
   }
   cptr       = XtNew(ContextList);
   cptr->cond = XtNewString(clptr);
   cptr->next = NULL;
   if (RTTinfo.conptr == NULL)
   {
      RTTinfo.conptr = cptr;
   }
   else
   {
      cptr2  = RTTinfo.conptr;
      upcond = MakeUpper(cptr->cond);
      for (cptr1 = RTTinfo.conptr; cptr1 != NULL; cptr1 = cptr1->next)
      {
	 upcond1 = MakeUpper(cptr1->cond);
	 if (strcmp(cptr1->cond, cptr->cond) == 0)
	 {
	    XtFree(cptr->cond);
	    XtFree(cptr);
	    break;
	 }
	 else if (strcmp(upcond1, upcond) > 0)
	 {
	    if (RTTinfo.conptr == cptr1)
	    {
	       cptr->next     = cptr1;
	       RTTinfo.conptr = cptr;
	       break;
	    }
	    else
	    {
	       cptr->next  = cptr1;
	       cptr2->next = cptr;
	       break;
	    }
	 }
	 XtFree(upcond1);
	 cptr2 = cptr1;
      }
      XtFree(upcond);
      if (cptr1 == NULL)
      {
	 cptr2->next = cptr;
      }
   }
   XtFree(clause);
   return;
}




/*
   FUNCTION: 		InsertTag

   Synopsis: Insert a tag alphabetically.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
InsertTag(head, tptr)

Tagspec
   *head,
   *tptr;

{
   String
      uptag,
      uptag1;
   Tagspec
      *tptr1,
      *tptr2;

   uptag = MakeUpper(tptr->tag);
   tptr2 = head;
   for (tptr1 = head; tptr1 != NULL; tptr1 = tptr1->next)
   {
      uptag1 = MakeUpper(tptr1->tag);
      if (strcmp(uptag1, uptag) > 0)
      {
	 tptr->next = tptr1;
	 XtFree(uptag1);
	 break;
      }
      XtFree(uptag1);
      tptr2 = tptr1;
   }
   if ((tptr2 == head) && (tptr1 == head))
   {
      if (head == RTTinfo.symptr)
      {
	 RTTinfo.symptr = tptr;
      }
      else if (head == RTTinfo.pinptr)
      {
	 RTTinfo.pinptr = tptr;
      }
   }
   else
   {
      tptr2->next = tptr;
   }
   XtFree(uptag);
   return;
}



/*
   FUNCTION: 		AddToList

   Synopsis: Add tag information into the data structure.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
AddToList(code, tagstr, rexstr, constr, oldflag)

TagCode
   code;
String
   tagstr,
   rexstr,
   constr;
int
   oldflag;

{
   String
      ident;
   Tagspec
      *head,
      *tptr,
      *tptr1,
      *tptr2;
   
   head  = NULL;
   ident = StripTagString(tagstr);
   tptr  = NewTagptr(ident, code);
   if (strcmp(rexstr, NOREGEXP) == 0)
   {
      tptr->regexp = XtNewString("");
   }
   else
   {
      tptr->regexp = XtNewString(rexstr);
   }
   if (*constr == '\0')
   {
      constr = XtNewString("None");
   }
   else
   {
      AddContext(constr);
   }
   
   switch (code)
   {
   default:
      break;
      
   case Symbol:
      tptr->tcode   = Symbol;
      tptr->context = XtNewString(constr);
      if (RTTinfo.symptr == NULL)
      {
	 RTTinfo.symptr = tptr;
      }
      else
      {
	 head = RTTinfo.symptr;
	 InsertTag(head, tptr);
	 return;
      }
      break;

   case ProcIn:
      tptr->tcode = ProcIn;
      tptr->context = XtNewString(constr);
      if (RTTinfo.pinptr == NULL)
      {
	 RTTinfo.pinptr = tptr;
      }
      else
      {
	 head = RTTinfo.pinptr;
	 InsertTag(head, tptr);
	 return;
      }
      break;

   case Comma:
      tptr->tcode   = Comma;
      tptr->context = XtNewString(constr);
      if (RTTinfo.sepptr == NULL)
      {
	 RTTinfo.sepptr = tptr;
      }
      else
      {
	 head = RTTinfo.sepptr;
      }
      break;
      
   case ImBeg:
      if ((tagstr[1] == '/') || (tagstr[2] == '/'))
      {
	 tptr->tcode = ImEnd;
      }
      else
      {
	 tptr->tcode = ImBeg;
      }
      tptr->context = XtNewString(constr);
      if (RTTinfo.impptr == NULL)
      {
	 RTTinfo.impptr = tptr;
      }
      else
      {
	 head = RTTinfo.impptr;
      }
      break;

   case ExBeg:
      if ((tagstr[1] == '/') || (tagstr[2] == '/'))
      {
	 tptr->tcode = ExEnd;
      }
      else
      {
	 tptr->tcode = ExBeg;
      }
      if (oldflag == 0)
      {
	 tptr->context = XtNewString(constr);
      }
      else
      {
	 tptr->context = CorrectContext(constr, tptr->tcode);
      }
      if (RTTinfo.expptr == NULL)
      {
	 RTTinfo.expptr = tptr;
      }
      else
      {
	 head = RTTinfo.expptr;
      }
      break;
   }
   
   if (head != NULL)
   {
      tptr2 = head;
      for (tptr1 = head; tptr1 != NULL; tptr1 = tptr1->next)
      {
	 tptr2 = tptr1;
      }
      tptr2->next = tptr;
   }

   if (tagstr[1] == '#')
   {
      RTTinfo.root = XtNewString(ident);
   }
   XtFree(ident);
   return;
}



/*
   FUNCTION: 		ResetLists()

   Synopsis: Reset all of the lists in the data structure.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
       pete	3/3/92	 I've changed the names of the editor widgets created
       			 in CreateEditorShell() so the title is set in the
			 resources instead of being hardcoded.  Needed to
			 update the XtNameToWidget() calls for the new names.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ResetLists()

{
   Widget
      expedit,
      impedit,
      pinedit,
      sepedit,
      symedit;
   
   Symfocus = NULL;
   Pinfocus = NULL;
   Sepfocus = NULL;
   Impfocus = NULL;
   Expfocus = NULL;
   
   symedit = XtNameToWidget(Toplevel, "*symsel-editor*editorform");
   pinedit = XtNameToWidget(Toplevel, "*pinsel-editor*editorform");
   sepedit = XtNameToWidget(Toplevel, "*sepsel-editor*editorform");
   impedit = XtNameToWidget(Toplevel, "*impsel-editor*editorform");
   expedit = XtNameToWidget(Toplevel, "*expsel-editor*editorform");

   XawFormDoLayout(symedit, False);
   XawFormDoLayout(pinedit, False);
   XawFormDoLayout(sepedit, False);
   XawFormDoLayout(impedit, False);
   XawFormDoLayout(expedit, False);
   
   ResetTaglist(RTTinfo.symptr);
   ResetTaglist(RTTinfo.pinptr);
   ResetTaglist(RTTinfo.sepptr);
   ResetTaglist(RTTinfo.impptr);
   ResetTaglist(RTTinfo.expptr);
   ResetConlist(RTTinfo.conptr);

   RTTinfo.symptr = NULL;
   RTTinfo.pinptr = NULL;
   RTTinfo.sepptr = NULL;
   RTTinfo.impptr = NULL;
   RTTinfo.expptr = NULL;
   RTTinfo.conptr = NULL;

   XawFormDoLayout(symedit, True);
   XawFormDoLayout(pinedit, True);
   XawFormDoLayout(sepedit, True);
   XawFormDoLayout(impedit, True);
   XawFormDoLayout(expedit, True);
   return;
}




/*
   FUNCTION: 		UpdateList

   Synopsis: Insert a context into the context list alphabetically.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
UpdateList(popup)

Widget
   popup;

{
   ContextList
      *cptr;
   int
      listptr;
   Widget
      dlglist;
   
   dlglist = XtNameToWidget(popup, "*dlglist");
   
   XtFree(Liststr);
   Liststr = NULL;

   listptr = 0;
   for (cptr = RTTinfo.conptr; cptr != NULL; cptr = cptr->next)
   {
      listptr++;
   }
   if (listptr == 0)
   {
      Liststr = (String *) XtCalloc(1, sizeof(String));
      Liststr[0] = NULL;
   }
   else
   {
      Liststr = (String *) XtCalloc(listptr, sizeof(String));
      for (listptr = 0, cptr = RTTinfo.conptr;
	   cptr != NULL;
	   listptr++, cptr = cptr->next)
      {
	 Liststr[listptr] = cptr->cond;
      }
   }
   
   XawListChange(dlglist, Liststr, listptr, 0, True);
   return;
}
