/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menubar.c,v $
   Purpose:     Create and handle the menubar.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.8 $
   Date:        $Date: 92/05/29 15:27:59 $

NOTE:
   AVAIL can be:
	HIDDEN for internal use only
	EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	menubar.c,v $
 * Revision 1.8  92/05/29  15:27:59  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.7  92/03/16  15:01:45  ware
 * Make titles consistent.
 * 
 * Revision 1.6  1992/01/30  14:22:53  cso
 * The help option is commented out until help is actually provided.
 *
 * Revision 1.5  1992/01/28  18:45:01  cso
 * Restructured dependencies.
 * Added border around menubar.
 *
 * Revision 1.4  1991/12/05  15:23:22  cso
 * Added the separator tag option in the editor menu.
 *
 * Revision 1.3  1991/11/27  00:29:15  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.2  1991/11/09  16:42:48  cso
 * Made some minor changes to the keyboard focus feedback mechanism so
 * that the interface does not shift around too much.
 * Added a function ShiftScrollbars to keep them up to date, but it did
 * not have the desired effect, so nothing calls it at the moment.
 *
 * Revision 1.1  1991/10/04  19:39:24  jbm
 * Initial revision
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"


/*  EXTERNS */

#include "callbacks.h"
#include "dialog.h"
#include "list.h"
#include "menucalls.h"
#include "retag.h"
#include "tagedit.h"
#include "Xica/help.h"

/*  GLOBALS */

Widget
   Contextmenu;




/*
   FUNCTION: 		CreateFileMenu

   Synopsis: Create the file menu choices with callbacks.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CreateFileMenu(parent)

Widget
   parent;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      editsel,  /* Edit  option widget */
      mergesel, /* Merge option widget */
      newsel,   /* New   option widget */
      quitsel,  /* Quit  option widget */
      resetsel, /* Reset option widget */
      savesel;  /* Save  option widget */
   
   /*
    * Offer RESET first, a line to separate, NEW, EDIT, MERGE and SAVE,
    * another line separator, and lastly QUIT
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Reset    "); cnt++;
   resetsel = XtCreateManagedWidget("resetsel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(resetsel, XtNcallback, Reset, NULL); 

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "New      "); cnt++;
   newsel = XtCreateManagedWidget("newsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(newsel, XtNcallback, NewFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Edit     "); cnt++;
   editsel = XtCreateManagedWidget("editsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(editsel, XtNcallback, EditFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Merge    "); cnt++;
   mergesel = XtCreateManagedWidget("mergesel", smeBSBObjectClass,
				    parent, args, cnt);
   XtAddCallback(mergesel, XtNcallback, MergeFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Save     "); cnt++;
   savesel = XtCreateManagedWidget("savesel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(savesel, XtNcallback, SaveFile, NULL);

   cnt = 0;
   XtSetArg(args[cnt], XtNlineWidth, 3); cnt++;
   (void) XtCreateManagedWidget("linesep", smeLineObjectClass,
				parent, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Quit     "); cnt++;
   quitsel = XtCreateManagedWidget("quitsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(quitsel, XtNcallback, Quit, NULL);
   return;
}



/*
   FUNCTION: 		CreateContextMenu

   Synopsis: Create the menu for all context clauses.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateContextMenu()

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      item;
   
   cnt = 0;
   Contextmenu = XtCreatePopupShell("Contextmenu", simpleMenuWidgetClass,
				    Toplevel, args, cnt);

   cnt = 0;
   item = XtCreateManagedWidget("IN", smeBSBObjectClass, Contextmenu,
				args, cnt);
   XtAddCallback(item, XtNcallback, ContextMenuSelect, In);

   cnt = 0;
   item = XtCreateManagedWidget("START", smeBSBObjectClass, Contextmenu,
				args, cnt);
   XtAddCallback(item, XtNcallback, ContextMenuSelect, Start);

   cnt = 0;
   item = XtCreateManagedWidget("END", smeBSBObjectClass, Contextmenu,
				args, cnt);
   XtAddCallback(item, XtNcallback, ContextMenuSelect, Stop);

   cnt = 0;
   item = XtCreateManagedWidget("None", smeBSBObjectClass, Contextmenu,
				args, cnt);
   XtAddCallback(item, XtNcallback, ContextMenuSelect, Nope);

   return;
}



/*
   FUNCTION: 		CreateEditorMenu

   Synopsis: Create the editor menu selections.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
       pete	3/3/92	No change here, but I've modified CreateEditorShell
       			so it does not set the "title" of the shell.  If
			one adds more editors here, be sure to add a resource
			for the title in the Fallback resources.

			There are several portions of code that rely on the
			names of the created widgets.  Look for calls
			to XtNameToWidget().

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CreateEditorMenu(parent)

Widget
   parent;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      symsel,  /* Symbol                 option widget */
      pinsel,  /* Processing Instruction option widget */
      sepsel,  /* Separator Tags         option widget */
      impsel,  /* Implicit Tag           option widget */
      expsel,  /* Explicit Tag           option widget */
      temp;    /* Temporary widget for the popup shell editors
		  that are created so that the option will be made
		  insensitive after popup and then sensitive again
		  when popped down. */
   

   CreateContextDialog();
   CreateContextMenu();
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Symbol Tags"); cnt++;
   symsel = XtCreateManagedWidget("symsel", smeBSBObjectClass,
				  parent, args, cnt);
   temp = CreateEditorShell(symsel);
   XtAddCallback(symsel, XtNcallback, XtCallbackNone, temp);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Processing Instructions"); cnt++;
   pinsel = XtCreateManagedWidget("pinsel", smeBSBObjectClass,
				  parent, args, cnt);
   temp = CreateEditorShell(pinsel);
   XtAddCallback(pinsel, XtNcallback, XtCallbackNone, temp);

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Separator Tags"); cnt++;
   sepsel = XtCreateManagedWidget("sepsel", smeBSBObjectClass,
				  parent, args, cnt);
   temp = CreateEditorShell(sepsel);
   XtAddCallback(sepsel, XtNcallback, XtCallbackNone, temp);
   XtSetSensitive(XtNameToWidget(temp, "*remove"), False);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Implicit Segmenting Tags"); cnt++;
   impsel = XtCreateManagedWidget("impsel", smeBSBObjectClass,
				  parent, args, cnt);
   temp = CreateEditorShell(impsel);
   XtAddCallback(impsel, XtNcallback, XtCallbackNone, temp);
   XtSetSensitive(XtNameToWidget(temp, "*remove"), False);
   
   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Explicit Segmenting Tags"); cnt++;
   expsel = XtCreateManagedWidget("expsel", smeBSBObjectClass,
				  parent, args, cnt);
   temp = CreateEditorShell(expsel);
   XtAddCallback(expsel, XtNcallback, XtCallbackNone, temp);
   XtSetSensitive(XtNameToWidget(temp, "*remove"), False);

   ResetLists();
   UpdateEditors(False);
   return;
}




/*
   FUNCTION: 		CreateCodeMenu

   Synopsis: Create the code generation menu.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CreateCodeMenu(parent)

Widget
   parent;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      makesel,  /* Make option widget */
      runsel;   /* Run  option widget */

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Make      "); cnt++;
   makesel = XtCreateManagedWidget("makesel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(makesel, XtNcallback, Make, NULL); 

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Run       "); cnt++;
   runsel = XtCreateManagedWidget("runsel", smeBSBObjectClass,
				  parent, args, cnt);
   XtAddCallback(runsel, XtNcallback, Run, NULL); 
   return;
}


/*
   FUNCTION: 		CreateHelpMenu

   Synopsis: Create the help menu.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
CreateHelpMenu(parent)

Widget
   parent;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      helpsel;  /* Help option widget */

   cnt = 0;
   XtSetArg(args[cnt], XtNlabel, "Introduction"); cnt++;
   helpsel = XtCreateManagedWidget("helpsel", smeBSBObjectClass,
				   parent, args, cnt);
   XtAddCallback(helpsel, XtNcallback, XicaHelp, "retag.hlp");
   return;
}



/*
   FUNCTION: 		CreateMenubar

   Synopsis: Create the menubar and its selections.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget CreateMenubar( parent )
Widget parent;
{
   Widget
      codebut,
      codemenu,
      editbut,
      editormenu,
      filebut,
      filemenu,
      helpbut,
      helpmenu,
      menubar;

   menubar = XtVaCreateManagedWidget( "menubar",
       groupWidgetClass, parent,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     1,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       NULL );
   
   /*********** File menu **************/

   filebut = XtVaCreateManagedWidget( "File",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   File   ",
       XtNmenuName, "filemenu",
       XtNmargin,   1,
       XtNstretch,  True,
       NULL );

   filemenu = XtVaCreatePopupShell( "filemenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateFileMenu(filemenu);

   /*********** Tag Editors **************/

   editbut = XtVaCreateManagedWidget( "TagEditors",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "  Tag Editors  ",
       XtNmenuName, "editormenu",
       XtNmargin,   1,
       XtNstretch,  True,
       NULL );

   editormenu = XtVaCreatePopupShell( "editormenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateEditorMenu(editormenu);

   /*********** Code Generation **************/

   codebut = XtVaCreateManagedWidget( "Code",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   Code   ",
       XtNmenuName, "codemenu",
       XtNmargin,   1,
       XtNstretch,  True,
       NULL );

   codemenu = XtVaCreatePopupShell( "codemenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateCodeMenu(codemenu);

#ifndef NOHELP
   helpbut = XtVaCreateManagedWidget( "Help",
       menuButtonWidgetClass, menubar,
       XtNlabel,    "   Help   ",
       XtNmenuName, "helpmenu",
       XtNmargin,   1,
       XtNstretch,  True,
       NULL );

   helpmenu = XtVaCreatePopupShell( "helpmenu",
       simpleMenuWidgetClass, parent,
       NULL );

   CreateHelpMenu(helpmenu);
#endif /* NOHELP */

   return(menubar);
}
