/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: menucalls.c,v $
   Purpose:     Contains the menu callbacks for the menu selections.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.16 $
   Date:        $Date: 92/05/29 15:28:05 $

NOTE:
   AVAIL can be:
	HIDDEN for internal use only
	EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	menucalls.c,v $
 * Revision 1.16  92/05/29  15:28:05  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.15  92/03/16  15:01:52  ware
 * Make titles consistent.
 * 
 * Revision 1.14  1992/03/10  19:49:30  cso
 * Fixed a Merge error; rearranged the order of file querying in Run
 *
 * Revision 1.13  1992/02/25  02:54:21  cso
 * Provided the use of a general Makefile
 *
 * Revision 1.12  1992/02/21  05:47:57  cso
 * Added a cancel option to the save changes dialog
 *
 * Revision 1.11  1992/02/20  04:56:42  cso
 * Fixed a problem with using yyrestart
 *
 * Revision 1.10  1992/02/11  16:42:26  cso
 * Corrected a problem with the order of resetting the interface without
 * regard of the possibility that the user may want to cancel the action.
 *
 * Revision 1.9  1992/02/06  05:01:18  cso
 * Fixed the titles that appear in the file dialogs to be correct
 * depending on context.
 *
 * Revision 1.8  1992/01/31  14:44:12  cso
 * Added more informative messages regarding the state of the tool and
 * what is occurring (during makes and runs).  Right now, these system
 * calls are no longer being done in the background so as to give better
 * information in the interface.
 *
 * Revision 1.7  1992/01/28  18:46:37  cso
 * Restructured dependencies.
 * Trying to eliminate the bogus messages during loads.
 * Corrected (Merge) for the new spec language.
 *
 * Revision 1.6  1992/01/02  23:37:14  cso
 * Upgraded to the Xica widget set from Xlwh.  Using Message boxes
 * instead of the alert shells, and the new file dialog.
 *
 * Revision 1.5  1991/12/05  15:42:02  cso
 * Fixed initialization problem in the rsf compiler by making the
 * context, reg. exp., and tag variables global, and initializing before
 * calling yyparse.
 *
 * With the addition of the separator tag editor and making it backward
 * compatible, I needed to update the tag editor and data structure with
 * old info, but I needed to work around the Dirtyflag setting as well,
 * so the user will know to save the new info structure.
 *
 * Added the appropriate sequence in the Merge function to support the
 * separator tag editor.
 *
 * Revision 1.4  1991/11/27  00:29:22  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.3  1991/11/07  21:32:03  cso
 * Checked that the user wanted to load a new file before resetting the
 * interface.
 *
 * Revision 1.2  1991/11/04  16:51:53  cso
 * generate.c, output.h: Upgraded the generation routines to use
 * structconcat() for two orders of magnitude speed up!  (Thanks to jbm
 * for the help).
 *
 * list.c: fixed some uninitialized variables as detected by QA_C
 *
 * menucalls.c: Added the ifdef FLEX and yyrestart call for a flex
 * generated scanner
 *
 * Revision 1.1.1.2  1991/10/04  20:20:03  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS */

#include "alpha.h"
#include "chtab.h"
#include "contab.h"
#include "fileio.h"
#include "generate.h"
#include "list.h"
#include "retag.h"
#include "status.h"
#include "tables.h"
#include "tagedit.h"
#include "utils.h"

/* Externals from rcomp.y */

extern char
   *Ycontxt,
   *Yregexp,
   *Ytagstr;
extern FILE
   *yyin;
extern int
   Ambigflag,
   Expflag,
   linecount,
   Loadflag,
   Oldflag;

/* External reference to Flex yyrestart */

#ifdef FLEX
extern void
   yyrestart();
#endif



/*
   FUNCTION: 		SaveChanges

   Synopsis: 
       Check if the  current session has changed a grammar description.
       Offer the user to save or forget changes AND if unsuccessful
       in saving let them know and let them leave the function
       invoking this one without losing the changes.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON
	3/5/92	pete	Working on making titles consistent

   Globals used:
       Dirtyflag  

   Globals Modified:
       none

   Return value:	Boolean

   Range:		True or False
*/

static Boolean
SaveChanges()

{
   Boolean
      continue_oper,
      response;
   FILE
      *fptr;

   /*
    * If the current session has changed, give the user a chance to
    * save the file or at least realize that changes have occurred.
    */
   continue_oper = True;
   if (Dirtyflag == True)
   {
      /*
       * Ask the user if they wish to save the changes, if Continue, 
       * then get a file name and save the file
       */
      continue_oper = YesNoCancelMessage(Toplevel, "Save changes?",
					 True, &response);
      
      if ((continue_oper == True) && (response == True))
      {
	 /*
	  * Get the file to save the current session
	  */
	 fptr = OpenFile("Save a specification [.rsf]",
			 "Save:",
			 "Save",
			 "*.rsf",
			 "rsf",
			 False);
	 
	 /*
	  * If successful in getting a file, save out the current session
	  * Else alert the user that changes will be lost
	  */
	 if (fptr != NULL)
	 {
	    fprintf(fptr, "DATA ALPHABET:\t%s\n",      RTTinfo.data);
	    fprintf(fptr, "TAG ALPHABET:\t%s\n",       RTTinfo.tags);
	    fprintf(fptr, "AMBIGUOUS ALPHABET:\t%s\n", RTTinfo.ambg);
	    
	    fprintf(fptr, "SPECIAL SYMBOLS:\n");
	    SaveTaglist(fptr, RTTinfo.symptr);
	    fprintf(fptr, "PROCESSING INSTRUCTIONS:\n");
	    SaveTaglist(fptr, RTTinfo.pinptr);
	    fprintf(fptr, "IMPLICIT TAGS:\n");
	    SaveTaglist(fptr, RTTinfo.impptr);
	    fprintf(fptr, "EXPLICIT TAGS:\n");
	    SaveTaglist(fptr, RTTinfo.expptr);
	    fprintf(fptr, "SEPARATOR TAGS:\n");
	    SaveTaglist(fptr, RTTinfo.sepptr);

	    fclose(fptr);
	    Dirtyflag = False;
	 } /***** IF the user chooses to save changes */
	 else
	 {
	    /*
	     * If the user does not wish to lose changes,
	     * then return False, preventing whatever action would lose
	     * the changes from occurring.
	     */
	    return(YesNoMessage(Toplevel, "Lose changes?", False));
	 } /***** ELSE a file was not chosen */
      } /***** IF the user wanted to save changes */
   } /***** IF the dirty bit is set and there is something to save */
   return(continue_oper);
}



/*
   FUNCTION: 		Reset

   Synopsis: Reset the interface and data structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
Reset(w, client_data, call_data)

Widget
   w;		/* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;	/* Unused */

{
   if (SaveChanges() == True)
   {
      /*
       * Initialize devegram, after confirming with user
       */
      if (YesNoMessage(Toplevel, "Reset interface?", True) == True)
      {
	 ResetLists();
	 XtFree(RTTinfo.data);
	 XtFree(RTTinfo.tags);
	 XtFree(RTTinfo.ambg);
	 XtFree(RTTinfo.root);
	 RTTinfo.data = XtNewString(ALPHABET);
	 RTTinfo.tags = XtNewString("");
	 RTTinfo.ambg = XtNewString("");
	 RTTinfo.root = XtNewString("");
	 UpdateInterface();
	 Dirtyflag = False;
      }
   }
   return;
}



/*
   FUNCTION: 		NewFile

   Synopsis: Reads in an ICA file to prime the editors.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
NewFile(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;
   
   if (SaveChanges() == True)
   {
      fptr = OpenFile("Load an ICA Specification [.ica]",
		      "Load:",
		      "Load",
		      "*.ica",
		      "ica",
		      True);

      if (fptr != NULL)
      {
	 ResetLists();
	 XicaStatus(Statw, "Loading new specification from ICA file\n");
	 if (ReadICA(fptr) == True)
	 {
	    UpdateEditors(True);
	    Dirtyflag = True;
	    XicaStatus(Statw, "ICA file load completed.\n");
	    XicaStatus(Statw, "Do not forget to SAVE this specification!\n");
	 }
	 else
	 {
	    XicaStatus(Statw, "ICA file load aborted.\n");
	 }
      }
   }
   return;
}



/*
   FUNCTION: 		EditFile

   Synopsis: Load in a previously saved out specification file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
	3/5/92	pete	Working on making titles consistent

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
EditFile(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   Boolean
      tempflag;
   
   if (SaveChanges() == True)
   {
      yyin = OpenFile("Edit a specification [.rsf]",
		      "Edit:",
		      "Edit",
		      "*.rsf",
		      "rsf",
		      True);

      if (yyin != NULL)
      {
	 ResetLists();
	 XicaStatus(Statw, "Loading Retag Spec. from RSF file\n");
	 Loadflag = 1;
	 Oldflag  = 0;
	 linecount = 1;
	 
	 Dirtyflag = False;
	 tempflag  = False;
	 if (yyparse() == 0)
	 {
	    if (Dirtyflag == True)
	    {
	       tempflag = True;
	       XicaStatus(Statw, "An old RSF file was loaded and upgraded\n");
	       XicaStatus(Statw, "Please remember to save.\n");
	    }
	    UpdateInterface();
	    UpdateEditors(True);
	    Dirtyflag = tempflag;
	    XicaStatus(Statw, "RSF file load completed.\n");
	 }
	 else
	 {
	    XicaStatus(Statw, "RSF file load aborted.\n");
	 }
	 fclose(yyin);
#ifdef FLEX
	 yyrestart(yyin);
#endif
      }

#ifdef DEBUG
      PrintSpec();
#endif
   }
   return;
}



/*
   FUNCTION: 		MergeFile

   Synopsis: Merge in an ICA file with an RSF file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
	3/5/92	pete	Working on making titles consistent

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
MergeFile(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   Boolean
      rootflag;
   char
      c,
      junkstr[BUFSIZ];  /* Consume unimportant info from the ICA file */
   FILE
      *fptr;
   int
      filestatus,
      maxtag;
   String
      ident;
   Tagspec 
      *eptr,
      *eptr1,
      *eptr2,
      *iptr,
      *iptr1,
      *iptr2,
      *sptr,
      *sptr1,
      *sptr2;
   
   if (SaveChanges() == False)
   {
      return;
   }
   yyin = OpenFile("Merging a specification [.rsf]\n\
 with an ICA Specification [.ica]",
		   "Load:",
		   "Load",
		   "*.rsf",
		   "rsf",
		   True);

   if (yyin == NULL)
   {
      return;
   }
   
   fptr = OpenFile("Merging a specification [.rsf]\n\
 with an ICA Specification [.ica]",
		   "Merge:",
		   "Merge",
		   "*.ica",
		   "ica",
		   True);
   if (fptr == NULL)
   {
      return;
   }
   
   ResetLists();
   XicaStatus(Statw, "Loading Retag Spec. from RSF file\n");
   Loadflag  = 1;
   Oldflag   = 0;
   linecount = 1;
   if (yyparse() == 0)
   {
      /*
       * Now, we have both an old (rsf) and a new (ica) file
       * In the ica file, find the marker and read until the
       * end of that line.
       * Now read and merge the new file into the old file.
       */
      while ((fscanf(fptr, "%s", junkstr) != EOF) &&
	     (strcmp(junkstr, "####")     != 0))
      {
      }
      if (strcmp(junkstr, "####") != 0)
      {
	 XicaStatus(Statw, "Unable to load ica file\n");
	 return;
      }
      
      while (((c = fgetc(fptr)) != '\n') && (c != EOF))
      {
      }
      if (c == EOF)
      {
	 XicaStatus(Statw, "Unable to load ica file\n");
	 return;
      }
      
      fscanf(fptr, "%d", &maxtag);
      ident = (String) XtMalloc((unsigned) maxtag + 8);
      
      eptr1 = RTTinfo.expptr;
      eptr2 = eptr1;
      iptr1 = RTTinfo.impptr;
      iptr2 = iptr1;
      sptr1 = RTTinfo.sepptr;
      sptr2 = sptr1;
      
      rootflag = False;
      
      filestatus = fscanf(fptr, "%s", ident);
      while ((filestatus != EOF) && (strcmp(ident, "####") != 0) &&
	     (eptr1 != NULL))
      {
#ifdef DEBUG
	 fprintf(stderr, "ica ident = [%s]\n", ident);
#endif
	 if (ident[0] == '#')
	 {
#ifdef DEBUG
	    fprintf(stderr, "root = [%s]\n", ident);
#endif
	    rootflag = True;
	    ident++;
	    if (strcmp(RTTinfo.root, ident) != 0)
	    {
	       RTTinfo.root = XtNewString(ident);
	    }
	 } /***** IF the read in symbol is the root symbol */
	 if (strcmp(ident, eptr1->tag) == 0)
	 {
	    while ((eptr1 != NULL) && (strcmp(ident, eptr1->tag) == 0))
	    {
	       eptr2 = eptr1;
	       eptr1 = eptr1->next;
	    }
	    while ((iptr1 != NULL) && (strcmp(ident, iptr1->tag) == 0))
	    {
	       iptr2 = iptr1;
	       iptr1 = iptr1->next;
	    }
	    while ((sptr1 != NULL) && (strcmp(ident, sptr1->tag) == 0))
	    {
	       sptr2 = sptr1;
	       sptr1 = sptr1->next;
	    }
	    if (rootflag == True)
	    {
	       ident--;
	       rootflag = False;
	    }
	    filestatus = fscanf(fptr, "%s", ident);
	 } /***** IF a match exists */
	 else if (strcmp(ident, eptr1->tag) < 0)
	 {
	    XicaStatus(Statw, "Inserting tag [%s] to specification\n",
		       ident);
	    
	    eptr = NewTagptr(ident, ExBeg);
	    if (eptr1 == RTTinfo.expptr)
	    {
	       RTTinfo.expptr = eptr;
	    }
	    else
	    {
	       eptr2->next = eptr;
	    }
	    eptr->next  = eptr1;
	    eptr2       = eptr;
	    
	    eptr        = NewTagptr(ident, ExEnd);
	    eptr->next  = eptr1;
	    eptr2->next = eptr;
	    eptr2       = eptr;
	    
	    iptr = NewTagptr(ident, ImBeg);
	    if (iptr1 == RTTinfo.impptr)
	    {
	       RTTinfo.impptr = iptr;
	    }
	    else
	    {
	       iptr2->next = iptr;
	    }
	    iptr->next = iptr1;
	    iptr2      = iptr;
	    
	    sptr = NewTagptr(ident, Comma);
	    if (sptr1 == RTTinfo.sepptr)
	    {
	       RTTinfo.sepptr = sptr;
	    }
	    else
	    {
	       sptr2->next = sptr;
	    }
	    sptr->next = sptr1;
	    sptr2      = sptr;
	    
	    if (rootflag == True)
	    {
	       ident--;
	       rootflag = False;
	    }
	    filestatus = fscanf(fptr, "%s", ident);
	 } /***** ELSE ident is inserted into structure */
	 else
	 {
	    XicaStatus(Statw, "Removing tags from old specification:\n");
	    while ((eptr1 != NULL) && (strcmp(ident, eptr1->tag) > 0))
	    {
	       if (eptr1->tcode == ExBeg)
	       {
		  XicaStatus(Statw, " Explicit tag spec:\nStart: %s\t%s\t%s\n",
			     eptr1->tag, eptr1->regexp, eptr1->context);
	       }
	       else
	       {
		  XicaStatus(Statw, " Explicit tag spec:\nEnd  : %s\t%s\t%s\n",
			     eptr1->tag, eptr1->regexp, eptr1->context);
	       }
	       
	       if (eptr2 == RTTinfo.expptr)
	       {
		  RTTinfo.expptr = eptr1->next;
		  eptr2          = RTTinfo.expptr;
		  DisposeTagptr(eptr1);
		  eptr1 = eptr2;
	       }
	       else
	       {
		  eptr2->next = eptr1->next;
		  DisposeTagptr(eptr1);
		  eptr1 = eptr2->next;
	       }
	    } /***** WHILE old tags exist */
	    
	    while ((iptr1 != NULL) && (strcmp(ident, iptr1->tag) > 0))
	    {
	       if (iptr1->tcode == ImBeg)
	       {
		  XicaStatus(Statw, " Implicit tag spec:\nStart: %s\t%s\t%s\n",
			     iptr1->tag, iptr1->regexp, iptr1->context);
	       }
	       else
	       {
		  XicaStatus(Statw, " Implicit tag spec:\nEnd  : %s\t%s\t%s\n",
			     iptr1->tag, iptr1->regexp, iptr1->context);
	       }
	       
	       if (iptr2 == RTTinfo.impptr)
	       {
		  RTTinfo.impptr = iptr1->next;
		  iptr2          = RTTinfo.impptr;
		  DisposeTagptr(iptr1);
		  iptr1 = iptr2;
	       }
	       else
	       {
		  iptr2->next = iptr1->next;
		  DisposeTagptr(iptr1);
		  iptr1 = iptr2->next;
	       }
	    } /***** WHILE old tags exist */
	    
	    while ((sptr1 != NULL) && (strcmp(ident, sptr1->tag) > 0))
	    {
	       XicaStatus(Statw, " Separator tag spec:\n\t%s\t%s\t%s\n",
			  sptr1->tag, sptr1->regexp, sptr1->context);
	       if (sptr2 == RTTinfo.sepptr)
	       {
		  RTTinfo.sepptr = sptr1->next;
		  sptr2          = RTTinfo.sepptr;
		  DisposeTagptr(sptr1);
		  sptr1 = sptr2;
	       }
	       else
	       {
		  sptr2->next = sptr1->next;
		  DisposeTagptr(sptr1);
		  sptr1 = sptr2->next;
	       }
	    } /***** WHILE old tags exist */
	 } /***** ELSE old tags need to disposed */
      } /***** WHILE new tags exist to consider */
      
      while ((filestatus != EOF) && (strcmp(ident, "####") != 0))
      {
#ifdef DEBUG
	 fprintf(stderr, "ica ident = [%s]\n", ident);
#endif
	 if (ident[0] == '#')
	 {
#ifdef DEBUG
	    fprintf(stderr, "root = [%s]\n", ident);
#endif
	    rootflag = True;
	    ident++;
	    if (strcmp(RTTinfo.root, ident) != 0)
	    {
	       RTTinfo.root = XtNewString(ident);
	    }
	 } /***** IF the read in symbol is the root symbol */
	 eptr        = NewTagptr(ident, ExBeg);
	 eptr2->next = eptr;
	 eptr2       = eptr;
	 eptr        = NewTagptr(ident, ExEnd);
	 eptr2->next = eptr;
	 eptr2       = eptr;
	 
	 iptr        = NewTagptr(ident, ImBeg);
	 iptr2->next = iptr;
	 iptr2       = iptr;
	 
	 sptr        = NewTagptr(ident, Comma);
	 sptr2->next = sptr;
	 sptr2       = sptr;
	 
	 if (rootflag == True)
	 {
	    ident--;
	    rootflag = False;
	 }
	 filestatus = fscanf(fptr, "%s", ident);
      } /***** WHILE more new tags need to be added */
      
      eptr2->next = NULL;
      iptr2->next = NULL;
      sptr2->next = NULL;
      
      if (eptr1 != NULL)
      {
	 XicaStatus(Statw, "Removing tag from old specification:\n");  
	 while (eptr1 != NULL)
	 {
	    eptr2 = eptr1->next;
	    XicaStatus(Statw, " Explicit tag spec:\n\t%s\t%s\t%s\n",
		       eptr1->tag, eptr1->regexp, eptr1->context);
	    DisposeTagptr(eptr1);
	    eptr1 = eptr2;
	 } /***** WHILE old tags exist */
	 
	 while (iptr1 != NULL)
	 {
	    iptr2 = iptr1->next;
	    XicaStatus(Statw, " Implicit tag spec:\n\t%s\t%s\t%s\n",
		       iptr1->tag, iptr1->regexp, iptr1->context);
	    DisposeTagptr(iptr1);
	    iptr1 = iptr2;
	 } /***** WHILE old tags exist */
	 
	 while (sptr1 != NULL)
	 {
	    sptr2 = sptr1->next;
	    XicaStatus(Statw, " Separator tag spec:\n\t%s\t%s\t%s\n",
		       sptr1->tag, sptr1->regexp, sptr1->context);
	    DisposeTagptr(sptr1);
	    sptr1 = sptr2;
	 } /***** WHILE old tags exist */
      } /***** IF old tags still need to be disposed */
      
      XtFree(ident);
      
      Dirtyflag = True;
      fclose(fptr);
      fclose(yyin);
   } /***** IF an old file was successfully loaded */
#ifdef FLEX
   if (fptr != NULL)
   {
      yyrestart(yyin);
   }
#endif
#ifdef DEBUG
   PrintSpec();
#endif
   UpdateInterface();
   UpdateEditors(True);
   return;
} /***** MergeFile() */



/*
   FUNCTION: 		SaveFile

   Synopsis: Save the RTT info into a file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
	3/5/92	pete	Working on making titles consistent

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
SaveFile(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   FILE
      *fptr;

   /*
    * If nothing has changed, give the user a chance to skip this
    */
   if (Dirtyflag == False)
   {
      if (YesNoMessage(Toplevel,
		       "No changes have been made.\n  Save anyway?",
		       False) == False)
      {
	 return;
      }
   }

   fptr = OpenFile("Save a specification [.rsf]",
		   "Save:",
		   "Save",
		   "*.rsf",
		   "rsf",
		   False);

   if (fptr != NULL)
   {
      XicaStatus(Statw, "Saving Retag Spec. to RSF file\n");

      fprintf(fptr, "DATA ALPHABET:\t%s\n",      RTTinfo.data);
      fprintf(fptr, "TAG ALPHABET:\t%s\n",       RTTinfo.tags);
      fprintf(fptr, "AMBIGUOUS ALPHABET:\t%s\n", RTTinfo.ambg);

      fprintf(fptr, "SPECIAL SYMBOLS:\n");
      SaveTaglist(fptr, RTTinfo.symptr);
      fprintf(fptr, "PROCESSING INSTRUCTIONS:\n");
      SaveTaglist(fptr, RTTinfo.pinptr);
      fprintf(fptr, "IMPLICIT TAGS:\n");
      SaveTaglist(fptr, RTTinfo.impptr);
      fprintf(fptr, "EXPLICIT TAGS:\n");
      SaveTaglist(fptr, RTTinfo.expptr);
      fprintf(fptr, "SEPARATOR TAGS:\n");
      SaveTaglist(fptr, RTTinfo.sepptr);
      fclose(fptr);
      Dirtyflag = False;
      
      XicaStatus(Statw,"Save Completed\n");
   }
   else
   {
      XicaStatus(Statw,"Save aborted\n");
   }
   return;
}



/*
   FUNCTION: 		Quit

   Synopsis: Quit out of the interface.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
Quit(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
#ifdef DEBUG
   fprintf(stderr, "Quit was selected\n");
#endif
   /*
    * Check if changes need to be saved.  If the user does not wish to lose
    * changes, but was unsuccessful in saving them, just return.
    */
   if (SaveChanges() == True)
   {
      ResetLists();
      XtDestroyWidget(Toplevel);
      exit(0);
   }
   return;
}



/*
   FUNCTION: 		Make

   Synopsis:  Make the specification files and compile them into a tool.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
	3/5/92	pete	Working on making titles consistent

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Make(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   String
      filestr,
      sysstr;
   
   yyin = OpenFile("Making a specification [.rsf] into a [.rx]",
		   "Load:",
		   "Load",
		   "*.rsf",
		   "rsf",
		   True);

   if (yyin != NULL)
   {
      filestr = GetFilename("Making a specification [.rsf] into a [.rx]",
			    "Make:",
			    "Make",
			    "*.rx",
			    "rx",
			    False);

      if (*filestr == '\0')
      {
	 fclose(yyin);
	 return;
      } /***** if the user did not choose an rsf file */
      
      ZeroChartab();
      Ambigflag = 0;
      Loadflag  = 0;
      Lexndx    = 0;
      Yaccndx   = 0;
      Ctxtndx   = 0;
      Ycontxt   = NULL;
      Yregexp   = NULL;
      Ytagstr   = NULL;
      linecount = 1;
      
      if (yyparse() == 0)
      {
	 XicaStatus(Statw, "Generating specifications for [%s]\n", filestr);
	 GenerateLex(filestr);
	 GenerateYacc(filestr);
	 FreeTables();

#ifdef MEMDEBUG
	 fprintf(stderr, "malloc_verify() returns [%d]\n", malloc_verify());
#endif

	 sysstr = XtMalloc(strlen(filestr) + ICAlen + 64);
	 sprintf(sysstr,
		 "make -f %s/lib/Makefile.gen 'TARGET=%s' 'MAINLIB=retag'",
		 ICApath, filestr);
#ifdef DEBUG
	 fprintf(stderr, "MAKE str = [%s]\n", sysstr);
#endif
	 XicaStatus(Statw, "Compiling [%s]\n", filestr);
	 errno = 0;
	 if (system(sysstr) < 0)
	 {
	    XicaStatus(Statw, "Compilation failed.\n");
	    perror("System call to MAKE replace tag tool failed");
	 } /***** IF the system call did not work */
	 else
	 {
	    XicaStatus(Statw, "Compilation completed.\n");
	 }
	 
	 XtFree(sysstr);
      } /***** IF an RSF file was successfully parsed */
      else
      {
	 XicaStatus(Statw, "There was an error in the specification.\n");
	 XicaStatus(Statw, "Compilation aborted.\n\n");
      } /***** ELSE there was an error in the spec */

      XtFree(filestr);

      fclose(yyin);
#ifdef FLEX
      yyrestart(yyin);
#endif
   } /***** The user chose a file to make from */
   
   return;
} /***** Make */



/*
   FUNCTION: 		Run

   Synopsis: Use a system call to run a previously compiled tool.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
Run(w, client_data, call_data)

Widget
   w;           /* Invoking widget */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   String
      execfile,
      infile,
      outfile,
      sysstr;

   /* pick a document to retag */
   infile = GetFilename("Choose an input file [*]",
			"Ok:",
			"Ok",
			"*",
			"*",
			True);
   if (*infile == '\0')
   {
      XtFree(infile);
      return;
   }
   
   /* choose an output file name */
   outfile = GetFilename("Choose an output file [*]\n\
 Cancel to use stdout",
			 "Ok:",
			 "Ok",
			 "*",
			 "*",
			 False);
   
   /* pick an executable */
   execfile = GetFilename("Execute a replace tag tool [.rx]",
			  "Run:",
			  "Run",
			  "*.rx",
			  "rx",
			  True);

   if (*execfile == '\0')
   {
      XtFree(infile);
      XtFree(outfile);
      XtFree(execfile);
      return;
   }

   if (*outfile != '\0')
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) +
			strlen(outfile) + 32);
      sprintf(sysstr, "%s < %s > %s", execfile, infile, outfile);
      XtFree(outfile);
   } /***** IF the user chose a file for output */
   else
   {
      sysstr = XtMalloc(strlen(execfile) + strlen(infile) + 32);
      sprintf(sysstr, "%s < %s", execfile, infile);
   } /***** ELSE put the output to stdout */
   
   XicaStatus(Statw, "Running [%s]\n", sysstr);
   errno = 0;
   if (system(sysstr) < 0)
   {
      XicaStatus(Statw, "Run failed\n");
      perror("System call to RUN replace tag tool failed");
   }
   else
   {
      XicaStatus(Statw, "Run completed\n");
   }
   
   XtFree(sysstr);
   XtFree(infile);
   XtFree(execfile);
   return;
} /***** Run() */

