/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: rcomp.y,v $
   Purpose:     To parse and validate a .rsf file.  If get_file ==
   True, it is used to load a file into the interface, else it is used to
   generate the temp.[yacc/lex] files from the Make option.

   Programmer:  Conleth O'Connell, Julie Barnes
   Reviewer:    
   Revision:    $Revision: 1.9 $
   Date:        $Date: 92/03/16 15:02:03 $

NOTE:
   AVAIL can be:
	HIDDEN for internal use only
	EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	rcomp.y,v $
 * Revision 1.9  92/03/16  15:02:03  ware
 * Make titles consistent.
 * 
 * Revision 1.8  1992/02/25  02:52:55  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.7  1992/02/20  04:55:35  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.6  1992/01/28  18:37:35  cso
 * Used allocation of constant strings so they can be freed
 *
 * Revision 1.5  1991/12/05  16:02:59  cso
 * Added the separator tag support code.
 * 	- changed the parsing language to find the sep. tag info
 * 	- added the capability of being backward compatible and
 * 		updating the data structures
 *
 * Changed the initialization of some variables.
 *
 * Added checks to catch user-modified spec files (that they aren't
 * supposed to do...|-), and to recover...
 *
 * Revision 1.4  1991/11/08  23:04:32  cso
 * Removed the possibility of generating bogus cpde from partial specs.
 * The situation was rare but possible, when a user would only specify
 * one of the two regular expressions necessary for explicit segmenting
 * tags.
 *
 * Revision 1.3  1991/11/07  21:59:49  cso
 * Modified the interface load parse to put a space after the context
 * clause keywords so that variable-width fonts will not make the
 * interface look bad.  Still need to adjust the dialog to do the same.
 * Although this changes the rsf spec language, the parse is backwards
 * compatible with old specifications so on subsequent loads and saves
 * the update is transparent to the user.
 *
 * Revision 1.2  1991/11/06  17:05:55  cso
 * *** empty log message ***
 *
 * Revision 1.1.1.2  1991/10/04  20:20:10  jbm
 * First HaL mods to ICA
 *
Revision 1.2  90/11/06  17:35:37  cso
Removed TABs.

Revision 1.1  90/10/05  13:19:13  cso
Initial revision

*/
/* Yacc specification for the retag specification language */
/* compiler.                                               */
/*                                                         */
/* Copyright 1990. The Ohio State University.              */

%{

#ifdef XGUI
#include "widgets.h"
#include "defs.h"
#include "types.h"

extern void
   AddToList(),
   CopyTags();

extern int
   linecount;
extern RTTspec
   RTTinfo;
extern Widget
   Statw;

TagCode
   Tcode;
#endif


extern char
   *concat();
extern void
   InstallAlfa(),
   InstallSpec();

char
   *Ycontxt,
   *Yregexp,
   *Ytagstr;
int
   Ambigflag,
   Expflag,
   Loadflag,
   Oldflag;
%}

%union
{
 int ival;
 char *trans;
}

%token <trans>
AMBIGCHAR
DATACHAR
DEFINE
DELTAG
END
EXPTAG
IMPTAG
IN
LINEFEED
NEST
PROCIN
SEPTAG
START
STRING
SYMBOL
TAGCHAR

%type <trans>
dataalph
datatagalph
delimiting
explicit
implicit
oldornew
oldtagline
oldtaglines
optctxtclause
optstring
procin
retagfile
segmenting
separatorOPTL
separator
symbol
tagalph
tagline
taglines
tagspec

%start retagfile

%%

retagfile	: dataalph tagalph datatagalph tagspec
		{
		}
		;

dataalph	: DATACHAR STRING LINEFEED
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      /*  place it in the data alphabet */
		      RTTinfo.data = XtNewString($2);
#endif
		   }
		   else
		   {
		      InstallAlfa($2, 1);
		   }
#ifdef XGUI
		   XtFree($2);
#else
		   free($2);
#endif
		}
		;

tagalph		: TAGCHAR optstring LINEFEED
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      /* place it in the tag alphabet */
		      RTTinfo.tags = XtNewString($2);
#endif
		   }
		   else
		   {
		      InstallAlfa($2, 2);
		   }
#ifdef XGUI
		   XtFree($2);
#else
		   free($2);
#endif
		}
		;

datatagalph	: AMBIGCHAR optstring LINEFEED
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      /* place it in ambiguous alphabet */
		      RTTinfo.ambg = XtNewString($2);
#endif
		   }
		   else
		   {
		      if ($2[0] != '\0')
		      {
			 Ambigflag = 1;
		      }
		      else
		      {
			 Ambigflag = 0;
		      }
		      InstallAlfa($2, 3);
		   }
#ifdef XGUI
		   XtFree($2);
#else
		   free($2);
#endif
		}
		;

optstring	: STRING
		{
		   $$ = concat($1, NULL);
		}
		| /* null */
		{
		   $$ = concat(NULL); 
		}
		;



tagspec		: symbol taglines procin taglines segmenting
		{
		}
		;

segmenting	: implicit taglines oldornew separatorOPTL
		{
		}
		;

oldornew	: delimiting oldtaglines
		{
		}
		| explicit taglines
		{
		}
		;

separatorOPTL	: separator taglines
		{
		}
		| /* empty */
		{
#ifdef XGUI
		   CopyTags();
#endif
		}
		;
   
symbol		: SYMBOL
		{
		   Expflag = 0;
		   Oldflag = 0;
#ifdef XGUI
		   Tcode = Symbol;
#endif
		}
		;

procin		: PROCIN
		{
#ifdef XGUI
		   Tcode = ProcIn;
#endif
		}
		;

implicit	: IMPTAG
		{
#ifdef XGUI
		   Tcode = ImBeg;
#endif
		}
		;

explicit	: EXPTAG
		{
		   Oldflag = 0;
		   Expflag = 1;
#ifdef XGUI
		   Tcode = ExBeg;
#endif
		}
		;

delimiting	: DELTAG
		{
		   Oldflag = 1;
		   Expflag = 1;
#ifdef XGUI
		   Tcode = ExBeg;
#endif
		}
		;

separator	: SEPTAG
		{
		   Expflag = 0;
#ifdef XGUI
		   Tcode = Comma;
#endif
		}
		;
   
tagline		: STRING STRING optctxtclause LINEFEED
		{
		   /* REXP TAG optctxtclause LINEFEED */
#ifdef DEBUG
		   fprintf(stderr,"tagline production\n"); 
#endif
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      AddToList(Tcode, $2, $1, $3, Oldflag);
		      XtFree($1);
		      XtFree($2);
		      XtFree($3);
#endif
		   }
		   else if (Expflag == 0)
		   {
		      InstallSpec($1, $2, $3, Expflag, Oldflag);
		      free($1);
		      free($2);
		      free($3);
		   }
		   else
		   {
		      if (Ytagstr == NULL)
		      {
			 Yregexp = $1;
			 Ytagstr = $2;
			 Ycontxt = $3;
		      } /***** IF the begin tag has been found */
		      else
		      {
			 int
			    i = 1;
			 
			 if (Ytagstr[i] == '#')
			 {
			    i++;
			 }
			 
			 if ((strcmp(&Ytagstr[i], &$2[i + 1]) == 0) &&
			     (strcmp(Yregexp, NOREGEXP) != 0) &&
			     (strcmp($1,      NOREGEXP) != 0))
			 {
			    InstallSpec(Yregexp, Ytagstr, Ycontxt,
					Expflag, Oldflag);
			    InstallSpec($1, $2, $3, Expflag, Oldflag);
			    free(Yregexp);
			    free(Ytagstr);
			    free(Ycontxt);
			    free($1);
			    free($2);
			    free($3);
			    Yregexp = NULL;
			    Ytagstr = NULL;
			    Ycontxt = NULL;
			 }
			 else if (strcmp(&Ytagstr[i], &$2[i + 1]) != 0)
			 {
#ifdef XGUI
			    XicaStatus(Statw, "Error in specification file");
			    XicaStatus(Statw, " on line [%d] : ",
				       linecount - 2);
			    XicaStatus(Statw, "start tag [%s] is unmatched\n",
				       Ytagstr);
			    XicaStatus(Statw, "Continuing...\n");
			    XtFree(Yregexp);
			    XtFree(Ytagstr);
			    XtFree(Ycontxt);
#else
			    fprintf(stderr, "Error in the [.rsf] file");
			    fprintf(stderr, " on line [%d] : ",
				    linecount - 2);
			    fprintf(stderr, "start tag [%s] is unmatched\n",
				    Ytagstr);
			    fprintf(stderr, "Continuing...\n");
			    free(Yregexp);
			    free(Ytagstr);
			    free(Ycontxt);
#endif
			    Yregexp = $1;
			    Ytagstr = $2;
			    Ycontxt = $3;
			 }
			 else
			 {
#ifdef XGUI
			    XtFree(Yregexp);
			    XtFree(Ytagstr);
			    XtFree(Ycontxt);
			    XtFree($1);
			    XtFree($2);
			    XtFree($3);
#else
			    free(Yregexp);
			    free(Ytagstr);
			    free(Ycontxt);
			    free($1);
			    free($2);
			    free($3);
#endif
			    Yregexp = NULL;
			    Ytagstr = NULL;
			    Ycontxt = NULL;
			 }
		      } /***** ELSE check the end tag spec */
		   } /***** ELSE compiling explicit seg. tags */
		}
		;

oldtagline	: STRING STRING STRING STRING optctxtclause LINEFEED
		{
		   /* REXP REXP BTAG ETAG optctxtclause LINEFEED */
#ifdef DEBUG
		   fprintf(stderr, "oldtagline production\n"); 
#endif
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      AddToList(Tcode, $3, $1, $5, Oldflag);
		      AddToList(Tcode, $4, $2, $5, Oldflag);
#endif
		   }
		   else
		   {
		      if ((strcmp(NOREGEXP, $1) != 0) &&
			  (strcmp(NOREGEXP, $2) != 0))
		      {
			 InstallSpec($1, $3, $5, Expflag, Oldflag);
			 InstallSpec($2, $4, $5, Expflag, Oldflag);
		      }
		   }
#ifdef XGUI
		   XtFree($1);
		   XtFree($2);
		   XtFree($3);
		   XtFree($4);
		   XtFree($5);
#else
		   free($1);
		   free($2);
		   free($3);
		   free($4);
		   free($5);
#endif
		}
		;

optctxtclause	: START STRING
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      $$ = concat(XtNewString("START: "), $2, NULL);
#endif
		   }
		   else
		   {
		      $$ = concat(XtNewString("S"), $2, NULL);
		   }
		}
		| END STRING
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      $$ = concat(XtNewString("END: "), $2, NULL);
#endif
		   }
		   else
		   {
		      $$ = concat(XtNewString("E"), $2, NULL);
		   }
		}
		| IN STRING
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      $$ = concat(XtNewString("IN: "), $2, NULL);
#endif
		   }
		   else
		   {
		      $$ = concat(XtNewString("I"), $2, NULL);
		   }
		}
		| NEST STRING IN STRING
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      $$ = concat(XtNewString("NEST: "), $2,
				  XtNewString("IN: "), $4, NULL);
#endif
		   }
		   else
		   {
		      $$ = concat(XtNewString("N"), $2, NULL);
		   }
		}
		| DEFINE STRING
		{
		   if (Loadflag == 1)
		   {
#ifdef XGUI
		      $$ = concat(XtNewString("DEFINES: "), $2, NULL);
#endif
		   }
		   else
		   {
		      $$ = concat(XtNewString("D"), $2, NULL);
		   }
		}
		| /* null */
		{
		   $$ = concat(NULL);
		}
		;

taglines	: taglines tagline
		{
		}
		| /* null */
		{
		}
		;

oldtaglines	: oldtaglines oldtagline
		{
		}
		| /* null */
		{
		}
		;

%%
