/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: retag.c,v $
   Purpose:     Contain the main().

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.9 $
   Date:        $Date: 92/05/29 15:28:11 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	retag.c,v $
 * Revision 1.9  92/05/29  15:28:11  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.8  92/05/20  11:31:20  ware
 * Removed unnecessary colors
 * 
 * Revision 1.7  92/03/16  15:02:06  ware
 * Make titles consistent.
 * 
 * Revision 1.6  1992/02/25  02:52:51  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.5  1992/01/28  18:40:51  cso
 * Restructured dependencies
 *
 * Revision 1.4  1991/11/27  15:57:42  cso
 * Made some aesthetic changes.
 *
 * Revision 1.3  1991/11/27  00:29:29  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.2  1991/11/09  16:38:44  cso
 * *** empty log message ***
 *
 * Revision 1.1.1.3  1991/10/04  20:20:13  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "alpha.h"
#include "callbacks.h"
#include "status.h"

/*  GLOBALS  */

RTTspec
   RTTinfo;
Widget
   Toplevel;                          /* toplevel widget, supreme parent */
XtAppContext
   Appcon;                     /* the application context         */
String
  ICApath;
int
  ICAlen;
static char
   *Fallbacks[] = {
   /************************/
   /***     Menubar     ***/
   /************************/
   /*** "*menubar*background:	gray", ***/
   /*** "*alphaform.background:	gray", ***/
   /*** "*datalabel.background:	gray", ***/
   /*** "*taglabel.background:	gray", ***/
   /*** "*amblabel.background:	gray", ***/
   "retag.title:		Replace Tags",
   "retag.iconName:		Replace Tags",
   "*symsel-editor.title:	Replace Tags: Symbol Tags",
   "*symsel-editor.iconName:	Replace Tags: Symbol Tags",
   "*pinsel-editor.title:	Replace Tags: Processing Instructions",
   "*pinsel-editor.iconName:	Replace Tags: Processing Instructions",
   "*sepsel-editor.title:	Replace Tags: Separator Tags",
   "*sepsel-editor.iconName:	Replace Tags: Separator Tags",
   "*impsel-editor.title:	Replace Tags: Implicit Segmenting Tags",
   "*impsel-editor.iconName:	Replace Tags: Implicit Segmenting Tags",
   "*expsel-editor.title:	Replace Tags: Explicit Segmenting Tags",
   "*expsel-editor.iconName:	Replace Tags: Explicit Segmenting Tags",
   /************************/
   /***     Sizing       ***/
   /************************/
   "*Statw.prefWidth:           500",
   "*Statw.minWidth:            100",
   "*Statw.prefHeight:          96",
   "*Statw.minHeight:           20",
   "*dlgview.prefWidth:         100",
   "*dlgview.minWidth:          50",
   "*dlgview.prefHeight:        75",
   "*dlgview.minHeight:         50",
   "*tagview.prefWidth:         200",
   "*tagview.minWidth:          100",
   "*tagview.prefHeight:        300",
   "*tagview.minHeight:         50",
   "*tagcol.stretch:            False",
   "*rexview.prefWidth:         300",
   "*rexview.minWidth:          100",
   "*rexview.prefHeight:        300",
   "*rexview.minHeight:         50",
   "*rexcol.stretch:            True",
   "*conview.prefWidth:         150",
   "*conview.minWidth:          100",
   "*conview.prefHeight:        300",
   "*conview.minHeight:         50",
   "*concol.stretch:            False",
   (char *) NULL
   };

static XtActionsRec
   TextActions[] = {
                     {"AlphaFocus",    AlphaFocus},
                     {"ContextSelect", ContextSelect},
                     {"DialogOkTrans", DialogOkTrans},
                     {"FocusSelect",   FocusSelect},
                     {"TagReturn",     TagReturn},
                     {"TextReturn",    TextReturn},
                     {NULL,            NULL}
		   };



/*
   FUNCTION: 		RTTInit

   Synopsis: This function initializes the RTTinfo data structure.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       RTTinfo

   Globals Modified:
       RTTinfo

   Return value:	void

   Range:		none
*/

static void
RTTInit()
{
    RTTinfo.data   = XtNewString(ALPHABET);
    RTTinfo.tags   = XtNewString("");
    RTTinfo.ambg   = XtNewString("");
    RTTinfo.root   = XtNewString("");
    RTTinfo.symptr = (Tagspec     *) NULL;
    RTTinfo.pinptr = (Tagspec     *) NULL;
    RTTinfo.impptr = (Tagspec     *) NULL;
    RTTinfo.expptr = (Tagspec     *) NULL;
    RTTinfo.conptr = (ContextList *) NULL;
    return;
}




/*
   FUNCTION: 		main

   Synopsis:  Initializes interface and then handles events.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


int
main(argc, argv)

int 
   argc;
char 
   **argv;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   
#ifdef MEMDEBUG
   fprintf(stderr, "malloc_debug() returns [%d]\n", malloc_debug(1));
#endif

   ICApath = (String) getenv("ICAROOT");
#ifdef DEBUG
   fprintf(stderr, "ICAROOT = [%s]\n", ICApath);
#endif
   if (ICApath == NULL)
   {
      ICApath = ".";
      XicaStatus(Statw, "WARNING: ICAROOT is not set\n");
      XicaStatus(Statw, "Using current directory\n");
   }
   ICAlen = strlen(ICApath);
   
   RTTInit();
   
   cnt = 0;
   XtSetArg(args[cnt], XtNallowShellResize, True);  cnt++;
   XtSetArg(args[cnt], XtNinput,            True);  cnt++;
#ifdef HARDCODEDTITLE
   XtSetArg(args[cnt], XtNtitle,            TITLE); cnt++;
#endif
   Toplevel = XtAppInitialize(&Appcon, TITLE, NULL, 0, &argc, argv,
			      Fallbacks, args, cnt);
   
#ifdef SYNC
   XSynchronize(XtDisplay(Toplevel), True);
#endif
   
   XtAppAddActions(Appcon, TextActions, XtNumber(TextActions));
   XawSimpleMenuAddGlobalActions(Appcon);

   CreateInterface();
   
   XtAppMainLoop(Appcon);

}

