/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: tables.c,v $
   Purpose:     

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.12 $
   Date:        $Date: 92/03/16 15:02:24 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------


   HISTORY
   $Log:	tables.c,v $
 * Revision 1.12  92/03/16  15:02:24  ware
 * Make titles consistent.
 * 
 * Revision 1.11  1992/02/25  02:52:49  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.10  1992/02/21  06:29:02  cso
 * Mistakenly freeing memory I shouldn't have
 *
 * Revision 1.9  1992/02/20  04:55:44  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.8  1992/02/11  16:43:44  cso
 * The commented out message also caused a problem - corrected it
 *
 * Revision 1.7  1992/02/06  04:56:11  cso
 * Removed an annoying warning message
 *
 * Revision 1.6  1992/01/28  18:40:42  cso
 * Restructured dependencies
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "chtab.h"
#include "status.h"
#include "utils.h"

/* External from rcomp.y */

extern int
   Ambigflag;

/*  GLOBALS  */

LexTable
   Lextab[TABLESIZE];
YaccTable
   Yacctab[TABLESIZE];
int
   Lexndx,
   Yaccndx;




int
InstallLex(regexp, context)

String
   regexp,
   context;

{
   ConCode
      clausecode;
   int
      i;
   
   /*
     Treat the nest as an IN type of clausecode
     */
   context = DetermineContext(context, &clausecode);
   for (i = 0; i < Lexndx; i++)
   {
      if (strcmp(Lextab[i].regexp, regexp) == 0)
      {
	 if (((Lextab[i].ccode == Nope)  && (clausecode == Start)) ||
	     ((Lextab[i].ccode == Start) && (clausecode == Nope)))
	 {
	    XicaStatus(Statw,
		    "\nWARNING The specification will not work as expected\n");
	    XicaStatus(Statw,
		       "The duplicate regular expression [%s]\n", regexp);
	    XicaStatus(Statw, "appears in the specification, once without a");
	    XicaStatus(Statw, "context clause, and at least once either");
	    XicaStatus(Statw, "starting or defining a context clause\n");
	    XicaStatus(Statw,
		    "Ignoring the duplicate with context [%s]\n\n",
		    context);
	    return(i);
	 } /***** IF the same reg. exp. has two action options */
	 else if (strcmp(Lextab[i].context, context) == 0)
	 {
	    if (((Lextab[i].ccode == Stop) &&
		 ((clausecode == In) || (clausecode == Nest))) ||
		(((Lextab[i].ccode == In) || (clausecode == Nest)) &&
		(clausecode == Stop)))
	    {
	       XicaStatus(Statw,
		       "WARNING: The specification will not work as expected");
	       XicaStatus(Statw,
		       "\nThe duplicate regular expression [%s]\n", regexp);
	       XicaStatus(Statw,
		       "appears in the specification, inside and ");
	       XicaStatus(Statw,
		       "either ending or defining the end");
	       XicaStatus(Statw,
		       "of the same context clause\n");
	       XicaStatus(Statw,
		       "Ignoring the duplicate with context [%s]\n\n",
		       context);
	       return(i);
	    } /***** ELSE IF the same reg. exp. has two context options */
	    else if (Lextab[i].ccode == clausecode)
	    {
	       /**********  Removing the following warning
	       XicaStatus(Statw,
		       "\nWARNING The specification has duplicate info\n");
	       XicaStatus(Statw, "The duplicate regular expression [%s]\n",
		       regexp);
	       XicaStatus(Statw, "appears in the specification, with ");
	       XicaStatus(Statw, "context clause [%s]\n", context);
	       XicaStatus(Statw, "Ignoring the duplicate\n\n");
	       ********* Removed message ************/
	       return(i);
	    } 
	    /***** ELSE IF the clause and reg. exp are the same */
	 } /***** ELSE IF the context and reg. exp are the same */
      } /***** IF the reg. exp is the same */
   } /***** FOR each entry in the lex table */

   /*
     for nest type this may be tricky, test for it and strip out the
     context that it starts and that it is within maybe add a new
     field in lex table to hold the two contexts for the code generator.
     */
   Lextab[Lexndx].ccode   = clausecode;
   Lextab[Lexndx].regexp  = XtNewString(regexp);
   Lextab[Lexndx].context = XtNewString(context);
   if (Ambigflag == 1)
   {
      if ((clausecode == Start) || (clausecode == Nope))
      {
	 Lextab[Lexndx].amb  = CheckAmbig(regexp);
      }
   }
   else
   {
      Lextab[Lexndx].amb  = False;
   }
   Lexndx++;
   return(Lexndx - 1);
} /* InstallLex */




void
InstallYacc(tagstr, segflag, lexndx, endndx)

String
   tagstr;
Boolean
   segflag;
int
   lexndx,
   endndx;

{
   char
      *stripstr;
   
   stripstr = StripTagString(tagstr);

   if (lexndx == -1)
   {
      Yacctab[Yaccndx - 1].endndx = endndx;
   }
   else if (lexndx >= 0)
   {
      Yacctab[Yaccndx].segflag = segflag;
      Yacctab[Yaccndx].root    = DetermineRoot(tagstr);
      Yacctab[Yaccndx].endndx  = endndx;
      Yacctab[Yaccndx].lexndx  = lexndx;
      Yacctab[Yaccndx].tcode   = DetermineTagCode(tagstr);
      Yacctab[Yaccndx].tag     = XtNewString(stripstr);
      Yaccndx++;
   }
   XtFree(stripstr);
   return;
} /* InstallYacc */




void
InstallSpec(regexp, tagstr, context, expflag, oldflag)

char
   *regexp,
   *tagstr,
   *context;
int
   expflag,
   oldflag;

{
   int
      index;
   TagCode
      tcode;

   if (strcmp(regexp, NOREGEXP) == 0)
   {
      return;
   }
   tcode = DetermineTagCode(tagstr);
   if ((tcode == Symbol) || (tcode == ProcIn) || (expflag == 0))
   {
      index = InstallLex(regexp, context);
      InstallYacc(tagstr, False, index, -1);
   }
   else if (tcode == Begin)
   {
      if (context[0] == 'D')
      {
	 context[0] = 'S';
      }
      else if ((context[0] == 'E') && (oldflag == 1))
      {
	 context[0] = 'I';
      }
      index = InstallLex(regexp, context);
      InstallYacc(tagstr, True, index, -1);
   }
   else
   {
      if (context[0] == 'D')
      {
	 context[0] = 'E';
      }
      else if ((context[0] == 'S') && (oldflag == 1))
      {
	 context[0] = '\0';
      }
      index = InstallLex(regexp, context);
      InstallYacc(tagstr, True, -1, index);
   }
   return;
}

