/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: tagedit.c,v $
   Purpose:     The Tag Editor module contains the creation routines
                   for the four tag editors.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.13 $
   Date:        $Date: 92/05/29 15:28:20 $

NOTE:
   AVAIL can be:
	HIDDEN for internal use only
	EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	tagedit.c,v $
 * Revision 1.13  92/05/29  15:28:20  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.11  92/03/16  15:02:31  ware
 * Make titles consistent.
 * 
 * Revision 1.10  1992/03/10  19:43:17  cso
 * Removed chaining of labels; rearranged scrollbars
 *
 * Revision 1.9  1992/02/06  04:52:29  cso
 * Added chaining to improve resizing looks
 *
 * Revision 1.8  1992/01/30  14:30:57  cso
 * Variety of mods:
 *  - Editor widgets have borderwidth 2, initialized to background, and
 * 	the display caret is hidden.
 *  - Global translations are capitalized.
 *  - The state for ending implicit tags is set to True.
 *  - Feedback is provided while the tag editors are creating their
 * 	widgets.
 *  - Keyboard focus is initialized in all tag editors.
 *
 * Revision 1.7  1992/01/28  18:47:35  cso
 * Restructured dependencies.
 * Changed chaining to prevent the collapse of regexp windows.
 *
 * Revision 1.6  1991/12/05  16:05:24  cso
 * Added the appropriate changes to incorporate the separator tag editor
 * (relatively minor).
 *
 * Revision 1.5  1991/11/27  00:29:40  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.4  1991/11/19  18:22:36  cso
 * Added the Enter translation to be the same as for Return.
 *
 * Revision 1.3  1991/11/14  15:02:06  cso
 * Added a translation for linefeed as well as return.
 *
 * Revision 1.2  1991/11/07  22:19:30  cso
 * Removed the possibility of cutting into the context clause widget.
 *
 * Revision 1.1.1.2  1991/10/04  20:20:22  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "alpha.h"
#include "callbacks.h"
#include "list.h"
#include "retag.h"
#include "status.h"
#include "utils.h"

/*  GLOBALS  */

XtTranslations 
   Menutrans,
   Tagtrans,
   Texttrans;
static String 
   menutransstr = "<Btn1Down>: ContextSelect() \
                               XawPositionSimpleMenu(Contextmenu) \
                               MenuPopup(Contextmenu)",
   tagtransstr  = "<Key>space: no-op()\n\
                   <Key>Tab:   no-op()",
   texttransstr = "<Key>Return:   TagReturn()\n\
                   <Key>KP_Enter: TagReturn()\n\
                   <Key>Linefeed: TagReturn()\n\
                   <Btn1Down>,<Btn1Up>: FocusSelect()";
   



/*
   FUNCTION: 		CreateEditorLayout

   Synopsis: 

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static Widget
CreateEditorLayout(parent)

Widget
   parent;

{
   Arg
      args[MAXARGS];
   int
      cnt,
      height,
      conwidth,
      rexwidth,
      tagwidth,
      minheight,
      minwidth;
   Widget
      conlab,
      conform,
      conview,
      menubar,
      done,
      duplicate,
      remove,
/*
      help,
*/
      table, labelrow, viewrow, tagcol,rexcol,concol,
      rexlab,
      rexform,
      rexview,
      taglab,
      tagform,
      tagview,
      vertical;
   
   height   = 300;
   tagwidth = 200;
   rexwidth = 300;
   conwidth = 150;
   minheight = 50;
   minwidth  = 100;

   menubar = XtVaCreateManagedWidget( "menubar",
       groupWidgetClass, parent,
       XtNorientation, XtorientHorizontal,
       XtNspacing,     1,
       XtNmargin,      8,
       NULL );

   done = XtVaCreateManagedWidget( "done",
       commandWidgetClass, menubar,
       XtNlabel,  "   Done   ",
       XtNmargin, 1,
       NULL );
   
   duplicate = XtVaCreateManagedWidget( "duplicate",
       commandWidgetClass, menubar,
       XtNlabel,  "  Duplicate  ",
       XtNmargin, 1,
       NULL );
   XtAddCallback( duplicate, XtNcallback, DuplicateTag, ShellParent(parent) );
   
   remove = XtVaCreateManagedWidget( "remove",
       commandWidgetClass, menubar,
       XtNlabel,  "   Remove   ",
       XtNmargin, 1,
       NULL );
   XtAddCallback( remove, XtNcallback, RemoveTag, ShellParent(parent) );
   
/*
   help = XtVaCreateManagedWidget( "help",
       commandWidgetClass, menubar,
       XtNlabel,  "   Help   ",
       XtNmargin, 1,
       NULL );
*/
   
   table = XtVaCreateManagedWidget( "table",
       tableWidgetClass, parent,
       XtNhorzSpacing,   8,
       XtNhorzPlacement, tpFlush,
       XtNalignment,     gaFill,
       XtNmargin,        8,
       XtNstretch,       True,
       NULL );
   labelrow = XtVaCreateWidget( "labelrow",
       tableRowObjectClass, table,
       NULL );
   viewrow = XtVaCreateWidget( "viewrow",
       tableRowObjectClass, table,
       XtNstretch, True,
       NULL );
   tagcol = XtVaCreateWidget( "tagcol",
       tableColObjectClass, table,
       NULL );
   rexcol = XtVaCreateWidget( "rexcol",
       tableColObjectClass, table,
       NULL );
   concol = XtVaCreateWidget( "concol",
       tableColObjectClass, table,
       NULL );

   taglab = XtVaCreateManagedWidget( "taglab",
       labelWidgetClass, table,
       XtNlabel,       "Tags:",
       XtNborderWidth, 0,
       XtNtableRow,    labelrow,
       XtNtableCol,    tagcol,
       NULL );

   rexlab = XtVaCreateManagedWidget( "rexlab",
       labelWidgetClass, table,
       XtNlabel,       "Regular Expressions:",
       XtNborderWidth, 0,
       XtNtableRow,    labelrow,
       XtNtableCol,    rexcol,
       NULL );

   conlab = XtVaCreateManagedWidget( "conlab",
       labelWidgetClass, table,
       XtNlabel,       "Context Clauses:",
       XtNborderWidth, 0,
       XtNtableRow,    labelrow,
       XtNtableCol,    concol,
       NULL );

   tagview = XtVaCreateManagedWidget( "tagview",
       viewportWidgetClass, table,
       XtNforceBars,     True,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNpreference,    False,
/*
       XtNprefHeight,    height,
       XtNminHeight,     minheight,
       XtNprefWidth,     tagwidth,
       XtNminWidth,      minwidth,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      tagcol,
       NULL );
   
   vertical = XtNameToWidget(tagview,"vertical");
   XtAddCallback(vertical, XtNscrollProc, ScrollTogether, ShellParent(parent));
   XtAddCallback(vertical, XtNjumpProc,   ScrollTogether, ShellParent(parent));
   
   tagform = XtVaCreateManagedWidget( "tagform",
       viewformWidgetClass, tagview,
       XtNresizable, True,
       NULL );
   
   rexview = XtVaCreateManagedWidget( "rexview",
       viewportWidgetClass, table,
       XtNforceBars,     True,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNuseRight,      True,
       XtNpreference,    False,
/*
       XtNprefHeight,    height,
       XtNminHeight,     minheight,
       XtNprefWidth,     rexwidth,
       XtNminWidth,      minwidth,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      rexcol,
       NULL );

   vertical = XtNameToWidget(rexview,"vertical");
   XtAddCallback(vertical, XtNscrollProc, ScrollTogether, ShellParent(parent));
   XtAddCallback(vertical, XtNjumpProc,   ScrollTogether, ShellParent(parent));

   XtUnmanageChild(vertical);
   XtVaSetValues( vertical,
       XtNwidth,       1,
       XtNborderWidth, 0,
       NULL );
   
   rexform = XtVaCreateManagedWidget( "rexform",
       viewformWidgetClass, rexview,
       XtNresizable, True,
       NULL );
   
   conview = XtVaCreateManagedWidget( "conview",
       viewportWidgetClass, table,
       XtNforceBars,     True,
       XtNallowHoriz,    True,
       XtNallowVert,     True,
       XtNuseRight,      True,
       XtNpreference,    False,
/*
       XtNprefHeight,    height,
       XtNminHeight,     minheight,
       XtNprefWidth,     conwidth,
       XtNminWidth,      minwidth,
*/
       XtNhorzAlignment, taFill,
       XtNvertAlignment, taFill,
       XtNtableRow,      viewrow,
       XtNtableCol,      concol,
       NULL );

   vertical = XtNameToWidget(conview,"vertical");
   XtAddCallback(vertical, XtNscrollProc, ScrollTogether, ShellParent(parent));
   XtAddCallback(vertical, XtNjumpProc,   ScrollTogether, ShellParent(parent));
   
   conform = XtVaCreateManagedWidget( "conform",
       viewformWidgetClass, conview,
       XtNresizable, True,
       NULL );
   
   return(done);
}



/*
   FUNCTION: 		CreateEditorShell

   Synopsis: Create the basic editor design - menus, buttons, and viewports

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
       pete	3/3/92	Making the names follow a common scheme.  Instead
       			of using the label of the "enable" widget, instead
			use the name of the "enable" widget and tack on
			"-editor".  Then rely on resources to set it
			appropriately.

			Note that this has ramifications in several other
			places.  In particular, one must update the Fallback
			resources and anywhere (e.g. list.c:ResetLists,
			callbacks.c:FocusSelect, tagedit.c:UpdateEditors)
			that calls XtNameToWidget() and expects a certain name.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget
CreateEditorShell(enable)

Widget
   enable;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   String
      editlab;
   char
      buf[BUFSIZ];
   Widget
      donebut,
      form,
      shell;
   XtPopdownID
      pdptr;   /* Pointer to the popdown record that will take care of */
	       /* sensitizing the menu button */

#ifdef HARDCODEDTITLE
   XtVaGetValues(enable, XtNlabel, &editlab, NULL );
#else
   strcpy (buf, XtName (enable));
   strcat (buf, "-editor");
   editlab = buf;
#endif
   
   shell = XtVaCreatePopupShell( editlab,
       applicationShellWidgetClass, Toplevel,
#ifdef HARDCODEDTITLE
       XtNtitle, editlab,
#endif
       NULL );

   form = XtVaCreateManagedWidget( "editorform",
       groupWidgetClass, shell,
       XtNspacing,  8,
       NULL );

   donebut = CreateEditorLayout(form);

   pdptr                = XtNew(XtPopdownIDRec);
   pdptr->shell_widget  = shell;
   pdptr->enable_widget = enable;
   XtAddCallback(donebut, XtNcallback, XtCallbackPopdown, pdptr);  

   XtRealizeWidget(shell);
   
   return(shell);
}




/*
   FUNCTION: 		CreateEditorWidgets

   Synopsis: Create a fresh set of widgets for an editor.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CreateEditorWidgets(editor, tptr, edittype, vptr)

Widget
   editor;
Tagspec
   *tptr;
XawTextEditType
   edittype;
Tagspec
   *vptr;

{
   Arg
      args[MAXARGS];
   Dimension
      textwidth;
   int
      cnt;
   Widget
      conform,
      rexform,
      tagform,
      vert;

   conform = XtNameToWidget(editor, "*conform");
   rexform = XtNameToWidget(editor, "*rexform");
   tagform = XtNameToWidget(editor, "*tagform");

   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->tagw;
   }
   tptr->tagw = XtVaCreateManagedWidget( "tagw",
       asciiTextWidgetClass, tagform,
       XtNbottom,       XtChainTop,
       XtNright,        XtChainLeft,
       XtNfromVert,     vert,
       XtNresizable,    True,
       XtNeditType,     edittype,
       XtNresize,       XawtextResizeWidth,
       XtNdisplayCaret, False,
       XtNborderWidth,  2,
       XtNborder,       Background,
       XtNstring,       tptr->tag,
       NULL );

   textwidth = GetStringWidthInPixels(tptr->tagw, tptr->tag);
   XtVaSetValues( tptr->tagw, XtNwidth, textwidth, NULL );

#ifdef ICA
   if ((tptr->tcode == Symbol) || (tptr->tcode == ProcIn))
   {
#endif
      XtOverrideTranslations(tptr->tagw, Tagtrans);
      XtOverrideTranslations(tptr->tagw, Texttrans);
#ifdef ICA
   }
#endif

   if ((tptr->tcode != Symbol) &&
       (tptr->tcode != ProcIn) &&
       (tptr->tcode != Comma))
   {
      cnt = 0;
      XtSetArg(args[cnt], XtNbottom,    XtChainTop);   cnt++;
      XtSetArg(args[cnt], XtNright,     XtChainLeft);  cnt++;
      XtSetArg(args[cnt], XtNfromHoriz, tptr->tagw);   cnt++;
      XtSetArg(args[cnt], XtNfromVert,  vert);         cnt++;
      if ((tptr->tcode == ImBeg) || (tptr->tcode == ExBeg))
      {
	 XtSetArg(args[cnt], XtNlabel, "S");  cnt++;
	 XtSetArg(args[cnt], XtNstate, True); cnt++;
      }
      else
      {
	 XtSetArg(args[cnt], XtNlabel, "E");  cnt++;
	 XtSetArg(args[cnt], XtNstate, True); cnt++;
      }
      if ((tptr->tcode == ImBeg) || (tptr->tcode == ImEnd))
      {
	 tptr->togw = XtCreateManagedWidget("togw", toggleWidgetClass,
					    tagform, args, cnt);
	 XtAddCallback(tptr->togw, XtNcallback, AttSet, tptr);
      }
      else
      {
	 cnt--;
	 tptr->togw = XtCreateManagedWidget("togw", labelWidgetClass,
					    tagform, args, cnt);
      }
   }
   else
   {
      tptr->togw = NULL;
   }
   
   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->rexw;
   }
   tptr->rexw = XtVaCreateManagedWidget( "rexw",
       asciiTextWidgetClass, rexform,
       XtNbottom,       XtChainTop,
       XtNright,        XtChainLeft,
       XtNfromVert,     vert,
       XtNresizable,    True,
       XtNeditType,     XawtextEdit,
       XtNresize,       XawtextResizeWidth,
       XtNdisplayCaret, False,
       XtNborderWidth,  2,
       XtNborder,       Background,
       XtNstring,       tptr->regexp,
       NULL );

   textwidth = GetStringWidthInPixels(tptr->rexw, tptr->regexp);
   XtVaSetValues( tptr->rexw, XtNwidth, textwidth, NULL );

   XtOverrideTranslations(tptr->rexw, Texttrans);

   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->conw;
   }
   tptr->conw = XtVaCreateManagedWidget( "conw",
       asciiTextWidgetClass, conform,
       XtNbottom,       XtChainTop,
       XtNright,        XtChainLeft,
       XtNfromVert,     vert,
       XtNresizable,    True,
       XtNeditType,     XawtextRead,
       XtNresize,       XawtextResizeWidth,
       XtNdisplayCaret, False,
       XtNborder,       Background,
       XtNborderWidth,  2,
       XtNstring,       tptr->context,
       NULL );

   textwidth = GetStringWidthInPixels(tptr->conw, tptr->context);
   XtVaSetValues( tptr->conw, XtNwidth, textwidth, NULL );

   XtOverrideTranslations(tptr->conw, Menutrans);

   return;
}



/*
   FUNCTION: 		UpdateEditors

   Synopsis: Update all of the editor shells after a file has been loaded.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON
       pete	3/3/92	 I've changed the names of the editor widgets created
       			 in CreateEditorShell() so the title is set in the
			 resources instead of being hardcoded.  Needed to
			 update the XtNameToWidget() calls for the new names.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
UpdateEditors(placeflag)

Boolean
   placeflag; /* Place holder flag for extra spots in editors */

{
   Arg
      args[MAXARGS];
   Boolean
      pflag = True;
   int
      cnt;
   Tagspec
      *tptr,
      *vptr;
   Widget
      editor;

   Menutrans = XtParseTranslationTable(menutransstr);
   Tagtrans  = XtParseTranslationTable(tagtransstr);
   Texttrans = XtParseTranslationTable(texttransstr);

   cnt = 0;
   XtSetArg(args[cnt], XtNdisplayCaret, True);       cnt++;
   XtSetArg(args[cnt], XtNborder,       Foreground); cnt++;

   if (placeflag == True)
   {
      XicaStatus(Statw, "Creating widgets\n");
   }

   editor = XtNameToWidget(Toplevel, "*symsel-editor");
   EditorDoLayout( editor, False );
   vptr = NULL;
   for (tptr = RTTinfo.symptr; tptr != NULL; tptr = tptr->next)
   {
      XicaStatus(Statw, ".");
      CreateEditorWidgets(editor, tptr, XawtextEdit, vptr);
      vptr = tptr;
   }
   EditorDoLayout( editor, True );
   
   /*
    * Create the startup Widgets.
    */
   if (placeflag == True)
   {
      XicaStatus(Statw, ".");
      if (vptr == NULL)
      {
	 RTTinfo.symptr = NewTagptr("", Symbol);
	 CreateEditorWidgets(editor, RTTinfo.symptr, XawtextEdit, vptr);
      }
      else
      {
	 vptr->next = NewTagptr("", vptr->tcode);
	 CreateEditorWidgets(editor, vptr->next, XawtextEdit, vptr);
      }
      XicaStatus(Statw, "\n");
   }

   if ((RTTinfo.symptr != NULL) && (RTTinfo.symptr->tagw != NULL))
   {
      XtSetValues(RTTinfo.symptr->tagw, args, cnt);
      XtSetKeyboardFocus(editor, RTTinfo.symptr->tagw);
      Symfocus = RTTinfo.symptr->tagw;
   }
   

   editor = XtNameToWidget(Toplevel, "*pinsel-editor");
   EditorDoLayout( editor, False );
   vptr = NULL;
   for (tptr = RTTinfo.pinptr; tptr != NULL; tptr = tptr->next)
   {
      XicaStatus(Statw, ".");
      CreateEditorWidgets(editor, tptr, XawtextEdit, vptr);
      vptr = tptr;
   }
   EditorDoLayout( editor, True );

   /*
    * Create the startup Widgets.
    */
   if (placeflag == True)
   {
      XicaStatus(Statw, ".");
      if (vptr == NULL)
      {
	 RTTinfo.pinptr = NewTagptr("", ProcIn);
	 CreateEditorWidgets(editor, RTTinfo.pinptr, XawtextEdit, vptr);
      }
      else
      {
	 vptr->next = NewTagptr("", vptr->tcode);
	 CreateEditorWidgets(editor, vptr->next, XawtextEdit, vptr);
      }
      XicaStatus(Statw, "\n");
   }
   if ((RTTinfo.pinptr != NULL) && (RTTinfo.pinptr->tagw != NULL))
   {
      XtSetValues(RTTinfo.pinptr->tagw, args, cnt);
      XtSetKeyboardFocus(editor, RTTinfo.pinptr->tagw);
      Pinfocus = RTTinfo.pinptr->tagw;
   }
   
   editor = XtNameToWidget(Toplevel, "*sepsel-editor");
   EditorDoLayout( editor, False );
   vptr = NULL;
   for (tptr = RTTinfo.sepptr; tptr != NULL; tptr = tptr->next)
   {
      XicaStatus(Statw, ".");
#ifdef ICA
      CreateEditorWidgets(editor, tptr, XawtextRead, vptr);
#else
      CreateEditorWidgets(editor, tptr, XawtextEdit, vptr);
#endif
      vptr = tptr;
   }
   EditorDoLayout( editor, True );

   if (placeflag == True)
   {
      XicaStatus(Statw, "\n");
   }

   if ((RTTinfo.sepptr != NULL) && (RTTinfo.sepptr->rexw != NULL))
   {
      XtSetValues(RTTinfo.sepptr->rexw, args, cnt);
      XtSetKeyboardFocus(editor, RTTinfo.sepptr->rexw);
      Sepfocus = RTTinfo.sepptr->rexw;
   }

   editor = XtNameToWidget(Toplevel, "*impsel-editor");
   EditorDoLayout( editor, False );
   vptr = NULL;
   for (tptr = RTTinfo.impptr; tptr != NULL; tptr = tptr->next)
   {
      XicaStatus(Statw, ".");
#ifdef ICA
      CreateEditorWidgets(editor, tptr, XawtextRead, vptr);
#else
      CreateEditorWidgets(editor, tptr, XawtextEdit, vptr);
#endif
      vptr = tptr;
   }
   EditorDoLayout( editor, True );

   if (placeflag == True)
   {
      XicaStatus(Statw, "\n");
   }

   if ((RTTinfo.impptr != NULL) && (RTTinfo.impptr->rexw != NULL))
   {
      XtSetValues(RTTinfo.impptr->rexw, args, cnt);
      XtSetKeyboardFocus(editor, RTTinfo.impptr->rexw);
      Impfocus = RTTinfo.impptr->rexw;
   }

   editor = XtNameToWidget(Toplevel, "*expsel-editor");
   EditorDoLayout( editor, False );
   vptr = NULL;
   for (tptr = RTTinfo.expptr; tptr != NULL; tptr = tptr->next)
   {
      if (pflag == True)
      {
	 XicaStatus(Statw, ".");
      }
      pflag = !pflag;
      
#ifdef ICA
      CreateEditorWidgets(editor, tptr, XawtextRead, vptr);
#else
      CreateEditorWidgets(editor, tptr, XawtextEdit, vptr);
#endif
      vptr = tptr;
   }
   EditorDoLayout( editor, True );

   if (placeflag == True)
   {
      XicaStatus(Statw, "\n");
   }

   if ((RTTinfo.expptr != NULL) && (RTTinfo.expptr->rexw != NULL))
   {
      XtSetValues(RTTinfo.expptr->rexw, args, cnt);
      XtSetKeyboardFocus(editor, RTTinfo.expptr->rexw);
      Expfocus = RTTinfo.expptr->rexw;
   }
   
   return;
}



