/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: utils.c,v $
   Purpose:     Provide some useful routines.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.12 $
   Date:        $Date: 92/05/29 15:28:26 $


   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	utils.c,v $
 * Revision 1.12  92/05/29  15:28:26  boyd
 * Changes to improve resizing, consisting mostly of replacing Form widgets with
 * Group, Table and Viewform.
 * 
 * 
 * Revision 1.10  92/03/16  15:02:43  ware
 * Make titles consistent.
 * 
 * Revision 1.9  1992/03/10  19:40:36  cso
 * Removed (un)managing calls
 *
 * Revision 1.8  1992/02/25  02:52:45  cso
 * Trying to trace down and fix memory leak
 *
 * Revision 1.7  1992/02/20  04:55:41  cso
 * Spec'ed out changes to add a new context type called Nest
 *
 * Revision 1.6  1992/02/06  04:49:06  cso
 * Modified the End tag code per ultrix user comment
 *
 * Revision 1.5  1992/01/28  18:48:50  cso
 * Restructured dependencies.
 * Trying to eliminate the bogus messages during loads.
 * Corrected for the new spec language.
 *
 * Revision 1.4  1991/12/05  16:18:43  cso
 * Added the appropriate code sequences for supporting the separator tags
 *
 * Added the function to update the sep. tag data structure in the case
 * of an old spec being parsed.
 *
 * Added a function to clean up a memory leak - freeing up space after
 * code generation.
 *
 * Revision 1.3  1991/11/27  00:29:47  cso
 * Made a number of minor changes:
 *  - Use of MAXARGS
 *  - Use of FRAMEWIDTH
 *  - Use of the Status widget
 *  - Use of the keypad enter for return also
 *  - Use of fallback resources
 *  - Reconfigured the interface
 *  - Removed some of the unnecessary unmanaging/managing to solve some
 * 	of the jumpiness in the interface
 *
 * Revision 1.2  1991/11/09  16:42:53  cso
 * Made some minor changes to the keyboard focus feedback mechanism so
 * that the interface does not shift around too much.
 * Added a function ShiftScrollbars to keep them up to date, but it did
 * not have the desired effect, so nothing calls it at the moment.
 *
 * Revision 1.1.1.3  1991/10/04  20:20:26  jbm
 * First HaL mods to ICA
 *
*/

/* #INCLUDES */

#include "widgets.h"
#include "defs.h"
#include "types.h"

/*  EXTERNS  */

#include "alpha.h"
#include "contab.h"
#include "list.h"
#include "retag.h"
#include "status.h"
#include "tables.h"



#include <X11/IntrinsicP.h>

void EditorDoLayout( editor, doit )
Widget editor;
Boolean doit;
{
    Widget tagform, rexform, conform;

    tagform = XtNameToWidget( editor, "*tagform" );
    rexform = XtNameToWidget( editor, "*rexform" );
    conform = XtNameToWidget( editor, "*conform" );

    XawFormDoLayout( tagform, doit );
    XawFormDoLayout( rexform, doit );
    XawFormDoLayout( conform, doit );
}


Widget ShellParent(w)
Widget w;
/*
   Return the nearest parent of w which is a shell widget.
*/
{
    Widget parent;

    while (NULL != (parent = XtParent(w)) && !XtIsShell(parent))  w = parent;

    return (parent);
}


/*
   FUNCTION: 		GetStringWidthInPixels

   Synopsis: 
       Using XTextExtents, find the width of a string in pixels, making
       the application font independent.
       
   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Dimension

   Range:		0 <=
*/

Dimension
GetStringWidthInPixels(w, str)

Widget
   w;   /* The widget under consideration */
String
   str; /* The string being measured */

{
   Arg
      args[MAXARGS];        /* List of arguments */
   Dimension
      width;           /* Width value to be returned */
   int
      cnt,             /* Number of arguments */
      direction_hint,  /* Direction of the font */
      ascent,          /* Amount above the baseline used by the font */
      descent,
      len;         /* Amount below the baseline used by the font */
   Position
      lmargin,         /* The left margin allowed in the widget */
      rmargin;         /* The right margin allowed in the widget */
   XCharStruct
      overall;         /* Structure describing the font */
   XFontStruct
      *font;           /* The font structure used by the widget */
   
   /*
    * Get the font and margins of the widget
    */
   cnt = 0;
   XtSetArg(args[cnt], XtNfont,        &font);    cnt++;
   XtSetArg(args[cnt], XtNleftMargin,  &lmargin); cnt++;
   XtSetArg(args[cnt], XtNrightMargin, &rmargin); cnt++;
   XtGetValues(w, args, cnt);
   
   /*
    * Get some of the dimensions used in the font of the widget
    */
   len = strlen(str);
   if (strchr(str, '\t') != NULL)
   {
      len += 7;
   }
   XTextExtents(font, str, len, &direction_hint, &ascent, &descent,
		&overall);
   
   /*
    * The width returned equals the margins of the widget (to make sure
    * we don't cut it off too short) plus the left and right bearings that
    * define the rectangle around the string in that font
    */
   width = lmargin + rmargin + overall.lbearing + overall.rbearing;
   return(width);
}




/*
   FUNCTION: 		DisposeTagptr

   Synopsis:  Free up a tag ptr structure.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
DisposeTagptr(tptr)

Tagspec
   *tptr;

{
   XtFree(tptr->tag);
   XtFree(tptr->regexp);
   XtFree(tptr->context);
   if (tptr->togw != NULL)
   {
      XtDestroyWidget(tptr->togw);
   }
   if (tptr->tagw != NULL)
   {
      XtDestroyWidget(tptr->tagw);
   }
   if (tptr->rexw != NULL)
   {
      XtDestroyWidget(tptr->rexw);
   }
   if (tptr->conw != NULL)
   {
      XtDestroyWidget(tptr->conw);
   }
      
   XtFree(tptr);
   return;
}



/*
   FUNCTION: 		NewTagptr

   Synopsis: Create a new tag structure.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Tagspec *

   Range:		none
*/

Tagspec *
NewTagptr(tagstr, tcode)

String
   tagstr;
TagCode
   tcode;

{
   Tagspec
      *tptr;
   
   tptr          = XtNew(Tagspec);
   tptr->tag     = XtNewString(tagstr);
   tptr->regexp  = XtNewString("");
   tptr->context = XtNewString("None");
   tptr->tcode   = tcode;
   tptr->conw    = NULL;
   tptr->rexw    = NULL;
   tptr->tagw    = NULL;
   tptr->togw    = NULL;
   tptr->next    = NULL;
   return(tptr);
}




/*
   FUNCTION: 		DetermineContext

   Synopsis: 

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


String
DetermineContext(constr, code)

String
   constr;
ConCode
   *code;

{
   switch (*constr)
   {
   default:
      XicaStatus(Statw, "DetermineContext: Bad clause code");
      break;
      
   case '\0':
      *code = Nope;
      return(constr);
      break;
      
   case 'S':
      *code = Start;
      break;
      
   case 'E':
      *code = Stop;
      break;

   case 'I':
      *code = In;
      break;

   case 'N':
      *code = Nest;
      break;
   }
   
   return(&constr[1]);
}




/*
   FUNCTION: 		DetermineRoot

   Synopsis: 

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


Boolean
DetermineRoot(tagstr)

String
   tagstr;

{
   if (tagstr[0] != '\0')
   {
      if (tagstr[1] == '#')
      {
	 return(True);
      }
   }
   return(False);
}




TagCode
DetermineTagCode(tagstr)

String
   tagstr;

{
   int
      i;
   
   i = 0;
   if (tagstr[i] == '&')
   {
      return(Symbol);
   }
   i++;
   if (tagstr[i] == '#')
   {
      i++;
   }
   switch (tagstr[i])
   {
   default:
      return(Begin);
      
   case '/':
      return(EndT);
	 
   case '?':
      return(ProcIn);

   case ',':
      return(Comma);
   }
}



String
StripTagString(tagstr)

String
   tagstr;

{
   int
      i,
      j;
   String
      tmpstr;

#ifdef DEBUG
   fprintf(stderr, "Stripped string [%s] to ", tagstr);
#endif

   tmpstr = XtNewString(tagstr);
   
   i = 1;
   if (tmpstr[i] == '#')
   {
      i++;
   }
   switch (tmpstr[i])
   {
   default:
      break;
      
   case '/':
   case '?':
   case ',':
      i++;
   }
   
   for (j = i; j < (strlen(tmpstr) - 1); j++)
   {
      tmpstr[j - i] = tagstr[j];
   }
   tmpstr[j - i] = '\0';
#ifdef DEBUG
   fprintf(stderr, "[%s]\n", tmpstr);
#endif
   return(tmpstr);
}




/*
   FUNCTION: 		PrintSpec

   Synopsis: Give a dump of the loaded structures.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PrintSpec()

{
   ContextList
      *cptr;
   Tagspec
      *tptr;
   
   fprintf(stderr, "Data characters = [%s]\n", RTTinfo.data);
   fprintf(stderr, "Tags characters = [%s]\n", RTTinfo.tags);
   fprintf(stderr, "Ambg characters = [%s]\n", RTTinfo.ambg);
   fprintf(stderr, "Root symbol     = [%s]\n", RTTinfo.root);
   fprintf(stderr, "Context clauses =\n");
   for (cptr = RTTinfo.conptr; cptr != NULL; cptr = cptr->next)
   {
      fprintf(stderr, "\t[%s]\n", cptr->cond);
   }
   fprintf(stderr, "Symbol tags =\n");
   for (tptr = RTTinfo.symptr; tptr != NULL; tptr = tptr->next)
   {
      fprintf(stderr, "\t[%s]\t[%s]\t[%s]\n",
	      tptr->tag, tptr->regexp, tptr->context);
   }
   fprintf(stderr, "Processing Instructions =\n");
   for (tptr = RTTinfo.pinptr; tptr != NULL; tptr = tptr->next)
   {
      fprintf(stderr, "\t[%s]\t[%s]\t[%s]\n",
	      tptr->tag, tptr->regexp, tptr->context);
   }
   fprintf(stderr, "Separator Tags =\n");
   for (tptr = RTTinfo.sepptr; tptr != NULL; tptr = tptr->next)
   {
      fprintf(stderr, "\t[%s]\t[%s]\t[%s]\n",
	      tptr->tag, tptr->regexp, tptr->context);
   }
   fprintf(stderr, "Implicit tags =\n");
   for (tptr = RTTinfo.impptr; tptr != NULL; tptr = tptr->next)
   {
      fprintf(stderr, "\t[%s]\t[%s]\t[%s]\n",
	      tptr->tag, tptr->regexp, tptr->context);
   }
   fprintf(stderr, "Explicit tags =\n");
   for (tptr = RTTinfo.expptr; tptr != NULL; tptr = tptr->next)
   {
      fprintf(stderr, "\t[%s]\t[%s]\t[%s]\n",
	      tptr->tag, tptr->regexp, tptr->context);
   }
   
   return;
}



/*
   FUNCTION: 		ReadICA

   Synopsis: Read in the list of segmenting tags.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


Boolean
ReadICA(fptr)

FILE
   *fptr;

{
   char
      c,
      junkstr[BUFSIZ];  /* Consume unimportant info from the ICA file */
   String
      ident,
      idptr;
   Tagspec 
      *eptr1,
      *eptr2,
      *iptr1,
      *iptr2,
      *sptr1,
      *sptr2;
   int
      maxtag;
 
   if (fptr == NULL)
   {
      return(False);
   }
   XicaStatus(Statw, "Loading New Retag Spec from ICA file\n");

   /*
    * Initialize the Explicit and Implicit pointers
    * Read characters in from an ICA file until the #### markers.
    * Read characters until the end of that line
    * Now read in the max size of an identifier
    * Allocate space in a buffer
    * Read in tag identifiers and put them in the proper place
    */
   eptr1 = RTTinfo.expptr;
   iptr1 = RTTinfo.impptr;
   sptr1 = RTTinfo.sepptr;
   
   while ((fscanf(fptr, "%s", junkstr) != EOF) &&
	  (strcmp(junkstr, "####")     != 0))
   {
   }
   if (strcmp(junkstr, "####") != 0)
   {
      return(False);
   }
   while (((c = fgetc(fptr)) != '\n') && (c != EOF))
   {
   }
   if (c == EOF)
   {
      return(False);
   }
   
   fscanf(fptr, "%d", &maxtag);
   ident = (String) XtMalloc((unsigned) maxtag + 8);
   idptr = ident;

   while ((fscanf(fptr, "%s", ident) != EOF) &&
	  (strcmp(ident, "####")     != 0))
   {
      if (ident[0] == '#')
      {
#ifdef DEBUG
	 fprintf(stderr, "root = [%s]\n", ident);
#endif
	 ident++;
	 RTTinfo.root = XtNewString(ident);
      }

#ifdef DEBUG
      fprintf(stderr, "ident = [%s]\n", ident);
#endif
      
      sptr2 = NewTagptr(ident, Comma);
      if (sptr1 == NULL)
      {
	 RTTinfo.sepptr = sptr2;
      }
      else
      {
	 sptr1->next = sptr2;
      }
      sptr1 = sptr2;

      iptr2 = NewTagptr(ident, ImBeg);
      if (iptr1 == NULL)
      {
	 RTTinfo.impptr = iptr2;
      }
      else
      {
	 iptr1->next = iptr2;
      }
      iptr1 = iptr2;

      eptr2 = NewTagptr(ident, ExBeg);
      if (eptr1 == NULL)
      {
	 RTTinfo.expptr = eptr2;
      }
      else
      {
	 eptr1->next = eptr2;
      }
      eptr1 = eptr2;

      eptr2       = NewTagptr(ident, ExEnd);
      eptr1->next = eptr2;
      eptr1       = eptr2;
   }
   
   fclose(fptr);
   XtFree(idptr);
   XicaStatus(Statw, "Load Completed\n");
#ifdef DEBUG
   PrintSpec();
#endif

   return(True);
}




/*
   FUNCTION: 		CorrectContext

   Synopsis: For backward compatibility.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		NULL or a char *
*/

String
CorrectContext(constr, tagcode)

String
   constr;
TagCode
   tagcode;

{
   String
      sptr1,
      sptr2,
      str;
   
   sptr2 = XtNewString(constr);
   sptr1 = sptr2;
   if (tagcode == ExBeg)
   {
      switch(*constr)
      {
      default:
	 break;

      case 'D':
	 sptr1++;
	 sptr1++;
	 sptr1[0] = 'S';
	 sptr1[1] = 'T';
	 sptr1[2] = 'A';
	 sptr1[3] = 'R';
	 sptr1[4] = 'T';
	 break;

      case 'E':
	 sptr1++;
	 sptr1[0] = 'I';
	 sptr1[1] = 'N';
	 break;

      case 'I':
	 break;

      case 'S':
	 break;
      }
   }
   else
   {
      switch(*constr)
      {
      default:
	 break;

      case 'D':
	 sptr1++;
	 sptr1++;
	 sptr1++;
	 sptr1++;
	 sptr1[0] = 'E';
	 sptr1[1] = 'N';
	 sptr1[2] = 'D';
	 break;

      case 'E':
	 break;

      case 'I':
	 break;

      case 'S':
	 sptr1 = XtNewString("None");
	 XtFree(sptr2);
	 return(sptr1);
	 break;
      }
   }
   str = XtNewString(sptr1);
   XtFree(sptr2);
   return(str);
}



/*
   FUNCTION: 		FindTagptr

   Synopsis: Find the tag ptr given a widget where the user hit return.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	Tagspec *

   Range:		NULL or valid
*/

Tagspec *
FindTagptr(w)

Widget
   w;

{
   Tagspec
      *tptr;
   
   tptr = NULL;
   for (tptr = RTTinfo.symptr; tptr != NULL; tptr = tptr->next)
   {
      if ((tptr->tagw == w) || (tptr->rexw == w) || (tptr->conw == w))
      {
	 return(tptr);
      }
   }
   for (tptr = RTTinfo.pinptr; tptr != NULL; tptr = tptr->next)
   {
      if ((tptr->tagw == w) || (tptr->rexw == w) || (tptr->conw == w))
      {
	 return(tptr);
      }
   }
   for (tptr = RTTinfo.sepptr; tptr != NULL; tptr = tptr->next)
   {
      if ((tptr->tagw == w) || (tptr->rexw == w) || (tptr->conw == w))
      {
	 return(tptr);
      }
   }
   for (tptr = RTTinfo.impptr; tptr != NULL; tptr = tptr->next)
   {
      if ((tptr->tagw == w) || (tptr->rexw == w) || (tptr->conw == w))
      {
	 return(tptr);
      }
   }
   for (tptr = RTTinfo.expptr; tptr != NULL; tptr = tptr->next)
   {
      if ((tptr->tagw == w) || (tptr->rexw == w) || (tptr->conw == w))
      {
	 return(tptr);
      }
   }
   
   return(tptr);
}



/*
   FUNCTION: 		MakeUpper

   Synopsis: 
       Make a string into the uppercase equivalent.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		An upper case version of the input string
*/

String
MakeUpper(str)

String
   str;  /* The string being transformed */

{
   String
      tmp; /* A temporary copy */
   int
      i;   /* Loop counter to examine at each character */
   
   /*
    * Make a copy of the string
    * For each character, consider if it should be changed
    */
   tmp = XtNewString(str);
   for (i = 0; i < strlen(tmp); i++)
   {
      /*
       * If the character is a lower case alphabetic character
       * Make it upper case
       */
      if (islower(tmp[i]))
      {
	 tmp[i] = toupper(tmp[i]);
      }
   }
   return(tmp);
}



/*
   FUNCTION: 		RemoveChar

   Synopsis: Remove a character from one of the alphabets.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
RemoveChar(str)

String
   str;

{
   while (str[0] != '\0')
   {
      str[0] = str[1];
      if (str[0] != '\0')
      {
	 str++;
      }
   }
   return;
}




/*
   FUNCTION: 		RemoveDuplicateChars

   Synopsis:  Remove any duplicate characters from a string.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	String

   Range:		"" or the input string
*/

String
RemoveDuplicateChars(str)

String
   str;

{
   String
      str1,
      str2;
   
#ifdef DEBUG
   fprintf(stderr, "Before [%s]", str);
#endif
   for (str1 = str; *str1 != '\0'; )
   {
      while (str1[0] != '\0')
      {
	 if (isalnum(str1[0]) != 0)
	 {
	    str1++;
	 }
	 else if ((str1[0] == '-') && (isalnum(str1[1]) != 0))
	 {
	    str1++;
	    str1++;
	 }
	 else
	 {
	    break;
	 }
      } /***** WHILE it is not any a-z combos */
      if (*str1 != '\0')
      {
	 str2 = strchr(str1 + 1, *str1);
	 if (str2 != NULL)
	 {
	    if ((str2[0] == '-') && (isalnum(str2[1]) != 0))
	    {
	       str1++;
	    }
	    else
	    {
	       RemoveChar(str2);
	    }
	 } /***** IF a duplicate was found in the rest of the string */
	 else
	 {
	    str1++;
	 }
      } /***** IF str1 is not already at the end, e.g., a-z would push */
	/***** it there */
   } /***** FOR each character in the string */
#ifdef DEBUG
   fprintf(stderr, " -> after [%s]\n", str);
#endif

   return(str);
}



/*
   FUNCTION: 		UpdateAlphabet

   Synopsis: Update an alphabet to enforce mutual exclusiveness.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


String
UpdateAlphabet(w, alphastr)

Widget
   w;
String
   alphastr;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   String
      str1,
      str2,
      textstr;
   
   cnt = 0;
   XtSetArg(args[cnt], XtNstring, &textstr); cnt++;
   XtGetValues(w, args, cnt);

   for (str1 = alphastr; *str1 != '\0'; str1++)
   {
      if (((str1[0] == '-') && (isalnum(str1[1]) != 0)) ||
	  (isalnum(str1[0]) != 0))
      {
	 continue;
      }

      /*
       * this is not a special character so lets do the simple
       * remove thing.
       */
      str2 = strrchr(textstr, *str1);
      if (str2 != NULL)
      {
	 RemoveChar(str2);
      }
   } /***** FOR each character in the alphabet string */

   return(textstr);
}



/*
   FUNCTION: 		FindFocus

   Synopsis: find the focus widget that is a child of the passed in Widget.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

Widget
FindFocus(parent)

Widget
   parent;

{
   if ((Expfocus != NULL) && (parent == ShellParent(Expfocus)))
   {
      return(Expfocus);
   }
   if ((Impfocus != NULL) && (parent == ShellParent(Impfocus)))
   {
      return(Impfocus);
   }
   if ((Sepfocus != NULL) && (parent == ShellParent(Sepfocus)))
   {
      return(Sepfocus);
   }
   if ((Pinfocus != NULL) && (parent == ShellParent(Pinfocus)))
   {
      return(Pinfocus);
   }
   if ((Symfocus != NULL) && (parent == ShellParent(Symfocus)))
   {
      return(Symfocus);
   }
   return(NULL);
}



/*
   FUNCTION: 		FixVerticals

   Synopsis: Fix the vertical widgets after duplicate/remove.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FixVerticals(tptr, vptr)

Tagspec
   *tptr,
   *vptr;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   Widget
      vert;

   if (tptr == NULL)
   {
      return;
   }
   
   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->tagw;
   }
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,  vert); cnt++;
   XtSetValues(tptr->tagw, args, cnt);

   if (tptr-> togw != NULL)
   {
      cnt = 0;
      XtSetArg(args[cnt], XtNfromVert,  vert); cnt++;
      XtSetValues(tptr->togw, args, cnt);
   } /**** IF that tag ptr has a toggle widget */
   
   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->rexw;
   }
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,  vert); cnt++;
   XtSetValues(tptr->rexw, args, cnt);

   if (vptr == NULL)
   {
      vert = NULL;
   }
   else
   {
      vert = vptr->conw;
   }
   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,  vert); cnt++;
   XtSetValues(tptr->conw, args, cnt);
   return;
}




/*
   FUNCTION: 		ShiftScrollbars

   Synopsis: 
       Shift the scrollbars so that new widgets do not disappear,
       e.g., in the symbol editor.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ShiftScrollbars(w)

Widget
   w;

{
   Arg
      args[MAXARGS];
   float
      shown,
      top;
   int
      cnt;
   Widget
      cvert,
      rvert,
      tvert,
      vertscroll;
   
   vertscroll = XtNameToWidget(XtParent(XtParent(w)), "*vertical");
   cnt = 0;
   XtSetArg(args[cnt], XtNshown, &shown); cnt++;
   XtSetArg(args[cnt], XtNtop,   &top);   cnt++;
   XtGetValues(vertscroll, args, cnt);
   tvert = XtNameToWidget(ShellParent(vertscroll), "*tagview*vertical");
   rvert = XtNameToWidget(ShellParent(vertscroll), "*rexview*vertical");
   cvert = XtNameToWidget(ShellParent(vertscroll), "*conview*vertical");
   XawScrollbarSetThumb(tvert, (float) (1.0 - shown), shown);
   XawScrollbarSetThumb(rvert, (float) (1.0 - shown), shown);
   XawScrollbarSetThumb(cvert, (float) (1.0 - shown), shown);
   return;
}



/*
   FUNCTION: 		CopyTags

   Synopsis: 
       Copy a unique list of segmenting tags to the separator list
       because we are reading in an old specification, and none were
       created before.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
CopyTags()

{
   Tagspec
      *iptr,
      *sptr;
   
   sptr = NULL;
   for (iptr = RTTinfo.impptr; iptr != NULL; iptr = iptr->next)
   {
      if (sptr == NULL)
      {
	 sptr           = NewTagptr(iptr->tag, Comma);
	 RTTinfo.sepptr = sptr;
	 sptr->regexp   = XtNewString("");
	 sptr->context  = XtNewString("None");
      }
      else if (strcmp(sptr->tag, iptr->tag) != 0)
      {
	 sptr->next    = NewTagptr(iptr->tag, Comma);
	 sptr          = sptr->next;
	 sptr->regexp  = XtNewString("");
	 sptr->context = XtNewString("None");
      }
   }
   Dirtyflag = True;
   return;
}



/*
   FUNCTION: 		FreeTables

   Synopsis: 
       Free up the space used by the Lex and Yacc tables.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Satish Menon, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
FreeTables()

{
   int
      i;
   
   for (i = 0; i < Lexndx; i++)
   {
      XtFree(Lextab[i].regexp);
      XtFree(Lextab[i].context);
   }
   for (i = 0; i < Yaccndx; i++)
   {
      XtFree(Yacctab[i].tag);
   }
   for (i = 0; i < Ctxtndx; i++)
   {
      XtFree(Ctxttab[i]);
   }
   return;
}
