#include <stdio.h>
#include "types.h"
#include "output.h"

extern void
   Output();
extern TokenMapping
   Maproot;
extern Production
   Grammar[];
extern int
   Gmax;

extern char *
   GetMapping();
extern SymbolTable 
   *MakeSymbolTable(),
   *MakeTokenTable();



void
PrintTokens(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *estr,
      *sstr;
   SymbolTable
      *tptr;
   
   fprintf(fptr, "%%token <data>\n");
   fprintf(fptr, "  ICA_Char_DATA\n\n");
   fprintf(fptr, "%%token <ival>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      estr = (char *) makeendtag(tptr->symbol);
      sstr = (char *) makestarttag(tptr->symbol);
      if (tptr->symbol[1] == '/')
      {
	 fprintf(fptr, "  %s\n", estr);
      }
      else
      {
	 fprintf(fptr, "  %s\n", sstr);
      }
      free(estr);
      free(sstr);
   }
   return;
}




void
PrintType(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   SymbolTable
      *tptr;
      
   fprintf(fptr, "\n%%type <trans>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "  %s\n", tptr->symbol);
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "  %sOPTL\n", tptr->symbol);
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "  %sSTAR\n", tptr->symbol);
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "  %sPLUS\n", tptr->symbol);
      }
   }
   return;
}




void
PrintProductions(fptr)

FILE
   *fptr;

{
   char
      *estr,
      *sstr;
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr,
      *sptr1;
   PermutationList
      *pptr;
   
   for (i = 0; i < Gmax; i++)
   {
      fprintf(fptr, "%s\t: ", Grammar[i].lhs);
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    estr = (char *) makeendtag(sptr->symbolstr);
	    sstr = (char *) makestarttag(sptr->symbolstr);
	    if ((sptr->symbolstr[0] == '<') && (sptr->symbolstr[1] == '/'))
	    {
	       fprintf(fptr, "%s ", estr);
	    }
	    else if (sptr->symbolstr[0] == '<')
	    {
	       fprintf(fptr, "%s ", sstr);
	    }
	    else
	    {
	       fprintf(fptr, "%s", sptr->symbolstr);
	       switch (sptr->occurstr[0])
	       {
	       default:
		  fprintf(fptr, " ");
		  break;

	       case '?':
		  fprintf(fptr, "OPTL ");
		  break;
		  
	       case '*':
		  fprintf(fptr, "STAR ");
		  break;

	       case '+':
		  fprintf(fptr, "PLUS ");
		  break;
	       }
	    }
	    free(estr);
	    free(sstr);
	 } /* for sptr */
      
	 fprintf(fptr, "\n\t\t{\n\t\t $$ = structconcat(");
	 pptr = rptr->perminstance;
	 if ((pptr != NULL) && (pptr->prev == NULL))
	 {
	    sptr1 = rptr->symbol;
	    if (pptr->permsymbol != NULL)
	    {
	       sptr = pptr->permsymbol;
	    }
	    else
	    {
	       sptr = rptr->symbol;
	    }
	    for ( ; sptr != NULL; sptr = sptr->next)
	    {
	       fprintf(fptr, "%s, ", GetMapping(sptr->symbolstr, sptr1, 1));
	    }
	 }
	 fprintf(fptr, "NULL);\n");
	 if (i == 0)
	 {
	    fprintf(fptr, "\t\t structprint($$);\n");
	 }
	 fprintf(fptr, "\t\t}\n");
	 if (rptr->next == NULL)
	 {
	    fprintf(fptr, "\t\t;\n\n");
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
      } /* for rptr */
   }
   return;
}




void
PrintExtras(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *str;
   SymbolTable
      *tptr;
      
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      str = tptr->symbol;
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "%sOPTL\t: %s\n", str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "%sSTAR\t: %sSTAR %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "%sPLUS\t: %sPLUS %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| %s\n", str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
   }
   fprintf(fptr, "ICA_Char_DATASTAR\t: ICA_Char_DATASTAR ICA_Char_DATA\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t $$ = structconcat($1, structnew($2), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t| /* empty content */\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t $$ = structconcat(structnew(\"\"), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t;\n\n");
   return;
}




SymbolTable *
GenerateYacc(fptr)

FILE
   *fptr;

{
   SymbolTable
      *stable,
      *ttable;
   
   Output(fptr, yacchead);
   ttable = NULL;
   ttable = MakeTokenTable(ttable);
   PrintTokens(fptr, ttable);
   stable = NULL;
   stable = MakeSymbolTable(stable);
   PrintType(fptr, stable);
   fprintf(fptr, "\n%%start %s\n\n%%%%\n", Grammar[0].lhs);
   PrintProductions(fptr);
   PrintExtras(fptr, stable);
   return(ttable);
}




void
PrintLexSpec(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *estr,
      *sstr;
   SymbolTable
      *tptr;
   TokenMapping
      *mptr;
   
   for (mptr = Maproot.next; mptr != NULL; mptr = mptr->next)
   {
      if (mptr->token[0] == '&')
      {
	 fprintf(fptr, "\"%s\"\t{\n", mptr->token);
	 fprintf(fptr, "\t yylval.data = makestr(%s);\n",
		 GetMapping(mptr->token, NULL, 0));
	 fprintf(fptr, "\t return(ICA_Char_DATA);\n\t}\n\n");
      }
   }
   
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "\"%s\"\t{\n", tptr->symbol);
      fprintf(fptr, "\t BEGIN 0;\n");
      fprintf(fptr, "\t return(");
      estr = (char *) makeendtag(tptr->symbol);
      sstr = (char *) makestarttag(tptr->symbol);
      if (tptr->symbol[1] == '/')
      {
	 fprintf(fptr, "%s", estr);
      }
      else
      {
	 fprintf(fptr, "%s", sstr);
      }	 
      fprintf(fptr, ");\n\t}\n\n");
      free(estr);
      free(sstr);
   }
   return;
}





void
GenerateLex(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   Output(fptr, lexhead);
   PrintLexSpec(fptr, table);
   Output(fptr, lextail);
   return;
}
