#include <stdio.h>
#include "types.h"

extern int
   Gmax;
extern Production
   Grammar[];
extern TokenMapping
   Maproot;



void
addmap(root, token, text)

TokenMapping
   *root;
char
   *token,
   *text;

{
   TokenMapping
      *temp,
      *new;
   
   /* find the end of the symbol table */
   temp = root;
   while (temp->next != NULL)
   {
      temp = temp->next;
   }
   
   /* allocate memory for the new node, and it's symbol entry */
   if ((new = (TokenMapping *) malloc(sizeof (TokenMapping))) == NULL)
   {
      (void) perror ("Can't allocate memory for new node in addmap");
      exit (1);
   }
   if ((new->token = (char *) malloc (strlen(token)+1)) == NULL)
   {
      (void) perror ("Can't allocate memory for new symbol in addmap");
      exit (1);
   }
   if ((new->text = (char *) malloc (strlen(text)+1)) == NULL)
   {
      perror ("Can't allocate memory for new symbol in addmap");
      exit (1);
   }
   
   /* and link it in */
   (void) strcpy(new->token, token);
   (void) strcpy(new->text, text);
   new->next = NULL;
   temp->next = new;
   return;
}



void
clear_map(root)

TokenMapping
   *root;

/* delete and free up all memory associated with a mapping table */

{
   TokenMapping
      *temp,
      *temptemp;
   
   temp         = root->next;
   root->next   = NULL;
   root->token  = NULL;
   root->text   = NULL;
   
   while (temp != NULL)
   {
      (void) free (temp->token);
      (void) free (temp->text);
      temptemp = temp;
      temp = temp->next;
      (void) free (temptemp);
   }
   return;
}



SymbolTable *
PutSymbol(tableroot, symbolstr)

SymbolTable
   *tableroot;
char
   *symbolstr;

{
   int
      newflag;
   SymbolTable
      *tmptab,
      *tptr;

   tmptab = (SymbolTable *) malloc((unsigned) sizeof(SymbolTable));
   if (tmptab == NULL)
   {
      fprintf(stderr, "PutSymbol: Could not allocate space\n");
      exit(1);
   }
   newflag = 1;
   tptr    = tableroot;
   if (tableroot != NULL)
   {
      if (strcmp(tableroot->symbol, symbolstr) == 0)
      {
	 newflag = 0;
	 tptr = tableroot;
      }
      else
      {
	 for (tptr = tableroot; tptr->next != NULL; tptr = tptr->next)
	 {
	    if (strcmp(tptr->next->symbol, symbolstr) == 0)
	    {
	       newflag = 0;
	       tptr = tptr->next;
	       break;
	    }
	 }
      }	 
   }
   if (newflag == 1)
   {
      tmptab->symbol   = symbolstr;
      tmptab->optional = 0;
      tmptab->star     = 0;
      tmptab->plus     = 0;
      tmptab->next     = NULL;
      
      if (tableroot == NULL)
      {
	 tableroot = tmptab;
      }
      else
      {
	 tptr->next = tmptab;
      }
   }
   else
   {
      free(tmptab);
      tmptab = tptr;
   }
   return(tmptab);
}




SymbolTable *
MakeSymbolTable(tableroot)

SymbolTable
   *tableroot;

{
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr;
   SymbolTable
      *tptr;
      
   tableroot = NULL;
   
   tptr = PutSymbol(tableroot, Grammar[0].lhs);
   if (tableroot == NULL)
   {
      tableroot = tptr;
   }
   
   for (i = 0; i < Gmax; i++)
   {
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    if ((sptr->symbolstr[0] != '<') &&
		(strcmp(sptr->symbolstr, "ICA_Char_DATA") != 0))
	    {
	       tptr = PutSymbol(tableroot, sptr->symbolstr);
	       if (tableroot == NULL)
	       {
		  tableroot = tptr;
	       }
	       
	       
	       switch (sptr->occurstr[0])
	       {
	       default:
		  break;
		  
	       case '*':
		  tptr->star++;
		  break;
		  
	       case '+':
		  tptr->plus++;
		  break;
		  
	       case '?':
		  tptr->optional++;
		  break;
	       }
	    }
	 }
      }
   }
   return(tableroot);
}




SymbolTable *
PutToken(tableroot, symbolstr)

SymbolTable
   *tableroot;
char
   *symbolstr;

{
   int
      newflag;
   SymbolTable
      *tmptab,
      *tptr;

   tmptab = (SymbolTable *) malloc((unsigned) sizeof(SymbolTable));
   if (tmptab == NULL)
   {
      fprintf(stderr, "PutToken: Could not allocate space\n");
      exit(1);
   }
   newflag = 1;
   tptr    = tableroot;
   if (tableroot != NULL)
   {
      if (strcmp(tableroot->symbol, symbolstr) == 0)
      {
	 newflag = 0;
	 tptr = tableroot;
      }
      else
      {
	 for (tptr = tableroot; tptr->next != NULL; tptr = tptr->next)
	 {
	    if (strcmp(tptr->next->symbol, symbolstr) == 0)
	    {
	       newflag = 0;
	       tptr = tptr->next;
	       break;
	    }
	 }
      }	 
   }
   if (newflag == 1)
   {
      tmptab->symbol   = symbolstr;
      tmptab->optional = 0;
      tmptab->star     = 0;
      tmptab->plus     = 0;
      tmptab->next     = NULL;
      
      if (tableroot == NULL)
      {
	 tableroot = tmptab;
      }
      else
      {
	 tptr->next = tmptab;
      }
   }
   else
   {
      free(tmptab);
      tmptab = tptr;
   }
   return(tmptab);
}




SymbolTable *
MakeTokenTable(tableroot)

SymbolTable
   *tableroot;

{
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr;
   SymbolTable
      *tptr;
      
   tableroot = NULL;
   
   for (i = 0; i < Gmax; i++)
   {
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	 {
	    if (sptr->symbolstr[0] == '<')
	    {
	       tptr = PutToken(tableroot, sptr->symbolstr);
	       if (tableroot == NULL)
	       {
		  tableroot = tptr;
	       }
	    }
	 }
      }
   }
   return(tableroot);
}




char *
GetMapping(tag, sptr1, yflag)

char
   *tag;
SymbolList
   *sptr1;
int
   yflag;

{
   char
      dollarstr[10],
      *str;
   int
      i;
   SymbolList
      *sptr;
   TokenMapping
      *mptr;

   if ((tag[0] == '<') || (tag[0] == '&'))
   {
      for (mptr = Maproot.next; mptr != NULL; mptr = mptr->next)
      {
	 if (strcmp(tag, mptr->token) == 0)
	 {
	    str = (char *) malloc((unsigned) (strlen(mptr->text) + 32));
	    if (str == NULL)
	    {
	       fprintf(stderr, "GetMapping: Could not allocate [%d] bytes\n",
		       strlen(mptr->text) + 32);
	       exit(1);
	    }
	    if (yflag == 1)
	    {
	       sprintf(str, "structnew(%s)", mptr->text);
	    }
	    else
	    {
	       sprintf(str, "%s", mptr->text);
	    }
	    return(str);
	 }
      }
      if (mptr == NULL)
      {
	 str = (char *) malloc((unsigned) (16));
	 if (str == NULL)
	 {
	    fprintf(stderr, "GetMapping: Could not allocate [16] bytes\n");
	    exit(1);
	 }
	 sprintf(str, "structnew(\"\")");
	 return(str);
      }
   }
   else
   {
      for (sptr = sptr1, i = 1;
	   ((sptr != NULL) &&
	    (strcmp(sptr->symbolstr, tag) != 0));
	   sptr = sptr->next, i++)
      {
      }
      if (sptr != NULL)
      {
	 str = (char *) malloc((unsigned) (16));
	 if (str == NULL)
	 {
	    fprintf(stderr, "GetMapping: Could not allocate [16] bytes\n");
	    exit(1);
	 }
	 sprintf(str, "$%d", i);
	 return(str);
      }
      else
      {
	 fprintf(stderr, "GetMapping Error\n");
      }
   }
   str = (char *) malloc((unsigned) (strlen(tag) + 16));
   if (str == NULL)
   {
      fprintf(stderr, "GetMapping: Could not allocate [%d] bytes\n",
	      strlen(tag) + 16);
      exit(1);
   }
   sprintf(str, "structnew(\"%s\")", tag);
   return(str);
}
