#include <stdio.h>

#include "types.h"

FILE
   *LEX,
   *YACC;
Production
   Grammar[20000];
int
   Gmax;
TokenMapping
   Maproot;

extern SymbolTable *
   GenerateYacc();



extern FILE
  *yyin;

char *
Initialize(argc, argv)

int
   argc;
char
   **argv;

{
   int
      tempargc;
   char
      **tempargv;
   char 
      *lexfile,
      *target,
      *yaccfile;
   
   if ((target = (char *) malloc(5)) == NULL)
   {
      fprintf(stderr, "downtg:Initialize:Unable to allocate space\n");
      fprintf(stderr, "downtg:Initialize:Cannot parse command line\n");
      exit(2);
   }
   
   (void) strcpy(target, "down");
   tempargc = argc;
   tempargv = argv;
   tempargc--;
   tempargv++;
   /* 
    * an ugly way to parse 2 arguments, but in the long run, 
    * hopefully this will be a fairly decent way to add more
    * position independent arguments.
    */
   if (argc > 1)
   {
      while (tempargc > 0)
      {
	 if ((strcmp(*tempargv, "-target") == 0) && (tempargc > 1))
	 {
	    tempargv++;
	    tempargc--;
	    target = (char *) realloc(target, strlen(*tempargv));
	    strcpy(target, *tempargv);
	 }
	 else if ((yyin = fopen(*tempargv, "r")) == NULL)
	 {
	    (void) fprintf(stderr, "Bad input file name: [%s] \n", *tempargv);
	    exit (1);
	 } 	/* end else if */
	 tempargv++;
	 tempargc--;
      }	/* end while */
   }		/* end if (argc) */
  
   lexfile = (char *) malloc((unsigned) (strlen(target) + 16));
   if (lexfile == NULL)
   {
      fprintf(stderr, "Initialize: unable to malloc [%d] bytes\n",
	      strlen(target) + 16);
      exit(1);
   }
   yaccfile = (char *) malloc((unsigned) (strlen(target) + 16));
   if (yaccfile == NULL)
   {
      fprintf(stderr, "Initialize: unable to malloc [%d] bytes\n",
	      strlen(target) + 16);
      exit(1);
   }
   sprintf(lexfile,  "%s.l", target);
   sprintf(yaccfile, "%s.y", target);
   if ((LEX = fopen(lexfile, "w")) == NULL) 
   {
      (void) perror("Error opening LEX file");
      exit(1);
   }
   if ((YACC = fopen(yaccfile, "w")) == NULL) 
   {
      (void) perror("Error opening YACC file");
      exit(1);
   }
   
   Maproot.next  = NULL;
   Maproot.text  = NULL;
   Maproot.token = NULL;
   
   return(target);
}



/*
 *  usage: downtg [-target targetfilename] icafilename
 *
 *
 */
int
main(argc, argv)

int
   argc;
char
   **argv;

{
   char
      *target;	/* name of the target files generated */
   extern int
      linecount;
   SymbolTable
      *ttable;
   
   target = Initialize(argc, argv);
   
   linecount = 1;
   if (yyparse() == 0) 
   {
#ifdef DEBUG
      (void) fprintf(stderr, "hunky and dory\n");
#endif
      ttable = GenerateYacc(YACC);
      fclose(YACC);
      GenerateLex(LEX, ttable);
      fclose(LEX);
      MakeTranslator(target);
   }
   else 
   {
      fclose(YACC);
      fclose(LEX);
      (void) fprintf(stderr, 
		     "not as hunky as it should be, and a lot less dory\n");
      exit(1);
   }
   fclose(yyin);
   return(0);
}



void
yyerror(errorstr)

char
   *errorstr;

{
   extern char
      yytext[];
   extern int
      linecount;
   
   (void) fprintf(stderr, "Error [%s] at or before [%s] on line %5d\n",
		  errorstr, yytext, linecount);
}
