#include <stdio.h>
#include "types.h"
#include "output.h"

extern void
   Output();
extern Production
   Grammar[];
extern int
   Gmax;

extern char
   *GetMapping(),
   *makeendtag(),
   *makestarttag();
extern SymbolTable 
   *MakeSymbolTable(),
   *MakeTokenTable();



void
PrintTokens(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *estr,
      *sstr;
   SymbolTable
      *tptr;
   
   fprintf(fptr, "%%token <data>\n");
   fprintf(fptr, "  ICA_Char_DATA\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      if (tptr->symbol[1] == '/')
      {
	 estr = (char *) makeendtag(tptr->symbol);
	 fprintf(fptr, "  %s\n", estr);
	 free(estr);
      }
      else
      {
	 sstr = (char *) makestarttag(tptr->symbol);
	 fprintf(fptr, "  %s\n", sstr);
	 free(sstr);
      }
   }
   return;
}




void
PrintType(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   SymbolTable
      *tptr;
      
   fprintf(fptr, "\n%%type <trans>\n");
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "  %s\n", tptr->symbol);
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "  %sOPTL\n", tptr->symbol);
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "  %sSTAR\n", tptr->symbol);
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "  %sPLUS\n", tptr->symbol);
      }
   }
   return;
}





void
PrintOccur(fptr, symbol, sptr)

FILE
   *fptr;
char
   *symbol;
SymbolList
   *sptr;

{
   for ( ; sptr != NULL; sptr = sptr->next)
   {
      if (strcmp(symbol, sptr->symbolstr) == 0)
      {
	 switch (sptr->occurstr[0])
	 {
	 default:
	    fprintf(fptr, " ");
	    break;
	    
	 case '?':
	    fprintf(fptr, "OPTL ");
	    break;
	    
	 case '*':
	    fprintf(fptr, "STAR ");
	    break;
	    
	 case '+':
	    fprintf(fptr, "PLUS ");
	    break;
	 }
      }
   }
   return;
}




void
PrintProductions(fptr)

FILE
   *fptr;

{
   char
      *estr,
      *mapstr,
      *sstr;
   int
      i;
   RhsList
      *rptr;
   SymbolList
      *sptr,
      *sptr1;
   PermutationList
      *pptr;
   
   for (i = 0; i < Gmax; i++)
   {
      fprintf(fptr, "%s\t: ", Grammar[i].lhs);
      for (rptr = Grammar[i].rhs; rptr != NULL; rptr = rptr->next)
      {
	 for (pptr = rptr->perminstance; pptr != NULL; pptr = pptr->next)
	 {
	    if (pptr->permsymbol != NULL)
	    {
	       sptr = pptr->permsymbol;
	    }
	    else
	    {
	       sptr = rptr->symbol;
	    }
	    for ( ; sptr != NULL; sptr = sptr->next)
	    {
	       if ((sptr->symbolstr[0] == '<') && (sptr->symbolstr[1] == '/'))
	       {
		  estr = (char *) makeendtag(sptr->symbolstr);
		  fprintf(fptr, "%s ", estr);
		  free(estr);
	       }
	       else if (sptr->symbolstr[0] == '<')
	       {
		  sstr = (char *) makestarttag(sptr->symbolstr);
		  fprintf(fptr, "%s ", sstr);
		  free(sstr);
	       }
	       else if (sptr->symbolstr[1] != '?')
	       {
		  fprintf(fptr, "%s", sptr->symbolstr);
		  PrintOccur(fptr, sptr->symbolstr, rptr->symbol);
	       }
	    } /* for sptr */
	 
	    fprintf(fptr, "\n\t\t{\n\t\t  $$ = structconcat(");
	    if (pptr->permsymbol != NULL)
	    {
	       sptr1 = pptr->permsymbol;
	    }
	    else
	    {
	       sptr1 = rptr->symbol;
	    }
	    for (sptr = rptr->symbol; sptr != NULL; sptr = sptr->next)
	    {
	       mapstr = GetMapping(sptr->symbolstr, sptr1);
	       fprintf(fptr, "%s, ", mapstr);
	       free(mapstr);
	    }
	    fprintf(fptr, "NULL);\n");
	    if (i == 0)
	    {
	       fprintf(fptr, "\t\t  structprint($$);\n");
	    }
	    fprintf(fptr, "\t\t}\n");
	    if (pptr->next != NULL)
	    {
	       fprintf(fptr, "\t\t| ");
	    }
	 }  /* for pptr */
	 if (rptr->next == NULL)
	 {
	    fprintf(fptr, "\t\t;\n\n");
	 }
	 else
	 {
	    fprintf(fptr, "\t\t| ");
	 }
      } /* for rptr */
   }
   return;
}




void
PrintExtras(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *str;
   SymbolTable
      *tptr;
      
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      str = tptr->symbol;
      if (tptr->optional > 0)
      {
	 fprintf(fptr, "%sOPTL\t: %s\n", str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->star > 0)
      {
	 fprintf(fptr, "%sSTAR\t: %sSTAR %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| /* empty */\n");
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
      if (tptr->plus > 0)
      {
	 fprintf(fptr, "%sPLUS\t: %sPLUS %s\n", str, str, str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, $2, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t| %s\n", str);
	 fprintf(fptr, "\t\t{\n");
	 fprintf(fptr, "\t\t  $$ = structconcat($1, NULL);\n");
	 fprintf(fptr, "\t\t}\n");
	 fprintf(fptr, "\t\t;\n\n");
      }
   }
   fprintf(fptr, "ICA_Char_DATASTAR\t: ICA_Char_DATASTAR ICA_Char_DATA\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat($1, structnew($2), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t| /* empty content */\n");
   fprintf(fptr, "\t\t{\n");
   fprintf(fptr, "\t\t  $$ = structconcat(structnew(\"\"), NULL);\n");
   fprintf(fptr, "\t\t}\n");
   fprintf(fptr, "\t\t;\n\n");
   return;
}




SymbolTable *
GenerateYacc(fptr)

FILE
   *fptr;

{
   SymbolTable
      *stable,
      *ttable;
   
   Output(fptr, yacchead);
   ttable = NULL;
   ttable = MakeTokenTable(ttable);
   PrintTokens(fptr, ttable);
   stable = NULL;
   stable = MakeSymbolTable(stable);
   PrintType(fptr, stable);
   fprintf(fptr, "\n%%start %s\n\n%%%%\n", Grammar[0].lhs);
   PrintProductions(fptr);
   PrintExtras(fptr, stable);
   return(ttable);
}




void
PrintLexSpec(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   char
      *estr,
      *sstr;
   SymbolTable
      *tptr;
   
   for (tptr = table; tptr != NULL; tptr = tptr->next)
   {
      fprintf(fptr, "\"%s\"\t{\n", tptr->symbol);
      fprintf(fptr, "\t BEGIN 0;\n");
      fprintf(fptr, "#ifndef NOINTERFACE\n");
      fprintf(fptr, "\t UpdateHighlight(yyleng);\n");
      fprintf(fptr, "#endif\n");
      fprintf(fptr, "\t return(");
      if (tptr->symbol[1] == '/')
      {
	 estr = (char *) makeendtag(tptr->symbol);
	 fprintf(fptr, "%s", estr);
	 free(estr);
      }
      else
      {
	 sstr = (char *) makestarttag(tptr->symbol);
	 fprintf(fptr, "%s", sstr);
	 free(sstr);
      }	 
      fprintf(fptr, ");\n\t}\n\n");
   }
   return;
}





void
GenerateLex(fptr, table)

FILE
   *fptr;
SymbolTable
   *table;

{
   Output(fptr, lexhead);
   PrintLexSpec(fptr, table);
   Output(fptr, lextail);
   return;
}
