/********************************************************************
**
**  This routine was developed from flex source.  The nfa and dfa
**  construction routines of flex were used to create a dfa machine
**  to represent the given regular expression.  All code-creating
**  and file-writing routines that appear in the original flex 
**  code have been eliminated.  The yylex() routine was altered to
**  accept input from a string in memory instead of data from a 
**  file.  The main() was renamed to compare_string(), and flexinit()
**  was altered so that it could be called more that once.  The 
**  routine string_test() was added to run the given string through
**  the dfa that gets constructed.  The dfa is stored in the array
**  dfa_trans[].  This array holds the state number of the next dfa
**  state to transfer to and the character which will cause that
**  transition.  The array start_state[] acts as an index into
**  the dfa_trans[] array.  For example, the n'th state in the dfa
**  starts at position indexed by start_state[n] in the dfa_trans[]
**  array.
**
**  All documentation appearing in the flex routines is as it
**  appears in the original flex source.  The new routines and
**  variables are documented as appropriate.
**
********************************************************************/


/* flex - tool to generate fast lexical analyzers */

/*-
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Vern Paxson.
 * 
 * The United States Government has rights in this work pursuant
 * to contract no. DE-AC03-76SF00098 between the United States
 * Department of Energy and the University of California.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1990 The Regents of the University of California.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /n/abyss/2/chameleon/staff/src/ICA/src/Utils/Rex/main.c,v 1.7 92/05/21 22:00:39 ware Exp $ (LBL)";
#endif


#include "flexdef.h"

static char flex_version[] = "2.3";

extern void ccl2ecl PROTO(());

/* declare functions that have forward references */

void flexinit PROTO((int, int));
void readin PROTO(());
void set_up_initial_allocations PROTO(());


/* these globals are all defined and commented in flexdef.h */
int syntaxerror, trace;
int caseins;
int csize;
int onestate[ONE_STACK_SIZE], onesym[ONE_STACK_SIZE];
int onenext[ONE_STACK_SIZE], onedef[ONE_STACK_SIZE], onesp;
int current_mns, lastnfa;
int *firstst, *lastst, *finalst, *transchar, *trans1, *trans2;
int *accptnum;
int numtemps, numprots, protprev[MSP], protnext[MSP], prottbl[MSP];
int protcomst[MSP], firstprot, lastprot, protsave[PROT_SAVE_SIZE];
int numecs, nextecm[CSIZE + 1], ecgroup[CSIZE + 1], nummecs, tecfwd[CSIZE + 1];
int tecbck[CSIZE + 1];
int start_state;
int current_max_dfa_size, current_max_xpairs;
int current_max_template_xpairs, current_max_dfas;
int lastdfa, *nxt, *chk, *tnxt;
int *base, *def, *nultrans, NUL_ec, tblend, firstfree, **dss, *dfasiz;
union dfaacc_union *dfaacc;
int *accsiz, *dhash, numas;
int numsnpairs, jambase, jamstate;
int lastccl, current_maxccls, *cclmap, *ccllen, *cclng, cclreuse;
int current_max_ccl_tbl_size;
Char *ccltbl;
char nmstr[MAXLINE];
int nummt, hshcol, dfaeql, numeps, eps2, num_reallocs;
int tmpuses, totnst, peakpairs, numuniq, numdup, hshsave;
int num_backtracking, bol_needed, eol_needed;
int end_of_buffer_state;

/*  These globals where added to the flex code for the rex routines. */

Dfa_trans_rec  *dfa_trans;
int             dfa_trans_max;
int             flexend_called;
char           *rex;
int            *state_start;
int             new_ccl;
char           *shortest_string;

/*********************************************************************
**
**  compare_string()
**
**  DESCRIPTION:  This is one entry point.  It will create
**     a dfa for the given regular expression and then run the given
**     test_string through it to see if it matches.
**
**  PARAMETERS:
**     reg_exp:      The regular expression.
**     test_string:  A pointer to the string to run through the 
**                      dfa created by the regular expression.
**     caseins_flag: A boolean flag that signals whether or not
**                      the dfa should be case insensitive.
**                   (true : create a case insensitive test)
**                   (false: create a case sensitive test)
**     trace_flag:   A boolean flag that signals whether or not
**                      trace statements should be printed out
**                      to help in debugging.
**                   (true : dump table info to stderr)
**                   (false: do not dump table info)
**
**  RETURN VALUES:
**     STRING_MATCH:
**        The string matches the regular expression.
**
**     STRING_MATCH_RECOPIED:
**        The regular expression contained context sensitive 
**        information (ie. it began with a '^' or ended with a
**        '$') and the string was altered slightly ( a '\n' was
**        added to the beginning or ending of it).  After being
**        altered slightly, the string matched the regular
**        expression, however the string pointer points to the
**        matching string upon return and not to the original
**        string.
**
**     STRING_NO_MATCH:
**        The string did not match the regular expression.
**
**     REX_SYNTAX_ERROR:
**        There was a syntax error in the regular expression.
**
**     STRING_OTHER_ERROR:
**        There was a non-syntax related error that occurred during
**        the building of the dfa machine.
**
***********************************************************************/
CompStrReturn compare_string( reg_exp, test_string, caseins_flag, trace_flag )
char   *reg_exp;
char   **test_string;
int    caseins_flag;
int    trace_flag;

{
char  *copy_string();
void  dump_tables();
CompStrReturn   string_test();

int   result;


/*
**  Copy the regular expression into a global that can be accessed
**     by the yylex() routine.
*/

    rex = copy_string(reg_exp, 0);

/*
**  Initialize the variables and allocate space for the tables if
**     necessary.
*/
    flexinit( caseins_flag, trace_flag );

/*
**  Process the regular expression and build the nfa machine.
*/
    readin();
    if ( syntaxerror )
      return(REX_SYNTAX_ERROR);

/*
**  Convert the nfa to a dfa.
*/
    ntod();

    if (trace)
      dump_tables();

/*
**  Run the string through the dfa
*/
    result = string_test(test_string);
    return(result);
}



/*********************************************************************
**
**  strgen()
**
**  DESCRIPTION:  This is second entry point.  It will create the
**     shortest string which matches the specified regular
**     expression.
**
**  PARAMETERS:
**     reg_exp:      The regular expression.
**
**  RETURN VALUES:
**     A pointer to the string dynamically created by this routine.
**
***********************************************************************/
char  *strgen( reg_exp )
char   *reg_exp;
{
void  readin_strgen();
char  *copy_string();
void  dump_tables();

int   result;


/*
**  Copy the regular expression into a global that can be accessed
**     by the yylex() routine.
*/

    rex = copy_string(reg_exp, 0);

/*
**  Initialize the variables and allocate space for the tables if
**     necessary.
*/
    flexinit( false, false);

/*
**  Process the regular expression and build the nfa machine.
*/
    readin_strgen();
    if ( syntaxerror )
      return("Error");

    return(shortest_string);
}



/*********************************************************************
**
**  dump_tables()
**
**  DESCRIPTION:  This is routine will dump the tables holding the
**     dfa machine.  These tabels are used by the string_test()
**     routine.
**
**  PARAMETERS:
**     None.
**
**  RETURN VALUES:
**     None.
**
***********************************************************************/
void dump_tables()

{
int	x, y, pflag;


   fprintf(stderr, 
      "\n\n DFA TABLES AS STORED IN DFA_TRANS and STATE_START \n\n");


/*
**  Print out the transition table.
**    The first illegal state_start is negative, so stop printing
**    when we come to it.
*/

   x = 1;
   while (state_start[x] >= 0)
     {
      fprintf(stderr, "state # %d:\n", x);
      y = state_start[x];
      while (y < abs(state_start[x+1]))
        {
         fprintf(stderr, "\t\t%d\t\t%d\n", dfa_trans[y].trans_char, 
            dfa_trans[y].next_state);
         y++;
        }
      x++;
     }
   fprintf(stderr, "state_state[%d] = %d \n", x, state_start[x]);


/*
**  Now print out the accepting states.
**    We don't need to go all the way up to current_max_dfas, but it
**    doesn't hurt.
*/

   pflag = 0;
   fprintf(stderr, "\n\ndfaacc tables before make_tables(); \n\n");
   for (x=0; x<current_max_dfas; x++)
     {
      if (dfaacc[x].dfaacc_state != 0) 
        {
         fprintf(stderr, "%3d:  %d \t", x, dfaacc[x].dfaacc_state);
         pflag = 1;
        }
         if ((pflag == 1) && (((x+1) % 5) == 0))
        {
         fprintf(stderr, "\n");
         pflag = 0;
        }
     }


/*
**  Finally, we print out the equivalence classes
*/

   fprintf(stderr, "\n\necgroup tables \n\n");
   for (x=0; x<CSIZE+1; x++)
     {
      fprintf(stderr, "%3d:  %d \t", x, ecgroup[x]);
      if (((x+1) % 5) == 0)
        {
         fprintf(stderr, "\n");
        }
     }
}


/*********************************************************************
**
**  flexend()
**
**  DESCRIPTION:  In the original flex code, flexend was called to
**     abort the program execution.  The routine could be called
**     from anywhere and program execution would terminate.  This
**     is no longer an acceptable situation since the flex code is
**     no longer a stand alone program but merely a subprogram
**     of a larger one.  For this reason, flexend was changed to
**     print an error message to stderr and set a flag to indicate
**     that program execution should terminate.  It does not
**     terminate program execution, however.
**
**  PARAMETERS:
**     None.
**
**  RETURN VALUES:
**     None.
**
**********************************************************************/
void  flexend()
{
   flexend_called = true;

   fprintf(stderr, "***   FLEXEND CALLED: PROGRAM SHOULD TERMINATE ");
   fprintf(stderr, "BUT WILL CONTINUE");
}



/*********************************************************************
**
**  flexinit()
**
**  DESCRIPTION:  This is routine will initialize all of the variables
**     that need to be initialized and allocate space for the tables.
**     If the tables have already been allocated, they will not be
**     allocated again.
**
**  PARAMETERS:
**     caseins_flag:  Indicates whether a case sensitive or case
**        insensitive dfa should be generated.
**     trace_flag:    Indicates whether various tables should be
**        dumped to stderr to aid in debugging.
**
**  RETURN VALUES:
**     None.
**
***********************************************************************/
void flexinit( caseins_flag, trace_flag)
int  caseins_flag;
int  trace_flag;

{
static int  first_call = true;   /*  So we'll only allocate arrays once */
int i, x;

/*
**  Set flags that based on parameters.
*/

    caseins = caseins_flag;
    trace = trace_flag;

/*
**  Set other variables.
*/

    syntaxerror = false;
    flexend_called = false;
    csize = DEFAULT_CSIZE;
    lastccl = 0;


/* initialize the statistics */

    lastdfa = lastnfa = numas = numsnpairs = tmpuses = 0;
    numecs = numeps = eps2 = num_reallocs = hshcol = dfaeql = totnst = 0;
    numuniq = numdup = hshsave = 0;
    num_backtracking = onesp = numprots = 0;
    bol_needed = eol_needed = false;

    firstprot = NIL;


/* used in mkprot() so that the first proto goes in slot 1
 * of the proto queue
 */

    lastprot = 1;


/*  Use equivalence classes. */

 { /* set up doubly-linked equivalence classes */
	/* We loop all the way up to csize, since ecgroup[csize] is the
	 * position used for NUL characters
	 */

   ecgroup[1] = NIL;

   for ( i = 2; i <= csize; ++i )
     {
      ecgroup[i] = i - 1;
      nextecm[i - 1] = i;
     }

   nextecm[csize] = NIL;
 }


/*
**  Only allocate memory for the first call, subsequent calls will use
**     the same space.  If it's not the first call, clear out the
**     ccltab[].
*/

   if (first_call == true)
     {
      set_up_initial_allocations();
      first_call = false;
     }
   else
     {
      for (x = 0; x < CCL_HASH_SIZE; x++)
         ccltab[x] = NIL;
     }
}



/*********************************************************************
**
**  readin()
**
**  DESCRIPTION:  This routine will parse the regular expression
**     and build the nfa.
**
**  PARAMETERS:
**     None.
**
**  RETURN VALUES:
**     None.
**
***********************************************************************/
void readin()

{
   if ( zzparse() )
     {
      fprintf(stderr, "Fatal Parse Error\n" );
      syntaxerror = true;
     }

   numecs = cre8ecs( nextecm, ecgroup, csize );

/* now map the equivalence class for NUL to its expected place */
    
   ecgroup[0] = ecgroup[csize];
   NUL_ec = abs( ecgroup[0] );

   ccl2ecl();
}


/*********************************************************************
**
**  readin_strgen()
**
**  DESCRIPTION:  This routine will parse the regular expression
**     and build the smallest matching string.
**
**  PARAMETERS:
**     None.
**
**  RETURN VALUES:
**     None.
**
***********************************************************************/
void readin_strgen()

{
   if ( syparse() )
     {
      fprintf(stderr, "Fatal Parse Error\n" );
      syntaxerror = true;
     }

}


/*********************************************************************
**
**  set_up_initial_allocations()
**
**  DESCRIPTION:  This is routine allocate the space needed for the
**     tables.
**
**  PARAMETERS:
**     None.
**
**  RETURN VALUES:
**     None.
**
***********************************************************************/
void set_up_initial_allocations()

{
    current_mns = INITIAL_MNS;
    firstst = allocate_integer_array( current_mns );
    lastst = allocate_integer_array( current_mns );
    finalst = allocate_integer_array( current_mns );
    transchar = allocate_integer_array( current_mns );
    trans1 = allocate_integer_array( current_mns );
    trans2 = allocate_integer_array( current_mns );
    accptnum = allocate_integer_array( current_mns );

    current_maxccls = INITIAL_MAX_CCLS;
    cclmap = allocate_integer_array( current_maxccls );
    ccllen = allocate_integer_array( current_maxccls );
    cclng = allocate_integer_array( current_maxccls );

    current_max_ccl_tbl_size = INITIAL_MAX_CCL_TBL_SIZE;
    ccltbl = allocate_character_array( current_max_ccl_tbl_size );

    current_max_dfa_size = INITIAL_MAX_DFA_SIZE;

    current_max_xpairs = INITIAL_MAX_XPAIRS;
    nxt = allocate_integer_array( current_max_xpairs );
    chk = allocate_integer_array( current_max_xpairs );

    current_max_template_xpairs = INITIAL_MAX_TEMPLATE_XPAIRS;
    tnxt = allocate_integer_array( current_max_template_xpairs );

    current_max_dfas = INITIAL_MAX_DFAS;
    base = allocate_integer_array( current_max_dfas );
    def = allocate_integer_array( current_max_dfas );
    dfasiz = allocate_integer_array( current_max_dfas );
    accsiz = allocate_integer_array( current_max_dfas );
    dhash = allocate_integer_array( current_max_dfas );
    dss = allocate_int_ptr_array( current_max_dfas );
    dfaacc = allocate_dfaacc_union( current_max_dfas );

    state_start = allocate_integer_array( current_max_dfas);

    dfa_trans = allocate_dfa_trans( current_max_dfas * AVG_OUT_TRANS);
    dfa_trans_max = current_max_dfas * AVG_OUT_TRANS;

    nultrans = (int *) 0;
}



/*********************************************************************
**
**  string_test()
**
**  DESCRIPTION:  This routine will run the string through the dfa
**     that has been created and return whether it matches of not.
**
**  PARAMETERS:
**     string:  A pointer to the string to put through the dfa.
**
**  RETURN VALUES:
**     STRING_MATCHED:  See description in compare_string()
**     STRING_MATCHED_RECOPIED:  See description in compare_string()
**     STRING_NOT_MATCHED:  See description in compare_string()
**
***********************************************************************/
CompStrReturn   string_test(string)
char  **string;

{
char  *copy_string();

int   x = 0, dt_index = 0, cur_state = 1;
int   changed_state;
char  *chr;
char  *beg_chr;
int   recopied = false;


/*
**  If the string should begin at the beginning of a line,
**     add a '\n' to the beginning of the string if it doesn't already
**     exist.
*/

   chr = *string;
   recopied = false;

   if ((bol_needed == true) && (*string[0] != '\n'))
	  {
		chr = copy_string("\n", strlen(*string));
      strcat(chr, *string);
      recopied = true;
     }


/*
**  If the string should be followed by an eol,
**     add a '\n' to the end of the string if it doesn't already
**     exist.
*/

   if ((eol_needed == true) && (chr[strlen(chr) - 1] != '\n')) 
	  {
		chr = copy_string(chr, 1);
      strcat(chr, "\n");
      recopied = true;
     }


/*
**  Now, put the string through the dfa.
**    Keep going until the end of the string is reached, or there is no
**    out transition from the current state for the current chr.
*/

   beg_chr = chr;
   while (*chr != '\0')
     {
      dt_index = state_start[cur_state];
      changed_state = false;

/*
**  If dt_index is a valid index into the dfa_trans[] table, try to find
**    an out transition for the current chr.
*/  

      if (dt_index >= 0)
        {
/*
**  Loop through all the out transitions for this state.
*/
         while (( dt_index < abs(state_start[cur_state+1])) && 
                (changed_state == false))
           {
/*
**  If out transition matches the current character, go to the next state,
**    otherwise kepp looking.
*/
            if (dfa_trans[dt_index].trans_char == abs (ecgroup[(int)(*chr)]))
              {
               cur_state = dfa_trans[dt_index].next_state;
               changed_state = true;
              }
            else
              {
               dt_index++;
              }
           }
        }


/*
**  If there was no change in state, the string contains a character 
**    that doesn't match the dfa.
*/
      if (changed_state == false) 
        {
         return(STRING_NO_MATCH);
        }

      chr++;
     }

/*
**  The string has been processed, now see if the state we're in
**     is an accpeting state.
*/

   if ((dfaacc[cur_state].dfaacc_state == 1) && (recopied == false))
     {
      return(STRING_MATCH);
     }
   else if (dfaacc[cur_state].dfaacc_state == 1)
     {
      *string = beg_chr;
      return(STRING_MATCH_RECOPIED);
     }
   else
     {
      return(STRING_NO_MATCH);
     }

}
