
/* parse.y - parser for flex input */

%token CHAR NUMBER SECTEND SCDECL XSCDECL WHITESPACE NAME PREVCCL EOF_OP

%{
/*-
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Vern Paxson.
 * 
 * The United States Government has rights in this work pursuant
 * to contract no. DE-AC03-76SF00098 between the United States
 * Department of Energy and the University of California.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /n/abyss/2/chameleon/staff/src/ICA/src/Utils/Rex/parse.y,v 1.3 92/03/18 12:49:24 ware Exp $ (LBL)";
#endif

#include "flexdef.h"

int pat, eps, anyccl, lastchar, i;
int  cclsorted;
Char clower();

static int madeany;      /* whether we've made the '.' character class */


%}

%%
goal            :  initlex sect2
                 { 
                 }
                ;


initlex         :
                 {
                 /* create default DFA start condition */
 
                  start_state = mkstate( SYM_EPSILON );
                  madeany = false;
                 }
                ;


sect2           :  flexrule '\n'
                ;


flexrule        :  rule
                 {
                  pat = $1;
                  finish_rule( pat );

                  start_state = mkbranch( start_state, pat );
                 }

                |  error
                 {
                  synerr( "unrecognized rule" ); 
                 }
                ;


rule            : re
                 {
                  $$ = $1;
                 }
                ;


re              :  re '|' series
                 {
                  $$ = mkor( $1, $3 );
                 }

                |  series
                 {
                  $$ = $1;
                 }
                ;


series          :  series singleton
                 {
                  /* this is where concatenation of adjacent patterns
                   * gets done
                   */
                  $$ = link_machines( $1, $2 );
                 }

                |  singleton
                 {
                  $$ = $1;
                 }
                ;

  
singleton       :  singleton '*'
                 {
                  $$ = mkclos( $1 );
                 }

                |  singleton '+'
                 {
                  $$ = mkposcl( $1 );
                 }

                |  singleton '?'
                 {
                  $$ = mkopt( $1 );
                 }

                |  singleton '{' NUMBER ',' NUMBER '}'
                 {
                  if ( $3 > $5 || $3 < 0 )
                    {
                     synerr( "bad iteration values" );
                     $$ = $1;
                    }
                  else
                    {
                     if ( $3 == 0 )
                       $$ = mkopt( mkrep( $1, $3, $5 ) );
                     else
                       $$ = mkrep( $1, $3, $5 );
                    }
                 }

                |  singleton '{' NUMBER ',' '}'
                 {
                  if ( $3 <= 0 )
                    {
                     synerr( "iteration value must be positive" );
                     $$ = $1;
                    }
                  else
                     $$ = mkrep( $1, $3, INFINITY );
                 }

                |  singleton '{' NUMBER '}'
                 {
                  /* the singleton could be something like "(foo)",
                   * in which case we have no idea what its length
                   * is, so we punt here.
                   */

                  if ( $3 <= 0 )
                    {
                     synerr( "iteration value must be positive" );
                     $$ = $1;
                    }
                  else
                     $$ = link_machines( $1, copysingl( $1, $3 - 1 ) );
                 }

                |  '.'
                 {
                  if ( ! madeany )
                    {
                     /* create the '.' character class */
                     anyccl = cclinit();
                     ccladd( anyccl, '\n' );
                     cclnegate( anyccl );

                     mkeccl( ccltbl + cclmap[anyccl],
                     ccllen[anyccl], nextecm,
                     ecgroup, csize, csize );
  
                     madeany = true;
                    }
                  $$ = mkstate( -anyccl );
                 }

                |  fullccl
                 {
                  if ( ! cclsorted )
                    /* sort characters for fast searching.  We use a
                     * shell sort since this list could be large.
                     */
                    cshell( ccltbl + cclmap[$1], ccllen[$1], true );

                  mkeccl( ccltbl + cclmap[$1], ccllen[$1],
                  nextecm, ecgroup, csize, csize );

                  $$ = mkstate( -$1 );
                 }

                |  PREVCCL
                 {
                  $$ = mkstate( -$1 );
                 }

                |  '"' string '"'
                 {
                  $$ = $2;
                 }

               |  '(' re ')'
                {
                 $$ = $2;
                }

               |  CHAR
                {
                 if ( caseins && $1 >= 'A' && $1 <= 'Z' )
                    $1 = clower( $1 );

                 $$ = mkstate( $1 );
                }
               ;


fullccl        :  '[' ccl ']'
                {
                 $$ = $2;
                }

               |  '[' '^' ccl ']'
                {
                 /* *Sigh* - to be compatible Unix lex, negated ccls
                  * match newlines
                  */

#ifdef NOTDEF
                 ccladd( $3, '\n' ); /* negated ccls don't match '\n' */
                 cclsorted = false; /* because we added the newline */
#endif
                 cclnegate( $3 );
                 $$ = $3;
                }
               ;

ccl            :  ccl CHAR '-' CHAR
                {
                 if ( $2 > $4 )
                    synerr( "negative range in character class" );
                 else
                   {
                    if ( caseins )
                      {
                       if ( $2 >= 'A' && $2 <= 'Z' )
                          $2 = clower( $2 );
                       if ( $4 >= 'A' && $4 <= 'Z' )
                          $4 = clower( $4 );
                      }

                    for ( i = $2; i <= $4; ++i )
                       ccladd( $1, i );

                    /* keep track if this ccl is staying in alphabetical
                     * order
                     */
                    cclsorted = cclsorted && ($2 > lastchar);
                    lastchar = $4;
                   }

                 $$ = $1;
                }

               |  ccl CHAR
                {
                 if ( caseins )
                    if ( $2 >= 'A' && $2 <= 'Z' )
                       $2 = clower( $2 );

                 ccladd( $1, $2 );
                 cclsorted = cclsorted && ($2 > lastchar);
                 lastchar = $2;
                 $$ = $1;
                }

               |
                {
                 cclsorted = true;
                 lastchar = 0;
                 $$ = cclinit();
                }
               ;


string         :  string CHAR
                {
                 if ( caseins )
                    if ( $2 >= 'A' && $2 <= 'Z' )
                       $2 = clower( $2 );

                 $$ = link_machines( $1, mkstate( $2 ) );
                }

               |
                {
                 $$ = mkstate( SYM_EPSILON );
                }
               ;

%%


/* synerr - report a syntax error */

void synerr( str )
char str[];

{
    syntaxerror = true;
    error_message( str );
}


/* error_message - write out a message */

void error_message( str )
char str[];

{
    fprintf( stderr, "%s\n", str );
}


/* yyerror - eat up an error message from the parser;
 *        currently, messages are ignore
 */

void yyerror( msg )
char msg[];

{
}
