/* sparse.y - parser for flex input */

%token CHAR NUMBER SECTEND SCDECL XSCDECL WHITESPACE NAME PREVCCL EOF_OP

%{
/*-
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Vern Paxson.
 * 
 * The United States Government has rights in this work pursuant
 * to contract no. DE-AC03-76SF00098 between the United States
 * Department of Energy and the University of California.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /n/abyss/2/chameleon/staff/src/ICA/src/Utils/Rex/sparse.y,v 1.2 92/03/18 12:49:32 ware Exp $ (LBL)";
#endif

#include "flexdef.h"

int pat, eps, anyccl, lastchar, i;
int  cclsorted;
Char clower();

static int madeany;      /* whether we've made the '.' character class */


%}

%%
goal            :  initlex sect2
                 {
                  shortest_string = makestr($2);
                 }
                ;


initlex         :
                 {
                 }
                ;


sect2           :  flexrule '\n'
                 {
                  $$ = $1;
                 }
                ;


flexrule        :  rule
                 {
                  $$ = $1;
                 }

                |  error
                 {
                  synerr("unrecognized rule");
                 }
                ;


rule            : re
                 {
                  $$ = $1;
                 }
                ;


re              :  re '|' series
                 {
                  $$ = $1;
                 }

                |  series
                 {
                  $$ = $1;
                 }
                ;


series          :  series singleton
                 {
                  $$ = (int) concat(makestr($1), makestr($2), NULL);
                 }

                |  singleton
                 {
                  $$ = $1;
                 }
                ;

  
singleton       :  singleton '*'
                 {
                  $$ = (int) makestr("");
                 }

                |  singleton '+'
                 {
                  $$ = $1;
                 }

                |  singleton '?'
                 {
                  $$ = (int) makestr("");
                 }

                |  singleton '{' NUMBER ',' NUMBER '}'
                 {
                  int    x;
                  char  *current;
                  char  *next;

                      current = makestr("");
                      for (x = 1; x <= $3; x++) {
                          next    = concat(current, makestr($1), NULL);
                          current = next;
                      }

                      $$ = (int) current;
                 }

                |  singleton '{' NUMBER ',' '}'
                 {
                  int    x;
                  char  *current;
                  char  *next;

                      current = makestr("");
                      for (x = 1; x <= $3; x++) {
                          next    = concat(current, makestr($1), NULL);
                          current = next;
                      }

                      $$ = (int) current;
                 }

                |  singleton '{' NUMBER '}'
                 {
                  int    x;
                  char  *current;
                  char  *next;

                      current = makestr("");
                      for (x = 1; x <= $3; x++) {
                          next    = concat(current, makestr($1), NULL);
                          current = next;
                      }

                      $$ = (int) current;
                 }

                |  '.'
                 {
                  $$ = (int) makestr(".");
                 }

                |  fullccl
                 {
                  $$ = $1;
                 }

                |  PREVCCL
                 {
                  char    temp[2];

                  temp[0] = (char) ccltbl[cclmap[$1]];
                  temp[1] = (char) '\0';
                  $$ = (int) makestr(temp);
                 }

                |  '"' string '"'
                 {
                  $$ = $2;
                 }

               |  '(' re ')'
                {
                 $$ = $2;
                }

               |  CHAR
                {
                 char   temp[5];
                 int    value, x;

                     if (yylval == '\b')       {
                         $$ = (int) makestr("\\b");}
                     else if (yylval == '\f')  {
                         $$ = (int) makestr("\\f");}
                     else if (yylval == '\n')  {
                         $$ = (int) makestr("\\n");}
                     else if (yylval == '\r')  {
                         $$ = (int) makestr("\\r");}
                     else if (yylval == '\t')  {
                         $$ = (int) makestr("\\t");}
                     else if (yylval == '\v')  {
                         $$ = (int) makestr("\\v");}
                     else if (iscntrl(yylval)) {
                         value = yylval;
                         temp[0] = '\\';
                         temp[4] = '\0';
                         for (x = 3; x > 0; x--) {
                            temp[x] = (value % 8) + (int) '0';
                            value = value / 8;
                         }

                         $$ = (int) makestr(temp);;
                     }
                     else {
                         temp[0] = (char) yylval;
                         temp[1] = (char) '\0';
                         $$ = (int) makestr(temp);
                     }
                }
               ;


fullccl        :  '[' ccl ']'
                {
                 char  temp[2];
                 char  *ccl_ptr;

                     ccl_ptr = (char *) $2;

                     temp[0] = *ccl_ptr;;
                     temp[1] = (char) '\0';

                     $$ = (int) makestr(temp);

                     ccladd(new_ccl, *ccl_ptr);   /* new_ccl is set in ccl */
                }


               |  '[' '^' ccl ']'
                { 
                 int  x;
                 char temp[2];

                 for (x = 0; x <= 255; x++) {
                     if (!(iscntrl((char) x)) && !(isin((char)x, $3))) {
                         break;
                     }
                 }

                 temp[0] = (char) x;
                 temp[1] = '\0';
                 $$ = (int) makestr(temp);
                }
               ;


ccl            :  ccl CHAR '-' CHAR
                {
                 int    x;
                 char   cur_chr[2];
                 char  *next;
                 char  *current;

                 current = makestr("");
                 for (x = $2; x <= $4; x++) {
                     cur_chr[0] = (char) x;
                     cur_chr[1] = (char) '\0';

                     next = concat(current, makestr(cur_chr), NULL);
                     current = next;
                 }
                 $$ = (int) current;
                }

               |  ccl CHAR
                {
                 char   temp[2];

                     temp[0] = (char) $2;
                     temp[1] = (char) '\0';
                     $$ = (int) concat(makestr($1), makestr(temp), NULL);
                }

               |
                {
                 new_ccl = cclinit();
                 $$ = (int) makestr("");
                }
               ;


string         :  string CHAR
                {
                 char   temp[2];

                     temp[0] = (char) yylval;
                     temp[1] = (char) '\0';
                     $$ = (int) concat(makestr($1), makestr(temp), NULL);

                }
               |
                {
                 $$ = (int) makestr("");
                }
               ;

%%


/* yyerror - eat up an error message from the parser;
 *        currently, messages are ignore
 */

void yyerror( msg )
char msg[];

{
}
