#include <stdio.h>
#include <ctype.h>

#include <icatypes.h>

#define	BOOLN		int
#define	TRUE		1
#define	FALSE		0

ValidateRexReturn  good_backslash();
ValidateRexReturn  good_brace();
ValidateRexReturn  good_bracket();
ValidateRexReturn  good_paren();
ValidateRexReturn  good_percent();
ValidateRexReturn  good_quotes();
ValidateRexReturn  good_trailer();

ValidateRexReturn  is_valid_rex(chr_ptr)
char  *chr_ptr;
/******************************************************************************
**
**   I S  _  V A L I D _ R E X 
**
**      The purpose of this function is to examine a lex regular expression
**   and see if it is syntactically correct.  If it is correct, this function
**   returns TRUE, else it returns the reason it failed.
**
******************************************************************************/

{
int    chr_count = 0;              /*  # chr's in the expression  */
int    paren_cnt = 0;              /*  Parentheses nesting level  */
char   *head;                      /*  1st chr of expression      */
BOOLN  nonblank_found = FALSE;     /*  Flag indicating content of */
                                   /*    parentheses              */
ValidateRexReturn  result;                     /*  The individual routines return */
				   /*  their own and sometimes        */
				   /*  multiple results               */

/*
**  Note the beginning of the exp. in case we need to print it out in an
**    error message.   
*/

   head = chr_ptr; 

 
/* 
**  Now, loop through the regular expression, character by character.
*/

   while (*chr_ptr != '\0')
     {

   /*
   **  First, if we are inside parentheses, check for a non-blank character.
   */

      if ((paren_cnt > 0) && (*chr_ptr != ' ') && (*chr_ptr != ')'))
        {
         nonblank_found = TRUE;
        }


   /*
   **  Now check the syntax of the current character if it has a special
   **  meaning to lex.
   */

      switch (*chr_ptr)
      {
	case '"':
	   result = good_quotes(head, &chr_ptr, &chr_count);
	   if (result != Valid)
	      return(result);
	   break;
         
	case '\\':
	   result = good_backslash(head, &chr_ptr, &chr_count);
	   if (result != Valid)
	      return(result);
	   break;

	case '[':
	   result = good_bracket(head, &chr_ptr, &chr_count);
	   if (result != Valid)
	      return(result);
	   break;

	case '?':
	case '*':
	case '+':
	case '<':
	case '/':
	case '|':
	   result = good_trailer(head, chr_ptr, chr_count);
	   if (result != Valid)
	      return(result);
	   break;

	case '(':
	case ')':
	   result = good_paren(head, chr_ptr, &paren_cnt, &nonblank_found);
	   if (result != Valid)
	      return(result);
	   break;

	case '{':
	case '}':
	   result = good_brace(head, &chr_ptr, &chr_count);
	   if (result != Valid)
	      return(result);
	   break;

	case '%':
	   result = good_percent(head, chr_ptr, chr_count);
	   if (result != Valid)
	      return(result);
	   break;

	case ' ':
#ifdef DEBUG
            fprintf(stderr, "   %s\n", head);
            fprintf(stderr, "** ERROR **  Regular expression may not contain ");
            fprintf(stderr, "a blank. \n");
#endif
            return(BlankFound);
            break;
        }

   /*
   **  Now look at the next char.
   */

      chr_count++;
      chr_ptr++;

     }

 
/*
**  The entire expression has been processed, now make sure that there
**    aren't any open parentheses that haven't been closed.  If there is,
**    it is an error, if not, the expression is legal.
*/  

   if (paren_cnt != 0)
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Missing corresponding ");
      fprintf(stderr, "closing parentheses. \n");
#endif
      return(MissingCloseParen);
     }
   else
     {
      return(Valid);
     }

}


ValidateRexReturn good_quotes(head, p_chr_ptr, p_chr_count)
char  *head;
char  **p_chr_ptr;
int   *p_chr_count;
/******************************************************************************
**
**   G O O D _ Q U O T E S
**
**      This function is called when an open quote is found in the regular
**   expression.  It will search the string until the close quote is found.
**   It should be realized the the close quote may be escaped if preceded by
**   a '\'.  If the closing quote is not found, FALSE is returned, otherwise
**   TRUE is returned.
**
******************************************************************************/
{
/*
**  Skip past the first quote to begin looking for the closing quote.
*/

   (*p_chr_count)++;
   (*p_chr_ptr)++;


/*
**  Continue looping until the close quote is found or we come the the end of
**    the expression.
*/

   while  ((**p_chr_ptr != '\0') && (**p_chr_ptr != '"'))
     {

   /*
   **  Check for the '\', which still has meaning in quotes.
   */

      if (**p_chr_ptr == '\\')      /*  If a '\' is in the quote,     */
        {                           /*   Skip the chr that follows.   */
         (*p_chr_count)++;
         (*p_chr_ptr)++;

         if (**p_chr_ptr == '\0')   /*  If the '\' was the last chr  */
            break;                  /*   in the string, break loop.  */
        }

   /*
   **  Now go on to the next chr.
   */


      (*p_chr_count)++;
      (*p_chr_ptr)++;

    }


/*
**  If the loop was broken for some reason other than finding the close quote,
**    it is an error, otherwise it is legal.
*/

   if (**p_chr_ptr != '"')
     {   
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Missing closing quote. \n");
#endif
      return(MissingCloseQuote);
     }
   else
     {
      return(Valid);
     }

}



ValidateRexReturn good_backslash(head, p_chr_ptr, p_chr_count)
char  *head;
char  **p_chr_ptr;
int   *p_chr_count;
/******************************************************************************
**
**   G O O D _ B A C K S L A S H
**
**      This function is called when a backslash is found in the regular
**   expression.  It will escape the character that follows the '\'.  If no
**   character follows it, FALSE is returned, otherwise TRUE is return.
**   
**
******************************************************************************/
{
/*
**  Look at the character that follows the backslash.
*/
   (*p_chr_ptr)++;
   (*p_chr_count)++;

/*
**  If the backslash is the last character in the regular expression,
**    it is an error, otherwise it is OK.
*/
   if (**p_chr_ptr == '\0')
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Regular expression ");
      fprintf(stderr, "may not end with a `\\`. \n");
#endif
      return(EndingBackslash);
     }
   else
     {
      return(Valid);
     }
}


ValidateRexReturn good_bracket(head, p_chr_ptr, p_chr_count)
char  *head;
char  **p_chr_ptr;
int   *p_chr_count;
/******************************************************************************
**
**   G O O D _ B R A C K E T
**
**      This function is called when an open bracket is found in the regular
**   expression.  It will process the characters that follow until the 
**   closing bracket is found.  During this processing, the `\`, `^`, and `-'
**   characters have special meaning.  Although the '^' character needs no
**   special processing, the '\' and '-' must be checked for correct syntax.
**   If there is a syntax error found in the bracket processing, FALSE is
**   returned, else TRUE is returned.
**
******************************************************************************/
{
char *prior_chr_ptr, *next_chr_ptr;

/*
**  Skip past the open bracket and begin processing.
*/
   (*p_chr_count)++;
   (*p_chr_ptr)++;

/*
**  Continue processing until the closing bracket is found or we come to the
**    end of the expression.
*/

   while  ((**p_chr_ptr != '\0') && (**p_chr_ptr != ']'))
     {

   /*
   **  Process any special characters that may appear in the brackets.
   */

      switch (**p_chr_ptr)
        {

      /*
      **  For the backslash, escape the character that follows and make sure
      **    that the backslash isn't the last character in the expression.
      */
         case '\\' :
            (*p_chr_ptr)++;

            if (**p_chr_ptr == '\0')
              {
#ifdef DEBUG
               fprintf(stderr, "   %s\n", head);
               fprintf(stderr, "** ERROR **  Missing corresponding closing ");
               fprintf(stderr, "bracket. \n");
#endif
               return(MissingCloseBracket);
              }
            break;

      /*
      **  For the dash, several tests are required.
      */
         case '-':

         /*
         **  First, find the characters preceding and succeeding the dash.
         */

            prior_chr_ptr = (*p_chr_ptr) - 1;
            next_chr_ptr  = (*p_chr_ptr) + 1;


         /*
         **  If the dash is preceeded by an open bracket or succeeded by a
         **    closed bracket, it is syntactically correct and no more
         **    processing is needed.
         */

            if (( *prior_chr_ptr == '[') || (*next_chr_ptr == ']'))
               break;


         /*
         **  If the dash is preceeded or succeeded by any other 
         **    non-alphanumeric chr, it is an error.
         */

            if ((!isalnum(*prior_chr_ptr)) || (!isalnum(*next_chr_ptr)))
              {
#ifdef DEBUG
               fprintf(stderr, "   %s\n", head);
               fprintf(stderr, "** ERROR **  Nonportable range specified ");
               fprintf(stderr, "with '-'.\n");
#endif
               return(NonportableRange);
              }


         /*
         **  If the dash is both preceeded and succeeded by alphanumeric
         **    characters, the dash must separate "like" characters.  That 
         **    is to say, two upper case characters, two lower case characters,
         **    or two digits.
         */

            if ((isupper(*prior_chr_ptr) != isupper(*next_chr_ptr)) ||
                (islower(*prior_chr_ptr) != islower(*next_chr_ptr)) ||
                (isdigit(*prior_chr_ptr) != isdigit(*next_chr_ptr)))
              {
#ifdef DEBUG
               fprintf(stderr, "   %s\n", head); 
               fprintf(stderr, "** ERROR **  Nonportable range specified ");
               fprintf(stderr, "with '-'.\n");
#endif
               return(NonportableRange);
              }
            break;

        }

   /*
   **  Now that all special characters have been processed, go to the next chr.
   */

      (*p_chr_count)++;
      (*p_chr_ptr)++;
     }


/*
**  If the loop was broken for some reason other than finding the close bracket,
**    it is an error.
*/

   if (**p_chr_ptr != ']')
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Missing closing bracket. \n");
#endif
      return(MissingCloseBracket);
     }
   else
     {
      return(Valid);
     }
   
}



ValidateRexReturn good_trailer(head, chr_ptr, chr_count)
char  *head;
char  *chr_ptr;
int   chr_count;
/******************************************************************************
**
**   G O O D _ T R A I L E R
**
**      This function is called whenever a character that must follow another
**   character is encountered.  If no character preceeds this character, 
**   FALSE is returned, otherwise TRUE is returned.
**   
**
******************************************************************************/
{
/*
**  If the character is the first character of the regular expression, it
**    is an error.
*/
   if (chr_count == 0)
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Regular expression may not begin ");
      fprintf(stderr, "with a '%c'. \n", *chr_ptr);
#endif
      return(BadStartingChar);
     }
   else
     {
      return(Valid);
     }
}



ValidateRexReturn good_paren(head, chr_ptr, p_paren_cnt, p_nonblank_found)
char  *head;
char  *chr_ptr;
int   *p_paren_cnt;
BOOLN *p_nonblank_found;
/******************************************************************************
**
**   G O O D _ P A R E N
**
**      This function is called whenever an open or closed parentheses is 
**   encountered in the regular expression.  If the the character is an open
**   parenthesis, the parentheses counter is incremented and the nonblank
**   flag is cleared.  If the character is a closed parentheses, the parentheses
**   counter is decremented.  If this counter is below zero or the nonblank
**   flag has not yet been set, FALSE is returned, otherwise TRUE is returned.
**
******************************************************************************/
{
/*
**  If an open parentheses was encountered, increment the parentheses
**    counter and reset the nonblank flag.
*/

   if (*chr_ptr == '(')
     {
      (*p_paren_cnt)++;
      *p_nonblank_found = FALSE;
     }

/*
**  Else, a closed parentheses was encountered so decrement the
**    parentheses counter and check for errors.
*/

   else
     {
      (*p_paren_cnt)--;


   /*
   **  If the paren_cnt falls below zero, the are too many closed parens.
   */

      if (*p_paren_cnt < 0)
        {
#ifdef DEBUG
         fprintf(stderr, "   %s\n", head);
         fprintf(stderr, "** ERROR **  Missing corresponding open ");
         fprintf(stderr, "parentheses. \n");
#endif
         return(MissingOpenParen);
        }


   /*
   **  If the nonblank_found flag is FALSE, there was nothing contained in
   **    the parentheses.
   */

      if ( ! *p_nonblank_found )   /* Nothing between the parens.   */
        {
#ifdef DEBUG
         fprintf(stderr, "   %s\n", head);   
         fprintf(stderr, "** ERROR **  Parentheses must contain ");
         fprintf(stderr, "nonblank character. \n");
#endif
         return(EmptyParens);
        }

     }

/*
**  Everything is OK.
*/

   return(Valid);
}



ValidateRexReturn good_brace(head, p_chr_ptr, p_chr_count)
char  *head;
char  **p_chr_ptr;
int   *p_chr_count;
/******************************************************************************
**
**   G O O D _ B R A C E
**
**      This function is called whenever an open or closed brace is 
**   encountered in the regular expression.  If the the character is a closing
**   brace, it is an error.  If it is an open brace, it must be of the form
**   {x} or {x,y} where x and y are repetition numbers of any number of
**   digits.
**
******************************************************************************/
{
/*
**  If a closing brace was encountered, it is an error.
*/

   if (**p_chr_ptr == '}')
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);   
      fprintf(stderr, "** ERROR **  No corresponding open brace. \n");
#endif
      return(MissingOpenBrace);
     }


/*
**  Now process the open brace.
**    It may not be the first chr of the regular expression.
*/
   if (*p_chr_count == 0)
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Regular expression may not ");
      fprintf(stderr, "begin with a '{'. \n");
#endif
      return(IllegalOpenBrace);
     } 


/*
**  Now, process the contents on the braces.  They should contain repetition
**    numbers, so look at the next chr.
*/

   (*p_chr_ptr)++;
   (*p_chr_count)++;


/*
**  If it is not a digit, it is an error.
*/

   if ( ! isdigit(**p_chr_ptr))
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Only repetition numbers ");
      fprintf(stderr, "may appear in braces. \n");
#endif
      return(BadRepetitionContent);
     }


/*
**  If it was a digit, skip past the rest of the chr's until a nondigit
**    is found.
*/

   while (isdigit(**p_chr_ptr))
     {
      (*p_chr_ptr)++;
      (*p_chr_count)++;
     }


/*
**  If this nondigit is a comma, a second repetition number must follow.
*/

   if (**p_chr_ptr == ',')
     {

   /*
   **  A second repetition number must follow, so look at the next chr.
   */

      (*p_chr_ptr)++;
      (*p_chr_count)++;

   /*
   **  Again, if it is not a digit, it is an error.
   */
      if ( ! isdigit(**p_chr_ptr))
        {
#ifdef DEBUG
         fprintf(stderr, "   %s\n", head);
         fprintf(stderr, "** ERROR **  Only repetition numbers may ");
         fprintf(stderr, "appear in braces. \n");
#endif
         return(BadRepetitionContent);
        }


   /*
   **  If it was a digit, skip past the rest of the chr's until a nondigit
   **    is found.
   */ 
      while (isdigit(**p_chr_ptr))
        {
         (*p_chr_ptr)++;
         (*p_chr_count)++;
        }

   }


/*
**  The nondigit that was encountered must now be a closing brace.  If it
**    is not, it is an error.
*/

   if (**p_chr_ptr != '}')
     {
#ifdef DEBUG
      fprintf(stderr, "   %s\n", head);
      fprintf(stderr, "** ERROR **  Closing brace not in expected ");
      fprintf(stderr, "position. \n");
#endif
      return(WhereCloseBrace);
     }
   else
     {
      return(Valid);
     }

}



ValidateRexReturn good_percent(head, chr_ptr, chr_count)
char  *head;
char  *chr_ptr;
int   chr_count;
/******************************************************************************
**
**   G O O D _ P E R C E N T
**
**      This function is called whenever a percent sign is encountered.  It
**   is an error only if two percent signs appear as the first two characters
*    of the regular expresion.  If this is the case, FALSE is returned,
**   otherwise TRUE is returned.
**
******************************************************************************/
{
char  *next_chr_ptr;

 
/*
**  Get a pointer to the next character.
*/
   next_chr_ptr = chr_ptr + 1;


/*
**  If the first two characters of the line are percent signs, 
**    it is an error, otherwise it is OK.
*/

   if ((chr_count == 0) && (*next_chr_ptr == '%'))
        {
#ifdef DEBUG
         fprintf(stderr, "   %s\n", head);
         fprintf(stderr, "** ERROR **  Regular expression may not begin "); 
         fprintf(stderr, "with '%%%%'. \n");
#endif
         return(IllegalPercent);
        }
   else
     {
      return(Valid);
     }

}
