#include <varargs.h>
#include <stdio.h>

#include <icatypes.h>

/* 
   concat a bunch of strings together format:
   newstring = concat("first", "second", ..., NULL);
  
   automatically free's its arguments.
 */

char *
concat(va_alist)

va_dcl

{
   va_list
      ap;
   char
      *arg,
      *str;
   int
      newlen = 0;
   
   /* valid initial value for str */
   if ((str = (char *) malloc((unsigned) 8)) == NULL)
   {
      (void) fprintf(stderr, "concat ERROR: unable to malloc initial str\n");
      exit(1);
   }
   str[0] = '\0';
   
   /* gather the argument pointers */
   va_start(ap);
   arg = va_arg(ap, char *);
   
   while (arg != NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "Current concat argument = [%s]\n", arg);
#endif
      /* figure out how long the new string needs to be */
      newlen += strlen(arg);
      newlen++;
      
      /* get the space for the new string */
      if ((str = (char *) realloc(str, (unsigned) newlen)) == NULL) 
      {
	 (void) fprintf(stderr,
			"concat ERROR: realloc NULL for [%5d] byte(s)\n",
			newlen);
	 exit(1);
      }
      
      /* copy the current argument into the new string */
      (void) strcat(str, arg);

      /* free up the current argument */
      (void) free(arg);
      
      /* get the next argument to consider */
      arg = va_arg(ap, char *);
   }
   va_end(ap);

#ifdef DEBUG
   fprintf(stderr, "Concatenated string = [%s]\n", str);
#endif   
   return(str);
}



/*
 * The structconcat() connects structures and not strings.
 */

ConcatStruct *
structconcat(va_alist)

va_dcl

{
   va_list
      ap;
   ConcatStruct
      *old,
      *new,
      *start,
      *trace;
   
   /*
    * Initialize pointers.
    */
   old   = NULL;
   new   = NULL;
   start = NULL;

   /*
    * Start the variable arguments.
    */
   va_start(ap);

   /*
    * For each argument, link it up.
    */
   for (new = va_arg(ap, ConcatStruct *);
	new != NULL;
	new = va_arg(ap, ConcatStruct *))
   {
#ifdef DEBUG
      fprintf(stderr, "Current concat argument = [%s]\n", new->string);
#endif
      if (old == NULL)
      {
	 old   = new;
	 start = new;
      }
      else
      {
	 if (old->next == NULL)
	 {
	    old->next = new;
	 }
	 else if (old->tail != NULL)
	 {
	    old->tail->next = new;
	 }
	 else
	 {
	    /*
	     * Just in case something is out of synch, i.e., old->next
	     * is not NULL, but its tail is NULL.  Reconcile it by
	     * following the next pointer put its real tail point next
	     * to new, so the next pointer is correct, and make old's
	     * tail be trace.  The next piece adjusts the tail so it is
	     * really correct anyway.
	     */
	    for (trace = old; trace->next != NULL; trace = trace->next)
	    {
	    }
	    trace->next = new;
	 }
	 if (new->tail == NULL)
	 {
	    old->tail = new;
	 }
	 else
	 {
	    old->tail = new->tail;
	 }
      } /***** ELSE it is not the first arg */
   } /***** FOR each argument */

   /*
    * End variable argument usage
    */
   va_end(ap);

   /*
    * Return the pointer to the beginning of the list
    */
   return(start);
}



ConcatStruct *
structnew(str)

char
   *str;

{
   ConcatStruct
      *new;
   
   new = (ConcatStruct *) malloc((unsigned) sizeof(ConcatStruct));
   if (new == NULL)
   {
      fprintf(stderr,
	      "structnew: Error allocating structure for [%s]\n",
	      str);
      exit(2);
   }
   new->string = str;
   new->next   = NULL;
   new->tail   = NULL;
   return(new);
}




void
structprint(csptr)

ConcatStruct
   *csptr;

{
   int
      scnt;
   
   for (scnt = 0; csptr != NULL; csptr = csptr->next, scnt++)
   {
      fputs(csptr->string, stdout);
   }
   fflush(stdout);
   fprintf(stderr,
	   "\n\nNumber of structures [%d] of size [%d] used.\n",
	   scnt, sizeof(ConcatStruct));
   return;
}

