/* CONTENTS
   Owner:       Copyright 1991 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: retagdriver.c,v $
   Purpose:     ITT (insert tag tool) driver, main program

   Programmer:  Julie Ann Barnes
   Reviewer:    
   Revision:    $Revision: 1.6 $
   Date:        $Date: 92/05/21 21:57:42 $

NOTE:
   AVAIL can be:
	HIDDEN for internal use only
	EXPORT for internal and external use

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* HISTORY
   $Log:	retagdriver.c,v $
 * Revision 1.6  92/05/21  21:57:42  ware
 * 
 * Allow files to be passed on command line instead of only stdin.  Clean
 * up several error messages.
 * 
 * Revision 1.5  92/03/17  18:04:30  ware
 * Use ICA.defs
 * 
 * Revision 1.4  1992/03/10  17:17:24  cso
 * Added the -DFLEX cflag for yytext
 *
 * Revision 1.3  1992/01/28  17:47:56  cso
 * Aesthetics
 *
 * Revision 1.2  1991/11/06  20:05:52  cso
 * added function headers
 * modified pop to keep popping the stack until the context that it was
 * called with is popped off (of course checks for the bottom of the
 * stack are done too).
 * increased stack size from 50 to 500.
 *
 * Revision 1.1.1.2  1991/10/04  19:41:40  jbm
 * Integrated Chameleon Architecture
 * Ohio State University
 * Alpha 2.0 release
 *
*/

/* #INCLUDES */
#include <stdio.h>
#include <ctype.h>

#define STACKSIZE 500
#define BOTTOM 0

/* EXTERNS */

#ifdef FLEX
extern char *yytext;
#else
extern char yytext[];
#endif
extern FILE *yyin;

/* GLOBALS */
int linecount;
int sc_stack[STACKSIZE];
int top;




int
cntlinefeeds(str)

char
   *str;

{
   int
      i,
      count;

   count = 0;
   for (i = 0; str[i] != '\0'; i++)
   {
      if (str[i] == '\n')
      {
         count++;
      }
   }
   return(count);
}




/*
   FUNCTION: 		pop

   Synopsis: 
       Pops a context off the context stack.

   Initial purpose:
       Same as above.

   Present purpose:
       Keep popping until the context passed in has been popped off,
       or the bottom of the stack has been reached.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	Conleth O'Connell

   Modifications:
       DATE     WHO      REASON
       91/11/04 cso      Added the continual popping to eliminate some
                          syntax errors and add spec. power.

   Globals used:
       top      The current index into the stack.
       sc_stack The context stack

   Globals Modified:
       top
       sc_stack

   Return value:	int

   Range:		> 0
*/


int
pop(ctxt)

int
   ctxt;

{
   if (top > BOTTOM)
   {
      while ((top > BOTTOM) && (sc_stack[top] != ctxt))
      {
	 top--;
      }
      if (top == BOTTOM)
      {
	 fprintf(stderr, "Start condition stack underflow.\n");
	 fprintf(stderr, "Trying to end an unstarted clause\n");
	 return(BOTTOM);
      }
      else
      {
	 top--;
	 return(sc_stack[top]);
      }
   }
   else
   {
      fprintf(stderr, "Start condition stack underflow.\n");
      fprintf(stderr, "Trying to end an unstarted clause\n");
      return(BOTTOM);
   }
}



/*
   FUNCTION: 		push

   Synopsis: 
       Push a context onto the stack.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       top      Index on the stack of the current context on the stack.
       sc_stack The context stack

   Globals Modified:
       top
       sc_stack

   Return value:	void

   Range:		none
*/


void
push(ctxt)

int
   ctxt;

{
   if (top < STACKSIZE)
   {
      top++;
      sc_stack[top] = ctxt;
   }
   else
   {
      fprintf(stderr, "Start condition stack overflow.\n");
      fprintf(stderr, "Change STACKSIZE in $ICAROOT/src/Utils/retagdriver\n");
      fprintf(stderr, "remake library, then remake retagger from retag.\n");
      exit(1);
   }
   return;
}



/*
   FUNCTION: 		

   Synopsis: RTT

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Ann Barnes
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Operating System issues:
       CALL		 PURPOSE

   System files modified:
       none

   Globals used:
       none

   Globals Modified:
       none

   Return value:	int

   Range:		0
*/

int
main(argc, argv)

int
   argc;
char
   *argv[];

{
   fprintf(stderr, "Retagging...\n");
   linecount     = 1;
   top           = BOTTOM;
   sc_stack[top] = BOTTOM;
   if (argc > 1)
   {
      if ((yyin = fopen(argv[1], "r")) == NULL)
      {
	 (void) fprintf(stderr, "Bad input file name: %s \n", argv[1]);
	 exit (1);
      }
   }
   if (yyparse() == 0)
   {
      fprintf(stderr, "Tag replacement process completed.\n");
   }
   else
   {
      fprintf(stderr, "Tag replacement process failed.\n");
   }
   return(0);
}


void
yyerror(errorstr)

char
   *errorstr;

{
   fprintf(stderr, "Error [%s] at or before [%s] on line [%5d]\n",
	   errorstr, yytext, linecount);
   return;
}

int
yywrap()

{
   fprintf(stderr, "Number of input lines read = %6d\n", linecount);
   return(1);
}
