/*
 * $Log:	txmain.c,v $
 * Revision 1.3  92/03/17  18:04:33  ware
 * Use ICA.defs
 * 
 * Revision 1.2  1992/03/10  17:18:16  cso
 * Added push(), pop(), Top, and ICA_stack for context translation
 *
 * Revision 1.1  1992/02/25  02:09:13  cso
 * Initial revision
 *
 * Revision 1.1  1992/01/28  17:45:42  cso
 * Initial revision
 *
 */

#include <ctype.h>
#include <stdio.h>
#include <varargs.h>

#ifdef FLEX
extern char *yytext;
#else
extern char yytext[];
#endif

#define STACKSIZE 500
#define BOTTOM 0

/* GLOBALS */
int ICA_stack[STACKSIZE];
int Top;

extern FILE *yyin;
extern int  linecount;



/*
   FUNCTION: 		pop

   Synopsis: 
       Pops a context off the stack.

   Initial purpose:
       Same as above.

   Present purpose:
       Keep popping until the context passed in has been popped off,
       or the bottom of the stack has been reached.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       top      The current index into the stack.
       sc_stack The context stack

   Globals Modified:
       top
       sc_stack

   Return value:	int

   Range:		> 0
*/


void
pop(ctxt)

int
   ctxt;

{
   if (Top > BOTTOM)
   {
      if (Top == BOTTOM)
      {
	 fprintf(stderr, "Stack underflow: ");
	 fprintf(stderr, "Trying to pop an empty stack\n");
      }
      else
      {
	 Top--;
      }
   }
   else
   {
      fprintf(stderr, "Stack underflow: ");
      fprintf(stderr, "Trying to pop an empty stack\n");
   }
   return;
}



/*
   FUNCTION: 		push

   Synopsis: 
       Push a context onto the stack.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Julie Barnes, Conleth O'Connell
   Reviewer:	

   Modifications:
       WHO     DATE      REASON

   Globals used:
       top      Index on the stack of the current context on the stack.
       sc_stack The context stack

   Globals Modified:
       top
       sc_stack

   Return value:	void

   Range:		none
*/


void
push(ctxt)

int
   ctxt;

{
   if (Top < STACKSIZE)
   {
      Top++;
      ICA_stack[Top] = ctxt;
   }
   else
   {
      fprintf(stderr, "Stack overflow: Trying to push on a full stack\n");
      fprintf(stderr, "Change STACKSIZE in $ICAROOT/src/Utils/txmain.c\n");
      fprintf(stderr,
	      "Reinstall library, then remake translator from gen2spec.\n");
      exit(1);
   }
   return;
}



void
UpdateHighlight(tokenlen)

int
   tokenlen;

{
   /* This is because the spec2gen interface generates code for both */
   /* batch and interactive translators, but only the interactive */
   /* cares */
   return;
}



int
main(argc, argv)

int
   argc;
char
   **argv;

{
   if (argc > 1)
   {
      if ((yyin = fopen(argv[1], "r")) == NULL)
      {
	 (void) fprintf(stderr, "Bad input file name: %s \n", argv[1]);
	 exit (1);
      }
   }
  
   linecount = 1;
  
   if (yyparse() == 0) 
   {
      (void) fprintf(stderr, "Translated with no problems\n");
   }
   else 
   {
      (void) fprintf(stderr, "Translation incomplete\n");
   }
   return(0);
}


void
yyerror(errorstr)

char
   *errorstr;

{
  (void) fprintf(stderr, "Error [%s] at or before [%s] on line [%5d]\n",
		 errorstr, yytext, linecount);
  return;
}

