#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>
   
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Viewport.h>
#include <Xica/Group.h>

#define MAXPRODS 10000
#define TITLE    "Specific->General Translation Interface"
   

extern char
   *yyrule[];
extern FILE
   *yyin;
extern int
   linecount;
extern short
   yycontab[];

XtAppContext
   Appcon;
Widget
   Toplevel;

static String
   Fallbacks[] = {
   "*title:                     Specific->General Translation Interface",
   "*iconName:                  Specific->General Translation Interface",
   "*framebox.background:       XtDefaultForeground",
   "*docview.prefWidth:         600",
   "*docview.minWidth:          150",
   "*docview.prefHeight:        400",
   "*docview.minHeight:         100",
   "*docview.stretch:           True",
   "*listviewport.prefWidth:    600",
   "*listviewport.minWidth:     150",
   "*listviewport.prefHeight:   250",
   "*listviewport.minHeight:    50",
   "*listviewport.stretch:      True",
   NULL };

String
   Choices[MAXPRODS],
   Waitlist[] = {"Please wait - PARSING...", NULL};

int
   Beghilight  = 0,
   Choicemade  = 0,
   Endhilight  = 0,
   Numchoices  = 0,
   Shiftflag   = 0;



void
UpdateHighlight(tokenlen)

int
   tokenlen;

{
   Beghilight =  Endhilight;
   Endhilight += tokenlen;
   return;
}



static void
GetChoiceMade(w, client_data, choice)

Widget
   w;
caddr_t
   client_data;
XawListReturnStruct
   *choice;

{
   Choicemade = choice->list_index;
   return;
}

static void
Exit(w, client_data, call_data)

Widget
   w;
caddr_t
   client_data,
   call_data;

{
   XtDestroyWidget(Toplevel);
   exit(0);
}     




void
yyerror(errorstr)

char
   *errorstr;

{
#ifdef FLEX
   extern char *yytext;
#else
   extern char yytext[];
#endif

  (void) fprintf(stderr, "Error [%s] at or before [%s] on line [%5d]\n",
		 errorstr, yytext, linecount);
  return;
}




static void
IntParserMainLoop()

{
   XEvent
      event;
   
   for ( ; ; )
   {
      XtAppNextEvent(Appcon, &event);
      XtDispatchEvent(&event);
      if ((Choicemade > 0) && (Choicemade < Numchoices))
      {
	 break;
      }
   }
   return;
}



/* Use yaccindex to look at the conflicts of the state-terminal pair
   under consideration.  If index (yycontab[yaccindex]) is -1 then it is a
   shift-reduce conflict, otherwise just reduce-reduce.  Index is then
   used as an index into the list of productions (yyrule[]).  */

int
CreateChoiceList(yaccindex)

int
   yaccindex;

{
   extern char
      *yytext;
   int
      index,
      i,
      j;
   char
      temp[BUFSIZ],
      *token;
   
   i          = 0;
   Shiftflag  = 0;
   token      = (char *) yytext;
   Choices[i] = XtMalloc(strlen(token) + 64);
   (void) sprintf(Choices[i], "Which production does [%s] end?", token);
   i++;

   for (index = yycontab[yaccindex];
	(index = yycontab[yaccindex]) != 0;
	yaccindex++)
   {
      /*
       * If index equals -1, it is a shift-reduce conflict
       * else get the productions that can be reduced.
       */
      if (index == -1)
      {   
	 Shiftflag = 1;
      }
      else
      {
	 for (j = 0; 
	      ((yyrule[index][j] != ':') && (j < BUFSIZ));
	      j++)
	 {
	    temp[j] = yyrule[index][j];
	 }
	 if (j == BUFSIZ)
	 {
	    (void) fprintf(stderr,
			   "CreateChoiceList: BUFSIZ [%5d] exceeded\n",
			   BUFSIZ);
	    (void) fprintf(stderr, "Reset and Recompile\n");
	    j--;
	 }
	 temp[j] = '\0';
	 
	 Choices[i] = XtMalloc(strlen(temp) + 16);
	 (void) sprintf(Choices[i], "%5d - %s", i, temp);
	 i++;
      } /***** ELSE a production to reduce */
   } /***** FOR each production in conflict */

   /*
    * If it is a shift-reduce conflict
    * add the none of the above choice
    */
   if (Shiftflag == 1)
   {
      Choices[i] = XtMalloc(64);
      sprintf(Choices[i], "%5d - None of the above", i);
      i++;
   }
   return(i);
}



/**********  User Interface called from yyparse ******/

int
ResolveConflict(yaccindex)

int
   yaccindex;

{
   int
      i;
   Widget
      docview,
      choicelist;
   XEvent
      event;
   
   /*
    * Obtain the widgets for viewing the document, and showing the
    * list of choices.
    * Select the token in the document view
    * Set the Insertion point at the end of the selection
    * Reset the choice value
    * Create the list of choices
    * Update the choice list
    * Process events until the user selects a choice
    * Swap choice list with a waiting message
    * Unselect the token
    * Process some events so the user sees the update
    */
   docview    = XtNameToWidget(Toplevel, "*docview");
   choicelist = XtNameToWidget(Toplevel, "*choicelist");
   XawTextSetSelection(     docview, Beghilight, Endhilight);
   XawTextSetInsertionPoint(docview, Endhilight);
   Choicemade = 0;
   Numchoices = CreateChoiceList(yaccindex);
   XawListChange(choicelist, Choices, Numchoices, -1, True);
   IntParserMainLoop();
   XawListChange(choicelist, Waitlist, -1, -1, True);
   XawTextUnsetSelection(docview);
   for (i = 0; i < Numchoices; i++)
   {
      XtFree(Choices[i]);
      Choices[i] = NULL;
   }
   
   for (i = 0; i < 4; i++)
   {
      XtAppNextEvent(Appcon, &event);
      XtDispatchEvent(&event);
   }
   
#ifdef DEBUG
   fprintf(stderr, "Choice made                                   = [%d]\n",
	   ChoiceMade);
   fprintf(stderr, "DEFAULT: yycontab[yaccindex]                  = [%d]\n",
	   yycontab[yaccindex]);
   fprintf(stderr, "SR     : yycontab[yaccindex + Choicemade]     = [%d]\n",
	   yycontab[yaccindex + Choicemade]);
   fprintf(stderr, "RR     : yycontab[yaccindex + Choicemade - 1] = [%d]\n",
	   yycontab[yaccindex + Choicemade - 1]);
#endif

   /*
    * If it was a shift-reduce conflict
    * * If the choice was the none of the above selection
    * * the user chose shift, so return the first value in the
    * * conflict list (-1)
    * * Else return the initial index plus the number of the choice
    * Else it was a reduce-reduce conflict
    * return the initial value plus one minus the choice made b/c the
    * initial index could also be chosen
    */
   if (Shiftflag == 1)
   {
      if (Choicemade == (Numchoices - 1))
      {
	 return(yycontab[yaccindex]);
      }
      else 
      {
	 return(yycontab[yaccindex + Choicemade]);
      }
   }
   else
   {
      return(yycontab[yaccindex + Choicemade - 1]);
   }
}



/**********  CreateInterface Widget Procedure  ***************/

static Widget
CreateInterface(parent)

Widget
   parent;

{
   Arg
      args[20];
   int
      cnt;
   Widget
      choicelist,
      cmdbut,
      docview,
      label,
      viewport;
   
   cmdbut = XtVaCreateManagedWidget( "exit",
       commandWidgetClass, parent,
       XtNlabel,     "EXIT",
       XtNalignment, gaFill,
       XtNmargin,    8,
       NULL );
   XtAddCallback(cmdbut, XtNcallback, Exit, NULL);
   
   label = XtVaCreateManagedWidget( "label",
       labelWidgetClass, parent,
       XtNlabel,       "Input Document",
       XtNborderWidth, 0,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       NULL );
   
   docview = XtVaCreateManagedWidget( "docview",
       asciiTextWidgetClass, parent,
       XtNscrollHorizontal, XawtextScrollAlways,
       XtNscrollVertical,   XawtextScrollAlways,
       XtNeditType,         XawtextAppend,
       XtNtype,             XawAsciiString,
       XtNpreference,       False,
/*
       XtNprefWidth,        600,
       XtNminWidth,         150,
       XtNprefHeight,       400,
       XtNminHeight,        100,
       XtNstretch,          True,
*/
       XtNalignment,        gaFill,
       XtNmargin,           8,
       NULL );
   XawTextEnableRedisplay(docview);

   label = XtVaCreateManagedWidget( "label",
       labelWidgetClass, parent,
       XtNlabel,       "Choices",
       XtNborderWidth, 0,
       XtNalignment,   gaFill,
       XtNmargin,      8,
       NULL );
   
   viewport = XtVaCreateManagedWidget( "listviewport",
       viewportWidgetClass, parent,
       XtNallowVert,    True,
       XtNallowHoriz,   True,
       XtNforceBars,    True,
       XtNpreference,   False,
/*
       XtNprefWidth,    600,
       XtNminWidth,     150,
       XtNprefHeight,   250,
       XtNminHeight,    50,
       XtNstretch,      True,
*/
       XtNalignment,    gaFill,
       XtNmargin,       8,
       NULL );
   
   choicelist = XtVaCreateManagedWidget( "choicelist",
       listWidgetClass, viewport,
       XtNverticalList,   True,
       XtNforceColumns,   True,
       XtNdefaultColumns, 1,
       XtNlist,           Waitlist,
       NULL );
   XtAddCallback(choicelist, XtNcallback, GetChoiceMade, NULL);
   Numchoices = 0;
   Choicemade = 0;
   return(docview);
}



static void
EchoFile(w, fp)

Widget
   w;
FILE
   *fp;

{
   char
      *buffer;
   int
      endpt;
   XawTextBlock
      src;       /* Contains information to append string to widget */
   
   endpt  = 0;
   buffer = XtMalloc(BUFSIZ);
   while ((buffer = fgets(buffer, BUFSIZ, fp)) != NULL)
   {
      src.firstPos = 0;
      src.length   = strlen(buffer);
      src.ptr      = buffer;
      src.format   = FMT8BIT;
      (void) XawTextReplace(w, endpt, endpt, &src);
      endpt += src.length;
      XawTextSetInsertionPoint(w, endpt);
   }

   XtFree(buffer);
   return;
}



int
main(argc, argv)

unsigned int
   argc;
char
   **argv;

{
   int
      i;
   Widget
      docview,
      framebox;
   XEvent
      event;
   
   /*********  Create Toplevel and frame widgets       ************/
   
   Toplevel = XtVaAppInitialize(
       &Appcon, "SGtrans", NULL, 0, &argc, argv, Fallbacks,
       XtNinput,            True,
/*
       XtNtitle,            TITLE,
       XtNiconName,         TITLE,
*/
       XtNallowShellResize, True,
       NULL );
   
   framebox  = XtVaCreateManagedWidget( "framebox",
       groupWidgetClass, Toplevel,
       XtNspacing,         10,
       NULL );
   
   /*********  Create the other widgets       ************/
   
   docview = CreateInterface(framebox);
   if (argc > 1)
   {
      if ((yyin = fopen(argv[1], "r")) == NULL)
      {
	 fprintf(stderr, "Unable to open [%s]\n", argv[1]);
	 exit(1);
      }
   }
   else
   {
      yyin = stdin;
   }

   EchoFile(docview, yyin);
   XtVaSetValues( docview, XtNeditType, XawtextRead, NULL );
   
   /*********  Final Mapping of Widget Toplevel **********/
   XtRealizeWidget(Toplevel);
   for (i = 0; i < 4; i++)
   {
      XtAppNextEvent(Appcon, &event);
      XtDispatchEvent(&event);
   }

   /*
    * Prepare to parse the document
    * Parse it
    * Clean up
    */
   
   if (fseek(yyin, (long) 0, 0) != 0)
   {
      fprintf(stderr, "Cannot use [%s] in a pipe\n", argv[0]);
      fprintf(stderr, " Usage options are:\n");
      fprintf(stderr, "   %s < file ... continue pipe\n", argv[0]);
      fprintf(stderr, "   %s file   ... continue pipe\n", argv[0]);
      exit(1);
   }
   
   if (yyparse() == 0)
   {
      (void) fprintf(stderr, "Translated with no problems\n");
   }
   else
   {
      (void) fprintf(stderr, "Translation incomplete\n");
   }

   fclose(yyin);

   XtDestroyWidget(Toplevel);
   return(0);
}
