/*
$Header: /n/abyss/2/chameleon/staff/src/ICA/src/Xica/FileDlg.c,v 1.12 92/05/25 17:46:05 ware Exp $
*/
/*
   FileDlg.c:  File Open Dialog Box Widget

   $Author: ware $

   $Revision: 1.12 $

   $Date: 92/05/25 17:46:05 $

   $Log:	FileDlg.c,v $
 * Revision 1.12  92/05/25  17:46:05  ware
 * Handle getwd() vs getcwd() on SYSV
 * 
 * Revision 1.11  92/05/21  22:05:26  ware
 * Removed duplicat <dirent.h> and unneeded <sys/unistd.h>
 * 
 * Revision 1.10  92/05/07  16:09:24  boyd
 * 
 * Oops... Almost but not quite.  This one will do it - keeps vp width dynamic
 * without the height changing.
 * 
 * Revision 1.9  92/05/07  15:51:29  boyd
 * 
 * JB - changed the file name list viewport to preference False, so that it
 * doesn't get resized so sporadically.
 * 
 * Revision 1.8  92/03/18  13:47:07  ware
 * Changes to support better geometry management
 * 
 * Revision 1.3  92/03/05  17:17:22  boyd
 * Changes for use with new version of Group;  also changed spacings/margins.
 * 
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <malloc.h>
#include <dirent.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/Misc.h>
#include <X11/Xaw/XawInit.h>

#include <X11/Shell.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Scrollbar.h>
#include "IcaText.h"
#include "Group.h"
#include "SensList.h"
#include "Wait.h"
#include "MsgBox.h"

#include "FileDlgP.h"

#define streq(a,b) (strcmp( (a), (b) ) == 0)

#define BEEP(w)    XBell( XtDisplay(w), 100 )

#define offset(field) XtOffset( FileDlgWidget, field )
/*
    {name, class, type, size,
        offset, default_type, default_addr},
*/
static XtResource resources[] = {
    {XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof(Dimension),
        offset(core.border_width), XtRImmediate, (XtPointer) 2},
    {XtNallowShellResize, XtCAllowShellResize, XtRBoolean, sizeof(Boolean),
        offset(shell.allow_shell_resize), XtRImmediate, (XtPointer) True},
    {XtNdataFont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
        offset(filedlg.dataFont),XtRString, "XtDefaultFont"},
    {XtNlabelFont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
        offset(filedlg.labelFont),XtRString, "XtDefaultFont"},
    {XtNbtnFont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
        offset(filedlg.btnFont),XtRString, "XtDefaultFont"},
    {XtNbtnCursor, XtCCursor, XtRCursor, sizeof(Cursor),
        offset(filedlg.btnCursor), XtRString, "hand2"},
    {XtNspacing, XtCThickness, XtRInt, sizeof(int),
        offset(filedlg.spacing), XtRImmediate, (XtPointer) 8},
    {XtNlines, XtCLines, XtRInt, sizeof(int),
        offset(filedlg.lines), XtRImmediate, (XtPointer) 6},
    {XtNpath, XtCPath, XtRString, sizeof(String),
        offset(filedlg.path), XtRString, NULL},
    {XtNspec, XtCSpec, XtRString, sizeof(String),
        offset(filedlg.spec), XtRString, "*"},
    {XtNname, XtCName, XtRString, sizeof(String),
        offset(filedlg.name), XtRString, NULL},
    {XtNdlgTitle, XtCTitle, XtRString, sizeof(String),
        offset(filedlg.title), XtRString, NULL},
    {XtNprompt, XtCPrompt, XtRString, sizeof(String),
        offset(filedlg.prompt), XtRString, "Open:"},
    {XtNbutton, XtCButton, XtRString, sizeof(String),
        offset(filedlg.button), XtRString, " OK "},
    {XtNmustMatch, XtCMustMatch, XtRBoolean, sizeof(Boolean),
        offset(filedlg.mustMatch), XtRImmediate, (XtPointer) True},
    {XtNmustExist, XtCMustExist, XtRBoolean, sizeof(Boolean),
        offset(filedlg.mustExist), XtRImmediate, (XtPointer) True},
    {XtNwillRead, XtCWillRead, XtRBoolean, sizeof(Boolean),
        offset(filedlg.willRead), XtRImmediate, (XtPointer) True},
    {XtNwillWrite, XtCWillWrite, XtRBoolean, sizeof(Boolean),
        offset(filedlg.willWrite), XtRImmediate, (XtPointer) False},
    {XtNwillExecute, XtCWillExecute, XtRBoolean, sizeof(Boolean),
        offset(filedlg.willExecute), XtRImmediate, (XtPointer) False},
    {XtNconfirm, XtCConfirm, XtRBoolean, sizeof(Boolean),
        offset(filedlg.confirm), XtRImmediate, (XtPointer) False},
    {XtNsorted, XtCSorted, XtRBoolean, sizeof(Boolean),
        offset(filedlg.sorted), XtRImmediate, (XtPointer) True},
    {XtNdirsFirst, XtCDirsFirst, XtRBoolean, sizeof(Boolean),
        offset(filedlg.dirsFirst), XtRImmediate, (XtPointer) True},
    {XtNfilesFirst, XtCFilesFirst, XtRBoolean, sizeof(Boolean),
        offset(filedlg.filesFirst), XtRImmediate, (XtPointer) False},
    {XtNshowDots, XtCShowDots, XtRBoolean, sizeof(Boolean),
        offset(filedlg.showDots), XtRImmediate, (XtPointer) False},
    {XtNfileCheck, XtCCallback, XtRCallback, sizeof(XtPointer),
        offset(filedlg.fileCheck), XtRCallback, NULL},
    {XtNselectProc, XtCCallback, XtRCallback, sizeof(XtPointer),
        offset(filedlg.selectProc), XtRCallback, NULL},
    {XtNcancelProc, XtCCallback, XtRCallback, sizeof(XtPointer),
        offset(filedlg.cancelProc), XtRCallback, NULL}
};
#undef offset

#define  RE_MATCH    1
#define  RE_NOMATCH  0
#define  RE_ERROR   -1

#define  UNSORTED    0
#define  SORT_MIXED  1
#define  SORT_DF     2
#define  SORT_FD     3

#ifdef SYSV
static char *re_comp(/* char * */);  /* returns NULL if OK */
static int re_exec(/* char * */);    /* returns RE_... */

#define getwd(dir) getcwd(dir,MAXPATHLEN)

#else
extern char *re_comp(/* char * */);  /* returns NULL if OK */
extern int re_exec(/* char * */);    /* returns RE_... */
#endif

static Cursor waitCursor;

char *ActualPath( dirOut, dirIn )
char *dirOut, *dirIn;
{
    struct passwd *pwd;  char *s, *d, *e, *t, dirTmp[MAXPATHLEN];

    if (dirIn && *dirIn)  {
        s = dirIn;  d = dirTmp;
        while (*s)  {
            if (*s == '$' && isalpha(*(s+1)))  {
                t = d;  s++;
                while (*s && isalnum(*s))  *(t++) = *(s++);
                *(t++) = '\0';
                if (e = getenv(d))  {
                    strcpy( d, e );  d += strlen(e);
                }  else
                    return NULL;
            }  else  {
                if (*s != '$')  *(d++) = *s;
                s++;
            }
        }
        *d = '\0';

        s = dirTmp;  d = dirOut;
        if (*s == '~')  {
            s++;
            if (!*s || *s == '/')  {
                if (pwd = getpwuid(getuid())) {
                    strcpy( dirOut, pwd->pw_dir );  d += strlen(dirOut);
                    if (*s != '/')  *(d++) = '/';
                }  else
                    return NULL;
            }  else  {
                while (*s && *s != '/') *(d++) = *(s++);
                *d = '\0';  d = dirOut;
                if (pwd = getpwnam(d))  {
                    strcpy( dirOut, pwd->pw_dir );  d += strlen(dirOut);
                    if (*s != '/')  *(d++) = '/';
                }  else
                    return NULL;
            }
        } else
        if (!*s || *s != '/')  {  /* start with working directory */
            getwd(dirOut);  d += strlen(dirOut);
            if (*s == '.' && (!*(s+1) || *(s+1) == '/'))  s++;
            if (*s != '/')  *(d++) = '/';
        }
        *d = '\0';

        while (*s)  {
            switch (*s)  {
                case '/':  
                    if (d == dirOut || *(d-1) != '/') *(d++) = *s;
                    s++;
                    break;
                case '.':
                    if (*(s+1) == '.' && (!*(s+2) || *(s+2) == '/'))  {
                        if (d != dirOut)  d--;
                        if (d != dirOut && *d == '/')  d--;
                        while (d != dirOut && *d != '/')  d--;
                        s += 2;
                    }  else
                    if (!*(s+1) || *(s+1) == '/')  s++;
                    else
                        while (*s && *s != '/')  *(d++) = *(s++);
                    break;
                default:
                    while (*s && *s != '/')  *(d++) = *(s++);
                    break;
            }
        }
        *d = '\0';
    }  else  {
        getwd(dirOut);  strcat( dirOut, "/" );
    }

    return dirOut;
}

static Boolean SplitActualPath( path, dir, file )
char *path, *dir, *file;
{
    char *p, *f;  struct stat st;  Boolean isDir, isSpec;

    p = path + strlen(path) - 1;

    isDir = (stat( path, &st ) == 0 && S_ISDIR(st.st_mode));
    if (!isDir)  {
        f = p;
        while (f != path && *f != '/')  f--;
        if (*f == '/')  f++;
    }  else
        f = p+1;

    p = path;
    while (*p && p != f)  *(dir++) = *(p++);
    if (p != path && *(p-1) != '/')  *(dir++) = '/';
    *dir = '\0';

    isSpec = !*p;  /* is a file spec if empty */
    while (*p)  {
        isSpec = isSpec || (*p == '*' || *p == '?');
        *(file++) = *(p++);
    }
    *file = '\0';

    return isSpec;
}

static char *FilePattern( fileSpec )
char *fileSpec;
{
    char *pattern, *d, *s;

    pattern = (String) XtMalloc( 2 * (strlen(fileSpec)+3) );

    s = fileSpec;  d = pattern;  *(d++) = '^';
    while (*s)  {   
        switch (*s)  {   
            case '.':
                *(d++) = '\\';  *(d++) = '.';
                break;
            case '?':
                *(d++) = '.';
                break;
            case '*':
                *(d++) = '.';  *(d++) = '*';
                break;
            default:
                *(d++) = *s;
                break;
        } 
        s++;
    } 
    *(d++) = '$';  *d = '\0';

    return(pattern);
}

static void FreePathList( dflist )
pDF_LIST dflist;
{
    int i;

    if (dflist->access)  XtFree(dflist->access);

    for (i = dflist->num-1;  i >= 0;  i--)  XtFree(dflist->name[i]);
    if (dflist->num)  XtFree(dflist->name);

    if (dflist->path)  XtFree(dflist->path);
    if (dflist->spec)  XtFree(dflist->spec);

    dflist->num    = dflist->nd = dflist->nf = 0;
    dflist->path   = NULL;
    dflist->spec   = NULL;
    dflist->name   = NULL;
    dflist->access = NULL;
}

#ifndef TOUPPER
#define TOUPPER(c)  (isalpha(c) ? ((c) & 0x5f) : (c))
#endif

static int DirsFirst( a, b )
char *a[], *b[];
{
    char *ac, *bc, *la, *lb;

    ac = *a;  bc = *b;
    /*
       put directories first:  these end with '/'
    */
    la = ac + strlen(ac) - 1;  lb = bc + strlen(bc) - 1;
    if (*la == '/' && *lb != '/')  return -1;
    else
    if (*la != '/' && *lb == '/')  return 1;
    else  {
        while (*ac && *bc && TOUPPER(*ac) == TOUPPER(*bc))  { ac++;  bc++; }

        return (TOUPPER(*ac) - TOUPPER(*bc));
    }
}


static int FilesFirst( a, b )
char *a[], *b[];
{
    char *ac, *bc, *la, *lb;

    ac = *a;  bc = *b;
    /*
       put directories last:  these end with '/'
    */
    la = ac + strlen(ac) - 1;  lb = bc + strlen(bc) - 1;
    if (*la == '/' && *lb != '/')  return 1;
    else
    if (*la != '/' && *lb == '/')  return -1;
    else  {
        while (*ac && *bc && TOUPPER(*ac) == TOUPPER(*bc))  { ac++;  bc++; }

        return (TOUPPER(*ac) - TOUPPER(*bc));
    }
}


static int Mixed( a, b )
char *a[], *b[];
{
    char *ac, *bc;

    ac = *a;  bc = *b;
    while (*ac && *bc && TOUPPER(*ac) == TOUPPER(*bc))  { ac++;  bc++; }

    return (TOUPPER(*ac) - TOUPPER(*bc));
}


static Boolean Access( pathSpec, st, user, group, R,W,X )
String pathSpec;
struct stat st;  uid_t user;  gid_t group;
Boolean R,W,X;
{
    Boolean isOwner, isInGroup, r,w,x;

    isOwner   = (user  == st.st_uid);
    isInGroup = (group == st.st_gid);
    r = (((st.st_mode & S_IRUSR) && isOwner)   ||
         ((st.st_mode & S_IRGRP) && isInGroup) ||
         (st.st_mode & S_IROTH));
    w = (((st.st_mode & S_IWUSR) && isOwner)   ||
         ((st.st_mode & S_IWGRP) && isInGroup) ||
         (st.st_mode & S_IWOTH));
    x = (((st.st_mode & S_IXUSR) && isOwner)   ||
         ((st.st_mode & S_IXGRP) && isInGroup) ||
         (st.st_mode & S_IXOTH));
    /*
      At least one mode of access must be intended.
       (Note: (!a || b) is (a => b)).
    */
    return (R || W || X) && (!R || r) && (!W || w) && (!X || x);
}


static Boolean MakePathList( pathReq, dflist, R,W,X, sortMode, showDots )
String pathReq;
pDF_LIST dflist;
Boolean R,W,X, showDots;
Cardinal sortMode;
{
    DIR *dir;  struct dirent *de;  struct stat st;
    uid_t user;  gid_t group;
    String pathSpec, dirSpec, fileSpec, psEnd, filePatt, parent;
    int i, n, nd, nf, len;
    int (*fncmp)();

    pathSpec = (String) XtMalloc( MAXPATHLEN );
    if (!ActualPath( pathSpec, pathReq ))  {
        XtFree(pathSpec);  return False;
    }

    dirSpec = (String) XtMalloc( MAXPATHLEN );
    fileSpec = (String) XtMalloc( MAXPATHLEN );
    SplitActualPath( pathSpec, dirSpec, fileSpec );

    if (strlen(fileSpec) == 0)  strcpy( fileSpec, "*" );
    filePatt = FilePattern( fileSpec );

    if (re_comp(filePatt))  {
        XtFree(pathSpec);  XtFree(dirSpec);  XtFree(fileSpec);
        XtFree(filePatt);
        return False;
    }

    strcpy( pathSpec, dirSpec );
    psEnd = pathSpec + strlen(pathSpec);
    /*
        Pass 1:  just count subdirectories & regular files
    */
    if (!(dir = opendir(dirSpec)))  {
        XtFree(pathSpec);  XtFree(dirSpec);  XtFree(fileSpec);  return False;
    }

    nd = nf = 0;
    while (de = readdir(dir))  {
        *psEnd = '\0';  strcat( pathSpec, de->d_name );
        if (stat( pathSpec, &st ) == 0 && S_ISDIR(st.st_mode))
            nd++;
        else
            nf++;
    }
    if (nd < 2)  {  /* must be at least 2: . and .. */
        XtFree(pathSpec);  XtFree(dirSpec);  XtFree(fileSpec);  return False;
    }
    n = nd + nf;
    /*
        Between passes: clear old list, allocate new one
    */
    FreePathList( dflist );

    dflist->path = (String) XtMalloc( strlen(dirSpec)+1 );
    strcpy( dflist->path, dirSpec );
    dflist->spec = (String) XtMalloc( strlen(fileSpec)+1 );
    strcpy( dflist->spec, fileSpec );
    dflist->name = (String *) XtMalloc( n * sizeof (String) );
    dflist->access = (Boolean *) XtMalloc( n * sizeof(Boolean) );

    /*
        Pass 2: Selectively build list

        Notes: - pattern matching is not applied to subdirs
               - directory ".." is placed in first entry, is not sorted
               - directory "." is not put in the list
    */
    dflist->name[0] = (String) XtMalloc(4);
    strcpy( dflist->name[0], "../" );
    dflist->len = 3;

    rewinddir(dir);
    n = nd = 1;  nf = 0;
    while (de = readdir(dir))  {
        len = strlen(de->d_name);
        *psEnd = '\0';  strcat( pathSpec, de->d_name );
        if (stat( pathSpec, &st ) == 0 && S_ISDIR(st.st_mode))  {
            if (streq( de->d_name, ".."))  {
                /* do nothing - it's in [0] already */
            }  else
            if (!streq( de->d_name, "." ))  {
                dflist->name[n] = (String) XtMalloc(strlen(de->d_name)+2);
                strcpy( dflist->name[n], de->d_name );
                strcat( dflist->name[n], "/" );
                n++;  nd++;  len++;
                if (len > dflist->len)  dflist->len = len;
            }
        }  else  {
            if (re_exec(de->d_name) == RE_MATCH &&
                (*(de->d_name) != '.' || *(de->d_name) == *fileSpec ||
                 showDots))  {
                dflist->name[n] = (String) XtMalloc(strlen(de->d_name)+1);
                strcpy( dflist->name[n], de->d_name );
                n++;  nf++;
                if (len > dflist->len)  dflist->len = len;
            }
        }
    }
    closedir(dir);

    XtFree(dirSpec);  XtFree(fileSpec);  XtFree(filePatt);

    /*
        Sort from second entry; leave ".." alone in first entry
    */
    dflist->num = n;  dflist->nd = nd;  dflist->nf = nf;
    if (dflist->num > 1)  {
        if (sortMode != UNSORTED)  {
            switch (sortMode)  {
	        case SORT_MIXED:
                     fncmp = Mixed;  break;
	        case SORT_DF:
                     fncmp = DirsFirst;  break;
                case SORT_FD:
                     fncmp = FilesFirst;  break;
            } 
            qsort( &(dflist->name[1]), dflist->num-1, sizeof (String), fncmp );
        }
    }

    /*
       Pass 3:  Determine access privileges.
                This requires a separate pass because the name list must
                be a list of strings for sorting, and moreover may be
                rearranged by sorting.
    */
    user = getuid();  group = getgid();

    if (n)  {
        for (i = 0;  i < n;  i++)  {
            *psEnd = '\0';  strcat( pathSpec, dflist->name[i] );

            if (stat( pathSpec, &st ) != 0)
                dflist->access[i] = False;
            else
            if (S_ISDIR(st.st_mode))  {
                /*
                   We always want execute (search) access for directories
                */
                if (streq( dflist->name[i], "../" ))  {
                    parent = (String) XtMalloc( MAXPATHLEN );
                    if (!ActualPath( parent, pathSpec ))
                        dflist->access[i] = False;
                    else
                        dflist->access[i] = Access( parent, 
                                                    st, user,group,
                                                    R,W, True );
                    XtFree(parent);
                }  else  {
                    dflist->access[i] = Access( pathSpec,
                                                st,  user,group,
                                                R,W, True );
                }
            }  else  {
                dflist->access[i] = Access( pathSpec,
                                            st, user,group, R, W, X );
            }
        }
    }
    XtFree(pathSpec);

    return True;
}


static void SetInputText( fdw )
FileDlgWidget fdw;
{
    FileDlgPart *fdp;
    char pathReq[MAXPATHLEN];

    fdp = &(fdw->filedlg);

    strcpy( pathReq, fdp->path );
    if (strlen(fdp->name) == 0)
        strcat( pathReq, fdp->spec );
    else
        strcat( pathReq, fdp->name );

    XtVaSetValues( fdp->input, XtNstring, pathReq, NULL );

    XawTextSetSelection( fdp->input, strlen(fdp->path), strlen(pathReq) );
    XawTextSetInsertionPoint( fdp->input, strlen(pathReq) );
}


void SetPosition( fdw )
FileDlgWidget fdw;
/*
    Place the dialog box  -  here, centered over the parent.
*/  
{
    FileDlgPart *fdp;
    Widget parent;
    Display *display;  Screen *screen;
    int x,y, px,py, sw,sh, w,h, bw;  Dimension pw,ph;

    fdp = &(fdw->filedlg);
/*
    display = XtDisplay(fdw);
    XQueryPointer( display, DefaultRootWindow(display), &root, &child,
                   &x,&y, &dx,&dy, &b );
*/
    w = fdw->core.width;
    h = fdw->core.height;
    bw = fdw->core.border_width;

    parent = XtParent(fdw);
    px = parent->core.x;
    py = parent->core.y;
    pw = parent->core.width;
    ph = parent->core.height;

    screen = XtScreen(fdw);
    sw = WidthOfScreen(screen) - w - 1;
    sh = HeightOfScreen(screen) - h - 1;

    x = px + (((int) pw) - (w + bw+bw)) / 2;  /* center over parent */
    y = py + (((int) ph) - (h + bw+bw)) / 2;

    if (x < 1)  x = 1;     /* keep it in screen */
    else
    if (x+bw+bw >= sw)  x = sw-bw-bw-1;
    if (y < 1)  y = 1;
    else
    if (y+bw+bw >= sh)  y = sh-bw-bw-1;

    XtVaSetValues( fdw,  XtNx, x,  XtNy, y,  NULL );
}


static void DoFileSearch( fdw )
FileDlgWidget fdw;
{
    FileDlgPart *fdp;
    Boolean Ok;  Cardinal sortMode;
    char pathSpec[MAXPATHLEN], *searchSpec, listlbl[MAXPATHLEN];
    Widget scroll;
    XFontStruct *font;  XCharStruct *max;
    Dimension cw,ch, cs,rs, iw,ih, h, vpw, sbw, w, mw;
    
    fdp = &(fdw->filedlg);

    GroupDoLayout( fdp->dlgbox, False );

    StartWaiting( (WaitWidget) fdp->wait );

    strcpy( pathSpec, fdp->path );  strcat( pathSpec, fdp->spec );

    sortMode = (!fdp->sorted ? UNSORTED :
                (fdp->dirsFirst == fdp->filesFirst ? SORT_MIXED :
                 (fdp->dirsFirst ? SORT_DF : SORT_FD)));

    Ok = MakePathList( pathSpec, &(fdp->dflist),
                        fdp->willRead, fdp->willWrite, fdp->willExecute,
                        sortMode, fdp->showDots );
    if (!Ok)  BEEP(fdw);

    strcpy( fdp->path, fdp->dflist.path );
    strcpy( fdp->spec, fdp->dflist.spec );
    SetInputText(fdw);

    sprintf( listlbl, "Files matching: %s", fdp->spec );
    XtVaSetValues( fdp->listlbl, XtNlabel, listlbl, NULL );

    SensListChange( fdp->list,
                    fdp->dflist.name, fdp->dflist.access,
                    fdp->dflist.num, 0, True );

    XtVaGetValues( fdp->list,
        XtNinternalWidth, &iw,
        XtNinternalHeight, &ih,
        XtNcolumnSpacing, &cs,
        XtNrowSpacing, &rs,
        NULL );
    scroll = XtNameToWidget( fdp->vp, "vertical" );
    XtVaGetValues( scroll, XtNthickness, &sbw, NULL );

    font = fdp->dataFont;  max = &(font->max_bounds);
    cw = max->width;  ch = max->ascent + max->descent;
    h = (fdp->lines * (ch+rs)) + ih - rs;
    vpw = (fdp->dflist.len * cw) + iw + cs + sbw;

    XtVaSetValues( fdp->vp,
        XtNwidth, vpw,  XtNprefWidth, vpw,
        XtNheight, h,   XtNminHeight, h,
        NULL );

    XawScrollbarSetThumb( scroll, 0.0, -1.0 );  /* set to top */
    XtMoveWidget( fdp->list, 0,0 );

    GroupDoLayout( fdp->dlgbox, True );

    EndWaiting( (WaitWidget) fdp->wait );
}

static void Reset( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    FileDlgWidget fdw = (FileDlgWidget) appData;
    FileDlgPart *fdp;
    char pathName[MAXPATHLEN];

    if (!(fdw->shell.popped_up))  return;

    fdp = &(fdw->filedlg);

    strcpy( fdp->path, fdp->defaultPath );
    strcpy( fdp->spec, fdp->defaultSpec );
    strcpy( fdp->name, fdp->defaultName );

    SetInputText(fdw);  DoFileSearch( fdw );
}

static void Select( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    FileDlgWidget fdw = (FileDlgWidget) appData;
    FileDlgPart *fdp;
    XawListReturnStruct *item = (XawListReturnStruct *) wData;
    char pathReq[MAXPATHLEN], pathSpec[MAXPATHLEN], *path, *name;

    if (!(fdw->shell.popped_up))  return;

    fdp = &(fdw->filedlg);
    strcpy( pathReq, fdp->dflist.path );
    strcat( pathReq, item->string );
    /*
       if directory selected, append the current (name or) spec
    */
    if (item->string[strlen(item->string)-1] == '/')  {
        ActualPath( pathSpec, pathReq );
        strcpy( fdp->name, "" );     /* force spec */
        if (strlen(fdp->name) > 0)
            strcat( pathSpec, fdp->name );
        else
            strcat( pathSpec, fdp->spec );
        XtVaSetValues( fdp->input, XtNstring, pathSpec, NULL );

        path = (String) XtMalloc( MAXPATHLEN );
        name = (String) XtMalloc( MAXPATHLEN );
        SplitActualPath( pathSpec, path, name );

        XawTextSetSelection( fdp->input, strlen(path), strlen(pathSpec) );
        XawTextSetInsertionPoint( fdp->input, strlen(pathSpec) );

        XtFree(path);  XtFree(name);
    }  else  {
        XtVaSetValues( fdp->input, XtNstring, pathReq, NULL );

        XawTextSetSelection( fdp->input, strlen(fdp->path), strlen(pathReq) );
        XawTextSetInsertionPoint( fdp->input, strlen(pathReq) );
    }
}

static void Open( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    FileDlgWidget fdw = (FileDlgWidget) appData;
    FileDlgPart *fdp;
    String selection, pathName, path, name, pattern, fmt, msg;
    FILECHK_PRM parm;  struct stat st;  Boolean isSpec, exists, yes;
    uid_t user;  gid_t group;

    if (!(fdw->shell.popped_up))  return;
    
    fdp = &(fdw->filedlg);

    XtVaGetValues( fdp->input, XtNstring, &selection, NULL );

    pathName = (String) XtMalloc( MAXPATHLEN );
    if (!ActualPath( pathName, selection ))  {
        OkMessage( (Widget) fdw, "Input doesn't have a valid path." );
        XtFree(pathName);
        return;
    }
    path = (String) XtMalloc( MAXPATHLEN );
    name = (String) XtMalloc( MAXPATHLEN );
    isSpec = SplitActualPath( pathName, path, name );
    strcpy( fdp->path, path );

    if (isSpec || !streq( fdp->path, fdp->dflist.path ))  {
        if (isSpec)
            strcpy( (strlen(name) ? fdp->spec : fdp->name), name );

        XtFree(pathName);  XtFree(path);  XtFree(name);
        DoFileSearch(fdw);
        return;
    }

    if (fdp->mustMatch)  {
        pattern = FilePattern( fdp->defaultSpec );
        if (re_comp(pattern))  {
            BEEP(w);
            XtFree(pathName);  XtFree(path);  XtFree(name);
            XtFree(pattern);
            return;
        }
        if (re_exec(name) != RE_MATCH)  {
            fmt = "File doesn't match '%s' - continue?";
            msg = (String) XtMalloc( strlen(fmt)+strlen(fdp->defaultSpec) );
            sprintf( msg, fmt, fdp->defaultSpec );
            if (!YesNoMessage( (Widget) fdw, msg, False ))  {
                XtFree(msg);
                XtFree(pathName);  XtFree(path);  XtFree(name);
                XtFree(pattern);
                return;
            }
            XtFree(msg);
        }
        XtFree(pattern);
    }

    exists = (stat( pathName, &st ) == 0);

    if (fdp->mustExist && !exists)  {
        BEEP(w);
        XtFree(pathName);  XtFree(path);  XtFree(name);
        return;
    }
    
    /*
        Basically for file creation, make sure the user has access to the
        selected directory.
    */
    user = getuid();  group = getgid();
    if (stat( path, &st ) != 0 ||
        !Access( path, st, user, group,
                 fdp->willRead, fdp->willWrite, fdp->willExecute ))  {
        BEEP(w);
        XtFree(pathName);  XtFree(path);  XtFree(name);
        return;
    }

    if (fdp->willWrite && fdp->confirm && exists)  {
        yes = YesNoMessage( (Widget) fdw,
                            "File exists - overwrite it?", True );
        if (!yes)  return;
    }

    if (fdp->fileCheck)  {
        parm.name = pathName;
        parm.exists = exists;
        parm.ok = True;
        XtCallCallbacks( fdw, XtNfileCheck, (XtPointer) &parm );
        if (!parm.ok)  {
            BEEP(w);
            XtFree(pathName);  XtFree(path);  XtFree(name);
            return;
        }
    }
/*
    here only if file name has passed all tests, so popdown, save
    the checked file name, and call user's open routine
*/
    XtPopdown(fdw);

    XtVaSetValues( fdp->input, XtNstring, pathName, NULL );
    XawTextSetSelection( fdp->input, strlen(fdp->path), strlen(pathName) );
    XawTextSetInsertionPoint( fdp->input, strlen(pathName) );
    XtFree(pathName);
    strcpy( fdp->path, path );
    strcpy( fdp->name, name );

    if (fdp->selectProc)  {
        XtVaGetValues( fdp->input, XtNstring, &selection, NULL );
        XtCallCallbacks( fdw, XtNselectProc, (XtPointer) selection );
    }
}
 
static void Cancel( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    FileDlgWidget fdw = (FileDlgWidget) appData;
    FileDlgPart *fdp;

    fdp = &(fdw->filedlg);  strcpy( fdp->name, "" );

    XtPopdown(fdw);

    if (fdp->cancelProc)
        XtCallCallbacks( fdw, XtNcancelProc, NULL );
}   


/* ARGSUSED */
static void Initialize( req, new, arg, nargs )
Widget req, new;
ArgList arg;
Cardinal *nargs;
{
    FileDlgWidget fdw = (FileDlgWidget) new;
    FileDlgPart *fdp;
    char pathSpec[MAXPATHLEN], *path, *spec, *name;
    int i;   
    XFontStruct *font;  XCharStruct *max;
    Dimension cw,ch, h, vpw;

    fdp = &(fdw->filedlg);

    path = fdp->path;  spec = fdp->spec;  name = fdp->name;
    fdp->path = (String) XtMalloc( MAXPATHLEN );
    fdp->spec = (String) XtMalloc( MAXPATHLEN );
    fdp->name = (String) XtMalloc( MAXPATHLEN );
    strcpy( fdp->path, (path ? path : ".") );
    strcpy( fdp->spec, (spec ? spec : "*") );
    strcpy( fdp->name, (name ? name : "") );

    fdp->defaultPath = (String) XtMalloc( MAXPATHLEN );
    fdp->defaultSpec = (String) XtMalloc( MAXPATHLEN );
    fdp->defaultName = (String) XtMalloc( MAXPATHLEN );
    strcpy( fdp->defaultPath, fdp->path );
    strcat( fdp->defaultPath, "/" );
    strcat( fdp->defaultPath, fdp->spec );
    if (ActualPath( pathSpec, fdp->defaultPath ))  {
        SplitActualPath( pathSpec, fdp->path, fdp->spec );
    }  else  {
        strcpy( fdp->path, "./" );  strcpy( fdp->spec, "*" );
    }
    strcpy( fdp->defaultPath, fdp->path );
    strcpy( fdp->defaultSpec, fdp->spec );
    strcpy( fdp->defaultName, fdp->name );

    fdp->dflist.num    = fdp->dflist.nd = fdp->dflist.nf = 0;
    fdp->dflist.len    = 0;
    fdp->dflist.path   = NULL;
    fdp->dflist.spec   = NULL;
    fdp->dflist.name   = NULL;
    fdp->dflist.access = NULL;

    font = fdp->dataFont;  max = &(font->max_bounds);
    cw = max->width;  ch = max->ascent + max->descent;
    h = (fdp->lines * (ch+4)) + 2 - 4 - 11;
    vpw = (fdp->dflist.len * cw) + 10 + 14;

    fdp->dlgbox = XtVaCreateManagedWidget( "dlgbox",
        groupWidgetClass, fdw,
        XtNspacing, fdp->spacing,
        XtNorientation, XtorientVertical,
        NULL );

    fdp->dlgname = XtVaCreateManagedWidget( "dlgname",
        labelWidgetClass, fdp->dlgbox,
        XtNlabel, fdp->title,
        XtNfont, fdp->labelFont,
        XtNborderWidth, 0,
        XtNmargin, fdp->spacing,
        NULL );

    fdp->textbox = XtVaCreateManagedWidget( "textbox",
        groupWidgetClass, fdp->dlgbox,
        XtNspacing, 0,
        XtNorientation, XtorientVertical,
        XtNalignment, gaFill,
        XtNmargin, 0,
        XtNpreference, True,
        NULL );
    fdp->iprompt = XtVaCreateManagedWidget( "iprompt",
        labelWidgetClass, fdp->textbox,
        XtNlabel, fdp->prompt,
        XtNfont, fdp->labelFont,
        XtNborderWidth, 0,
        XtNmargin, fdp->spacing,
        NULL );
    fdp->input = XtVaCreateManagedWidget( "input",
        icaTextWidgetClass, fdp->textbox,
        XtNfont, fdp->dataFont,
        XtNeditType, XawtextEdit,
        XtNlength, MAXPATHLEN,
        XtNalignment, gaFill,
        XtNminHeight, ch,
        XtNmargin, fdp->spacing,
        NULL );

    fdp->listbox = XtVaCreateManagedWidget( "listbox",
        groupWidgetClass, fdp->dlgbox,
        XtNspacing, 0,
        XtNorientation, XtorientVertical,
        XtNalignment, gaFill,
        XtNmargin, 0,
        NULL );

    fdp->listlbl = XtVaCreateManagedWidget( "listlbl",
        labelWidgetClass, fdp->listbox,
        XtNlabel, fdp->title,
        XtNfont, fdp->labelFont,
        XtNborderWidth, 0,
        XtNmargin, fdp->spacing,
        NULL );
    fdp->vp = XtVaCreateManagedWidget( "vp",
        viewportWidgetClass, fdp->listbox,
        XtNforceBars, True,
        XtNallowVert, True,
        XtNpreference, False,
        XtNwidth, vpw, XtNminWidth, vpw,
        XtNheight, h, XtNminHeight, h,
        XtNuseRight, False,
        XtNverticalList, True,
        XtNalignment, gaFill,
        XtNmargin, fdp->spacing,
        NULL );
    fdp->list = XtVaCreateManagedWidget( "list",
        sensListWidgetClass, fdp->vp,
        XtNfont, fdp->dataFont,
        XtNdefaultColumns, 1,
        XtNforceColumns, True,
        XtNlist, NULL,
        XtNnumberStrings, 0,
        NULL );

    fdp->btnbox = XtVaCreateManagedWidget( "btnbox",
        groupWidgetClass, fdp->dlgbox,
        XtNspacing, fdp->spacing,
        XtNorientation, XtorientHorizontal,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNpreference, True,
        NULL );

    fdp->openbtn = XtVaCreateManagedWidget( "openbtn",
        commandWidgetClass, fdp->btnbox,
        XtNlabel, fdp->button,
        XtNfont, fdp->btnFont,
        XtNcursor, fdp->btnCursor,
        XtNborderWidth, 2,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        NULL );
    fdp->resetbtn = XtVaCreateManagedWidget( "resetbtn",
        commandWidgetClass, fdp->btnbox,
        XtNlabel, "Reset",
        XtNfont, fdp->btnFont,
        XtNcursor, fdp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        NULL );
    fdp->cancelbtn = XtVaCreateManagedWidget( "cancelbtn",
        commandWidgetClass, fdp->btnbox,
        XtNlabel, "Cancel",
        XtNfont, fdp->btnFont,
        XtNcursor, fdp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        NULL );

    if (!fdp->title)  XtUnmanageChild(fdp->dlgname);
    if (!fdp->prompt) XtUnmanageChild(fdp->iprompt);

    fdp->wait = XtVaCreateWidget( "wait",
        waitWidgetClass, fdw,
        NULL );

    SetInputText(fdw);

    XtAddCallback( fdp->input, XtNacceptProc, Open, fdw );
    XtAddCallback( fdp->list, XtNcallback, Select, fdw );
    XtAddCallback( fdp->list, XtNacceptProc, Open, fdw );
    XtAddCallback( fdp->openbtn, XtNcallback, Open, fdw );
    XtAddCallback( fdp->resetbtn, XtNcallback, Reset, fdw );
    XtAddCallback( fdp->cancelbtn, XtNcallback, Cancel, fdw );
}

static void Realize( w, valueMask, attributes )
Widget w;
XtValueMask *valueMask;
XSetWindowAttributes *attributes;
/*
    This is being used here as a point at which to place the dialog box - 
    after the first call to change_managed, but before mapping
*/
{
    FileDlgWidget fdw = (FileDlgWidget) w;
    FileDlgPart *fdp;
    WidgetClass superclass = w->core.widget_class->core_class.superclass;

    fdp = &(fdw->filedlg);

    SetPosition(fdw);

    (*superclass->core_class.realize)( w, valueMask, attributes );

    XSetTransientForHint( XtDisplay(w), XtWindow(w), XtWindow(XtParent(w)) );

    XtSetKeyboardFocus( fdw, fdp->input );
}


static void Destroy( w )
Widget w;
{
    FileDlgWidget fdw = (FileDlgWidget) w;
    FileDlgPart *fdp;

    fdp = &(fdw->filedlg);

    if (fdp->defaultName)  XtFree(fdp->defaultName);
    if (fdp->defaultSpec)  XtFree(fdp->defaultSpec);
    if (fdp->defaultPath)  XtFree(fdp->defaultPath);
    if (fdp->name)  XtFree(fdp->name);
    if (fdp->spec)  XtFree(fdp->spec);
    if (fdp->path)  XtFree(fdp->path);

    FreePathList( fdp->dflist );
}


/* ARGSUSED */
static Boolean SetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    FileDlgWidget fdw = (FileDlgWidget) new;
    FileDlgWidget fdOld = (FileDlgWidget) old;
    FileDlgPart *fdp, *fpOld;
    char pathSpec[MAXPATHLEN], *argname, *path, *spec, *name;
    int i;
    Boolean chgdDataFont, chgdLabelFont, chgdBtnFont, chgdBtnCursor,
            chgdTitle, chgdPrompt, chgdButton,
            chgdPath, chgdSpec, chgdName,
            chgdSpacing;

    fdp = &(fdw->filedlg);  fpOld = &(fdOld->filedlg);

    chgdDataFont =  chgdLabelFont = chgdBtnFont = chgdBtnCursor = False;
    chgdTitle = chgdPrompt = chgdButton = False;
    chgdPath = chgdSpec = chgdName = chgdSpacing = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
        if (streq( argname, XtNdataFont ))   chgdDataFont = True;
        if (streq( argname, XtNlabelFont ))  chgdLabelFont = True;
        if (streq( argname, XtNbtnFont ))    chgdBtnFont = True;
        if (streq( argname, XtNbtnCursor ))  chgdBtnCursor = True;
        if (streq( argname, XtNtitle ))      chgdTitle = True;
        if (streq( argname, XtNprompt ))     chgdPrompt = True;
        if (streq( argname, XtNbutton ))     chgdButton = True;
        if (streq( argname, XtNpath ))       chgdPath = True;
        if (streq( argname, XtNspec ))       chgdSpec = True;
        if (streq( argname, XtNname ))       chgdName = True;
        if (streq( argname, XtNspacing ))    chgdSpacing = True;
    }

    if (chgdDataFont)  {
        XtVaSetValues( fdp->input, XtNfont, fdp->dataFont, NULL );
        XtVaSetValues( fdp->list,  XtNfont, fdp->dataFont, NULL );
    }

    if (chgdLabelFont)  {
        XtVaSetValues( fdp->iprompt, XtNfont, fdp->labelFont, NULL );
        XtVaSetValues( fdp->dlgname, XtNfont, fdp->labelFont, NULL );
        XtVaSetValues( fdp->listlbl, XtNfont, fdp->labelFont, NULL );
    }

    if (chgdBtnFont)  {
        XtVaSetValues( fdp->openbtn,    XtNfont, fdp->btnFont, NULL );
        XtVaSetValues( fdp->resetbtn,   XtNfont, fdp->btnFont, NULL );
        XtVaSetValues( fdp->cancelbtn,  XtNfont, fdp->btnFont, NULL );
    }

    if (chgdBtnCursor)  {
        XtVaSetValues( fdp->openbtn,    XtNcursor, fdp->btnCursor, NULL );
        XtVaSetValues( fdp->resetbtn,   XtNcursor, fdp->btnCursor, NULL );
        XtVaSetValues( fdp->cancelbtn,  XtNcursor, fdp->btnCursor, NULL );
    }

    if (chgdTitle)  {
        XtVaSetValues( fdp->dlgname, XtNlabel, fdp->title, NULL );
        if (fdp->title && !XtIsManaged(fdp->dlgname))
            XtManageChild(fdp->dlgname);
        else
        if (!fdp->title && XtIsManaged(fdp->dlgname))
            XtUnmanageChild(fdp->dlgname);
    }

    if (chgdPrompt)  {
        XtVaSetValues( fdp->iprompt, XtNlabel, fdp->prompt, NULL );
        if (fdp->prompt && !XtIsManaged(fdp->iprompt))
            XtManageChild(fdp->iprompt);
        else
        if (!fdp->prompt && XtIsManaged(fdp->iprompt))
            XtUnmanageChild(fdp->iprompt);
    }

    if (chgdButton)  {
        XtVaSetValues( fdp->openbtn, XtNlabel, fdp->button, NULL );
    }

    if (chgdPath || chgdSpec)  {
        if (chgdPath)  {
            path = fdp->path;  fdp->path = fpOld->path;
            strcpy( fdp->path, (path ? path : "./") );
        }
        if (chgdSpec)  {
            spec = fdp->spec;  fdp->spec = fpOld->spec;
            strcpy( fdp->spec, (spec ? spec : "*") );
        }

        path = (String) XtMalloc( MAXPATHLEN );
        strcpy( path, fdp->path );
        strcat( path, "/" );
        strcat( path, fdp->spec );
        if (ActualPath( pathSpec, path ))  {
            SplitActualPath( pathSpec, fdp->path, fdp->spec );
        }  else  {
            strcpy( fdp->path, "./" );  strcpy( fdp->spec, "*" );
        }
        XtFree(path);

        if (chgdPath)  strcpy( fdp->defaultPath, fdp->path );
        if (chgdSpec)  strcpy( fdp->defaultSpec, fdp->spec );

        SetInputText(fdw);
    }

    if (chgdName)  {
        name = fdp->name;  fdp->name = fpOld->name;
        strcpy( fdp->name, (name ? name : "") );
        strcpy( fdp->defaultName, fdp->name );

        SetInputText(fdw);
    }
/*
    if (chgdSpacing)  {
        XtVaSetValues( fdp->dlgbox, XtNspacing, fdp->spacing, NULL );
    }
*/
    if (!(fdp->willRead || fdp->willWrite || fdp->willExecute))
        fdp->willRead = True;

    if (!fdp->willWrite)  fdp->mustExist = True;

    return XtIsRealized(fdw);
}


static void VisibilityEvent( w, event )
Widget w;
XVisibilityEvent *event;
{
/*
   Need something here to keep from being obscured by an ancestor
   (in fact, this is needed for _any_ popup;  I don't know why it isn't
   built into Xt, twm, or somewhere...
*/

/*
    Widget p;

    if (event->state != VisibilityUnobscured)  {
        printf( "\nFD: visible: %s ", XtName(w) );
        p = XtParent(w);
        while (p)  {
            printf( " %s ", XtName(p) );
            p = XtParent(p);
        }
    }
*/
}


static void PointerEvent( w, event )
Widget w;
XEvent *event;
{
    XMapRaised( XtDisplay(w), XtWindow(w) );
}


static XtActionsRec actions[] = {
    {"visible", VisibilityEvent},
    {"mouse",   PointerEvent}
};

static char translations[] = "\
<ButtonPress>: mouse()    \n\
<Visible>:     visible()  \
";


FileDlgClassRec fileDlgClassRec = {
  { /* core_class fields */
    /* superclass         */    (WidgetClass) &transientShellClassRec,
    /* class_name         */    "FileDlg",
    /* widget_size        */    sizeof(FileDlgRec),
    /* class_initialize   */    XawInitializeWidgetSet,
    /* class_part init    */    NULL,
    /* class_inited       */    False,
    /* initialize         */    Initialize,
    /* initialize_hook    */    NULL,
    /* realize            */    Realize,
    /* actions            */    actions,
    /* num_actions        */    XtNumber(actions),
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    True,
    /* compress_exposure  */    True,
    /* compress_enterleave*/    True,
    /* visible_interest   */    True,
    /* destroy            */    Destroy,
    /* resize             */    XtInheritResize,
    /* expose             */    NULL,
    /* set_values         */    SetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */    NULL,
    /* accept_focus       */    NULL,
    /* version            */    XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    translations,
    /* query_geometry     */	XtInheritQueryGeometry,
    /* display_accelerator*/	NULL,
    /* extension          */	NULL
  },
  { /* composite_class fields */
    /* geometry_manager   */   XtInheritGeometryManager,
    /* change_managed     */   XtInheritChangeManaged,
    /* insert_child       */   XtInheritInsertChild,
    /* delete_child       */   XtInheritDeleteChild,
    /* extension          */   NULL
  },
  { /* shell_class fields */
    /* extension          */   NULL
  },
  { /* wm_shell_class fields */
    /* extension          */   NULL
  },
  { /* vendor_shell_class fields */
    /* extension          */   NULL
  },
  { /* transient_shell_class fields */
    /* extension          */   NULL
  },
  { /* filedlg_class fields */
    /* empty              */   0
  }
};

WidgetClass fileDlgWidgetClass = (WidgetClass) &fileDlgClassRec;


FileDlgWidget SetupFileDialog( parent )
Widget parent;
{
    Widget filedlg;

    filedlg = XtVaCreatePopupShell( "filedlg",
        fileDlgWidgetClass, parent,
        NULL );

    return (FileDlgWidget) filedlg;
}


void DoFileDialog( fdw )
FileDlgWidget fdw;
{
    FileDlgPart *fdp;
    XtAppContext app;  XEvent event;

    fdp = &(fdw->filedlg);

    if (XtIsRealized(fdw))  SetPosition(fdw);

    XtPopup( fdw, XtGrabNonexclusive );

    DoFileSearch(fdw);

    app = XtWidgetToApplicationContext(fdw);
    do  {
        XtAppNextEvent( app, &event );  XtDispatchEvent( &event );
    }  while (fdw->shell.popped_up);
}

#ifdef SYSV
/*
 * We define re_comp() and re_exec() in terms of the SYS V functions
 */
#include <regex.h>

static regex_t	compiled_regex;
static int	compiled_one;

static char *
re_comp (pattern)
	char	*pattern;
{
	if (!pattern)
		return ((char *) NULL);
	if (compiled_one)
	{
		regfree (&compiled_regex);
		compiled_one = 0;
	}
	if (regcomp (&compiled_regex, pattern, 0)  != 0)
		return ("Unable to compiled with regcomp()");
	compiled_one = 1;
	return ((char *) NULL);
}

static int
re_exec (line)
	char	*line;
{
	if (!compiled_one)
		return (RE_ERROR);
	if (regexec (&compiled_regex, line, 0, (regmatch_t *) NULL, 0) == 0)
		return RE_MATCH;
	else
		return RE_NOMATCH;
	
}
#endif /* SYSV */
