/*
$Header: /n/abyss/2/chameleon/staff/src/ICA/src/Xica/Group.c,v 1.8 92/05/29 14:59:50 boyd Exp $
*/
/*
   Group.c  -  Row/Column Hierarchical Layout Widget

   $Author: boyd $

   $Revision: 1.8 $

   $Date: 92/05/29 14:59:50 $

   $Id: Group.c,v 1.8 92/05/29 14:59:50 boyd Exp $

   $Log:	Group.c,v $
 * Revision 1.8  92/05/29  14:59:50  boyd
 * Added support for dynamic reordering of children, and GetValues on insertAfter
 * and fromSibling.
 * 
 * 
 * Revision 1.7  92/05/07  16:46:43  boyd
 * 
 * A small fix to semantics - minimum sizes are lower bounds on preferred sizes
 * when preference is false.
 * 
 * Revision 1.6  92/05/07  16:36:32  boyd
 * 
 * Added ConstraintSetValues so that insertAfter and fromSibling could be used
 * dynamically to rearrange widgets in the Group.  This capability is NOT being
 * used anywhere yet, but may be in the near future.
 * 
 * NOTE that GetValues for either of these (especially fromSibling) will return
 * meaningless info - these resources are not used during layout, as in Form or
 * Box, but are used immediately to rearrange the composite part's child list,
 * and are ignored thereafter (i.e., they are _write-only_ resources).
 *   Eventually, I may have to put in a GetValues hook to return them,
 * but it is not yet necessary.
 * 
 * Revision 1.5  92/03/24  11:12:42  ware
 * Fixed centering problem.
 * 
 * Revision 1.4  92/03/18  13:47:12  ware
 * Changes to support better geometry management
 * 
 * Revision 1.5  92/03/05  17:13:04  boyd
 * Added placement resource at group level, analogous to alignment for
 * constraints (see Group.h).
 * 
 * Revision 1.3  92/03/04  19:57:20  boyd
 * Added insertAfter and fromSibling resources to allow relative positioning
 * of children
 * 

*/
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Misc.h>
#include <X11/Xmu/Converters.h>
#include <X11/Xmu/CharSet.h>
#include <X11/Xaw/XawInit.h>

#include "GroupP.h"

#define streq(a,b) (strcmp((a),(b)) == 0)

/* Private Definitions */

static Cardinal GroupInsertPosition( /* Widget */ );

#define offset(field) XtOffset(GroupWidget, field)
static XtResource resources[] = {
    {XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof(Dimension),
        offset(core.border_width), XtRImmediate, (XtPointer) 0},
    {XtNorientation, XtCOrientation, XtROrientation, sizeof(XtOrientation),
        offset(group.orientation), XtRImmediate, (XtPointer) XtorientVertical},
    {XtNspacing, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(group.spacing), XtRImmediate, (XtPointer) 0},
    {XtNplacement, XtCPlacement, XtRPlacement, sizeof(GroupPlace),
        offset(group.place), XtRImmediate, (XtPointer) gpFill},
    {XtNinsertPosition, XtCInsertPosition, XtRFunction, sizeof(XtOrderProc),
        offset(composite.insert_position), XtRImmediate,
        (XtPointer) GroupInsertPosition}
};
#undef offset

#define offset(field) XtOffset(GroupConstraints, group.field)
static XtResource groupConstraintResources[] = {
    {XtNalignment, XtCAlignment, XtRAlignment, sizeof(GroupAlign),
        offset(align), XtRAlignment, (XtPointer) gaTopOrLeft},
    {XtNmargin, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(margin), XtRImmediate, (XtPointer) 0},
    {XtNstretch, XtCStretch, XtRBoolean, sizeof(Boolean),
        offset(stretch), XtRImmediate, (XtPointer) False},
    {XtNminWidth, XtCWidth, XtRDimension, sizeof(Dimension),
        offset(mw), XtRImmediate, (XtPointer) 0},
    {XtNminHeight, XtCHeight, XtRDimension, sizeof(Dimension),
        offset(mh), XtRImmediate, (XtPointer) 0},
    {XtNpreference, XtCPreference, XtRBoolean, sizeof(Boolean),
        offset(preference), XtRImmediate, (XtPointer) True},
    {XtNprefWidth, XtCWidth, XtRDimension, sizeof(Dimension),
        offset(pw), XtRImmediate, (XtPointer) 0},
    {XtNprefHeight, XtCHeight, XtRDimension, sizeof(Dimension),
        offset(ph), XtRImmediate, (XtPointer) 0},
    {XtNinsertAfter, XtCBoolean, XtRBoolean, sizeof(Boolean),
        offset(after), XtRImmediate, (XtPointer) True},
    {XtNfromSibling, XtCWidget, XtRWidget, sizeof(Widget),
        offset(from), XtRWidget, (XtPointer) NULL}
};
#undef offset

static void ClassInitialize(), ClassPartInitialize(), Initialize(), Resize();
static void ConstraintInitialize();
static Boolean SetValues(), ConstraintSetValues();
static void ConstraintGetValuesHook();
static XtGeometryResult GeometryManager(), PreferredGeometry();
static void ChangeManaged();
static void Layout();

static ConstraintClassExtensionRec constraintClassExt = {
    /* next_extension	*/	NULL,
    /* record_type	*/	NULLQUARK,
    /* version		*/	XtConstraintExtensionVersion,
    /* record_size	*/	sizeof(ConstraintClassExtensionRec),
    /* get_values_hook	*/	ConstraintGetValuesHook
};

GroupClassRec groupClassRec = {
  { /* core_class fields */
    /* superclass         */    (WidgetClass) &constraintClassRec,
    /* class_name         */    "Group",
    /* widget_size        */    sizeof(GroupRec),
    /* class_initialize   */    ClassInitialize,
    /* class_part_init    */    ClassPartInitialize,
    /* class_inited       */    FALSE,
    /* initialize         */    Initialize,
    /* initialize_hook    */    NULL,
    /* realize            */    XtInheritRealize,
    /* actions            */    NULL,
    /* num_actions        */    0,
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    TRUE,
    /* compress_exposure  */    TRUE,
    /* compress_enterleave*/    TRUE,
    /* visible_interest   */    FALSE,
    /* destroy            */    NULL,
    /* resize             */    Resize,
    /* expose             */    NULL,
    /* set_values         */    SetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */    NULL,
    /* accept_focus       */    NULL,
    /* version            */    XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */    PreferredGeometry,
    /* display_accelerator*/    XtInheritDisplayAccelerator,
    /* extension          */    NULL
  },
  { /* composite_class fields */
    /* geometry_manager   */   GeometryManager,
    /* change_managed     */   ChangeManaged,
    /* insert_child       */   XtInheritInsertChild,
    /* delete_child       */   XtInheritDeleteChild,
    /* extension          */   NULL
  },
  { /* constraint_class fields */
    /* subresourses       */   groupConstraintResources,
    /* subresource_count  */   XtNumber(groupConstraintResources),
    /* constraint_size    */   sizeof(GroupConstraintsRec),
    /* initialize         */   NULL,
    /* destroy            */   NULL,
    /* set_values         */   ConstraintSetValues,
    /* extension          */   (XtPointer) &constraintClassExt
  },
  { /* group_class fields */
    /* layout             */   Layout
  }
};

WidgetClass groupWidgetClass = (WidgetClass)&groupClassRec;

/****************************************************************
 *
 * Private Procedures
 *
 ****************************************************************/


static XrmQuark XtQTopOrLeft, XtQBottomOrRight, XtQCenter, XtQFill, XtQFlush;


/* ARGSUSED */
static Boolean _CvtStringToAlignment( display,
                                      args, nargs, fromVal, toVal,
                                      data )
Display    *display;
XrmValue   *args;
Cardinal   *nargs;
XrmValue   *fromVal;
XrmValue   *toVal;
XtPointer  *data;
{
    XrmQuark q;
    char lowerName[512];

    XmuCopyISOLatin1Lowered ( lowerName, (char*) fromVal->addr );
    q = XrmStringToQuark(lowerName);

    if (toVal->addr != NULL)  {
        toVal->size = sizeof(GroupAlign);
        if (q == XtQTopOrLeft)  {
            *((GroupAlign *) toVal->addr) = gaTopOrLeft;
            return True;
        } else
        if (q == XtQBottomOrRight)  {
            *((GroupAlign *) toVal->addr) = gaBottomOrRight;
            return True;
        } else
        if (q == XtQCenter)  {
            *((GroupAlign *) toVal->addr) = gaCenter;
            return True;

        } else
        if (q == XtQFill)  {
            *((GroupAlign *) toVal->addr) = gaFill;
            return True;
        }
    }

    XtStringConversionWarning( fromVal->addr, "alignment" );
    return False;
}


/* ARGSUSED */
static Boolean _CvtStringToPlacement( display,
                                      args, nargs, fromVal, toVal,
                                      data )
Display   *display;
XrmValue  *args;
Cardinal  *nargs;
XrmValue  *fromVal;
XrmValue  *toVal;
XtPointer *data;
{
    XrmQuark q;
    char lowerName[512];

    XmuCopyISOLatin1Lowered ( lowerName, (char*) fromVal->addr );
    q = XrmStringToQuark(lowerName);

    if (toVal->addr != NULL)  {
        toVal->size = sizeof(GroupPlace);
        if (q == XtQTopOrLeft)  {
            *((GroupPlace *) toVal->addr) = gpTopOrLeft;
            return True;
        } else
        if (q == XtQBottomOrRight)  {
            *((GroupPlace *) toVal->addr) = gpBottomOrRight;
            return True;
        } else
        if (q == XtQCenter)  {
            *((GroupPlace *) toVal->addr) = gpCenter;
            return True;

        } else
        if (q == XtQFill)  {
            *((GroupPlace *) toVal->addr) = gpFill;
            return True;
        }  else
        if (q == XtQFlush)  {
            *((GroupPlace *) toVal->addr) = gpFlush;
            return True;
        }
    }

    XtStringConversionWarning( fromVal->addr, "placement" );
    return False;
}


static void ClassInitialize()
{
    static XtConvertArgRec parentCvtArgs[] = {
        {XtBaseOffset, (caddr_t)XtOffset(Widget, core.parent), sizeof(Widget)}
    };

    XawInitializeWidgetSet();
    XtQTopOrLeft      = XrmStringToQuark("topOrLeft");
    XtQBottomOrRight  = XrmStringToQuark("bottomOrRight");
    XtQCenter         = XrmStringToQuark("center");
    XtQFill           = XrmStringToQuark("fill");
    XtQFlush          = XrmStringToQuark("flush");

    XtSetTypeConverter( XtRString, XtRAlignment, _CvtStringToAlignment,
        NULL, 0, XtCacheNone, NULL );

    XtSetTypeConverter( XtRString, XtRPlacement, _CvtStringToPlacement,
        NULL, 0, XtCacheNone, NULL );

    XtAddConverter( XtRString, XtRWidget, XmuCvtStringToWidget,
        parentCvtArgs, XtNumber(parentCvtArgs) );
}


static void ClassPartInitialize(class)
WidgetClass class;
{
    register GroupWidgetClass c = (GroupWidgetClass)class;

    if (c->group_class.layout == XtInheritLayout)
        c->group_class.layout =
           ((GroupWidgetClass) c->core_class.superclass)->group_class.layout;
}


/* ARGSUSED */
static void Initialize( req, new, arg, nargs )
Widget req, new;
ArgList arg;
Cardinal nargs;
{
    GroupWidget gw = (GroupWidget) new;

    gw->group.no_refigure    = False;
    gw->group.needs_relayout = False;
    gw->group.reordered      = True;  /* force reset @ GetValues */
}


static Cardinal GroupInsertPosition( w )
Widget w;
/*
    Return the number of children _before_ this child, in the Group's list
    of children.
*/
{
    GroupWidget gw = (GroupWidget) w->core.parent;
    WidgetList child;
    GroupConstraints group;  GroupConstraintsPart *gcp;
    Cardinal i, nc;

    child = gw->composite.children;
    nc = gw->composite.num_children;

    group = (GroupConstraints) w->core.constraints;
    gcp = &(group->group);

    /*
        Set reordered here, since this routine is called exactly when the
	list of children is changed.
    */
    gw->group.reordered = True;

    if (gcp->from == NULL)  return (gcp->after ? nc : 0);
    for (i = 0;  i < nc;  i++)
        if (child[i] == gcp->from)  return (gcp->after ? i+1 : i);

    return (gcp->after ? nc : 0);
}


static void RefigureLocations( w )
GroupWidget w;
{
    if (w->group.no_refigure) {
        w->group.needs_relayout = True;
    }
    else {
        (*((GroupWidgetClass)w->core.widget_class)->group_class.layout)
            ( w );
        w->group.needs_relayout = False;
    }
}

/* ARGSUSED */
static void SetPreferred( gw, resize )
GroupWidget gw;
Boolean resize;
{
    GroupPart *gp;
    Dimension minw, minh, i, bw, pw,ph;
    Boolean vertical, chgW, chgH;
    Widget child;
    GroupConstraints group;  GroupConstraintsPart *gcp;
    XtWidgetGeometry request, reply;

    gp = &(gw->group);

    vertical = (gp->orientation == XtorientVertical);
    minw = (vertical ? 0 : gp->spacing);
    minh = (vertical ? gp->spacing : 0);

    for (i = 0;  i < gw->composite.num_children;  i++)  {
        child = gw->composite.children[i];
        if (XtIsManaged(child))  {
            group = (GroupConstraints) child->core.constraints;
            gcp = &(group->group);

            if (vertical)  {
                chgW = (gcp->align == gaFill);  chgH = gcp->stretch;
            }  else  {
                chgW = gcp->stretch;  chgH = (gcp->align == gaFill);
            }
            if (gcp->preference)  {
                request.request_mode = (CWWidth | CWHeight);
                request.width = gcp->mw;  request.height = gcp->mh;
                XtQueryGeometry( child, &request, &reply );
                gcp->pw = reply.width;  gcp->ph = reply.height;
                if (!XtIsComposite(child))  {
		    gcp->mw = gcp->pw;  gcp->mh = gcp->ph;
                }
            }  else  {
                if (gcp->pw < gcp->mw)  {
                    gcp->pw = (chgW ? gcp->mw :
                                      Max( gcp->mw, child->core.width ));
                }
                if (gcp->ph < gcp->mh)  {
                    gcp->ph = (chgH ? gcp->mh :
                                      Max( gcp->mh, child->core.height ));
                }
            }
            pw = gcp->pw;  ph = gcp->ph;  bw = 2 * child->core.border_width;
            if (vertical)  {
                pw += bw + 2*gcp->margin;
                if (minw < pw)  minw = pw;
                minh += (ph + bw) + gp->spacing;
            }  else  {
                ph += bw + 2*gcp->margin;
                if (minh < ph)  minh = ph;
                minw += (pw + bw) + gp->spacing;
            }
        }
    }
    gp->pw = Max( minw, 2*gp->spacing );
    gp->ph = Max( minh, 2*gp->spacing );

    if (resize)  {
        XtMakeResizeRequest( gw, gp->pw, gp->ph,  &pw, &ph );
    }
}


static void Resize( w )
Widget w;
{
    GroupWidget gw = (GroupWidget) w;
    GroupPart *gp;
    Dimension minh, i, cw, ch, bw, dw, nm, ns;
    Dimension margin, spacing, stretch, fill, excess;
    Dimension width, height;
    int extra;
    Boolean vertical;
    int x,y;
    Widget child;
    GroupConstraints group;  GroupConstraintsPart *gcp;
    XtWidgetGeometry request, reply;

    gp = &(gw->group);

    vertical = (gp->orientation == XtorientVertical);

    minh = gp->spacing;
    nm = ns = 0;
    for (i = 0;  i < gw->composite.num_children;  i++)  {
        child = gw->composite.children[i];
        if (XtIsManaged(child))  {
            group = (GroupConstraints) child->core.constraints;
            gcp = &(group->group);

            nm++;
            if (gcp->stretch)  {
                ch = (vertical ? gcp->mh : gcp->mw);
                ns++;
            } else
                ch = (vertical ? gcp->ph : gcp->pw);
            bw = 2 * child->core.border_width;
            minh += (ch + bw) + gp->spacing;
        }
    }

    if (vertical)  {
        extra = gw->core.height - minh;
    }  else  {
        extra = gw->core.width - minh;
    }
    if (extra < 0)  {
        margin = spacing = gp->spacing;  stretch = excess = 0;
    }  else  {
        if (ns > 0)  {
            margin = spacing = gp->spacing;
            stretch = extra / ns;
	    excess = extra - (stretch * ns);
        }  else  {
            stretch = 0;  excess = 0;
            switch (gp->place)  {
                case gpTopOrLeft:
                    margin = spacing = gp->spacing;
                    break;
                case gpBottomOrRight:
                    margin = gp->spacing + extra;
                    spacing = gp->spacing;
                    break;
                case gpCenter:
                    margin = gp->spacing + (extra / 2);
                    spacing = gp->spacing;
                    break;
                case gpFill:
                    fill = extra / (nm+1);
                    margin = spacing = gp->spacing + fill;
                    excess = extra - (fill * (nm+1));
                    break;
                case gpFlush:
                    if (nm > 1)  {
                        margin = gp->spacing;
                        fill = extra / (nm-1);
                        spacing = gp->spacing + fill;
			excess = extra - (fill * (nm-1));
                    }  else  {
		        fill = extra / 2;
                        margin = gp->spacing + fill;
			excess = extra - (fill * 2);
                        spacing = 0;
                    }
                    break;
            }
        }
    }
    /*
        place managed children
    */
    if (vertical)  {
        x = 0;  y = margin;
        width = gw->core.width;  height = gw->core.height;
    }  else  {
        x = margin;  y = 0;
        width = gw->core.height;  height = gw->core.width;
    }

    for (i = 0;  i < gw->composite.num_children;  i++)  {
        child = gw->composite.children[i];
        if (XtIsManaged(child))  {
            group = (GroupConstraints) child->core.constraints;
            gcp = &(group->group);
            bw = child->core.border_width;

            if (vertical)  {
                cw = gcp->pw;
                ch = (gcp->stretch ? gcp->mh : gcp->ph);
            }  else  {
                cw = gcp->ph;
                ch = (gcp->stretch ? gcp->mw : gcp->pw);
            }
            if (gcp->stretch)  {
	        ch += stretch + excess;  excess = 0;
	    }
            switch (gcp->align)  {
                case gaFill:
                    dw = gcp->margin;
                    cw = width - (2*dw + 2*bw);
		    excess = 0;
                    break;
                case gaTopOrLeft:
                    dw = gcp->margin;
                    break;
                case gaBottomOrRight:
                    dw = width - (cw + 2*bw + gcp->margin);
                    break;
                case gaCenter:
                    dw = (width - (cw + 2*bw)) / 2;
                    break;
            }

	    /* In the following, core values are changed to force an update */

            if (vertical)  {
		child->core.width = child->core.height = 0;
	        child->core.y = -1;
                XtConfigureWidget( child, x+dw, y, cw, ch, bw );
                y += child->core.height + 2*bw + spacing + excess;
            }  else  {
		child->core.width = child->core.height = 0;
	        child->core.x = -1;
                XtConfigureWidget( child, x, y+dw, ch, cw, bw );
                x += child->core.width + 2*bw + spacing + excess;
            }
            if (!gcp->preference)  {
                gcp->pw = child->core.width;  gcp->ph = child->core.height;
            }
	    excess = 0;  /* use only once */
        }
    }
}


static void Layout( gw )
GroupWidget gw;
{
    SetPreferred( gw, True );  Resize( (Widget) gw );
}


/* ARGSUSED */
static XtGeometryResult GeometryManager( w, request, reply )
Widget w;
XtWidgetGeometry *request;
XtWidgetGeometry *reply;        /* RETURN */
{
    GroupConstraints group = (GroupConstraints) w->core.constraints;
    Dimension width, height, border_width;

    if (request->request_mode &
        ~(XtCWQueryOnly | CWWidth | CWHeight | CWBorderWidth))
        return XtGeometryNo;

    if (request->request_mode & CWWidth)
        width = request->width;
    else
        width = w->core.width;

    if (request->request_mode & CWHeight)
        height = request->height;
    else
        height = w->core.height;

    if (request->request_mode & CWBorderWidth)
        border_width = request->border_width;
    else
        border_width = w->core.border_width;

    if (width == w->core.width &&
        height == w->core.height &&
        border_width == w->core.border_width)
        return XtGeometryNo;

    if (!(request->request_mode & XtCWQueryOnly)) {
        w->core.width  = width;
        w->core.height = height;
        w->core.border_width = border_width;
        RefigureLocations( (GroupWidget) XtParent(w) );
    }
    return XtGeometryYes;
}


/* ARGSUSED */
static Boolean SetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    return( FALSE );
}


/* ARGSUSED */
static Boolean ConstraintSetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    GroupConstraints group = (GroupConstraints) new->core.constraints;
    GroupWidget gw = (GroupWidget) new->core.parent;
    GroupPart *gp;
    Boolean chgdFrom, chgdAfter;
    char *argname;
    int i;  Cardinal n, ip;
    WidgetList child;
    
    gp = &(gw->group);

    chgdFrom = chgdAfter = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
	if (streq( argname, XtNinsertAfter ))  chgdAfter = True;
	if (streq( argname, XtNfromSibling ))  chgdFrom  = True;
    }

    if (!(chgdFrom || chgdAfter))  return (FALSE);

    n = gw->composite.num_children;  child = gw->composite.children;
    for (ip = 0;  ip < n;  ip++)
        if (child[ip] == new)  break;

    n--;  (gw->composite.num_children)--;
    for (i = ip;  i < n;  i++)  child[i] = child[i+1];

    ip = GroupInsertPosition( new );
    for (i = n;  i > ip;  i--)  child[i] = child[i-1];
    child[ip] = new;

    (gw->composite.num_children)++;

    RefigureLocations(gw);

    return (TRUE);
}


/* ARGSUSED */
static void ConstraintGetValuesHook( w, arg, nargs )
Widget w;
ArgList arg;
Cardinal *nargs;
{
    GroupWidget gw = (GroupWidget) w->core.parent;
    GroupPart *gp;
    GroupConstraints group;  GroupConstraintsPart *gcp;
    Cardinal i, n;
    WidgetList child;
    
    gp = &(gw->group);

    /*
        The reason for this hook is to set values for after and from, if
	necessary.  The reordered flag is set if the values may be
	wrong or not set for all children.

	A canonical form is used for all children, namely, after is True
	and from is the previous child in the list.
    */
    if (!gp->reordered)  {
        child = gw->composite.children;  n = gw->composite.num_children;
	for (i = 0;  i < n;  i++)  {
            group = (GroupConstraints) child[i]->core.constraints;
            gcp = &(group->group);
	    
	    gcp->after = True;
	    gcp->from = (i > 0 ? child[i-1] : NULL);
	}
	gp->reordered = False;
    }

    XtGetSubvalues( w,
		    groupConstraintResources,
		    XtNumber(groupConstraintResources),
		    arg, *nargs );
}


static void ChangeManaged(w)
Widget w;
{
    GroupWidget gw = (GroupWidget) w;

    RefigureLocations( gw );
}


static XtGeometryResult PreferredGeometry( w, request, reply  )
Widget w;
XtWidgetGeometry *request, *reply;
{
    GroupWidget gw = (GroupWidget) w;

    SetPreferred( gw, False );

    reply->width = gw->group.pw;
    reply->height = gw->group.ph;
    reply->request_mode = CWWidth | CWHeight;
    if (request->request_mode & (CWWidth | CWHeight) ==
          reply->request_mode & CWWidth | CWHeight &&
          request->width == reply->width &&
          request->height == reply->height)
        return XtGeometryYes;
    else
    if (reply->width == gw->core.width && reply->height == gw->core.height)
        return XtGeometryNo;
    else
        return XtGeometryAlmost;
}


/**********************************************************************
 *
 * Public routines
 *
 **********************************************************************/

/* 
 * Set or reset figuring (ignored if not realized)
 */

void GroupDoLayout( w, doit )
Widget w;
Boolean doit;
{
    register GroupWidget gw = (GroupWidget) w;

    gw->group.no_refigure = !doit;

    if (XtIsRealized(w) && gw->group.needs_relayout)
        RefigureLocations( gw );
}
