#ifndef _Group_h
#define _Group_h

#include <X11/Constraint.h>
#include <X11/Shell.h>            /* XtNminWidth, XtNminHeight */
#include <X11/Xmu/Converters.h>   /* XtOrientation */

/***********************************************************************
 *
 * Group  Widget
 *
 ***********************************************************************/

/* Resources:

 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 borderWidth         BorderWidth        Dimension       0
 orientation         Orientation        XtOrientation   vertical (1)
 spacing             Thickness          Dimension       0
 placement           Placement          Placement       gpFill (2)
Note (1):
   From a resource file, use "horizontal" or "vertical".  From a procedure
   call, use "XtorientHorizontal" or "XtorientVertical".
Note (2):
   This resource is only used when no children are being stretched.
*/

/* Constraint resources:

 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 alignment           Alignment          Alignment       TopOrLeft (3)
 margin              Thickness          Dimension       0
 stretch             Stretch            Boolean         False
 minWidth            Width              Dimension       0 (4)
 minHeight           Height             Dimension       0 (4)
 preference          Preference         Boolean         True
 prefWidth           Width              Dimension       core.width  (5)
 prefHeight          Height             Dimension       core.height (5)
 insertAfter         Boolean            Boolean         True (6)
 fromSibling         Widget             Widget          NULL (6)

Note (3):
   From a resource file, use the enum names (below) without the "ga" prefix.
   From a call, use the full enum names.
Note (4):
   These are used only if preference is False or the child is a composite
   widget (a subclass of Composite).  If a non-composite widget has
   preference True, its preferred and minimum sizes are taken to be the
   same, as determined by querying the widget.
Note (5):
   These are used only if preference is False.  If preference is True, the
   widget is queried for preferred width and height.
   When used, they determine only the _initial_ size of the widget.  If the
   user resizes, the new values become the preferred values.
Note (6):
   If insertAfter and fromSibling is NULL, the widget will be inserted as
   the last child.  If !insertAfter and fromSibling is NULL, the widget will
   be inserted before the first child.
*/


#define XtNspacing     "spacing"
#define XtNplacement   "placement"
#define XtNalignment   "alignment"
#define XtNmargin      "margin"
#define XtNstretch     "stretch"
#define XtNpreference  "preference"
#define XtNprefWidth   "prefWidth"
#define XtNprefHeight  "prefHeight"
#define XtNinsertAfter "insertAfter"
#define XtNfromSibling "fromSibling"

#define XtCPlacement   "Placement"
#define XtCAlignment   "Alignment"
#define XtCStretch     "Stretch"
#define XtCPreference  "Preference"

/*
   Because the following are also defined by Form.h, this file should
   be included after Form.h.
*/
#ifndef XtCWidget
#define XtCWidget      "Widget"
#endif
#ifndef XtRWidget
#define XtRWidget      "Widget"
#endif
/* */

typedef enum {
    gpTopOrLeft,      /* place children at top (horiz) or left (vert)  */
    gpBottomOrRight,  /* place children at bottom or right */
    gpCenter,         /* put extra space before first and after last  */
    gpFill,           /* space uniformly around children */
    gpFlush           /* space uniformly between children, no extra
                         before first or after last */
} GroupPlace;

typedef enum {
    gaTopOrLeft,      /* align to top (horiz) or left (vert)  */
    gaBottomOrRight,  /* align to bottom or right */
    gaCenter,         /* center  */
    gaFill            /* resize to fill between margins */
} GroupAlign;

typedef struct _GroupClassRec   *GroupWidgetClass;
typedef struct _GroupRec        *GroupWidget;

extern WidgetClass groupWidgetClass;

extern void GroupDoLayout();    /* widget, doit */
    /* Widget widget */
    /* Boolean doit */
 
#endif /* _Group_h */
