#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/XawInit.h>
#include "IcaTextP.h"

#define streq(a,b) (strcmp( (a), (b) ) == 0)

#define offset(field) XtOffset( IcaTextWidget, field )
/*
    {name, class, type, size,
        offset, default_type, default_addr},
*/
static XtResource resources[] = {
    {XtNresize, XtCResize, XtRResizeMode, sizeof(XawTextResizeMode),
       offset(text.resize), XtRImmediate, (caddr_t) XawtextResizeBoth},
    {XtNacceptProc, XtCCallback, XtRCallback, sizeof(caddr_t),
        offset(ica_text.acceptProc), XtRCallback, NULL},
};
#undef offset

static void Accept();

static XtActionsRec actions[] = {
    {"accept",      Accept}
};

static char translations[] = "\
<Key>Return:    accept()    \n\
<Key>Linefeed:  accept()    \n\
<Key>KP_Enter:  accept()      \
";

static void Initialize(), Realize();
static Boolean SetValues();
static XtGeometryResult PreferredGeometry();

IcaTextClassRec icaTextClassRec = {
  { /* core fields */
    /* superclass       */      (WidgetClass) &asciiTextClassRec,
    /* class_name       */      "IcaText",
    /* widget_size      */      sizeof(IcaTextRec),
    /* class_initialize */      XawInitializeWidgetSet,
    /* class_part_init  */	NULL,
    /* class_inited     */      FALSE,
    /* initialize       */      Initialize,
    /* initialize_hook  */	NULL,
    /* realize          */      Realize,
    /* actions          */      actions,
    /* num_actions      */      XtNumber(actions),
    /* resources        */      resources,
    /* num_ resource    */      XtNumber(resources),
    /* xrm_class        */      NULLQUARK,
    /* compress_motion  */      TRUE,
    /* compress_exposure*/      XtExposeGraphicsExpose,
    /* compress_enterleave*/	TRUE,
    /* visible_interest */      FALSE,
    /* destroy          */      NULL,
    /* resize           */      XtInheritResize,
    /* expose           */      XtInheritExpose,
    /* set_values       */      SetValues,
    /* set_values_hook  */	NULL,
    /* set_values_almost*/	XtInheritSetValuesAlmost,
    /* get_values_hook  */	NULL,
    /* accept_focus     */      XtInheritAcceptFocus,
    /* version          */	XtVersion,
    /* callback_private */      NULL,
    /* tm_table         */      XtInheritTranslations,
    /* query_geometry	*/	PreferredGeometry
  },
  { /* Simple fields */
    /* change_sensitive	*/	XtInheritChangeSensitive
  },
  { /* text fields */
    /* empty            */      0
  },
  { /* ascii fields */
    /* empty            */      0
  },
  { /* ica_text fields */
    /* empty            */      0
  }
};

WidgetClass icaTextWidgetClass = (WidgetClass) &icaTextClassRec;


static void PreferredTextSize( tw, width, height )
IcaTextWidget tw;
Dimension *width, *height;
{
    String text;
    XFontStruct	*fsp;  XCharStruct cs;
    int dh, fa, fd;
    Dimension tm,bm, lm,rm;

    XtVaGetValues( tw,
        XtNfont, &fsp,
        XtNstring, &text,
        XtNtopMargin, &tm,
        XtNbottomMargin, &bm,
        XtNleftMargin, &lm,
        XtNrightMargin, &rm,
        NULL );

    if (text && strlen(text))  {
	XTextExtents( fsp, text, strlen(text), &dh, &fa, &fd, &cs );
        *width = (Dimension) (cs.rbearing - cs.lbearing + lm + rm);
        *height = (Dimension) (fa + fd + tm + bm);
    }  else  {
        *width = tw->core.width;  *height = tw->core.height;
    }
}


/* ARGSUSED */
static void
Initialize( req, new, arg, nargs )
Widget req, new;
ArgList arg;
Cardinal *nargs;
{
    IcaTextWidget tw = (IcaTextWidget) new;
    int i;
    String argname;
    Boolean redisplay, reqWidth, reqHeight;
    Dimension w, h;

    reqWidth = reqHeight = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
        if (streq( argname, XtNwidth  ))  reqWidth = True;
        if (streq( argname, XtNheight ))  reqHeight = True;
    }

    PreferredTextSize( tw, &w, &h );
    if (!reqWidth)   tw->core.width = w;
    if (!reqHeight)  tw->core.height = h;

    XtOverrideTranslations( tw, XtParseTranslationTable(translations) );    
}


static void Realize( w, valueMask, attributes )
Widget w;
XtValueMask *valueMask;
XSetWindowAttributes *attributes;
{
    WidgetClass superclass = w->core.widget_class->core_class.superclass;

    (*superclass->core_class.realize)( w, valueMask, attributes );
}


/* ARGSUSED */
static Boolean SetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    WidgetClass superclass = new->core.widget_class->core_class.superclass;
    int i;
    String argname;
    Boolean redisplay, chgdString, chgdFont, newSize;
    Dimension w, h;
    XtGeometryResult result;

    chgdString = chgdFont = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
        if (streq( argname, XtNstring ))  chgdString = True;
        if (streq( argname, XtNfont   ))  chgdFont = True;
    }
    newSize = False;
    if (chgdString || chgdFont)  {
        PreferredTextSize( (IcaTextWidget) new, &w, &h );
        result = XtMakeResizeRequest( new, w, h, NULL, NULL );
        newSize = (result == XtGeometryYes || result == XtGeometryDone);
    }

    return newSize && XtIsRealized(new);
}


static XtGeometryResult PreferredGeometry( w, request, reply )
Widget w;
XtWidgetGeometry *request, *reply;
{
    IcaTextWidget tw = (IcaTextWidget) w;

    reply->request_mode = CWWidth | CWHeight;
    PreferredTextSize( tw, &(reply->width), &(reply->height) );

    if (((request->request_mode & (CWWidth | CWHeight))	==
                                  (CWWidth | CWHeight)) &&
	  request->width == reply->width &&
	  request->height == reply->height)
	return XtGeometryYes;
    else
    if (reply->width == w->core.width && reply->height == w->core.height)
	return XtGeometryNo;
    else
	return XtGeometryAlmost;
}


/* ARGSUSED */
static void Accept( w, event )
Widget w;
XEvent *event;
{
    IcaTextWidget tw = (IcaTextWidget) w;  IcaTextPart *tp;
    String text;

    tp = &(tw->ica_text);
    if (tp->acceptProc)  {
        XtVaGetValues( w, XtNstring, &text, NULL );
        XtCallCallbacks( w, XtNacceptProc, (caddr_t) text );
    }
}









