/*
    MsgBox.c:  Message Box Widget
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <malloc.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/Misc.h>
#include <X11/Xaw/XawInit.h>

#include <X11/Shell.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include "Group.h"

#include "MsgBoxP.h"

#define streq(a,b) (strcmp( (a), (b) ) == 0)

#define offset(field) XtOffset( MsgBoxWidget, field )
/*
    {name, class, type, size,
        offset, default_type, default_addr},
*/
static XtResource resources[] = {
    {XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof(Dimension),
        offset(core.border_width), XtRImmediate, (XtPointer) 2},
    {XtNallowShellResize, XtCAllowShellResize, XtRBoolean, sizeof(Boolean),
        offset(shell.allow_shell_resize), XtRImmediate, (XtPointer) True},
    {XtNlabelFont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
        offset(msgbox.labelFont),XtRString, "XtDefaultFont"},
    {XtNbtnFont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
        offset(msgbox.btnFont),XtRString, "XtDefaultFont"},
    {XtNbtnCursor, XtCCursor, XtRCursor, sizeof(Cursor),
        offset(msgbox.btnCursor), XtRString, "hand2"},
    {XtNmessage, XtCMessage, XtRString, sizeof(String),
        offset(msgbox.message), XtRString, NULL},
    {XtNresponse, XtCResponse, XtRBoolean, sizeof(Boolean),
        offset(msgbox.response), XtRImmediate, (XtPointer) True},
    {XtNdefaultResponse, XtCDefaultResponse, XtRBoolean, sizeof(Boolean),
        offset(msgbox.defaultResponse), XtRImmediate, (XtPointer) True},
    {XtNdone, XtCDone, XtRBoolean, sizeof(Boolean),
        offset(msgbox.done), XtRImmediate, (XtPointer) False},
};
#undef offset


static void Ok( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    MsgBoxWidget mbw = (MsgBoxWidget) appData;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);

    mbp->response = mbp->defaultResponse;  mbp->done = True;

    XtPopdown(mbw);
}   

static void Yes( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    MsgBoxWidget mbw = (MsgBoxWidget) appData;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);  mbp->response = True;  mbp->done = True;

    XtPopdown(mbw);
}


static void No( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    MsgBoxWidget mbw = (MsgBoxWidget) appData;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);  mbp->response = False;  mbp->done = True;

    XtPopdown(mbw);
}


static void Cancel( w, appData, wData )
Widget w;
XtPointer appData, wData;
{
    MsgBoxWidget mbw = (MsgBoxWidget) appData;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);  

    mbp->response = mbp->defaultResponse;  mbp->done = False;

    XtPopdown(mbw);
}   


/* ARGSUSED */
static void Initialize( req, new, arg, nargs )
Widget req, new;
ArgList arg;
Cardinal *nargs;
{
    MsgBoxWidget mbw = (MsgBoxWidget) new;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);

    mbp->msggrp = XtVaCreateManagedWidget( "msggrp",
        groupWidgetClass, mbw,
        XtNspacing, 12,
        XtNorientation, XtorientVertical,
        NULL );

    mbp->msglbl = XtVaCreateManagedWidget( "msglbl",
        labelWidgetClass, mbp->msggrp,
        XtNlabel, mbp->message,
        XtNborderWidth, 0,
        XtNfont, mbp->labelFont,
        XtNalignment, gaCenter,
        XtNmargin, 12,
        XtNstretch, False,
        NULL );
    mbp->btngrp = XtVaCreateManagedWidget( "btngrp",
        groupWidgetClass, mbp->msggrp,
        XtNspacing, 12,
        XtNorientation, XtorientHorizontal,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        NULL );

    mbp->okbtn = mbp->yesbtn = mbp->nobtn = mbp->cancelbtn = NULL;
}

/* ARGSUSED */
static Boolean SetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    MsgBoxWidget mbw = (MsgBoxWidget) new;
    MsgBoxWidget mbOld = (MsgBoxWidget) old;
    MsgBoxPart *mbp, *mbpOld;
    int i;  String argname;
    Boolean chgdLabelFont, chgdBtnFont,  chgdMessage;

    mbp = &(mbw->msgbox);  mbpOld = &(mbOld->msgbox);

    chgdLabelFont = chgdBtnFont = chgdMessage = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
        if (strcmp( argname, XtNlabelFont ) == 0)  chgdLabelFont = True;
        if (strcmp( argname, XtNbtnFont ) == 0)    chgdBtnFont = True;
        if (strcmp( argname, XtNmessage ) == 0)    chgdMessage = True;
    }

    if (chgdLabelFont)  {
        XtVaSetValues( mbp->msglbl, XtNfont, mbp->labelFont, NULL );
    }

    if (chgdBtnFont)  {
        if (mbp->okbtn)
            XtVaSetValues( mbp->okbtn,  XtNfont, mbp->btnFont, NULL );
        if (mbp->yesbtn)
            XtVaSetValues( mbp->yesbtn,  XtNfont, mbp->btnFont, NULL );
        if (mbp->nobtn)
            XtVaSetValues( mbp->nobtn,  XtNfont, mbp->btnFont, NULL );
        if (mbp->cancelbtn)
            XtVaSetValues( mbp->cancelbtn,  XtNfont, mbp->btnFont, NULL );
    }

    if (chgdMessage)  {
        XtVaSetValues( mbp->msglbl, XtNlabel, mbp->message, NULL );
    }

    return XtIsRealized(mbw);
}


static void Realize( w, valueMask, attributes )
Widget w;
XtValueMask *valueMask;
XSetWindowAttributes *attributes;
/*
    This is being used here as a point at which to place the dialog box - 
    after the first call to change_managed, but before mapping
*/
{
    WidgetClass superclass = w->core.widget_class->core_class.superclass;
    Display *display;  Screen *screen;
    Window root, child;  int x,y, dx,dy, b;
    int sw, sh, bw;
    XWMHints hints;
    XWindowAttributes attr;

    display = XtDisplay(w);
    XQueryPointer( display, DefaultRootWindow(display), &root, &child,
                   &x,&y, &dx,&dy, &b );

    screen = XtScreen(w);
    bw = w->core.border_width;
    sw = WidthOfScreen(screen) - w->core.width - 1;
    sh = HeightOfScreen(screen) - w->core.height - 1;
    x -= w->core.width/2;  /* try to center at pointer */
    y -= w->core.height/2;
    if (x < 0)  x = 0;     /* otherwise, keep it in screen */
    else
    if (x+bw > sw)  x = sw-bw;
    if (y < 0)  y = 0;
    else
    if (y+bw > sh)  y = sh-bw;
    XtVaSetValues( w, XtNx, x,  XtNy, y,  NULL );

    (*superclass->core_class.realize)( w, valueMask, attributes );

    XSetTransientForHint( display, XtWindow(w), XtWindow(XtParent(w)) );

    XtSetKeyboardFocus( w, None );
}


static Boolean AcceptFocus( w, time )
Widget w;
Time *time;
{
    return True;
}


static void VisibilityEvent( w, event )
Widget w;
XVisibilityEvent *event;
{
/*
   Need something here to keep from being obscured by an ancestor
   (in fact, this is needed for _any_ popup;  I don't know why it isn't
   built into Xt, twm, or somewhere...
*/
/*
    Widget p;

    if (event->state != VisibilityUnobscured)  {
        printf( "\nMB: visible: %s ", XtName(w) );
        p = XtParent(w);
        while (p)  {
            printf( " %s ", XtName(p) );
            p = XtParent(p);
        }
    }
*/
}


static void PointerEvent( w, event )
Widget w;
XEvent *event;
{
    XMapRaised( XtDisplay(w), XtWindow(w) );
}


static void AcceptEvent( w, event )
Widget w;
XEvent *event;
{
    MsgBoxWidget mbw = (MsgBoxWidget) w;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);  

    mbp->response = mbp->defaultResponse;  mbp->done = True;

    XtPopdown(mbw);
}


static void CancelEvent( w, event )
Widget w;
XEvent *event;
{
    MsgBoxWidget mbw = (MsgBoxWidget) w;
    MsgBoxPart *mbp;

    mbp = &(mbw->msgbox);  

    mbp->response = mbp->defaultResponse;  mbp->done = False;

    XtPopdown(mbw);
}


static XtActionsRec actions[] = {
    {"accept",  AcceptEvent},
    {"cancel",  CancelEvent},
    {"visible", VisibilityEvent},
    {"mouse",   PointerEvent}
};

static char translations[] = "\
<Key>Return:    accept()   \n\
<Key>Linefeed:  accept()   \n\
<Key>KP_Enter:  accept()   \n\
<ButtonPress>:  mouse()    \n\
<Visible>:      visible()  \
";


MsgBoxClassRec msgBoxClassRec = {
  { /* core_class fields */
    /* superclass         */    (WidgetClass) &transientShellClassRec,
    /* class_name         */    "MsgBox",
    /* widget_size        */    sizeof(MsgBoxRec),
    /* class_initialize   */    XawInitializeWidgetSet,
    /* class_part init    */    NULL,
    /* class_inited       */    False,
    /* initialize         */    Initialize,
    /* initialize_hook    */    NULL,
    /* realize            */    Realize,
    /* actions            */    actions,
    /* num_actions        */    XtNumber(actions),
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    False,
    /* compress_exposure  */    True,
    /* compress_enterleave*/    False,
    /* visible_interest   */    True,
    /* destroy            */    NULL,
    /* resize             */    XtInheritResize,
    /* expose             */    NULL,
    /* set_values         */    SetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */    NULL,
    /* accept_focus       */    AcceptFocus,
    /* version            */    XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    translations,
    /* query_geometry     */	XtInheritQueryGeometry,
    /* display_accelerator*/	NULL,
    /* extension          */	NULL
  },
  { /* composite_class fields */
    /* geometry_manager   */   XtInheritGeometryManager,
    /* change_managed     */   XtInheritChangeManaged,
    /* insert_child       */   XtInheritInsertChild,
    /* delete_child       */   XtInheritDeleteChild,
    /* extension          */   NULL
  },
  { /* shell_class fields */
    /* extension          */   NULL
  },
  { /* wm_shell_class fields */
    /* extension          */   NULL
  },
  { /* vendor_shell_class fields */
    /* extension          */   NULL
  },
  { /* transient_shell_class fields */
    /* extension          */   NULL
  },
  { /* msgbox_class fields */
    /* empty              */   0
  }
};

WidgetClass msgBoxWidgetClass = (WidgetClass) &msgBoxClassRec;


void OkMessage( parent, message )
Widget parent;
String message;
/*
   For now, place the dialog box at the current pointer position.
*/
{
    MsgBoxWidget mbw;  MsgBoxPart *mbp;
    XtAppContext app;  XEvent event;
    Dimension mw, bw, dw, ds;
    Boolean disp;

    mbw = (MsgBoxWidget) XtVaCreatePopupShell( "Message",
        msgBoxWidgetClass, parent,
        NULL );

    mbp = &(mbw->msgbox);

    mbp->okbtn = XtVaCreateManagedWidget( "okbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " OK ",
        XtNborderWidth, 2,
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );

    XtAddCallback( mbp->okbtn, XtNcallback, Ok, mbw );

    if (message)
        XtVaSetValues( mbp->msglbl, XtNlabel, message, NULL );

    XtPopup( mbw, XtGrabNonexclusive );

    app = XtWidgetToApplicationContext(mbw);
    do  {
        XtAppNextEvent( app, &event );  XtDispatchEvent( &event );
    }  while (mbw->shell.popped_up);

    XtDestroyWidget(mbw);
}

Boolean OkCancelMessage( parent, message )
Widget parent;
String message;
/*
   For now, place the dialog box at the current pointer position.
*/
{
    MsgBoxWidget mbw;  MsgBoxPart *mbp;
    XtAppContext app;  XEvent event;
    Dimension mw, bw, dw, ds;
    Boolean done;

    mbw = (MsgBoxWidget) XtVaCreatePopupShell( "Message",
        msgBoxWidgetClass, parent,
        NULL );

    mbp = &(mbw->msgbox);

    mbp->okbtn = XtVaCreateManagedWidget( "okbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " OK ",
        XtNborderWidth, 2,
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );
    mbp->cancelbtn = XtVaCreateManagedWidget( "cancelbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, "Cancel",
        XtNborderWidth, 1,
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );

    XtAddCallback( mbp->okbtn,     XtNcallback, Ok,     mbw );
    XtAddCallback( mbp->cancelbtn, XtNcallback, Cancel, mbw );

    if (message)
        XtVaSetValues( mbp->msglbl, XtNlabel, message, NULL );

    XtPopup( mbw, XtGrabNonexclusive );

    app = XtWidgetToApplicationContext(mbw);
    do  {
        XtAppNextEvent( app, &event );  XtDispatchEvent( &event );
    }  while (mbw->shell.popped_up);

    done = mbp->done;

    XtDestroyWidget(mbw);

    return done;
}

Boolean YesNoMessage( parent, message, defaultResp )
Widget parent;
String message;
Boolean defaultResp;
/*
   For now, place the dialog box at the current pointer position.
*/
{
    MsgBoxWidget mbw;  MsgBoxPart *mbp;
    XtAppContext app;  XEvent event;
    Dimension mw, bw, dw, ds;
    Boolean response;

    mbw = (MsgBoxWidget) XtVaCreatePopupShell( "Message",
        msgBoxWidgetClass, parent,
        NULL );

    mbp = &(mbw->msgbox);  mbp->defaultResponse = defaultResp;

    mbp->yesbtn = XtVaCreateManagedWidget( "yesbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " Yes ",
        XtNborderWidth, (defaultResp ? 2 : 1),
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );
    mbp->nobtn = XtVaCreateManagedWidget( "nobtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " No ",
        XtNborderWidth, (defaultResp ? 1 : 2),
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );

    XtAddCallback( mbp->yesbtn, XtNcallback, Yes, mbw );
    XtAddCallback( mbp->nobtn,  XtNcallback, No,  mbw );

    if (message)
        XtVaSetValues( mbp->msglbl, XtNlabel, message, NULL );

    XtPopup( mbw, XtGrabNonexclusive );

    app = XtWidgetToApplicationContext(mbw);
    do  {
        XtAppNextEvent( app, &event );  XtDispatchEvent( &event );
    }  while (mbw->shell.popped_up);

    response = mbp->response;

    XtDestroyWidget(mbw);

    return response;
}


Boolean YesNoCancelMessage( parent, message, defaultResp, response )
Widget parent;
String message;
Boolean defaultResp, *response;
/*
   For now, place the dialog box at the current pointer position.
*/
{
    MsgBoxWidget mbw;  MsgBoxPart *mbp;
    XtAppContext app;  XEvent event;
    Dimension mw, bw, dw, ds;
    Boolean done;

    mbw = (MsgBoxWidget) XtVaCreatePopupShell( "Message",
        msgBoxWidgetClass, parent,
        NULL );

    mbp = &(mbw->msgbox);  mbp->defaultResponse = defaultResp;

    mbp->yesbtn = XtVaCreateManagedWidget( "yesbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " Yes ",
        XtNborderWidth, (defaultResp ? 2 : 1),
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );
    mbp->nobtn = XtVaCreateManagedWidget( "nobtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, " No ",
        XtNborderWidth, (defaultResp ? 1 : 2),
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );
    mbp->cancelbtn = XtVaCreateManagedWidget( "cancelbtn",
        commandWidgetClass, mbp->btngrp,
        XtNlabel, "Cancel",
        XtNborderWidth, 1,
        XtNfont, mbp->btnFont,
        XtNcursor, mbp->btnCursor,
        XtNalignment, gaCenter,
        XtNmargin, 0,
        XtNstretch, False,
        NULL );

    XtAddCallback( mbp->yesbtn,    XtNcallback, Yes,    mbw );
    XtAddCallback( mbp->nobtn,     XtNcallback, No,     mbw );
    XtAddCallback( mbp->cancelbtn, XtNcallback, Cancel, mbw );

    if (message)
        XtVaSetValues( mbp->msglbl, XtNlabel, message, NULL );

    XtPopup( mbw, XtGrabNonexclusive );

    app = XtWidgetToApplicationContext(mbw);
    do  {
        XtAppNextEvent( app, &event );  XtDispatchEvent( &event );
    }  while (mbw->shell.popped_up);

    done = mbp->done;  *response = mbp->response;

    XtDestroyWidget(mbw);

    return done;
}


