/******************************************************************************
 *									      *
 * Status.c			Source file for the StatusWidget              *
 *									      *
 * VERSION:	1.00		FOR X Version 11, Release 4		      *
 *									      *
 * COPYRIGHT NOTICE							      *
 *									      *
 *	Copyright 1991	Robert M. Griffiths	All Rights Reserved	      *
 *									      *
 ******************************************************************************
 */

#include <stdio.h>

#include <X11/StringDefs.h>
#include <X11/IntrinsicP.h>

#include "StatusP.h"

#include <varargs.h>

/*********************************************************************/
static Dimension defWidth  = 550;
static Dimension defHeight = 250;

#define offset(field) XtOffset( StatusWidget, field )
/*
    {name, class, type, size,
        offset, default_type, default_addr},
*/
static XtResource resources[] = {
  {XtNscrollVertical, XtCScroll, XtRScrollMode, sizeof(XawTextScrollMode),
     offset(text.scroll_vert), XtRImmediate,
     (caddr_t) XawtextScrollWhenNeeded},
  {XtNscrollHorizontal, XtCScroll, XtRScrollMode, sizeof(XawTextScrollMode),
     offset(text.scroll_horiz), XtRImmediate, 
     (caddr_t) XawtextScrollWhenNeeded},
  {XtNwidth, XtCWidth, XtRDimension, sizeof(Dimension),
     offset(core.width), XtRDimension, (caddr_t)&defWidth},
  {XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
     offset(core.height), XtRDimension, (caddr_t)&defHeight}
};
#undef offset
/*********************************************************************/

static char defaultTranslations[] = "None<Key>:  no-op() \n\
                                     Shift<Key>: no-op() \n\
                                     Lock<Key>:  no-op() \n";

/*********************************************************************/

static void Initialize();

/*********************************************************************/

StatusClassRec statusClassRec = {
   {
      /* Core class part initialization -- required for Widget  */
      (WidgetClass) &asciiTextClassRec, /* superclass           */
      "StatusWindow",                   /* class name           */
      sizeof(StatusRec),                /* widget_size          */
      NULL,                             /* class initialize     */
      NULL,                             /* class-part-init      */
      FALSE,                            /* class_inited         */
      Initialize,                       /* initialize           */
      NULL,                             /* initialize_hook      */
      XtInheritRealize,                 /* realize              */
      NULL,                             /* actions              */
      0,                                /* num actions          */
      resources,                        /* resources            */
      XtNumber(resources),              /* num_resources        */
      NULLQUARK,                        /* xrm class            */
      TRUE,                             /* compress_motion      */
      TRUE,                             /* compress_exposure    */
      TRUE,                             /* compress_enterleave  */
      FALSE,                            /* visible interest     */
      NULL,                             /* destroy              */
      XtInheritResize,                  /* resize               */
      XtInheritExpose,                  /* expose               */
      NULL,                             /* set-values           */
      NULL,                             /* set_values_hook      */
      XtInheritSetValuesAlmost,         /* set_values_almost    */
      NULL,                             /* get_values_hook      */
      NULL,                             /* accept_focus         */
      XtVersion,                        /* version              */
      NULL,                             /* callback_private     */
      XtInheritTranslations,            /* tm_table             */
      XtInheritQueryGeometry,           /* query geometry       */
      XtInheritDisplayAccelerator,      /* display accelerator  */
      NULL                              /* extension            */
   },
   {
      /* Simple class member initialization */
      XtInheritChangeSensitive          /* change_sensitive	*/
   },
   {
      /* Text class member initialization */
      0                                 /* empty                */
   },
   {
      /* Ascii class member initialization */
      0                                 /* empty                */
   },
   {
      /* Status class member initialization */
      0                                 /* empty                */
   }
};


WidgetClass statusWidgetClass = (WidgetClass)&statusClassRec;

/*********************************************************************
 *                              Methods
 *********************************************************************/

static void Initialize(request_w, new_w, in_args, in_nargs)
Widget	  request_w, new_w;
ArgList	  in_args;
Cardinal *in_nargs;
{
  StatusWidget 
    new = (StatusWidget)new_w;  /* The StatusWidget to be initialized */
  XtTranslations trans;         /* The translation table after parsing */
  StatusPart *statpt;           /* Pointer to the status instance variables */
  TextPart *txt;                /* Pointer to the text instance variables */
  AsciiSrcObject src;           /* The text widget's AsciiSrc resource */
  Arg args[10];                 /* Arguments to set AsciiSrc resources */
  int cnt;                      /* Number of AsciiSrc resources being set */

  /* 
   * Set variables to the various parts of the status widget for
   * easy access to resources and instance variables.
   */

  statpt = &(new->status);
  txt    = &(new->text);
  src    = (AsciiSrcObject)txt->source;

  /* Override the translations that were inherited from the
   * AsciiTextWidget. Only the translations listed in
   * defaultTranslations are overridden.
   */

  trans = XtParseTranslationTable(defaultTranslations);
  XtOverrideTranslations(new, trans);

  /* Set status resources and instance variables */

  statpt->buffer_size = 0;

  /* Set the AsciiSrc widget's resources */
  
  XtVaSetValues( src, 
     XtNeditType, XawtextAppend,
     XtNtype, XawAsciiString,
     NULL );

  XawTextDisplayCaret( new, False );
  XawTextEnableRedisplay( new );
}

/*********************************************************************
 *                     Visible Functions
 *********************************************************************/

/******************************************************************
 * void XicaStatus(Widget widget, char *format, ...);
 * 
 * This function will behave just like printf.
 *
 ******************************************************************/

void XicaStatus(va_alist)
va_dcl
{
  va_list ap;           /* List of parameters */
  Widget widget;        /* First parameter, string will be displayed to this */
  char *format;         /* Second parameter, format string for printf */
  FILE *foo;            /* File pointer to /dev/null */
  unsigned long size;   /* Size of src string */
  XawTextBlock src;     /* Contains information to append string to widget */
  StatusWidget stat;    /* The widget typecasted to StatusWidget */
  StatusPart *statpart; /* Pointer to StatusWidget instance variables */

  /* Open /dev/null for write */

  if (NULL == (foo = fopen("/dev/null","w"))) {
    (void) fprintf(stderr, "Cannot open /dev/null for write");
    return;
  }
 
  va_start(ap);      /* Start to process variable argument list */

  /* The first two parameters must be the widget and the format string */

  widget = va_arg(ap, Widget);
  format = va_arg(ap, char *);

  /* Cast widget to StatusWidget and set pointer to point
   * to the instance variables.
   */

  stat = (StatusWidget)widget;
  statpart = &(stat->status);

  /* Print the string to /dev/null just to get the size of the formatted
   * string.
   */

  size = vfprintf(foo, format, ap);

  (void) fclose(foo);    /* Close /dev/null, that's all I needed it for */

  /* 
   * Fill up the TextBlock structure that will be sent to the widget.
   */

  src.firstPos = 0;
  src.length   = size;
  src.ptr      = (char *)XtMalloc(size + 2);
  src.format   = FMT8BIT;

  (void) vsprintf(src.ptr, format, ap);    /* Print the string into src */
  va_end(ap);

  /* 
   * Append the new string to the string currently in the widget.
   */

  XawTextDisableRedisplay( widget );
/*
  XawTextDisplayCaret( widget, True);
*/
  (void) XawTextReplace(widget, statpart->buffer_size, 
			statpart->buffer_size, &src);

  /* Increment the instance variable, buffer_size, in order to
   * keep track of the size of the string being displayed.
   */
  
  statpart->buffer_size += size;

  /* Set the insertion point to buffer_size, which is the end
   * of the string. This will force the widget to scroll when
   * new text is added.
   */

  XawTextSetInsertionPoint( widget, statpart->buffer_size );
  XawTextDisplayCaret( widget, False );
  XawTextEnableRedisplay( widget );

  /* Free the strings that I allocated space for. */

  XtFree(src.ptr);
}
