/*
$Header: 
*/
/*
   Table.c  -  Two-dimensional Layout Widget

   $Author: boyd $

   $Revision: 1.1 $

   $Date: 92/05/29 15:03:44 $

   $Log:	Table.c,v $
 * Revision 1.1  92/05/29  15:03:44  boyd
 * Original version of Table.
 * 
 * 

*/
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Misc.h>
#include <X11/Xmu/Converters.h>
#include <X11/Xmu/CharSet.h>
#include <X11/Xaw/XawInit.h>

#include "TableP.h"

#define streq(a,b) (strcmp((a),(b)) == 0)

/* Private Definitions */

#define offset(field) XtOffset( TableWidget, field )
static XtResource resources[] = {
    {XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof(Dimension),
        offset(core.border_width), XtRImmediate, (XtPointer) 0},
    {XtNhorzSpacing, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(table.hspacing), XtRImmediate, (XtPointer) 0},
    {XtNvertSpacing, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(table.vspacing), XtRImmediate, (XtPointer) 0},
    {XtNhorzMargin, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(table.hmargin), XtRImmediate, (XtPointer) 0},
    {XtNvertMargin, XtCThickness, XtRDimension, sizeof(Dimension),
        offset(table.vmargin), XtRImmediate, (XtPointer) 0},
    {XtNhorzPlacement, XtCPlacement, XtRPlacement, sizeof(TablePlace),
        offset(table.hplace), XtRImmediate, (XtPointer) tpFill},
    {XtNvertPlacement, XtCPlacement, XtRPlacement, sizeof(TablePlace),
        offset(table.vplace), XtRImmediate, (XtPointer) tpTopOrLeft}
};
#undef offset

#define offset(field) XtOffset( TableConstraints, table.field )
static XtResource tableConstraintResources[] = {
    {XtNhorzAlignment, XtCAlignment, XtRAlignment, sizeof(TableAlign),
        offset(halign), XtRAlignment, (XtPointer) taTopOrLeft},
    {XtNvertAlignment, XtCAlignment, XtRAlignment, sizeof(TableAlign),
        offset(valign), XtRAlignment, (XtPointer) taTopOrLeft},
    {XtNpreference, XtCPreference, XtRBoolean, sizeof(Boolean),
        offset(preference), XtRImmediate, (XtPointer) True},
    {XtNminWidth, XtCWidth, XtRDimension, sizeof(Dimension),
        offset(mw), XtRImmediate, (XtPointer) 0},
    {XtNminHeight, XtCHeight, XtRDimension, sizeof(Dimension),
        offset(mh), XtRImmediate, (XtPointer) 0},
    {XtNprefWidth, XtCWidth, XtRDimension, sizeof(Dimension),
        offset(pw), XtRImmediate, (XtPointer) 0},
    {XtNprefHeight, XtCHeight, XtRDimension, sizeof(Dimension),
        offset(ph), XtRImmediate, (XtPointer) 0},
    {XtNtableRow, XtCTableRow, XtRWidget, sizeof(Widget),
        offset(row), XtRWidget, (XtPointer) NULL},
    {XtNtableCol, XtCTableCol, XtRWidget, sizeof(Widget),
        offset(col), XtRWidget, (XtPointer) NULL},
};
#undef offset

static void ClassInitialize(), ClassPartInitialize();
static void Initialize(), Destroy(), Resize();
static void ConstraintInitialize();
static Boolean SetValues(), ConstraintSetValues();
static XtGeometryResult GeometryManager(), PreferredGeometry();
static void ChangeManaged(), InsertChild(), DeleteChild();
static void Layout();

static CompositeClassExtensionRec compositeClassExt = {
    /* next_extension	*/	NULL,
    /* record_type	*/	NULLQUARK,
    /* version		*/	XtCompositeExtensionVersion,
    /* record_size	*/	sizeof(CompositeClassExtensionRec),
    /* accepts_objects	*/	TRUE
};

TableClassRec tableClassRec = {
  { /* core_class fields */
    /* superclass         */    (WidgetClass) &constraintClassRec,
    /* class_name         */    "Table",
    /* widget_size        */    sizeof(TableRec),
    /* class_initialize   */    ClassInitialize,
    /* class_part_init    */    ClassPartInitialize,
    /* class_inited       */    FALSE,
    /* initialize         */    Initialize,
    /* initialize_hook    */    NULL,
    /* realize            */    XtInheritRealize,
    /* actions            */    NULL,
    /* num_actions        */    0,
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    TRUE,
    /* compress_exposure  */    TRUE,
    /* compress_enterleave*/    TRUE,
    /* visible_interest   */    FALSE,
    /* destroy            */    Destroy,
    /* resize             */    Resize,
    /* expose             */    NULL,
    /* set_values         */    SetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */    NULL,
    /* accept_focus       */    NULL,
    /* version            */    XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */    PreferredGeometry,
    /* display_accelerator*/    XtInheritDisplayAccelerator,
    /* extension          */    NULL
  },
  { /* composite_class fields */
    /* geometry_manager   */   GeometryManager,
    /* change_managed     */   ChangeManaged,
    /* insert_child       */   InsertChild,
    /* delete_child       */   DeleteChild,
    /* extension          */   (XtPointer) &compositeClassExt
  },
  { /* constraint_class fields */
    /* subresourses       */   tableConstraintResources,
    /* subresource_count  */   XtNumber(tableConstraintResources),
    /* constraint_size    */   sizeof(TableConstraintsRec),
    /* initialize         */   ConstraintInitialize,
    /* destroy            */   NULL,
    /* set_values         */   ConstraintSetValues,
    /* extension          */   NULL
  },
  { /* table_class fields */
    /* layout             */   Layout
  }
};

WidgetClass tableWidgetClass = (WidgetClass)&tableClassRec;

/****************************************************************
 *
 * Private Procedures
 *
 ****************************************************************/


static XrmQuark XtQTopOrLeft, XtQBottomOrRight, XtQCenter, XtQFill, XtQFlush;


/* ARGSUSED */
static Boolean _CvtStringToAlignment( display,
                                      args, nargs, fromVal, toVal,
                                      data )
Display    *display;
XrmValue   *args;
Cardinal   *nargs;
XrmValue   *fromVal;
XrmValue   *toVal;
XtPointer  *data;
{
    XrmQuark q;
    char lowerName[512];

    XmuCopyISOLatin1Lowered ( lowerName, (char*) fromVal->addr );
    q = XrmStringToQuark(lowerName);

    if (toVal->addr != NULL)  {
        toVal->size = sizeof(TableAlign);
        if (q == XtQTopOrLeft)  {
            *((TableAlign *) toVal->addr) = taTopOrLeft;
            return True;
        } else
        if (q == XtQBottomOrRight)  {
            *((TableAlign *) toVal->addr) = taBottomOrRight;
            return True;
        } else
        if (q == XtQCenter)  {
            *((TableAlign *) toVal->addr) = taCenter;
            return True;

        } else
        if (q == XtQFill)  {
            *((TableAlign *) toVal->addr) = taFill;
            return True;
        }
    }

    XtStringConversionWarning( fromVal->addr, "alignment" );
    return False;
}


/* ARGSUSED */
static Boolean _CvtStringToPlacement( display,
                                      args, nargs, fromVal, toVal,
                                      data )
Display   *display;
XrmValue  *args;
Cardinal  *nargs;
XrmValue  *fromVal;
XrmValue  *toVal;
XtPointer *data;
{
    XrmQuark q;
    char lowerName[512];

    XmuCopyISOLatin1Lowered ( lowerName, (char*) fromVal->addr );
    q = XrmStringToQuark(lowerName);

    if (toVal->addr != NULL)  {
        toVal->size = sizeof(TablePlace);
        if (q == XtQTopOrLeft)  {
            *((TablePlace *) toVal->addr) = tpTopOrLeft;
            return True;
        } else
        if (q == XtQBottomOrRight)  {
            *((TablePlace *) toVal->addr) = tpBottomOrRight;
            return True;
        } else
        if (q == XtQCenter)  {
            *((TablePlace *) toVal->addr) = tpCenter;
            return True;

        } else
        if (q == XtQFill)  {
            *((TablePlace *) toVal->addr) = tpFill;
            return True;
        }  else
        if (q == XtQFlush)  {
            *((TablePlace *) toVal->addr) = tpFlush;
            return True;
        }
    }

    XtStringConversionWarning( fromVal->addr, "placement" );
    return False;
}


static void ClassInitialize()
{
    static XtConvertArgRec parentCvtArgs[] = {
        {XtBaseOffset, (caddr_t)XtOffset(Widget, core.parent), sizeof(Widget)}
    };

    XawInitializeWidgetSet();
    XtQTopOrLeft      = XrmStringToQuark("topOrLeft");
    XtQBottomOrRight  = XrmStringToQuark("bottomOrRight");
    XtQCenter         = XrmStringToQuark("center");
    XtQFill           = XrmStringToQuark("fill");
    XtQFlush          = XrmStringToQuark("flush");

    XtSetTypeConverter( XtRString, XtRAlignment, _CvtStringToAlignment,
        NULL, 0, XtCacheNone, NULL );

    XtSetTypeConverter( XtRString, XtRPlacement, _CvtStringToPlacement,
        NULL, 0, XtCacheNone, NULL );

    XtAddConverter( XtRString, XtRWidget, XmuCvtStringToWidget,
        parentCvtArgs, XtNumber(parentCvtArgs) );
}


static void ClassPartInitialize(class)
WidgetClass class;
{
    register TableWidgetClass c = (TableWidgetClass)class;

    if (c->table_class.layout == XtInheritLayout)
        c->table_class.layout =
           ((TableWidgetClass) c->core_class.superclass)->table_class.layout;
}


/* ARGSUSED */
static void Initialize( req, new, arg, nargs )
Widget req, new;
ArgList arg;
Cardinal nargs;
{
    TableWidget tw = (TableWidget) new;
    TablePart *tp;

    tp = &(tw->table);
    tp->no_refigure    = False;
    tp->needs_relayout = False;
    tp->reordered      = True;
    tp->nrows = tp->ncols = 0;
    tp->arows = tp->ncols = 0;
    tp->row = NULL;
    tp->col = NULL;
}


static void ConstraintInitialize( req, new )
Widget req, new;
{
    /*
       If this is a row or column gadget, the constraint part is just
       wasting space, so get rid of it.
    */
    if (XtIsTableRow(new) || XtIsTableCol(new))  {
        XtFree(new->core.constraints);
	new->core.constraints = NULL;
    }
}


static void Destroy( w )
Widget w;
{
    TableWidget tw = (TableWidget) w;
    TablePart *tp;

    tp = &(tw->table);
    
    if (tp->row)  XtFree( tp->row );
    if (tp->col)  XtFree( tp->col );
}


static Cardinal InsertPosition( top, child, nc )
TableObjPart *top;
WidgetList child;
Cardinal nc;
/*
    Return the number _before_ this child, in the list of children.
*/
{
    Cardinal i, n;

    if (top->from == NULL)  return (top->after ? nc : 0);

    for (i = 0;  i < nc;  i++)
        if (child[i] == top->from)  return (top->after ? i+1 : i);

    return (top->after ? nc : 0);
}


static void InsertChild( w )
Widget w;
{
    CompositeWidgetClass superclass;
    TableWidget tw = (TableWidget) w->core.parent;
    TablePart *tp;
    TableRowObject row = (TableRowObject) w;
    TableColObject col = (TableColObject) w;
    TableObjPart *top;
    Cardinal i, ip;

    tp = &(tw->table);

    superclass = (CompositeWidgetClass)
                  tw->core.widget_class->core_class.superclass;

    /* if a row/col, add to row/col list */

    if (XtIsTableRow(w))  {
        top = &(row->table);
        ip = InsertPosition( top, tp->row, tp->nrows );
        if (tp->nrows >= tp->arows)  {
            /* allocate more space */
            tp->arows += (tp->arows / 2) + 2;
            tp->row = (WidgetList) XtRealloc( tp->row,
                                              tp->arows * sizeof(Widget) );
        }
        /* move up those after ip */
        for (i = tp->nrows;  i > ip;  i--)  tp->row[i] = tp->row[i-1];

        tp->row[ip] = w;
        tp->nrows++;
	tp->reordered = True;
    }  else
    if (XtIsTableCol(w))  {
        top = &(col->table);
        ip = InsertPosition( top, tp->col, tp->ncols );
        if (tp->ncols >= tp->acols)  {
            /* allocate more space */
            tp->acols += (tp->acols / 2) + 2;
            tp->col = (WidgetList) XtRealloc( tp->col,
                                              tp->acols * sizeof(Widget) );
        }
        /* move up those after ip */
        for (i = tp->ncols;  i > ip;  i--)  tp->col[i] = tp->col[i-1];

        tp->col[ip] = w;
        tp->ncols++;
	tp->reordered = True;
    }  else  {
        /* this is a cell widget - add to composite.children */

        (*superclass->composite_class.insert_child)( w );
    }
}


static void DeleteChild( w )
Widget w;
{
    CompositeWidgetClass superclass;
    TableWidget tw;  TablePart *tp;
    Cardinal i, dp;

    tw = (TableWidget) w->core.parent;
    tp = &(tw->table);

    superclass = (CompositeWidgetClass)
                  tw->core.widget_class->core_class.superclass;

    /* if a row/col, delete from row/col list */

    if (XtIsTableRow(w))  {
        for (dp = 0;  dp < tp->nrows;  dp++)  {
            if (tp->row[dp] == w)  break;
        }
        if (dp >= tp->nrows)  return;  /* not in list */

        tp->nrows--;
        for (i = dp;  i < tp->nrows;  i++)  tp->row[i] = tp->row[i+1];
	tp->reordered = True;
    }  else
    if (XtIsTableCol(w))  {
        for (dp = 0;  dp < tp->ncols;  dp++)  {
            if (tp->col[dp] == w)  break;
        }
        if (dp >= tp->ncols)  return;  /* not in list */

        tp->ncols--;
        for (i = dp;  i < tp->ncols;  i++)  tp->col[i] = tp->col[i+1];
	tp->reordered = True;
    }  else  {
        /* a cell widget - delete from composite.children */

        (*superclass->composite_class.delete_child)( w );
    }
}


static void RefigureLocations( w )
TableWidget w;
{
    if (w->table.no_refigure) {
        w->table.needs_relayout = True;
    }
    else {
        (*((TableWidgetClass)w->core.widget_class)->table_class.layout)
            ( w );
        w->table.needs_relayout = False;
    }
}

/* ARGSUSED */
static void SetPreferred( tw, resize )
TableWidget tw;
Boolean resize;
{
    TablePart *tp;
    Dimension i, bw;
    Widget child;
    TableConstraints table;  TableConstraintsPart *tcp;
    TableRowObject row;
    TableColObject col;
    TableObjPart *rop, *cop;
    XtWidgetGeometry request, reply;
    Boolean chgW, chgH;

    tp = &(tw->table);

    /*  Initialize row heights and col widths to 0 */
    for (i = 0;  i < tp->nrows;  i++)  {
        row = (TableRowObject) tp->row[i];
        row->table.ph = row->table.mh = 0;
    }
    for (i = 0;  i < tp->ncols;  i++)  {
        col = (TableColObject) tp->col[i];
        col->table.pw = col->table.mw = 0;
    }

    /* Find preferred cell dimensions, update row ph/col pw */

    for (i = 0;  i < tw->composite.num_children;  i++)  {
        child = tw->composite.children[i];
        if (XtIsManaged(child))  {
            table = (TableConstraints) child->core.constraints;
            tcp = &(table->table);

            if (XtIsTableRow(tcp->row) && XtIsTableCol(tcp->col))  {
                row = (TableRowObject) tcp->row;  rop = &(row->table);
                col = (TableColObject) tcp->col;  cop = &(col->table);

		chgH = (tcp->valign ==taFill || rop->stretch);
		chgW = (tcp->halign ==taFill || cop->stretch);
                if (tcp->preference)  {
                    request.request_mode = (CWWidth | CWHeight);
                    request.width = tcp->mw;  request.height = tcp->mh;
                    XtQueryGeometry( child, &request, &reply );
                    tcp->pw = reply.width;  tcp->ph = reply.height;
                    if (!XtIsComposite(child))  {
		        tcp->mw = tcp->pw;  tcp->mh = tcp->ph;
                    }
                }  else  {
                    if (tcp->pw < tcp->mw)  {
                        tcp->pw = (chgW ? tcp->mw :
                                   Max( tcp->mw, child->core.width ));
                    }
                    if (tcp->ph < tcp->mh)  {
                        tcp->ph = (chgH ? tcp->mh :
                                   Max( tcp->mh, child->core.height ));
                    }
                }
                bw = 2 * child->core.border_width;

                if (cop->pw < tcp->pw+bw)  cop->pw = tcp->pw+bw;
                if (cop->mw < tcp->mw+bw)  cop->mw = tcp->mw+bw;

                if (rop->ph < tcp->ph+bw)  rop->ph = tcp->ph+bw;
                if (rop->mh < tcp->mh+bw)  rop->mh = tcp->mh+bw;
            }
        }
    }

    /* Compute table pw/ph by summing row ph/col pw values */

    tp->ph = 2*tp->vmargin;
    for (i = 0;  i < tp->nrows;  i++)  {
        row = (TableRowObject) tp->row[i];  rop = &(row->table);
        tp->ph += rop->ph;
    }
    tp->pw = 2*tp->hmargin;
    for (i = 0;  i < tp->ncols;  i++)  {
        col = (TableColObject) tp->col[i];  cop = &(col->table);
        tp->pw += cop->pw;
    }
    tp->pw += (tp->ncols - 1) * tp->hspacing;
    tp->ph += (tp->nrows - 1) * tp->vspacing;
 
    if (resize)  {
        XtMakeResizeRequest( tw, tp->pw, tp->ph,  NULL, NULL );
    }
}


static void Resize( w )
Widget w;
{
    TableWidget tw = (TableWidget) w;
    TablePart *tp;
    Widget child;
    TableConstraints table;  TableConstraintsPart *tcp;
    TableRowObject row;
    TableColObject col;
    TableObjPart *rop, *cop;
    XtWidgetGeometry request, reply;
    int i, sr, sc, ew,eh, xw,xh;
    Dimension minw,minh, hm,vm, hs,vs;
    Dimension sw,sh, fw,fh;
    Position x,y, ox,oy;
    Dimension cw,ch, ow,oh, bw;

    tp = &(tw->table);

    /*
        Initialize row/col width or height, and count stretches
    */
    minh = 2*tp->vmargin + (tp->nrows-1)*tp->vspacing;
    minw = 2*tp->hmargin + (tp->ncols-1)*tp->hspacing;

    sr = 0;
    for (i = 0;  i < tp->nrows;  i++)  {
        row = (TableRowObject) tp->row[i];  rop = &(row->table);
        if (rop->stretch)  sr++;
        row->rectangle.height = (rop->stretch ? rop->mh : rop->ph);
	minh += row->rectangle.height;
    }
    sc = 0;
    for (i = 0;  i < tp->ncols;  i++)  {
        col = (TableColObject) tp->col[i];  cop = &(col->table);
        if (cop->stretch)  sc++;
        col->rectangle.width = (cop->stretch ? cop->mw : cop->pw);
	minw += col->rectangle.width;
    }

    hm = tp->hmargin;   vm = tp->vmargin;
    hs = tp->hspacing;  vs = tp->vspacing;
    /*
        Compute and distribute any extra size
    */
    eh = tw->core.height - minh;  xh = 0;
    if (eh > 0)  {
        if (sr > 0)  {
            sh = eh / sr;  xh = eh - (sh * sr);
            for (i = 0;  i < tp->nrows;  i++)  {
                row = (TableRowObject) tp->row[i];
                if (row->table.stretch)  {
                    row->rectangle.height += sh + xh;
                    xh = 0;
                }
            }
        }  else  {  /* distribute by placement */
            switch (tp->vplace)  {
                case tpBottomOrRight:
                    vm += eh;  break;
                case tpCenter:
                    vm += (eh+1) / 2;  break;
                case tpFill:
                    fh = (eh / (tp->nrows+1));
                    xh = eh - (fh * (tp->nrows+1));
                    vm += fh;  vs += fh;
                    break;
                case tpFlush:
                    fh = (eh / (tp->nrows-1));
                    xh = eh - (fh * (tp->nrows-1));
                    vs += fh;
                    break;
                case tpTopOrLeft:
                    /* no changes */ ;
            }
        }
    }
    ew = tw->core.width - minw;  xw = 0;
    if (ew > 0)  {
        if (sc > 0)  {
            sw = ew / sc;  xw = ew - (sw * sc);
            for (i = 0;  i < tp->ncols;  i++)  {
                col = (TableColObject) tp->col[i];
                if (col->table.stretch)  {
                    col->rectangle.width += sw + xw;
                    xw = 0;
                }
            }
        }  else  {  /* distribute by placement */
            switch (tp->hplace)  {
                case tpBottomOrRight:
                    hm += ew;  break;
                case tpCenter:
                    hm += (ew+1) / 2;  break;
                case tpFill:
                    fw = (ew / (tp->ncols+1));
                    xw = ew - (fw * (tp->ncols+1));
                    hm += fw;  hs += fw;
                    break;
                case tpFlush:
                    fw = (ew / (tp->ncols-1));
                    xw = ew - (fw * (tp->ncols-1));
                    hs += fw;
                    break;
                case tpTopOrLeft:
                    /* no changes */ ;
            }
        }
    }

    /*
        Place rows and columns based on computed sizes
    */
    y = vm;
    for (i = 0;  i < tp->nrows;  i++)  {
        row = (TableRowObject) tp->row[i];
        row->rectangle.y = y;
        y += row->rectangle.height + vs + xh;
        xh = 0;
    }
    x = hm;
    for (i = 0;  i < tp->ncols;  i++)  {
        col = (TableColObject) tp->col[i];
        col->rectangle.x = x;
        x += col->rectangle.width + hs + xw;
        xw = 0;
    }

    /*
        Size and place managed cell widgets by row/col
    */
    for (i = 0;  i < tw->composite.num_children;  i++)  {
        child = tw->composite.children[i];
        if (XtIsManaged(child))  {
            table = (TableConstraints) child->core.constraints;
            tcp = &(table->table);
            bw = child->core.border_width;

            if (XtIsTableRow(tcp->row) && XtIsTableCol(tcp->col))  {
                row = (TableRowObject) tcp->row;
                col = (TableColObject) tcp->col;
                ox = col->rectangle.x;  ow = col->rectangle.width;
                oy = row->rectangle.y;  oh = row->rectangle.height;

                switch (tcp->valign)  {
                    case taTopOrLeft:
                        y = oy;  ch = tcp->ph;
                        break;
                    case taBottomOrRight:
                        y = oy + (oh - tcp->ph - 2*bw);  ch = tcp->ph;
                        break;
                    case taCenter:
                        y = oy + (oh - tcp->ph - 2*bw + 1) / 2;
                        ch = tcp->ph;
                        break;
                    case taFill:
                        y = oy;  ch = oh - 2*bw;
                        break;
                }

                switch (tcp->halign)  {
                    case taTopOrLeft:
                        x = ox;  cw = tcp->pw;
                        break;
                    case taBottomOrRight:
                        x = ox + (ow - tcp->pw - 2*bw);  cw = tcp->pw;
                        break;
                    case taCenter:
                        x = ox + (ow - tcp->pw - 2*bw + 1) / 2;
                        cw = tcp->pw;
                        break;
                    case taFill:
                        x = ox;  cw = ow - 2*bw;
                        break;
                }

                /* the following will force a configuration change */
		child->core.x = child->core.y = -1;
                child->core.width = child->core.height = 0;

                XtConfigureWidget( child, x, y, cw, ch, bw );

                if (!tcp->preference)  {
                    tcp->pw = child->core.width;
                    tcp->ph = child->core.height;
                }
            }
        }
    }
}


static void Layout( tw )
TableWidget tw;
{
    SetPreferred( tw, True );  Resize( (Widget) tw );
}


/* ARGSUSED */
static XtGeometryResult GeometryManager( w, request, reply )
Widget w;
XtWidgetGeometry *request;
XtWidgetGeometry *reply;        /* RETURN */
{
    TableConstraints table = (TableConstraints) w->core.constraints;
    Dimension width, height, border_width;

    if (request->request_mode &
        ~(XtCWQueryOnly | CWWidth | CWHeight | CWBorderWidth))
        return XtGeometryNo;

    if (request->request_mode & CWWidth)
        width = request->width;
    else
        width = w->core.width;

    if (request->request_mode & CWHeight)
        height = request->height;
    else
        height = w->core.height;

    if (request->request_mode & CWBorderWidth)
        border_width = request->border_width;
    else
        border_width = w->core.border_width;

    if (width == w->core.width &&
        height == w->core.height &&
        border_width == w->core.border_width)
        return XtGeometryNo;

    if (!(request->request_mode & XtCWQueryOnly)) {
        w->core.width  = width;
        w->core.height = height;
        w->core.border_width = border_width;
        RefigureLocations( (TableWidget) XtParent(w) );
    }
    return XtGeometryYes;
}


/* ARGSUSED */
static Boolean SetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    return( FALSE );
}


/* ARGSUSED */
static Boolean ConstraintSetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    TableConstraints Table = (TableConstraints) new->core.constraints;
    TableWidget tw = (TableWidget) new->core.parent;
    TablePart *tp;
    TableRowObject row = (TableRowObject) new;
    TableColObject col = (TableColObject) new;
    TableObjPart *top;
    Boolean chgdFrom, chgdAfter;
    char *argname;
    int i;  Cardinal n, ip;
    WidgetList obj;
    
    tp = &(tw->table);

    chgdFrom = chgdAfter = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
	if (streq( argname, XtNinsertAfter ))  chgdAfter = True;
	if (streq( argname, XtNfromSibling ))  chgdFrom  = True;
    }

    if (!(chgdFrom || chgdAfter) &&
	!XtIsTableRow(new) && !XtIsTableCol(new))  return (FALSE);

    tp->reordered = True;

    if (XtIsTableRow(new))  {
        top = &(row->table);  obj = tp->row;  n = tp->nrows;
    }  else  {
        top = &(col->table);  obj = tp->col;  n = tp->ncols;
    }

    for (ip = 0;  ip < n;  ip++)
        if (obj[ip] == new)  break;

    n--;
    for (i = ip;  i < n;  i++)  obj[i] = obj[i+1];

    ip = InsertPosition( top, obj, n );
    for (i = n;  i > ip;  i--)  obj[i] = obj[i-1];
    obj[ip] = new;

    RefigureLocations(tw);

    return (TRUE);
}


static void ChangeManaged(w)
Widget w;
{
    TableWidget tw = (TableWidget) w;

    RefigureLocations( tw );
}


static XtGeometryResult PreferredGeometry( w, request, reply  )
Widget w;
XtWidgetGeometry *request, *reply;
{
    TableWidget tw = (TableWidget) w;

    SetPreferred( tw, False );

    reply->width = tw->table.pw;
    reply->height = tw->table.ph;
    reply->request_mode = CWWidth | CWHeight;
    if (request->request_mode & (CWWidth | CWHeight) ==
          reply->request_mode & CWWidth | CWHeight &&
          request->width == reply->width &&
          request->height == reply->height)
        return XtGeometryYes;
    else
    if (reply->width == tw->core.width && reply->height == tw->core.height)
        return XtGeometryNo;
    else
        return XtGeometryAlmost;
}


/**********************************************************************
 *
 * Public routines
 *
 **********************************************************************/

/* 
 * Set or reset figuring (ignored if not realized)
 */

void TableDoLayout( w, doit )
Widget w;
Boolean doit;
{
    register TableWidget tw = (TableWidget) w;

    tw->table.no_refigure = !doit;

    if (XtIsRealized(w) && tw->table.needs_relayout)
        RefigureLocations( tw );
}


/**********************************************************************
 *
 * TableRow and TableCol definitions
 *
 **********************************************************************/


#define offset(field) XtOffset( TableObject, table.field )
static XtResource rowResources[] = {
    {XtNstretch, XtCStretch, XtRBoolean, sizeof(Boolean),
        offset(stretch), XtRImmediate, (XtPointer) False},
    {XtNinsertAfter, XtCBoolean, XtRBoolean, sizeof(Boolean),
        offset(after), XtRImmediate, (XtPointer) True},
    {XtNfromSibling, XtCWidget, XtRWidget, sizeof(Widget),
        offset(from), XtRWidget, (XtPointer) NULL}
};
#undef offset

static void ObjectClassInitialize(), ObjectInitialize();
static XtGeometryResult ObjectPreferredGeometry();
static Boolean ObjectSetValues();
static void ObjectGetValuesHook();

TableObjClassRec tableRowClassRec = {
  {
    /* superclass         */    (WidgetClass) &rectObjClassRec,
    /* class_name         */    "TableRow",
    /* size               */    sizeof(TableObjRec),
    /* class_initialize   */    ObjectClassInitialize,
    /* class_part_initialize*/  NULL,
    /* Class init'ed      */    FALSE,
    /* initialize         */    NULL,
    /* initialize_hook    */    NULL,
    /* realize            */    NULL,
    /* actions            */    NULL,
    /* num_actions        */    0,
    /* resources          */    rowResources,
    /* resource_count     */    XtNumber(rowResources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    FALSE, 
    /* compress_exposure  */    FALSE,
    /* compress_enterleave*/    FALSE,
    /* visible_interest   */    FALSE,
    /* destroy            */    NULL,
    /* resize             */    NULL,
    /* expose             */    NULL,
    /* set_values         */    ObjectSetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,  
    /* get_values_hook    */    ObjectGetValuesHook,
    /* accept_focus       */    NULL,
    /* intrinsics version */    XtVersion,
    /* callback offsets   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */    ObjectPreferredGeometry,
    /* display_accelerator*/    NULL,
    /* extension          */    NULL
  },{
    /* table object class Fields */
    /* empty              */    NULL                            
  }
};

WidgetClass tableRowObjectClass = (WidgetClass) &tableRowClassRec;

#define offset(field) XtOffset( TableObject, table.field )
static XtResource colResources[] = {
    {XtNstretch, XtCStretch, XtRBoolean, sizeof(Boolean),
        offset(stretch), XtRImmediate, (XtPointer) False},
    {XtNinsertAfter, XtCBoolean, XtRBoolean, sizeof(Boolean),
        offset(after), XtRImmediate, (XtPointer) True},
    {XtNfromSibling, XtCWidget, XtRWidget, sizeof(Widget),
        offset(from), XtRWidget, (XtPointer) NULL}
};
#undef offset

TableObjClassRec tableColClassRec = {
  {
    /* superclass         */    (WidgetClass) &rectObjClassRec,
    /* class_name         */    "TableCol",
    /* size               */    sizeof(TableObjRec),
    /* class_initialize   */    ObjectClassInitialize,
    /* class_part_initialize*/  NULL,
    /* Class init'ed      */    FALSE,
    /* initialize         */    NULL,
    /* initialize_hook    */    NULL,
    /* realize            */    NULL,
    /* actions            */    NULL,
    /* num_actions        */    0,
    /* resources          */    colResources,
    /* resource_count     */    XtNumber(colResources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    FALSE, 
    /* compress_exposure  */    FALSE,
    /* compress_enterleave*/    FALSE,
    /* visible_interest   */    FALSE,
    /* destroy            */    NULL,
    /* resize             */    NULL,
    /* expose             */    NULL,
    /* set_values         */    ObjectSetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,  
    /* get_values_hook    */    ObjectGetValuesHook,
    /* accept_focus       */    NULL,
    /* intrinsics version */    XtVersion,
    /* callback offsets   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */    ObjectPreferredGeometry,
    /* display_accelerator*/    NULL,
    /* extension          */    NULL
  },{
    /* table object class Fields */
    /* empty              */    NULL                            
  }
};

WidgetClass tableColObjectClass = (WidgetClass) &tableColClassRec;


static void ObjectClassInitialize()
{
    static XtConvertArgRec parentCvtArgs[] = {
        {XtBaseOffset, (caddr_t)XtOffset(Widget, core.parent), sizeof(Widget)}
    };

    XawInitializeWidgetSet();

    XtAddConverter( XtRString, XtRWidget, XmuCvtStringToWidget,
        parentCvtArgs, XtNumber(parentCvtArgs) );
}


/* ARGSUSED */
static Boolean ObjectSetValues( old, req, new, arg, nargs )
Widget old, req, new;
ArgList arg;
Cardinal *nargs;
{
    TableWidget tw = (TableWidget) new->core.parent;
    TablePart *tp;
    TableRowObject row = (TableRowObject) new;
    TableColObject col = (TableColObject) new;
    TableObjPart *top;
    Boolean chgdFrom, chgdAfter;
    char *argname;
    int i;  Cardinal n, ip;
    WidgetList obj;
    
    tp = &(tw->table);

    chgdFrom = chgdAfter = False;
    for (i = 0;  i < *nargs;  i++)  {
        argname = arg[i].name;
        if (streq( argname, XtNinsertAfter ))  chgdAfter = True;
        if (streq( argname, XtNfromSibling ))  chgdFrom  = True;
    }

    if (!(chgdFrom || chgdAfter))  return (FALSE);

    if (XtIsTableRow(new))  {
        n = tp->nrows;  obj = tp->row;  top = &(row->table);
    }  else  {
        n = tp->ncols;  obj = tp->col;  top = &(col->table);
    }
    for (ip = 0;  ip < n;  ip++)
        if (obj[i] == new)  break;

    n--;
    for (i = ip;  i < n;  i++)  obj[i] = obj[i+1];

    ip = InsertPosition( top, obj, n );
    for (i = n;  i > ip;  i--)  obj[i] = obj[i-1];

    obj[ip] = new;

    RefigureLocations(tw);

    return (False);
}


/* ARGSUSED */
static void ObjectGetValuesHook( w, arg, nargs )
Widget w;
ArgList arg;
Cardinal *nargs;
{
    TableWidget tw = (TableWidget) w->core.parent;
    TablePart *tp;
    TableObjPart top;
    Cardinal i, n;
    TableObject *obj;
    XtResourceList resources;

    tp = &(tw->table);

    /*
        The reason for this hook is to set values for after and from, if
	necessary.  The reordered flag is set if the values may be
	wrong or not set for all children.

	A canonical form is used for all children, namely, after is True
	and from is the previous child in the list.
    */
    if (tp->reordered)  {
        obj = (TableObject *) tp->row;  n = tp->nrows;
	for (i = 0;  i < n;  i++)  {
	    top = obj[i]->table;
	    top.after = True;
	    top.from = (i > 0 ? (Widget) obj[i-1] : NULL);
	}
        obj = (TableObject *) tp->col;  n = tp->ncols;
	for (i = 0;  i < n;  i++)  {
	    top = obj[i]->table;
	    top.after = True;
	    top.from = (i > 0 ? (Widget) obj[i-1] : NULL);
	}
	tp->reordered = False;
    }

    resources = (XtIsTableRow(w) ? rowResources : colResources);
    XtGetSubvalues( w, resources, XtNumber(resources), arg, *nargs );
}


static XtGeometryResult ObjectPreferredGeometry( w, request, reply ) 
Widget w;
XtWidgetGeometry *request, *reply;
{
    TableObject obj = (TableObject) w;
    TableWidget tw = (TableWidget) w->core.parent;
    TablePart *tp;

    tp = &(tw->table);

    if (XtIsTableRow(w))  {
        reply->height = obj->table.ph;  reply->width = tp->pw;
    }  else  {
        reply->width = obj->table.pw;  reply->height = tp->ph;
    }
    reply->request_mode = CWWidth | CWHeight;
    if (request->request_mode & (CWWidth | CWHeight) ==
          reply->request_mode & CWWidth | CWHeight &&
          request->width == reply->width &&
          request->height == reply->height)
        return XtGeometryYes;
    else
    if (reply->width == obj->rectangle.width &&
        reply->height == obj->rectangle.height)
        return XtGeometryNo;
    else
        return XtGeometryAlmost;
}
