#ifndef _Table_h
#define _Table_h

#include <X11/Constraint.h>
#include <X11/Shell.h>            /* XtNminWidth, XtNminHeight */
#include "Group.h"                /* class resource names */

/***********************************************************************
 *
 * Table Widget, TableRow and TableCol Gadgets
 *
 ***********************************************************************/

/* Resources:

Table:

 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 borderWidth         BorderWidth        Dimension       0
 horzSpacing         Thickness          Dimension       0
 vertSpacing         Thickness          Dimension       0
 horzMargin          Margin             Dimension       0
 vertMargin          Margin             Dimension       0
 horzPlacement       Placement          Placement       tpFill (1,2)
 vertPlacement       Placement          Placement       tpTopOrLeft (1,2)

(Cell Widget) Constraints:

 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 horzAlignment       Alignment          Alignment       TopOrLeft (2)
 vertAlignment       Alignment          Alignment       TopOrLeft (2)
 preference          Preference         Boolean         True
 minWidth            Width              Dimension       0 (3)
 minHeight           Height             Dimension       0 (3)
 prefWidth           Width              Dimension       core.width  (4)
 prefHeight          Height             Dimension       core.height (4)
 tableRow            TableRow           Widget          NULL (6)
 tableCol            TableCol           Widget          NULL (6)

Rows/Columns:
 Name                Class              RepType         Default Value
 ----                -----              -------         -------------
 stretch             Stretch            Boolean         False
 insertAfter         Boolean            Boolean         True (5)
 fromSibling         Widget             Widget          NULL (5)

Note (1):
   This resource is only used when no row/cols are being stretched.
Note (2):
   From a resource file, use the enum names (below) without the "tp" or
  "ta" prefix. From a call, use the full enum names.
Note (3):
   These are used only if preference is False or the child is a composite
   widget (a subclass of Composite).  If a non-composite widget has
   preference True, its preferred and minimum sizes are taken to be the
   same, as determined by querying the widget.
Note (4):
   These are used only if preference is False.  If preference is True, the
   widget is queried for width and height.
   When used, they determine only the _initial_ size of the widget.  If the
   user resizes, the new values become the preferred values.
Note (5):
   If insertAfter and fromSibling is NULL, the object will be inserted as
   the last child.  If !insertAfter and fromSibling is NULL, the object will
   be inserted before the first child.
   Of course, fromSibling must be of the same type; i.e., a row can only be
   from another row, and a column from another column.
Note (6):
   These are _required_ resources; results are unpredictable (at the moment)
   if they are not properlly defined.
   Although the type is Widget for convenience, tableRow must be of class
   tableRowObjectClass (or a subclass of it), and tableCol must be of class
   TableColObjectClass.
*/


#define XtNhorzSpacing     "horzSpacing"
#define XtNvertSpacing     "vertSpacing"
#define XtNhorzPlacement   "horzPlacement"
#define XtNvertPlacement   "vertPlacement"
#define XtNhorzAlignment   "horzAlignment"
#define XtNvertAlignment   "vertAlignment"
#define XtNhorzMargin      "horzMargin"
#define XtNvertMargin      "vertMargin"
#define XtNtableRow        "tableRow"
#define XtNtableCol        "tableCol"

#define XtCTableRow        "TableRow"
#define XtCTableCol        "TableCol"

/*  The following are all in Group.h:
#define XtNstretch        "stretch"
#define XtNpreference     "preference"
#define XtNprefWidth      "prefWidth"
#define XtNprefHeight     "prefHeight"
#define XtNinsertAfter    "insertAfter"
#define XtNfromSibling    "fromSibling"

#define XtCPlacement      "Placement"
#define XtCAlignment      "Alignment"
#define XtCStretch        "Stretch"
#define XtCPreference     "Preference"
*/

typedef enum {
    tpTopOrLeft,      /* place children at top (horz) or left (vert)  */
    tpBottomOrRight,  /* place children at bottom or right */
    tpCenter,         /* put extra space before first and after last  */
    tpFill,           /* space uniformly around children */
    tpFlush           /* space uniformly between children, no extra
                         before first or after last */
} TablePlace;

typedef enum {
    taTopOrLeft,      /* align to top (horz) or left (vert)  */
    taBottomOrRight,  /* align to bottom or right */
    taCenter,         /* center  */
    taFill            /* resize to fill between margins */
} TableAlign;

typedef struct _TableClassRec   *TableWidgetClass;
typedef struct _TableRec        *TableWidget;

extern WidgetClass tableWidgetClass;

typedef struct _TableObjClassRec   *TableRowObjectClass,
                                   *TableColObjectClass;
typedef struct _TableObjRec        *TableRowObject,
                                   *TableColObject;

extern WidgetClass tableRowObjectClass, tableColObjectClass;

extern void TableDoLayout();    /* widget, doit */
    /* Widget widget */
    /* Boolean doit */
 
#define XtIsTableRow(w)    XtIsSubclass( (w), tableRowObjectClass )
#define XtIsTableCol(w)    XtIsSubclass( (w), tableColObjectClass )

#endif /* _Table_h */
