#ifndef _TableP_h
#define _TableP_h

#include <X11/ConstrainP.h>     /* superclass of Table */
#include <X11/RectObjP.h>       /* superclass of TableRow/TableCol */

#include "Table.h"

#define XtRAlignment "Alignment"
#define XtRPlacement "Placement"

#define XtInheritLayout ((void (*)())_XtInherit)

/*
    Table structure definitions
*/

typedef struct {
    void (*layout)(/* TableWidget */);
} TableClassPart;

typedef struct _TableClassRec {
    CoreClassPart       core_class;
    CompositeClassPart  composite_class;
    ConstraintClassPart constraint_class;
    TableClassPart      table_class;
} TableClassRec;

extern TableClassRec tableClassRec;

typedef struct _TablePart {
    /* resources */
    Dimension  hspacing,
               vspacing;         /* default distance between children  */
    Dimension  hmargin,
               vmargin;          /* default distance at edges          */
    TablePlace hplace,
               vplace;           /* placement for unstretched children */
    /* private */
    int        no_refigure;      /* no re-layout while > 0             */
    Boolean    needs_relayout;   /* next time no_refigure == 0         */
    Dimension  pw, ph;           /* preferred width/height             */
    Boolean    reordered;        /* have rows/cols changed order?      */

    Cardinal   nrows, ncols;     /* number of rows and columns         */
    Cardinal   arows, acols;     /* total number allocated             */
    WidgetList row;              /* row gadgets                        */
    WidgetList col;              /* column gadgets                     */
} TablePart;

typedef struct _TableRec {
    CorePart            core;
    CompositePart       composite;
    ConstraintPart      constraint;
    TablePart           table;
} TableRec;

typedef struct _TableConstraintsPart {
    /* resources */
    TableAlign  halign,
                valign;         /* alignment for this child widget        */
    Boolean     preference;     /* query child for preferred geometry     */
    Dimension   mw, mh;         /* minimum width & height                 */
    Dimension   pw, ph;         /* preferred width & height               */
    Widget      row, col;       /* row/col objects (required)             */
} TableConstraintsPart;

typedef struct _TableConstraintsRec {
    TableConstraintsPart        table;
}
TableConstraintsRec, *TableConstraints;


/*
    TableRow, TableCol structure definitions
*/
typedef struct {
    XtPointer empty;       /* nothing for now */
} TableObjClassPart, TableRowClassPart, TableColClassPart;

typedef struct _TableObjClassRec {
    RectObjClassPart  rect_class;
    TableObjClassPart table_class;
} TableObjClassRec;

extern TableObjClassRec tableRowClassRec, tableColClassRec;

typedef struct _TableObjPart {
    /* resource */
    Boolean     stretch;        /* will this row/col absorb extra space?   */
    Boolean     after;          /* insert new child after?                 */
    Widget      from;           /* sibling to insert from                  */
    /* private */
    Dimension   pw, ph;         /* preferred width/height (only 1 is used) */
    Dimension   mw, mh;         /* minimum width/height          ''        */
} TableObjPart;

typedef struct _TableObjRec {
    ObjectPart          object;
    RectObjPart         rectangle;
    TableObjPart        table;
} TableObjRec, *TableObject;

#endif /* _TableP_h */



