
#include <stdio.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xlibint.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xresource.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/XawInit.h>
#include <X11/cursorfont.h>

#include "WaitP.h"
#include "clkfaces.h"

#define offset(field) XtOffset( WaitWidget, field )
/*
    {name, class, type, size,
        offset, default_type, default_addr},
*/
static XtResource resources[] = {
    {XtNmappedWhenManaged, XtCMappedWhenManaged, XtRBoolean, sizeof(Boolean),
         offset(core.mapped_when_managed), XtRImmediate, (XtPointer) False}
};
#undef offset

static void Realize(), Initialize();

WaitClassRec waitClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"Wait",
    /* widget_size		*/	sizeof(WaitRec),
    /* class_initialize		*/	XawInitializeWidgetSet,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	NULL,
    /* resize			*/	NULL,
    /* expose			*/	NULL,
    /* set_values		*/	NULL,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* wait fields */
    /* clockFace                */      NULL
  }
};

WidgetClass waitWidgetClass = (WidgetClass)&waitClassRec;


static XColor   black = { 0,      0,      0,      0 },
                white = { 0, 0xffff, 0xffff, 0xffff };

Cursor waitCursor;

static void InitializeCursors( ww, wc )
WaitWidget ww;  WaitWidgetClass wc;
{
    Display *dpy;
    Window   win;
    Pixmap   src, msk;
    XImage   ximage;
    GC       gc;
    int i;
    Cursor  *clockFace;
   
    clockFace = (Cursor *) XtMalloc( 12 * sizeof(Cursor) );

    dpy = XtDisplay(ww);
    win = DefaultRootWindow(dpy);

    waitCursor = XCreateFontCursor( dpy, XC_watch );

    src = XCreatePixmap( dpy, win, clock_width, clock_height, 1 );
    msk = XCreatePixmap( dpy, win, clock_width, clock_height, 1 );
    gc = XCreateGC( dpy, src, 0L, (XGCValues *) 0 );

    ximage.height = clock_height;
    ximage.width = clock_width;
    ximage.depth = 1;
    ximage.xoffset = 0;
    ximage.format = ZPixmap;
    ximage.byte_order = LSBFirst;
    ximage.bitmap_unit = 8;
    ximage.bitmap_bit_order = LSBFirst;
    ximage.bitmap_pad = 8;
    ximage.bytes_per_line = (clock_width+7)/8; 
 
    for (i = 0;  i < 12;  i++)  {
        ximage.data = clock_source_bits[i];
        XPutImage( dpy, src, gc, &ximage,
                   0,0,0,0, clock_width, clock_height );
        ximage.data = clock_mask_bits[i];
        XPutImage( dpy, msk, gc, &ximage,
                   0,0,0,0, clock_width, clock_height );
/*
        clockFace[i] = XCreatePixmapCursor( dpy, src, msk,
                                            &black, &white,
                                            clock_x_hot, clock_y_hot );
*/
/*
    since Timer events are queued (synchronous), the above is pointless,
    so just use the watch cursor
*/
        clockFace[i] = waitCursor;
    }
    wc->wait_class.clockFace = clockFace;
    
    XFreeGC( dpy, gc );
    XFreePixmap( dpy, src );
    XFreePixmap( dpy, msk );
}

static void Initialize( req, new )
Widget req, new;
{
    WaitWidget ww = (WaitWidget) new;
    WaitPart *wp;  WaitWidgetClass wc;

    wp = &(ww->wait);  wp->waiting = FALSE;  wp->count = 0;

    wc = (WaitWidgetClass) XtClass(new);
    if (wc->wait_class.clockFace == NULL)  InitializeCursors( ww, wc );
}
    
static void Realize( w, valueMask, attributes )
Widget w;
XtValueMask *valueMask;
XSetWindowAttributes *attributes;
{
    Widget parent;  WaitWidgetClass wc;

    parent = XtParent(w);  wc = (WaitWidgetClass) XtClass(w);

    w->core.width = parent->core.width;   /* not used */
    w->core.height = parent->core.height;
    w->core.x = w->core.y = 0;
    w->core.depth = parent->core.depth;
    w->core.border_width = 0;

    attributes->cursor = wc->wait_class.clockFace[0];

    attributes->override_redirect = True;
    *valueMask = CWCursor | CWOverrideRedirect;

    XtCreateWindow( w, InputOutput, CopyFromParent, *valueMask, attributes );
}


static void Tick( w )
WaitWidget w;
{
    WaitPart *wp;  WaitWidgetClass wc;
    Widget parent;
 
    wp = &(w->wait);  wc = (WaitWidgetClass) XtClass(w);

    parent = XtParent(w);
    if (w->core.width != parent->core.width ||
        w->core.height != parent->core.height)  {
        XtResizeWidget( w, parent->core.width, parent->core.height, 0 );
        if (wp->waiting)  XMapRaised( XtDisplay(w), XtWindow(w) );
    }
    if (wp->waiting)  {
        wp->count = (++(wp->count)) % 12;
        XDefineCursor( XtDisplay(w), XtWindow(w),
                       wc->wait_class.clockFace[wp->count] );

        wp->timer = XtAppAddTimeOut( XtWidgetToApplicationContext(w),
                        1000, Tick, w );
    }
}

void StartWaiting( w )
WaitWidget w;
{
    WaitPart *wp;  WaitWidgetClass wc;

    wp = &(w->wait);  wc = (WaitWidgetClass) XtClass(w);

    wp->waiting = True;  wp->count = 0;
    XDefineCursor( XtDisplay(w), XtWindow(w),
                   wc->wait_class.clockFace[0] );
    XMapRaised( XtDisplay(w), XtWindow(w) );  XFlush( XtDisplay(w) );

    wp->timer = XtAppAddTimeOut( XtWidgetToApplicationContext(w),
                    1000, Tick, w );
}


void EndWaiting( w )
WaitWidget w;
{
    WaitPart *wp;

    wp = &(w->wait);  wp->waiting = False;    

    XUnmapWindow( XtDisplay(w), XtWindow(w) );  XFlush( XtDisplay(w) );
}


