/* CONTENTS
   Owner:       Copyright 1990 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: helpmenu.c,v $
   Purpose:     
       Create the choices for help information and the functions
       necessary to create a shell to display the information.  Within
       that shell also offer the ability to print out the text.

   Programmer:  Con O'Connell
   Reviewer:    
   Revision:    $Revision: 1.1 $
   Date:        $Date: 92/03/18 13:47:03 $

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------
   HIDDEN  HelpPrint

   HIDDEN  HelpDone

   HIDDEN  Help

   EXPORT  XicaHelpMenu
*/

/* HISTORY
 *  $Log:	helpmenu.c,v $
 * Revision 1.1  92/03/18  13:47:03  ware
 * Initial revision
 * 
 */

/* #INCLUDES */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include "help.h"

extern char *getenv ();



/*
   FUNCTION: 		HelpPrint

   Synopsis: 
       Print out the contents (help file) displayed currently by the shell.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.


   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Operating system:
       system      Used to print the help file displayed in the help shell.

   Return value:	void

   Range:		none
*/

static void
HelpPrint(w, filename, call_data)

Widget
   w;         /* Unused */
String
   filename;  /* The help file currently displayed */
caddr_t
   call_data; /* Unused */

{
   String
      sysstr; /* Contains the system string to be executed */

   /*
    * Allocate space for the system string.
    * Create the system string to be executed.
    * Execute the system call.
    */
#ifdef DEBUG
   fprintf(stderr, "Printing help file [%s]\n", filename);
#endif
   sysstr = XtMalloc((unsigned) strlen(filename) + 32);
   sprintf(sysstr, "/usr/ucb/lpr %s", filename);
   system(sysstr);
   XtFree (sysstr);
   return;
}




/*
   FUNCTION: 		HelpDone

   Synopsis: 
       The user is done with this shell, so destroy it.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.


   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static void
HelpDone(w, client_data, call_data)

Widget
   w;           /* Used to access the shell */
caddr_t
   client_data, /* Unused */
   call_data;   /* Unused */

{
   /*
    * Destroy the help shell (parent of the parent of w)
    */
   XtDestroyWidget(XtParent(XtParent(w)));
   return;
}




/*
   FUNCTION: 		HelpGetFilename

   Synopsis: 
       Return the a filename to do this help.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.


   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       91-05-23   cso    Added the use of ICAROOT.
       6 Mar 92	  pete	 Modified from devegram's CreateHelpMenu().

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

static String
HelpGetFilename (parent, file)
Widget	parent;				/* The parent of the help shell */
String	file;				/* name of the help file */
{
	String	filename;		/* full path to the file */
	String	icaroot;		/* where to start looking */
   

   /*
    * FIX: Use a #define to fill in a default value.  Better yet, use
    *	   XtFindFile() to look it up.
    */
   if (!(icaroot = getenv ("ICAROOT")))
	   icaroot = ".";
   filename = XtMalloc(strlen(icaroot) + strlen("/help/") + strlen (file) + 1);
   sprintf (filename, "%s/help/%s", icaroot, file);

   return (filename);
}



/*
   FUNCTION: 		Help

   Synopsis: 
       Create the shell to display a specific help file.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.


   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE       WHO    REASON
       6 Mar 92	  pete	 Moved into Xica so it is globally available.

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
XicaHelp(w, helpfile, call_data)

Widget
   w;         /* Unused */
String
   helpfile;  /* The file to be displayed */
caddr_t
   call_data; /* Unused */

{
   int
      cnt;      /* Number of arguments */
   Arg
      args[10];/* List of arguments */
   Widget
      helpdis,  /* The display widget */
      helpdone, /* The done widget */
      helpframe,/* The frame containing command and text widgets */
      helpprint,/* The print widget */
      shell,	/* The shell parent */
      helpshell;/* The shell */
   String
	   filename;			/* how to find the help file */

   /*
    * -----------------------------------------
    * |  shell(shell)                         |
    * | ------------------------------------- |
    * | |  frame(form)                      | |
    * | |                                   | |
    * | | [print(command)]  [done(command)] | |
    * | |                                   | |
    * | | [display(text)]                   | |
    * | |                                   | |
    * | ------------------------------------- |
    * -----------------------------------------
    */
   for (shell = w; !XtIsShell (shell); shell = XtParent (shell))
	   /* empty */;
   cnt = 0;
   XtSetArg(args[cnt], XtNallowShellResize, True); cnt++;
   helpshell  = XtCreatePopupShell ("Help_Display", topLevelShellWidgetClass,
				    shell, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNresizable, True); cnt++;
   helpframe = XtCreateManagedWidget("helpframe", formWidgetClass,
				     helpshell, args, cnt);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromHoriz, NULL);    cnt++;
   XtSetArg(args[cnt], XtNfromVert,  NULL);    cnt++;
   XtSetArg(args[cnt], XtNlabel,     "Print"); cnt++;
   helpprint = XtCreateManagedWidget("helpprint", commandWidgetClass,
				     helpframe, args, cnt);
   XtAddCallback(helpprint, XtNcallback, HelpPrint, helpfile);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromHoriz, helpprint);  cnt++;
   XtSetArg(args[cnt], XtNfromVert,  NULL);       cnt++;
   XtSetArg(args[cnt], XtNlabel,     "Done");     cnt++;
   helpdone = XtCreateManagedWidget("helpdone", commandWidgetClass,
				   helpframe, args, cnt);
   XtAddCallback(helpdone, XtNcallback, HelpDone, NULL);

   filename = HelpGetFilename (w, helpfile);

   cnt = 0;
   XtSetArg(args[cnt], XtNfromVert,         helpprint);           cnt++;
   XtSetArg(args[cnt], XtNfromHoriz,        NULL);                cnt++;
   XtSetArg(args[cnt], XtNwidth,	    700);		  cnt++;
   XtSetArg(args[cnt], XtNheight,           300);                 cnt++;
   XtSetArg(args[cnt], XtNstring,           filename);            cnt++;
   XtSetArg(args[cnt], XtNtype,             XawAsciiFile);        cnt++;
   XtSetArg(args[cnt], XtNscrollHorizontal, XawtextScrollAlways); cnt++;
   XtSetArg(args[cnt], XtNscrollVertical,   XawtextScrollAlways); cnt++;
   helpdis = XtCreateManagedWidget("helpdis", asciiTextWidgetClass,
				   helpframe, args, cnt);
   XtFree (filename);

   XtPopup (helpshell, XtGrabNone);
   return;
}


